
/** Klasa AI odpowiedzialana za przemieszczanie sie z punktu do punktu.
 */
public class AIMover extends AIBase
{
    /** Znacznik, ze to AI moze otworzyc sobie brame, jesli brama na to pozwala
     */
    public boolean bCanOpenGates = true;
    
    /** Interwal, co jaki ma sie odbywac uaktualnianie kierunku poruszania sie AI [s]
     */
    public float fUpdateDirTime = .5f;

    /** Interwal, co jaki ma sie odbywac sprawdzanie, czy AI nie utknal [s]
     */
    public float fMoveWatchDogTime = .5f;

    /** Jaka czesc dystansu wynikajacego z predkosci poruszania sie 
     *  musi przebyc AI zeby nie uznalo, ze utknelo 
     */
    public float fStuckDistRatio = 0.3f;

    /** Tolerancja dojscia do punktu koncowego [cm]
     *  (jesli AI znajdzie sie w odleglosci mniejszej od fOnDestinationTolerance od
     *   punktu docelowego - nie jest powtornie znajdowana droga
     *   zalozenie: fOnDestinationTolerance>=fOnPlaceTolerance)  
     */
    public float fOnDestinationTolerance = 100.f;

    /** Tolerancja dojscia do punktu wyznaczonego przez PathFindera [cm].
     */
    public float fOnPlaceTolerance = 100.f;
    
    /** Czas co jaki ma probowac sie ruszyc kiedy utknie 
     *  (np. walac glowa w nie do konca otwarte drzwi) [s]
     */
    public float fStuckWaitTime = 1.f;//#AI
        
        
    /** Zmienne pozwalajace zapamietac droge i parametry spaceru */
    /** Aktualna sciezka 'wyminiec' dla dojscia do punktu docelowego 
     */
    Vector aCurrentPath[] = null;        
    /** Aktualny punkt, do ktorego ma dojsc AI
     */
    Vector vCurrentPoint = null;
    /** Koncowy punkt, do ktorego ma dojsc AI wzdluz znalezionej drogi
     */
    Vector vDestPoint = null;
    /** Indeks aktualnego punktu w tablicy aCurrentPath
     */
    int nCurrentPoint = 0;               

    /** Predkosc z jaka porusza sie AI 
     */
    float fMovementSpeed;
    
    /** Wektor kierunku predkosci AI (jednostkowy)
     */
    Vector vMovementSpeedDir = Vector.Forward;
    
    /** Nazwa funkcji, jaka ma byc zawolana po dotarciu na miejsce
     *
     */
    String sArrivedHandler = null;
    
    /** Flagi do sterowania sposobem poruszania sie przez GoTo
     */
    final static int _GOTO_USE_PATHFINDER      = 1;//Znacznik, ze AI ma uzywac PathFindera dla GoTo do punktu
    final static int _GOTO_UPDATE_MOTION       = 2;//Znacznik, ze po dojsciu do punktu koncowego ma byc zrobiony UpdateMotion
    
    final static int _GOTO_DEFAULT             = _GOTO_USE_PATHFINDER | _GOTO_UPDATE_MOTION;

    
    /** Flagi ustawione przez ostatnie GoTo
     */
    int nCurrentGotoFlags = 0;
    
    
    /** Predkosci chodzenia AI dla roznych trybow MotionState.SPEED_(STILL, SWIM, CRAWL, DUCK, WALK, RUN)
     */
    // przyspieszanie AI by MK, bylo: {0, 50, 85, 130, 400};
    float aMotionVelocity[] = {0, 70/*swim*/, 70, 100, 165, 500};
    

    /** Mnozniki predkosci dla strafowania */
    float aSpeedMultiplierStrafe[] = {0.f, 1.0f/*swim*/, 1.f, .6f, .5f, 1.f};
    /** Mnozniki predkosci dla kierunku do przodu/tylu ze strafe */
    float aSpeedMultiplierFwdStrafe[] = {0.f, 1.0f/*swim*/, 1.f, 1.f, 1.f, 1.f};
    
	
    /** Identyfikator metody, ktora ma byc zawolana po osiagnieciu kierunku zadanego przez SetDestDir
     *  Jesli = 0 to nic nie jest wolane (jak w przypadku AIMovera); 
     *  Musi byc ustawiane przed kazdym SetDestDir bo jest kasowane po osiagnieciu kierunku
     */    
    int METHOD_DestDirReached           = 0;
    
    int METHOD_MoveWatchDog             = 0;
    
    /** Zapamietany handler do wolania metod
     */
    int METHOD_UpdateDestinationDir     = 0;

    
    
    /** Jaka czesc dystansu wynikajacego z predkosci poruszania sie 
     *  musi przebyc AI zeby nie uznalo, ze utknelo 
     *  (ustawiane w StartMoveWatchDog)
     */
    float fStuckRatio = 0;
    
    /** Zapamietana ostatnia pozycja do badania przesuniecia
     */
    Vector vLastMovePos = null;

    /** Zapamietana czestotliwosc wywolywania WatchDoga (do pomiaru drogi)
     */
    float fMoveWatchDogCallingPeriod = 0;
    
    /** Nazwa animacji otwierania bramy
     */
    String sGateOpeningAnim = "Idle04";

    /** Identyfikator animacji otwierania bramy (liczony w OnCreate)
     */
    int nUnstuckAnimID = 0;

    /** Tablica z obiektami pochodzacymi od Gate w poblizu 'utknietego' AI
     */
    ControlObject aPotentialGate[];

    /** Obiekt, do ktorego ustawia sie przodem AI
     */
    ControlObject cWalkTrackedObject = null;

    /** Ponizej jakiej wartosci [deg] musi byc roznica kata i kierunku wzdluz osi
     *  (przod, tyl, prawo, lewo), zeby uznac, ze ma byc odpalana animacja dla osi
     */
    float fCrossDirTolerance = 20.f;

	WaypointSystem m_WpSystem = null;
	
    public void OnCreate()
    {
        super.OnCreate();

        METHOD_UpdateDestinationDir = RegisterCallMethod("UpdateDestinationDir");
        METHOD_MoveWatchDog         = RegisterCallMethod("MoveWatchDog");
    }
    
    /** Zakladamy ze to, czy AI jest w budynku czy nie,
     * mozna rozpoznac po obecnosci systemu way pointow
     */
    boolean InBuilding()
    {
        return (m_WpSystem != null);
    }
    
//    void InitHumanBody()
//    {
//        super.InitHumanBody();
//    }
    
    void DefineAnimationSet()
    {
        super.DefineAnimationSet();
        nUnstuckAnimID = GetAnimID(ROOT_BONE, "Idle04");
    }
    
    public void DisableVitalActivities()
    {
        StopGoTo();
        SetPhysicsActive(false);
        super.DisableVitalActivities();
    }
    
    
    /** Zwraca sledzony obiekt (patrz SetWalkTrackedObject)
     */
    public ControlObject GetWalkTrackedObject()
    {
       return cWalkTrackedObject; 
    }

    /** Ustawia obiekt do sledzenia, gdy null - wylacza sledzenie obiektu
     *  (sledzenie = nakierowywanie cialem bez zmiany wektora predkosci)
     */
    public void SetWalkTrackedObject(ControlObject cObject)
    {
        LogD((cObject != null ? cObject.GetName() : "(null)") + "\n");
        cWalkTrackedObject = cObject;
    }
    
    
    /** Zawolanie GoTo z PathFinderem (dla kompatybilnosci)
     */    
    public void GoTo(Vector vDest, byte eHow2Go, String sDestHandler)
    {
        GoTo(vDest, eHow2Go, sDestHandler, _GOTO_DEFAULT);
    }
        
    /** Metoda sluzaca do rozpoczecia przemieszczania AI w okreslone miejsce<br>
     *  z omijaniem przeszkod realizowanym przy uzyciu mapy kolizji<br>
     *  @param vDest punkt docelowy (<code>Vector</code>)
     *  @param eHow2Go sposob, w jaki chcemy isc (<code>byte = MotionState.SPEED_(CRAWL|DUCK|WALK|RUN)</code>)
     *  @param sDestHandler nazwa funkcji, jaka ma byc wywolana po dojsciu na miejsce (<code>String</code>)
     */
    public void GoTo(Vector vDest, byte eHow2Go, String sDestHandler, int nFlags)
    {
//        CrashLog("DBG: GoTo from " + GetPosition() + " to " + vDest + 
//                 " how: " + eHow2Go + "\n");
        boolean bUsePathFinder = (nFlags & _GOTO_USE_PATHFINDER) != 0 ;
                                
		if ( (m_WpSystem != null) && (bUsePathFinder) )
		{
			m_WpSystem.GetCloserPoint(vDest);
			bUsePathFinder = false;
// MZM zaremowane przy save waypointow (organizacja trzymania danych)
//			vDest.Set(m_WpSystem.m_CurrWaypoint.m_vPos);
			Vector vLen = GetPosition();
			vLen.Sub(vDest);
			vLen.fY = 0;
			if (vLen.Len2() < 10f)
				return;
		}
		
        if (eHow2Go == MotionState.SPEED_STILL)
            return; // tymczasowe, zeby sie nie pomylic, potem mozna wywalic
        METHOD_DestDirReached = 0; //zeby sie nic nie wolalo przy chodzeniu
        sArrivedHandler = sDestHandler;
        nCurrentGotoFlags = nFlags;
        
        if (bUsePathFinder)
            aCurrentPath = FindPath(GetPosition(), vDest);
        else
        {
            aCurrentPath = new Vector[2];
            aCurrentPath[0] = GetPosition();
            aCurrentPath[1] = vDest;
        }
        if ((aCurrentPath != null) && (aCurrentPath.length > 1))
        {
            //LogD("tam mamy dojsc = " + vDest.toString() + "\n");
            vDestPoint = new Vector(vDest);
            cMotion.iSpeed = eHow2Go;

            nCurrentPoint = 1;
            vCurrentPoint = aCurrentPath[nCurrentPoint];

            SetPhysicsActive( true );
            bGoToInProgress = true;
            StartMoveWatchDog( fMoveWatchDogTime, fStuckDistRatio );

            SetDestinationPos(vCurrentPoint);
            SetDestinationRadius(fOnPlaceTolerance);

            nLastMotionIdx = -1; //zaznacz, ze trzeba odswiezyc cMotion
            UpdateDestinationDir();
        }
        else
        {
            LogD("No path to " + vDest.toString() + "\n");
        }
    }
    
    boolean bStopGotoClearsDestDir = true;
    /** Funkcja powodujaca natychmiastowe zatrzymanie AI i uniewaznienie jego
     *  dotychczasowej sciezki.
     */
    public void StopGoTo()
    {
        ClearDestinationPos();
        if (bStopGotoClearsDestDir)
            ClearDestinationDir();
        vCurrentPoint = null; 
        aCurrentPath = null;
        cMotion.iMoveX = 0;
        cMotion.iMoveZ = 0;
        SetLocalSpeed(Vector.Zero);
        fMovementSpeed = 0;

        if ((nCurrentGotoFlags & _GOTO_UPDATE_MOTION) != 0)
            UpdateMotion();

        StopMoveWatchDog();
        bGoToInProgress = false;
        if (bShouldDisablePhysics)
        {
            bShouldDisablePhysics = false;
            SetPhysicsActive( false );
        }
    }

    /** Zwraca kat pomiedzy wektorem do przodu a wektorem predkosci
     *  TODO: jak bedzie dobrze dzialac i nie trzeba bedzie podawac dokladnego kata
     *  - zwracac tylko Idx
     */
    float GetFwdToSpeedAngle()
    {
        float fDotF = vMovementSpeedDir.Dot(GetForwardVector()),
              fDotR = vMovementSpeedDir.Dot(GetRightVector());
        
        float fAngle = (float)(180.0/Math.PI * Math.acos(Crop(fDotF, -1, 1)));
        if (fDotR < 0)
            return -fAngle;
        else
            return fAngle;
    }

    /** Podaje indeks ustawien MotionState w zaleznosci od kata
     */
    int Angle2MotionIdx(float fAngle)
    {
        if (fAngle < 0)
            fAngle = 360 + fAngle;
        
        int nQuarter = (int)(fAngle / 90);
        float fQuarterAngle = fAngle - (float)(nQuarter * 90);

        int nIdx;
        if (fQuarterAngle <= fCrossDirTolerance)
            nIdx = nQuarter * 2;
        else if (fQuarterAngle >= 90 - fCrossDirTolerance)
            nIdx = nQuarter * 2 + 2;
        else
            nIdx = nQuarter * 2 + 1;
        nIdx %= 8;
        //LogD(fAngle + ": " + nIdx + "\n");
        return nIdx;
    }

    /** Funkcja oblicza wartosc predkosci w zaleznosci od aktualnego MotionState
     */
    void CalcSpeedValue()
    {
        float fMultiplier = 1;

        byte iSpeed = GetReducedMotionSpeed(cMotion);

        if (cMotion.iMoveX != 0) //jesli strafuje
        {
            if (cMotion.iMoveZ != 0) // idzie po skosie
                fMultiplier = aSpeedMultiplierFwdStrafe[iSpeed];
            else // tylko strafe
                fMultiplier = aSpeedMultiplierStrafe[iSpeed];
        }
        fMovementSpeed = aMotionVelocity[iSpeed] * fMultiplier;
    }
    
    
    int nLastMotionIdx = 0;
    /** Funkcja pomocnicza ustawiajaca kierunek poruszania sie na nastepny punkt na sciezce;
     *  Wywoluje sie periodycznie (co fUpdateDirTime) aby uaktualnic kierunek np po kolizji.
     */
    void UpdateDestinationDir()
    {
        if (vCurrentPoint != null)
        {
            Vector vDir = new Vector(vCurrentPoint);
            vDir.Sub(GetPosition());
            vDir.fY = 0.0f;
            vDir.Normalize();
            if (cWalkTrackedObject == null)
            {
                SetDestDir(vDir);

                CalcSpeedValue();
       		    SetLocalSpeed(new Vector(0,0, fMovementSpeed));
                vMovementSpeedDir = GetForwardVector();
                
                if (nLastMotionIdx < 0) // Trzeba zrobic UpdateMotion
                {
                    cMotion.iMoveX = 0;
                    cMotion.iMoveZ = 1;
                    UpdateMotion();
                    nLastMotionIdx = 0;
                }
            }
            else
            {
                int nMotionIdx = Angle2MotionIdx(GetFwdToSpeedAngle());
                if (nMotionIdx != nLastMotionIdx)
                {
                    nLastMotionIdx = nMotionIdx;
                    switch (nMotionIdx)
                    {
                    case 0: cMotion.iMoveZ = 1; cMotion.iMoveX = 0;
                            break;
                    case 1: cMotion.iMoveZ = 1; cMotion.iMoveX = 1;
                            break;
                    case 2: cMotion.iMoveZ = 0; cMotion.iMoveX = 1;
                            break;
                    case 3: cMotion.iMoveZ = -1; cMotion.iMoveX = 1;
                            break;
                    case 4: cMotion.iMoveZ = -1; cMotion.iMoveX = 0;
                            break;
                    case 5: cMotion.iMoveZ = -1; cMotion.iMoveX = -1;
                            break;
                    case 6: cMotion.iMoveZ = 0; cMotion.iMoveX = -1;
                            break;
                    case 7: cMotion.iMoveZ = 1; cMotion.iMoveX = -1;
                            break;
                    default:
                            LogD("Wrong motion idx returned!\n");
                            break;
                    }
                    UpdateMotion();
                }

                Vector vTrkDir = cWalkTrackedObject.GetPosition();
                vTrkDir.Sub(GetPosition());
                //SetMaxAngSpeed(360);
                SetFacingDestDir(vTrkDir);

                vMovementSpeedDir = new Vector(vDir);

                CalcSpeedValue();
                vDir.Mul(fMovementSpeed);
                SetSpeed(vDir);
            }
            
            CallMethodOnceIn(METHOD_UpdateDestinationDir, fUpdateDirTime);
        }
    }

    
    /** Handler wolany po dojsciu do punktu okreslonego przez SetDestinationPos 
     *  na odleglosc okreslona przez SetDestinationRadius (fOnPlaceTolerance)
     */
    void OnDestinationPos()
    {
        if (vCurrentPoint == null) //to sie nie powinno zdarzyc
        {
            LogD("vCurrentPoint == null!\n");
            return; 
        }
        
        // TODO: sprawdzic, czy jest odpowiednio blisko, zmodyfikowac predkosc i takie tam
               
		nCurrentPoint++;
        // Czy doszedl do ostatniego punktu?
		if( aCurrentPath.length <= nCurrentPoint ) 
        {
            aCurrentPath = null;
            vCurrentPoint = null;
            //LogD("ostatni punkt sciezki\n");
		    if ( ((nCurrentGotoFlags & _GOTO_USE_PATHFINDER) != 0) && 
                 vDestPoint.Distance(GetPosition()) > fOnDestinationTolerance ) 
            {   //jeszcze nie doszedl do celu - powtorz szukanie
                //LogD("nie doszedl za pierwszym razem \n");
                GoTo(vDestPoint, cMotion.iSpeed, sArrivedHandler, nCurrentGotoFlags);
            }
            else //doszedl - zatrzymaj i powiadom
            {
                StopGoTo();
                if (sArrivedHandler != null)
                {
                    //LogD("WOLAMY METODE: \"" + sArrivedHandler + "\"\n");
                    CallMethodOnceIn(sArrivedHandler, 0);
                }
            }
        }
        else // przejdz do nastepnego
        {
            vCurrentPoint = aCurrentPath[nCurrentPoint];

            SetDestinationPos(vCurrentPoint);
            SetDestinationRadius(fOnPlaceTolerance);
            UpdateDestinationDir();
        }
    }
   
    void OnDestinationDir()
    {
//        LogD(METHOD_DestDirReached + "\n");
        if (METHOD_DestDirReached != 0)
        {
            CallMethodOnceIn(METHOD_DestDirReached, 0);
            METHOD_DestDirReached = 0;
        }
        
        if (bShouldDisablePhysics && !bGoToInProgress)
        {
            bShouldDisablePhysics = false;
            SetPhysicsActive( false );
        }
    }
    
    
    //////////////////////////////////////////////////
    // Podmienione metody natywne - dla optymalizacji
    
    /** Znacznik, ze fizyka nie byla aktywna i nalezy ja wylaczyc
     */
    boolean bShouldDisablePhysics = false;
    /** Znacznik, ze AI dochodzi do podanego punktu i nie mozna wylaczyc fizyki
     */
    boolean bGoToInProgress = false;
    
    public void SetDestDir(Vector v)
    {
        // wlacz fizyke jesli nie jest wlaczona
        if (!GetPhysicsActive())
        {
            SetPhysicsActive( true );
            bShouldDisablePhysics = true;
        }
        super.SetDestDir(v);
    }

    public void SetFacingDestDir(Vector v)
    {
        // wlacz fizyke jesli nie jest wlaczona
        if (!GetPhysicsActive())
        {
            SetPhysicsActive( true );
            bShouldDisablePhysics = true;
        }
        super.SetFacingDestDir(v);
    }
    
    public void ClearDestinationDir()
    {
        super.ClearDestinationDir();
        if (bShouldDisablePhysics && !bGoToInProgress)
        {
            bShouldDisablePhysics = false;
            SetPhysicsActive( false );
        }
    }
    
    
    //////////////////////////////////////////////////
    // WatchDog odpowiedzialny za poruszanie sie
    void StartMoveWatchDog(float fUpdateTime, float fStuckDistRatio)
    {
        fMoveWatchDogCallingPeriod = fUpdateTime;
        fStuckRatio = fStuckDistRatio;
        vLastMovePos = null;
        MoveWatchDog(); //inicjalizacja        
        CallMethodEvery( METHOD_MoveWatchDog, fMoveWatchDogCallingPeriod );
    }
    
    void StopMoveWatchDog()
    {
        DisableCallMethod( METHOD_MoveWatchDog );
        DisableCallMethod( "GoOn" );
        DisableCallMethod( "GateOpen" );
        vLastMovePos = null;
    }

    void RestoreMoveWatchDog()
    {
        vLastMovePos = null;
        MoveWatchDog(); //inicjalizacja        
        CallMethodEvery( METHOD_MoveWatchDog, fMoveWatchDogCallingPeriod );
    }

    ControlObject cMoveObstacle = null;
    void MoveWatchDog()
    {
        Vector vCurrPos = GetPosition();
        
        // jesli w budynku to pootwieraj pobliskie drzwi
        ((ChromeModule)GetModule()).GetDoorControler().IsNearDoorable(this);
        if (cMoveObstacle == null)
        {
            if (vLastMovePos != null) //jesli inicjalizacja
            {
                float fMinDist = fMovementSpeed * fMoveWatchDogCallingPeriod * fStuckRatio;
                if (vLastMovePos.Distance(vCurrPos) < fMinDist)
                {
                    LogD("###Utklam!###\n");
                    Unstuck();
                    return;
                }
            }
            float s = fMovementSpeed * fMoveWatchDogCallingPeriod + fEllipsoidRadius[cMotion.iSpeed];
            //            LogD(s + "\n");
            Vector vStart = vCurrPos;
            vStart.fY += 100.f;
            
            Vector vEnd = new Vector(vMovementSpeedDir);
            vEnd.Mul(s);
            vEnd.Add(vStart);

            Collision cCol = TraceObjectsExclude(vStart, vEnd, this);
            if (cCol != null)
            {
                cMoveObstacle = cCol.Obstacle;
//                LogD("Kolizja z:" + cMoveObstacle + ", dist=" + cCol.fDistance + "\n");
                float t;
                if ((fMovementSpeed > 0) && (cCol.fDistance > fEllipsoidRadius[cMotion.iSpeed]))
                    t = (cCol.fDistance - fEllipsoidRadius[cMotion.iSpeed]) / fMovementSpeed;
                else
                    t = 0; // lepiej teraz niz nigdy
                CallMethodOnceIn("Unstuck", t);
            }
        }
        vLastMovePos = vCurrPos;
    }
    
    
    void Unstuck()
    {
        //CrashLog("DBG: Unstuck!\n");

        StopMoveWatchDog();
        MotionState cMovingMotion = new MotionState(cMotion);
        //SetLocalSpeed(Vector.Zero);
        SetSpeedValue(0);
        cMotion.Stop();
        UpdateMotion();

        DisableCallMethod( METHOD_UpdateDestinationDir );
        
        if ((cMoveObstacle != null) && (cMoveObstacle instanceof Gate))
        {
            aPotentialGate = new ControlObject[1];
            aPotentialGate[0] = cMoveObstacle;
        }
        else
            aPotentialGate = GetObjectsInRadiusFromClass(1000.f, "Gate");

        cMoveObstacle = null;

        if (aPotentialGate.length > 0)
        {
            if (bCanOpenGates)
            {
                PlayAnim(nUnstuckAnimID, 1);
                // rzadko sie to zdarza, wiec wolanie po nazwie
                CallMethodOnceIn( "GateOpen", GetAnimLength(nUnstuckAnimID) );
            }
            else
                StopGoTo(); //trudno nie wolno - zatrzymaj sie
            // TODO: w klasach pochodnych moze byc konieczne zaznaczenie, ze jakies zadanie sie zakonczylo...
        }
        else
            CallMethodOnceIn( "GoOn", fStuckWaitTime);

        cMotion = cMovingMotion;
    }
    

    /** Wolane po czasie potrzebnym na odegranie animacji otwierania bramy
     */
    void GateOpen()
    {
        for (int i=0; i<aPotentialGate.length; i++)
        {
            Gate cGate = (Gate)aPotentialGate[i];
            /*if (cGate.CanAIOpenIt(this))
                cGate.Execute(this);*/
        }
        PlayStandAnim();
//        PlayAnim( Anims[A_STAND], -1 ); //reset pozycji
        CallMethodOnceIn( "GoOn", 1.f);
    }
    
    void GoOn()
    {
        UpdateDestinationDir();
        UpdateMotion();
        RestoreMoveWatchDog();
    }
	
	void OnBlockMove()
	{
		if ((nCurrentGotoFlags & _GOTO_USE_PATHFINDER) != 0)
		{
			// probujemy isc jeszcze raz
			GoTo(vDestPoint, cMotion.iSpeed, sArrivedHandler, nCurrentGotoFlags);
		}
		else
			ClearFlag(EFlags._STOPPED_BY_PATHFINDER);

	}
    
/** TESTOWE
    boolean bTest1 = true; 
    
    public void SetPhysicsActive(boolean bActive)
    {
        if (bTest1 || bActive)
            super.SetPhysicsActive(bActive);
    }
 */
/*    public void OnCollisionFullInfo(ControlObject Col,Vector vPoint,Vector vNormal)
    {
		CrashLog("Object:" + Col + " normal: " + vNormal + "\n");
	}
*/	
}
