/** Klasa AI reprezentujaca Walkera (maszyne kroczaca)
 *  Wersja: Walker_w - 'wojskowa' z dwoma gunami i dwoma
 *          rakietnicami
 */
public class AIWalker_w extends AIWalker
{
    /** Tablica z gunami i rakietnicami Walkera
     */
    Weapon aWkrWeapons[] = null;

    
    int nWalkerLeftCannonID = 0;
    int nWalkerRightCannonID = 0;
    
    /** Indeksy dla broni
     */
    final static int _WKR_WPN_GUN_L         = 0;
    final static int _WKR_WPN_GUN_R         = 1;
    final static int _WKR_WPN_ROCKET_L      = 2;
    final static int _WKR_WPN_ROCKET_R      = 3;
    
    AIWalker_w()
    {
        super();
        MeshName = "Data/Meshes/AI/Walker_w.3da";

        ROOT_BONE = "Walker Pelvis";
        TORSO_BONE = "Walker Neck";

        fTwistHorizMin = -25;
        fTwistHorizMax = 25;
        fTwistVertMin = -45;
        fTwistVertMax = 65;
        fStartHealth = 4000.f;
        aMotionVelocity = new float[] { 0, 330/*swim*/, 330, 330, 330, 330 };

        //sAnimSeqFile = "Data/Meshes/AI/Walker_w_anim.scr";
        aWalkerHeight = new float[] { 520, 290/*swim*/, 290, 540, 470, 290 };
        sWalkerEye = "Rocket L";
    
        sCabinElemName = "Walker Cover";
        
        /** Dzwieki Walker_w
         */
        sStepSound = "Data/Sounds/Walker/Walker_Walk_w";
    
        sCabinOpenSound = "Data/Sounds/Walker/Walker_OpenCabin_w";
        sCabinCloseSound = "Data/Sounds/Walker/Walker_CloseCabin_w";
        sStandbySound = "Data/Sounds/Walker/Walker_Standby_w";
        sWakeUpSound = "Data/Sounds/Walker/Walker_WakeUp_w";
        sNormUpSound = "Data/Sounds/Walker/Walker_NormUp_w";
        sUpNormSound = "Data/Sounds/Walker/Walker_UpNorm_w";
        sNormDownSound = "Data/Sounds/Walker/Walker_NormDown_w";
        sDownNormSound = "Data/Sounds/Walker/Walker_DownNorm_w";

        sHeadLoopSound = "Data/Sounds/Walker/Walker_HeadLoop_w";
        sGunRotateLoopSound = "Data/Sounds/Walker/Walker_GunRotateLoop_w";

    }
    
    
    void InitWalker()
    {
        nWalkerEyeID = GetElementID(sWalkerEye);
        nWalkerLookAtID = GetElementID("Walker Fotel");
        nWalkerHeadID = GetElementID("Walker Head");
        nWalkerNeckID = GetElementID("Walker Neck");
        
        nWalkerLeftCannonID = GetElementID("Walker L Cannon");
        nWalkerRightCannonID = GetElementID("Walker R Cannon");
        
        InitHeadElementID();
    }
    
    
    void InitHeadElementID()
    {
        String aElemName[] = { "Walker Neck", "Walker Head" };
        aHeadElementID = new int[aElemName.length];
        for (int i=0; i<aElemName.length; i++)
            aHeadElementID[i] = GetElementID(aElemName[i]);
    }

    
    void InitializeAIWeapon()
    {
        String aWeaponElems[] = { "Gun L", "Gun R", "Rocket L", "Rocket R" };
        String aWeaponClassNames[] = { "WeaponWalker_w_Gun", "WeaponWalker_w_Gun",
                                     "WeaponWalker_w_Rocket", "WeaponWalker_w_Rocket" };
        
        // nie wolamy z bazowej, bo inna koncepcja

        aWkrWeapons = new Weapon[4];
        for (int i=0; i<4; i++)
        {
            Weapon cWea = (Weapon)CreateObject(aWeaponClassNames[i]);
            cWea.Execute(this);
            ChangeWeapon(cWea);

            AttachChild(cWea, aWeaponElems[i]);
            cWea.SetDirLocal(new Vector(1,0,0));

            cWea.EnableRendering(false);
            cWea.ClearFlag(EFlags._BLOCK_TRACE);
            cWea.ClearFlag(EFlags._BLOCK_MOVE);

            cWea.SetWeaponID(GetElementID(aWeaponElems[i]));
            aWkrWeapons[i] = cWea;
        }
    }
   

    void DefineAnimationSet()
	{
        super.DefineAnimationSet();
        SetElementAnimType(nWalkerLeftCannonID, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType(nWalkerRightCannonID, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
	}

    
    
    
    Vector GetFireOrigin(Weapon cWpn)
    {
        return GetElementPos(cWpn.GetWeaponID());    
    }

    Vector GetFireDir(Weapon cWpn)
    {
        // liczymy kierunek do celu
        Vector vDir = cEnemy.GetPointToLookAt(nTargetPointIdx);
        if (vDir == null)
        {
            vDir = cEnemy.GetPointToLookAt(0);
        }
        if (vDir != null)
            vDir.Sub(GetFireOrigin(cWpn));
        else 
            vDir = GetForwardVector();
        
        // dodajemy rozrzut
        Vector vDev = new Vector(RandomFloat(1), RandomFloat(1), RandomFloat(1));

        if (cWpn.nCurrentMode == Weapon.MODE_SINGLE)
            vDev.Mul(RandomFloat(fAimingDeviation));
        else //auto
            vDev.Mul(RandomFloat(fBurstAimingDeviation));

        vDev.Mul(vDir.Len());
        vDir.Add(vDev);
        
        return vDir;
    }

    void MessageWeapon(int nState, Weapon cWpn)
    {
        if ((nState & Weapon.STATE_SHOT) != 0)
        {
            //PlayAnim( Anims[A_WEA_FIRE], 1 );
            //TODO: dodac odrzut guna
        }
        if ((nState & Weapon.STATE_NO_AMMO) != 0)
        {
            cWpn.nAmmo = 100000;
        } 
        else if ((nState & (Weapon.STATE_OVERHEAT | Weapon.STATE_ERROR)) != 0)
        { 
            //za czesto strzelamy albo cos nie wyszlo 
            LogD("Shooting to often!\n");
        }
    }
    
    void ExecuteFXBarrel(String sFXName, Weapon cWpn)
    {
        ExecuteFX(this, cWpn.GetWeaponID(), sFXName, Vector.Zero);
    }

    
    /** Rozpoczecie strzelania z 'marszowego' guna
     */
    void PrimaryWeaponStartFire()
    {
        aWkrWeapons[_WKR_WPN_GUN_L].StartFire(Weapon.MODE_AUTO);
        //aWkrWeapons[_WKR_WPN_GUN_R].StartFire(Weapon.MODE_AUTO);
        CallMethodOnceIn("Start2ndPrimary", GetPrimaryWeaponDelay() * 0.5f); 
    }
    
    void Start2ndPrimary()
    {
        aWkrWeapons[_WKR_WPN_GUN_R].StartFire(Weapon.MODE_AUTO);
    }
    
    /** Zatrzymanie strzelania z 'marszowego' guna
     */
    void PrimaryWeaponStopFire()
    {
        aWkrWeapons[_WKR_WPN_GUN_L].StopFire();
        aWkrWeapons[_WKR_WPN_GUN_R].StopFire();
    }
    
    /** Pobranie opoznienia dla 'marszowego' guna
     */
    float GetPrimaryWeaponDelay()
    {
        return aWkrWeapons[_WKR_WPN_GUN_L].fDelay;
    }
    
    /** Odpalenie strzelania ze 'stacjonarnego' guna
     */
    void SecondaryWeaponStartFire()
    {
        aWkrWeapons[_WKR_WPN_ROCKET_L].StartFire(Weapon.MODE_AUTO);
        aWkrWeapons[_WKR_WPN_ROCKET_R].StartFire(Weapon.MODE_AUTO);
        //CallMethodOnceIn("Start2ndSecondary", GetSecondaryWeaponDelay() * 0.5f);
    }
    
//    void Start2ndSecondary()
//    {
//        aWkrWeapons[_WKR_WPN_ROCKET_R].StartFire(Weapon.MODE_AUTO);
//    }
    
    /** Zatrzymanie strzelania ze 'stacjonarnego' guna
     */
    void SecondaryWeaponStopFire()
    {
        aWkrWeapons[_WKR_WPN_ROCKET_L].StopFire();
        aWkrWeapons[_WKR_WPN_ROCKET_R].StopFire();
    }

    /** Pobranie opoznienia dla 'stacjonarnego' guna
     */
    float GetSecondaryWeaponDelay()
    {
        return aWkrWeapons[_WKR_WPN_ROCKET_L].fDelay;
    }


    void UpdateFiringPrimary()
    {
        ActiveWeapon = aWkrWeapons[_WKR_WPN_GUN_L];
        super.UpdateFiringPrimary();
    }

    void UpdateFiringSecondary()
    {
        ActiveWeapon = aWkrWeapons[_WKR_WPN_ROCKET_R];
        super.UpdateFiringSecondary();
    }

    
    void TwistBody(float fHorizAngle, float fVertAngle, int nTwistType)
    {   
        //LogD("fHorizAngle = " + fHorizAngle + " fVertAngle = " + fVertAngle + "\n");
        fVertAngle *= -1;
        switch (nTwistType)
        {
        case _TWIST_TYPE_NONE:
            {
                RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, 0);
                RotateElementWithAnim(nWalkerHeadID, vAlongYAxis, 0);
                RotateElementWithAnim(nWalkerLeftCannonID, vAlongXAxis, 0);
                RotateElementWithAnim(nWalkerRightCannonID, vAlongXAxis, 0);
                if (fLastAngleH != 0)
                    PlayHeadRotateSound(GetAnimMorphTime());
                if (fLastAngleV != 0)
                    PlayGunRotateSound(GetAnimMorphTime());
                fLastAngleH = fLastAngleV = 0;
            }
            break;
        case _TWIST_TYPE_HEAD_ONLY:
        case _TWIST_TYPE_HEAD_USE_TORSO:
        case _TWIST_TYPE_TORSO:
            {
                RotateElementWithAnim(nWalkerHeadID, vAlongYAxis, fHorizAngle);
                float fAddAng = 0;
                if (Math.abs(fHorizAngle) > 10.f)
                    fAddAng = fLegSafeAngleVert;
                    
                RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, fAddAng);
                RotateElementWithAnim(nWalkerLeftCannonID, vAlongXAxis, -fAddAng - fVertAngle);
                RotateElementWithAnim(nWalkerRightCannonID, vAlongXAxis, -fAddAng - fVertAngle);
                
                if (fLastAngleH != fHorizAngle)
                {
                    PlayHeadRotateSound(GetAnimMorphTime());
                    fLastAngleH = fHorizAngle;
                }
                if (fLastAngleV != fVertAngle)
                {
                    PlayGunRotateSound(GetAnimMorphTime());
                    fLastAngleV = fVertAngle;
                }

                break;

            }
        default:
            LogD("Incorrent twist type value!\n");
            break;
        }
        nLastTwistType = nTwistType;
    }

    
    
}
