public class AIHologram extends AIAttacker
{
    ////////////////////////////////////////////////////////////
    // Sterowanie zachowaniem hologramow 
    //
    
    /** Znacznik, ze hologram ma uciekac, gdy wrog opusci hotSpot
     */
    public boolean bEscapeWhenEnemyMoved = true;//#AIHologram
    
    /** Znacznik, ze hologram ma zniknac, gdy wrog opusci hotSpot
     *  (UWAGA: wyklucza sie z bEscapeWhenEnemyMoved, wlaczany - ma wyzszy priorytet)
     */
    public boolean bHideWhenEnemyMoved = false;//#AIHologram
    
    /** Znacznik, ze AI ma uciekac, gdy oberwie, ale nie zostanie zabite
     */
    public boolean bEscapeOnDamage = false;//#AIHologram
  
    
    /** Znacznik, ze hologram zadaje damage i nie znika */
    public boolean bIsARealMan = false;//#AIHologram

    
    
    /** Jak daleko ma uciekac w kierunku swego ustawienia hologram 
     *  zanim zniknie (tylko, jesli nie ma sciezki) [cm]
     */
    public float fEscapeDistance = 500.f;//#AIHologram
                                 
    /** Jak dlugo maksymalnie ma uciekac hologram zanim zniknie [s]
     */
    public float fMaxEscapeTime = 5.f;//#AIHologram

    /** Dzwiek odgrywany przy znikaniu hologramu */
    public String sHologramHideSound = "Data/Sounds/Ai/HologramHide";//#AIHologram
    
    /** Znacznik, ze po zabiciu/zniknieciu hologramu moze on zmartwychwstac */
    public boolean bMultipleUse = false;//#AIHologram
    
    
    
    
    /** Znacznik, ze hologram jest juz aktywowany
     */
    boolean bIsHoloActive = false;
    
    
    /** Zapamietany fDamage ActiveWeapona */
    float fOldDamage = 40;
    
    public void OnCreate()
    {
        super.OnCreate();
        ClearFlag(EFlags._BLOCK_MOVE);
        nReadinessToSearch = 0;
        ActivateAI(false, _FREEZE_ADVENTURE);
        if (ActiveWeapon != null)
        {
            fOldDamage = ActiveWeapon.fDamage;
        }
        else
            CrashLog("ERROR: ActiveWeapon == null (" + this.GetName() + ")\n");
    }
    
    
    public void Run()
    {
    }
    
    /////////////////////////////////////////////////////////////////
    // Komunikaty przychodzace z HotSpota
    //
    public void ActorEnteredHotSpot(ChromeActor cActor)
    {
        StartAttack(cActor);
    }
    
    public void ActorLeftHotSpot(ChromeActor cActor)
    {
        if (bHideWhenEnemyMoved)
        {
            HideHolo();
        }
        else if (bEscapeWhenEnemyMoved)
        {
            StartEscape();
        }
    }
    
    ///////////////////////////////////////////////////////////////////
    
    /** Wyzwolenie ataku przez hologram
     */
    public void StartAttack(ChromeActor cNewEnemy)
    {
        if (!bDead && !bIsHoloActive)
        {
            bIsHoloActive = true;
            ActivateAI(true, _FREEZE_ADVENTURE);
            
            if (ActiveWeapon != null)
            {
                if (bIsARealMan)
                    ActiveWeapon.fDamage = fOldDamage;
                else
                    ActiveWeapon.fDamage = 0.f;
            }
            
            if (!bIsARealMan)
                bUsePhysicsToDeathFall = false;
            
            
            Vector vNewDir = cNewEnemy.GetPosition();
            vNewDir.Sub(GetPosition());
            vNewDir.Normalize();
            SetDir(vNewDir);
            SetPositionWorld(vRespawnPos);
            
            nAttackCause = _ATTACK_CAUSE_SIGHT;
            nTargetPointIdx = 0;
            Attack(cNewEnemy);
        }
    }
    
    ///////////////////////////////////////////////////////////////////
    
    /** Wyzwolenie ucieczki 
     *  - w pierwotnym kierunku ustawienia hologramu, jesli nie ma sciezki 
     *    (na odleglosc fEscapeDistance)
     *  - po sciezce (jesli ma ja ustawiona)
     *  Ucieczka moze trwac max. fMaxEscapeTime
     */
    public void StartEscape()
    {
        if (!bDead && bIsHoloActive)
        {
            DisableCallMethod("StartEscape");
            
            if (aPath == null || aPath.GetPointsNum() == 0)
            {
                aPath = new AIPatrolerPath();

                aPath.ConvertToFlags(AIPatrolerPath._PATROLPATH_ONCE);
                aPath.nPathID = -1;
                aPath.sStringID = GetName() + "_Path";
                
                Vector vNewPos = new Vector(vRespawnDir);
                vNewPos.Normalize();
                vNewPos.Mul(fEscapeDistance);
                vNewPos.Add(GetPosition());
                
                AIPatrolerPathPoint point = new AIPatrolerPathPoint(vNewPos);
                point.fRadius = 100;
                point.nFlags = AIPatrolerPathPoint._PNF_NOPATHFINDER;
                point.fStopTimeInterval = 0f;
                point.iSpeed = MotionState.SPEED_RUN;
                //point.sTextField = "EscapeDone";
                aPath.aPoints.addElement(point);
            }
            
            /* W ostatnim punkcie zawsze zawolaj wymagana metode */
            PathPoint lastPoint = aPath.GetPoint(aPath.GetPointsNum() - 1);
            lastPoint.sTextField = "EscapeDone";
            
            nCurrentNode = -1;
            SetState("Escape");
            CallMethodOnceIn("EscapeDone", fMaxEscapeTime);
        }
    }
    
    
    void EscapeDone()
    {
        DisableCallMethod("EscapeDone");
        ActivateAI(false, _FREEZE_ADVENTURE);
        bIsHoloActive = false;
    }
    
    ///////////////////////////////////////////////////////////////////

/*    public void OnTakeDamage(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nDamageType)
    {
        super.OnTakeDamage(fDamage, Assassin, vHitLocation, vSpeed, nDamageType);
        if (!bDead && bEscapeOnDamage)
            StartEscape();
    }
*/    
	// poprawione OnTakeDamage //////////////////////////////////////////////////////////////
	public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation,
							 Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
	{
        super.OnTakeDamage(fDamage, fPhSimDamage, Assassin, vHitLocation, vSpeed, nMeshElementorBoneID, nDamageType);
        if (!bDead && bEscapeOnDamage)
            StartEscape();
	}
		
    ////////////////////////////////////////////////////////////////////////
    
    /** Ukrywanie hologramu - 'migocze' i znika
     */
    void HideHolo()
    {
        if (bIsHoloActive)
        {
            bIsHoloActive = false;
            SetScaleAbsolute(new Vector(1.2f, 1.2f, 1.2f));
            CallMethodIn("HideHolo_1", 0.1f);
            CallMethodIn("HideHolo_2", 0.2f);
            CallMethodIn("HideHolo_3", 0.3f);
            CallMethodIn("HideHoloDone", 0.5f);
            PlaySound(sHologramHideSound);
            DisableCallMethod("EscapeDone");
            if (!bMultipleUse)
                bDead = true;
        }
    }
    
    void HideHolo_1()
    {
        SetScaleAbsolute(new Vector(1, 1.3f, 1));
    }
    
    void HideHolo_2()
    {
        SetScaleAbsolute(new Vector(0.5f, 1.3f, 0.5f));
    }

    void HideHolo_3()
    {
        SetScaleAbsolute(new Vector(0.1f, 0.9f, 0.1f));
    }

    
    void HideHoloDone()
    {
        DisableCallMethod("HideHoloDone");
        SetScaleAbsolute(new Vector(1,1,1));
        ActivateAI(false, _FREEZE_ADVENTURE);
    }
    
    ////////////////////////////////////////////////////////////////////////

    /** Pomocnicze: Ustawia skale obiektu (a nie domnaza macierz skalowania)
     */
    void SetScaleAbsolute(Vector vNewScale)
    {
        Vector vOldScale = GetScale();
        SetScale(new Vector(vNewScale.fX / vOldScale.fX,
                            vNewScale.fY / vOldScale.fY,
                            vNewScale.fZ / vOldScale.fZ));
    }

    ////////////////////////////////////////////////////////////////////////
    // Pokryte stany
    //
    
    public class Escape extends AIPatroler.Patrol
    {
        public void OnReact()
        {
        }
        
        public void OnHearNoise(Actor Noisy, float fNoiseLevel, int nNoiseType)
        {
        }
        
/*        public void OnTakeDamage(float fDamage, GameObject Assassin, Vector vHitLocation,
								 Vector vSpeed, int nDamageType)
        {
            //CrashLog("Escape.OnTakeDamage!\n");
        }
*/		
		// poprawione OnTakeDamage //////////////////////////////////////////////////////////////
		public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation,
								 Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
		{
		}
    }
    
    
    public class Die extends AIBase.Die
    {
        public void OnEnter()
        {
            if (bIsARealMan)
                super.OnEnter();
            else
            {
                DisableVitalActivities();
                if (bIsHoloActive)
                    HideHolo();
                if (bMultipleUse)
                    bDead = false;
            }
        }
        
        public void OnReact()
        {
        }
        
        public void OnHearNoise(Actor Noisy, float fNoiseLevel, int nNoiseType)
        {
        }
    }
    
    
}

