/** Rozbudowana kamera. Potrafi robic rozblyski
 */
public class CameraExtended extends Camera
{
    /** to powinno byc w C++ */
    boolean m_bActive = false; 
       
    /** zmienne zwiazane z rozblyskami */
    Sprite m_cFlashSprite = null;
    int   m_aFlashTex[]    = null;

    float m_fFlashIntensity   = 0.0f;
    float m_fFlashLastTime    = 0.0f;
    float m_fFlashSpeed       = 0.2f;
    
    /** Pozycja spritea do flasha
     */
    Vector m_vFlashSpritePos  = new Vector(0.0f,0.0f,0.0f);
    Vector m_vFlashSpriteSize = new Vector(1024.0f, 1024.0f, 0.0f);

    /** identyfikatory rozblyskow */
    final static int _FLASH_BLOOD    = 0;
    final static int _FLASH_WHITE    = 1;
    final static int _FLASH_MAXCNT   = 2;
    
    /** Flaga mowiaca czy kamera ma zapamietac stan efektu flasha
     *  podczas deaktywacji, oraz jesli wywolanie flasha nastapilo w
     *  momencie gdy kamera nie jest aktywna
     *  Jesli tak, to przy ponownej aktywacji nastapi odtworzenie 
     *  faktycznego stanu flasha (kontynuacja z uwzglednieniem czasu wywolania) */
    boolean m_bKeepFlashIfNotActive = false;

    /** Flaga mowiaca o tym, czy wszelkie flasze wywolane na _tej_ kamerze
     *  maja byc rozpropagowane do innych kamer. 
     *  W tej chwili takimi docelowymi kamerami sa te obserwujace gadajace glowy 
     */
    boolean m_bPropagateFlash = true;
    
    /** Flaga mowiaca o tym, czy kamera ma akceptowac _przychodzace_
     * propagacje flashy
     */
    boolean m_bAcceptPropagatedFlash = false;
    
    /**
     */
    void OnCreate()
    {
        //flash
        m_cFlashSprite = (Sprite)CreateObject("Sprite");
        m_cFlashSprite.SetPos(m_vFlashSpritePos);
        m_cFlashSprite.SetSize(m_vFlashSpriteSize);
        m_cFlashSprite.SetName("m_cFlashSprite");
        
        m_aFlashTex = new int[ _FLASH_MAXCNT ];
        m_aFlashTex[ _FLASH_BLOOD ] = m_cFlashSprite.AddTexture("Data/Interface/ER_T_Red.png");
        m_aFlashTex[ _FLASH_WHITE ] = m_cFlashSprite.AddTexture("Data/Interface/ER_T_White.png");

        m_cFlashSprite.SetTexture(m_aFlashTex[0], new Rect(0, 0, 1, 1));
        m_cFlashSprite.SetAlpha(0f);
        AddSprite( m_cFlashSprite );
        m_cFlashSprite.MoveTop();
    }
    
    public void OnDestroy()
    {
        if(m_aFlashTex != null)
            for(int i=0; i< m_aFlashTex.length; ++i)
                m_cFlashSprite.ReleaseTexture( m_aFlashTex[i] );
        
        if(m_cFlashSprite != null)
        {
            RemoveSprite(m_cFlashSprite);
            m_cFlashSprite.delete();
            m_cFlashSprite = null;
        }
    }
    
    int m_nViewId    = -1;
    
    /** skojarzenie widoku z kamera, po to aby wiedziec jakie parametry
     * ustawic spritowi do flasha, domyslnie flash jest na caly ekran
     * docelowo bedzie to przychodzilo automatycznie jako handler
     * TODO: ponadto nalezy zrobic obsluga dla sytuacji gdy ta sama kamera
     * obsluguje kilka widokow
     */
    public void OnAttachedToView(int nViewId)
    {
        // wiemy juz jaki jest nasz widok
        m_nViewId = nViewId;
        
        // pobiezmy jego parametry zeby wiedziec gdzie umiescic
        // sprite'a do flasha (docelowo wiele spritow dla kazdego z widokow)
        Module cModule = GetModule();
        if ( cModule == null )
        {
            return;
        }
        
        float fSpriteSizeX = (float)cModule.GetViewSizeX( m_nViewId );
        float fSpriteSizeY = (float)cModule.GetViewSizeY( m_nViewId );
        float fSpritePosX  = (float)cModule.GetViewOriginX( m_nViewId );
        float fSpritePosY  = (float)(GetScreenHeight() - cModule.GetViewOriginY( m_nViewId ) - fSpriteSizeY);
              
        m_vFlashSpritePos  = new Vector( fSpritePosX, fSpritePosY, 0.0f );
        m_vFlashSpriteSize = new Vector( fSpriteSizeX,fSpriteSizeY,0.0f );
        
        if (m_cFlashSprite != null)
        {
            m_cFlashSprite.SetPos( m_vFlashSpritePos );
            m_cFlashSprite.SetSize( m_vFlashSpriteSize );
        }
    }
    
    /**
     */
    public void OnActivate()
    {
        m_bActive = true;

        if (m_bKeepFlashIfNotActive)
        {
            //jesli byl wczesniej zawolany flash to go odtwozmy
            if ( m_cFlashSprite != null && m_fFlashIntensity > 0.0f )
            {
                m_cFlashSprite.MoveTop();
                m_cFlashSprite.Show();
                
                // a to nam zagwarantuje, ze jesli jego czas juz minal
                // to zostanie od razu schowany
                ProcessFlashOnFrame();
            }
        }
    }

    /**
     */
    public void OnDeactivate()
    {
        m_bActive = false;

        //chowanie sprite'a rozblysku
        if ( m_cFlashSprite != null && m_fFlashIntensity > 0.0f )
        {
            if ( !m_bKeepFlashIfNotActive )
                m_fFlashIntensity = 0.0f;
            
            m_cFlashSprite.Hide();
        }
    }

    /**
     */
    public void Flash( int nId, float fDuration,float fValue, float fMaxValue )
    {
        LogD(" "+this+" m_bPropagateFlash "+m_bPropagateFlash+"\n");
        
        if ( m_bPropagateFlash )
        {
            // musimy o tym flashu powiadomic inne aktywne kamery
            PropagateFlash( nId, fDuration, fValue, fMaxValue );
        }
        
        if ( m_cFlashSprite == null ) 
        {
            CrashLog(" CameraExtended::Flash(), m_cFlashSprite == null!\n");
            return;
        }
        
        if ( nId < 0 || nId >= _FLASH_MAXCNT )
        {
            CrashLog(" CameraExtended::Flash(), flash id "+nId+" not found!\n");
            return;
        }
        
        if ( m_bActive || m_bKeepFlashIfNotActive )
        {
            LogD(" 2\n");
            m_fFlashIntensity += fValue;
            if (m_fFlashIntensity > fMaxValue)
                m_fFlashIntensity = fMaxValue;
            m_fFlashLastTime = GetTime();
            m_fFlashSpeed = 1.0f/fDuration * m_fFlashIntensity;
            m_cFlashSprite.SetTexture( m_aFlashTex[ nId ], new Rect(0, 0, 1, 1) );
            if ( m_bActive )
            {
                LogD(" 3\n");
				m_cFlashSprite.MoveTop();
                m_cFlashSprite.Show();
            }
        }
    }

    /**
     */
    public void OnFrame()
    {
        ProcessFlashOnFrame();
    }
    
    void ProcessFlashOnFrame()
    {
        if ( m_cFlashSprite == null ) 
            return;
        
        if ( m_fFlashIntensity > 0.0f )
        {
            float fNowTime   = GetTime();
            float fDeltaTime = fNowTime - m_fFlashLastTime;
            m_fFlashLastTime = fNowTime;

            m_fFlashIntensity -= m_fFlashSpeed * fDeltaTime;

            if( m_fFlashIntensity <= 0.0f )
            {
                m_fFlashIntensity = 0.0f;
                m_cFlashSprite.Hide();
            }
            else
            {
                if (m_fFlashIntensity > 1.f)
                    m_cFlashSprite.SetAlpha( 1.f );
                else
                    m_cFlashSprite.SetAlpha( m_fFlashIntensity );
            }
        }
    }

    /** Metoda majaca za zadanie rozpropagowac flashe do wszystkich
     * innych aktywnych kamer
     * TODO: zmienic to, powinien byc jakis manager aktualnie aktywnych kamer (widokow)
     * Na razie proka wola flashe dla kamer gadajacych glow
     */
    void PropagateFlash( int nId, float fDuration,float fValue, float fMaxValue )
    {
        
        
        Module cModule = GetModule();
        
        if ( ! (cModule instanceof ChromeModuleSingle) )
        {
            LogD(" "+this+" skipped\n");
            return;
        }
        
        if (((ChromeModuleSingle)cModule).MainPlayer == null)
        {
            CrashLog(" MainPlayer not found in ChromeModuleSingle!\n");
            return;
        }
            
        if (((ChromeModuleSingle)cModule).MainPlayer.HUD == null )
        {
            CrashLog(" MainPlayer.HUD not found in ChromeModuleSingle!\n");
            return;
        }
        
        HUDCompNiunia cHudNiunia = ((ChromeModuleSingle)cModule).MainPlayer.HUD.GetHUDNiunia();
                
        if ( cHudNiunia == null )
        {
            CrashLog(" MainPlayer.HUD. HUDCompNiunia not found in ChromeModuleSingle!\n");
            return;
        }
        
        cHudNiunia.PropagatedFlash( nId, 
                                    fDuration,
                                    fValue, 
                                    fMaxValue );
    }

    
    public void PropagatedFlash( int nId, float fDuration,float fValue, float fMaxValue )
    {
        
        // najpierw sprawdzmy czy w ogole obslugujemy przychodzace propagacje flashy
        if ( ! m_bAcceptPropagatedFlash )
        {
            LogD(" "+this+" skipped\n");
            return;
        }
        
        
        LogD(" "+this+"\n");
        // zapamietajmy sobie nasza zdolnosc do propagacji
        boolean bPropagateFlash =  m_bPropagateFlash;
        // wyzerujmy na chwile flage, zeby nie propagowalo sie dalej
        m_bPropagateFlash = false;
        
        // wywolajmy normalnego flasha
        Flash( nId, fDuration, fValue, fMaxValue );
        
        // i przywrocmy stan flagi zeby "nasze" flasze w przyszlosci mogly byc przez
        // nas propagowane
        m_bPropagateFlash = bPropagateFlash;
    }
    
    /** Ustawianie flagi m_bKeepFlashIfNotActive
     */
    void SetKeepFlashIfNotActive( boolean bKeep )
    {
        m_bKeepFlashIfNotActive = bKeep;
    }

    /** Ustawianie flagi m_bPropagateFlash
     */
    void SetPropagateFlash( boolean bPropagate )
    {
        m_bPropagateFlash = bPropagate;
    }
    
    /** Ustawianie flagi m_bAcceptPropagatedFlash
     */
    void SetAcceptPropagatedFlash( boolean bAccept )
    {
        m_bAcceptPropagatedFlash = bAccept;
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean m_bKeepFlashIfNotActiveSG = false;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[CameraExtended.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunksChrome._CHK_CAMERA_EXTENDED)
        {
			super.SGLoadChunk(cFC);			

			m_bActive = cFC.LoadBool();
       
			// zmienne zwiazane z rozblyskami
			m_fFlashIntensity = cFC.LoadFloat();
			m_fFlashLastTime = cFC.LoadFloat();
			m_fFlashSpeed = cFC.LoadFloat();
    
			// Pozycja spritea do flasha
			m_vFlashSpritePos = cFC.LoadVector();
			m_vFlashSpriteSize = cFC.LoadVector();
				
			m_nViewId = cFC.LoadInt();			
			
			m_bKeepFlashIfNotActiveSG = cFC.LoadBool();			
        }
		
		cFC.delete();
	}
	
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[CameraExtended.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_CAMERA_EXTENDED, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveBool(m_bActive);
       
		// zmienne zwiazane z rozblyskami
		cFC.SaveFloat(m_fFlashIntensity);
		cFC.SaveFloat(m_fFlashLastTime);
		cFC.SaveFloat(m_fFlashSpeed);
    
		// Pozycja spritea do flasha
		cFC.SaveVector(m_vFlashSpritePos);
		cFC.SaveVector(m_vFlashSpriteSize);
			
		cFC.SaveInt(m_nViewId);
		
		cFC.SaveBool(m_bKeepFlashIfNotActive);
							
		cFC.delete();
	}	    	        
}
