/** Klasa bazowa dla wszystkich bram potrzebna miedzy innymi dla AI
 */
public class Gate extends TriggerObject implements iact_OpenClose, iact_LockUnlock, iact_LockOpened
{
    /** flaga mowiaca czy AI moze otwierac brame
     */
    public boolean bCanAIOpenIt = true;//#\GATE
    
    /** flaga mowiaca czy dla danej bramy maja byc specjalnie liczone kolizje trace 
     *  chodzi o bramy siatkowe 
     **/
    public boolean bFenceGateType = false;//#GATE
    
    public float fMinSound = 2000f;//#GATE
    public float fMaxSound = 5000f;//#GATE
    
    /** dzwiek, ktory ma byc odgrywany podczas otwierania */
    public String sOpenSound = "Data/Sounds/DoorE_Open";//#GATE
    
    /** dzwiek, ktory ma byc odgrywany podczas zamykania */
    public String sCloseSound = "Data/Sounds/DoorE_Close";//#GATE
    
    /** dzwiek, ktory ma byc odegrany gdy uzyjemy zamka elektronicznego */
    public String sLockSystemAccept = "Data/Sounds/doors/LockSystemAccept";//#GATE
    
    /** dzwiek, ktory ma byc odegrany gdy nie mozna otworzyc drzwi */
    public String sLockSystemReject = "Data/Sounds/doors/LockSystemReject";//#GATE
    
    /** Tablica obiektow indykujacych stan drzwi, skopiowane z doorsobject*/
    TriggerObject[] aIndicatorsFound = null;
    
    String[] m_sIndicatorElems = new String[]{
                                    "TerminalA",
                                    "Terminal"
                                    };
    
    
    /** Czy moze byc uzywany wiele razy */
    public boolean bMultipleUse = true;//#\GATE
    
    /** Stan poczatkowy */
    public boolean bOpened = false;//#\GATE

    /** Stan poczatkowy zamkniecia */
    public boolean bLocked = false;//#\GATE
    
    public Action[] aOnOpen = null;//#\GATE
    public Action[] aOnClose = null;//#\GATE
    
    /** klatki animacji otwierania */
    int nOpenStartFrame = 0;
    int nOpenEndFrame = 0;
    
    /** klatki animacji zamykania */
    int nCloseStartFrame = 0;
    int nCloseEndFrame = 0;
    
    int nAnimOpenID = -1;
    int nAnimCloseID = -1;
    
    Gate()
    {
        super();
        ActiveElement = "zamek_elektroniczny";
    }
    
        
    void OnCreate()
    {
        bSG = true;
        
        super.OnCreate();
        
        int nLastFrame = (int)GetNumFrames();

        // ustawiamy klatki animacji
        if(nOpenStartFrame == nOpenEndFrame && nOpenStartFrame == 0)
        {
            nOpenStartFrame = 0;
            nOpenEndFrame = nLastFrame/2;
            CrashLog("[Gate.OnCreate] nOpenStartFrame="+nOpenStartFrame+" nOpenEndFrame="+nOpenEndFrame+"\n");
        }
        
        if(nCloseStartFrame == nCloseEndFrame && nCloseStartFrame == 0)
        {
            nCloseStartFrame = nLastFrame/2;
            nCloseEndFrame = nLastFrame;
            CrashLog("[Gate.OnCreate] nCloseStartFrame="+nCloseStartFrame+" nCloseEndFrame="+nCloseEndFrame+"\n");
        }

        //inicjacja animacji wlaczania/wylaczania
        InitAnimSeq("open", nOpenStartFrame, nOpenEndFrame);
        InitAnimSeq("close", nCloseStartFrame, nCloseEndFrame);
        
        nAnimOpenID = GetAnimID(0, "open");
        nAnimCloseID = GetAnimID(0, "close");

        SetAnimMorphTime(0.0f);
        
        SetFlag(EFlags._BLOCK_TRACE);
        ClearFlag(EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH);
        
        // ten obiekt ma wylaczone kolizje z trace'ami
        if (bFenceGateType)
            ClearFlag(EFlags._BLOCK_TRACE);
        
        //jesli poczatkowa flaga bOpened ustawiona na true trzeba otworzyc brame
        if (bOpened)
        {
            // ustawiamy ostatnia klatke sekwencji otwarcia
            SetFrame(nOpenEndFrame, 0);
        }
        
        GetNearestDoorsIndicators();

        UpdateIndicators();
        
    }

    public void OnDestroy()
    {
    }
    
    void Execute( Actor executor )
    {
        if(bLocked) return;
        
        super.Execute(executor);
        
        if (!bOpened) //sa zamkniete
        {
            float fTime = GetWaveDuration(sLockSystemAccept);
            PlaySound(sLockSystemAccept, GetElementPos(GetElementID(ActiveElement)), fMinSound, fMaxSound);
            CallMethodIn("Open", fTime);
        }
        else //sa otwarte
        {
            float fTime = GetWaveDuration(sLockSystemAccept);
            PlaySound(sLockSystemAccept, GetElementPos(GetElementID(ActiveElement)), fMinSound, fMaxSound);
            CallMethodIn("Close", fTime);
        }
    }
    
    String GetTip()
    {
        return bOpened ? "[&GateClose&]" : "[&GateOpen&]";
    }
    
    public void Open()
    {
        if(bLocked) return;
        
        if (!bMultipleUse) bActiveTrigger = false;
        bOpened = true;
        PlayAnim(nAnimOpenID, 1);
        CallMethodIn("AfterOpen", GetAnimLength(nAnimOpenID));
        PlaySound(sOpenSound, fMinSound, fMaxSound);
    }
    
    void AfterOpen()
    {
        InvokeActionChain(aOnOpen);
    }
    
    public void Close()
    {
        if(bLocked) return;
        
        if (!bMultipleUse) bActiveTrigger = false;
        bOpened = false;
        PlayAnim(nAnimCloseID, 1);
        CallMethodIn("AfterClose", GetAnimLength(nAnimCloseID));
        PlaySound(sCloseSound, fMinSound, fMaxSound);
    }
    
    public void ToggleOpenClose()
    {
        if(bOpened)
            Close();
        else
            Open();
    }
    
    void AfterClose()
    {
        InvokeActionChain(aOnClose);
    }

    public void Lock()
    {
        bLocked = true;
        
        UpdateIndicators();
    }
    
    public void Unlock()
    {
        bLocked = false;
        
        UpdateIndicators();
    }
    
    public void ToggleLockUnlock()
    {
        if(bLocked)
            Unlock();
        else
            Lock();
    }
    
    public boolean IsLocked()
    {
        return bLocked;
    }
    
    public boolean IsOpened()
    {
        return bOpened;
    }
    
    public void LockOpened()
    {
        // wymuszamy wykonanie otwarcia nawet jak brama jest zalokowana
        bLocked = false;
        
        // otwieramy brame
        Open();
        
        // blokujemy ja
        Lock();
    }
	
    void GetNearestDoorsIndicators()
    {
        Vector vPos = GetPosition();
        GameObject[] aLampkas = GetObjectsInRadiusFromClass(vPos, 750.0f, "TriggerObject");
        float fMinDist = 100000000000.0f;
        float fMinDist2 = 100000000000.0f;
        int nMinLampka = -1;
        int nMinLampka2 = -1;
        for(int i=0; i<aLampkas.length; i++)
        {
            if (!IsIndicator((MeshObject)aLampkas[i]))
                continue;
            
            float fDist = vPos.Distance2(((ControlObject)aLampkas[i]).GetPosition());
            if(fDist < fMinDist)
            {
                fMinDist = fDist;
                nMinLampka = i;
            }
            else
            {
                if(fDist < fMinDist2)
                {
                    fMinDist2 = fDist;
                    nMinLampka2 = i;
                }
            }
        }

        if(nMinLampka != -1 && nMinLampka2 != -1)
        {
            aIndicatorsFound = new TriggerObject[2];
            aIndicatorsFound[0] = (TriggerObject)aLampkas[nMinLampka];
            aIndicatorsFound[1] = (TriggerObject)aLampkas[nMinLampka2];
        }
        else
            if (nMinLampka != -1)
            {
                aIndicatorsFound = new TriggerObject[1];
                aIndicatorsFound[0] = (TriggerObject)aLampkas[nMinLampka];
            }
            else
                if (nMinLampka2 != -1)
                {
                    aIndicatorsFound = new TriggerObject[1];
                    aIndicatorsFound[0] = (TriggerObject)aLampkas[nMinLampka2];
                }
        
    }
    
    boolean IsIndicator(MeshObject cObj)
    {
        for (int i = 0; i < m_sIndicatorElems.length; i++)
        {
            if (cObj.GetElementID(m_sIndicatorElems[i]) != -1)
                return true;
        }
        return false;
        
    }
    
    /** Update stanu lampek
     */
    void UpdateIndicators()
    {
        if(aIndicatorsFound==null) 
            return;
        for(int i=0; i<aIndicatorsFound.length; ++i)
                if(IsLocked() && !IsOpened())
                    aIndicatorsFound[i].SetSkin("Locked");//DoorClose();
                else
                    aIndicatorsFound[i].SetSkin("Unlocked");//DoorOpen();
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Gate.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_GATE)
        {
			super.SGLoadChunk(cFC);
			
            boolean bPlayingAnim = false;
            
			if (cFC.LoadBool())
            {
				//PlayAnim(nAnimOpenID, 1);
                bPlayingAnim = true;
            }
			
			if (cFC.LoadBool())
            {
				//PlayAnim(nAnimCloseID, 1);
                bPlayingAnim = true;
            }
            
            bLocked = cFC.LoadBool();
            bOpened = cFC.LoadBool();
            
            if(!bPlayingAnim)
            {
                if(bOpened)
                    SetFrame(GetLastFrame(nAnimOpenID), -1);
                else
                    SetFrame(GetLastFrame(nAnimCloseID), -1);
            }
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Gate.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_GATE, cFCParent);
		
		super.SGSaveChunk(cFC);
								
		cFC.SaveBool(IsPlaying(nAnimOpenID));
		cFC.SaveBool(IsPlaying(nAnimCloseID));
		
        cFC.SaveBool(bLocked);
        cFC.SaveBool(bOpened);
        
		cFC.delete(); 
	}		
    
    public void SGUpdate()
	{
		FileChunk.Log("[Gate.SGUpdate]\n");
        super.SGUpdate();
        UpdateIndicators();
        
    }
}
