public class UIRotorText extends UserInterface
{
    /** ramie (promien) tekstu po jakikm bedzie sie obracal */
    protected final static float      _TEXT_ARM           = -1f; //jezeli -1, to rozmiar ramienia jest rowny rozmiarowi elementu
    
    protected final static float      _FULL_ANGLE         = 360f;
    
    protected final static float      _ARM_COEF           = 15f;
    
    /** lista elementow */
    protected java.util.Vector m_cElements = null;
    /** ramie */
    protected Vector m_vArm = null;
    /** dlugosc ramienia */
    protected float m_fArmLength = -1f;
    /** element bazowy */
    protected UIStatic m_cBase = null;
    /** wskaznik waypointa (strzalka)*/
    protected UIStatic m_cPointer = null;
    /** wskaznik obiektu (punkt na mapie)*/
    protected UIStatic m_cLocationPoint = null;
    /** pozycja */
    protected Vector m_vRotorPos = null;
    /** rozmiar */
    protected Vector m_vRotorSize = null;
    /** flaga ustalenia rozmiarow, pozycji.... */
    protected boolean m_bElementsPositionsUpdated = false;
    
    int _idx = 0;
   
    void OnCreate()
    {
        if( _TEXT_ARM > 0 )
        {
            m_vArm = new Vector( _TEXT_ARM, 0, 0 );
        }
        ActivateOnModule();
        super.OnCreate();
    }
    
    /** zaladowanie ze skryptu */
    boolean Load(ParseKeyword wndKey)
    {
        super.Load( wndKey );
        
        LoadBaseButton();
        
        return true;
    }
    /** zaladowanie elementow */
    protected void LoadBaseButton()
    {
        m_cBase = (UIStatic)GetElem("ID_BASE");
        m_cBase.SetVisible(false);
        m_cPointer = (UIStatic)GetElem("ID_BASE_POINT");
        m_cPointer.SetVisible(false);
        m_cLocationPoint = (UIStatic)GetElem("ID_OBJECT_POINT");
        m_cLocationPoint.SetVisible(false);
    }
    
    int AddElem( String sName )
    {
        return AddElem( sName, 0, null );
    }
    
    int AddElem( String sName, float fAngle, DefLocation cLocation )
    {
        RotorTextElement cElem = new RotorTextElement();
        
        cElem.AddElem( sName, fAngle, null , cLocation );
        m_cElements.addElement( cElem );
        cElem.m_nIndex = _idx;
        _idx ++;
        return m_cElements.indexOf( cElem );
    }
    
    RotorTextElement GetElem(int nElem)
    {
        return (RotorTextElement)m_cElements.elementAt(nElem);
    }
    
    /**Usuwa element z polem m_nIndex o wartosci nElemIdx, nie jest to indeks w wektorze
     */
    void RemoveElem(int nElemIdx)
    {
        for (int i = 0; i < m_cElements.size(); i ++)
        {
            RotorTextElement cElem = (RotorTextElement)m_cElements.elementAt(i);
            if (cElem.m_nIndex == nElemIdx)
            {
                cElem.delete();
                m_cElements.removeElement(cElem);           
            }
        }
        
    }
    
    /** odswiezanie ctl'a */
    public void OnFrame()
    {
        if( !m_bInited )
            return;
        
        super.OnFrame();
        
        //ustalenie pozycji elemntow
        if( !m_bElementsPositionsUpdated )
        {
            OnElementsPositionsUpdate();
            m_bElementsPositionsUpdated = true;
        }
    }
    
    void OnElementsPositionsUpdate()
    {
        if( !m_bInited || m_bElementsPositionsUpdated )
            return;

        //pozycja
        m_vRotorPos = GetPos();
        
        //rozmiar
        m_vRotorSize = GetSize();
        
        //ustalenie ramienia po jakim maja sie poruszac elementy - jezeli
        //nie zostalo zdefiniowane w skrypcie przyjmuje rozmiar menu (polowe)
        if( m_vArm == null )
        {
            m_vArm = new Vector( this.GetSize().fX / 2f, 0f, 0f );
        }
        //ramie
        //m_vArm.fX *= GetResCoeffX();
        m_fArmLength = m_vArm.Len();
    }
    
    float GetResCoeffX()
    {
        return GetScreenWidth()/800f;
    }
    
    void MoveTopElement(int nIndex)
    {
        if( m_cElements != null )
        {
            if( nIndex >= 0 && nIndex < m_cElements.size() )
            {
                ((RotorTextElement)m_cElements.elementAt( nIndex )).MoveTopElement();
            }
        }
    }
    
    /** podswietlenie aktualnie wybranego waypointa */
    void SetElementTextColor( int nIndex, float fR, float fG, float fB )
    {
        SetElementTextColor( nIndex, fR, fG, fB, 1f );
    }
    void SetElementTextColor( int nIndex, float fR, float fG, float fB, float fAlpha)
    {
        if( m_cElements != null )
        {
            if( nIndex >= 0 && nIndex < m_cElements.size() )
            {
                ((RotorTextElement)m_cElements.elementAt( nIndex )).SetElementTextColor( fR, fG, fB, fAlpha );
            }
        }
    }
        
    int GetElementsSize()
    {
        if( m_cElements != null )
        {
            return m_cElements.size();
        }
        return -1;
    }
    
    void UpdateElemPos( int nIndex, float fAngle, float fRadius, float fDistance, float fMapRadius )
    {
        ((RotorTextElement)m_cElements.elementAt( nIndex )).UpdateElem( fAngle, fRadius, fDistance, fMapRadius );
    }
        
    
    float GetArmLength()
    {
        if( m_vArm != null )
            return m_vArm.Len();
        else
            return -1f;
    }
    
    public class RotorTextElement
    {
        /**id przyznawany przy dodawaniu, potrzebny do kasowania*/
        int m_nIndex = -1;
        /** button */
        UIStatic m_cElement = null;
        /** punkt wskazujacy */
        UIStatic m_cPoint = null;
        /** objectiv */
        UIStatic m_cObjectPoint = null;
        /** aktualny kat */
        float m_fAngle = -1f;
        /** zmienne do wyliczania pozycji elementu */
        protected float fElemPosX, fElemPosY, fCalcX, fCalcY;
        /** zmienne do wyliczania pozycji elementu */
        protected float fObjectPosX, fObjectPosY, fObjectCalcX, fObjectCalcY;
        /** aktualne pozycje elementu w X i Y */
        protected float m_fPosX = -1f;
        protected float m_fPosY = -1f;
        /** aktualne pozycje elementu w X i Y */
        protected float m_fObjectPosX = -1f;
        protected float m_fObjectPosY = -1f;
        /** pozycja elementu */
        protected Vector vElementPos = new Vector();
        /** pozycja elementu */
        protected Vector vObjectPos = new Vector();
        protected float fObjectRot = 0;
        /** zmienna sluzaca do obliczenia dlugosci ramienia */
        protected Vector vLen = new Vector();
        
        protected DefLocation m_cLocation = null;
        
        void AddElem( String sName )
        {
            AddElem( sName, 0, null, null );
        }
        
        void AddElem( String sName, float fAngle )
        {
            AddElem( sName, fAngle, null, null );
        }
        
        void AddElem( String sName, float fAngle, Object cData, DefLocation cLocation)
        {
            m_cLocation = cLocation;
            if( m_cBase == null || m_cPointer == null || m_cLocationPoint == null)
            {
                CrashLog("UIRotorText::AddElem() - brak elementu lub punktu bazowego; m_cBase == null || m_cPointer == null");
                return;
            }
            if( m_cElements == null )
            {
                m_cElements = new java.util.Vector();
            }
            
            UIStatic cWndNew = (UIStatic)CreateObject("UIStatic");
            //nadanie id
            cWndNew.m_sId = "ID_BUTTON";
            //kopiowanie bazowego do nowego - nadanie wlasciwosci
            m_cBase.Copy( cWndNew, m_cBase );
            cWndNew.m_cSupervisor = m_cBase.m_cSupervisor;
            cWndNew.m_cData = cData;
            cWndNew.m_bVisible = true;
            cWndNew.SetActive( false );
            AddElement( cWndNew );
            if (cLocation != null)
            {
                //TODO inaczej zorganizowac tekstury
                float fTexX = cLocation.GetTextureWidth(cLocation.m_iTexNo);
                float fTexY = cLocation.GetTextureHeight(cLocation.m_iTexNo);
                
                cWndNew.SetNormalTexture(new UITexture(cLocation.m_TexName, fTexX, fTexY,
                                        cLocation.m_vTexRect.fLeft * fTexX, 
                                        cLocation.m_vTexRect.fTop * fTexY, 
                                        cLocation.m_vTexRect.fRight * fTexX, 
                                        cLocation.m_vTexRect.fBottom * fTexY));
                cWndNew.UpdateLogicSizeAndPos(cLocation.GetSize(), new Vector());
                cWndNew.OnSizeAndPos();
            }
            else
            {
                cWndNew.SetText( sName );
                cWndNew.UpdateLogicSizeAndPos(new Vector(16, 16, 0), new Vector());
                cWndNew.OnSizeAndPos();
            }
           //Reload();
            
            m_cElement = cWndNew;
            
            //dodanie punktu waypointa
            
            cWndNew = (UIStatic)CreateObject("UIStatic");
            //nadanie id
            cWndNew.m_sId = "ID_POINT";
            //kopiowanie bazowego do nowego - nadanie wlasciwosci
            m_cBase.Copy( cWndNew, m_cPointer );
            cWndNew.m_cSupervisor = m_cBase.m_cSupervisor;
            cWndNew.SetActive( true );
            cWndNew.SetText( "" );
            cWndNew.m_cData = cData;
            cWndNew.m_bVisible = true;
            AddElement( cWndNew );
            
            //Reload();
            
            m_cPoint = cWndNew;
            
            cWndNew = (UIStatic)CreateObject("UIStatic");
            //nadanie id
            cWndNew.m_sId = "ID_OBJECT_POINT";
            //kopiowanie bazowego do nowego - nadanie wlasciwosci
            m_cBase.Copy( cWndNew, m_cLocationPoint );
            cWndNew.m_cSupervisor = m_cBase.m_cSupervisor;
            cWndNew.SetActive( true );
            cWndNew.SetText( "" );
            cWndNew.m_cData = cData;
            cWndNew.m_bVisible = true;
            AddElement( cWndNew );
            if (cLocation != null)
            {
                //TODO inaczej zorganizowac tekstury
                float fTexX = cLocation.GetTextureWidth(cLocation.m_iTexNo);
                float fTexY = cLocation.GetTextureHeight(cLocation.m_iTexNo);
                
                cWndNew.SetNormalTexture(new UITexture(cLocation.m_TexName, fTexX, fTexY,
                                        cLocation.m_vTexRect.fLeft * fTexX, 
                                        cLocation.m_vTexRect.fTop * fTexY, 
                                        cLocation.m_vTexRect.fRight * fTexX, 
                                        cLocation.m_vTexRect.fBottom * fTexY));
                cWndNew.UpdateLogicSizeAndPos(cLocation.GetSize(), new Vector());
                cWndNew.OnSizeAndPos();
            }
            else
            {
                cWndNew.UpdateLogicSizeAndPos(new Vector(16, 16, 0), new Vector());
                cWndNew.OnSizeAndPos();
            }
            Reload();
            
            m_cObjectPoint = cWndNew;
            
            SetElemStartPos( fAngle );
        }
        
        protected void UpdateElem( float fAngle, float fRadius, float fDistance, float fMapRadius )
        {
                
            CalculateElemPos( fAngle, fRadius, fDistance, fMapRadius);
            SetElemPos();
        }
        
        
        protected boolean CalculateElemPos( float fAngle, float fRadius, float fDistance, float fMapRadius )
        {
            //CrashLog("fangle "+fAngle);
            //uaktualnienie kata
            if( !bFirstCalculate )
            {
                fAngle -= m_fAngle;
                m_fAngle += fAngle;
            }
            
            if( m_fAngle >= _FULL_ANGLE )
            {
                m_fAngle -= _FULL_ANGLE;
            }
                
            if( m_fAngle < 0 )
            {
                m_fAngle = _FULL_ANGLE + m_fAngle;
            }
            
            float fMulCoef;
            
            if( fDistance <= fRadius && fDistance > 0 && fRadius > 0)
            {
                float fLength;
                
                
                fLength = fMapRadius * (fDistance/fRadius);// * GetResCoeffX();
                
                vLen.fX = m_fPosX;
                vLen.fY = m_fPosY;
                vLen.fZ = 0f;
                
                fMulCoef = fLength / vLen.Len();
                vLen.Mul( fMulCoef );
                
                m_fObjectPosX = vLen.fX;
                m_fObjectPosY = vLen.fY;
                
                vLen.fX = m_fPosX;
                vLen.fY = m_fPosY;
                vLen.fZ = 0f;
                
                fMulCoef += _ARM_COEF * GetResCoeffX() / vLen.Len();
                vLen.Mul( fMulCoef  );
                
                m_fPosX = vLen.fX;
                m_fPosY = vLen.fY;
                
                if (m_cPoint != null)
                    m_cPoint.Hide();
                if (m_cElement != null)
                    m_cElement.Hide();
                if (CanShowObjectPoint())
                {
                    m_cObjectPoint.SetActive(true);
                    m_cObjectPoint.Show();
                }
                
            }
            else
            {
                vLen.fX = m_fPosX;
                vLen.fY = m_fPosY;
                vLen.fZ = 0f;
                
                fMulCoef = m_vArm.Len() / vLen.Len();
                vLen.Mul( fMulCoef );
                m_fObjectPosX = vLen.fX;
                m_fObjectPosY = vLen.fY;
                
                vLen.fX = m_fPosX;
                vLen.fY = m_fPosY;
                vLen.fZ = 0f;
                
                fMulCoef += _ARM_COEF * GetResCoeffX() / vLen.Len();
                
                vLen.Mul( fMulCoef  );
                
                m_fPosX = vLen.fX;
                m_fPosY = vLen.fY;
                
                if (m_cPoint != null)
                    m_cPoint.Show();
                if (CanShowElement())
                    m_cElement.Show();
                if (m_cObjectPoint != null)
                {
                    m_cObjectPoint.SetActive(false);
                    m_cObjectPoint.Hide();
                }
                
            }
            
            
                
            //wyliczenie pozycji aktualnej pozcji w stosunku do rodzica
            fElemPosX = m_vRotorPos.fX + m_vRotorSize.fX / 2f;
            fElemPosY = m_vRotorPos.fY + m_vRotorSize.fY / 2f;
            
            fObjectPosX = fElemPosX;
            fObjectPosY = fElemPosY;
                
            //obrocenie o kat bazowy
            fCalcX = CalculatePosX( fAngle, true );
            fCalcY = CalculatePosY( fAngle, true );
            //obrocenie o kat bazowy
            fObjectCalcX = CalculatePosX( fAngle, false );
            fObjectCalcY = CalculatePosY( fAngle, false );
            
            //CrashLog("fCalcX "+fCalcX+" "+fObjectCalcX);
            //CrashLog("fCalcY "+fCalcY+" "+fObjectCalcY);
                
            fElemPosX += fCalcX;
            fElemPosY += fCalcY;
            
            fObjectPosX += fObjectCalcX;
            fObjectPosY += fObjectCalcY;
            
            //CrashLog("fElemPosX "+fElemPosX+" "+fObjectPosX);
            //CrashLog("fElemPosY "+fElemPosY+" "+fObjectPosY);
                
            m_fPosX = fCalcX;
            m_fPosY = fCalcY;
            
            m_fObjectPosX = fObjectCalcX;
            m_fObjectPosY = fObjectCalcY;
                
            if (m_cElement != null)
            {
                vElementPos.fX = fElemPosX - m_cElement.GetSize().fX / 2f;
                vElementPos.fY = fElemPosY - m_cElement.GetSize().fY / 2f;
            }
            
            if (m_cObjectPoint != null)
            {
                vObjectPos.fX = fObjectPosX - m_cObjectPoint.GetSize().fX / 2f;
                vObjectPos.fY = fObjectPosY - m_cObjectPoint.GetSize().fY / 2f;
            }
            
            return false;
        }
        
        //czy mozna pokazac punkt na kompasie, przydatne glownie w sieci
        boolean CanShowObjectPoint()
        {
            return m_cObjectPoint != null;
        }
        
        boolean CanShowElement()
        {
            return m_cElement != null;
        }
        
        /** ustalenie pozycji */
        protected void SetElemPos()
        {
            if (m_cElement != null)
                m_cElement.SetPos( vElementPos );
            if (vElementPos.Len() == 0)
            {
                /*Tools.CrashLogWarning("LOCATION POS 0,0 !!!");
                Thread.dumpStack();*/
            }
            if (m_cObjectPoint != null)
            {
                m_cObjectPoint.SetPos( vObjectPos );
                //m_cObjectPoint.SetRotation( m_cLocation.GetRotation() );
            }
            
            if (m_cPoint != null)
                m_cPoint.SetRotation( m_fAngle+90f );
        }
        /** zmiana koloru tekstu */
        protected void SetElementTextColor( float fR, float fG, float fB, float fAlpha)
        {
            if (m_cElement != null)
                m_cElement.SetTextColor( fR, fG, fB, fAlpha );
        }
        
        boolean bFirstCalculate = true;
        
        protected void SetElemStartPos( float fAngle )//, Vector vParentPos, Vector vParentSize )
        {
            
            //ustalenie kata poczatkowego i aktualnego dla elementu
            if( fAngle >= _FULL_ANGLE )
                fAngle -= _FULL_ANGLE;
            
            m_fAngle = fAngle;

            m_fPosX = m_vArm.fX;
            m_fPosY = m_vArm.fY;
            
            m_fObjectPosX = m_vArm.fX;
            m_fObjectPosY = m_vArm.fY;
            
            CalculateElemPos( fAngle, -1f, -1f, HUDCompass.GetArmLength() );
            
            bFirstCalculate = false;
            
            SetElemPos();
        }
        /** wylicza pozycje na okregu w X */
        protected float CalculatePosX( float fAngle, boolean bText )//, float fPosX, float fPosY )
        {
            float fDeg = fAngle * (float)Math.PI / 180f;
            float a = (float)Math.cos( fDeg ) * (bText?m_fPosX:m_fObjectPosX) - (float)Math.sin( fDeg ) * (bText?m_fPosY:m_fObjectPosY);
            return a;
        }
        /** wylicza pozycje na okregu w Y */
        protected float CalculatePosY( float fAngle, boolean bText )//, float fPosX, float fPosY )
        {
            float fDeg = fAngle * (float)Math.PI / 180f;
            float b = (float)Math.sin( fDeg ) * (bText?m_fPosX:m_fObjectPosX) + (float)Math.cos( fDeg ) * (bText?m_fPosY:m_fObjectPosY);
            return b;
        }
        
        void MoveTopElement()
        {
            if (m_cElement != null)
            {
                m_cElement.MoveTop();
            }
            if (m_cPoint != null)
            {
                m_cPoint.MoveTop();
            }
            if (m_cObjectPoint != null)
            {
                m_cObjectPoint.MoveTop();
            }
        }
        
        //te metody maja znaczenie w sieciowej wersji tej klasy
        public void Hide()
        {
        }
        public void Show()
        {
        }
        
        public void delete()
        {
            if (m_cElement != null)
            {
                m_cElement.Remove();
                m_cElement.ReleaseTextures();
                m_cElement.GetModule().RemoveSprite(m_cElement);
                m_cElement.delete();
                m_cElement = null;
            }
            if (m_cPoint != null)
            {
                m_cPoint.Remove();
                m_cPoint.ReleaseTextures();
                m_cPoint.GetModule().RemoveSprite(m_cPoint);
                m_cPoint.delete();
                m_cPoint = null;
            }
            if (m_cObjectPoint != null)
            {
                m_cObjectPoint.Remove();
                m_cObjectPoint.ReleaseTextures();
                m_cObjectPoint.GetModule().RemoveSprite(m_cObjectPoint);
                m_cObjectPoint.delete();
                m_cObjectPoint = null;
            }
        }
        
    }
}
