/** klasa przechowujaca 'piramidke' ControlObjectow,
 * ktorymi pozniej mozna sterowac - glownie animacje:
 * obracanie, skalowanie, alpha, przesuwanie... itd. itp
 */
public class UserInterfacePyramid extends UserInterface
{
    /** pelny kat */
    final static float _FULL_ANGLE  = 360f;
    /** liczba elementow */
    private int m_nElements = -1;
    /** nazwa elementow w skrypcie */
    private String m_sElementID = null;
    /** lista elementow */
    private java.util.Vector m_vElements = new java.util.Vector();
    /** czas obrotu obejmujacy wszystkie elementy piramidki */
    private float m_fRotateGlobalTime = 0.0f;
    
    
    void SortPyramidElems( boolean bFirstOnTop )
    {
        if( m_vElements != null )
        {
            if( m_vElements.size() > 1 )
            {
                if( bFirstOnTop )
                {
                    for( int i = m_vElements.size() - 1; i >= 0; i--)
                    {
                        GetPyramidElem( i ).MoveTop();
                    }
                }
                else
                {
                    for( int i = 0; i < m_vElements.size(); i++ )
                    {
                        GetPyramidElem( i ).MoveTop();
                    }
                }
            }
        }       
    }
    
    UIWindow GetPyramidElem( int nIndex )
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                return ((PyramidElement)m_vElements.elementAt( nIndex )).GetPyramidElem();
            }
        }
        return null;
    }
    /** zwraca Rotation elementu */
    float GetRotation( int nIndex )
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                return ((PyramidElement)m_vElements.elementAt( nIndex )).GetRotation();
            }
        }
        return 0;
    }
    
    /** wypisanie tekstu na elemencie */
    void SetElementText( int nIndex, String sText )
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                ((PyramidElement)m_vElements.elementAt( nIndex )).SetElementText( sText );
            }
        }
    }
    /** zmiana globalnego czasu obrotu - 'globalny' = dotyczy wszystkich elementow */
    void SetRotateGlobalTime( float fVal )
    {
        m_fRotateGlobalTime = fVal;
    }
    /** powrot elementu do pozycji z ktorej 'przybyl' */
    void MoveElemOut( int nIndex, float fTime )
    {
        MoveElemOut(nIndex, fTime, 0.0f);
    }
    /** powrot elementu do pozycji z ktorej 'przybyl' */
    void MoveElemOut(int nIndex, float fTime, float fPauseTime)
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                ((PyramidElement)m_vElements.elementAt( nIndex )).MoveElemOut( fTime, fPauseTime );
            }
        }
    }
    
    /** zmiana pozycji elementu - animacja */
    void MoveElem(int nIndex, float fTime)
    {
        MoveElem( nIndex, null, null, fTime, 0.0f );
    }
    
    void MoveElem(int nIndex, Vector vEffectDestPos, float fTime)
    {
        MoveElem( nIndex, null, vEffectDestPos, fTime, 0.0f );
    }
    
    void MoveElem( int nIndex, Vector cDestPos, float fTime, float fPauseTime )
    {
        MoveElem( nIndex, null, cDestPos, fTime, fPauseTime );
    }
    
    void MoveElem( int nIndex, Vector cStartPos, Vector cDestPos, float fTime, float fPauseTime )
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                ((PyramidElement)m_vElements.elementAt( nIndex )).MoveElem( cStartPos, cDestPos, fTime, fPauseTime );
            }
        }
    }
    
    /** zatrzymanie obracania */
    void StopRotateElement( int nIndex )
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                ((PyramidElement)m_vElements.elementAt( nIndex )).StopRotateElement();
            }
        }
    }
    
    /** obracanie ciagle */
    void RotateElementRightContinuous( int nIndex )
    {
        RotateElementLeftContinuous( nIndex, m_fRotateGlobalTime );
    }
    void RotateElementRightContinuous( int nIndex, float fTime )
    {
        RotateElement( nIndex, -_FULL_ANGLE + 1, fTime, false, true, false, false );
    }
    void RotateElementRightContinuous( int nIndex, float fTime, boolean bMask, boolean bMaskDirection )
    {
        RotateElement( nIndex, -_FULL_ANGLE + 1, fTime, false, true, bMask, bMaskDirection );
    }
    
    /** obracanie ciagle */
    void RotateElementLeftContinuous( int nIndex )
    {
        RotateElementLeftContinuous( nIndex, m_fRotateGlobalTime );
    }
    void RotateElementLeftContinuous( int nIndex, float fTime )
    {
        RotateElement( nIndex, _FULL_ANGLE - 1, fTime, false, true, false, false );
    }
    void RotateElementLeftContinuous( int nIndex, float fTime, boolean bMask, boolean bMaskDirection )
    {
        RotateElement( nIndex, _FULL_ANGLE - 1, fTime, false, true, bMask, bMaskDirection );
    }

    /** obrot elementu */
    void RotateElement( int nIndex, float fAngle )
    {
        //if( m_fRotateGlobalTime > 0 )
        {
            RotateElement( nIndex, fAngle, m_fRotateGlobalTime, true, false, false, false );
        }
    }
    
    void RotateElement( int nIndex, float fAngle, boolean bMask, boolean bMaskDirection )
    {
        //if( m_fRotateGlobalTime > 0 )
        {
            RotateElement( nIndex, fAngle, m_fRotateGlobalTime, true, false, bMask, bMaskDirection );
        }
    }
    
    void RotateElement( int nIndex, float fAngle, float fTime )
    {
        RotateElement( nIndex, fAngle, fTime, false, false, false, false );
    }
    
    void RotateElement( int nIndex, float fAngle, float fTime, boolean bMask, boolean bMaskDirection )
    {
        RotateElement( nIndex, fAngle, fTime, false, false, bMask, bMaskDirection );
    }
    
    void RotateElement( int nIndex, float fAngle, float fTime, boolean bGlobal, boolean bContinuous, boolean bMask, boolean bMaskDirection )
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                ((PyramidElement)m_vElements.elementAt( nIndex )).RotateElement( fAngle, fTime, bGlobal, bContinuous, bMask, bMaskDirection );
            }
        }
    }

    /** ukrywanie/pokazywanie elementu */
    void SetElementVisible( int nIndex, boolean bVal )
    {
        if( m_vElements != null )
        {
            if( nIndex < m_vElements.size() && nIndex >= 0 )
            {
                ((PyramidElement)m_vElements.elementAt( nIndex )).SetVisible( bVal );
            }
        }
    }
    /** zaladowanie elementow ze skryptu */
    void LoadElements( int nElements, String sId )
    {
        m_nElements = nElements;
        
        m_sElementID = sId;
        
        if( m_nElements <= 0 || m_sElementID == null )
            return;
        
        for( int i = 0; i < m_nElements; i++ )
        {
            AddPyramidElement( (UIButton)GetElem( m_sElementID + i ) );
        }
    }
    /** dodanie elementu */
    private void AddPyramidElement( UIWindow cElem )
    {
        PyramidElement cElement = new PyramidElement();
        cElement.AddPyramidElement( cElem );
        
        m_vElements.addElement( cElement );
    }
    /** sprzatanie */
    public void OnDestory()
    {
        if( m_vElements != null )
        {
            if( m_vElements.size() > 0 )
            {
                for(int i = 0; i < m_vElements.size(); i++)
                {
                    ((PyramidElement)m_vElements.elementAt( i )).delete();
                }
            }
            m_vElements = null;
        }
    }
    
    ///////////////////////////////
    // ELEMENT PIRAMIDKI
    ///////////////////////////////
    /** element piramidki */
    public class PyramidElement
    {
        /** element piramidki */
        private UIWindow m_cElement = null;

        
        /** zmiana napisu na elemencie */
        void SetElementText( String sText )
        {
            if( m_cElement != null )
            {
                if( m_cElement instanceof UIStatic )
                {
                    ((UIStatic)m_cElement).SetText( sText );
                }
                else
                {
                    CrashLog("PyramidElement::SetElementText() - nie moge nic napisac, bo wybrany element nie jest instancja UIStatic");
                }
            }
        }
        
        /** powrot elementu do pozycji z ktorej 'przybyl' */
        void MoveElemOut( float fTime, float fPauseTime )
        {
            if( m_cElement != null )
            {
                m_cElement.SetEffectPosOut( fTime, fPauseTime );
            }
        }
        /** zmiana pozycji - animacja */
        void MoveElem( Vector cStartPos, Vector cDestPos, float fTime, float fPauseTime )
        {
            if( m_cElement != null )
            {
                if( cDestPos == null )
                    cDestPos = new Vector();
                if( cStartPos == null )
                    cStartPos = m_cElement.GetLogicPos();
                
                m_cElement.SetEffectDestPos( cStartPos, cDestPos, fTime, fPauseTime );
                //m_cElement.SetEffectDestPos( cStartPos, new Vector(SpeedwayGame.GetRandom(0.0f, 1.0f),SpeedwayGame.GetRandom(0.0f, 1.0f),0), fTime, fPauseTime );
            }
        }
        /** obrot elementu */
        private void RotateElement( float fAngle, float fTime, boolean bGlobal, boolean bContinuous, boolean bMask, boolean bMaskDirection )
        {
            float fDiff;
            if( m_cElement != null )
            {
                //fAngle %= _FULL_ANGLE;
                
                fDiff = Math.abs( m_cElement.GetRotation() - fAngle );
                if( bGlobal )
                {
                   fTime = ( fTime * fDiff ) / _FULL_ANGLE;
                }
                if (fTime != 0)
                    m_cElement.SetEffectDestAngle( fAngle, fTime, bContinuous, bMask, bMaskDirection );
                else
                {
                    m_cElement.SetAngleNoEffect(fAngle, bMask, bMaskDirection);
                    
                }
            }
        }

        /** zatrzymanie elementu */
        private void StopRotateElement()
        {
            if( m_cElement != null )
            {
                m_cElement.m_bEffectAngle = false;
            }
        }
        /** ukrywanie/pokazywanie elementu */
        private void SetVisible( boolean bVal )
        {
            if( m_cElement != null )
            {
                m_cElement.SetVisible( bVal );
            }
        }
        
        /** dodawanie elementu */
        void AddPyramidElement( UIWindow cElem )
        {
            m_cElement = cElem;
        }
        /** sprzatanie */
        public void delete()
        {
            if( m_cElement != null )
            {
                m_cElement = null;
            }
        }
        
        float GetRotation()
        {
            if( m_cElement != null )
                return m_cElement.GetRotation();
            else
                return 0;
        }
        
        UIWindow GetPyramidElem()
        {
            return m_cElement;
        }
    }
}
