public class HitIndicators extends GameObject
{
	static int _SPACE_SCREEN = 8;
	static int _SPACE_HUD = 62;
	
	static int _UP = 1;
	static int _RIGHT = 2;
	static int _DOWN = 4;
	static int _LEFT = 8;
	
	static float _DOT_TABLE[] = { 0.9238f,
								  0.3826f,
								  0f,
								  -0.3826f,
								  -0.9238f };
	
	static float _DISAPPEAR_SPEED = 2f;
    static float _DISAPPEAR_SPEED_OVERLOAD = 0.75f;
	static float _DISAPPEAR_SPEED_DROWNING = 0.6f;
	static int _NOF_HIT_IND = 4;
	
	PlayerHUD m_Player = null;
	
	String m_sHitTextureName = "CH_T_Cyber.png";
	Sprite m_HitIndSprites[] = new Sprite[_NOF_HIT_IND];
    Sprite m_cOverloadSprite = null;
    Sprite m_cDrowningSprite = null;
	int m_HitIndTextNo[] = new int[_NOF_HIT_IND];
    int m_nOverloadTexNo = 0;
    int m_nDrowningTexNo = 0;
    
	Vector m_vHitSpriteSize = new Vector(359f, 24f, 0f);
	float m_Alpha[] = new float[4];
    float m_fOverloadAlpha = 0;
    final static int _OVERLOAD_WAVES = 7;
    float m_fOverloadAlphaCurWave = 0;
    int m_nOverloadAlphaWaves = 0;
	
    float m_fDrowningAlpha = 0f;
    float m_fDrowningDestAlpha = -1;
    
	float m_fLastUpdate = 0;
	
    public void OnCreate()
    {
		int iRotation = 0;
		for (int i = 0; i < _NOF_HIT_IND; i++)
		{
			m_HitIndSprites[i] = (Sprite) CreateObject("Sprite");
			m_HitIndTextNo[i] = m_HitIndSprites[i].AddTexture(m_sHitTextureName);
			m_HitIndSprites[i].SetBlendingMode(EBlendingModes.S_BLEND_MIX);
			m_HitIndSprites[i].SetTexture(m_HitIndTextNo[i],
										  Rect.WHOLE_TEX);
			m_HitIndSprites[i].SetSize(m_vHitSpriteSize);
			m_HitIndSprites[i].SetRotation(iRotation);
			iRotation += 90;
			m_HitIndSprites[i].ProcessMouse(false);
			AddSprite(m_HitIndSprites[i]);
			m_HitIndSprites[i].MoveTop();
		}
		
        m_cOverloadSprite = (Sprite) CreateObject("Sprite");
        AddSprite(m_cOverloadSprite);
        m_cOverloadSprite.SetBlendingMode(EBlendingModes.S_BLEND_MIX);
        m_cOverloadSprite.SetSize(new Vector(GetScreenWidth(), GetScreenHeight(), 0));
        m_nOverloadTexNo = m_cOverloadSprite.AddTexture("er_t_black.png");
        m_cOverloadSprite.SetTexture(m_nOverloadTexNo, new Rect(0,0,1,1));
        m_cOverloadSprite.MoveTop();
        
		m_HitIndSprites[0].SetPos(new Vector((GetScreenWidth() - m_vHitSpriteSize.fX) / 2f,
											 _SPACE_SCREEN,
											 0f));
		m_HitIndSprites[1].SetPos(new Vector(GetScreenWidth() - m_vHitSpriteSize.fY -
											 m_vHitSpriteSize.fX / 2f - _SPACE_SCREEN,
											 GetScreenHeight() / 2f - m_vHitSpriteSize.fY / 2,
											 0f));
		m_HitIndSprites[2].SetPos(new Vector((GetScreenWidth() - m_vHitSpriteSize.fX) / 2f,
											 GetScreenHeight() - m_vHitSpriteSize.fY - _SPACE_HUD - _SPACE_SCREEN,
											 0f));
		m_HitIndSprites[3].SetPos(new Vector(_SPACE_SCREEN - m_vHitSpriteSize.fX / 2f,
											 GetScreenHeight() / 2f - m_vHitSpriteSize.fY / 2,
											 0f));
        
        m_cDrowningSprite = (Sprite) CreateObject("Sprite");
        AddSprite(m_cDrowningSprite);
        //m_cDrowningSprite.SetBlendingMode(EBlendingModes.S_BLEND_MUL);
        //m_cDrowningSprite.SetSpriteMaterial("sprite_blend_mul_walpha.mat");
        m_cDrowningSprite.SetSize(new Vector(GetScreenWidth(), GetScreenHeight(), 0));
        m_nDrowningTexNo = m_cDrowningSprite.AddTexture("er_t_purple.png");
        m_cDrowningSprite.SetTexture(m_nDrowningTexNo, new Rect(0,0,1,1));
        m_cDrowningSprite.MoveTop();
	}
	
    boolean m_bDisableInNetGame = false;
	public void SetPlayer(PlayerHUD player)
	{
		m_Player = player;
        if (m_Player instanceof PlayerNetBase)
        {
            m_bDisableInNetGame = ! m_Player.bNetIsOwner;
        }
	}
	
	public void Hit(Vector vShotPoint)
	{
		if (m_Player == null)
			return;
		
		if (!m_Player.HUD.m_bActivateAllowed)
			return;
        
        if (m_bDisableInNetGame)
            return;

		int iShow = CalculatePlace(vShotPoint);

		if ((iShow & _UP) > 0)
			ShowHit(0);
		if ((iShow & _RIGHT) > 0)
			ShowHit(1);
		if ((iShow & _DOWN) > 0)
			ShowHit(2);
		if ((iShow & _LEFT) > 0)
			ShowHit(3);
	}
	
	void ShowHit(int iNum)
	{
		m_HitIndSprites[iNum].Show();
		m_Alpha[iNum] = 1f;
		m_HitIndSprites[iNum].SetAlpha(m_Alpha[iNum]);
	}
	
    void ShowOverload()
    {
        m_cOverloadSprite.Show();
        m_fOverloadAlpha = 1f - RandomFloat(0.1f);
        m_cOverloadSprite.SetAlpha(m_fOverloadAlpha);
        m_nOverloadAlphaWaves = _OVERLOAD_WAVES;
        m_fOverloadAlphaCurWave = 0.2f + RandomFloat(0.33f);
    }
    
    void HideOverload()
    {
        m_cOverloadSprite.Hide();
    }

    void HideForMovie()
    {
        m_cOverloadSprite.Hide();
        m_cDrowningSprite.Hide();
    }
    
    void ShowDrowning()
    {
        m_cDrowningSprite.Show();
        m_fDrowningDestAlpha = 0.6f;
    }
    
    void HideDrowning()
    {
        //m_cDrowningSprite.Hide();
        m_fDrowningDestAlpha = 0;
    }
    
	public void OnFrame()
	{
		float fCurrTime = GetTime();

		DecreaseAlphas(fCurrTime - m_fLastUpdate);
		
		m_fLastUpdate = fCurrTime;
	}
	
	void DecreaseAlphas(float fTime)
	{
		for (int i = 0; i < _NOF_HIT_IND; i++)
		{
			if (m_Alpha[i] > 0f)
			{
				m_Alpha[i] -= fTime * _DISAPPEAR_SPEED;
				if (m_Alpha[i] < 0)
				{
					m_Alpha[i] = 0;
					m_HitIndSprites[i].Hide();
				}
				m_HitIndSprites[i].SetAlpha(m_Alpha[i]);
			}
		}
        
        if (m_fOverloadAlpha > 0f)
        {
            m_fOverloadAlpha -= fTime * _DISAPPEAR_SPEED_OVERLOAD;
            if (m_nOverloadAlphaWaves > 0 && m_fOverloadAlpha <= m_fOverloadAlphaCurWave)
            {
                m_nOverloadAlphaWaves --;
                m_fOverloadAlphaCurWave = 0.5f + RandomFloat(0.33f);
                m_fOverloadAlpha = m_fOverloadAlphaCurWave + RandomFloat(0.5f);
            }
                
            if (m_fOverloadAlpha < 0)
                m_fOverloadAlpha = 0;
            if (m_fOverloadAlpha > 1)
                m_fOverloadAlpha = 1;
            
            m_cOverloadSprite.SetAlpha(m_fOverloadAlpha);
        }
        
        if (m_fDrowningDestAlpha > 0f)
        {
            m_fDrowningAlpha += fTime * _DISAPPEAR_SPEED_DROWNING;
            if (m_fDrowningAlpha >= m_fDrowningDestAlpha)
                m_fDrowningAlpha = m_fDrowningDestAlpha;
            
            m_cDrowningSprite.SetAlpha(m_fDrowningAlpha);
        }
        else
        {
            if (m_fDrowningDestAlpha == 0)
            {
                m_fDrowningAlpha -= fTime * _DISAPPEAR_SPEED_DROWNING;
                if (m_fDrowningAlpha <= m_fDrowningDestAlpha)
                {
                    m_fDrowningAlpha = m_fDrowningDestAlpha;
                    m_fDrowningDestAlpha = -1;
                    m_cDrowningSprite.Hide();
                }
                
                m_cDrowningSprite.SetAlpha(m_fDrowningAlpha);
                
            }
        }
	}
	
	int CalculatePlace(Vector vFrom)
	{
		int iReturn = 0;
		Vector vDir = new Vector(vFrom);
		vDir.Sub(m_Player.GetPosition());
		vDir.fY = 0;
		vDir.Normalize();

		Vector vPlayerFwd = m_Player.GetForwardVector();
		vPlayerFwd.fY = 0f;
		vPlayerFwd.Normalize();
		
		float fDot = vPlayerFwd.Dot(vDir);
		float fCross = vPlayerFwd.Cross(vDir).fY;
		
		if (fDot > _DOT_TABLE[0])
			iReturn |= _UP;
		else
		if (fDot > _DOT_TABLE[1])
		{
			if (fCross < 0f)
				iReturn |= (_UP | _RIGHT);
			else
				iReturn |= (_UP | _LEFT);
		}
		else
		if (fDot > _DOT_TABLE[2])
		{
			if (fCross < 0f)
				iReturn |= (_RIGHT);
			else
				iReturn |= (_LEFT);
		}
		else
		if (fDot > _DOT_TABLE[3])
		{
			if (fCross < 0f)
				iReturn |= (_DOWN | _RIGHT);
			else
				iReturn |= (_DOWN | _LEFT);
		}
		else
			iReturn |= _DOWN;
		return iReturn;
	}
	
	public void OnDestroy()
	{
        for (int i = 0; i < m_HitIndSprites.length; i++)
		{
			if (m_HitIndTextNo[i] != 0)
				m_HitIndSprites[i].ReleaseTexture(m_HitIndTextNo[i]);
			RemoveSprite(m_HitIndSprites[i]);
			m_HitIndSprites[i].delete();
			m_HitIndSprites[i] = null;
		}
		
		m_HitIndSprites = null;
		
        m_cOverloadSprite.ReleaseTexture(m_nOverloadTexNo);
        RemoveSprite(m_cOverloadSprite);
        m_cOverloadSprite.delete();
        m_cOverloadSprite = null;
        
        m_cDrowningSprite.ReleaseTexture(m_nDrowningTexNo);
        RemoveSprite(m_cDrowningSprite);
        m_cDrowningSprite.delete();
        m_cDrowningSprite = null;
        
        //CrashLog("[HitIndicators.OnDestroy]: Destroing DONE. \n");
	}
	
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_HUD_MANAGER, cFCParent);
		
		super.SGSaveChunk(cFC);
        
        cFC.SaveFloat(m_fDrowningAlpha);
        cFC.SaveFloat(m_fDrowningDestAlpha);  
        
        cFC.delete(); 
    }
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
		super.SGLoadChunk(cFC);
        
        m_fDrowningAlpha = cFC.LoadFloat();
        m_fDrowningDestAlpha = cFC.LoadFloat();  
        
        cFC.delete();
        
    }
    
    
	public void SGUpdate()
	{
		super.SGUpdate();
		
        m_fLastUpdate = GetTime();
            
		for (int i = 0; i < _NOF_HIT_IND; i++)
		{
			m_Alpha[i] = 0;
			m_HitIndSprites[i].SetAlpha(m_Alpha[i]);
			m_HitIndSprites[i].Hide();
		}
		
		if (m_cOverloadSprite != null)
		{
			m_cOverloadSprite.SetAlpha(0);
			m_cOverloadSprite.Hide();
		}
        
        if (m_cDrowningSprite != null)
        {
            if (m_fDrowningAlpha > 0)
                m_cDrowningSprite.Show();
            else
                m_cDrowningSprite.Hide();
            
            m_cDrowningSprite.SetAlpha(m_fDrowningAlpha);
        }
	}
}
