/** Klasa odpowiadajaca za podpowiadanie graczowi
 */
public class HintManager extends GameObject
{
    // czas co jaki bedzie sie robil upadate
    final static float _CHECK_TIME        = 1.0f;
    // minimalny czas przerwy miedzy wyswietleniem jednego i drugiego hinta
    static float _MIN_DISPLAY_PAUSE = 5.0f;
    // czas opoznienia pierwszego hinta po starcie systemu
    static float _TIME_START_DELAY  = 5.0f;//45f;

    /** Flaga mowiaca czy system hintow jest aktywny
     */
    boolean m_bActive = false;
    
    // lista hintow czekajacych na obsluzenie
    java.util.Vector m_aHints = null;    
    // lista hintow juz obsluzonych i wycofanych
    java.util.Vector m_aRemovedHints = null;    
        
    /** Referencja do aktualnie wyswietlanego hinta
     * jesli null, to znaczy ze zaden hint nie jest aktywny
     */
    protected Hint  m_cActiveHint = null;
    
    /** Czas gry w ktorym moze byc wyswietleony nastepny hint
     */
    float m_fNextHintTime = 0.0f;
    
    
    /** Sprite z napisami
     */
    protected Sprite m_cText = null;
    
    /**Tlo dla tekstow*/
    protected Sprite m_cTextBkg = null;

    protected int m_nTextDownMargin = 150;
    
    /**
     * Miganie
     */
    
    /* czy napis ma migac */
    boolean m_bBlinking = true;
    
    /* czas pomiedzy pojawieniem sie a zniknieciem spritea */
    final static float _BLINKING_ON_TIME   = 0.15f;
    /* czas pomiedzy zniknieciem a pojawieniem spritea */
    final static float _BLINKING_OFF_TIME   = 0.15f;
    
    /* ilosc migniec */
    static int _BLINKING_CNT = 3;
    
    boolean m_bBlinkingOn = false;
    
    int m_nBlinkingCnt = 0;

    /** czy napis ma sie blendowac przy znikaniu */
    boolean m_bBlendingHide = false;
    
    /** czas blendowania */
    final static float _BLEND_TIME = 5.0f;
    /** czas rozpoczecia blendowania */
    float m_fBlendStartTime        = 0.0f;
    
    int METHOD_CheckHints     = 0;
    int METHOD_DeactivateHint = 0;
    int METHOD_BlinkText      = 0;
    int METHOD_BlendText      = 0;

	ChromeModule cModule = null;
	
    boolean m_bTutorial = false;
    
    void OnCreate()
    {
        m_cTextBkg = (Sprite)CreateObject("Sprite");
		//m_cText.SetNoTexture();
        m_cTextBkg.SetTexture(m_cTextBkg.AddTexture("er_t_black.png"), new Rect(0,0,1,1));
        m_cTextBkg.SetAlpha(0.4f);
		m_cTextBkg.ProcessMouse(false);
		m_cTextBkg.SetSize(new Vector(1024.0f, 768.0f,0));
		m_cTextBkg.SetPos(new Vector(10.0f, 455.0f, 0));
        AddSprite( m_cTextBkg );
        m_cTextBkg.MoveTop();
        
        m_cText = (Sprite)CreateObject("Sprite");
		m_cText.ProcessMouse(false);
        m_cText.SetTextColor( 1.0f, 1.0f, 1.0f, 1.0f );
		m_cText.SetSize(new Vector(1024.0f, 768.0f,0));
		m_cText.SetPos(new Vector(10.0f, 455.0f, 0));
        m_cText.Show();
        AddSprite( m_cText );        
        m_cText.MoveTop();
        
        m_cTextBkg.Hide();
        m_cText.Hide();

                
        m_aHints        = new java.util.Vector(0,8);
        m_aRemovedHints = new java.util.Vector(0,8);
        
        METHOD_CheckHints     = RegisterCallMethod("CheckHints");
        METHOD_DeactivateHint = RegisterCallMethod("DeactivateHint");
        METHOD_BlinkText      = RegisterCallMethod("BlinkText");
        METHOD_BlendText      = RegisterCallMethod("BlendText");
		
		cModule = (ChromeModule) GetModule();
    }
    
    void OnDestroy()
    {
        DisableCallMethods();
        
        if ( m_cText != null )
        {
            RemoveSprite(m_cText);
            m_cText.delete();
            m_cText = null;
        }
        if (m_cTextBkg != null)
        {
            RemoveSprite(m_cTextBkg);
            m_cTextBkg.delete();
            m_cTextBkg = null;
        }
    }
    
    
    /** Metoda wolana na stacie systemu,
     * powoduje zainicjowanie zegarow i zaplanowanie wolania
     * metody CheckHints co okreslony czas
     * @param bTutorialOn czy wlaczamy tutorial
     */
    void Start(boolean bTutorialOn)
    {
        m_bTutorial = bTutorialOn;
        m_fNextHintTime = GetTime() + _TIME_START_DELAY;
        CallMethodEvery( METHOD_CheckHints , _CHECK_TIME );
    }
    
    /** Metoda wolana po zabiciu playera,
     * powoduje deaktywacje systemu hintow
     */
    void Stop()
    {
        DisableCallMethod( METHOD_CheckHints );
        Deactivate();
    }
    
    /** Metoda wolana cyklicznie, i spawdzajaca czy czasem nie
     *  trzeba wyswietlic jakiegos hinta
     *  Uwaga: metoda ta moze byc zawolana tez "recznie", po
     *  wykonaniu jakies akcji
     */
    void CheckHints()
    {
		// jezeli odgrywamy wlasnie film, wychodzimy
		if (cModule.cMoviePlaying != null)
			return;
		
		// jezeli odgrywamy wlasnie gadke, wychodzimy
		if (cModule instanceof ChromeModuleSingle)
		{
			if ( ((ChromeModuleSingle) cModule).m_bPlayingScene )
				return;
		}
		
        if ( m_cActiveHint != null )
        {
            // jesli jakis hint aktualnie jest wyswietlany, to nie ma co kombinowac
            return;
        }
    
        if ( GetTime() <  m_fNextHintTime )
        {
            // jeszcze nie czas na nowego hinta
            return;
        }
        
        //przegladamy liste hintow, celem przeniesienia z jednej listy do drugiej
        CheckHintsToRemove();
        
        if ( m_bActive )
        {
            //przegladamy liste hintow, celem znalezienia takiego do wyswietlenia
            CheckHintsToActivate();
        }
    }
    
    /** Metoda aktywujaca hinta na ekranie, 
     * sprawdza z jakiego pliku wyswietlic tekst i na jak dlugo
     * po czym, wola metoda wykonawaqcza Show z odpowiednimy parametrami
     */
    void ActivateHint( Hint hint )
    {
        LogD("\n");
        if ( hint == null )
            return;
        
        m_cActiveHint = hint;
        
        ShowText( m_cActiveHint.GetText() );
        DisableCallMethod( METHOD_DeactivateHint );
        CallMethodIn( METHOD_DeactivateHint , m_cActiveHint.GetTimeDuration() );
        m_cActiveHint.DisplayStarted();
    }
    
    /** deaktywacja aktualnego hinta
     */
    void DeactivateHint()
    {
        LogD("\n");
        if ( m_cActiveHint == null )
            return;
        
        if (m_cActiveHint.m_fMinDisplayPause != -1)
            m_fNextHintTime = GetTime() + m_cActiveHint.m_fMinDisplayPause;
        else
            m_fNextHintTime = GetTime() + _MIN_DISPLAY_PAUSE;
        
        if ( m_bBlendingHide )
        {
            HideTextBlend();
            m_fNextHintTime += _BLEND_TIME;
        }
        else
            HideText();
        
        if ( m_cActiveHint.DisplayOnce() )
            m_cActiveHint.SetToRemove(); 
        else
            m_cActiveHint.ResolveNextCheckTimeAfterActivate( GetTime() );
        m_cActiveHint.DisplayFinished();
        m_cActiveHint = null;
    }
    
    /** deaktywacja aktualnego hinta
     */
    void CancelHint()
    {
        LogD("\n");
        if ( m_cActiveHint == null )
            return;
        
        m_fNextHintTime = GetTime() + _MIN_DISPLAY_PAUSE;
        
        if ( m_bBlendingHide )
        {
            HideTextBlend();
            m_fNextHintTime += _BLEND_TIME;
        }
        else
            HideText();
                
        m_cActiveHint.ResolveNextCheckTimeAfterActivate( GetTime() );
        m_cActiveHint.DisplayCanceled();
        m_cActiveHint = null;
    }


    /** Metoda wolana z zewnatrz, pozwalajaca na deaktywacje hinta
     */
    void Deactivate()
    {
        LogD("\n");
        
        if (!m_bActive)
            return;
        
        m_bActive = false;
        
        if ( m_cActiveHint == null )
            return;
 
        HideText();
    }
    
    /** Metoda wolana z zewnatrz, pozwalajaca na ponowna aktywacja hinta
     */
    void Activate()
    {
        LogD("\n");
        
        if (m_bActive)
            return;
        
        m_bActive = true;
        
        if ( m_cActiveHint == null )
            return;
        
        ShowText( m_cActiveHint.GetText() );
    }
    
    /** Wyswietlenie spritea z tekstem
     */
    protected void ShowText(String sText)
    {
        LogR("HintManager.ShowText <"+ sText +"> hint " + m_cActiveHint + "\n");
        if ( m_cText == null )
            return;
        
        m_cText.SetFont( ChromeFonts.FONT_HINT );
        m_cText.SetTextPosFlags(ESpriteText.MULTILINE);
        int nTextLeftMargin = GetScreenWidth()/4;
        //m_nTextDownMargin = GetScreenHeight()/2;
            
        int nSizeX = GetScreenWidth()/3;// - 2 * nTextLeftMargin;
        int nSizeY = GetScreenHeight();
        
        //musimy pobrac faktyczny tekst, bo w chwili obecnej w sText mamy
        //tylko jego _identyfikator_ !!!
        //TODO:jak zostanie to poprawione na poziomie Sprite, to tu usunac!
        /*String sTextJustify = m_cText.GetTextJustify(Text.Get( sText ), nSizeX);
        sTextJustify = m_cText.GetTextJustify(Text.Get( sText ), nSizeX);*/
        m_cText.SetSize(new Vector(nSizeX, nSizeY, 0));
        m_cText.DisplayText(Text.Get( sText ));
        
        Vector vMargin = new Vector(15,10,0);
        
        Vector vTextSize = m_cText.GetTextExtent(sText);
        //vTextSize.fX += 2f * vMargin.fX;
        //vTextSize.fY += 2f * vMargin.fY;
        m_cText.SetSize(vTextSize);
        m_cText.DisplayText(sText);
        
        //m_cText.DisplayTextFitSizeWithMargin(sTextJustify, vMargin);
        
        //z lewej pod celownikiem
        Vector vPos = new Vector ( 30,
                                   GetScreenHeight() * 2f/3f - m_cText.GetSize().fY/2f,
                                   0);
        m_cText.SetPos( vPos );
        
        Vector vBkSize = vTextSize.Add2(vMargin.Mul2(2));
        Vector vBkPos = vPos.Sub2(vMargin);

        m_cTextBkg.SetSize(vBkSize);
        m_cTextBkg.SetPos(vBkPos);
        
        m_cTextBkg.MoveTop();
        m_cTextBkg.Show();
        m_cText.MoveTop();
        m_cText.Show();
        
        
        
        if ( m_bBlinking )
            InitBlinking();
    }
    
    void InitBlinking()
    {
        m_nBlinkingCnt = _BLINKING_CNT;
        m_bBlinkingOn  = true;
        if ( m_nBlinkingCnt > 0 )
            CallMethodIn( METHOD_BlinkText ,_BLINKING_ON_TIME );
    }
    
    
    void BlinkText()
    {
        if (m_bBlinkingOn)
        {
            m_cTextBkg.Hide();
            m_cText.Hide();
            
            CallMethodIn( METHOD_BlinkText ,_BLINKING_OFF_TIME );
        }
        else
        {
            m_cTextBkg.Show();
            m_cText.Show();
            
            if ( --m_nBlinkingCnt > 0)
                CallMethodIn( METHOD_BlinkText ,_BLINKING_ON_TIME );
        }
        
        m_bBlinkingOn = !m_bBlinkingOn;
    }
    
    /** Schowanie spritea z tekstem
     */
    void HideText()
    {
        LogD("\n");
        
        m_cTextBkg.Hide();
        m_cText.Hide();
        // po schowaniu musimy wylaczyc miganie
        DisableCallMethod( METHOD_BlinkText );
        // i blendowanie
        DisableCallMethod( METHOD_BlendText );
    }
    
        
    void HideTextBlend()
    {
        m_fBlendStartTime = GetTime();
        CallMethodIn( METHOD_BlendText, 0.0f );
    }
    
    void BlendText()
    {
        float fCoeff = (GetTime()-m_fBlendStartTime)/_BLEND_TIME;
        
        if (fCoeff >= 1.0f)
        {
            float fAlpha = 1.0f - (fCoeff * 0.5f);
            LogD(" "+GetTime()+" "+fAlpha+"\n");
                
            if (m_cText != null)
                m_cText.SetAlpha( fAlpha );   
            DisableCallMethod( METHOD_BlendText );
            CallMethodIn( METHOD_BlendText, 0.0f );
        }
        else
        {
            HideText();
        }
    }
    
    
    /**
     * System hintow w managerze
     * 
     */
    
    /** Dodanie hinta do managera
     */
    void AddHint( Hint hint )
    {
        if ( hint == null )
            return;
        
        if ( m_aHints == null )
            return;
        
        m_aHints.addElement( hint );
        hint.SetHintManager( this );
    }
    
    /** Usuniecie hinta z managera
     */
    void RemoveHint( Hint hint )
    {
        if ( hint == null )
            return;
        
        if ( m_aHints == null )
            return;
        
        m_aHints.remove( hint );
    }
      
    /** Przeniesienie hinta do listy juz obsluzonych
     * do zastosowania w przyszlosci
     */
    void MoveHintToRemoved( int nIndex )
    {
        if ( m_aHints == null )
            return;

        if ( m_aHints.size() <= nIndex )
            return;
        
        Hint hint = (Hint)m_aHints.elementAt( nIndex );
                 
        m_aHints.remove( nIndex );
                
        if ( m_aRemovedHints == null )
            return;
        
        m_aRemovedHints.addElement( hint );
    }
    
    /** Sprawdzenie, czy nie ma jakis hintow do wyswietlenia
     */
    void CheckHintsToActivate()
    {
        if ( m_aHints == null )
            return;

        float fTimeNow = GetTime();
        
        // hint przeznaczony do wyswietlenia        
        Hint cHint  = null;
        
        for ( int i=0; i<m_aHints.size(); i++)
        {
            Hint hint = (Hint)m_aHints.elementAt( i );
            
            if ( hint == null )
                continue;
            
            if ( !hint.Available() || fTimeNow < hint.GetNextCheckTime() )
                continue;
             
            hint.ResolveNextCheckTime( fTimeNow );
                   
            // jest hint przeznaczony do wyswietlenia
            if ( hint.ShouldBeActivated( fTimeNow ) )
            {
                if ( cHint == null )
                {
                    // to jest pierwszy hint do wyswietlenia, zapamietajmy go
                    cHint  = hint;
                    
                    // nie analizujemy identyfikatorow jako priorytetow
                    // w sytuacjach konfliktowych istotne jest kolejnosc dodawania
                    // do hint managera
                    break;
                }
                else 
                {
                    // sprawdzmy jego priorytet, w porownaniu z tym poprzednio zapamietanym
                    if ( cHint.GetId() > hint.GetId() )
                    {
                        // ok, ten jest "lepszy" niz ten poprzedni zapamietajmy go
                        cHint  = hint;
                    }    
                }
            }
        }
        
        if ( cHint != null )
        {
            cHint.ResolveNextCheckTime( fTimeNow );
            ActivateHint( cHint );
        }
    }
    
    /** Sprawdzenie, czy nie ma jakis hintow do przeniesienia do listy obsluzonych
     */
    void CheckHintsToRemove()
    {
        if ( m_aHints == null )
            return;
        
        if ( m_aRemovedHints == null )
            return;
        
        for ( int i=0; i<m_aHints.size(); i++ )
        {
            Hint hint = (Hint)m_aHints.elementAt( i );
            
            if ( hint == null )
                continue;
            
            if ( hint.GetToRemove() )
            {
                MoveHintToRemoved( i );
                i--;
            }
        }
    }
        
 
    /** 
     * System komunikatow przychdzacych z modulu 
     * Komunikaty powinny byc wolana po okreslonych akcjach
     * 
     */
    
    final static int _MSG_MAPUSED        = 0;
    final static int _MSG_BINOCULARSUSED = 1;
    final static int _MSG_JEEPUSED       = 2;
    final static int _MSG_TAKENDAMAGE    = 3;
    final static int _MSG_HEALTHEXUSED   = 4;
    final static int _MSG_NOAMMO         = 5;
    final static int _MSG_WEAPONRELOADED = 6;
    final static int _MSG_IMPLANTSUSED   = 7;
    final static int _MSG_DEAD_BODY      = 8;
	
    final static int _MSG_TUTORIAL             = 101;
    final static int _MSG_TUTORIAL_STEP1_ENDED = 101;
    final static int _MSG_TUTORIAL_STEP2_ENDED = 102;
    final static int _MSG_TUTORIAL_STEP4_ENDED = 104;
    final static int _MSG_TUTORIAL_STEP5_ENDED = 105;
    final static int _MSG_TUTORIAL_STEP6_ENDED = 106;
    final static int _MSG_TUTORIAL_STEP7_ENDED = 107;
    final static int _MSG_TUTORIAL_STEP8_ENDED = 108;
    final static int _MSG_TUTORIAL_STEP9_ENDED = 109;
    final static int _MSG_TUTORIAL_STEP10_ENDED = 110;
    final static int _MSG_TUTORIAL_STEP11_ENDED = 111;
    final static int _MSG_TUTORIAL_STEP12_ENDED = 112;
    final static int _MSG_TUTORIAL_STEP13_ENDED = 113;
    final static int _MSG_TUTORIAL_STEP14_ENDED = 114;
    final static int _MSG_TUTORIAL_STEP15_ENDED = 115;
 
    /** Obsluga przychodzacego komunikatu
     */
    void IncomingMessage(int nId)
    {
        //zanim nie skonczy sie tutorial nie przyjmujemy innych message'y
        if (m_bTutorial && nId < _MSG_TUTORIAL)
            return;
        // wyslijmy wszystkim komunikat
        for ( int i=0; i<m_aHints.size(); i++)
        {
            Hint hint = (Hint)m_aHints.elementAt( i );
            
            if ( hint == null )
                continue;
            
            hint.IncomingMessage( nId );
        }
        
        // sprawdzmy czy czasem ten aktualnie wyswietlany nie powinien sie schowac
        if ( m_cActiveHint != null && m_cActiveHint.GetToRemove() )
        {
            CancelHint();
        }
        
        // sprawdzmy czy nie trzeba czasem czegos wyswietlic
        CheckHints();
    }
    
    void MoveTop()
    {
        m_cTextBkg.MoveTop();       
        m_cText.MoveTop();
    }
    
    void Clear()
    {
        HideText();
        
        if (m_cActiveHint != null)
        {
            m_cActiveHint.SetToRemove(); 
        
            m_cActiveHint.DisplayFinished();
            m_cActiveHint = null; 
        }

        if ( m_aRemovedHints != null )
            m_aRemovedHints.removeAllElements();
        if ( m_aHints != null )
            m_aHints.removeAllElements();
        
        ClearTime();
        DisableCallMethods();
        
    }
    
    void ClearTime()
    {
        m_fNextHintTime = GetTime() + _TIME_START_DELAY;
        DisableCallMethod(METHOD_CheckHints);
        CallMethodEvery(METHOD_CheckHints, _CHECK_TIME);
        for ( int i=0; i<m_aHints.size(); i++)
        {
            Hint hint = (Hint)m_aHints.elementAt( i );
            
            if ( hint == null )
                continue;
            
            hint.ResolveNextCheckTime( GetTime() );
        }
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean m_bActiveSG = false;
	Hint m_cActiveHintSG = null;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	        
		FileChunk.Log("[HintManager.SGLoadChunk]\n");						
		
		Deactivate();
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_CHROMEMODULE_SINGLE)
        {	
			super.SGLoadChunk(cFC);

			m_bActiveSG = cFC.LoadBool();

			m_aHints = cFC.LoadObjectsVectorRef();
			m_aRemovedHints = cFC.LoadObjectsVectorRef();
			
			for (int i=0; i < m_aHints.size(); i++)
				((Hint)m_aHints.elementAt(i)).SetHintManager(this);
			
			for (int i=0; i < m_aRemovedHints.size(); i++)
				((Hint)m_aRemovedHints.elementAt(i)).SetHintManager(this);			
		
			m_cActiveHint = (Hint)cFC.LoadObjectRef();
		
			m_fNextHintTime = cFC.LoadFloat();
				
			m_bBlinking = cFC.LoadBool();
			m_bBlinkingOn = cFC.LoadBool();
	    
			m_nBlinkingCnt = cFC.LoadInt();
    
			m_bBlendingHide = cFC.LoadBool();
    
			m_fBlendStartTime = cFC.LoadFloat();
    
			m_bTutorial = cFC.LoadBool();							
            
            _MIN_DISPLAY_PAUSE = cFC.LoadFloat();
            _TIME_START_DELAY = cFC.LoadFloat();
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[HintManager.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_CHROMEMODULE_SINGLE, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveBool(m_bActive);

		cFC.SaveObjectsVectorRef(m_aHints);
		cFC.SaveObjectsVectorRef(m_aRemovedHints);
		
		cFC.SaveObjectRef(m_cActiveHint);
		
		cFC.SaveFloat(m_fNextHintTime);
			
		cFC.SaveBool(m_bBlinking);
		cFC.SaveBool(m_bBlinkingOn);
	    
		cFC.SaveInt(m_nBlinkingCnt);
    
		cFC.SaveBool(m_bBlendingHide);
    
		cFC.SaveFloat(m_fBlendStartTime);
    
		cFC.SaveBool(m_bTutorial);				
        
        cFC.SaveFloat(_MIN_DISPLAY_PAUSE);
        cFC.SaveFloat(_TIME_START_DELAY);
        
        cFC.delete();
    }		
	
	public void SGUpdate()
	{
		super.SGUpdate();		
		
		if (m_bActiveSG)					
			Activate();						
	}
	
}