/** klasa - kontener odpowiedzialna za przechowywanie i logike implantow
 */
public class ImplantSystem extends GameObject
{
    final static String _IMPLANTS_FILENAME = "data/implants.scr";
	/** dzwiek odgrywany przy wylaczeniu implantow */
	final public static String _IMPLANTS_OUT_SOUND = "Data/Sounds/Menu/implants_out.wav";

    final static float _OVERLOAD_WARNING_THRESHOLD = 80;
    final static float _OVERLOAD_DAMAGE = 50;

    final static String _OVERLOAD_SOUND = "data/sounds/Ai/DefaultSoldier/shot 2.wav";
    
    boolean m_bConstantOverload = false;
    
    /**ID sa przyznawane przy ladowaniu skryptu
     */
    public static int   _IMPLANT_SCOPED_EYES            = -1, 
                        _IMPLANT_DERMAL_ARMOR		    = -1,
                        _IMPLANT_TENDO_AMPLIFY          = -1,
                        _IMPLANT_TARGETING_ASSISTANCE   = -1,
                        _IMPLANT_MUSCLE_IMPROVER        = -1,
                        _IMPLANT_TERMOVISION_SYSTEM     = -1,
                        _IMPLANT_SLOMO                  = -1,
                        _IMPLANT_HACK                   = -1;
    
    private static String _IMPLANT_NAMES[] = null;
    
	public static int _IMPLANTS_NUM = 7;
	
	Implant m_Implants[] = null;
    
    Implant m_tImplantsFreezed[] = null;
	
	boolean m_bIsWorking = false;
	
	float m_fSystemOverload = 0;
    
	float m_fNaturalOverloadReduction = 1f;
	
	float m_fLastUpdateTime = 0;
	float m_fLastUpdateTimeInterval = 0;
	float m_fLastUpdateOverloadChange = 0;

	boolean m_bSystemOverload = false;
	
	HUDCompPlayer m_HUDOverload = null;
	
    final static float _UPDATE_INTERVAL = 0.25f;
    
    int METHOD_Update = -1;
	
	public void StartUpdate()
	{
        if (METHOD_Update == -1)
            METHOD_Update = RegisterCallMethod("Update");
        
        m_bIsWorking = true;
		m_fLastUpdateTime = GetTime();
		CallMethodEvery(METHOD_Update, _UPDATE_INTERVAL);
	}
	
	public void StopUpdate()
	{
        if (METHOD_Update == -1)
            METHOD_Update = RegisterCallMethod("Update");
        
        m_bIsWorking = false;
		DisableCallMethod(METHOD_Update);
	}
	
	public void Update()
	{
		Update(GetTime());
	}
    
    ImplantSystem()
	{
		bSG = false;
	}
    
    Implant[] GetImplants()
    {
        return m_Implants;
    }
    
	int GetImplantNumByClass(String name)
	{
		for (int i = 0; i < _IMPLANTS_NUM; i++)
        {
			if (name.equalsIgnoreCase(m_Implants[i].getClass().getName()))
				return i;
		}
        //Tools.CrashLogError("IMPLANT " + name + " NOT FOUND");
        //GameObject.printStackTrace("");
		return -1;
	}
	
	Implant GetImplant(int iNum)
	{
		if ( (iNum < 0) || (iNum >= _IMPLANTS_NUM) )
			return null;
        //TODO: tymczasowe dodatkowe zabezpieczenie, bo w sieci lecialy Exception'y
        if ( m_Implants != null && iNum < m_Implants.length )
		    return m_Implants[iNum];
        else
        {
            return null;
        }
              
	}
	
    /**Zwraca nazwe implantu o danym numerze, jesli jeszcze nie ma implantow to odczytuje
     * dane ze skryptu (zapisuje tylko nazwy)*/
    static String GetImplantName(int nImplant)
    {
        if (_IMPLANT_NAMES == null)
        {
            ParseKeyword Root = GameObject.ParseFile(_IMPLANTS_FILENAME);
        
            java.util.Vector vTempImpl = new java.util.Vector();
            Implant cTmp = null;
            java.util.Vector vImplantNames = new java.util.Vector();
            if(Root == null)
            {
                GameObject.CrashLog("No file "+ _IMPLANTS_FILENAME +"!\n");
            }
            else
            {
                ParseKeyword Key, SubKey;
                for (Key = Root; Key != null; Key = Key.GetNext())
                {
                    if(Key.Equals("Implant") ||
                       Key.Equals("ImplantSP"))
                    { 
                        vImplantNames.addElement((Object)Key.GetSParam(1));
                    }
                }
            }
            
            if (Root != null)
                Root.delete();
            
            _IMPLANT_NAMES = new String[vImplantNames.size()];
            for (int i = 0; i < vImplantNames.size(); i++)
                _IMPLANT_NAMES[i] = (String)vImplantNames.elementAt(i);
        }
        
        if (nImplant >= _IMPLANT_NAMES.length)
            return null;
        else
            return _IMPLANT_NAMES[nImplant];
    }
    
    static ParseKeyword ImplantScript = null;
    
	void InitImplants(Module mod, Player player)
	{
        if(ImplantScript == null)
            ImplantScript = GameObject.ParseFile(_IMPLANTS_FILENAME);
        
        ParseKeyword Root = ImplantScript;
        
        java.util.Vector vTempImpl = new java.util.Vector();
        Implant cTmp = null;
        if(Root == null)
        {
            GameObject.CrashLog("No file "+ _IMPLANTS_FILENAME +"!\n");
        }
        else
        {
            ParseKeyword Key, SubKey;
            for (Key = Root; Key != null; Key = Key.GetNext())
            {
                if(Key.Equals("Implant") ||
                  (Key.Equals("ImplantSP") && (mod instanceof ChromeModuleSingle)) )
                {
                    //cTmp = (Implant) mod.CreateObject(Key.GetSParam(0));
                    String sImplantClass = Key.GetSParam(0);
                    cTmp = (Implant) mod.GetObjectOfClass(sImplantClass);
                    cTmp.cModule = (ChromeModule)mod;
                    cTmp.OnCreate();
                    cTmp.Load(Key);
                    vTempImpl.addElement(cTmp);
                    //ustalamy id dla implantu
                    UpdateIDs(cTmp.getClass().getName(), vTempImpl.size() - 1);
                }
                if (Key.Equals("OverloadRegeneration"))
                {
                    m_fNaturalOverloadReduction = Key.GetFParam(0);
                }
            }
        }
        
        _IMPLANTS_NUM = vTempImpl.size();
        
        //GameObject.CrashLog("Creating implants");
        m_Implants = new Implant[_IMPLANTS_NUM];
		for (int i = 0; i < _IMPLANTS_NUM; i++)
        {
            m_Implants[i] = (Implant) vTempImpl.elementAt(i);
            m_Implants[i].SetPlayer(player);
        }
        
            
        
	}
	
    /**Metoda updejtuje id implantu o podanej klasie, id zalezy od kolejnosci wczytywania 
     * ze skryptu*/
    void UpdateIDs(String sImplantClass, int nID)
    {

        if (sImplantClass.equals("ImplantZoom"))
            _IMPLANT_SCOPED_EYES = nID; 
        else
            if (sImplantClass.equals("ImplantArmor"))
                _IMPLANT_DERMAL_ARMOR = nID; 
            else
                if (sImplantClass.equals("ImplantSpeed"))
                    _IMPLANT_TENDO_AMPLIFY = nID; 
                else
                    if (sImplantClass.equals("ImplantAccuracy"))
                        _IMPLANT_TARGETING_ASSISTANCE = nID; 
                    else
                        if (sImplantClass.equals("ImplantStrength"))
                            _IMPLANT_MUSCLE_IMPROVER = nID; 
                        else
                            if (sImplantClass.equals("ImplantTermovision"))
                                _IMPLANT_TERMOVISION_SYSTEM = nID; 
                            else
                                if (sImplantClass.equals("ImplantSlomo"))
                                    _IMPLANT_SLOMO = nID; 
                                else
                                    if (sImplantClass.equals("ImplantHack"))
                                        _IMPLANT_HACK = nID; 
        else
        {
            Tools.CrashLogError("UNKNOWN IMPLANT CLASS. PLEASE UPDATE ImplantSystem.UpdateIDs METHOD");
            GameObject.printStackTrace("");
        }
                        		  
    }
    
	public void Update(float fNow)
	{
		m_fLastUpdateTimeInterval = fNow - m_fLastUpdateTime;
		m_fLastUpdateTime = fNow;
		UpdateImplants(m_fLastUpdateTimeInterval);
        ApplyChanges();
        UpdateHud();
	}
    
	void UpdateImplants(float fTime)
	{
		if (!m_bIsWorking)
			return;
		if (m_Implants == null)
			return;

        int nActiveImplantsCount = GetActiveImplantsCount();
        
		float fOverloadToAdd = 0;
		
		for (int i = 0; i < _IMPLANTS_NUM; i++)
        {
			fOverloadToAdd += m_Implants[i].Update( fTime );
        }

        if (nActiveImplantsCount > 0 && this instanceof NetImplantSystem)
        {
            //pierwiastek 3 stopnia
            fOverloadToAdd /= (float)java.lang.Math.pow(nActiveImplantsCount, 1.0/3.0);
         }
        
        /*if (fOverloadToAdd > 0)
    		m_fLastUpdateOverloadChange = fOverloadToAdd;
        else*/
        
        if (!m_bConstantOverload)
        {
            if (nActiveImplantsCount > 0)
                m_fLastUpdateOverloadChange = fOverloadToAdd - fTime * m_fNaturalOverloadReduction;
            else
            {
                //zaden implant nie jest w uzyciu
                if (this instanceof NetImplantSystem)
                    m_fLastUpdateOverloadChange = fOverloadToAdd - fTime * 7f * m_fNaturalOverloadReduction;
                else
                    m_fLastUpdateOverloadChange = fOverloadToAdd - fTime * 3f * m_fNaturalOverloadReduction;
            }
            
            m_fSystemOverload += m_fLastUpdateOverloadChange;
        }        
        
	}
    
    void ApplyChanges()
    {
        if (m_fSystemOverload >= 100.0f)
			Overload();
        else
        {
            m_bSystemOverload = false;
            if (m_fSystemOverload < 0.0f)
			    m_fSystemOverload = 0.0f;
        }
    }
	
    void UpdateHud()
    {
        if (m_HUDOverload != null && 
            GetPlayer() != null && 
            GetPlayer().bNetIsOwner)
		{
			m_HUDOverload.SetOverloadBelt((int) m_fSystemOverload);
			m_HUDOverload.SetTextOverload((int) m_fSystemOverload);
		}
    }
    
    void UpdateHudIndicators()
    {
        if ( GetPlayer() != null && 
             GetPlayer().bNetIsOwner )
        {
            GetPlayer().HUD.GetHUDImplantIndicators().SetActiveImplants(m_Implants);
        }
    }

        
    public void Update(float fNow,float fOverload)
	{
	}
    
    void Overload()
    {
        if (GetPlayer() == null || (!GetPlayer().bNetIsOwner && Role != ERole._AUTHORITY))
            return;
        
        m_fSystemOverload = 100f;
        
        DeactivateAllImplants();
        
        if ( m_bSystemOverload )
            return;
        m_bSystemOverload = true;
        
        if ( GetPlayer().Role == ERole._AUTHORITY )
            GetPlayer().TakeDamage(_OVERLOAD_DAMAGE, 0, GetPlayer(), null, null, EBones._NON_BONE, EnumDamageType._BULLET);
        
        if (GetPlayer().bNetIsOwner)
        {
            GetPlayer().PlaySound(_IMPLANTS_OUT_SOUND);
            GetPlayer().TextMessage(Text.Get("&SYSTEM_OVERLOAD&"));
        
            GetPlayer().AttachOneShotSound(_OVERLOAD_SOUND);
        
            if (GetPlayer().HUD.m_HitIndicators != null)
                GetPlayer().HUD.m_HitIndicators.ShowOverload();
            GetPlayer().HUD.GetHUDImplantIndicators().SetActiveImplants(m_Implants);
            if (GetPlayer().HUD.GetHUDHack() != null)
                GetPlayer().HUD.GetHUDHack().Overload();
        }
        
    }
    
    void ConstantOverload()
    {
        Overload();
        m_bConstantOverload = true;
    }
    
    java.util.Vector vHints = new java.util.Vector();
	void ApplyImplantChanges(java.util.Vector implantModifiers)
	{
        vHints.removeAllElements();
		if (implantModifiers == null)
			return;
		if (m_Implants == null)
			return;
		
		ImplantModifier implantModifier = null;
		for (int i = 0; i < implantModifiers.size(); i++)
		{
			implantModifier = (ImplantModifier) implantModifiers.elementAt(i);
			
			int iImplantID = GetImplantNumByClass(implantModifier.m_sClassName);
			
			if (iImplantID != -1)
            {
				m_Implants[iImplantID].SetAssimilation(implantModifier.m_fChange);
                //jesli trzeba to ustatwiamy hinta
                if (implantModifier.m_bNew)
                    m_Implants[iImplantID].m_bShowHint = true;
            }
		}
        
        UpdateHudIndicators();
        
        for (int i = 0; i < _IMPLANTS_NUM; i++ )
        {
            if (m_Implants[i].m_bShowHint)
                vHints.addElement(new Integer(m_Implants[i].m_nHint));
        }
        
        if (vHints.size() > 0 && 
            GetModule() instanceof ChromeModuleSingle && 
            ((ChromeModuleSingle)GetModule()).m_bImplantsAvailable)
        {
            ((ChromeModuleSingle)GetModule()).InitImplantHints(vHints);
        }
	}

    void OnInstantActionEnableImplants()
    {
        GetPlayer().TextMessage("&IMPLANTS_CHANGED&");
        ((HUDManager)GetPlayer().HUD).GetHUDImplantIndicators().Activate();
        UpdateHudIndicators();
        ((ChromeModuleSingle)GetModule()).AddImplantHints(vHints);
    }
    
    float GetSystemOverload()
	{
		return m_fSystemOverload;
	}
	
	float GetSystemTimeToShutDown()
	{
		if (m_fLastUpdateOverloadChange > 0)
			return m_fLastUpdateTimeInterval * (100 - m_fSystemOverload) / m_fLastUpdateOverloadChange;
		else
			return 0;
	}
	
	public void SetHUDToUpdate(HUDCompPlayer hud)
	{
		m_HUDOverload = hud;
	}
	
	boolean IsImplantActive(int iNum)
	{
		if (iNum < 0 || iNum >= _IMPLANTS_NUM)
			return false;
		if ( (m_Implants == null) || (m_Implants[iNum] == null) )
			return false;
		return m_Implants[iNum].IsActive();
	}
    
    //MK: z jakiej racji tu zwraca MAinPlayer'a?
    PlayerHUD GetPlayer()
    {
        return (PlayerHUD)((ChromeModule)GetModule()).GetMainPlayer();
    }
    
    boolean CanActivateImplants()
    {
        return (m_fSystemOverload < _OVERLOAD_WARNING_THRESHOLD && m_bIsWorking);
    }
    
    void Clean()
    {
        m_fSystemOverload   = 0.0f;        
        for (int i = 0; i < m_Implants.length; i++)
            m_Implants[ i ].Deactivate();
        UpdateHud();
        UpdateHudIndicators();
    }
    
    int GetActiveImplantsCount()
    {
        int nActive = 0;
        for (int i = 0; i < _IMPLANTS_NUM; i++)
            if (IsImplantActive(i))
                nActive++;
        
        return nActive;
    }
    
    /**Freezuje implanty np. na czas filmu*/
    void FreezeImplants()
    {
        m_tImplantsFreezed = new Implant[m_Implants.length];
        for (int i = 0; i < m_tImplantsFreezed.length; i++)
        {
            if (m_Implants[i].IsActive())
            {
                m_tImplantsFreezed[i] = m_Implants[i];
                m_tImplantsFreezed[i].Deactivate();
            }
            m_Implants[i].m_bImplantSystemWorking = false;
        }
        m_bIsWorking = false;
    }
    
    /**Odfrizowuje implanty (przywraca sfreezowane metoda FreezeImplants)*/
    void UnFreezeImplants()
    {
        if (Precache.bPrecaching)
        {
            CallMethodIn("UnFreezeImplants", 0.1f);
            return;
        }
        
        if (m_tImplantsFreezed == null)
            return;
        
        for (int i = 0; i < m_Implants.length; i++)
        {
            m_Implants[i].m_bImplantSystemWorking = true;
            if (m_tImplantsFreezed[i] != null)
                m_tImplantsFreezed[i].Activate();
        }
        m_tImplantsFreezed = null;
        m_bIsWorking = true;
        
        UpdateHud();                
        UpdateHudIndicators();
    }
    
    void DeactivateAllImplants()
    {        
        for (int i = 0; i < m_Implants.length; i++)
        {
            if (m_Implants[i] != null)
                m_Implants[i].Deactivate();
        }
    }
    
    public void OnDestroy()
    {
        
		if (m_Implants != null)
			for (int i = 0; i < _IMPLANTS_NUM; i++)
			{
				if (m_Implants[i] != null)
				{
					m_Implants[i].OnDestroy();
					m_Implants[i] = null;
				}
			}
		m_Implants = null;
        _IMPLANTS_NUM = 0;
        _IMPLANT_SCOPED_EYES          = -1;
        _IMPLANT_DERMAL_ARMOR		  = -1;
        _IMPLANT_TENDO_AMPLIFY        = -1;
        _IMPLANT_TARGETING_ASSISTANCE = -1;
        _IMPLANT_MUSCLE_IMPROVER      = -1;
        _IMPLANT_TERMOVISION_SYSTEM   = -1;
        _IMPLANT_SLOMO                = -1;
        _IMPLANT_NAMES = null;
    }
    
    void Cheat()
    {
        for (int i = 0; i < _IMPLANTS_NUM; i++)
        {
            if (m_Implants[i] != null)
                m_Implants[i].Masta();
        }
    }
    
    void Cheat2()
    {
        for (int i = 0; i < _IMPLANTS_NUM; i++)
        {
            if (m_Implants[i] != null && 
                !m_Implants[i].m_bDisabled && !m_bConstantOverload)
                m_Implants[i].Masta();
        }
        
    }
    
    void Cheat3()
    {
        if (!m_bConstantOverload)
        {
            m_fSystemOverload = 0;
            UpdateHud();                
            UpdateHudIndicators();
        }
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[ImplantSystem.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_OBJECT_IMPLANT_SYSTEM)
        {	
			m_fSystemOverload = cFC.LoadFloat();
            m_bIsWorking = cFC.LoadBool();
            m_bSystemOverload = cFC.LoadBool();
            m_bConstantOverload = cFC.LoadBool();
            
            m_tImplantsFreezed = new Implant[m_Implants.length];
        
            for (int i = 0; i < _IMPLANTS_NUM; i ++)
                if (cFC.LoadBool())
                    m_tImplantsFreezed[i] = m_Implants[i];
            
            
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[ImplantSystem.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_OBJECT_IMPLANT_SYSTEM, cFCParent);
		
		cFC.SaveFloat(m_fSystemOverload);
        cFC.SaveBool(m_bIsWorking);
        cFC.SaveBool(m_bSystemOverload);
    	cFC.SaveBool(m_bConstantOverload);
        
        if (m_bIsWorking || m_tImplantsFreezed == null)
        {
            for (int i = 0; i < _IMPLANTS_NUM; i ++)
            {
                if (i == _IMPLANT_TERMOVISION_SYSTEM)
                    cFC.SaveBool(GetImplantTermovision().m_bActiveSG);
                else
                    cFC.SaveBool(GetImplant(i).IsActive());
            }
        }
        else
        {
            if (m_tImplantsFreezed != null)
            {
                for (int i = 0; i < _IMPLANTS_NUM; i ++)
                    cFC.SaveBool(m_tImplantsFreezed[i] != null);
            }
            else
            {
                for (int i = 0; i < _IMPLANTS_NUM; i ++)
                    cFC.SaveBool(false);
            }
        }
        cFC.delete();
    }	    	
	
	public void SGUpdate()
	{
		FileChunk.Log("[ImplantSystem.SGUpdate]\n");
		//DeactivateAllImplants();
        m_fLastUpdateTime = GetTime();
        METHOD_Update = -1;
        StartUpdate();
        /*if (!m_bIsWorking)
        {
            FreezeImplants();
        }
        else*/
        {
            for (int i = 0; i < m_Implants.length; i++)
            {
                if (m_Implants[i] != null)
                {
                    m_Implants[i].m_bImplantSystemWorking = true;  
                    m_Implants[i].SGUpdate();
                }
                
            }
        }
        SetHUDToUpdate(GetPlayer().HUD.GetHUDPlayer());
        UpdateHud();                
        UpdateHudIndicators();
	}
    
    void UpdateTermovision()
    {
        ImplantTermovision cTermo = ((ImplantTermovision)GetImplant(ImplantSystem._IMPLANT_TERMOVISION_SYSTEM));
        cTermo.Update();
    }
    
    ImplantTermovision GetImplantTermovision()
    {
        return ((ImplantTermovision)GetImplant(ImplantSystem._IMPLANT_TERMOVISION_SYSTEM));
    }
    
}
