/** implant Termovision System
 */
public class ImplantTermovision extends Implant
{
	static float _TERMOVISION_BLEND_START   = 500.0f;
    static float _TERMOVISION_BLEND_END     = 3500.0f;
	static float _TERMOVISION_MARGIN        = 1000.0f;
    
    {
        m_nHint = Hint._HINT_IMPLANT_TERMOVISION;
    }
    
    void SetBlendStart(int nBlendStart)
    {
        _TERMOVISION_BLEND_START = nBlendStart;
    }
    
    void SetBlendEnd(int nBlendEnd)
    {
        _TERMOVISION_BLEND_END = nBlendEnd;
    }
    
	/** zakresy widzialnosci obiektow przywracane po wylaczeniu implantu */
	float m_fBlendDistance = 0f;
	float m_fFadeDistance = 0f;
	float m_fTreesVisRange = 0f;
    float m_fTreesVisBlendRange = 0f;
	float m_fCamFar = 0f;
    float m_fFogNear = 0;
    float m_fFogFar  = 0;

    int m_nScreenTex = 0;
    
    int m_nFogType = -1;
    float m_fFogDensity = -1;
    
    Sprite m_cScreen = null;
    
    /**Lista meshy uzywanych do wizualizacji termowizji. Meshe sa dynamicznie przydzielane
     * widocznym aktualnie obiektom, "wolne" meshe sa zawsze na poczatku listy
     */
    private static java.util.Vector g_vTermoMeshes = new java.util.Vector();
    private static java.util.Vector g_vObjects = new java.util.Vector();
    private static java.util.Vector g_vObjectsRenderable = new java.util.Vector();
                                   
    static MeshObject GetTermoMesh()
    {
        
        MeshObject cMesh = null;
        if (g_vTermoMeshes.size() > 0)
            cMesh = (MeshObject)g_vTermoMeshes.elementAt(0);
        
        if (cMesh == null || cMesh.IsActiveOnModule())
        {
            //zabraklo wolnych meshy, tworzymy nowego
            cMesh = CreateTermoMesh(ObjectivesManager.m_Module);
            //niestety zjeba w stylu reacmenagera         
        }
        else
        {
            //wrzucamy mesha na koniec listy (jest uzywany)
            g_vTermoMeshes.removeElementAt(0);
            g_vTermoMeshes.addElement(cMesh);
        }
        
        return cMesh;
    }
    
    static void ReleaseTermoMesh(MeshObject cMesh)
    {
        cMesh.DeactivateOnModuleAlways();
        g_vTermoMeshes.removeElement(cMesh);
        g_vTermoMeshes.insertElementAt(cMesh, 0);
    }
    
    static MeshObject CreateTermoMesh(GameObject cParent)
    {
        MeshObject cMesh = (MeshObject)cParent.CreateObjectNoInit("MeshObject");
        cMesh.MeshName = "data/meshes/ai/Termowizja.3DA";
        cMesh.InitObject();
        //tego chyba nie musi byc, jak bedzie dzialalo to wywalic...
        //cParent.GetModule().AddControlObject(cMesh);
        cMesh.DeactivateOnModuleAlways();
        cMesh.SetFacingTowardsObserver(true);
        cMesh.ClearFlag(EFlags._COLLISION);
        cMesh.ClearFlag(EFlags._BLOCK_TRACE);
        cMesh.SetFlag(EFlags._PORTAL_INDEPENDENT);
        cMesh.ClearFlag(EFlags._FXPARTICLE_COLLISION);
        cMesh.ClearFlag(EFlags._NOT_STATIC);
        //cMesh.LoadMesh("data/interface/bBox.3da");
        cMesh.SetMeshVisibilityRange(_TERMOVISION_BLEND_START, _TERMOVISION_BLEND_END);                
        
        if (g_vTermoMeshes.size() > 0)
            g_vTermoMeshes.insertElementAt(cMesh, 0);
        else
            g_vTermoMeshes.addElement(cMesh);
        
        //LogR("TERMOVISION Creating TermoMesh. Count " + g_vTermoMeshes.size());
        return cMesh;    
    }
    
    static void AddToSystem(ChromeActor cActor)
    {
        int nActorIndex = g_vObjects.indexOf(cActor);
        boolean bActive = true;
        if (cActor instanceof NewAI)
            bActive = ((NewAI)cActor).m_bIsActive;

        //jesli nie ma to dodajemy
        if (nActorIndex == -1)
        {
            g_vObjects.addElement(cActor);
            
            g_vObjectsRenderable.addElement(new Boolean(cActor.IsRenderable() && bActive ));
            Player cPlayer = ((ChromeModule)cActor.GetModule()).GetMainPlayer();

            if (cPlayer != null && cPlayer instanceof PlayerNetBase)
            {
                ImplantSystem cImplantSys = cPlayer.GetImplantSystem();
                if (cImplantSys != null)
                {
                    ImplantTermovision cTermo = ((ImplantTermovision)cImplantSys.GetImplant(ImplantSystem._IMPLANT_TERMOVISION_SYSTEM));
                    if (cTermo != null)
                        cTermo.Update();
                }
            }
        }
        else
        {
            g_vObjectsRenderable.setElementAt(new Boolean(cActor.IsRenderable() && bActive ), nActorIndex);
        }
        //LogR("TERMOVISION adding ai " + g_vObjects.size());
    }
    
    static void RemoveFromSystem(ChromeActor cActor)
    {
        int nIdx = g_vObjects.indexOf(cActor);
        if (nIdx != -1)
        {
            g_vObjects.removeElement(cActor);
            
            if (cActor.IsTermovisionOn())
                cActor.TermovisionOff();
            
            if (cActor instanceof PlayerNetBase)
            {
                if (((Boolean)g_vObjectsRenderable.elementAt(nIdx)).booleanValue())
                    ActorEnableRendering(cActor, true);
            }
            
            g_vObjectsRenderable.removeElementAt(nIdx);
        }
    }
    
    public void OnCreate()
    {
		super.OnCreate();
        //na starcie tworzymy n meshy, jak zabraknie to sie dorobi
		/*for (int i = 0; i < 100; i++)
            CreateTermoMesh(this);*/
        
        m_cScreen = (Sprite)cModule.CreateObject("Sprite");
        m_cScreen.SetSize(new Vector(GameObject.GetScreenWidth(), GameObject.GetScreenHeight(), 0));
        cModule.AddSprite(m_cScreen);
        m_cScreen.SetSpriteMaterial("heatvis_mask.mat");
        m_nScreenTex = m_cScreen.AddTexture("CH_T_VisionSpirte.png");
        m_cScreen.SetMask(0, new Rect(0,0,1,1));
        m_cScreen.SetTexture(m_nScreenTex, new Rect(0,0,1,1));
    }   
	
    public boolean Activate()
    {
        if (cModule instanceof ModuleNet)
            return false;
		if (super.Activate())
		{
            ChromeModule module = cModule;
			if (m_Player.bNetIsOwner &&
                module != null       &&
                module.iViewID != -1)
			{
                m_cScreen.MoveTop();
                m_cScreen.Show();
                
                m_Player.SetTermovision(true);
                
				m_fTreesVisRange = module.GetTreesVisibilityRange();
                m_fTreesVisBlendRange = module.GetTreesVisibilityBlendRange();
				module.SetTreesVisibilityRange(_TERMOVISION_BLEND_END-1000.f, _TERMOVISION_BLEND_END);
                m_fFogNear = module.FogGetNear();
                m_fFogFar = module.FogGetFar();
    
                /* Pobiera typ mgly */
                m_nFogType = module.FogGetType();
                /* Pobiera gestosc mgly exponencjalnej */
                m_fFogDensity = module.FogGetDensity();            
                /* Ustawia typ mgly
                 * 0 - none, 1 - linear, 2 - exp, 3 - exp2 
                 */
                module.FogSetType(1);
                module.FogSetNear(_TERMOVISION_BLEND_START);
                module.FogSetFar(_TERMOVISION_BLEND_END);
				module.ViewRenderSky(module.iViewID, false);
                /**new*/
                ObjectsEnableRendering(false);
                Update();
                ImplantSystem cImplantSystem = m_Player.GetImplantSystem();
                cImplantSystem.CallMethodEvery("UpdateTermovision", 0.1f);
			}
			return true;
		}
		else
			return false;
	}
	
    void ObjectsEnableRendering(boolean bEnable)
    {
        for (int i = 0; i < g_vObjects.size(); i++)
        {
            ChromeActor cActor = (ChromeActor)g_vObjects.elementAt(i);
            
            if (((Boolean)g_vObjectsRenderable.elementAt(i)).booleanValue())
                ActorEnableRendering(cActor, bEnable);
        }
    }
            
    
    void Update()
    {
        if (!m_bActiveFlag || !bNetIsOwner)
            return;
        ChromeActor cActor = null;
        for (int i = 0; i < g_vObjects.size(); i++)
        {
            cActor = (ChromeActor)g_vObjects.elementAt(i);
            
            //actor zaczal sie renderowac w trakcie dzialania termowizji
            if (cActor.IsRenderable())
            {
                g_vObjectsRenderable.addElement(new Boolean(true)); 
                
                ActorEnableRendering(cActor, false);
            }
            
            if (cActor.GetDistanceTo(m_Player) < _TERMOVISION_BLEND_END + _TERMOVISION_MARGIN &&
                cActor.IsActiveOnModule())
            {
                if (!cActor.IsTermovisionOn())
                    cActor.TermovisionOn();
            }
            else
            {
                if (cActor.IsTermovisionOn())
                    cActor.TermovisionOff();
            }
        }
    }
    
    
    static void ActorEnableRendering(ChromeActor cActor, boolean bEnable)
    {
        boolean bNoAnimRefresh = cActor.IsSetFlag(EFlags._NO_ANIMATION_REFRESH);
        boolean bNoAnim = cActor.IsSetFlag(EFlags._NO_ANIMATION);

        if (bNoAnimRefresh)
            cActor.SetFlag(EFlags._NO_ANIMATION_REFRESH);
        else
            cActor.ClearFlag(EFlags._NO_ANIMATION_REFRESH);

        cActor.EnableRendering(bEnable);
        
        if (bNoAnim)
            cActor.SetFlag(EFlags._NO_ANIMATION);
        else
            cActor.ClearFlag(EFlags._NO_ANIMATION);
    }
    
    public void Deactivate()
    {
        if (!m_bImplantSystemWorking || !bNetIsOwner)
            return;
        
        if (m_bActiveFlag)
        {
		    ChromeModule module = cModule;
            if (m_Player.bNetIsOwner &&
                module != null       &&
                module.iViewID != -1)
		    {
                m_Player.SetTermovision(false);
                
		    	if (m_fTreesVisRange != 0)
		    		module.SetTreesVisibilityRange(m_fTreesVisBlendRange, m_fTreesVisRange);
		    	//module.ViewVisionEffect(module.iViewID, EVision.VE_NONE);
		    	if ( m_Player        != null && 
                     m_Player.Camera != null &&
                     m_fCamFar       != 0.0f )
                {
		    		//m_Player.Camera.SetClipFar(m_fCamFar);
                }
		    	module.ViewRenderSky(module.iViewID, true);
                
                module.FogSetType(m_nFogType);
                
                module.FogSetNear(m_fFogNear);
                module.FogSetFar(m_fFogFar);
                
                /* Ustawia gestosc mgly exponencjalnej */
                module.FogSetDensity(m_fFogDensity);
                
                ObjectsEnableRendering(true);
                for (int i = 0; i < g_vObjects.size(); i++)
                {
                    ((ChromeActor)g_vObjects.elementAt(i)).TermovisionOff();
                }
                ImplantSystem cImplantSystem = m_Player.GetImplantSystem();
                cImplantSystem.DisableCallMethod("UpdateTermovision");
                
                m_cScreen.Hide();
		    }
        }
        super.Deactivate();
	}
    
    boolean m_bActiveSG = false;
    void DeactivateToSave()
    {
        if (IsActive())
            m_bActiveSG = true;
        else
            m_bActiveSG = false;
        
        Deactivate();
    }
    
    void ActivateAfterSave()
    {
        Activate();
        m_bActiveSG = false;
    }
    
    public void SGUpdate()
	{
        super.SGUpdate();
        //g_vTermoMeshes.removeAllElements();
        //g_vObjects.removeAllElements();
        //g_vObjectsRenderable.removeAllElements();
    }
    
    public void OnDestroy()
    {
        if (bNetIsOwner)
        {
            m_cScreen.ReleaseTexture(m_nScreenTex);
            m_Player.GetModule().RemoveSprite(m_cScreen);
            m_cScreen.delete();
            g_vTermoMeshes.removeAllElements();
            g_vObjects.removeAllElements();
            g_vObjectsRenderable.removeAllElements();
        }
    }
}
