/** Klasa jetpacka (na razie dla MingTse na lvl09)
 */
public class Jetpack extends InvObjectInst implements iact_StartStop
{
    public float _ENGINE_FX_UPDATE_TIME = 0.2f;
    public float _GLOW_FX_UPDATE_TIME = 60f;
    
    public String _ENGINE_FX_NAME = "FXJetpackEngine";
    public String _GLOW_FX_NAME = "FXJetpackGlow";
    public String _ENGINES_SOUND = "Data/Sounds/Vehicles/Jetpack";
    
    // nazwy dummiesow do attachowania fx'ow (tablice, bo graficy lubia bardzo zmieniac nazwy na lepsze...)
    String[] m_aEnginieDummies = new String[] { "Engine01", "Engine02", "Engine03", "Engine04" };
    String[] m_aGlowDummies = new String[] { "Glow01", "Glow02" };

    /** Tablica ID meshelementow silnikow */
    int[] m_aEngineDummiesIDs = null;
    /** Tablica ID meshelementow flar */
    int[] m_aGlowDummiesIDs = null;

    /** ID dzwieku silnika */
    int m_nEngineSoundID = 0;
    
    int[] m_aGlowFXsIDs = null;
    
    int METHOD_UpdateEngineFXs = 0;
    int METHOD_UpdateGlowFXs   = 0;

    
    
    Jetpack()
    {
        super();
        MeshName="Data/Meshes/Equipment/jetpack.3da";
        
        nInvIndex = InvObject._E_UNIFORM;
        nInvType = EInvObjType._EQUIP;
        sName = "&JETPACK&";
        sTexFileName = "Data/Textures/CH_I_Weapons4.png";
      	nCellsWidth = 1;
	    nCellsHeight = 1;
        
        //wspolrzedne polozenia na teksturze
        nOnTexPosX = 200.0f;
        nOnTexPosY = 60.0f;
        
        //rozmiary tekstury
        nTexSizeX = 256.0f;
        nTexSizeY = 256.0f;
    }
    
    
    public void OnCreate()
    {
        super.OnCreate();
        bSG = true;
        METHOD_UpdateEngineFXs = RegisterCallMethod("UpdateEngineFXs");
        METHOD_UpdateGlowFXs = RegisterCallMethod("UpdateGlowFXs");
        // zainicjuj id meshelementow do fx'ow
        m_aEngineDummiesIDs = new int[m_aEnginieDummies.length];
        for (int i = 0; i < m_aEnginieDummies.length; i++)
            m_aEngineDummiesIDs[i] = GetElementID(m_aEnginieDummies[i]);
        
        m_aGlowDummiesIDs = new int[m_aGlowDummies.length];
        for (int i = 0; i < m_aGlowDummies.length; i++)
            m_aGlowDummiesIDs[i] = GetElementID(m_aGlowDummies[i]);
        
        m_aGlowFXsIDs = new int[m_aGlowDummiesIDs.length];
        for (int i = 0; i < m_aGlowFXsIDs.length; i++)
            m_aGlowFXsIDs[i] = -1;
        
    }

    
    public boolean IsActive(ControlObject activator)
    {
        return bActiveTrigger && super.IsActive(activator);
    }
    /////////////////////////////////////////////////////////////////////////////
    // implementacja iact_StartStop
    //
    private boolean m_bIsStarted = false;
    public void Start()
    {
        EnableEngineFXs(true);
        if (m_nEngineSoundID == 0)
            m_nEngineSoundID = AttachSound(_ENGINES_SOUND);
        m_bIsStarted = true;
    }
    
    /** Metoda wolana po przyjsciu zdarzenia act_Stop */
    public void Stop()
    {
        EnableEngineFXs(false);
        if (m_nEngineSoundID != 0)
        {
            DetachSound(m_nEngineSoundID);
            m_nEngineSoundID = 0;
        }
        m_bIsStarted = false;
    }
    
    public void ToggleStartStop()
    {
        if (m_bIsStarted)
            Stop();
        else
            Start();
    }
    
    public void EnableEngineFXs(boolean bEnable)
    {
        if (bEnable)
        {
            CallMethodEvery(METHOD_UpdateEngineFXs, _ENGINE_FX_UPDATE_TIME);
            CallMethodEvery(METHOD_UpdateGlowFXs, _GLOW_FX_UPDATE_TIME);
            UpdateEngineFXs();
            UpdateGlowFXs();
        }
        else
        {
            DisableCallMethod(METHOD_UpdateEngineFXs);
            DisableCallMethod(METHOD_UpdateGlowFXs);
            // wylacz dlugo trwajace fx glow'a
            for (int i = 0; i < m_aGlowDummiesIDs.length; i++)
            {
                if (m_aGlowFXsIDs[i] >= 0)
                    DeleteFXSequence(m_aGlowFXsIDs[i]);
            }
        }
    }
    
    void UpdateEngineFXs()
    {
        for (int i = 0; i < m_aEngineDummiesIDs.length; i++)
            ExecuteFX(this, m_aEngineDummiesIDs[i], _ENGINE_FX_NAME, new Vector());
    }

    
    void UpdateGlowFXs()
    {
        for (int i = 0; i < m_aGlowDummiesIDs.length; i++)
        {
            if (m_aGlowFXsIDs[i] >= 0)
                DeleteFXSequence(m_aGlowFXsIDs[i]);
            m_aGlowFXsIDs[i] = ExecuteFX(this, m_aGlowDummiesIDs[i], _GLOW_FX_NAME, new Vector());
        }
    }

    
    //*********************************************************
    //					SaveGame
    //*********************************************************
	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Jetpack.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_JETPACK)
        {
			super.SGLoadChunk(cFC);
            m_bIsStarted = cFC.LoadBool();
        }
		
		cFC.delete();
	}
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Jetpack.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_JETPACK, cFCParent);
		
		super.SGSaveChunk(cFC);

        cFC.SaveBool(m_bIsStarted);
        
        cFC.delete(); 
	}				
	
	public void SGUpdate()
	{
		FileChunk.Log("[Jetpack.SGUpdate]\n");
		super.SGUpdate();
        
        m_nEngineSoundID = 0; // zaznacz, zeby odpail dzwiek, bo save wylacza
        if (m_aGlowFXsIDs != null) // zaznacz, ze nie sa odpalone fx, bo jw.
            for (int i = 0; i < m_aGlowFXsIDs.length; i++)
            {
                if (m_aGlowFXsIDs[i] > 0)
                    DeleteFXSequence(m_aGlowFXsIDs[i]);
                m_aGlowFXsIDs[i] = 0;
            }
        if (m_bIsStarted)
            Start();
	}

    public void OnAudioReinitialize()
    {
        
        if (m_nEngineSoundID != 0)
            m_nEngineSoundID = AttachSound(_ENGINES_SOUND);

    }
    
}
