public class LaserMine extends Explosive
{
    /**Czas po jakim mina sie uaktywnia*/
    public float m_fActivationTime = 3f;
    public float m_fSeekTargetTime = 0.075f;    
    public float m_fSeekTargetRange = 250;
    public float m_fLaserRange = 500f;
    public boolean m_bLaserTriggered = true;
    public Vector m_vLaserTarget = null;  
    public Vector m_vLaserPos = null;
    
    /**Czy mina ma sie sama uzbroic po stworzeniu*/
    public boolean m_bArmOnCreate = false;//#LaserMine
    
    LaserMineCollision m_cLaserCollision = null;
    
    String m_sLaserFX = "FXLaserMine";
    LaserPointer m_cLaser = null;
    
    LaserMine()
    {
        super();
        MeshName="Data/Meshes/Equipment/LaserMine.3DA";
        //MeshName="Data/Meshes/Equipment/bomba_zdalna.3DA";
        
        nInvIndex = InvObject._E_PROXIMITY_MINE;
        nInvType = EInvObjType._EQUIP;
        sName = "&E_N_LASERMINE&";
        sTexFileName = "er_t_Red.png";
        nCellsWidth = 2;
        nCellsHeight = 2;
        
        nOnTexPosX = 160.0f;
        nOnTexPosY = 180.0f;
        
        nTexSizeX = 256.0f;
        nTexSizeY = 256.0f;

        fHealth = 1.0f;
        
        /**Czas po jakim mina sie uaktywnia*/
        m_fActivationTime = 3f;
        
        /** Zasieg wybuchu obiektu w jakim odbierana jest energia */
        fBlastRange = 500.0f;
        
        /** Blast damage odbierany w centrum wybuchu */
        fBlastDamage = 300.0f;
        
        /** Nazwa FXow do odpalenia przy wybuchu */
        sBlowFX = "FXBlowGrenade";

        /** Nazwa dzwieku odpalanego przy wybuchu*/
        sBlowSound = "Data/Sounds/Weapon/GrenadeExplosion.wav";
        
        /** Nazwa dzwieku odgywanego, gdy bombka jest aktywna */
        sActiveSound = "Data/Sounds/BombBeep.wav";
        
        /** Numer dzwieku odgywanego, gdy bombka jest aktywna */
        nActiveSound_id = -1;
        
        /** odleglosc od obiektu, terenu z jakiej mozna przykleic bombke*/
        fDistance = 200.0f;

        /** Znacznik, ze ma sie attachowac do celu na ktorym sie go mountuje */
        bAttachToTarget = true;

    }
    
    public void OnCreate()
    {
        super.OnCreate();
        
		//SetPhysicsType(EPhysics._NONE);
        if (m_bLaserTriggered)
        {
            m_cLaser = Tools.CreateLaserPointer(GetModule(), m_sLaserFX, 1, m_fLaserRange, true);
            //m_cLaser.AttachAsChildTo(this);
            m_cLaser.SetPosition(Vector.Zero);
            
            m_cLaserCollision = (LaserMineCollision)CreateObject("LaserMineCollision");
            GetModule().AddControlObject(m_cLaserCollision);
			//m_cLaserCollision.AttachAsChildTo(this);
            m_cLaserCollision.Init(this);
        }
        
        if(m_bArmOnCreate)
            PlantBomb();
            //bActiveTrigger = true;
    }
	
    void OnRemoveFromInventory()
    {
        super.OnRemoveFromInventory();
        
		m_cLaser.AttachAsChildToAndKeepWorldPos(this);
        m_cLaserCollision.AttachAsChildToAndKeepWorldPos(this);
    }	
    
    void LaserCollision(ControlObject cObj, Vector vPoint)
    {
        Explode();
    }
    
    /** metoda powodujaca uzycie przedmiotu - przyklejenie bombki do sciany,
     * terenu, obiektu...
     */
    public void Use()
    {
        // szukam punktu, w ktorym umieszczamy bombke
        Vector vStart = ((ChromeModuleSingle) GetModule()).MainPlayer.Camera.GetPosition();
        m_vEnd = ((ChromeModuleSingle)GetModule()).MainPlayer.Camera.GetForwardVector();
        m_vEnd.Mul(-fDistance);
        m_vEnd.Add(vStart);
        
        m_Collision = null;
        
        Collision coll = TraceTerrain(vStart, m_vEnd);
        if (coll != null)
            m_Collision = coll;

        coll = TraceTrees(vStart, m_vEnd);
        if (coll != null)
            m_Collision = coll;
        
        coll = TraceObjectsExclude(vStart, m_vEnd, Owner);
        if (coll != null)
            m_Collision = coll;
        
        if (m_Collision != null)
        {
            //CrashLog("DBG: Planting on: " + m_Collision.Obstacle + "\n");
            if ( !(m_Collision.Obstacle instanceof Pawn) )
            {
                PlantBomb();
                return;
            }
        }
        
    }
    
    void PlantBomb()
    {
        if (m_bLaserTriggered)
        {
            if (!PlantBombLaser())
            {
                m_bArmOnCreate = false;
                return;
            }
        }
        else
        {
            super.PlantBomb();
        }
        
        if (!m_bArmOnCreate)
        {
            CallMethodIn("Activate", m_fActivationTime);
            // wywalamy bombe z inventory
            Owner.cInventory.RemoveInvObject(this);
        }
        else
        {
            CallMethodIn("Activate", 0);
        }
        bActiveTrigger = false;
        
    }
    
    boolean PlantBombLaser()
    {
        if (!m_bArmOnCreate)
        {
            Vector vNormal = m_Collision.vNormal;
            vNormal.Normalize();

		    if (java.lang.Math.abs(vNormal.Dot(Vector.Up)) > 0.999f)
		    {
		    	Vector vUp = Owner.GetRightVector();
		    	vUp = vUp.Cross(Vector.Up);
		    	FromUpForwardPos(vUp, vNormal, m_vEnd);
		    }
		    else
		    {
		    	SetPosition(m_vEnd);
		    	SetDir(vNormal);
		    }
        }		
        Vector vDir = GetForwardVectorVolatile();
        Vector vOrigin = GetPositionVectorVolatile();
        Vector vEnd = new Vector(vDir);
        vEnd.Mul(m_fLaserRange);
        vEnd.Add(vOrigin);

        Collision stColl = null;
        Collision stCollTerrain=TraceTerrain(vOrigin, vEnd);
        Collision stCollTrees=TraceTrees(vOrigin, vEnd);
        Collision stCollObjects=TraceObjectsExclude(vOrigin, vEnd, Owner);
        
        if(stCollObjects != null)
            stColl = stCollObjects;
        else
            if (stCollTrees != null)
                stColl = stCollTrees;
            else
                if (stCollTerrain != null)
                    stColl = stCollTerrain;
                else    
                    return false;
        
        ActivateOnModule();
        EnableRendering(true);
        if (bAttachToTarget && 
            !m_bArmOnCreate &&
            m_Collision.Obstacle instanceof MeshObject) // to glownie sprawdzenie, czy nie null
        {
            if (m_Collision.ObstacleElement != -1)
                ((MeshObject)m_Collision.Obstacle).AttachChildAndKeepWorldPos(this, m_Collision.ObstacleElement);
            else
                ((MeshObject)m_Collision.Obstacle).AttachChildAndKeepWorldPos(this, 0);//Podlacz do roota
        }
        
        m_cLaser.SetRange(stColl.fDistance);
        m_cLaser.SetDir(GetForwardVector());
        
        //dodajemy 3 cm dla pewnosci
        m_vLaserTarget = PointWorldToLocal(vEnd.Add2(vDir.Mul2(3)));
        m_vLaserPos = GetPosition();
        return true;
    }
    
       
    /** Odebranie energii obiektom wokol wybuchu
     */
    void Blast()
    {
        
        MakeBlastDamage(fBlastDamage, fBlastDamage, GetPosition(), fBlastRange, true,
                        null, EnumDamageType._BLAST);
    }
    
    void Activate()
    {
        if (m_bLaserTriggered)
        {
            m_cLaser.PointForwardAttach(m_cLaser.m_fRange);
            m_cLaserCollision.FromUpForwardPos(GetUpVector(), GetForwardVector(), GetPosition());
            m_cLaserCollision.SetLength(m_cLaser.m_fRange);
            m_cLaserCollision.ActivateOnModule();
        }
        else
        {
            CallMethodEvery("SeekTarget", m_fSeekTargetTime);
        }
        if (!m_bArmOnCreate)   
            PlaySound(sActiveSound);
        
        CallMethodEveryMinMax("CheckLaserEndPoint", 0.25f, 2f);
    }

    
    
    float fLastCollDistance = 0;
    void CheckLaserEndPoint()
    {
        Collision stColl = TraceForCollision(GetPositionVectorVolatile(), PointLocalToWorld(m_vLaserTarget), this);
        if (stColl == null)
        {
            //laser wisi w powietrzu, nie ma na czym sie "oprzec" 
            Explode();
        }
        else
        {
            //czy zmienila sie odleglosc do drugiego konca promienia
            if (Math.abs(stColl.fDistance - fLastCollDistance) > 10 && fLastCollDistance != 0)
                Explode();
            fLastCollDistance = stColl.fDistance;
        }
        
    }
    
    void Explode()
    {
        DisableCallMethods();
        super.Explode();
        if (m_cLaser != null)
        {
            m_cLaser.DeactivateOnModuleAlways();
            m_cLaser.CallMethodIn("delete", 0);
        }
        if (m_cLaserCollision != null)
        {
            m_cLaserCollision.ClearFlag(EFlags._BLOCK_MOVE);
            m_cLaserCollision.DeactivateOnModuleAlways();
            m_cLaserCollision.CallMethodIn("delete", 0);
        }
        m_cLaser = null;
		m_cLaserCollision = null;
    }
    
    /**wyszukiwanie celu w wariancie zblizeniowym (m_bLaserTriggered = false)*/
    void SeekTarget()
    {
        GameObject tab[] = ((GameObject)this).GetObjectsInRadiusFromClass(GetPositionVectorVolatile(), m_fSeekTargetRange, "ChromeActor");         
        ChromeActor cActor = null;
        for (int i = 0; i < tab.length; i++)
        {
            cActor = (ChromeActor)tab[i];
            if( cActor==this ) continue;
            // umarlaki sie nie licza
            if(cActor.bDead) continue;
            
            // liczymy trace'a czy czasem cos nie zaslania obiektu
            Vector vActorMin = new Vector();
            Vector vActorMax = new Vector();
            cActor.GetMeshExtents(vActorMin, vActorMax);
            vActorMin.fX = 0; vActorMin.fZ = 0;
            // inaczej sprawdzamy czy nie schowal sie ktos za przeszkoda
            int nSamples = 4;
            float fStep = (vActorMax.fY - vActorMin.fY) / ((float)nSamples);
            for(int j=0; j<nSamples; ++j)
            {
                if(CheckTrace(cActor, new Vector(), vActorMin))
                {
                    Explode();
                    return;
                }
                vActorMin.fY += fStep;
            }
                
        }
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[LaserMine.SGLoadChunk]\n");
		
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_EXPLOSIVE)
        {	
			super.SGLoadChunk(cFC);
            
			m_vEnd = cFC.LoadVector();
			m_vLaserTarget = cFC.LoadVector();
			m_vLaserPos = cFC.LoadVector();
			fLastCollDistance = cFC.LoadFloat();
			
			m_cLaser = (LaserPointer)cFC.LoadObject(m_cLaser,this);
			m_cLaserCollision = (LaserMineCollision)cFC.LoadObject(m_cLaserCollision,this);
        }
		
		cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[LaserMine.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_EXPLOSIVE, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveVector(m_vEnd);
		cFC.SaveVector(m_vLaserTarget);
		cFC.SaveVector(m_vLaserPos);		
		cFC.SaveFloat(fLastCollDistance);
		
		cFC.SaveObject(m_cLaser);
		cFC.SaveObject(m_cLaserCollision);
    
        cFC.delete();
    }	    		
	
    public void SGUpdate()
    {	
		FileChunk.Log("[LaserMine.SGUpdate]\n");		
		super.SGUpdate();		
    }    
	
    void Acquire(Module mod, Pawn owner)
    {
        super.Acquire(mod, owner);
		
		m_vLaserTarget = null;  
		m_vLaserPos = null;
		if (m_cLaserCollision != null)
			this.AcquireObjectModule(m_cLaserCollision);
    }		
}

