public class StationaryEnergyCannonControllerNet extends StationaryCannonControllerNet
{
    ControlObject m_cDummyBullet = null;

    static int m_nActorClassId = 0;
    
    float m_fBlastRadius = 1000.0f;
    float m_fBlastDamage = 300.0f;
    
    public void OnCreate()
    {
        if (m_nActorClassId == 0) 
            m_nActorClassId = GetClassID( "Actor" );
        
        super.OnCreate();
        
        m_cDummyBullet = (ControlObject)CreateObject("ControlObject");
    }
    
    
    void Hit(float fDistance, ControlObject cObstacle, int nObstacleElement, 
             Vector vHitPoint, Vector vDir, Vector vNormal, 
             int nMaterial, int nMaterialFlags, boolean bWaterHit)
    {
        Actor cExcludeObj = (cObstacle instanceof Actor)?
                              (Actor)cObstacle : null;
        MakeBlastDamage( m_fBlastDamage, 
                         m_fBlastDamage, 
                         vHitPoint, 
                         m_fBlastRadius, 
                         false, 
                         cExcludeObj );
        
        super.Hit( fDistance, cObstacle, nObstacleElement, 
                   vHitPoint, vDir,      vNormal, 
                   nMaterial, nMaterialFlags, 
                   bWaterHit );
    }
    
    public void MakeBlastDamage(float fInBlastDamage, 
                                float fPhSimDamage, 
                                Vector vPos, float fBlastRange, 
                                boolean bExcludeActor, Actor cExcludeObj)
    {
        //LogR("HIT "+((cExcludeObj!=null)?cExcludeObj.GetNetInfo():"none")+" Damage "+fDamage); 
        
        m_cDummyBullet.SetPosition( vPos );
        
        ControlObject[] aObjects = m_cDummyBullet.GetObjectsInRadiusFromClass(
                                                  fBlastRange, 
                                                  m_nActorClassId );
        Actor  cActor = null;
        Vector vSpeed = null;
        for ( int i=0; i<aObjects.length; ++i )
        {
            float fBlastDamage = fInBlastDamage;
                
            cActor = (Actor)aObjects[i]; 
            
            if ( cActor instanceof Player )
            {
                Player cPlayer = (Player)cActor;
                //ShakeCameraPlayer( cPlayer );
                
                if ( (cPlayer.cSeatInfo != null)&&
                     (cPlayer.cSeatInfo.GetOccupiedObject() instanceof Vehicle) )
                {
                    fBlastDamage *= 
                        ((Vehicle)cPlayer.cSeatInfo.GetOccupiedObject()).GetOccupantsBlastDamageMul();
                }
            }
            
            if ( Role < ERole._AUTHORITY )
                continue;
            
            if ( cExcludeObj == cActor )
                continue;
            
            // sprawdzamy czy sobie nie odejmowac energii
            if ( bExcludeActor && ( cActor == this ) ) 
                continue;
                
            // umarlaki nie maja co miec zabieranego damagu
            if (cActor.bDead) 
                continue;
                
            float fRange = vPos.Distance(cActor.GetPosition());
            if (fRange < fBlastRange)
            {
                float fDamage = (fBlastRange - fRange) / fBlastRange;
                float fPhDamage = fDamage;
                fDamage *= fBlastDamage;
                fPhDamage *= fPhSimDamage;
                // liczymy kierunek predkosci
                vSpeed = cActor.GetPosition();
                vSpeed.Sub(vPos);
                vSpeed.Normalize();
                vSpeed.Mul(100000.0f);
                    
                // w odleglosci 30% zasiegu dzialania bulleta i tak zabiera energie
                if (fRange > fBlastRange*0.3f)
                {
                    // liczymy trace'a czy czasem cos nie zaslania obiektu
                    Vector vActorMin = new Vector();
                    Vector vActorMax = new Vector();
                    cActor.GetMeshExtents(vActorMin, vActorMax);
                    
                    vActorMin.fX = 0; vActorMin.fZ = 0;
                    
                    // inaczej sprawdzamy czy nie schowal sie ktos za przeszkoda
                    int nSamples = 4;
                    float fStep = (vActorMax.fY - vActorMin.fY) / ((float)nSamples);
                    float fDamageStep = fDamage/((float)nSamples);
                    float fPhDamageStep = fPhDamage/((float)nSamples);
                    for(int j=0; j<nSamples; ++j)
                    {
                        if(!CheckTrace(cActor, new Vector(), vActorMin))
                        {
                            // nie ma trafienia, wiec odbieramy mniej energii
                            fDamage -= fDamageStep;
                            fPhDamage -= fPhDamageStep;
                        }
                        vActorMin.fY += fStep;
                    }
                }
            
                //LogR("BLAST "+cActor.GetNetInfo()+" Damage "+fDamage); 
                cActor.TakeDamage( fDamage, fPhDamage, Owner, 
                                  Vector.Up, vSpeed, 
                                  EBones._SPINE , EnumDamageType._BULLET );
            }
        }
    }
}
