/**
 * Klasa przechowuje informacje o broni pracujacej na
 * rzecz walkera-vehicla
 * Za jej pomoca walker komunikuje sie z weaponem
 */
public class WeaponWalker implements ISaveGame
{
    final static int _MODE_PRIMARY   = 0;
    final static int _MODE_SECONDARY = 1;
    
    Weapon  m_cWeapon = null;
    boolean m_bFire = false;
    int     m_nMode = _MODE_PRIMARY;
    String  m_sWalkerElementBarrelExit = "";
    
    WeaponWalker(Weapon cWeapon,int nMode,String sElementBarrel)
    {
        m_cWeapon                   = cWeapon;
        m_nMode                     = nMode;
        m_sWalkerElementBarrelExit  = sElementBarrel;
    }
    
    int     GetMode() {  return m_nMode; }
    String  GetWalkerElementBarrelExit(){ return m_sWalkerElementBarrelExit; }
    Weapon  GetWeapon(){ return m_cWeapon; }
        
    void SetFire(boolean bFire)
    {
        SetFire(bFire, 0f);
    }
    
    void SetFire(boolean bFire, float fDelay)
    {
        if (m_bFire != bFire)
        {
            m_bFire = bFire;
            
            if (fDelay != 0f)
            {
                /*m_cWeapon.DisableCallMethod("StartFire");*/
                m_cWeapon.DisableCallMethod("StopFire");
            }
            
            if ( m_bFire )
            {
                if (fDelay != 0f)
                    m_cWeapon.CallMethodIn("StartFire", fDelay);
                else
                    m_cWeapon.StartFire();
            }
            else
            {
                if (fDelay != 0f)
                    m_cWeapon.CallMethodIn("StopFire", fDelay);
                else
                    m_cWeapon.StopFire();
            }
        }
    }
    
    void ForceStopFire()
    {
        m_cWeapon.StopFire();
    }
    
    void SetOwner( Pawn cOwner )
    {
        if ( m_cWeapon != null )
        {
            m_cWeapon.Owner = cOwner;
            //w zasadzie to powinno nie przeszkadzac zadnym
            //broniom walkerowym, ale do pewnosci zrobie
            //to tylko do rakiet
            if ( m_cWeapon instanceof WeaponWalker_w_Rocket )
                m_cWeapon.ChangeNetOwner( cOwner );    
        }
    }
    
    void Destroy()
    {
        if ( m_cWeapon != null )
        {
            m_cWeapon.StopFire();
            m_cWeapon.Owner = null;
            m_cWeapon.delete();
            m_cWeapon = null;
        }
    }
    
    //*********************************************************
    //                     SaveGame
    //*********************************************************
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[WeaponWalker.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_WEAPON_WALKER)
        {
            m_cWeapon = (Weapon)cFC.LoadObjectRef();
            m_bFire = cFC.LoadBool();
            m_nMode = cFC.LoadInt();
            m_sWalkerElementBarrelExit = cFC.LoadString();
        }
        
        cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[WeaponWalker.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_WEAPON_WALKER, cFCParent);
        
        cFC.SaveObjectRef(m_cWeapon);
        cFC.SaveBool(m_bFire);
        cFC.SaveInt(m_nMode);
        cFC.SaveString(m_sWalkerElementBarrelExit);
                
        cFC.delete();
    }
    
    public void SGUpdate() 
    {
        FileChunk.Log("[WeaponWalker.SGUpdate]\n");

    }
    
}

/** Lista weaponow walkera-vehicla
 */
class WeaponWalkerList implements ISaveGame
{
    java.util.Vector m_aList = new java.util.Vector();
    
    Walker m_cWalkerOwner;
    
    WeaponWalkerList(Walker cOwner)
    {
        m_cWalkerOwner = cOwner;
    }
    
    int GetSize(){ return m_aList.size(); }
    
    void Add(WeaponWalker cWeapon)
    {
        if ( cWeapon != null )
            m_aList.addElement( cWeapon );
    }
    
    WeaponWalker Get(int i)
    {
        if ( i >= 0 && i < m_aList.size() )
            if ( m_aList.get(i) instanceof WeaponWalker )
                return (WeaponWalker)m_aList.get(i);
        return null;            
    }
    
    void SetWeaponsOwner( Pawn cPawn )
    {
        for ( int i=0; i< m_aList.size(); i++ )
        {
            WeaponWalker cWeapon = Get( i );
            if ( cWeapon != null )
                cWeapon.SetOwner( cPawn );
        }
    }
    
    void ForceStopFire()
    {
        for ( int i=0; i< m_aList.size(); i++ )
        {
            WeaponWalker cWeapon = Get( i );
            if ( cWeapon != null )
                cWeapon.ForceStopFire();
        }
        //callback wykorzystywany do replikacji w sieci
        m_cWalkerOwner.SetWeaponFiring( WeaponWalker._MODE_PRIMARY, false );
        m_cWalkerOwner.SetWeaponFiring( WeaponWalker._MODE_SECONDARY, false );
    }

    
    
    void SetFire(int nMode,boolean bFire)
    {
        Weapon cPrevWeapon = null;
        for ( int i=0; i< m_aList.size(); i++ )
        {
            WeaponWalker cWeapon = Get( i );
            if ( cWeapon != null && cWeapon.GetMode() == nMode )
            {
                float fDelay = 0f;
                if (cPrevWeapon != null &&
                    cPrevWeapon.getClass() == cWeapon.m_cWeapon.getClass())
                    fDelay = ((Weapon)cWeapon.m_cWeapon).fDelay/2f;
                if (fDelay > 0.25f)
                    fDelay = 0.25f;
                
                cWeapon.SetFire( bFire, fDelay );
                cPrevWeapon = cWeapon.m_cWeapon;
            }
        }
        //callback wykorzystywany do replikacji w sieci
        m_cWalkerOwner.SetWeaponFiring( nMode, bFire );
    }
    
    void Destroy()
    {
        for ( int i=0; i< m_aList.size(); i++ )
        {
            WeaponWalker cWeapon = Get( i );
            if ( cWeapon != null )
                cWeapon.Destroy();
        }
        m_aList.removeAllElements();
    }
    
    //*********************************************************
    //                    SaveGame
    //*********************************************************
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[WeaponWalkerList.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_WEAPON_WALKER_LIST)
        {
            cFC.LoadObjectsVector(m_aList);
        }
        
        cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[WeaponWalkerList.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_WEAPON_WALKER_LIST, cFCParent);
        
        cFC.SaveObjectsVector(m_aList);
        
        cFC.delete();
    }
    
    public void SGUpdate() 
    {
        FileChunk.Log("[WeaponWalker.SGUpdate]\n");
    }
}

/**
 * Obiekt przechowujacy informacje o broni walkera
 * Bedzie on tworzony na klientach po replikacji walkera z serwera
 * az do czasu podpiecia rzeczywistej broni do walkera po stronie
 * klienckiej
 * Tak naprawde to powinna byc odwrotna kolejnosc dziedziczenia
 * bo WeaponWalker spelnia wiecej funkcji, ale zeby nie mieszac
 * juz z sejwami to zrobie na szybko tak
 * Aha, nie dopisuje tu sejwowania, bo ta klasa bedzie
 * wykorzystywana _tylko_ w sieci
 **/
class WeaponWalkerInfo extends WeaponWalker
{
    private 
    int m_nNetId = -1;
    int GetNetId(){ return m_nNetId; }
    
    WeaponWalkerInfo(int nNetId,int nMode,String sElementBarrel)
    {
        //nie wiemy jaka to bron, wiec pierwszy prarametr null
        super( null, nMode, sElementBarrel );
        //ale znamy jej sieciowy identyfikator, zatem go zapamietamy
        //i po tym bedziemy szykac wlasciwej broni
        m_nNetId = nNetId;
    }

    /**
     * Jak juz bylo wspomniane wyzej klasa nadrzedna
     * miala wieksze "umiejetnosci" wiec teraz trzeba
     * je dla bezpieczenstwa ograniczyc
     */
    void SetFire(boolean bFire){}
    void SetFire(boolean bFire, float fDelay) {}
    void ForceStopFire(){}
    void SetOwner( Pawn cOwner ) {}
}
