/** Bariera w postaci face'a nie pozwalajacego podanym obiektom przejsc dalej
 */
public class Barrier extends MeshObject implements iact_EnableDisable
{
    public ControlObject[] aBlockedObjects = null;//#\BARRIER;Objects, that are blocked by active barrier

    public ControlObject[] aPassedObjects = null;//#\BARRIER;Objects, that aren't blocked barrier
    
    public boolean bBlockAllByDefault = false;//#\BARRIER;When true, all objects not specified in aPassedObejcts are blocked
    
    public boolean bEnabledOnStart = true;//#\BARRIER;When true, barrier is active on start
    
    protected boolean m_bIsEnabled;
    
    public boolean m_bBlockNetPlayers  = true;//#Net;When true, net players are blocked by active barrier
    public boolean m_bBlockNetVehicles = true;//#Net;When true, net vehicles are blocked by active barrier
    
	public void OnCreate()
	{
        super.OnCreate();
        bSG = true;
        EnableRendering(false);
        
        SetFlag(EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH |
                EFlags._COLLISION | EFlags._NOT_STATIC);
        ClearFlag(EFlags._ALWAYS_ON_TOP | EFlags._BLOCK_TRACE | 
                  EFlags._CAST_SHADOWS | EFlags._RECEIVE_SHADOWS | 
                  EFlags._FXPARTICLE_COLLISION);
		SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
		SetCollAction(ECollFlags.OBJCOL_ACTION_SLIDE);
		SetCollHandler(ECollFlags.OBJCOL_HANDLER_NONE);
        
        if (!bBlockAllByDefault)
            SetCollFlags(ECollFlags.OBJCOL_FLAG_INTERACTIONONLY);
            
        CreateInteractionMap();

        if (bEnabledOnStart)
            Enable();
        else
            Disable();
	}

    void CreateInteractionMap()
    {
        RegisterInInteractionMap();
        if (aBlockedObjects != null)
            for (int i=0; i<aBlockedObjects.length; i++)
                if (aBlockedObjects[i] != null)
                {
                    aBlockedObjects[i].RegisterInInteractionMap();
                    aBlockedObjects[i].SetInteraction(this, true);
                }
        if (aPassedObjects != null)
            for (int i=0; i<aPassedObjects.length; i++)
                if (aPassedObjects[i] != null)
                {
                    aPassedObjects[i].RegisterInInteractionMap();
                    aPassedObjects[i].SetInteraction(this, false);
                }
    }            

    /**
     * Dodanie obiektu do systemu kolizji z barierami.
     * Metoda wolana jest z levelu _sieciowego_ dla wybranych
     * obiektow.
     * Bariera staje sie dla podanego obiektu albo przezroczysta, 
     * albo go blokuje, a zalezy to od zmiennych
     * - m_bBlockNetPlayers - okresla czy bariera ma blokowac
     *  graczy sieciowych (domyslnie true)
     * - m_bBlockNetVehicles - okresla czy bariera ma blokowac
     *  pojazdy s grze sieciowej (domyslnie na true)
     * Pozostale obiekty dodaja sie bezwarunkowo do listy blokowanych.
     */
    void SetInteractionWithObject( ControlObject cObject )
    {
        if ( cObject instanceof PlayerNet )
        {
            SetInteractionWithObject( cObject, m_bBlockNetPlayers );
            return;
        } 

        if ( cObject instanceof Vehicle )
        {
            SetInteractionWithObject( cObject, m_bBlockNetVehicles );
            return;
        }
        
        SetInteractionWithObject( cObject, true );
    }
    
    /**
     * Dodanie obiektu do mapy interakcji do blokowania/ignorowania 
     * przez bariere
     * wykorzystywane w sieci do recznego "podlaczania" obiektow
     */
    private void SetInteractionWithObject( ControlObject cObject, boolean bBlock )
    {
        if ( cObject == null )
            return;
        
        cObject.RegisterInInteractionMap();
        cObject.SetInteraction( this, bBlock );
    }
    
    /////////////////////////////////////////////////////////////
    // implementacje interfejsu iact_Appear
    //
    
    public void Disappear(ControlObject cDestinationObj)
    {
    }

    public void Appear(ControlObject cDestinationObj)
    {
    }
    
    /////////////////////////////////////////////////////////////
    // implementacje interfejsu iact_EnableDisable
    //
    
    public void Enable()
    {
        SetFlag(EFlags._BLOCK_MOVE);
        SetFlag(EFlags._COLLISION);
        ActivateOnModule();
        m_bIsEnabled = true;
    }
    
    public void Disable()
    {
        ClearFlag(EFlags._BLOCK_MOVE);
        ClearFlag(EFlags._COLLISION);
        DeactivateOnModule();
        m_bIsEnabled = false;
    }

    public boolean IsEnabled()
    {		
        return m_bIsEnabled;
    }
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[Barrier.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_BARRIER)
        {
            super.SGLoadChunk(cFC);
            m_bIsEnabled = cFC.LoadBool();
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[Barrier.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_BARRIER, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveBool(m_bIsEnabled);
        cFC.delete(); 
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[Barrier.SGUpdate]\n");
        super.SGUpdate();
		
		if (m_bIsEnabled == true)
			Enable();
		else
			Disable();
    }


}
