public class Container extends TriggerObject
{
    public String sOpenSound = "Data/Sounds/Pojemnik.wav";//#CONTAINER

    /** Poczatkowa klatka animacji otwierania
     */
    public int nOpenStartFrame = 0;//#CONTAINER
    
    /** Koncowa klatka animacji otwierania
     */
    public int nOpenEndFrame = 29;//#CONTAINER
    
    /** Szybkosc animacji
     */
    public float fAnimSpeed = 50.f;//#CONTAINER
    
    
    final static int _STATE_CLOSED = 0;
    final static int _STATE_OPENED = 1;
    
    int nState = _STATE_CLOSED;
    
    
    float fUpperExtentsEpsilon = 5.f;
    
    
    void OnCreate()
    {
		bSG = true;
        super.OnCreate();
        SetFlag(EFlags._BLOCK_MOVE);
        SetFlag(EFlags._BLOCK_TRACE);
        SetFlag(EFlags._COLLISION);
        SetFlag(EFlags._FXPARTICLE_COLLISION);
        ClearFlag(EFlags._NO_ANIMATION);
        ClearFlag(EFlags._NO_ANIMATION_REFRESH);
        SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        SetTraceCollType(ECollTraceFlags._HULL);
        
        SetLighting(true);
        
        //inicjacja animacji otwierania
        InitAnimSeq("open", nOpenStartFrame, nOpenEndFrame);
        SetAnimFreq("open", fAnimSpeed);

        // pompujemy extenty, zeby nie znikala otwarta skrzynka
        Vector vMin = new Vector();
        Vector vMax = new Vector();
        GetMeshExtents(vMin, vMax);
        float fDist = 3f*vMin.Distance(vMax);
        vMin.fX -= fDist; vMin.fY -= fDist; vMin.fZ -= fDist;
        vMax.fX += fDist; vMax.fY += fDist; vMax.fZ += fDist;

        SetExtents(vMin, vMax);
    }

    void OnPostCreate()
    {
        ActivateInsideObjects(false);
    }
    
    /** Przelaczenie toggle'a
     */
    void Execute( Actor executor )
    {
        super.Execute(executor);
        
        if (bActiveTrigger)
        {
            ActivateInsideObjects(true);
            bActiveTrigger = false;
            nState = _STATE_OPENED;
            
            PlayAnim("", "open", 1);
            if (sOpenSound != null && sOpenSound.length() > 0)
                PlaySound(sOpenSound, GetPosition());
            
            // Gdyby cos nie dzialalo ze zbieraniem przedmiotow wewnatrz - trzeba odremowac:
            //ClearFlag(EFlags._BLOCK_TRACE);
        }
    }
    
    void Execute( float fTime )
    {
        if (bActiveTrigger)
        {
            ActivateInsideObjects(true);
            bActiveTrigger = false;
            nState = _STATE_OPENED;
            
            PlayAnimAtTime("open", fTime);
            if (sOpenSound != null && sOpenSound.length() > 0)
                PlaySoundAtTime(sOpenSound, GetPosition(), fTime );
            
            // Gdyby cos nie dzialalo ze zbieraniem przedmiotow wewnatrz - trzeba odremowac:
            //ClearFlag(EFlags._BLOCK_TRACE);
        }
    }

    void ServerExecuteRequestFromClient()
    {
        if ( CanExecuteRequestFromClient() )
        {
            ExecuteDo();
            // dla niektorych triggerow bedzie mozna wybrac SendClientExecute();
            MsgSendClientExecute();
        }
    }
    
    /** Podaje opis obiektu
     * @return string z opisem obiektu (<code>String</code>)
     */
    String GetTip()
    {
        return "&Open&";
    }
    
    /** Powoduje aktywacje obiektow znajdujacych sie wewnatrz pojemnika
     *  (przyattachowanych)
     */
    void ActivateInsideObjects(boolean bActivate)
    {
        ControlObject aObjs[] = GetChildren();
        if (aObjs != null)
        {
            Vector vMinExt = new Vector(),
                vMaxExt = new Vector();
            GetMeshExtents(vMinExt, vMaxExt);
            vMaxExt.fY -= fUpperExtentsEpsilon;
            
            for (int i=0; i<aObjs.length; i++)
            {
                if (aObjs[i] instanceof MeshObject)
                {
                    MeshObject obj = (MeshObject)aObjs[i];
                    Vector vPos = obj.GetPositionVectorVolatile();
                    vPos = PointWorldToLocal(vPos);
                    
                    // tylko, jesli wewnatrz extentow
                    if (vMinExt.fX < vPos.fX && vPos.fX < vMaxExt.fX &&
                        vMinExt.fY < vPos.fY && vPos.fY < vMaxExt.fY &&
                        vMinExt.fZ < vPos.fZ && vPos.fZ < vMaxExt.fZ)
                    {
                        if (bActivate)
                            obj.Appear(null);
                        else
                            obj.Disappear(null);
                    }
                }
            }
        }
    }
    
    void MsgSendClientState()
    {
        MessageNew( MSG_CLIENT_STATE );
        MessagePackInt( nState );
        if ( nState == _STATE_OPENED )
            MessagePackFloat( m_fExecuteTime );
        MessageSend();
    }
    
    void MsgRecvClientState()
    {
        int nNewState = MessageUnPackInt();
        if ( nNewState == _STATE_OPENED )
            m_fExecuteTime = MessageUnPackFloat();
        MessageEnd();
        
        if ( nNewState == _STATE_OPENED &&
             nNewState != nState )
            Execute( m_fExecuteTime );
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Container.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_CONTAINER)
        {	
			super.SGLoadChunk(cFC);
			
			nState = cFC.LoadInt();
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Container.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_CONTAINER, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveInt(nState);
		
        cFC.delete();
    }	    			

	public void SGUpdate()
	{
		FileChunk.Log("[Container.SGUpdate]\n");
		super.SGUpdate();		
	}    
}

