import java.util.*;

public class Locker extends TriggerObject
{
    public String sDoorElementMask = "Drzwi";//#LOCKER;Prefix of doors element name (postfix must be '00' - '99')
    public String sPlaceholdersMask = "Place";//#LOCKER;Prefix of dummies with positions of items inside
    
    public String sOpenSound = "Data/Sounds/DoorD_Open";//#LOCKER
    public String sCloseSound = "Data/Sounds/DoorD_Close";//#LOCKER
    
    public String sDefaultItems = null;//#LOCKER;All possible items than can be found in this locker with chances to appear [%]. Syntax: '%chance1 item1[,%chance2 item2..]'
    public int nItemsChance = -1;//#LOCKER;Chance to find item on single shelf [%]
    
    ArrayList m_aDoorState = new ArrayList();
    
    public Locker()
    {
        super();
        ActiveElement = null; // to musi byc zrobione recznie, bo potrzebne nazwy elementow do animacji
    }
    
    public void OnCreate()
    {
        super.OnCreate();

        bSG = true;
        
        SetFlag(EFlags._COLLISION | EFlags._BLOCK_MOVE | EFlags._BLOCK_TRACE |
                EFlags._FXPARTICLE_COLLISION |
                EFlags._NOT_STATIC);
        
        ClearFlag(EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH);

        SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        SetTraceCollType(ECollTraceFlags._HULL);

        if (GetModule() instanceof ChromeModuleSingle)
        {
            ChromeModuleSingle module = (ChromeModuleSingle)GetModule();
            if (sDefaultItems == null)
                sDefaultItems = module.sLockerDefaultItems;
            if (nItemsChance < 0)
                nItemsChance = module.nLockerItemsChance;
        }
        
        InitDoors();
        InitObjectsInside();
    }
    

    void InitDoors()
    {
        // zainicjowanie nazwy pliku z animacjami
        int nDotPos = MeshName.lastIndexOf('.');
        if (nDotPos > 0)
            InitAnimSeqFile(MeshName.substring(0, nDotPos) + ".scr");
        else
            CrashLog("[Locker.OnCreate] ERROR: Incorrect mesh name (" + MeshName + ")\n");


        // wyszukanie elementow o podanej masce (+ dwucyfrowy numerek kolejny)
        int nID;
        String sElemName;
        aActiveElements = new java.util.Vector();
        
        for(int i=0; i<200; i++)
        {
            sElemName = sDoorElementMask + ( (i<10) ? "0" : "" ) + i;
            nID = GetElementID(sElemName);
            if (nID != -1)
            {
                // dodaj do tablicy elementow aktywnych
                aActiveElements.addElement(new java.lang.Integer(nID));

                // zapamietaj identyfikatory animacji
                LockerDoorState doorState = new LockerDoorState();
                doorState.nOpenAnimID = GetAnimID(sElemName, "open");
                doorState.nCloseAnimID = GetAnimID(sElemName, "close");
                m_aDoorState.add(doorState);
            }
        }
    }

    
    void InitObjectsInside()
    {
        if (sDefaultItems != null)
        {
            RandomItemizer itemizer = new RandomItemizer();
            itemizer.Unlock();

            // sparsuj Stringa z mozliwymi itemami
            StringTokenizer tokenizer = new StringTokenizer(sDefaultItems, " ,%", true);
            String token, name = null;
            int probability = 0;
            
            while (tokenizer.hasMoreTokens())
            {
                token = tokenizer.nextToken();
                if (token.equals("%"))
                {
                    if (tokenizer.hasMoreTokens())
                    {
                        token = tokenizer.nextToken();
                        try 
                        {
                            probability = Integer.valueOf(token).intValue();
                        } 
                        catch (java.lang.NumberFormatException e)
                        {
                            LogD("Incorrect probability value!\n");
                            probability = 0;
                        }
                    }
                    else
                    {
                        LogD("Default items names parsing error!\n");
                        return;
                    }
                }
                else if (token.equals(" "))
                    ;
                else if (token.equals(",") && name != null)
                {
                    if (GetClassID(name) != 0)
                        itemizer.AddItem(name, probability); 
                }
                else
                    name = new String(token);
            }

            if (GetClassID(name) != 0)
                itemizer.AddItem(name, probability); 

            itemizer.Lock();

            // wylosuj elementy i umiesc je na polkach
            java.util.Vector places = GetElementsWithMask(sPlaceholdersMask);
            for (int i = 0; i < places.size(); i++)
            {
                if (RandomInt(100) < nItemsChance)
                {
                    String itemClass = itemizer.DrawItemName();
                    if (itemClass != null)
                    {
                        InvObject item = (InvObject)CreateObject(itemClass); 
                        if (item != null)
                        {
                            AttachChild(item, ((Integer)places.elementAt(i)).intValue());
                            Vector vPos = new Vector( RandomFloat(-15, 15), 0, RandomFloat(-7, 7));
                            item.SetPosition(vPos);
                        }
                    }
                }
            }
        }
    }
    
    
    String GetTip()
    {
        int index = FindElementIndex(nActiveElement);
        if (index < 0)
            return "";
        
        LockerDoorState doorState = (LockerDoorState)m_aDoorState.get(index);
        if (doorState.nState == LockerDoorState._OPENED)
            return "&close&";
        else
            return "&open&";
    }

    
    public boolean IsActive(ControlObject Activator, int nCheckingElement)
    {
        int index = FindElementIndex(nCheckingElement);
        if (index < 0)
            return false;
        
        LockerDoorState doorState = (LockerDoorState)m_aDoorState.get(index);
        if (doorState.nState == LockerDoorState._IS_OPENING ||
            doorState.nState == LockerDoorState._IS_CLOSING)
            return false;
        else
            return true;
    }
    
    
    public void Execute(Actor executor)
    {
        super.Execute(executor);
        int index = FindElementIndex(nActiveElement);
        if (index >= 0)
        {
            LockerDoorState doorState = (LockerDoorState)m_aDoorState.get(index);
            if (doorState.nState == LockerDoorState._CLOSED)
            {
                PlayAnim(doorState.nOpenAnimID, 1);
                if (sOpenSound != null)
                    PlaySound(sOpenSound);
                doorState.nState = LockerDoorState._IS_OPENING;
            }
            else if (doorState.nState == LockerDoorState._OPENED)
            {
                PlayAnim(doorState.nCloseAnimID, 1);
                if (sCloseSound != null)
                    PlaySound(sCloseSound);
                doorState.nState = LockerDoorState._IS_CLOSING;
            }
        }
    }

    
    public void OnAnimEnd(int nAnimID)
    {
       for (int i = 0; i < m_aDoorState.size(); i++)
       {
           LockerDoorState doorState = (LockerDoorState)m_aDoorState.get(i);
           if (nAnimID == doorState.nOpenAnimID)
               doorState.nState = LockerDoorState._OPENED;
           else if (nAnimID == doorState.nCloseAnimID)
               doorState.nState = LockerDoorState._CLOSED;
       }
    }
    
    
    
    private int m_nLastFoundElement = -1;
    private int m_nLastFoundIndex = -1;
    protected int FindElementIndex(int nElement)
    {
        if (nElement == m_nLastFoundElement)
            return m_nLastFoundIndex;
        
        for (int i = 0; i < aActiveElements.size(); i++)
            if (((Integer)aActiveElements.elementAt(i)).intValue() == nElement)
            {
                m_nLastFoundElement = nElement;
                m_nLastFoundIndex = i;
                return i;
            }
        return -1;
    }
    
    //*********************************************************
    //                          SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[Locker.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_LOCKER)
        {
            super.SGLoadChunk(cFC);
            if (m_aDoorState != null)
            {
                int size = cFC.LoadInt();
                if (size >= 0)
                    for (int i = 0; i < size; i++)
                        ((LockerDoorState)m_aDoorState.get(i)).nState = cFC.LoadInt();
            }
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[Locker.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_LOCKER, cFCParent);
        
        super.SGSaveChunk(cFC);
        if (m_aDoorState != null)
        {
            cFC.SaveInt(m_aDoorState.size());
            for (int i = 0; i < m_aDoorState.size(); i++)
                cFC.SaveInt(((LockerDoorState)m_aDoorState.get(i)).nState);
        }
        else
            cFC.SaveInt(-1);
        
        cFC.delete();
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[Locker.SGUpdate]\n");
        super.SGUpdate();
    }

}


class LockerDoorState 
{
    public final static int _CLOSED       = 0;
    public final static int _OPENED       = 1;
    public final static int _IS_OPENING   = 2;
    public final static int _IS_CLOSING   = 3;
    
    public int nState = _CLOSED;
    
    public int nOpenAnimID = -1;
    public int nCloseAnimID = -1;
}


class RandomItemizer
{
    class ItemData implements Comparable
    {
        public String sName = null;
        public int nProbability;

        public ItemData(String name, int prob)
        {
            sName = name;
            nProbability = prob;
        }
        
        public int compareTo(Object to)
        {
            int toVal = ((ItemData)to).nProbability;
            return (nProbability > toVal) ? -1 : (nProbability < toVal) ? 1 : 0;
        }
    }
    
    ArrayList m_aItemDataList = new ArrayList();
    ItemData[] m_aItemData = null;
    int m_nProbSum = 0;
    
    boolean bLocked = false;
    
    
    public void Lock()
    {
        if (!bLocked)
        {
            bLocked = true;
            int size = m_aItemDataList.size();
            m_aItemData = new ItemData[size];
            m_nProbSum = 0;
            for (int i = 0; i < size; i++)
            {
                ItemData itemData = (ItemData)m_aItemDataList.get(i);
                m_aItemData[i] = itemData;
                m_nProbSum += itemData.nProbability;
            }
            Arrays.sort(m_aItemData);
        }
    }
    
    public void Unlock()
    {
        bLocked = false;
    }
    
    public boolean AddItem(String sName, int nProbability)
    {
        if (!bLocked)
            m_aItemDataList.add(new ItemData(sName, nProbability));
        return !bLocked;
    }

    
    public String DrawItemName()
    {
        if (bLocked && 
            m_aItemData != null && 
            m_aItemData.length > 0)
        {
            int nRandom = GameObject.RandomInt(m_nProbSum);
            GameObject.LogR("Locker.nRandom " + nRandom);
            int nLastIndex = 0;
            int nObjProb = 0;
            do 
            {
                nObjProb += m_aItemData[nLastIndex].nProbability;
                if (nRandom <= nObjProb)
                    break;
                nLastIndex++;
            } while (nLastIndex < m_aItemData.length);
            int nRandIdx = nLastIndex;
            /*if (nLastIndex > 0)
                nRandIdx = GameObject.RandomInt(nLastIndex);*/
            return m_aItemData[nRandIdx].sName;
        }
        return null;
    }
}