/** klasa odpowiedzialna za system alarmow na levelu
 */
public class AlarmsManager extends GameObject
{
    static AlarmsManager m_AlarmsManager = null;
    
	/** defaultowa wartosc identyfikatora grupy alarmu */
	static int _DEFAULT_ALARM_ID = 0;
    /** Maksymalny czas przez jaki pozostaje wlaczone alarm */
    static float _MAX_ALARM_DURATION = 60f;
    /** dzwiek alarmu w bazie */
    static String _ALARM_SOUND_NAME = "Data/Sounds/alarm.wav";
	/** minimalny czas pomiedzy update alarmu */
	static float _MIN_TIME_BTW_UPDATES = 2f;
	
    /** nastepny automatyczny id alarmu */
    static int _AUTO_ALARM_ID = 1000000;
    
	/** grupy obiektow alarmu */
	java.util.Vector m_AlarmGroups = new java.util.Vector();
	
	/** czy jest wiecej niz jeden punkt alarmu */
	boolean m_bAlarmMultiple = false;
	
	/** modul, w ktorym jest AlarmsManager */
	ChromeModuleSingle m_Module = null;
	
	void SetModule(ChromeModuleSingle module)
	{
		m_Module = module;
        m_AlarmsManager = this;
	}
    
    static int GetNextAutoAlarmID()
    {
        return _AUTO_ALARM_ID++;
    }
	
    /** alarmsmanager jest tworzony w onpostcreate chromemodulesingle */
	public void OnCreate()
	{
		bSG = false;
		
		// wlaczenie OnFrame i handlerow czasowych
		ActivateOnModule();
		
        // zainicjowanie swiatel alarmowych
        InitLightSignals();
		
		// zainicjowanie terminali alarmowych
		InitAlarmTerminals();
		
		CallMethodEvery("UpdateAlarmGroups", _MIN_TIME_BTW_UPDATES);
	}
	
	public void UpdateAlarmGroups()
	{
		float fCurrTime = GetTime();
		for (int i = 0; i < m_AlarmGroups.size(); i++)
			((AlarmGroup) m_AlarmGroups.elementAt(i)).CheckForDisable(fCurrTime);
	}
	
	public void OnDestroy()
	{
		DisableCallMethod("UpdateAlarmGroups");
	}

	/** zawolane z levelu OnPostCreate, by ustawic zolnierzy w odpowiednim stanie
	 */
	void StartAlarmSystem()
	{
		// inicjujemy alarmerow - gosci ktorzy wyskakuja gdy wlaczy sie alarm		
		InitAlarmGuards();
	}
	
    void InitLightSignals()
	{
        GameObject aTab[] = m_Module.GetObjectsFromClass("LightSignal");
		
		if (aTab.length == 0)
			return;
		
		for (int i = 0; i < aTab.length; i++)
		{
			LightSignal light = (LightSignal) aTab[i];
			light.Stop();
			
			AlarmGroup alarmGroup = GetAlarmGroupByID(light.m_iAlarmID);
			if (alarmGroup == null)
			{
				alarmGroup = new AlarmGroup(light.m_iAlarmID);
				m_AlarmGroups.addElement(alarmGroup);
			}
			alarmGroup.AddAlarmLight(light);
		}
	}
	
	/** zainicjowanie terminali z alarmem
	 */
	void InitAlarmTerminals()
	{
        GameObject aTab[] = m_Module.GetObjectsFromClass("AlarmTerminal");
		
		if (aTab.length == 0)
			return;
		
		for (int i = 0; i < aTab.length; i++)
		{
			AlarmTerminal terminal = (AlarmTerminal) aTab[i];
			
			AlarmGroup alarmGroup = GetAlarmGroupByID(terminal.m_iAlarmID);
			if (alarmGroup == null)
			{
				alarmGroup = new AlarmGroup(terminal.m_iAlarmID);
				m_AlarmGroups.addElement(alarmGroup);
			}
			alarmGroup.AddAlarmTerminal(terminal);
		}
	}
	
	/** zainicjowanie zolnierzy reagujacych na alarm
	 */
	void InitAlarmGuards()
	{
        /*GameObject aTab[] = m_Module.GetObjectsFromClass("AIAlarmed");
		
		if (aTab.length == 0)
			return;
		
		for (int i = 0; i < aTab.length; i++)
		{
			AIAlarmed guard = (AIAlarmed) aTab[i];
			
			AlarmGroup alarmGroup = GetAlarmGroupByID(guard.m_iAlarmID);
			if (alarmGroup == null)
			{
				alarmGroup = new AlarmGroup(guard.m_iAlarmID);
				m_AlarmGroups.addElement(alarmGroup);
			}
			alarmGroup.AddAlarmGuard(guard);
		}*/
	}

	/** wlaczenie alarmu dla wybranej grupy obiektow alarmu
	 */
	void EnableAlarm(boolean bEnable, int iAlarmGroupID, float fDuration)
	{
		AlarmGroup alarmGroup = GetAlarmGroupByID(iAlarmGroupID);
		
		if (alarmGroup != null)
			alarmGroup.EnableAlarm(bEnable, GetTime(), fDuration); // podajemy czas, by bylo wiadomo kiedy wylaczyc
	}
    
	/** wlacza / wylacza alarm na calym levelu
	 */
    void EnableAlarm(boolean bEnable, float fDuration)
    {
		float fCurrTime = GetTime();
		
		for (int i = 0; i < m_AlarmGroups.size(); i++)
		{
			AlarmGroup alarmGroup = ((AlarmGroup) m_AlarmGroups.elementAt(i));
			alarmGroup.EnableAlarm(bEnable, fCurrTime, fDuration);
		}
    }

	/** czy jest wlaczony alarm NA CALYM LEVELU
	 */
	boolean IsAlarmEnabled()
	{
		if (m_AlarmGroups.size() == 0)
			return false;
		boolean bEnabled = true;
		for (int i = 0; i < m_AlarmGroups.size(); i++)
		{
			AlarmGroup alarmGroup = (AlarmGroup) m_AlarmGroups.elementAt(i);
			bEnabled = bEnabled && alarmGroup.m_bAlarmEnabled;
		}
		return bEnabled;
	}
	
	/** czy jest wlaczony alarm dla grupy
	 */
	boolean IsAlarmEnabled(int iAlarmID)
	{
		AlarmGroup group = GetAlarmGroupByID(iAlarmID);
		
		if (group == null)
			return false;
		
		return group.IsAlarmEnabled();
	}
	
	AlarmGroup GetAlarmGroupByID(int iAlarmGroupID)
	{
		for (int i = 0; i < m_AlarmGroups.size(); i++)
		{
			AlarmGroup alarmGroup = (AlarmGroup) m_AlarmGroups.elementAt(i);
			if (alarmGroup.m_iAlarmID == iAlarmGroupID)
				return alarmGroup;
		}
		return null;
	}
	
	java.util.Vector GetTerminalsByID(int iAlarmGroupID)
	{
		AlarmGroup group = GetAlarmGroupByID(iAlarmGroupID);
		
		if (group == null)
			return null;
		
		if (group.m_aAlarmTerminals.size() == 0)
			return null;
		
		return group.m_aAlarmTerminals;
	}
    
    public void LogAlarmsGroups()
    {
        for (int i = 0; i < m_AlarmGroups.size(); i++)
            ((AlarmGroup) m_AlarmGroups.get(i)).LogAlarmGroup();
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[AlarmsManager.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_ALARMS_MANAGER)
        {
			super.SGLoadChunk(cFC);
			
			// grupy obiektow alarmu
			m_AlarmGroups = cFC.LoadObjectsVectorRef();			
        }
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[AlarmsManager.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_ALARMS_MANAGER, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		// grupy obiektow alarmu
		cFC.SaveObjectsVectorRef(m_AlarmGroups);
		
		cFC.delete();
	}				
	
	public void SGUpdate()
	{
		FileChunk.Log("[AlarmsManager.SGUpdate]\n");
		
		super.SGUpdate();
	
	}			
}

class AlarmGroup implements ISaveGame
{
	/** identyfikator tego alarmu */
	int m_iAlarmID = AlarmsManager._DEFAULT_ALARM_ID;
    /** Znacznik, ze aktualnie alarm jest wlaczony */
	boolean m_bAlarmEnabled = false;
	/** kiedy nalezy wylaczyc alarm */
	float m_fDisableTime = 0f;
	
	/** zolnierze, ktorzy moga zostac zaalarmowani przez zolnierza wlaczajacego alarm */
	java.util.Vector m_aAlarmedGuards = new java.util.Vector();
	/** terminale, ktorymi mozna wlaczyc / wylaczyc alarm */
	java.util.Vector m_aAlarmTerminals = new java.util.Vector();
    /** Bajeranckie koguty od alarmu */
	java.util.Vector m_aAlarmLights = new java.util.Vector();
	
	// Tylko do SaveGame
	AlarmGroup()
	{
		m_iAlarmID = 0;
	}

	AlarmGroup(int iAlarmID)
	{
		m_iAlarmID = iAlarmID;
	}
	
	void AddAlarmLight(LightSignal light)
	{
		m_aAlarmLights.addElement(light);
	}
	
	/*void AddAlarmGuard(AIAlarmed guard)
	{
		m_aAlarmedGuards.addElement(guard);
	}*/
	
	void  AddAlarmTerminal(AlarmTerminal terminal)
	{
		m_aAlarmTerminals.addElement(terminal);
	}
	
    void LightDestoyred(LightSignal cSignal)
    {
        m_aAlarmLights.remove(cSignal);
    }
    
    void EnableAlarm(boolean bEnable, float fTime, float fDuration)
    {
		m_bAlarmEnabled = bEnable;
		
		if (bEnable && (fDuration != 0f))
			m_fDisableTime = fTime + fDuration;
				
		for (int i = 0; i < m_aAlarmLights.size(); i++)
		{
			if (bEnable)
			{
			    ((LightSignal) m_aAlarmLights.elementAt(i)).Start();
				((LightSignal) m_aAlarmLights.elementAt(i)).AttachAlarmSound();
			}
			else
			{				
			    ((LightSignal) m_aAlarmLights.elementAt(i)).Stop();
				((LightSignal) m_aAlarmLights.elementAt(i)).DetachAlarmSound();
			}
		}

		for (int i = 0; i < m_aAlarmTerminals.size(); i++)
		{
			if (bEnable)
				((AlarmTerminal) m_aAlarmTerminals.elementAt(i)).SetSkin("AlertOn");
			else
				((AlarmTerminal) m_aAlarmTerminals.elementAt(i)).SetSkin("AlertOff");
		}
		
		if (bEnable)
			AlarmGuards();
	}
	
	void CheckForDisable(float fTime)
	{		
		if (!m_bAlarmEnabled)
			return;
		
		if (fTime > m_fDisableTime)
			EnableAlarm(false, 0f, 0f);
	}
	
	void AlarmGuards()
	{
		/*for (int i = 0; i < m_aAlarmedGuards.size(); i++)
			((AIAlarmed) m_aAlarmedGuards.elementAt(i)).ActivateAlarmed();*/
	}
	
	boolean IsAlarmEnabled()
	{
		return m_bAlarmEnabled;
	}
    
    void LogAlarmGroup()
    {
        GameObject.CrashLog("AlarmGroup:" + m_iAlarmID + "\n");
        GameObject.CrashLog("AlarmTerminals:" + m_aAlarmTerminals.size() + "\n");
        for (int i = 0; i < m_aAlarmTerminals.size(); i++)
            GameObject.CrashLog("Terminal:" +
                                ((AlarmTerminal) m_aAlarmTerminals.get(i)).m_iAlarmID +
                                "\n");
        GameObject.CrashLog("AlarmLights:" + m_aAlarmLights.size() + "\n");
        for (int i = 0; i < m_aAlarmLights.size(); i++)
            GameObject.CrashLog("Light:" +
                                ((LightSignal) m_aAlarmLights.get(i)).m_iAlarmID +
                                "\n");
        GameObject.CrashLog("AlarmedGuards:" + m_aAlarmedGuards.size() + "\n");
        /*for (int i = 0; i < m_aAlarmedGuards.size(); i++)
            GameObject.CrashLog("Guard:" +
                                ((AIAlarmed) m_aAlarmedGuards.get(i)).m_iAlarmID +
                                "\n");*/
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[AlarmGroup.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_ALARM_GROUP)
        {
			m_iAlarmID = cFC.LoadInt();
			
			// znacznik, ze aktualnie alarm jest wlaczony
			m_bAlarmEnabled = cFC.LoadBool();
			// kiedy nalezy wylaczyc alarm
			m_fDisableTime = cFC.LoadFloat();			
		
			// zolnierze, ktorzy moga zostac zaalarmowani przez zolnierza wlaczajacego alarm
			m_aAlarmedGuards = cFC.LoadObjectsVectorRef();
			// terminale, ktorymi mozna wlaczyc / wylaczyc alarm
			m_aAlarmTerminals = cFC.LoadObjectsVectorRef();
			// Bajeranckie koguty od alarmu
			m_aAlarmLights = cFC.LoadObjectsVectorRef();			
        }
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[AlarmGroup.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_ALARM_GROUP, cFCParent);

		cFC.SaveInt(m_iAlarmID);			
		
		// znacznik, ze aktualnie alarm jest wlaczony
		cFC.SaveBool(m_bAlarmEnabled);
		// kiedy nalezy wylaczyc alarm
		cFC.SaveFloat(m_fDisableTime);

		// zolnierze, ktorzy moga zostac zaalarmowani przez zolnierza wlaczajacego alarm
		cFC.SaveObjectsVectorRef(m_aAlarmedGuards);
		// terminale, ktorymi mozna wlaczyc / wylaczyc alarm
		cFC.SaveObjectsVectorRef(m_aAlarmTerminals);
		// Bajeranckie koguty od alarmu
		cFC.SaveObjectsVectorRef(m_aAlarmLights);

		cFC.delete();
	}				
	
	public void SGUpdate()
	{
		FileChunk.Log("[AlarmGroup.SGUpdate]\n");			
				
		EnableAlarm(m_bAlarmEnabled, 0f, 0f);
	}			
}
