public class Dialog extends GameObject
{
    static Dialog cPlayingDialog = null;
    
    /** Przerwa miedzy tekstami w dialogu */
    public float fWavePause = 0.5f;//#DIALOG;Time interval between text lines
    
    /** Pojedyncza kwestia wypowiadana przez   */
    public DialogLine[] aLines;//#\DIALOG;Line of dialog text
    
    /** Gdy inny dialog jest odgrywany okresla czy podczas odegrania tego
     * dialogu zkolejkowac jego odegranie czy go poprostu pominac. */
    public boolean bQueuing = true;//#\DIALOG;
    
    /** Module do odpalania tekstow */
    ChromeModuleSingle cModule = null;
    
    /** Numer aktualnie odgrywanej kwestii dialogu */
    int nCurrentLine = -1;
    
    /** Sciezka do dzwiekow pobrana z levelu */
    String sWavePath = "";

    /** Czas trwania dialogu, wyliczany podczas odgrywania */
    float fDuration = 0.0f;
    
    /** Czas kiedy rozpoczete zostalo odgrywanie dialogu */
    float fStartTime = 0.0f;
	
	Dialog()
	{
		super();
		
		bSG = true;
	}
    
    /** Metoda wolana przez ChromeModuleSingle dla wszystkich obiektow Dialog */
    public void Init()
    {
        try {
            cModule = (ChromeModuleSingle)GetModule();
                        
            // dla wszystkich kwestii updejtujemy osoby je mowiace
            for(int i=0; i<aLines.length; ++i)
            {
                aLines[i].cCharacter = cModule.UpdateCharacter(aLines[i].cCharacter);
                //LogR("aLines[i].cCharacter="+aLines[i].cCharacter+"\n");
            }
            
        } catch(ClassCastException e) {
            CrashLog("[Dialog.OnCreate] Dialog on non single player module!\n");
        }
    }
    
    float fPlayDuration = 0.0f;
    
    public float GetPlayDuration()
    {
        return fPlayDuration;
    }
    
    public boolean IsActorInDialog(AnimActor cActor)
    {
        for (int i = 0; i < aLines.length; i++)
        {
            if (aLines[i].cCharacter.cActor == cActor)
                return true;
        }
        return false;
    }
        
    
    /** Odegranie dialogu.
     * @return czas trwania calego dialogu
     */
    public void Play()
    {
        
        if (GetModule() instanceof ChromeModuleSingle)
        {
            if (((ChromeModuleSingle)GetModule()).m_bMissionEnded)
                return;
        }
        
        fPlayDuration = 0.0f;
        
        //CrashLog("[Dialog.Play] this='"+this+"' cPlayingDialog='"+cPlayingDialog+"'\n");
        // sprawdzamy czy odgrywa sie jakas scenka juz
        if(cPlayingDialog != null)
        {
            // odgrywa sie scenka, trzeba sie zdecydowac czy kolejkujemy
            if(bQueuing)
            {
                // liczymy kiedy sie skonczy odegrywany dialog
                float fCallInterval = cPlayingDialog.fDuration - (GetTime() - cPlayingDialog.fStartTime);
                //CrashLog("[Dialog.Play] Dialog '"+cPlayingDialog.GetName()+"' already plying! Delaying to "+fCallInterval+"\n");
                CallMethodIn("Play", fCallInterval);
            }
            return;
        }

        if(nCurrentLine > -1)
        {
            CrashLog("[Dialog.Play] Dialog already playing!\n");
            return;
        }

        fStartTime = GetTime();
        cPlayingDialog = this;
        
        // liczymy czas trwania calej scenki
        fDuration = 0;
        for (int i=0; i<aLines.length; i++)
            fDuration += aLines[i].GetDuration();
        
        // dodajemy przerwy miedzy wave'ami
        fDuration += fWavePause * (aLines.length-1);

        // ustawiamy odgrywanie pierwszej kwestii
        nCurrentLine = -1;

        PlayLine();
        
        CallMethodIn("ZeroPlayingDialog", fDuration);
        
        LogR("[Dialog.Play] this='"+GetName()+"' fDuration="+fDuration+"\n");
        fPlayDuration = fDuration;
    }

    void ZeroPlayingDialog()
    {
        cPlayingDialog = null;
    }
    
    void Stop()
    {
        if (nCurrentLine >= aLines.length)
		{            
			nCurrentLine = -1;			
			return;
		}
        
        DialogLine cLine = aLines[nCurrentLine];
        // wyszukujemy kto to ma gadac
        DialogCharacter cSpeaker = cLine.cCharacter;
        
        if(cSpeaker != null)
        {
            if(cSpeaker.nTexture != 0)
            {
                ((HUDCompNiunia)cModule.cNiunia).Deactivate();
            }
            else
            {
                if(cSpeaker.cTalkingHeadModule != null)
                {
                    cModule.cNiunia.Deactivate();
                }
                
                if(cSpeaker.cActor!=null)
                {
                    if(cSpeaker.bSimpleText)
                    {
                        cSpeaker.cActor.StopText();
                    }
                    else
                    {
                        cSpeaker.cActor.StopSpeak();
                    }
                }
            }
        
        }
        
        DisableCallMethods();
        ZeroPlayingDialog();
    }
    
    protected void PlayLine()
    {
        if (GetModule() instanceof ChromeModuleSingle)
        {
            if (((ChromeModuleSingle)GetModule()).m_bMissionEnded)
                return;
            else
                if (Precache.bPrecaching ||
                    ((ChromeModuleSingle)GetModule()).m_cBlackLoading.IsActuallyVisible())
                {
                    CallMethodIn("PlayLine", 0.1f);
                    return;
                }
        }
        //CrashLog("[Dialog.PlayLine]\n");
        /*TODO
        if(nCurrentLine > 0)
        {
            // wyslanie informacji o zakonczeniu odtwarzania
            DialogLine cPrevLine = aLines[nCurrentLine];
            cPrevLine.WaveFinished();
        }
        */
		
		nCurrentLine++;
		
		// koniec scenki
		if (nCurrentLine >= aLines.length)
		{            
			nCurrentLine = -1;			
			return;
		}
        
        DialogLine cLine = aLines[nCurrentLine];
        
        float fTime = cLine.PlayWave();
        String sText = cLine.GetText();
        String sWave = cLine.GetWave();
        
        // wyszukujemy kto to ma gadac
        DialogCharacter cSpeaker = cLine.cCharacter;
        
        if(cSpeaker != null)
        {
            boolean bShownText = true;
            if(cSpeaker.nTexture != 0)
            {
                // odpalamy teksture
                cModule.cNiunia.Message(cSpeaker.nTexture, sText, fTime);
            }
            else
            {
                if(cSpeaker.cTalkingHeadModule != null)
                {
                    // odpalamy gadajaca glowe
                    cModule.cNiunia.Message( sWave, sText, fTime, cSpeaker.cTalkingHeadModule, cLine.nPlayingWave);
                    bShownText = true;
                }
                
                // odpalamy aktorowi gadanie
                if(cSpeaker.cActor!=null)
                {
                    if(cSpeaker.bSimpleText)
                        cSpeaker.cActor.SpeakText(sText, fTime);
                    else
                    {
                        cSpeaker.cActor.bPlayWave = false;
                        cSpeaker.cActor.Speak(sWave);
                        cSpeaker.cActor.SetExternalPlayingWave(cLine.nPlayingWave);
                    }
                }
            }
        
		    // dodajemy tekst ze sceny do okna logow
		    if(cModule.MainPlayer.HUD.GetHUDLogs() != null)
		    	cModule.MainPlayer.HUD.GetHUDLogs().InputText(cSpeaker.sName+": "+sText);
        }
        
        // nastepna kwestia
        if (nCurrentLine < aLines.length)
            CallMethodIn("PlayLine", fTime + fWavePause);
    }

    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[Dialog.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_DIALOG)
        {
			super.SGLoadChunk(cFC);
			
			// Numer aktualnie odgrywanej kwestii dialogu
			nCurrentLine = cFC.LoadInt();
		    
			// Sciezka do dzwiekow pobrana z levelu
			sWavePath = cFC.LoadString();
		
			// Czas trwania dialogu, wyliczany podczas odgrywania
			fDuration = cFC.LoadFloat();
			// Czas kiedy rozpoczete zostalo odgrywanie dialogu
			fStartTime = cFC.LoadFloat();			
			
			fPlayDuration = cFC.LoadFloat();
			
			cFC.LoadObjectsTable(aLines);
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Dialog.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_DIALOG, cFCParent);
		
		super.SGSaveChunk(cFC);
		                
		// Numer aktualnie odgrywanej kwestii dialogu
		cFC.SaveInt(nCurrentLine);
	    
		// Sciezka do dzwiekow pobrana z levelu
		cFC.SaveString(sWavePath);
		
		// Czas trwania dialogu, wyliczany podczas odgrywania
		cFC.SaveFloat(fDuration);
		// Czas kiedy rozpoczete zostalo odgrywanie dialogu
		cFC.SaveFloat(fStartTime);
		
		cFC.SaveFloat(fPlayDuration);
		
		cFC.SaveObjectsTable(aLines);
		
		cFC.delete();
	}				
	
	public void SGUpdate()
	{
		FileChunk.Log("[Dialog.SGUpdate]\n");
		
		super.SGUpdate();
		
		for (int i=0; i < aLines.length; i++)
			aLines[i].BreakWave();				
		
		DisableCallMethod( "Play" );
		DisableCallMethod( "PlayLine" );
		ZeroPlayingDialog();
		
		if (nCurrentLine != -1)
		{
			nCurrentLine--;
            //missionend w module moze byc jeszcze na true
            CallOnFrameEnd("PlayLine");
			CallMethodIn("ZeroPlayingDialog",fDuration);
			fPlayDuration = fDuration;			
			for (int i=0; i < aLines.length; i++)
				aLines[i].SGUpdate();					
		}
	}			
    
}
