/** Klasa reprezentuje pojedyncza kwestie mowiona w gadce */
public abstract class DialogLine extends GameObject
{
    public static float _DEFAULT_WAVE_DURATION = 3.0f;
    public static float _MIN_WAVE_DURATION = 3.0f;
    public static float _MAX_WAVE_DURATION = 15.0f;
    public static float _DEFAULT_CHAR_DURATION = 0.06f;
    
    {sEditorComment = "Dialog text line";}
    
    public DialogCharacter cCharacter = null;//#;Character that read line
    
    public String sTextID = "";//#;Text identifier from localized strings
    
    /** Nazwa pliku na dysku z wavem gadki */
    String sWaveFileName = null;
    
    /** Czas trwania gadki */
    float fDuration = 0.0f;
	float fTimeHelp = 0.0f;

    /** okresla czy jest plik na dysku z wavem */
    boolean bWaveValid = false;

    int nPlayingWave = 0;
    
    static String CheckFile(String sPath, String sTextID, String sExt)
    {
        String sWaveFileName = null;
        StringBuffer sbOut = new StringBuffer(sTextID);
        int nPointPos = sbOut.length()-1;
                
        if(sbOut.length() == 0)
            sWaveFileName = "";
        else
        {
            if(nPointPos == -1)
                sWaveFileName = sPath + sbOut + sExt;
            else
                sWaveFileName = sPath + sbOut.substring(1, nPointPos) + sExt;
        }
        
        // sprawdzamy czy jest plik na dysku
        java.io.File cFile = new java.io.File(sWaveFileName);
        String sFileName = null;
        try {
            sFileName = cFile.getCanonicalPath();
        } catch (Exception e) {
        }
        return cFile.canRead() ? sFileName : null;
    }
    
    public void OnCreate()
    {
		bSG = false;
		
        try {
            ChromeModuleSingle cModule = (ChromeModuleSingle)GetModule();
                        
            String sWavePath = cModule.GetDialogSoundPath();
            
            // narazie wave nie jest dobry
            bWaveValid = false;
            
            // sprawdzamy czy jest jakis tekst wpisany
            if(sTextID != null)
            {
                if(sWaveFileName == null)
                    sWaveFileName = CheckFile(sWavePath, sTextID, ".wav");
                if(sWaveFileName == null)
                    sWaveFileName = CheckFile("Data\\Sounds\\Others\\", sTextID, ".wav");
                
                if(sWaveFileName == null)
                    sWaveFileName = CheckFile(sWavePath, sTextID, ".ogg");
                if(sWaveFileName == null)
                    sWaveFileName = CheckFile("Data\\Sounds\\Others\\", sTextID, ".ogg");
                
                bWaveValid = (sWaveFileName != null);
            }
            
            if(bWaveValid)
                fDuration = GetWaveDuration( sWaveFileName );
            else
            {
                // liczmy czas trwania na podstawie dlugosci tekstu
                fDuration = _DEFAULT_CHAR_DURATION * Text.Get(sTextID).length();
                if(fDuration < _MIN_WAVE_DURATION) fDuration = _MIN_WAVE_DURATION;
                if(fDuration > _MAX_WAVE_DURATION) fDuration = _MAX_WAVE_DURATION;
            }
        
        } catch(ClassCastException e) {
            CrashLog("[Dialog.OnCreate] Dialog on non single player module!\n");
        }
    }

    String GetText()
    {
        return sTextID;
    }

    String GetWave()
    {
        return sWaveFileName;
    }
    
    /** Odgrywa wave'a gadki jak jest i zwraca jego czas trwania */
    float PlayWave()
    {
		fTimeHelp = GetTime();
		
        if(bWaveValid)
		{
			//CrashLog("[Dialog.PlayWave] File = <"+sWaveFileName+">\n");
			if (nWavePos != -1)
			{
				nPlayingWave = PlayWaveFromPos(sWaveFileName,nWavePos);
				nWavePos = -1;
				return GetDuration()- (float)GetPlayingWaveTimePos(nPlayingWave);
			}
			else
				nPlayingWave = PlayWave(sWaveFileName);
		}

        return GetDuration();
    }
	    
    /** Zwraca dlugosc trwania gadki */
    float GetDuration()
    {
        return fDuration;
    }
    
    void BreakWave()
    {
        // zatrzymujemy wave'a
        if (nPlayingWave != 0)
            if(IsPlayingWave(nPlayingWave))
                StopWave(nPlayingWave);

        nPlayingWave = 0;
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	long nWavePos = -1;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[DialogLine.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_DIALOG_LINE)
        {
			super.SGLoadChunk(cFC);
			
			if (cFC.LoadBool())
			{
				fTimeHelp = cFC.LoadFloat();
                nWavePos = cFC.LoadLong();
			}
			else
				nWavePos = -1;	
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[DialogLine.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_DIALOG_LINE, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		if ((nPlayingWave > 0) && !IsPlayingWave(nPlayingWave))
			nPlayingWave = 0;
		
		cFC.SaveBool(nPlayingWave > 0);
			
		if (nPlayingWave > 0)
		{
			cFC.SaveFloat(GetTime()-fTimeHelp);
			cFC.SaveLong(GetPlayingWavePos(nPlayingWave));
		}
				                		
		cFC.delete();
	}		
    
    public void OnAudioPreReinitialize()
    {
        if (nPlayingWave != 0 && IsPlayingWave(nPlayingWave))
        {
            nWavePos = GetPlayingWavePos(nPlayingWave);
        }
    }
    
    public void OnAudioReinitialize()
    {
        
        if (nWavePos != -1 && nPlayingWave != 0)
            PlayWave();
    }
    
}
