import java.io.*;
import java.net.*;
 

public class Chat extends GameObject implements IChat
{
    boolean m_bClientsChanged = false;
    int m_nNewConnectionState = -1;
    boolean m_bMessagesChanged = false;
    
    
    final static String sm_sHost             = "chat.chromethegame.com";
    final static int    sm_nPort             = 12468;
    final static int    sm_nMsgHistoryLength = 50;

    private
    String m_sNick = "Nick";
    String GetNick(){ return m_sNick; }
    
    private ChatClient m_cClient;
        
    private 
    int     m_nConnectionState  = EChatConnectionState._DISCONNECTED;
    boolean IsConnected(){ return (m_nConnectionState == EChatConnectionState._CONNECTED); }
    
    
    ChatMsgHistory  m_cMsgHistory   = new ChatMsgHistory( sm_nMsgHistoryLength );
    ChatMsgInfoList m_cMsgToOutput  = new ChatMsgInfoList();
        
    IChatWindow     m_cIChatWindow  = null;
       
        
    int METHOD_OutputMsg        = 0;
    int METHOD_TryToConnect     = 0;
    int METHOD_ConnectDo        = 0;
    int METHOD_TryToDisconnect  = 0;
    int METHOD_DisconnectDo     = 0;
    int METHOD_EnableConnect    = 0;
    int METHOD_EnableDisconnect = 0;
    int METHOD_UnlockConnecting = 0;
    
        
    void OnCreate()
    {
        METHOD_OutputMsg        = RegisterCallMethod( "OutputMsg" );
        METHOD_TryToConnect     = RegisterCallMethod( "TryToConnect" );
        METHOD_ConnectDo        = RegisterCallMethod( "ConnectDo" );
        METHOD_TryToDisconnect  = RegisterCallMethod( "TryToDisconnect" );
        METHOD_DisconnectDo     = RegisterCallMethod( "DisconnectDo" );
        METHOD_EnableConnect    = RegisterCallMethod( "EnableConnect" );
        METHOD_EnableDisconnect = RegisterCallMethod( "EnableDisconnect" );
        METHOD_UnlockConnecting = RegisterCallMethod( "UnlockConnecting" );
        CallMethodEvery("CheckClient", 0.1f);
    }
    
    void CheckClient()
    {
        if (m_bClientsChanged)
        {
            m_bClientsChanged = false;
            ClientsChanged();
        }

        if (m_bMessagesChanged)
        {
            m_bMessagesChanged = false;
            OutputMsg();
        }
        if (m_nNewConnectionState != -1)
        {
            SetConnectionState(m_nNewConnectionState);
            m_nNewConnectionState = -1;
        }
    }
    
    void OnDestroy()
    {
    }    
    
    /**
     * Implementacja interfejsu IChat
     */
    public void Acquire( IChatWindow cIChatWindow )
    {
        //CrashLogD(cIChatWindow + "...");
        if ( m_cIChatWindow != null )
            m_cIChatWindow.Acquire( (IChat) null );
                
        m_cIChatWindow = cIChatWindow;
        
        if ( m_cIChatWindow != null )
        {
            m_cIChatWindow.Acquire( (IChat) this );
        }
        //CrashLogD("...OK");
    }
    
    public void Unacquire()
    {
        //CrashLogD("...");
        Acquire( (IChatWindow) null );
        //CrashLogD("...OK");
    }
    
    public void Connect()
    {
        //CrashLogD("");
        m_nNickPostfix = 0;
        
        if ( (! IsConnected()) && (! m_bLockConnecting) )
        {
            OutputMsg("Connecting...",EChatMessageType._SYSTEM);
            m_sNick = NetPlayerProfile.GetCurrentNickName();
            CallMethodIn( METHOD_TryToConnect, 0.1f );
        }
    }
    
    public void Disconnect( String sReason )
    {
        //CrashLogD("");
        if ( IsConnected() )
            TryToDisconnect();
    }
    
    synchronized public void InputMsg( String sText )
    {
        if ( IsConnected() )
        {
            OutputMsg( "> " + sText, NetMsg._COL_INMSG_CHAT_MY );
            
            try 
            {
                if ( m_cClient != null )
                     m_cClient.SendChatMsg( sText + "\n" );
            }
            catch (IOException b) 
            {
                OutputMsg( "Can't send message.", EChatMessageType._ERROR );
	            m_cClient.ConnectionLost();
            }
        }
        else
            OutputMsg( "Can't send message. You are disconnected",EChatMessageType._ERROR );
    }
    
    public void LocalInputMsg( String sText, Vector vColor )
    {
        OutputMsg( sText, vColor );
    }
    
    public ChatState GetState()
    {
        String[]       aClients     = null;
        ChatMsgHistory cMsgHistory  = null;
        
        if ( IsConnected() )
        {
            if ( m_cClient != null )
                 aClients = m_cClient.GetClients();
                    
            synchronized( m_cMsgHistory )
            {
                cMsgHistory = new ChatMsgHistory( m_cMsgHistory );
            }
        }
        
        return new ChatState( aClients, 
                              cMsgHistory, 
                              IsConnected(),
                              GetConnectionStateString( m_nConnectionState ) );
    }
    
    //////////////////////////////////////////////////////////////////////
    
    private boolean m_bConnectEnabled    = true;
    void EnableConnect(){ m_bConnectEnabled = true; }
    
    private boolean m_bDisconnectEnabled = true;
    void EnableDisconnect(){ m_bDisconnectEnabled = true; }
        
    private boolean m_bLockConnecting = false;
    void UnlockConnecting(){ m_bLockConnecting = false; }
    
    final static float _TOGGLE_CONNECTION_DELAY = 1.0f;
    
    void TryToConnect()
    {
        SetConnectionState( EChatConnectionState._CONNECTING );
        ConnectDo();
    }
    
    private synchronized void ConnectDo()
    {
        //CrashLogD("");
        if ( m_cClient != null )
        {
            //CrashLogD("Destroying client...");
            m_cClient.Destroy();
            //CrashLogD("...OK");
            m_cClient = null;
            DisableCallMethod( METHOD_TryToDisconnect );
            
            m_bConnectEnabled = false;
            CallMethodIn( METHOD_EnableConnect, _TOGGLE_CONNECTION_DELAY );
            DisableCallMethod( METHOD_ConnectDo );
            CallMethodIn( METHOD_ConnectDo, _TOGGLE_CONNECTION_DELAY );
            return;
        }
        
        if ( ! m_bConnectEnabled )
        {
            CallMethodIn( METHOD_ConnectDo, 0.01f );
            return;
        }
        
        //CrashLogD("Creating new client... " + Thread.currentThread().activeCount());
        
	    m_cClient = new ChatClient( sm_sHost, sm_nPort, m_sNick, this );
        //CrashLogD("...OK");
        DisableCallMethod( METHOD_ConnectDo );
        m_bDisconnectEnabled = false;
        CallMethodIn( METHOD_EnableDisconnect, _TOGGLE_CONNECTION_DELAY );
        m_bLockConnecting = true;
        CallMethodIn( METHOD_UnlockConnecting, _TOGGLE_CONNECTION_DELAY );
    }
    
    

    private synchronized void TryToDisconnect()
    {
        SetConnectionState( EChatConnectionState._DISCONNECTING );
        DisconnectDo();
    }
    
    private void DisconnectDo()
    {
        //CrashLogD("");
        if ( m_cClient != null )
        {
            //CrashLogD("");
            if ( ! m_bDisconnectEnabled )
            {
                CallMethodIn( METHOD_DisconnectDo, 0.01f );
                return;
            }
            
            //CrashLogD("Disconnecting client...");
            m_cClient.Disconnect("");
            //CrashLogD("...OK");
            m_cClient = null;
        }
        
        DisableCallMethod( METHOD_DisconnectDo );
        m_bConnectEnabled = false;
        CallMethodIn( METHOD_EnableConnect, _TOGGLE_CONNECTION_DELAY );
    }
        
    private int m_nNickPostfix = 0;
    
    public void ScheduleReconnect()
    {
        //CrashLogD("");
        m_nNickPostfix++;
        m_sNick     = NetPlayerProfile.GetCurrentNickName() + m_nNickPostfix;
        m_cClient   = null;
        DisconnectDo();//to tylko do zaprogramowanie opoznienia connecta
        TryToConnect();
    }
    
    protected void SetConnectionState( int nState )
    {
        m_nConnectionState = nState;
        UpdateConnnetionState();
    }
    
    private void UpdateConnnetionState()
    {
        String sText = GetConnectionStateString( m_nConnectionState );
        
        if ( m_cIChatWindow != null )
            m_cIChatWindow.SetConnectionStatus( (m_nConnectionState == EChatConnectionState._CONNECTED),
                                                sText );
    }
    
    private String GetConnectionStateString(int nConnectionState)
    {
        String sText = null;
        switch ( nConnectionState )
        {
        case EChatConnectionState._DISCONNECTED:
            sText = "Disconnected";
            break;
            
        case EChatConnectionState._CONNECTING:
            sText = "Connecting to "+sm_sHost+" ...";
            break;            
            
        case EChatConnectionState._CONNECTED:
            sText = "Connected to "+sm_sHost+" as "+m_sNick;
            break;                        
        }
        return sText;
    }
    
    synchronized protected void OutputMsg( String sMsg, int nType )
    {
        //CrashLogD("1 " + sMsg  + "...");
        Vector vColor = NetMsg._COLOR_DEFAULT;
        switch ( nType )
        {
            case EChatMessageType._CHAT:
                sMsg = NetGameTools.RemoveLastSign( sMsg );
                vColor = NetMsg._COL_INMSG_CHAT;
                break;
            case EChatMessageType._PRIVATE_CHAT:            
                sMsg = "priv from " + NetGameTools.RemoveLastSign( sMsg );
                vColor = NetMsg._COLOR_YELLOW;
                break;
            case EChatMessageType._SPECIAL:                
            case EChatMessageType._STORED:                
                sMsg = NetGameTools.RemoveLastSign( sMsg );
                vColor = NetMsg._COLOR_YELLOW;
                break;    
            case EChatMessageType._FROM_SERVER:
                vColor = NetMsg._COL_INMSG_SERVER;
                break;
            case EChatMessageType._SYSTEM:
                vColor = NetMsg._COL_INMSG_SYSTEM;
                break;
            case EChatMessageType._ERROR:
                vColor = NetMsg._COLOR_RED;
                break;
        }
        
        OutputMsg( sMsg, vColor );
        //CrashLogD("1...OK");
    }
   
    synchronized private void OutputMsg( String sText, Vector vColor )
    {
        //CrashLogD("2...");
        ChatMsgInfo cMsgInfo = new ChatMsgInfo( sText, vColor );
            
        synchronized ( m_cMsgHistory )
        {
            m_cMsgHistory.Add( cMsgInfo );
        }
        
        synchronized ( m_cMsgToOutput )
        {
            m_cMsgToOutput.Add( cMsgInfo );
        }
        
        m_bMessagesChanged = true;
        //CrashLogD("2...OK");
    }
    
    synchronized void OutputMsg()
    {
        //CrashLogD("...");
        synchronized ( m_cMsgToOutput )
        {
            if ( m_cIChatWindow != null )
            {
                for ( int i = 0; i<m_cMsgToOutput.GetSize(); ++i )
                {
                    ChatMsgInfo cMsgInfo = m_cMsgToOutput.GetMsgInfoAt( i );
                    m_cIChatWindow.OutputMsg( cMsgInfo.GetText(),
                                              cMsgInfo.GetColor() );
                }
            }
            
            m_cMsgToOutput.RemoveAll();
        }
        //CrashLogD("...OK");
    }
        
    
    synchronized protected void ClientsChanged()
    {
        String[]        aClients = null;
        if ( m_cClient != null )
            aClients = m_cClient.GetClients();
        
        if ( m_cIChatWindow != null )        
            m_cIChatWindow.UpdateClients( aClients );
    }
    
    
}
