import java.util.Vector;

class GameServerWaiter
{
    int Port;
    int Address;
    float MessageTime;
    int MessageNumber;
    
    GameServerWaiter(int address, int port)
    {
        this.Port = port;
        this.Address = address;
        MessageNumber = 0;
        MessageTime = 0;
    }
    
    boolean Equal(String address, int port)
    {
        String s;
        s = Integer.toString((Address>>24)& 255) + "." +					
            Integer.toString((Address>>16)& 255) + "." +					
            Integer.toString((Address>> 8)& 255) + "." +					
            Integer.toString((Address>> 0)& 255);
        return Port == port && s.equals(address);
    }
    
    
    boolean Equal(GameServerWaiter w)
    {
        return Port == w.Port && Address == w.Address;
    }
    
    boolean Equal(int address, int port)
    {
        return Port == port && Address == address;
    }
    
    void SetTime(float time)
    {
        MessageTime = time;
    }
    
    float GetTime()
    {
        return MessageTime;
    }
    
    void IncMessageNumber()
    {
        MessageNumber++;
    }
    
    void ClearMessageNumber()
    {
        MessageNumber = 0;
    }
    
    int GetMessageNumber()
    {
        return MessageNumber;
    }
    
    int GetAddress()
    {
        return Address;
    }
    
    int GetPort()
    {
        return Port;
    }
}

public class GameServerList extends GameObject
{
    // ile maksymalnie razy odpytujemy server
    final int MAX_MESSAGE_NUMBER = 4;
    // w jakich minimalnych odstepach czasu
    final float MIN_DELTA_TIME = 4.0f;
    
    Vector GameServers;
    int LastGameServer;
    
    private static GameServerList GSList;
    
    public static GameServerList GetObject()
    {
        if (GSList == null)
        {
            GSList = new GameServerList();
        }
        return GSList;
    }

    private GameServerList()
    {
        LastGameServer = 0;
        GameServers = new Vector();
    }
    
    void AddServer(GameServerWaiter w)
    {
        if (FindServer(w) == -1)
        {
            GameServers.addElement(w);
        }
    }
    
    int FindServer(GameServerWaiter w)
    {
        for (int i = 0; i < GameServers.size(); i++)
        {
            if (((GameServerWaiter)GameServers.elementAt(i)).Equal(w))
            {
                return i;
            }
        }
        return -1;
    }
        
    int FindServer(String address, int port)
    {
        for (int i = 0; i < GameServers.size(); i++)
        {
            if (((GameServerWaiter)GameServers.elementAt(i)).Equal(address, port))
            {
                return i;
            }
        }
        return -1;
    }
    
    GameServerWaiter GetNextServer(float time)
    {
        GameServerWaiter GSWaiter = null;
        
        RemoveInactiveServers();
        
        if (LastGameServer >= GameServers.size())
            LastGameServer = 0;
        
        int tempLastGameServer = LastGameServer;
        
        do{
            if (LastGameServer >= GameServers.size())
                LastGameServer = 0;
            if (LastGameServer < GameServers.size())
            {
                GameServerWaiter w =(GameServerWaiter)GameServers.elementAt(LastGameServer);
                if (w.GetTime() + MIN_DELTA_TIME < time)
                {
                    w.IncMessageNumber();
                    w.SetTime(time);
                    GSWaiter = w;
                }
                LastGameServer++;
                if (LastGameServer >= GameServers.size())
                    LastGameServer = 0;
            }
        }
        while (LastGameServer != tempLastGameServer && GSWaiter == null);
        return GSWaiter;
    }
    
    void Reset()
    {
        LastGameServer = 0;
        for (int i = 0; i < GameServers.size(); i++)
        {
            ((GameServerWaiter)GameServers.elementAt(i)).ClearMessageNumber();
        }
    }
    
    private void RemoveInactiveServers()
    {
        for (int i = 0; i < GameServers.size(); i++)
        {
            if (((GameServerWaiter)GameServers.elementAt(i)).GetMessageNumber() >= MAX_MESSAGE_NUMBER)
            {
                GameServers.removeElementAt(i);
                i--;
            }
        }
    }
    
    void RemoveServer(GameServerWaiter w)
    {
        int i = FindServer(w);
        if (i != -1)
        {
            GameServers.removeElementAt(i);
        }
    }
    
    void RemoveServer(String address, int port)
    {
        int i = FindServer(address, port);
        if (i != -1)
        {
            GameServers.removeElementAt(i);
        }
    }
    
    void RemoveAllServers()
    {
        GameServers.removeAllElements();
    }
}
