/**
 * Zmiana podanego parametru mapy
 */
public class CommandMapSetParam extends ConsoleCommandParams
{
    CommandMapSetParam()
    {
        super();
        m_nRequiredAdminRights  = EAdminRights._STANDARD;
        m_nMinParams            = 
        m_nMaxParams            = 2;
        m_bResultsToAll         = true;
    }
    
    
    boolean   m_bBoolSettingParam = false;
    
    void Clean()
    {
        super.Clean();
        m_bBoolSettingParam = false;
    }
   
    final static int _PARAM_SETTING_NAME  = 0;
    final static int _PARAM_SETTING_PARAM = 1;
   
    String GetSParam(int nParam)
    { 
        String sParam = super.GetSParam( nParam );
    
        //drugi parametr (jesli jest stringowy) zawsze 
        //dotyczy przelacznikow "on" "off"
        //tu mozemy wykryc czy tak jest
        if ( nParam == _PARAM_SETTING_PARAM )
        {
            if ( (sParam != null) &&
                 ((sParam.equalsIgnoreCase("yes")||
                   sParam.equalsIgnoreCase("no")||
                   sParam.equalsIgnoreCase("on")||
                   sParam.equalsIgnoreCase("off"))) )
            {
                m_bBoolSettingParam = true;
                SetParam( _PARAM_SETTING_PARAM, sParam.toUpperCase() );
            }
            else
            {
                SetErrorId( EConsoleCommandError._WRONG_PARAM );
            }
        }
        return sParam; 
    }
    
    int GetIParam(int nParam)
    { 
        String sParam = super.GetParam( nParam );
    
        if ( nParam == _PARAM_SETTING_PARAM )
        {
            if ( (sParam != null) &&
                 (sParam.equalsIgnoreCase("no")||
                  sParam.equalsIgnoreCase("off")) )
            {
                m_bBoolSettingParam = true;
                SetParam( _PARAM_SETTING_PARAM, sParam.toUpperCase() );
                return -1;
            }
        }
        
        return super.GetIParam( nParam );
    }

    float GetFParam(int nParam)
    { 
        String sParam = super.GetParam( nParam );
    
        if ( nParam == _PARAM_SETTING_PARAM )
        {
            if ( (sParam != null) &&
                 (sParam.equalsIgnoreCase("no")||
                  sParam.equalsIgnoreCase("off")) )
            {
                m_bBoolSettingParam = true;
                SetParam( _PARAM_SETTING_PARAM, sParam.toUpperCase() );
                return -1.0f;
            }
        }
        
        return super.GetFParam( nParam );
    }
    
        
    /**
     * Callback z GameSettings podczas wyszukiwania parametru
     * do zmiany.
     * Tu mozemy stwierdzic czy cos poszlo nie tak,
     * jesli byly jakies problemy to akcja zastosowania
     * parametru nie odbedzie sie.
     */
    boolean VerifyModifier(ModifierNetGameSettings cModifier)
    {
        if ( ErrorOccured() )
        {
            //cos po drodze bylo nie tak
            //TODO: jaki to mogl byc blad?
            SetErrorId();
            return false;
        }
        
        if ( cModifier == null )
        {
            //nie znalazl zadnego modyfikatora, czyli wpisalismy
            //zla nazwe
            SetErrorId( EConsoleCommandError._MAP_PARAMETER_UNKNOWN );
            return false;    
        }
        
        

        if ( ! EModifiersNetGameSettings.AllowedInConsoleCommandMapSetParam( cModifier.GetId() ) )
        {
            //tego parametru nie mozna modyfikowac z consoli
            SetErrorId( EConsoleCommandError._MAP_PARAMETER_CHANGE_NOT_ALLOWED );
            return false;    
        }
        
        return true;
    }
    
    /**
     * Callback z GameSettings podczas wyszukiwania parametru
     * do zmianu.
     * Informuje ze wlasnie znaleziono (ignorecase) odpowiednia
     * komende i tu przekazywana jest jej poprawna postac
     * (do wyswietlenia w odpowiedzi)
     */
    void FoundSettingName( String sName )
    {
        SetParam( _PARAM_SETTING_NAME, sName );
    }
    
    
    
    void Perform()
    {
        boolean bApplied = m_cGameInfo.GetGameSettings().ParseGameSettingsCommand( this );
        if ( ! ErrorOccured() ) 
        {
            if ( bApplied )
                m_cGameInfo.EventGameSettingsChanged();
            else
                SetErrorId( EConsoleCommandError._MAP_PARAMETER_NOT_CHANGED );
        }
    }
    
    
    void PackResults(GameObject cMessager)
    {
        super.PackResults( cMessager );
        cMessager.MessagePackStringBoolTable( GetParams() );
        cMessager.MessagePackBoolean( m_bBoolSettingParam );
    }
    
    void UnPackResults(GameObject cMessager)
    {
        super.UnPackResults( cMessager );
        SetParams( cMessager.MessageUnPackStringBoolTable( "" ) );
        m_bBoolSettingParam = cMessager.MessageUnPackBoolean();
    }
    
    void ShowResults()
    {
        String sResults  = null;
        
        sResults = m_bBoolSettingParam ?
                   
                   (GetSParam( _PARAM_SETTING_NAME )+
                    " "+
                    GetSParam( _PARAM_SETTING_PARAM )):
                   
                   (NetGameTools.MixString(
                        Text.Get("&ConsoleCommandParameterSetTo&"),
                        GetSParam( _PARAM_SETTING_NAME ),
                        GetSParam( _PARAM_SETTING_PARAM ) ));

        WriteResults( GetTextCommandAccepted() + " " + sResults );
    }
    

    void PackErrors(GameObject cMessager)
    {
        super.PackErrors( cMessager );
        cMessager.MessagePackStringBoolTable( GetParams() );
    }
    
    void UnPackErrors(GameObject cMessager)
    {
        super.UnPackErrors( cMessager );
        SetParams( cMessager.MessageUnPackStringBoolTable( "" ) );
    }
    
    void ShowErrors()
    {
        String sError = GetTextCommandNotAccepted();
        switch ( GetErrorId() )
        {
        case EConsoleCommandError._MAP_PARAMETER_UNKNOWN:            
            sError = sError + " " + Text.Get("&ConsoleCommandParameterUnknown&");
            break;
            
        case EConsoleCommandError._MAP_PARAMETER_CHANGE_NOT_ALLOWED:            
            sError = sError + " " + Text.Get("&ConsoleCommandParameterChangeNotAllowed&");
            break;
            
        case EConsoleCommandError._MAP_PARAMETER_NOT_CHANGED:            
            sError = sError + " " + Text.Get("&ConsoleCommandParameterNotChanged&");
            break;            
            
        default:
            sError = null;
            super.ShowErrors();
        }
        if ( sError != null )
            WriteResults( sError );
    }
    

}
