/**
 * Akstrakcyjna klasa bazowa dla komend wywolywanych
 * z konsoli
 * Wszystkie "wlasciwe" komendy musza(!) miec klase rozpoczynajaca
 * sie od slowa "Command", (np "CommandKillMe", "CommandMapNext")
 * wtedy reszta nazwy znajdujaca sie po przedrostku "Command" staje
 * sie automatycznie nazwa komendy do wpisania z konsoli.
 * Jezeli chcemy zeby bylo inaczej nalezy w konstruktorze danej
 * klasy dokonac zmiany nazwy poprzez metode SetName.
 * 
 * Aby mozna bylo korzystac z danej komendy musi ona zostac
 * umieszczona w ConsoleCommandsManager'ze 
 */
public abstract class ConsoleCommand
{
    ConsoleCommand()
    {
        //pobieramy nazwe klasy
        String sClassName = this.getClass().getName();
        //po usunieciu przedrostka "Command" uzyskujemy nazwe komendy
        m_sName = NetGameTools.GetTextWithoutPrefix( sClassName,
                                                     "Command" );
    }

    final static String sm_sResultsPrefix = "|>";
    static String GetResultsPrefix(){ return sm_sResultsPrefix; }
    final static String sm_sAdminPrefix = "!>";
    static String GetAdminPrefix(){ return sm_sAdminPrefix; }
    
    /**
     * Stale parametry komendy (nie beda sie zmieniac
     * podczas dzialania programu)
     */
    /** NetGameInfo na ktorym wolana jest ta komenda */
    protected NetGameInfoHud     m_cGameInfo       = null;
    protected NetPlayerInfoList  m_cPlayerInfoList = null;
    void SetGameInfo(NetGameInfoHud cGameInfo) 
    { 
        m_cGameInfo       = cGameInfo; 
        m_cPlayerInfoList = (m_cGameInfo != null)?
                             m_cGameInfo.GetPlayerInfoList() : null;
    }
    /** sprawdzenie rodzaju gry */
    boolean IsInternetGame() { return (m_cGameInfo!=null)&&(m_cGameInfo.IsInternetGame()); }
    
    /** Nazwa komendy (do wpisania w konsoli w celu jej wywolania)*/
    private String m_sName   = null;
    String GetName(){ return m_sName; }
    void SetName(String sName){ m_sName = sName; }
    
    /** Identyfikator komendy, (tak naprawde to index w tablicy) */
    private int    m_nId       = -1;
    void SetId(int nId){ m_nId = nId; }
    int GetId() { return m_nId; }
    
    /** Czy komenda przeznaczona jest tylko dla administratorow */
    int m_nRequiredAdminRights = EAdminRights._NONE;
    int GetRequiredAdminRights(){ return m_nRequiredAdminRights; }
    boolean IsForAdminsOnly(){ return m_nRequiredAdminRights > EAdminRights._NONE; }

    /** Czy nalezy oczekiwac parametrow */
    //boolean m_bParamsExpected = false;
    //boolean AreParamsExpected(){ return m_bParamsExpected; }
        
    /** Czy nalezy spodziewac sie jakiegos _tekstowego_ rezultatu */
    boolean m_bResultsExpected = true;
    boolean AreResultsExpected(){ return m_bResultsExpected || (m_sResults!=null); }
        
    /** Czy resultaty maja byc wyslane do wyszystkich czy tylko do "autora"*/
    boolean m_bResultsToAll = false;
    boolean AreResultsToAll(){ return m_bResultsToAll; }
    
    /** Czy komenda jest lokalna, to znaczy mozna ja zawolac na kliencie
     * bez wiedzy serwera*/
    //boolean m_bLocal = false;
    //boolean IsLocal(){ return m_bLocal; }
    boolean m_bShouldExecuteOnlyOnServer = true;
    boolean ShouldExecuteOnlyOnServer(){ return m_bShouldExecuteOnlyOnServer; }
    
    
    /**
     * 
     * Zmienne parametry komendy, czyli te ktore beda
     * sie zmieniac podczas kolejnych wywolan komendy
     * Uwaga: w metodzie Clean nalezy "wyzerowac" wszystkie
     * z tych zmiennych
     * 
     */
    
    private 
    int m_nCurrentStatus = EConsoleCommandStatus._CLEAN;
    int GetCurrentStatus(){ return m_nCurrentStatus; }
    
    /** Id hosta ktory wywolal komende */
    private 
    int m_nHost = 0;
    int GetHost(){ return m_nHost; }
     
    /** gracz glownego hosta ustawiany na podstawie m_nHost
     * Jesli ten member przyda sie do czegos innego to
     * mozna go spokojnie wykorzystac */
    NetPlayerInfo m_cPlayerInfo = null;
        
    /** Czy komenda wykonuje sie wlasnie 
     * (sami ja wlasnie wklepalismy) czy nie*/
    private 
    boolean m_bLocalPerform = false;
    boolean IsLocalPerform(){ return m_bLocalPerform; }
    void SetLocalPerform(boolean bLocal){ m_bLocalPerform = bLocal; }
    
    /*sprawdza czy zwrocony kod, to jakis blad */
    boolean IsError( int nErrorId ){ return (nErrorId != EConsoleCommandError._NONE); }
    
    /** kod bledu ktory wystapil podczas wywolania komendy */    
    private 
    int  m_nErrorId = EConsoleCommandError._NONE;
    int  GetErrorId(){ return m_nErrorId; }
    void SetErrorId( int nId ){ m_nErrorId = nId; }
    void SetErrorId(){ SetErrorId(EConsoleCommandError._UNKNOWN); }
    /** Czy podczas wykonywania komendy cos sie nie posypalo? */
    boolean ErrorOccured(){ return (m_nErrorId != EConsoleCommandError._NONE); }
    

    /** Rezultat wykonania komendy */
    String m_sResults = null;
    /** Zlozony rezultat eykonania komendy, kazda klasa
     * bedzie musiala sobie potrafic to zinterpretowac */            
    NetDataContainer m_aResultsContainer = null;
    
    /**
     * Metoda czyszczace zmienne pola komendy, 
     * wolana po kazdym uzyciu (wykonaniu) danej komendy
     * Nalezy nadpisac w poszczegolnych klasach o 
     * dodatkowe pola.
     */
    void Clean()
    {
        m_nCurrentStatus = EConsoleCommandStatus._CLEAN;
        m_nHost = 0;
        m_bLocalPerform = false;
        m_sResults = null;
        m_aResultsContainer = null;
        m_nErrorId = EConsoleCommandError._NONE;
        m_cPlayerInfo = null;
    }
    
    /** Metoda pakujaca dane komendy
     * Bedzie wykorzystana dla komend dziedziczaych 
     * po ConsoleCommandParams
     **/
    void Pack(GameObject cMessager)
    {
    }
    
    /** Metoda odpakowujaca dane komendy
     * Bedzie wykorzystana dla komend dziedziczaych 
     * po ConsoleCommandParams, tam zostanie metoda nadpisana
     * Przy nadpisywaniu nalezy bezwzglednie zawolac _tez_ super 
     * (ze wzgledu na to ze zawsze nalezy odczytac MessageGetSenderId()).
     **/
    void UnPack(GameObject cMessager)
    {
        m_nHost   = cMessager.MessageGetSenderId();
    }
    

    /**
     * Metoda przepisujaca parametry komendy z 
     * z innej komendy
     */
    void Init( ConsoleCommand cCommand )
    {
        m_nHost   = cCommand.GetHost();
    }
    
    /**
     * Wolane tuz przed zawolaniem Perform()
     */
    void InitPerform()
    {
        if ( IsForAdminsOnly() && (!ShouldExecuteOnlyOnServer()) )
        {
            NetLog.Error(" ConsoleCommand "+GetNetInfo()+" IsForAdminsOnly but not ShouldExecuteOnlyOnServer");
            SetErrorId( EConsoleCommandError._UNKNOWN );
        }
        
        m_cPlayerInfo = m_cGameInfo.GetPlayerInfoList().GetPlayerByHostId( GetHost() );
    }
    
    /**
     * Metoda wykonawcza komendy,
     */
    void Perform()
    {
    }
    
    /**
     * Metoda wolana na samym koncu (w przypadku nie
     * stwierdzenie bledow podczas InitPerform i ew Perform)
     * Po wyswietleniu komunikatow i rozeslaniu ich w 
     * sieci
     * Mozna to wykonac jakas akcje, juz PO pojawieniu
     * sie komunikatu "Command accepted"
     */
    void PostPerform()
    {
    }
    
    /**
     * Zapakowanie resultatow w komunikat (do klienta/ow)
     * Jesli ta metoda nie zostanie nadpisana, to wysylana
     * bedzie wartosc m_sResults.
     * Jesli wyniki wykonanie komendy beda wymagaly
     * nadpisania tej metody, to warto uzyc tez super.PackResults,
     * zeby wyslany tez zostal id hosta ktory wykonal komende
     * (byc moze to bedzie potrzebne do wypisania wynikow)
     */
    void PackResults(GameObject cMessager)
    {
        cMessager.MessagePackInt( m_nHost );
        cMessager.MessagePackStringBool( m_sResults );

        boolean bResultsContainer = (m_aResultsContainer != null);
        cMessager.MessagePackBoolean( bResultsContainer );
        if ( bResultsContainer )
            m_aResultsContainer.ToMessage( cMessager );
    }
    
    /**
     * Odpakowanie resultatow z komunikatu (od serwera)
     * Jesli ta metoda nie zostanie nadpisana, to wysylana
     * bedzie wartosc m_sResults
     * Ponadto jesli w danej klasie nie zostanie nadpisana
     * metoda ShowResults() to ten tekst pojawi sie na konsoli
     * Jesli chodzi o nadpisywanie tej metody, to
     * patrz (@link #PackResults(GameObject cMessager))
     */
    void UnPackResults(GameObject cMessager)
    {
        m_nHost                   = cMessager.MessageUnPackInt();
        m_sResults                = cMessager.MessageUnPackStringBool( null );
        boolean bResultsContainer = cMessager.MessageUnPackBoolean();
        if ( bResultsContainer )
        {
            m_aResultsContainer = new NetDataContainer();
            m_aResultsContainer.FromMessage( cMessager );
        }
    }
    
    void UnPackAddResults( GameObject cMessager )
    {
        
    }

    /** Metoda wyswietlajaca rezultat
     * Zwykle bedzie ona nadpisywana przez poszczegolne klasy
     * komend, jednak jesli odpowiedzia ma byc zwykly tekst, to
     * wystarczy w Perform ustawic zmienna m_sResults i nie
     * nadpisywac metod PackResults UnPackResults, ShowResults(), 
     * wtedy automatycznie jako rezultat dostaniemy ten takst
     * Ponadto jesli zdefiniujemy m_bResultsExpected na true
     * a nie ustawimu m_sResults to domyslnie wyswietli sie
     * komunikat typu "Komanda jakastam zostala wykonana"
     **/
    void ShowResults()
    {
        boolean bNullResults = false;
        if ( m_sResults == null )
        {
            m_sResults = GetTextCommandAccepted();
            bNullResults = true;
        }
        WriteResults( m_sResults );
                
        //to jest robione po to, bo gdyby ktos kiedys zmienil
        //kolejnosc i wyswietlalby resultaty lokalnie zanim
        //zostaly one rozeslane w siec to wyslalby sie string
        //zamiast nulla (zamiast nulla u klientow tez pojawi
        //sie napis ze wykonano komende)
        if ( bNullResults )
            m_sResults = null;
    }
        

    /**
     * Zapakowanie bledu w komunikat (do klienta)
     */
    void PackErrors(GameObject cMessager)
    {
        cMessager.MessagePackInt( m_nErrorId );
    }
    
    /**
     * Odpakowanie bledu wykonania metody z komunikatu (od serwera)
     */
    void UnPackErrors(GameObject cMessager)
    {
        m_nErrorId = cMessager.MessageUnPackInt();
    }
    
    void ShowErrors()
    {
        String sError = GetTextCommandNotAccepted();
        switch ( m_nErrorId )
        {
        case EConsoleCommandError._COMMAND_UNKNOWN:            
            sError = sError + " " + Text.Get("&ConsoleCommandUnknown&");
            break;
        case EConsoleCommandError._YOU_ARE_NOT_ADMIN:
            sError = sError + " " + Text.Get("&ConsoleCommandYouAreNotAdmin&");
            break;
        case EConsoleCommandError._YOU_ARE_NOT_AUTHORIZED:
            sError = sError + " " + Text.Get("&ConsoleCommandYouAreNotAuthorizedToExecuteThisCommand&");
            break;
        case EConsoleCommandError._WRONG_PARAM:
            sError = sError + " " + Text.Get("&ConsoleCommandWrongParameter&");
            break;            
        case EConsoleCommandError._WRONG_PARAM_TYPE:
            sError = sError + " " + Text.Get("&ConsoleCommandWrongParameterType&");
            break;                        
        case EConsoleCommandError._WRONG_PARAMS_CNT:
            sError = sError + " " + Text.Get("&ConsoleCommandWrongParameterNumber&");
            break;                        
        case EConsoleCommandError._PLAYER_NOT_FOUND:
            sError = sError + " " + Text.Get("&ConsoleCommandPlayerNotFound&");
            break;
        case EConsoleCommandError._PLAYER_IS_DEAD:
            sError = sError + " " + Text.Get("&ConsoleCommandPlayerIsDead&");
            break;                  
        case EConsoleCommandError._GAME_OVER:
            sError = sError + " " + Text.Get("&ConsoleCommandGameOver&");
            break;
        case EConsoleCommandError._CURRENT_MAP_IS_NOT_A_TEAM_GAMEPLAY:
            sError = sError + " " + Text.Get("&ConsoleCommandNotTeamGameplayMode&");
            break;            
            
        case EConsoleCommandError._UNKNOWN:            
            break;            
        default:
            sError = null;
            NetLog.Warning("ConsoleError "+GetErrorId()+" not supported");
        }
        if ( sError != null )
            WriteResults( sError );
    }
    
    
    /** Metoda wykonawcza przesylajaca do GameInfo tekst 
     * jako rezultat wywolania komendy. 
     * Metoda powinna byc wolana w obsludze ShowResults
     * Uwaga: Jako ze dla danej komendy moze byc kilka linii 
     * tekstu, zatem ta metode moze byc zawolana wielokrotnie
     * w ShowResults  */
    protected void WriteResults( String sText )
    {
        if ( sText != null )
            m_cGameInfo.WriteCommandResults( sText );
    }
    
    /**
     * jw tylko tekst trafia na konsole admina
     */
    protected void WriteSpecialResults( String sText )
    {
        if ( sText != null )
            m_cGameInfo.WriteSpecialCommandResults( sText );
    }
    
    /** Porownuje nazwe komendy z podana */
    boolean IsEqual( String sName )
    {
        if ( (sName == null) || (m_sName == null) )
            return false;
        return (m_sName.compareToIgnoreCase( sName )==0);
    }
    
    /** Porownuje id komendy z podana */
    boolean IsEqual( int nId ) { return (m_nId == nId); }
            
    
    String GetTextCommandName()
    {
        return GetName();
    }
            

    String GetTextCommandAccepted()
    {
        return NetGameTools.MixString(
                    Text.Get("&ConsoleCommandAccepted&"),
                    GetTextCommandName());
    }
    
    String GetTextCommandNotAccepted()
    {
        return NetGameTools.MixString(
                    Text.Get("&ConsoleCommandNotAccepted&"),
                    GetTextCommandName());
    }
    
    
    
    String GetNetInfo(){ return ""+m_sName+" id:"+m_nId; }
    
    void InitStreamParams(String sText)
    {
    }

    
    /** Pobranie nazwy identyfikatora w zaleznosci od trybu gry*/
    String GetPlayerIdName() 
    {   
        return IsInternetGame() ? 
               "PID": 
               "HostId"; 
    }
    
    /**Zwraca identyfikator playera w zaleziosco od trybu gry*/
    int GetPlayerId( NetPlayerInfo cPlayerInfo )
    {
        return IsInternetGame() ? 
               cPlayerInfo.GetPID():
               cPlayerInfo.GetClientOwnerId();
    }
}

