public class AdminsManager extends GameObject
{
    final static String sm_sServerAdminsFileName = "Data/ServerAdmins.scr";
    
    private
    UsersList       m_cAvailableAdmins = null;
    
    private
    LoggedUsersList m_cLoggedAdmins    = null;
    LoggedUsersList GetLoggedAdmins(){ return m_cLoggedAdmins; }
    
    
    AdminsManager()
    {
        m_cAvailableAdmins = new UsersList();
        m_cLoggedAdmins    = new LoggedUsersList();
    }
    
    void OnCreate()
    {
        m_cAvailableAdmins.RemoveAll();
        m_cLoggedAdmins.RemoveAll();
    
        ParseAvailableAdmins();
    }
    
    void OnDestroy()
    {
    }
    
    
    int AdminLogin( String sUserName, String sPassword, int nHostId )
    {
        //if already logged return error code
        if ( m_cLoggedAdmins.GetUserByHostId( nHostId ) != null )
            return EConsoleCommandError._YOU_ARE_ALREADY_LOGGED_IN;
        
        //find given user
        UserInfo cUserInfo = m_cAvailableAdmins.GetUserByUserName( sUserName );
        
        //if not found return error code
        if ( cUserInfo == null )
            return EConsoleCommandError._INCORRECT_LOGINNAME_OR_PASSWORD;
        
        //if incorrect password return error code
        if ( ! sPassword.equals( cUserInfo.GetPassword()) )
            return EConsoleCommandError._INCORRECT_LOGINNAME_OR_PASSWORD;

        //ok, log in
        m_cLoggedAdmins.Add( new LoggedUserInfo( cUserInfo, nHostId ) );
        
        return EConsoleCommandError._NONE;            
    }
    
    int AdminLogout(int nHostId)
    {
        LoggedUserInfo cUserInfo = m_cLoggedAdmins.GetUserByHostId( nHostId );
        if ( cUserInfo == null )
            return EConsoleCommandError._NOT_LOGGED_YET;
        m_cLoggedAdmins.Remove( cUserInfo );
        return EConsoleCommandError._NONE;            
    }
    
   
    int GetAdminRights( int nHostId )
    {
        //server has always full privileges
        if (nHostId == 0)
            return EAdminRights._FULL;
        
        LoggedUserInfo cInfo = m_cLoggedAdmins.GetUserByHostId( nHostId );
        if ( cInfo != null )
            return cInfo.GetRights();
        
        return EAdminRights._NONE;
    }
    
    //what if called during game?
    private void ParseAvailableAdmins()
    {
        ParseAvailableAdmins( sm_sServerAdminsFileName );
    }
    private void ParseAvailableAdmins(String sFileName)
    {
        m_cAvailableAdmins.RemoveAll();
            
        ParseKeyword cRoot = ParseFile( sFileName );
        if (cRoot != null)
        {
            for (ParseKeyword cKey = cRoot; cKey != null; cKey = cKey.GetNext())
            {
                if (cKey.Equals("Admin"))
                {
                    /*NetLog.Test("Admin "+cKey.GetSParam(0)+" "+
                                         cKey.GetSParam(1)+" "+
                                         cKey.GetIParam(2));*/
                    m_cAvailableAdmins.Add(
                                    new UserInfo( cKey.GetSParam(0),
                                                  cKey.GetSParam(1),
                                                  cKey.GetIParam(2) )
                                          );
                }
            }
            cRoot.delete();
        }
        else
            NetLog.Error("No file "+ sFileName +"!\n");    
    }
}
