 /** Obiekt zawiera informacje o grze sieciowej w trakcie jej trwania.
 * Atrybuty tego obiektu sa replikowane na clientow. Obiekt ten sluzy rowniez do komunikacji
 * miedzy levelami na clientach a serwerem.
 */
public class NetGameInfo extends NetGameInfoHud
{
    /** Czy dane gry (a w konsekwencji zakonczenie PrecacheNet)
     * maja byc na kliencie
     * true  - juz po podlaczeniu gracza na serwerze
     * false - dopiero po wyspawnowaniu (i zainicjowaniu) gracza na kliencie
     */
    static boolean sm_bSendGameSettingOnJoin = false;
    
    
    /** Identyfikatory messagy wysylanych przez ta klase.
     */
    //z klienta na serwer
    final static int _MSG_CREATE_PLAYER            = 10;
    final static int _MSG_I_AM_READY               = 11;
    final static int _MSG_PING_SERVER              = 12;
            
    //z serwera na klienta
    final static int _MSG_CREATE_PLAYER_CONFIRM    = 15;
    final static int _MSG_CREATE_PLAYER_CANCEL     = 16;
    final static int _MSG_PLAYERS_STATE            = 17;
    final static int _MSG_TIME_INFO                = 18;
    
    final static int _MSG_GAME_SETTINGS            = 20;
    final static int _MSG_UPDATE_GAME_SETTINGS     = 21;
    final static int _MSG_PLAYER_JOINED            = 22;
    final static int _MSG_PLAYER_LEFT              = 23;
    final static int _MSG_PLAYER_DIED              = 24;
    final static int _MSG_PLAYER_CHANGED_NICK      = 25; 
    final static int _MSG_PLAYER_RESPAWN_LIMIT_OUT = 26;
    final static int _MSG_PLAYER_EXCLUDED          = 27;
    final static int _MSG_PLAYER_KICKED_OUT        = 28;
    final static int _MSG_PLAYER_BANNED            = 29;
    final static int _MSG_GAMEOVER                 = 30;
    final static int _MSG_GAMERESTART              = 31;
    final static int _MSG_NEXTLEVEL                = 32;
    final static int _MSG_OBJECTIVE_UPDATED        = 33; 
    final static int _MSG_GAMESTATUSMSG_UPDATED    = 34; 

    final static int _MSG_FX_RESPAWN_ON_LEVEL      = 40;
    final static int _MSG_NEW_GAME_PARAMS          = 42;
    final static int _MSG_AFTER_GAME_OVER_PING     = 43;
    
    //Komunikaty wspolne dozwolone do 80, pozostale w Teamach
    
    ///////////////////////////////////////////////////////
    // inicjalizacja obiektu tak aby replikowal atrybuty //
    ///////////////////////////////////////////////////////
    
    NetGameInfo()
    {
        Role = ERole._PROXY;
        SetGameType( ENetGameType.DEFAULT );
    }
    
    int METHOD_ClientCreateMyPlayerRequest          = 0;
    int METHOD_ServerCreateMyPlayer                 = 0;
    int METHOD_OnSetGameSettings                    = 0;     
    int METHOD_EventAfterGameOver                   = 0;
    int METHOD_LoadNextLevel                        = 0; 
    int METHOD_ServerDelayedLoadNextLevel           = 0;
    int METHOD_QuitGameOnGameOver                   = 0;
    int METHOD_ForceQuitGame                        = 0;
    int METHOD_StartSendingMsgClientPlayersState    = 0; 
    int METHOD_MsgSendClientPlayersState            = 0;
    int METHOD_VerifyPlayersState                   = 0;
    int METHOD_MsgSendServerPing                    = 0;
    int METHOD_ServerSendClientNextLevel            = 0;
    
    void OnCreate()
    {
        METHOD_ClientCreateMyPlayerRequest       = RegisterCallMethod("ClientCreateMyPlayerRequest");
        METHOD_ServerCreateMyPlayer              = RegisterCallMethod("ServerCreateMyPlayer");
        METHOD_OnSetGameSettings                 = RegisterCallMethod("OnSetGameSettings");
        METHOD_EventAfterGameOver                = RegisterCallMethod("EventAfterGameOver");    
        METHOD_LoadNextLevel                     = RegisterCallMethod("LoadNextLevel");
        METHOD_ServerDelayedLoadNextLevel        = RegisterCallMethod("ServerDelayedLoadNextLevel");
        METHOD_QuitGameOnGameOver                = RegisterCallMethod("QuitGameOnGameOver");
        METHOD_ForceQuitGame                     = RegisterCallMethod("ForceQuitGame");
        METHOD_StartSendingMsgClientPlayersState = RegisterCallMethod("StartSendingMsgClientPlayersState");
        METHOD_MsgSendClientPlayersState         = RegisterCallMethod("MsgSendClientPlayersState");
        METHOD_VerifyPlayersState                = RegisterCallMethod("VerifyPlayersState");
        METHOD_MsgSendServerPing                 = RegisterCallMethod("MsgSendServerPing");
        METHOD_ServerSendClientNextLevel         = RegisterCallMethod("ServerSendClientNextLevel");
        
        super.OnCreate();
        
        if (Role < ERole._AUTHORITY)
        {
            // zapisujemy sie do modulu, bo inaczej nie bedzie nic o nas wiedzial
            m_cModule.SetGameInfo( this );
            InitTimeCompensator();
                
            LogD("New player request\n");
            // wysylamy zadanie stworzenia gracza dla tego clienta o podanych parametrach do serwera
            ClientCreateMyPlayerRequest();
        }
        else
        {
            /*
            if ( PlayerNet.sm_bTestMoving || 
                 PlayerNet.sm_bTestAdjust || 
                 PlayerNet.sm_bTestJumping )
                SetPlayerOnServer( false );*/
        
            // tylko do testow, wywalic to!
            if ( m_cModule.bBotsForTest )
            {
                if ( sm_nBotsCnt > 0 )
                    nPlayersCntToAdd = sm_nBotsCnt;
                CallMethodEvery("CheckAdditionalPlayersToCreate", 1.0f);
            }
                                    
            ServerInitGame();
            ServerPostInitGame();
        }
        
        EnableMessages();
    }
    
    void OnPostCreate()
    {
        if (Role == ERole._AUTHORITY)
        {
            ServerCleanGame();
            if ( IsPlayerOnServer() )
                CallMethodIn( METHOD_ServerCreateMyPlayer, 0.0f );
            CallMethodIn( METHOD_OnSetGameSettings, 1.0f );
        }
    }
    
    void CreateHUDNet()
    {
        super.CreateHUDNet();
        if ( GetHUDNet() != null )
            GetHUDNet().Init( this );
    }

    /**SERVER replikacja wszystkich informacji o grze
     */
    //static int sm_nSetData;
        
    public void ReplicationInit()
    {
        //sm_nSetData = NewReplicationSetReliable("m_fGameTime");
        SetNetPriority( 1.0f );
    }
  
    /** Replikacja odpowiednich zestawow dla odpowiednich warunkow
     */
    
    public void OnReplicateServer()
    {
		// repliujemy informacje o grze do kazdego, gdzie by nie byl
        //Replicate( sm_nSetData );
    }
    
    //////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////
    
    NetPlayerInfo ServerAddPlayerInfo(PlayerNet cNewPlayer,int nClientId, int nClientGameVersion )
    {
        NetPlayerInfo cPlayerInfo = 
            super.ServerAddPlayerInfo( cNewPlayer, nClientId, nClientGameVersion );
        
        EventPlayerJoined( cNewPlayer );
        
        //po kazdym dolaczeniu gracza od nowa inicjuje 
        //wysylanie stanow graczy
        ScheduleSendingMsgClientPlayersState();
        
        return cPlayerInfo;
    }
    
    void AddPlayerInfo(NetPlayerInfo cPlayerInfo)
    {
        super.AddPlayerInfo( cPlayerInfo );
        
        if ( cPlayerInfo != null )
        {
            int nInfoId = cPlayerInfo.GetNetId();
            UpdateEnableObjectivesIfOwner( nInfoId );
        }
    }
        
    /**Usuniecie informacji o grze gracza.
     */
    void ServerDeletePlayerInfo(PlayerNet cPlayer)
    {
        EventPlayerLeft( cPlayer );
    
        super.ServerDeletePlayerInfo( cPlayer );
        
        if ( GetPlayerInfoList().GetSize() <= 0 )
            StopSendingMsgClientPlayersState();
    } 
    
    ///////////////////////////////////////////////////////////////////////
    // metody sluzace wysylaniu komunikatow o waznych zdarzeniach w grze //
    ///////////////////////////////////////////////////////////////////////

    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_PING_SERVER:
            MessageEnd();
            break;
                
        case _MSG_TIME_INFO:
            MsgRecvClientTimeInfo();
            break;            
        
        case _MSG_CREATE_PLAYER:
            LogD("_MSG_CREATE_PLAYER\n");
            MsgRecvServerCreatePlayer();
            break;

        case _MSG_PLAYER_DIED:
            LogD("_MSG_PLAYER_DIED\n");
            MsgRecvClientPlayerDied();
            break;
            
        case _MSG_PLAYER_CHANGED_NICK:
            LogD("_MSG_PLAYER_CHANGED_NICK\n");
            MsgRecvClientPlayerChangedNick();
            break;            
        
        case _MSG_I_AM_READY:
            LogD("_MSG_I_AM_READY\n");
            MsgRecvServerIAmReady();
            break;
            
        case _MSG_CREATE_PLAYER_CONFIRM:
            LogD("_MSG_CREATE_PLAYER_CONFIRM\n");
            MsgRecvClientCreatePlayerConfirm();
            break;
            
        case _MSG_CREATE_PLAYER_CANCEL:
            LogD("_MSG_CREATE_PLAYER_CANCEL\n");
            MsgRecvClientCreatePlayerCancel();
            break;
            
        case _MSG_PLAYERS_STATE:
            LogD("_MSG_PLAYERS_STATE\n");
            MsgRecvClientPlayersState();
            break;
            
        case _MSG_GAME_SETTINGS:
            LogD("_MSG_GAME_SETTINGS\n");
            MsgRecvClientGameSettings();
            break;            
            
        case _MSG_UPDATE_GAME_SETTINGS:
            LogD("_MSG_UPDATE_GAME_SETTINGS\n");
            MsgRecvClientUpdateGameSettings();
            break;                        
                            
        case _MSG_PLAYER_JOINED:
            LogD("_MSG_PLAYER_JOINED\n");
            MsgRecvClientPlayerJoined();
            break;
            
        case _MSG_PLAYER_LEFT:
            LogD("_MSG_PLAYER_LEFT\n");
            MsgRecvClientPlayerLeft();
            break;

        case _MSG_PLAYER_RESPAWN_LIMIT_OUT:
            LogD("_MSG_PLAYER_RESPAWN_LIMIT_OUT\n");
            MsgRecvClientPlayerRespawnLimitOut();
            break;
        
        case _MSG_PLAYER_EXCLUDED:
            LogD("_MSG_PLAYER_EXCLUDED\n");
            MsgRecvClientPlayerExcluded();
            break;
            
        case _MSG_PLAYER_KICKED_OUT:
            LogD("_MSG_PLAYER_KICKED_OUT\n");
            MsgRecvClientPlayerKickedOut();
            break;    
        
        case _MSG_PLAYER_BANNED:
            LogD("_MSG_PLAYER_BANNED\n");
            MsgRecvClientPlayerBanned();
            break;            
            
        case _MSG_FX_RESPAWN_ON_LEVEL:
            LogD("_MSG_FX_RESPAWN_ON_LEVEL\n");
            MsgRecvClientFXRespawnOnLevel();
            break;
            
		case _MSG_GAMEOVER:
            LogD("_MSG_GAMEOVER\n");
			MsgRecvClientGameOver();
			break;
        
        case _MSG_GAMERESTART:
            LogD("_MSG_GAMERESTART\n");
			MsgRecvClientGameRestart();
			break;
            
        case _MSG_NEW_GAME_PARAMS:
            LogD("_MSG_NEW_GAME_PARAMS\n");
			MsgRecvClientNewGameParams();
			break;
            
        case _MSG_NEXTLEVEL:
            LogD("_MSG_NEXTLEVEL\n");
            MsgRecvClientNextLevel();
			break;
                
        case _MSG_OBJECTIVE_UPDATED:
            LogD("_MSG_OBJECTIVE_UPDATED\n");
            MsgRecvClientObjectiveUpdated();
            break;
            
        case _MSG_GAMESTATUSMSG_UPDATED:
            LogD("_MSG_GAMESTATUSMSG_UPDATED\n");
            MsgRecvClientGameStatusMsgUpdated();
            break;
        
        case _MSG_AFTER_GAME_OVER_PING:
            LogD("_MSG_AFTER_GAME_OVER_PING\n");
			MsgRecvClientGameOverPing();
			break;
            
        default:
            super.OnMessage();
        }
    }
    
    /** 
     * Komunikaty wysylane z serwera na klientow
     */
    
    /**SERVER 
     * Obsluga zgloszenia clienta, aby mu stworzyc gracza.
     */
    void MsgRecvServerCreatePlayer()
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        int[]   aSkins          = new int[ NetPlayerModelInfo.GetSkinsPartialCnt() ];
                
        int     nClientId	    = MessageGetSenderId();
        int     nGameVersion    = MessageUnPackInt();
        String  sPlayerName     = MessageUnPackString();
        String  sMeshName       = MessageUnPackString();
        for (int i=0; i<aSkins.length; i++ )
            aSkins[ i ]         = MessageUnPackInt();
        String  sLevelName      = MessageUnPackString();
        int     nLevelCRC       = MessageUnPackInt();
        boolean bAutoPlayer     = MessageUnPackBoolean();
        MessageEnd();
        
        NetPlayerProfile cProfile = new NetPlayerProfile();
        if ( bAutoPlayer )
        {
            sPlayerName = "autoplayer: " + sPlayerName;
            cProfile.SetAutoPlayer( true );
        }
        cProfile.SetPlayerName( sPlayerName );
        cProfile.SetMeshName( sMeshName );
        cProfile.SetSkins( aSkins );
        
        // wyslijmy mu info, ze juz przyjelismy jego prosbe zeby sie nie niepokoil :-)
        MsgSendClientCreatePlayerConfirm( nClientId );
        
        // i probujemy stworzyc gracza
        int nResults = ServerCreatePlayer( cProfile, nClientId, 
                                           sLevelName, nLevelCRC, nGameVersion );
        
        if ( nResults != ECreatePlayerResults._CREATED_SUCCESFULL && nClientId != 0 )
        {
            //nie udalo sie, powiedz klientowi o tym,
            MsgSendClientCreatePlayerCancel( nClientId, nResults );
            // i jesli trzeba to go wykop
            CheckKickClientOnFailedCreatePlayer( nClientId, nResults );
        }
    }
    
    void CheckKickClientOnFailedCreatePlayer( int nClientId, int nResults )
    {
        boolean bKick = true;
            
        switch ( nResults )
        {
            //za takie cos nie wywalamy
            case ECreatePlayerResults._CANCELED_CAUSE_ALREADY_CREATED:
                bKick = false;
                break;
            
            case ECreatePlayerResults._CANCELED_CAUSE_WRONG_MAP:
            case ECreatePlayerResults._CANCELED_CAUSE_WRONG_CRCMAP:
            case ECreatePlayerResults._CANCELED_CAUSE_WRONG_GAME_VERSION:
            case ECreatePlayerResults._CANCELED_CAUSE_TOO_MANY_PLAYERS:                
            case ECreatePlayerResults._CANCELED_CAUSE_NOREASON:                
                bKick = true;
                break;
                            
            default:                
                CrashLog(" ECreatePlayerResults "+nResults+" unknown, cant check kick ");
        }
        
        //jesli potrzeba to go wykopujemy
        if ( bKick )
            ForceKickClient( nClientId );
        //TODO: zrobic to kikanie z malym opoznieniem
    }        
    
    int ServerCreatePlayer( NetPlayerProfile cProfile, int nClientId,
                            String           sLevelName, int nLevelCRC, int nGameVersion )
    {
        //i czy ma ta sama wersje
        if ( ! NetGameVersion.IsVersionCompatible( nGameVersion, GetGameVersion() ) )
            return ECreatePlayerResults._CANCELED_CAUSE_WRONG_GAME_VERSION;

        //sprawdzamy czy klient ma ten sam level zaladowany        
        if (!sLevelName.equals(m_cModule.GetGameInitInfo().GetCurrentLevelName()))
            return ECreatePlayerResults._CANCELED_CAUSE_WRONG_MAP;
        
        //no to probujemy stworzyc gracza
        int nResults = ServerCreatePlayer( cProfile, nClientId, nGameVersion );
        
        return nResults;
    }
    
    void MsgRecvServerIAmReady()
    {
        int nHost = MessageGetSenderId();
        MessageEnd();
        
        if ( !sm_bSendGameSettingOnJoin )
        {
            MsgSendClientGameSettings( nHost );
        }
    }
        
    /** 
     * Komunikaty wysylane z serwera na klientow
     */
    
    /**SERVER
     * Odpowiedz klientowi ze przyjeto jego zadanie 
     * wyspawnowania klienta
     * Uwaga, msg jest wysylany TYLKO do okreslonego
     * klienta
     */
    void MsgSendClientCreatePlayerConfirm( int nClientId )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_CREATE_PLAYER_CONFIRM);
        MessageSendHost( nClientId );
    }
    
    /**SERVER
     * Odpowiedz klientowi ze jego zadanie 
     * wyspawnowania klienta nie moze zostac zrealizowane
     * Uwaga, msg jest wysylany TYLKO do okreslonego klienta
     */
    void MsgSendClientCreatePlayerCancel(int nClientId,int nReason)
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_CREATE_PLAYER_CANCEL);
        MessagePackInt(nReason);
        MessageSendHost(nClientId);
        
    }
    
    /**SERVER Wysyla (co jakis czas) stan wszystkich graczy
     * bioracych udzial w grze.
     * Jesli ktorys z klientow stwierdzi ze jakis 
     * gracz u niego ma jakis inny stan, to poprosi
     * (juz na poziomie PlayerNet'a o doslanie aktualnego 
     * pelnego stanu)
     */
    void MsgSendClientPlayersState()
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_PLAYERS_STATE);
        GetPlayerInfoList().MessagePackPlayersState( GetTime() );
        MessageSend();
    }
    
    /** SERWER
     *  Obsluga czasu gry i pingow
     */
    void MsgSendClientTimeInfo()
    {
        MessageNewUnreliable( _MSG_TIME_INFO );
        MessagePackFloat( GetGameTime() );
        GetPlayerInfoList().MessagePackPlayersPing();
        MessageSend();
    }       
        
    void MsgSendClientPlayerJoined( int nJoinedInfoId )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_PLAYER_JOINED);
        MessagePackInt( nJoinedInfoId );
        MessageSend();
    }
    
    void MsgSendClientGameSettings( int nHost )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_GAME_SETTINGS);
        MessagePackBoolean( IsInternetGame() );
        MessagePackInt( GetGameResults() );
        MessagePackGameSettings();
        //MessagePackLevelContent( GetGameSettings().GetAvailableInventoryRoom() );
        MessagePackObjectives();
        MessagePackCurrentGameStatus();
        MessagePackPlayersTeamSkins();
        MessagePackTeamRole();
        MessageSendHost( nHost );
        
        if ( m_bGameParamsChanged )
            MsgSendClientNewGameParams();
    }
    
    void MsgSendClientUpdateGameSettings()
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_UPDATE_GAME_SETTINGS);
        MessagePackGameSettings();
        MessageSend();
    }
    
    void MsgSendClientPlayerDied( int nVictimInfoID, int nAssassinInfoID, boolean bTeamKill )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_PLAYER_DIED);
        MessagePackInt( nVictimInfoID );
        MessagePackInt( nAssassinInfoID );
        MessagePackBoolean( bTeamKill );
        MessageSend();
    }
    
    void MsgSendClientPlayerChangedNick( String   sOldNick,
                                         String   sNewNick,
                                         int      nPlayerInfoId,
                                         boolean  bForced )
    {
        if (Role < ERole._AUTHORITY) 
			return;
        MessageNew(_MSG_PLAYER_CHANGED_NICK);
        MessagePackStringBool( sOldNick );
        MessagePackStringBool( sNewNick );
        MessagePackInt( nPlayerInfoId );
        MessagePackBoolean( bForced );
        MessageSend();
    }
    
    
    void MsgSendClientPlayerLeft(int nLeftInfoId)
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_PLAYER_LEFT);
        MessagePackInt( nLeftInfoId );
        MessageSend();
    }
        
    

    void MsgSendClientPlayerRespawnLimitOut(int nPlayerInfoId)
    {
        MessageNew( _MSG_PLAYER_RESPAWN_LIMIT_OUT );
        MessagePackInt( nPlayerInfoId );
        MessageSend();
    }
    void MsgSendClientPlayerExcluded(int nPlayerInfoId)
    {
        MessageNew( _MSG_PLAYER_EXCLUDED );
        MessagePackInt( nPlayerInfoId );
        MessageSend();
    }
     
    void MsgSendClientPlayerKickedOut(int nPlayerInfoId,int nReason,String sReason)
    {
        boolean bEnumedReason = ( nReason != EKickReason._NOT_ENUMED_REASON );
        
        MessageNew( _MSG_PLAYER_KICKED_OUT );
        MessagePackInt( nPlayerInfoId );
        MessagePackBoolean( bEnumedReason );
        if ( bEnumedReason )
            MessagePackInt( nReason );            
        else
            MessagePackString( (sReason != null) ? sReason : "" );
        MessageSend();
    }

    void MsgSendClientPlayerBanned(int nPlayerInfoId,String sReason)
    {
        //TODO: jak dorobimy autobany to tu rozpatrzyc eReason
        MessageNew( _MSG_PLAYER_BANNED );
        MessagePackInt( nPlayerInfoId );
        MessagePackString( sReason );
        MessageSend();
    }
    
    void MsgSendClientGameOver()
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_GAMEOVER);
        MessagePackInt( GetGameResults() );
        MessageSend();
    }
    
    void MsgSendClientGameRestart()
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_GAMERESTART);
        MessagePackBoolean( m_bFirstRestart );
        MessagePackInt( m_nRestartMode );
        MessageSend();
    }
    
    
    void MsgSendClientNewGameParams()
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_NEW_GAME_PARAMS);
        MessagePackFloat( PlayerNet.sm_fPhysicsGranularity );
        MessagePackFloat( ( m_cModule != null )? m_cModule.GetPhysicsLastUpdateTime() : -1.0f );
        MessagePackFloat( PlayerNet._MAX_RESEND_MOVE_TIME );
        MessagePackBoolean( PlayerNetTestBot.sm_bCanUseWeapon );
        MessagePackBoolean( m_bClientGameShutdown );
        MessageSend();
        
        //zapamietujemy ze wysylalismy ten komunikat, zeby go 
        //doslac tym co sie dopiero dolacza
        m_bGameParamsChanged = true;
    }
    
    void MsgRecvClientNewGameParams()
    {
        float   fPhysicsGranularity     = MessageUnPackFloat();
        float   fPhysicsLastUpdateTime  = MessageUnPackFloat();
        float   fMaxResendMoveTime      = MessageUnPackFloat();
        boolean bAutoPlayersCanFire     = MessageUnPackBoolean();
        boolean bClientGameShutdown     = MessageUnPackBoolean();
        
        MessageEnd();
                
        SetPhysicsGranularity( fPhysicsGranularity );
        if ( m_cModule != null && fPhysicsLastUpdateTime >= 0.0f )
             m_cModule.SetPhysicsLastUpdateTime( fPhysicsLastUpdateTime );
        SetMaxResendMoveTime( fMaxResendMoveTime );
        SetAutoPlayersFire( bAutoPlayersCanFire );
        if ( bClientGameShutdown )
            SetClientShutdown();
    }
    
    void MsgSendClientFXRespawnOnLevel(Vector vPos,byte nFx)
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNewUnreliable(_MSG_FX_RESPAWN_ON_LEVEL);
        MessagePackVector( vPos );
        MessagePackByte( nFx );
        MessageSend();
    }
        
    
    
    ////////////////////////////////////////////////
    boolean m_bGameParamsChanged = false;
    
    boolean SetMaxResendMoveTime( float fMaxResendMoveTime )
    {
        boolean bRet = super.SetMaxResendMoveTime( fMaxResendMoveTime );
        if (bRet && Role == ERole._AUTHORITY)   
            MsgSendClientNewGameParams();
        return bRet;
    }
    
    boolean SetPhysicsGranularity(float fPhysicsGranularity)
    {
        boolean bRet = super.SetPhysicsGranularity( fPhysicsGranularity );
        if (bRet && Role == ERole._AUTHORITY)
            MsgSendClientNewGameParams();
        return bRet;
    }
    
    boolean SetAutoPlayersFire( boolean bFire )
    {
        boolean bRet = super.SetAutoPlayersFire( bFire );
        if (bRet && Role == ERole._AUTHORITY)
            MsgSendClientNewGameParams();
        return bRet;
    }
    
    boolean SetClientShutdown()
    {
        boolean bRet = super.SetClientShutdown();
        if (bRet && Role == ERole._AUTHORITY)
            MsgSendClientNewGameParams();
        return bRet;
    }
    
    
    /**
     * Zdarzenia zachodzace w grze, 
     * 
     * W wiekszosci sa to metody wywolywane na serwerze, ktore pozniej
     * musza zostac zreplikowane na klientow,
     * Na chwile obecna wyjatkiem jest tu EventTypedText (zarowno na serwerze
     * jak i na kliencie) oraz EventDisconnectFromServer (tylko na kliencie)
     * 
     */
        
    /** Serwer, 
     * podlaczenie sie nowego gracza
     */
    void EventPlayerJoined(PlayerNet player)
    {
        NetPlayerInfo cJoinedInfo   = null;
        int nJoinedInfoId   = -1;
        
        if ( player != null )
        {
            cJoinedInfo   = player.GetPlayerInfo();
            if (cJoinedInfo != null)
                nJoinedInfoId = cJoinedInfo.GetNetId();
            else
                CrashLog("ERROR: "+this+" PlayerJoined "+player+" cNetPlayerInfo == null\n");
        }
        
        if ( cJoinedInfo == null )
            return;
        
        if ( sm_bSendGameSettingOnJoin )
        {
            int nJoinedHost = cJoinedInfo.GetClientOwnerId();
            MsgSendClientGameSettings( nJoinedHost );
        }
        
        if ( cJoinedInfo.GetTeamId() != NetTeamInfo.NONE )
        {
            NetTeamInfo cJoinedTeamInfo = GetTeamInfoById( cJoinedInfo.GetTeamId() );
            
            int nJoinedTeamInfoId   = -1;
            int nSkin               = NetTeamInfo.NONE;
            int nChangeSkinCounter  = PlayerNetBase.GetNextGlobalChangeSkinCounter();
            if ( cJoinedTeamInfo != null )
            {
                nJoinedTeamInfoId   = cJoinedTeamInfo.GetNetId();
                nSkin               = cJoinedTeamInfo.GetTeamId();
            }
        
            MsgSendClientPlayerJoinedTeam( nJoinedInfoId, nJoinedTeamInfoId, nSkin, nChangeSkinCounter );
            AddIncomingMessage( NetIncomingMsg._PLAYER_JOINED_TEAM, nJoinedInfoId, -1, nJoinedTeamInfoId ); 
            
            OnTeamChange();
            ChangePlayerTeamSkins( nJoinedInfoId, nSkin, nChangeSkinCounter );
        }
        else
        {
            MsgSendClientPlayerJoined( nJoinedInfoId );
            AddIncomingMessage( NetIncomingMsg._PLAYER_JOINED, nJoinedInfoId ); 
        }
                
        cJoinedInfo.UpdateRespawnLimitCnt();
        UpdateEnableObjectivesIfOwner( nJoinedInfoId );
                        
        if ( ! IsGameStarted() )
        {
            StartGameIfNeeded();
        }
    }
                
    /** Serwer, aktualizacja wynikow gry,
     * Respawn gracza
     */
    void EventPlayerSpawned( PlayerNet cPlayer )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        NetPlayerInfo cPlayerInfo   = null;
        if ( cPlayer != null )
        {
            cPlayerInfo = cPlayer.GetPlayerInfo();
            if (cPlayerInfo != null)
            {
                //zliczmy kolejny respawn 
                cPlayerInfo.IncRespawnCnt();
                
                cPlayerInfo.UpdateRespawnLimitCnt();
                                
                if ( ! cPlayerInfo.RespawnLimitAvailable() )
                {
                    int nPlayerInfoId = cPlayerInfo.GetNetId();
                                        
                    //skonczyl mu sie limit, wyslac komunikat
                    MsgSendClientPlayerRespawnLimitOut( nPlayerInfoId );
        
                    AddIncomingMessage( NetIncomingMsg._PLAYER_RESPAWN_LIMIT_OUT, 
                                        nPlayerInfoId ); 
                }
            }
            else
                CrashLog("ERROR: "+this+" EventPlayerSpawned cPlayer "+cPlayer+" cNetPlayerInfo == null\n");
        }
    }
    
    
    void EventPlayersDamaged( PlayerNetBaseMortal victim, PlayerNetBaseMortal assassin )
    {

    }
    
    /** Serwer, aktualizacja wynikow gry,
     * Smierc gracza
     */
    void EventPlayerDied( PlayerNetBaseMortal victim, PlayerNetBaseMortal assassin, int nMode )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        NetPlayerInfo cVictimInfo   = null;
        int nVictimInfoId   = -1;
        if ( victim != null )
        {
            cVictimInfo = victim.GetPlayerInfo();
            if (cVictimInfo != null)
            {
                nVictimInfoId = cVictimInfo.GetNetId();
            }
            else
                CrashLog("ERROR: "+this+" PlayerDied victim "+victim+" cNetPlayerInfo == null\n");
        }
        
        NetPlayerInfo cAssassinInfo   = null;
        int nAssassinInfoId   = -1;
        if ( assassin != null )
        {
            cAssassinInfo = assassin.GetPlayerInfo();
            if (cAssassinInfo != null)
                nAssassinInfoId = cAssassinInfo.GetNetId();
            else
                CrashLog("ERROR: "+this+" PlayerDied assassin "+assassin+" cNetPlayerInfo == null\n");
        }
        
        if (cVictimInfo != null)
        {
            //sprawdzmy czy czasem to nie jest jego koniec
            if (cVictimInfo.CheckFailed())
            {
                //wlasnie zostaje wykluczeny
                cVictimInfo.SetExcluded( true );
            }
            else     
            {
                //jeszcze ma szanse pograc, wiec
                float fMinRespawnTime = GetGameSettings().GetMinRespawnTime();
                //zacznijmy odliczac czas do respawnu
                if ( fMinRespawnTime > 0.0f                 &&
                     nMode == EPlayerAction.DIE_MODE_NORMAL )
                {
                    //TODO: GetTime czy GetGameTime?
                    float fNextRespawnTime = GetTime() + fMinRespawnTime;
                    cVictimInfo.SetNextRespawnTime( fNextRespawnTime ); 
                }
            }
        }
        
        
        
        if ( ! MessagesDisabled() )
        {
            boolean bTeamKill = ((cVictimInfo             != null) && 
                                 (cAssassinInfo           != null) &&
                                 (cVictimInfo             != cAssassinInfo) &&
                                 (cVictimInfo.GetTeamId() == cAssassinInfo.GetTeamId()));
            EventMessageOnPlayerDied( nVictimInfoId, nAssassinInfoId, bTeamKill );
        }
    }
    
    /**
     * Wymuszenie wykopania gracza z gry z podaniem powodu
     */
    private void EventPlayerToKick( int nClientId, int nReason, String sReason )
    {
        NetPlayerInfo cPlayerInfo = GetPlayerInfoList().GetPlayerByHostId( nClientId );
        if ( cPlayerInfo != null )
        {
            //wysylamy komunikat ze gracz juz zostal wykopany
            MsgSendClientPlayerKickedOut( cPlayerInfo.GetNetId(), nReason, sReason );
            //wymuszamy wyslanie komunikatu, zeby doszedl zanim goscie na prawde
            //odlaczmy (zeby on sam dostal tez ten komunikat)
            FlushReplicationManagerBuffers();
            
            //jesli powod byl enumerowany, to zamienmy go na tekst
            if ( nReason != EKickReason._NOT_ENUMED_REASON )
                 sReason = EKickReason.GetText( nReason );
            
            //wyswietlmy na serwerze odpowiedni komunikat
            AddIncomingMessage( NetIncomingMsg._PLAYER_KICKED_OUT, 
                                sReason, 
                                cPlayerInfo.GetNetId() );
            
            //i zaplanujmy faktyczne wyrzucenie klienta z gry
            cPlayerInfo.ScheduleKickOutFromTheGame();
        }
    }
    
    /**
     * Wymuszenie wykopania gracza z gry z powodem enumerowanym
     */
    void EventPlayerToKick( int nClientId, int nReason )
    {
        EventPlayerToKick( nClientId, nReason, null );
    }
    
    /**
     * Wymuszenie wykopania gracza z gry z powodem stringowym (podanym z konsoli)
     */
    void EventPlayerToKick( int nClientId, String sReason )
    {
        EventPlayerToKick( nClientId, EKickReason._NOT_ENUMED_REASON , sReason );
    }
        
    
    /** W grze sieciowej bedzie mozna tej komunikat nadpisac innym
     * np w przypadku gdy ktos zabil kogos z tej samej druzyny
     */
    void EventMessageOnPlayerDied( int nVictimInfoId, int nAssassinInfoId, boolean bTeamKill )
    {
        AddIncomingMessage( NetIncomingMsg._PLAYER_DIED, nVictimInfoId, nAssassinInfoId );
        MsgSendClientPlayerDied( nVictimInfoId, nAssassinInfoId, false );        
    }
    
    void EventPlayerChangedNick(NetPlayerInfo cPlayerInfo,String sOldNick)
    {
        String sNewNick = cPlayerInfo.GetNick();
        AddIncomingMessage( NetIncomingMsg._PLAYER_CHANGED_NICK,
                            sOldNick,
                            sNewNick,
                            cPlayerInfo.GetNetId() );
        MsgSendClientPlayerChangedNick( sOldNick,
                                        sNewNick,
                                        cPlayerInfo.GetNetId(),
                                        false );        
        
        ChangeNickInProfileIfOwner( cPlayerInfo.GetNetId(), sNewNick );
    }
    
    void EventPlayerChangedNickForced(NetPlayerInfo cPlayerInfo,String sOldNick)
    {
        String sNewNick = cPlayerInfo.GetNick();
        AddIncomingMessage( NetIncomingMsg._PLAYER_CHANGED_NICK_FORCED,
                            sOldNick,
                            sNewNick,
                            cPlayerInfo.GetNetId() );
        MsgSendClientPlayerChangedNick( sOldNick,
                                        sNewNick,
                                        cPlayerInfo.GetNetId(),
                                        true );        
        
        ChangeNickInProfileIfOwner( cPlayerInfo.GetNetId(), sNewNick );
    }
    
    /** Serwer, 
     * odlaczenie sie gracza
     */
    void EventPlayerLeft(PlayerNet cPlayer)
    {
        NetPlayerInfo cLeftInfo   = null;
        int nLeftInfoId   = -1;
    
        if ( cPlayer != null )
        {
            cLeftInfo   = cPlayer.GetPlayerInfo();
            if (cLeftInfo != null)
                nLeftInfoId = cLeftInfo.GetNetId();
            else
                CrashLog("ERROR: "+this+" PlayerLeft "+cPlayer+" cPlayerInfo == null\n");
        }
        
        if ( ! MessagesDisabled() && !(cLeftInfo != null && cLeftInfo.IsKickedOutFromTheGame()) )
        {
            AddIncomingMessage( NetIncomingMsg._PLAYER_LEFT, nLeftInfoId ); 
            MsgSendClientPlayerLeft( nLeftInfoId );
        }
        
        boolean bMessagesDisabled = MessagesDisabled();
        DisableMessages();
        cPlayer.ForceDieOnLeftGame();
        if ( ! bMessagesDisabled )
            EnableMessages();
    }
	    
    
    void EventPlayerExcluded(NetPlayerInfo cPlayerInfo)
    {
        //jakis gracz wykluczony
        int nPlayerInfoId = cPlayerInfo != null ? cPlayerInfo.GetNetId() : -1;
        //obwiesmy to wszem i wobec
        AddIncomingMessage( NetIncomingMsg._PLAYER_EXCLUDED, nPlayerInfoId ); 
        MsgSendClientPlayerExcluded( nPlayerInfoId );
    }
    
    void MsgSendClientPlayerJoinedTeam( int nJoinedInfoId, int nTeamInfoId, int nSkin, int nChangeSkinCounter )
    {
    }
        
    void EventTeamExcluded(NetTeamInfo cTeamInfo)
    {
        //do napdisania w grach druzynowych
    }
    
    void EventObjectiveUpdated(int nUpdate, int iGroupNo, int iSubObjNo, boolean bMesssagesActive)
    {
        MsgSendClientObjectiveUpdated( nUpdate, iGroupNo, iSubObjNo, bMesssagesActive );   
    }
    

    /**
     * 
     * Called from ConsoleCommand's
     * 
     */

    /**
     * Called from CommandMapSetParam
     */
    void EventGameSettingsChanged()
    {
        super.EventGameSettingsChanged();
        MsgSendClientUpdateGameSettings();
    }
    
    /**
     * Tu jest wszystko co sie dzieje po zakonczeniu gry
     */

    //czas zawolania EventAfterGameOver po GameOver na serwerze
    final static float _EVENT_AFTER_GAME_OVER_TIME               =  5.0f;
    //czas ladowania kolejnej mapy na serverze po EventAfterGameOver
    final static float _LOAD_NEXT_LEVEL_ON_SERVER_DEFAULT_TIME   =  5.0f;
    //dodatkowe opoznienie na serwerze ladowania mapy (na rozeslanie komunikatow)
    final static float _LOAD_NEXT_LEVEL_ON_SERVER_ADD_DELAY_TIME =  5.0f;
    //czas ladowania kolejnej mapy po EventAfterGameOver
    final static float _LOAD_NEXT_LEVEL_ON_CLIENT_TIME           = 12.0f;
    //czas opuszczenia gry po disconnect'cie
    final static float _QUIT_GAME_AFTER_DISCONNECT_TIME          =  5.0f;
    //opoznienie ladowania mapy lub wyjscia z gry, jesli gracz jest w ingamemenu
    final static float _IN_GAME_MENU_DELAY                       =  3.0f;


    //czy gra sie juz skonczyla i czekamy na powrot do menu?    
    private boolean m_bWaitForQuitGameOnGameOver = false;
    //czy wlasnie czekamy na zaladowania nastepnego levelu?
    private boolean m_bWaitForLoadNextLevel = false;
    //jesli tak, to jaki to jest level?
    LevelContent    m_cNextLevel            = null;
        
    /**
     * Metoda wolana na serwerze w momencie GameOver
     */    
    void ServerGameOver()
    {
        if (Role < ERole._AUTHORITY) 
			return;
        
        super.ServerGameOver();
        
        CallMethodIn( METHOD_EventAfterGameOver, _EVENT_AFTER_GAME_OVER_TIME );
    }

    
    /** Metoda wolana chwilke po zakonczeni gry na serwerze,
     * ustala czy bedzie ladowany kolejny level, czy to juz faktycznie koniec
     */
    void EventAfterGameOver()
    {
        m_cNextLevel = m_cModule.GetGameInitInfo().GetNextLevel();
        //jesli jest nastepny level to przygotowyjemy sie to jego ladowania
        //jesli nie to bedzienmy wychodzic z gry
        if ( m_cNextLevel != null )
            ScheduleLoadNextLevel();
        else
            ScheduleQuitGameOnGameOver();
    }
    
    void ScheduleLoadNextLevel()
    {
        RegisterKeyRequest();
        m_bWaitForLoadNextLevel = true;
        
        String sLevel = m_cNextLevel.GetLevelName()+" "+
                        ENetGameType.GetGameNameShort( m_cNextLevel.GetNetGameType() );
        
        AddIncomingMessage( NetIncomingMsg._NEXT_LEVEL, sLevel );
        float fTime = 0.0f;
        if ( Role == ERole._AUTHORITY )
        {
            fTime = _LOAD_NEXT_LEVEL_ON_SERVER_DEFAULT_TIME;
            ActivateStatisticsOnWaitForSpaceToContinue();
        }
        else
        {
            fTime = _LOAD_NEXT_LEVEL_ON_CLIENT_TIME;
            ActivateStatisticsOnPleaseWait();
        }
        
        CallMethodIn( METHOD_LoadNextLevel, fTime );
    }
    
    void LoadNextLevel()
    {
        DisableCallMethod( METHOD_LoadNextLevel );
        if (((ChromeGame)ChromeGame.cSingleton).Menu.bInGameMenuAct)
        {
            CallMethodIn( METHOD_LoadNextLevel, _IN_GAME_MENU_DELAY );
            return;
        }
        
        CleanBeforeQuitGame();
        
        if ( Role == ERole._AUTHORITY )
        {
            ActivateStatisticsOnPleaseWait();

            //teraz trzeba powiadomic klientow o nowej mapie
            //jako, ze za chwilke wszystko bedzie skasowane a serwer odlaczy 
            //klientow, wiec nie mamy pewnosci ze nasz komunikat dotrze do wszystkich
            //w tym celu bedziemy go wysylac wielokrotnie (az do momentu faktycznego
            //przeladowania levelu) za kazdym razem flushujac bufory wyjsciowe 
            //To wszystko zalatwi nam metoda ServerSendClientNextLevel(), ktora 
            //bedzie sie wolala cyklicznie co pewien czas
            ServerSendClientNextLevel();
            //teraz zaplanujmy faktyczne przeladowanie levelu za pewien czas
            //opoznienie dajemy na to, zeby dac czas na wyslanie powyzszych
            //komunikatow
            CallMethodIn( METHOD_ServerDelayedLoadNextLevel, _LOAD_NEXT_LEVEL_ON_SERVER_ADD_DELAY_TIME );
            
        }
        else
            m_cModule.ClientLoadNextLevel();
    }

    /**
     * SERVER
     * Metoda wysyla kilka klukukrotnie do klientow 
     * nazwe nastepnego levelu, za kazdym wyslaniem 
     * flushowane sa bufory.
     * Nastepnie metoda wpisuje sie do time handlerow
     * aby zawolac sie za pewien czas (zeby powtorzyc akcje)
     */
    void ServerSendClientNextLevel()
    {
        for ( int i = 0; i < 5; ++i )
        {
            MsgSendClientNextLevel( m_cNextLevel );
            FlushReplicationManagerBuffers();
        }
        DisableCallMethod( METHOD_ServerSendClientNextLevel );        
        CallMethodIn( METHOD_ServerSendClientNextLevel, 0.1f );
    }
    
    void ServerDelayedLoadNextLevel()
    {
        DisableCallMethod( METHOD_ServerDelayedLoadNextLevel );        
        DisableCallMethod( METHOD_ServerSendClientNextLevel );        
        m_cModule.CallMethodIn( m_cModule.METHOD_ServerLoadNextLevel, 0.0f );
    }
        
    void ScheduleQuitGameOnGameOver()
    {
        RegisterKeyRequest();
        m_bWaitForQuitGameOnGameOver = true;
        
        ActivateStatisticsOnGameOverAndContinue();
        
        if ( Role == ERole._AUTHORITY )
            MsgSendClientNextLevel( null );
        
        //tu nie wymuszamy wyjscia z gry
        //CallMethodIn( METHOD_QuitGameOnGameOver, 10.0f );
    }
    
    void QuitGameOnGameOver()
    {
        DisableCallMethod( METHOD_QuitGameOnGameOver );
        if (((ChromeGame)ChromeGame.cSingleton).Menu.bInGameMenuAct)
        {
            CallMethodIn( METHOD_QuitGameOnGameOver, _IN_GAME_MENU_DELAY );
            return;
        }
        
        CleanBeforeQuitGame();
        ForceQuitGame();
    }
    
    private void CleanBeforeQuitGame()
    {
        DisableCallMethod( METHOD_LoadNextLevel );
        DisableCallMethod( METHOD_QuitGameOnGameOver );
        m_bWaitForLoadNextLevel         = false;
        m_bWaitForQuitGameOnGameOver    = false;
        UnregisterKeyRequest();
        
        //tu poprzerywac wszystkie akcje ktore moga posypac gre:
        //konsola
        HudNetStop();
        //TODO: cos jeszcze?
    }
    
    void ForceQuitGame()
    {
        DisableCallMethod( METHOD_ForceQuitGame );
        if ( m_cModule != null )
            m_cModule.ForceQuitGame();
    }
    
    
    void EventDisconnectFromServer( int nReason )
    {
        super.EventDisconnectFromServer( nReason );

        //jesli przyszlo rozlaczenie a nie jestesmy w stanie oczekiwania
        //na nowy level, albo podczas wychodzenia z gry to za chwile wyjdzmy
        if ( ! (m_bWaitForLoadNextLevel || m_bWaitForQuitGameOnGameOver) )
            CallMethodIn( METHOD_ForceQuitGame, _QUIT_GAME_AFTER_DISCONNECT_TIME );
    }
    
    void EventConnectionToServerLost()
    {
        super.EventConnectionToServerLost();

        //jesli przyszlo rozlaczenie a nie jestesmy w stanie oczekiwania
        //na nowy level, albo podczas wychodzenia z gry to za chwile wyjdzmy
        if ( ! (m_bWaitForLoadNextLevel || m_bWaitForQuitGameOnGameOver) )        
            CallMethodIn( METHOD_ForceQuitGame, _QUIT_GAME_AFTER_DISCONNECT_TIME );
    }

    void EventDisconnectByConnectionStatusManager()
    {
        super.EventDisconnectByConnectionStatusManager();

        //jesli przyszlo rozlaczenie a nie jestesmy w stanie oczekiwania
        //na nowy level, albo podczas wychodzenia z gry to za chwile wyjdzmy
        //nie daje zadnego opoznienia, bo juz w ConnetionStatusManagerze
        //bylo opoznienie faktycznego wyrzucenia
        if ( ! (m_bWaitForLoadNextLevel || m_bWaitForQuitGameOnGameOver) )
            CallMethodIn( METHOD_ForceQuitGame, 0.0f );
    }
    
    void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
    {
        if ( !bKeyState )
            return;
            
        if ( nKeyCode == EKey._SPACE )
        {
            if ( m_bWaitForLoadNextLevel && (Role == ERole._AUTHORITY) )
                CallMethodIn( METHOD_LoadNextLevel, 0.0f );
            else if ( m_bWaitForQuitGameOnGameOver )
                CallMethodIn( METHOD_QuitGameOnGameOver, 0.0f );
        }
        
        if ( Game.CheatsEnabled() )
        {
            if ( nKeyCode == EKey._Z )
            {
                String sText = ""+GetTime();
                if ( m_cTimeCompensator != null )
                    sText += " "+m_cTimeCompensator.GetInfo();
                AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, sText );
            }
        }
    }
    
    static boolean sm_bBreakpoint = false;
    static void SetCheckBreakpoint(boolean bVal){ sm_bBreakpoint=bVal; }
    static boolean CheckBreakPoint()
    {
        return sm_bBreakpoint;
    }
    
    //////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////
    
    /** CLIENT
     *  Pobranie z PlayerProfile parametrow gracza i wyslanie serwerowi
     *  
     */
    void ClientCreateMyPlayerRequest()
    {
        MsgSendServerCreateMyPlayer( NetPlayerProfile.GetCurrent(), m_cModule );
        
        // bedziemy powtarzac nasza prosbe, az nam serwer odpowie ze
        // przyjal to do wiadomosci
        CallMethodIn( METHOD_ClientCreateMyPlayerRequest, 5.0f );
    }
    
    static int    GetGameVersion(){ return NetGameVersion.Get(); }
    static String GetGameVersionShortString(){ return NetGameVersion.GetShortName(); }
    static String GetGameVersionFullString(){  return NetGameVersion.GetFullName(); }
    static String GetGameVersionNumberString(){return NetGameVersion.GetNumberName(); }
    static String GetNetGameVersionNumberString(){return NetGameVersion.GetNetNumberName(); }
    //TODO:wywalic jak dostane JoinMenu
    //static String GetGameVersionString(){ return NetGameVersion.GetShortName(); }
    
    /**CLIENT Zgloszenie nowego gracza do serwera. 
     * Wysyla message razem z informacjiami niezbednymi do stworzenia gracza, czyli mesh, 
     * skin ogolnie informacje, ktore sa potrzebne przy CreatObject i nie beda mogly byc 
     * pozniej doslane.
     * TODO Na razie nie ma zadnych informacji, trzeba wysmyslec jakie, ale napewno 
     * bedzie to 
     */
    void MsgSendServerCreateMyPlayer(NetPlayerProfile cProfile,ModuleNet cModule)
    {
        int[] aSkins = cProfile.GetSkinsIndex();
        
        MessageNew(_MSG_CREATE_PLAYER);
        MessagePackInt( GetGameVersion() );
        /*if ( sm_bAutoPlayer )
            MessagePackString( SystemInformation.getComputerName() );
        else*/
            MessagePackString( cProfile.GetPlayerName() );
        MessagePackString( cProfile.GetMeshName() );
        for (int i=0; i<aSkins.length; i++ )
            MessagePackInt( aSkins[ i ] );        
        MessagePackString( cModule.GetGameInitInfo().GetCurrentLevelName() );
        //TODO: dolaczyc CRC
        MessagePackInt( 0 );
        MessagePackBoolean( sm_bAutoPlayer );
        MessageSend();
    }
    
    /** CLIENT
     *  Obsluga potwierdzenia, ze serwer odebral juz nasza prosbe
     *  o wyspawnowanie playera
     */
    void MsgRecvClientCreatePlayerConfirm()
    {
        // upewniamy sie ze sie nie zawola na serwerze
        if (Role == ERole._AUTHORITY) 
			return;
        MessageEnd();
        
        // no to zablokujmy juz ewentualne ponawianie requestow o playera
        DisableCallMethod( METHOD_ClientCreateMyPlayerRequest );
    }    
    
    
    /** CLIENT
     *  Obsluga potwierdzenia, ze serwer odrzucil nasza prosbe
     *  o wyspawnowanie playera
     */
    void MsgRecvClientCreatePlayerCancel()
    {
        int nResult = MessageUnPackInt();
        MessageEnd();
        
        // upewniamy sie ze sie nie zawola na serwerze
        if (Role == ERole._AUTHORITY) 
			return;
        
        // no to zablokujmy juz ewentualne ponawianie requestow o playera
        DisableCallMethod( METHOD_ClientCreateMyPlayerRequest );
        
        if ( nResult == ECreatePlayerResults._CANCELED_CAUSE_ALREADY_CREATED )
            return;
        
        //zabezpieczenie zeby 2 raz nie przyszlo to samo
        if ( m_bCreatePlayerCancelReceived )
            return;
        m_bCreatePlayerCancelReceived = true;
        
        if ( nResult == ECreatePlayerResults._CANCELED_CAUSE_WRONG_MAP )
        {
            //ladujemy aktualna mape
            CallMethodIn( METHOD_LoadNextLevel, 0.0f );
            //TODO: czy nexllevel to na pewno to co chcemy?
            return;    
        }
               
        
        //wyswietlic komunikat o tym ze nie ma szans sie podlaczyc...
        if ( m_cModule.m_cPrecache != null )
             m_cModule.m_cPrecache.StartGame();
        
        String sMsg = NetMsg.sYouCantJoinGame;
        switch ( nResult )
        {
        case ECreatePlayerResults._CANCELED_CAUSE_WRONG_CRCMAP:
            sMsg = NetMsg.sYouCantJoinGameBecauseWrongCRCMap;
            break;
        case ECreatePlayerResults._CANCELED_CAUSE_WRONG_GAME_VERSION:
            sMsg = NetMsg.sYouCantJoinGameBecauseWrongGameVersion;            
            break;
        case ECreatePlayerResults._CANCELED_CAUSE_TOO_MANY_PLAYERS:
            sMsg = sMsg +" "+ Text.Get( "&SERVER_FULL&" );//TODO: wrzucic to do localized
            break;    
        }
        AddIncomingMessage( sMsg );
    }   
    
    /**
     * Czy dostalismy juz komunikat od serwera ze on
     * odrzucil nasza prosbe o stworzenie gracza
     */
    boolean m_bCreatePlayerCancelReceived = false;
    
    /** CLIENT
     *  Obsluga stanow poszczegolnych graczy
     */
    void MsgRecvClientPlayersState()
    {
        GetPlayerInfoList().MessageUnPackPlayersState();
        MessageEnd();
        
        // upewniamy sie ze sie nie zawola na serwerze
        if (Role == ERole._AUTHORITY) 
			return;
        
        //weryfikacja odebranych stanow odbedzie sie dopiero po jakims
        //czasie, aby gracze mogli sie przelaczyc do swojego
        //stanu w "normalny" sposob i zeby jedno nie kolidowalo
        //z drugim.
        DisableCallMethod( METHOD_VerifyPlayersState );
        CallMethodIn( METHOD_VerifyPlayersState, 
                      sm_fVerifyPlayersStateDelay );
    }       
    
    void VerifyPlayersState()
    {
        GetPlayerInfoList().VerifyPlayersState();
    }
    
    
    /** CLIENT
     *  Obsluga czasu gry i pingow
     */
    void MsgRecvClientTimeInfo()
    {
        SetGameTime( MessageUnPackFloat() );
        GetPlayerInfoList().MessageUnPackPlayersPing();
        MessageEnd();
    }       
    
    void MsgRecvClientPlayerDied()
    {
        int     nVictimInfoId   = MessageUnPackInt();
        int     nAssassinInfoId = MessageUnPackInt();
        boolean bTeamKill       = MessageUnPackBoolean();
        MessageEnd();
                
        if ( bTeamKill )
        {
            AddIncomingMessage( NetIncomingMsg._PLAYER_DIED_TEAM_KILL, 
                                nVictimInfoId, 
                                nAssassinInfoId );
        }
        else
        {
            AddIncomingMessage( NetIncomingMsg._PLAYER_DIED, 
                                nVictimInfoId, 
                                nAssassinInfoId );
        }
    }
    
    void MsgRecvClientPlayerChangedNick()
    {
        String  sOldNick      = MessageUnPackStringBool("");
        String  sNewNick      = MessageUnPackStringBool("");
        int     nPlayerInfoId = MessageUnPackInt();
        boolean bForced       = MessageUnPackBoolean();
        MessageEnd();
        
        int nMsgId = bForced ? 
                     NetIncomingMsg._PLAYER_CHANGED_NICK_FORCED:
                     NetIncomingMsg._PLAYER_CHANGED_NICK;
            
        AddIncomingMessage( nMsgId,
                            sOldNick,
                            sNewNick,
                            nPlayerInfoId );
        
        ChangeNickInProfileIfOwner( nPlayerInfoId, sNewNick );
    }
    
    
    void MsgRecvClientGameSettings()
    {
        boolean bInternetGame = MessageUnPackBoolean();
        int     nResults      = MessageUnPackInt();
        MessageUnPackGameSettings();
        //MessageUnPackLevelContent();
        MessageUnPackObjectives();
        MessageUnPackCurrentGameStatus();
        MessageUnPackPlayersTeamSkins();
        MessageUnPackTeamRole();
        
        MessageEnd();
        
        SetInternetGame( bInternetGame );

        SetGameResults( nResults );
        OnSetGameSettings();
    }
    
    void MsgRecvClientUpdateGameSettings()
    {
        boolean bChanged = MessageUnPackGameSettings();
        MessageEnd();

        if ( bChanged )
            OnUpdateGameSettings();
    }
    
    void MsgRecvClientPlayerJoined()
    {
        int     nJoinedInfoId = MessageUnPackInt();
        MessageEnd();
          
        AddIncomingMessage( NetIncomingMsg._PLAYER_JOINED, 
                            nJoinedInfoId );
        UpdateEnableObjectivesIfOwner( nJoinedInfoId );
    }
    
    
    
    
    void MsgRecvClientPlayerLeft()
    {
        int nLeftInfoId   = MessageUnPackInt();
        MessageEnd();
                
        AddIncomingMessage( NetIncomingMsg._PLAYER_LEFT, nLeftInfoId );
    }
    
    void MsgRecvClientPlayerRespawnLimitOut()
    {
        int nPlayerInfoId   = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._PLAYER_RESPAWN_LIMIT_OUT, 
                            nPlayerInfoId ); 
    }
    
    void MsgRecvClientPlayerExcluded()
    {
        int nPlayerInfoId   = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._PLAYER_EXCLUDED, 
                            nPlayerInfoId ); 
    }
    
    void MsgRecvClientPlayerKickedOut()
    {
        String  sReason     =  null;
            
        int nPlayerInfoId   = MessageUnPackInt();
        if ( MessageUnPackBoolean() )
        {
            //reason enumowany
            int nReason = MessageUnPackInt();            
            sReason     = EKickReason.GetText( nReason );
        }
        else
            sReason = MessageUnPackString();
        MessageEnd();
               
        if ( sReason == null )
             sReason = "";
                
        AddIncomingMessage( NetIncomingMsg._PLAYER_KICKED_OUT, 
                            sReason, nPlayerInfoId ); 
    }
    
    void MsgRecvClientPlayerBanned()
    {
        int nPlayerInfoId   = MessageUnPackInt();
        String sReason      = MessageUnPackString();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._PLAYER_BANNED, 
                            sReason, nPlayerInfoId ); 
    }

    void MsgSendClientObjectiveUpdated( int nUpdate, int iGroupNo, int iSubObjNo, boolean bMesssagesActive )
    {
        MessageNew( _MSG_OBJECTIVE_UPDATED );
        MessagePackInt( nUpdate ); 
        MessagePackInt( iGroupNo ); 
        MessagePackInt( iSubObjNo ); 
        MessagePackBoolean( bMesssagesActive ); 
        MessageSend();
    }
    
    void MsgRecvClientObjectiveUpdated()
    {
        int     nUpdate         = MessageUnPackInt(); 
        int     iGroupNo        = MessageUnPackInt(); 
        int     iSubObjNo       = MessageUnPackInt(); 
        boolean bMesssagesActive= MessageUnPackBoolean();
        MessageEnd();
        
        ObjectivesManager  cObjectivesManager = m_cModule.GetObjectivesManager();
        if ( cObjectivesManager != null )
            cObjectivesManager.ObjectiveUpdated( nUpdate, 
                                                 iGroupNo, 
                                                 iSubObjNo, 
                                                 bMesssagesActive );
    }
        
    void MsgSendClientGameStatusMsgUpdated( NetDataContainer cContainer )
    {
        MessageNew( _MSG_GAMESTATUSMSG_UPDATED );
        cContainer.ToMessage( this );
        MessageSend();
    }
    
    void MsgRecvClientGameStatusMsgUpdated()
    {
        NetDataContainer cContainer = new NetDataContainer();
        cContainer.FromMessage( this );
        MessageEnd();
        
        if ( GetGameStatusMsgManager() != null )
             GetGameStatusMsgManager().StatusUpdated( cContainer );
    }

    void MsgSendClientGameOverPing()
    {
        MessageNew( _MSG_AFTER_GAME_OVER_PING );
        // empty msg
        MessageSend();
    }
    
    void MsgRecvClientGameOverPing()
    {
        //empty by design
        MessageEnd();
    }
    
    void MsgSendClientNextLevel(LevelContent cLevel)
    {
        MessageNew( _MSG_NEXTLEVEL );
        if ( cLevel != null )
        {
            MessagePackBoolean( true );
            MessagePackString( cLevel.sLevelName  );
            MessagePackInt( cLevel.GetNetGameType() );    
        }
        else
            MessagePackBoolean( false );
        MessageSend();
    }
    
    void MsgRecvClientNextLevel()
    {
        String sLevelName = null;
        int    nGameType  = ENetGameType.DEFAULT;
        if ( MessageUnPackBoolean() )
        {
            sLevelName = MessageUnPackString();
            nGameType  = MessageUnPackInt();
        }
        MessageEnd();

        //jesli odebralismy od serwera info co dalej, to
        //zaniechamy wychodzenia z gry (jesli tak mialo byc)
        DisableCallMethod( METHOD_ForceQuitGame );
        
        if ( sLevelName != null )
        {
            //jest nastepny level
            
            //ten komunikat obslugujemy tylko jak pojawi sie po raz pierwszy,
            if ( ! m_bWaitForLoadNextLevel )
            {
                m_cNextLevel = m_cModule.ClientSetNextLevel(sLevelName,nGameType);
                ScheduleLoadNextLevel();
            }    
        }
        else
        {
            //nie ma kolejnego levelu, bedzie trzeba wyjsci z gry 
            
            if ( ! m_bWaitForQuitGameOnGameOver )
            {
                ScheduleQuitGameOnGameOver();
            }    
        }
    }
    
    void MsgSendServerIAmReady()
    {
        MessageNew( _MSG_I_AM_READY );
        MessageSend();
    }
        
    void MsgSendServerPing()
    {
        MessageNew( _MSG_PING_SERVER );
        MessageSend();
    }
    
    
    
	void MsgRecvClientGameOver()
	{
        int nResults = MessageUnPackInt();
		MessageEnd();
        
        SetGameResults( nResults );
		SetGameOver();
	}
    
    void MsgRecvClientGameRestart()
	{
        m_bFirstRestart = MessageUnPackBoolean();
        m_nRestartMode  = MessageUnPackInt();
		MessageEnd();
        
        //SetGameResults( ENetGameResults._GAME_IS_READY_TO_START );
        SetGameResults( ENetGameResults._GAME_IS_GOING );
		SetGameRestart();
	}
    
	void MsgRecvClientFXRespawnOnLevel()
	{
        Vector vPos = MessageUnPackVector();
        byte   nFx  = MessageUnPackByte();
		MessageEnd();
		
		if ( m_cModule != null )
            m_cModule.RespawnFX( vPos, nFx );
	}
    
    
    int GetDominationBaseLocalState( DominationBase cBase, NetPlayerInfo cPlayerInfo )
    {
        return -1;
    }

    static boolean sm_bAutoPlayer = false;
    static void SetAutoPlayer(boolean bAutoPlayer)
    {
        sm_bAutoPlayer = bAutoPlayer;
    }
    
    
    /**
     * Ustalenie id winnera
     * id gracza lub druzyny (jesli to gra teamowa)
     */
    int GetWinnerId()
    {
        return GetPlayerInfoList().GetLeaderInfoId();
    }
    
    boolean IsPlayerWinner(NetPlayerInfo cPlayerInfo,int nWinnerId)
    {
        if ( cPlayerInfo == null )
            return false;
        NetPlayerInfo cWinnerInfo = GetPlayerInfoByNetId( nWinnerId );
        if ( cWinnerInfo == null )
            return false;
        return cPlayerInfo == cWinnerInfo;
    }

    /** Co ile czasu serwer bedzie informowal wszystkich klientow 
     *  o stanie graczy */
    static float sm_fSendingMsgClientPlayersStatePeriod = 20.0f;
    /** Opoznienie rozpoczacie wysylania stanu od momentu
     * zawolania ScheduleSendingMsgClientPlayersState */
    static float sm_fSendingMsgClientPlayersStateDelay  = 10.0f;
    /** Czas po jakim bedzie zweryfikowany stan graczy od momentu odebrania
     * komunikatu, 
     * uwaga: czasn ten musi byc mniejszy niz sm_fSendingMsgClientPlayersStatePeriod
     * zeby jeden komunikat nie nadpisywal poprzedniego (nieobsluzonego jeszcze)
     * komunikatu */
    static float sm_fVerifyPlayersStateDelay = 1.0f;
    
                 
    void ScheduleSendingMsgClientPlayersState()
    {
        StopSendingMsgClientPlayersState();
        CallMethodIn( METHOD_StartSendingMsgClientPlayersState, 
                      sm_fSendingMsgClientPlayersStateDelay );
    }

    void StartSendingMsgClientPlayersState()
    {
        StopSendingMsgClientPlayersState();
        CallMethodEvery( METHOD_MsgSendClientPlayersState, 
                         sm_fSendingMsgClientPlayersStatePeriod );
    }
    void StopSendingMsgClientPlayersState()
    {
        DisableCallMethod( METHOD_MsgSendClientPlayersState );
    }

    /** Tymczasowo bedziemy pingowali automatycznie (az do 
     * uruchomienia TimeCompensatora)
     */
   
    //po odebraniu inicjalizujacego komunikatu z game settings wlaczamy pingi
    void OnSetGameSettings()
    {
        super.OnSetGameSettings();
        
        if ( Role < ERole._AUTHORITY )
        {
            CallMethodEvery( METHOD_MsgSendServerPing, 1.0f );
        }
    }
    
    /**
     * Called when game settings changes
     */
    void OnUpdateGameSettings()
    {
        super.OnUpdateGameSettings();
        
        //say player game settings changed
        //he should refresh HudMap
        if ( GetMyPlayerNet() != null )
            GetMyPlayerNet().OnUpdateGameSettings();
    }
    
    
    //po wystartowaniu kompensatora wylaczamy
    void StartTimeCompensator()
    {
        DisableCallMethod( METHOD_MsgSendServerPing );
        super.StartTimeCompensator();
    }
}
