public class NetGameInfoHud extends NetGameInfoBase
{
    final static int _MSG_PLAYER_MESSAGE                = 1; 
    final static int _MSG_TYPED_COMMAND                 = 2; 
    final static int _MSG_COMMAND_RESULTS               = 3; 
    final static int _MSG_COMMAND_ERRORS                = 4; 
    final static int _MSG_MESSAGE_FROM_SERVER           = 5; 
    final static int _MSG_FORCE_AUTO_VOICE              = 6; 
    //komunikaty dozwolone do 9, pozostale w potomnych klasach

    //manager huda sieciowego
    private HUDNetManager m_cHUDNet = null;
    HUDNetManager GetHUDNet(){ return m_cHUDNet; }
       
    //manager systemu komend konsolowych
    private ConsoleCommandsManager m_cConsoleCommandsManager = null;
    ConsoleCommandsManager GetConsoleCommandsManager(){ return m_cConsoleCommandsManager; }
    // system komunikatow przychodzacych czekajacych na obsluzenie
    private IncomingMsgManager m_cIncomingMsgManager = null;
    // system komunikatow o stanie gry
    private GameStatusMsgManager m_cGameStatusMsgManager = null;
    GameStatusMsgManager GetGameStatusMsgManager(){ return m_cGameStatusMsgManager; }
    // system Voice'ow
    private VoiceManager m_cVoiceManager = null;
    VoiceManager GetVoiceManager(){ return m_cVoiceManager; }
    
    //system automatycznych message'y, tym razem wykowyrzywane do TIPow
    private ServerAutoMessageMngr m_cServerAutoMessageMngr = null;
    
    //flaga mowiaca czy w danej chwili komunikaty sa zablokowane
    private boolean m_bMessagesDisabled = true;
    //sprawdzenie stanu
    boolean MessagesDisabled(){ return m_bMessagesDisabled; }
    //zezwolenie na komunikaty
    void    EnableMessages(){ m_bMessagesDisabled = false; }
    //zablokowanie komunikatow
    void    DisableMessages(){ m_bMessagesDisabled = true; }
    
    
    int METHOD_ActivateStatisticsOnJoin  = 0;
    int METHOD_ActivateStatisticsOnDeath = 0;
    
    
    void OnCreate()
    {
        METHOD_ActivateStatisticsOnJoin  = RegisterCallMethod("ActivateStatisticsOnJoin");
        METHOD_ActivateStatisticsOnDeath = RegisterCallMethod("ActivateStatisticsOnDeath");
        
        super.OnCreate();
        
        //tworzymy managera komend konsolowych
        m_cConsoleCommandsManager = (ConsoleCommandsManager)CreateObject("ConsoleCommandsManager");
        if ( Role == ERole._AUTHORITY )
            m_cConsoleCommandsManager.AddCommands();
        m_cConsoleCommandsManager.SetGameInfo( this );
            
        // tworzymy liste komunikatow
        m_cIncomingMsgManager   = (IncomingMsgManager)CreateObject("IncomingMsgManager");
        m_cIncomingMsgManager.SetGameInfo( this );
                        
        // tworzymy HUDa
        CreateHUDNet();
        
        //tworzymy manager komunikatow statusu gry
        String sClassName  = ENetGameType.GetGameStatusMsgManagerClassName( GetGameType() );
        GameObject cObject = CreateObject( sClassName );
        if ( cObject instanceof GameStatusMsgManager)
            SetGameStatusMsgManager( (GameStatusMsgManager)cObject );
        else
            CrashLog("ERROR: GameStatusMsgManager create failed "+cObject+"\n"); 
        
        //no i vioce manager
        m_cVoiceManager = ENetGameType.NewVoiceManager( GetGameType() );
        
        if ( Role == ERole._AUTHORITY )
        {
            m_cServerAutoMessageMngr = (ServerAutoMessageMngr)CreateObjectNoInit("ServerAutoMessageMngr");
            m_cServerAutoMessageMngr.SetGameInfo( this );
            m_cServerAutoMessageMngr.InitObject();
        }
    }
    
    public void OnDestroy()
    {
        super.OnDestroy();
        
        if ( m_cConsoleCommandsManager != null )
        {
            m_cConsoleCommandsManager.delete();
            m_cConsoleCommandsManager = null;
        }
        if ( m_cIncomingMsgManager != null )
        {
            m_cIncomingMsgManager.delete();
            m_cIncomingMsgManager = null;
        }
        if ( m_cGameStatusMsgManager != null )
        {
            m_cGameStatusMsgManager.delete();
            m_cGameStatusMsgManager = null;
        }
        if ( m_cServerAutoMessageMngr != null )
        {
            m_cServerAutoMessageMngr.delete();
            m_cServerAutoMessageMngr = null;
        }
        DeleteHUDNet();
    }
    
    void CreateHUDNet()
    {
        m_cHUDNet = (HUDNetManager)CreateObject("HUDNetManager");
    }
    
    void DeleteHUDNet()
    {
        if ( m_cHUDNet != null )
        {
            m_cHUDNet.delete();
            m_cHUDNet = null;
        }
    }
    
    void SetGameStatusMsgManager(GameStatusMsgManager cManager)
    {
         m_cGameStatusMsgManager = cManager;
         if ( m_cGameStatusMsgManager != null )
            m_cGameStatusMsgManager.SetGameInfo( this ); 
    }
    
    ///////////////////////////////////////////////////////
    //////////////////////////////////////////////////////
    
    /** Wlacza statystyki
     */
    void OnSetGameSettings()
    {
        super.OnSetGameSettings();
        
        m_cHUDNet.SetGameRule( GetGameSettings().GetTextGameRule() );
        m_cGameStatusMsgManager.Init( IsGameGoing(), GetGameSettings() );
        
        ShowWelcomeMessage();
        
        if ( Role == ERole._AUTHORITY )
        {
            EndPrecache();
        }
        else
        {
            CallMethodIn( "StartTimeCompensator",45.0f );
            CallMethodIn( "EndPrecache",0.0f );
        }
        
    }
    
    void OnUpdateGameSettings()
    {
        m_cHUDNet.SetGameRule( GetGameSettings().GetTextGameRule() );
        m_cGameStatusMsgManager.Init( IsGameGoing(), GetGameSettings() );
    }
    
    void HudNetStart()
    {
        if ( m_cHUDNet != null )
            m_cHUDNet.Start();
    }
    
    void HudNetStop()
    {
        if ( m_cHUDNet != null )
            m_cHUDNet.Stop();
    }
    
    void EndPrecache()
    {
        
        if ( Role == ERole._AUTHORITY )
        {
            if ( IsPlayerOnServer() )
                ActivateStatisticsOnJoin();
            else
                ActivateStatisticsOnStartDedicatedServer();
        }
        else
        {
            //TODO: jak bedzie mozna sie podlaczyc na kliencie bez gracza
            //to to uwzglednic
            ActivateStatisticsOnJoin();
        }
                
        if ( m_cModule.m_cPrecache != null )
             m_cModule.m_cPrecache.StartGameInTime( 0.0f );
    }
    
    final static int _STATISTICS_STATE_HIDDEN         = 0;
    final static int _STATISTICS_STATE_RESPAWN        = 1;
    final static int _STATISTICS_STATE_RESPAWN_REJECT = 2;
    int m_nStatisticsState     = _STATISTICS_STATE_HIDDEN;
    
    //tu w przypadku gry bedziemy mieli stan _STATISTICS_STATE_RESPAWN_REJECT
    //bedziemy zapamietywali przyczyne poprzedniego odrzucenia
    //checi naszego spawnu
    int m_nRespawnRejectReason = ERespawnRequestResults._ENABLED;
        
    void ActivateStatisticsOnStartDedicatedServer()
    {
        if ( m_cHUDNet != null )
            m_cHUDNet.ActivateStatistics( NetMsg.sTipWelcomeOnDedicatedServer, true );
    }
    
    boolean ActivateStatisticsOnJoin()
    {
        //jesli jestesmy juz w inventory menu, to ignorujemy 
        //wyswietlenie tych statystyk
        if ( m_cModule.IsActiveInventoryMenu() )
            return false;
        
        //jesli juz bylo zadania spawnu to tez nie pokazujemy
        //tych statystyk
        if ( WasLocalPlayerPressedFireToRespawn() )
            return false;
                
        String sPressKey = IsInventoryRoomAvailable() ? 
                           NetMsg.GetTipRespawnOrInventoryRoom() : 
                           NetMsg.sTipPressEnterToRespawn;
        m_nStatisticsState = _STATISTICS_STATE_RESPAWN;
        if ( m_cHUDNet != null )
            m_cHUDNet.ActivateStatistics( sPressKey, true );
        return true;
    }
    
    void ActivateStatisticsOnRespawnRequest()
    {
        DeactivateStatisticsOnRespawnRequest();
        
        m_nStatisticsState = _STATISTICS_STATE_HIDDEN;    
        
        if ( m_cHUDNet != null )
             m_cHUDNet.ActivateStatistics( NetMsg.sTipPleaseWait, true );
        //przy pierwszej probie spawnowania startujemuy huda
        HudNetStart();
    }
           
    void ActivateStatisticsOnRespawnReject()
    {
        ActivateStatisticsOnRespawnReject( m_nRespawnRejectReason );
    }
    void ActivateStatisticsOnRespawnReject(int nReason)
    {
        DeactivateStatisticsOnRespawnRequest();
        
        m_nStatisticsState     = _STATISTICS_STATE_RESPAWN_REJECT;
        m_nRespawnRejectReason = nReason;
        
        if ( m_cHUDNet != null )
        {
            String sText = "";
            switch ( nReason )
            {
            case ERespawnRequestResults._DISABLED_GAME_NOTSTARTED: 
                //sText = NetMsg.sTipCantRespawnWaitForGameStart;
                sText = NetMsg.sTipWaitingForOtherPlayers;
                break;
                
            case ERespawnRequestResults._DISABLED_GAME_OVER: 
                sText = NetMsg.sTipCantRespawnCauseGameOver;
                break;
                
            case ERespawnRequestResults._DISABLED_RESPAWN_LIMIT_OUT: 
                sText = NetMsg.sTipCantRespawnCauseLimitOut;
                break;
                
            case ERespawnRequestResults._DISABLED_NO_REASON: 
            default:
                sText = NetMsg.sTipCantRespawn;
                break;
            }
            
            m_cHUDNet.ActivateStatistics( sText, true );
        }
    }
    
    void ActivateStatisticsOnDeath(float fTime)
    {
        CallMethodIn( METHOD_ActivateStatisticsOnDeath,fTime);
        m_nStatisticsState = _STATISTICS_STATE_RESPAWN;
    }
    
    void ActivateStatisticsOnDeath()
    {
        //jesli jestesmy juz w inventory menu, to ignorujemy 
        //wyswietlenie tych statystyk
        if ( m_cModule.IsActiveInventoryMenu() )
            return;
        
        String sPressKey = IsInventoryRoomAvailable() ? 
                           NetMsg.GetTipRespawnOrInventoryRoom() : 
                           NetMsg.sTipPressEnterToRespawn;
        m_nStatisticsState = _STATISTICS_STATE_RESPAWN;
        if ( m_cHUDNet != null )
             m_cHUDNet.ActivateStatistics( sPressKey, true );
    }
    
    void ActivateStatisticsOnGameRestart()
    {
        //jesli jestesmy juz w inventory menu, to ignorujemy 
        //wyswietlenie tych statystyk
        if ( m_cModule.IsActiveInventoryMenu() )
            return;
        
        // najpierw na chwilke wylaczmy statystyki...
        DeactivateStatistics();
            
        String sPressKey = IsInventoryRoomAvailable() ? 
                           NetMsg.GetTipRespawnOrInventoryRoom() : 
                           NetMsg.sTipPressEnterToRespawn;

        m_nStatisticsState = _STATISTICS_STATE_RESPAWN;
        
        if ( m_cHUDNet != null )
             m_cHUDNet.ActivateStatistics( /*NetMsg.sTipGameRestart + " "+ */
                                           sPressKey, true );
    }
    
    void ActivateStatisticsOnChooseTeam()
    {
        if ( m_cHUDNet != null )
             m_cHUDNet.ActivateStatistics( NetMsg.sTipChooseTeam, true );
    }
    
    void ActivateStatisticsOnGameOver()
    {
        HideInventoryMenuIfActive();
        
        // najpierw na chwilke wylaczmy statystyki...
        DeactivateStatistics();
            
        if ( m_cHUDNet != null )
             m_cHUDNet.ActivateStatistics( NetMsg.sTipGameOver, true );
    }
    
    
    void ActivateStatisticsOnGameOverAndContinue()
    {
        HideInventoryMenuIfActive();
        
        // najpierw na chwilke wylaczmy statystyki...
        DeactivateStatistics();
            
        if ( m_cHUDNet != null )
             m_cHUDNet.ActivateStatistics( NetMsg.sTipGameOver + " "+Text.Get("&PRESS_SPACE_TO_CONTINUE&"), 
                                           true );
    }
    
    void ActivateStatisticsOnWaitForSpaceToContinue()
    {
        HideInventoryMenuIfActive();
        
        // najpierw na chwilke wylaczmy statystyki...
        DeactivateStatistics();
        
        if ( m_cHUDNet != null )
        {
            m_cHUDNet.ActivateStatistics( Text.Get("&PRESS_SPACE_TO_CONTINUE&"), true );
        }
    }
    
    void ActivateStatisticsOnPleaseWait()
    {
        HideInventoryMenuIfActive();
        
        // najpierw na chwilke wylaczmy statystyki...
        DeactivateStatistics();
        
        if ( m_cHUDNet != null )
        {
            m_cHUDNet.ActivateStatistics( NetMsg.sTipPleaseWait, true );
        }
    }
    
    void ActivateStatisticsOnDisconnect()
    {
        HideInventoryMenuIfActive();
        
        if ( m_cHUDNet != null )
             m_cHUDNet.ActivateStatistics( "", true );
    }
    
    void DeactivateStatisticsOnRespawnRequest()
    {
        if ( ! IsGameOver() )
            DeactivateStatistics();
    }
    
    void DeactivateStatisticsOnRespawn()
    {
        if ( ! IsGameOver() )
            DeactivateStatistics();
    }
    
    /**
     * jesli jestesmy wlasnie w inventory menu, 
     * to chowamy to menu
     */
    void HideInventoryMenuIfActive()
    {
        if ( m_cModule.IsActiveInventoryMenu() )
             m_cModule.RestoreAfterInventoryMenu();
    }
    
    /** Wylacza statystyki
     */
    void DeactivateStatistics()
    {
        DisableCallMethod( METHOD_ActivateStatisticsOnJoin );
        DisableCallMethod( METHOD_ActivateStatisticsOnDeath );
        if ( m_cHUDNet != null )
             m_cHUDNet.DeactivateStatistics( true );
        m_nStatisticsState = _STATISTICS_STATE_HIDDEN;
    }
    
    void LockHUDNet( boolean bLock )
    {
        if ( m_cHUDNet != null )
             m_cHUDNet.Lock( bLock );
    }
    
    void RefreshStatistics()
	{
		if ( m_cHUDNet != null )
			 m_cHUDNet.RefreshStatistics();
	}
    
          
    ////////////////////////////////////////////////////////////////
    
    void EventServerNotResponding()
    {
        super.EventServerNotResponding();
        if ( ! IsGameOver() )
        {
            AddHUDSystemMessage( NetMsg.sServerNotResponding );
            GetConnectionStatusMngr().TimeCompensatorServerNotResponding();
        }
    }
    
    void EventServerResponding()
    {
        super.EventServerResponding();
        SystemMessageHide();
        GetConnectionStatusMngr().TimeCompensatorOk();
    }
        
    void EventConnectionToServerLost()
    {
        super.EventConnectionToServerLost();
        if ( ! IsGameOver() )
        {
            AddHUDSystemMessage( NetMsg.sConnectionToServerLost );
            GetConnectionStatusMngr().TimeCompensatorConnectionToServerLost();
        }
    }
    
    void EventDisconnectFromServer( int nReason )
    {
        super.EventDisconnectFromServer( nReason );
        ShowInfoOnDisconnect( nReason );
    }
    
    void EventDisconnectByConnectionStatusManager()
    {
        super.EventDisconnectByConnectionStatusManager();
        //TODO: a moze tez przekazac z ConnectionStatusManagera
        //jakas przyczyna odlaczenia?
        ShowInfoOnDisconnect( EDisconnectReason._NO_REASON );
    }
    
    void ShowInfoOnDisconnect(int nReason)
    {
        if ( ! IsGameOver() )
        {
            ActivateStatisticsOnDisconnect();
            
            String sDisconnectFromServerReason = NetMsg.sDisconnectFromServer;
            switch ( nReason )
            {
            }
            AddHUDSystemMessage( sDisconnectFromServerReason );
        }
    }

    void AddHUDSystemMessage(String sText)
    {
        m_cHUDNet.AddSystemMessage( sText );
    }    
    
    void SystemMessageHide()
    {
        AddHUDSystemMessage( null );        
    }

    
    
    ///////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////
    ////// HUD sieciowy
    ///////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////
    
    
    void LocalEventTypedText(String sText,int nMode,int nDestHostId)
    {
        if (sText == null || sText.length() == 0)
            return;
        
        //stwierdzmy czy czasem nie mamy do czynienia z jakas komenda
        ConsoleCommand cCommand = m_cConsoleCommandsManager.CheckCommand( sText );
        if ( cCommand != null )
        {
            //tak, to jakas komenda, trzeba ja zawolac
            LocalEventTypedCommand( cCommand );
        }
        else     
        {
            //to tylko zwykly chat msg
            NetLog.Test("MY CHAT MSG: "+sText);
            LocalEventPlayerMessage( new NetMsg( sText ,nMode, nDestHostId ) );
        }
    }
    
    void LocalEventTypedText(String sText,int nMode)
    {
        LocalEventTypedText( sText, nMode, -1 /* nDestHostId */);
    }

    void LocalEventVoice(int nVoice)
    {
        if ( m_cVoiceManager == null )
            return;
        //TODO: jesli bedzie dostepne tylko w teamowych, to przeniesc do NetGameInfoTeam
        LocalEventPlayerMessage( new NetMsg( nVoice ) );
    }
    
    void LocalEventVoiceMessageOnGrenadeThrow()
    {
        if ( m_cVoiceManager == null )
            return;
        NetVoice cVoice = m_cVoiceManager.GetAutoVoiceOnGrenadeThrow();
        if ( cVoice != null )
            LocalEventVoice( cVoice.GetId() );
    }
    
    void LocalEventTypedCommand(String sCommand)
    {
        ConsoleCommand cCommand = m_cConsoleCommandsManager.CheckCommand( sCommand );
        if ( cCommand != null )
        {
            //tak, to jakas komenda, trzeba ja zawolac
            LocalEventTypedCommand( cCommand );
        }
    }
    
    private void LocalEventTypedCommand(ConsoleCommand cCommand)
    {
        if (Role == ERole._AUTHORITY)
        {
            m_cConsoleCommandsManager.ServerTypedCommand( cCommand );
        }
        else
        {
            //jestesmy klientem, 
            //czy to komenda lokalna czy serwerowa?
            if ( cCommand.ShouldExecuteOnlyOnServer() )
            {
                //niestety serwerowa, wiec nie mozemy niz sami zdzialac
                //wysylamy do serwera wklepana przez nas komende
                MsgSendServerTypedCommand( cCommand );
            }
            else
            {
                //moze byc wykonana u nas, wiec wykonajmy ja
                m_cConsoleCommandsManager.ClientTypedCommand( cCommand );
            }
        }
    }
    
    private void LocalEventPlayerMessage(NetMsg cMsg)
    {
        if (Role == ERole._AUTHORITY)
        {
            NetPlayerInfo cWriterInfo = GetPlayerInfoList().GetPlayerByHostId(0);
            EventPlayerMessage( cMsg, cWriterInfo );
        }
        else
        {
            MsgSendServerPlayerMessage( cMsg );
            
            //na kliencie pokazujemy komunikat i serwer nie bedzie musial
            //nam go z powrotem przesylac...
            NetPlayerInfo cWriterInfo   = GetPlayerInfoList().GetMyPlayer();
            int           nWriterInfoId = (cWriterInfo != null) ? 
                                           cWriterInfo.GetNetId() : -1;
            AddTextOrVoiceMessage( cMsg, nWriterInfoId );
        }
    }
        
    private void EventPlayerMessage( NetMsg        cMsg, 
                                     NetPlayerInfo cWriterInfo )
    {
        //jesli cWriterInfo == null to znaczy ze pisano na serwerze dedykowanym        
        int nWriterInfoId = (cWriterInfo != null) ? 
                             cWriterInfo.GetNetId() : -1 ;
        int nWriterStatus = (cWriterInfo != null) ? 
                             cWriterInfo.GetStatus(): NetPlayerInfo._STATUS_ALIVE;
        
        //jezeli gracz piszacy zyje i mamy tryb "do wszystkich"
        //to mozemy wyslac do wszystkich
        if ( nWriterStatus == NetPlayerInfo._STATUS_ALIVE && cMsg.IsAllMode() )
        {
            int nWriterHost = (cWriterInfo != null) ? 
                              cWriterInfo.GetClientOwnerId() : -1;
            MsgSendClientPlayerMessage( cMsg, nWriterInfoId, nWriterHost, false );

            AddTextOrVoiceMessage( cMsg, nWriterInfoId );
            return;
        }
        
        //mamy albo tryb teamowy, albo nickowy
        
        //trzeba ustalic id druzyny do ktorej nalezy graz wysylajacy
        int nWriterTeamId = cWriterInfo.GetTeamId();
        
        for ( int i=0; i< GetPlayerInfoList().GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerInfoList().GetPlayerByIndex( i );
            
            if ( cPlayerInfo == null )
                continue;
            
            //jezeli piszacy wiadomosc ma nizszy status niz
            //potencjalny adresat, to nie mozna mu wyslac
            if ( nWriterStatus < cPlayerInfo.GetStatus() )
            {
                //TODO: wyslac info do piszacego o niepowodzeniu
                continue;
            }
            
            //jesli mamy tryb druzynywy i gosc jest z innej druzyny to go omijamy
            if ( cMsg.IsTeamMode() && cPlayerInfo.GetTeamId() != nWriterTeamId )
                continue;
            
            
            
            //jesli mamy tryb nickowy i to nie jest ten gosc 
            //do ktorego chcemy pisac, to omijamy
            if ( cMsg.IsNickMode() && cPlayerInfo.GetClientOwnerId() != cMsg.m_nDestHostId )
                continue;
                 
            if ( cPlayerInfo.bNetIsOwner )
            {
                AddTextOrVoiceMessage( cMsg, nWriterInfoId );
            }
            else
            {
                if ( cPlayerInfo != cWriterInfo )
                {
                    int nHost = cPlayerInfo.GetClientOwnerId();
                    MsgSendClientPlayerMessage( cMsg, nWriterInfoId, 
                                                nHost, true );
                }
            }
        }

        //a to na serwerze dedykowanym
        if ( GetPlayerInfoList().GetMyPlayer() == null )
            AddTextOrVoiceMessage( cMsg, nWriterInfoId );
    }
    
    void EventServerMessage(String sText, int nMode)
    {
        if ( sText == null || sText.length() == 0 )
            return;
                       
        if (Role < ERole._AUTHORITY)
            return;
        
        if ( ServerAutoMessageMngr.AllowIncomingMsg( nMode ) )        
            AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, sText );
        
        MsgSendClientMessageFromServer( sText, nMode );
    }
    
    void EventAutoVoice(NetVoice cVoice,int nTeamId,Vector vPos)
    {
        EventAutoVoice( cVoice, nTeamId, vPos, null );
    }
    void EventAutoVoice(NetVoice cVoice,int nTeamId,Vector vPos,NetPlayerInfo cExclude)
    {
        if ( vPos != null )
        {
            NetPlayerInfo cPlayerInfo = 
                     GetPlayerInfoList().GetNearestAlivePlayer( nTeamId, vPos, cExclude );
        
            EventAutoVoice( cVoice, cPlayerInfo );
        }
        else
            EventAutoVoice( cVoice, nTeamId );
    }
    
    void EventAutoVoice(NetVoice cVoice,int nTeamId)
    {
        NetPlayerInfo cPlayerInfo = 
                     GetPlayerInfoList().GetAlivePlayer( nTeamId );
        
        EventAutoVoice( cVoice, cPlayerInfo );
    }
        
    void EventAutoVoice(NetVoice cVoice,NetPlayerInfo cPlayerInfo)
    {
        if ( cVoice == null )
            return;
        if ( cPlayerInfo == null )
            return;

        int nVoiceId = cVoice.GetId();
        int nHostId  = cPlayerInfo.GetClientOwnerId();
        if ( nHostId == 0 )
        {
            if ( IsLocalPlayerAlive() )
                LocalEventVoice( nVoiceId );
        }
        else
            MsgSendClientForceAutoVoice( nVoiceId, nHostId );
    }

    /** Metoda wykonawcza wyswietlajaca tekst jako rezultat komendy
     * Tekst pojawi sie w grze (w przychodzacych komunikatach)
     * oraz na konsoli tekstowej
     **/
    void WriteCommandResults( String sText )
    {
        AddIncomingMessage( NetIncomingMsg._ANSWER_COMMAND, sText );
        NetLog.CommandAnswer(sText);
    }
    
    void WriteSpecialCommandResults( String sText )
    {
        AddAdminMessage( sText );
        NetLog.CommandAnswer(sText);
    }
    
    /** 
     * Zgloszenie nowego komunikatu o podanych parametrach
     */
    void AddIncomingMessage( int nId )
    {
        AddIncomingMessage( nId, -1 );
    }
    
    void AddIncomingMessage( int nId, int nPlayerInfo1Id )
    {
        AddIncomingMessage( nId, nPlayerInfo1Id, -1 );
    }
    
    void AddIncomingMessage( int nId, int nPlayerInfo1Id, int nPlayerInfo2Id )
    {
        NetIncomingMsg cMsg = new NetIncomingMsg( this, nId, nPlayerInfo1Id, nPlayerInfo2Id );
        AddIncomingMessage( cMsg );
    }
    
    void AddIncomingMessage( int nId, int nPlayerInfo1Id, int nPlayerInfo2Id, int nTeamInfoId )
    {
        NetIncomingMsg cMsg = new NetIncomingMsg( this, nId, nPlayerInfo1Id, nPlayerInfo2Id, nTeamInfoId );
        AddIncomingMessage( cMsg );
    }
    
    void AddIncomingMessage( int nId, int nPlayerInfo1Id, int nPlayerInfo2Id, int nTeamInfoId, int nAddInfo )
    {
        NetIncomingMsg cMsg = new NetIncomingMsg( this, nId, nPlayerInfo1Id, nPlayerInfo2Id, nTeamInfoId, nAddInfo );
        AddIncomingMessage( cMsg );
    }
    
    void AddIncomingMessage( int nId, int nPlayerInfo1Id, int nPlayerInfo2Id, int nTeamInfoId, int nAddInfo, int nAddObjectId )
    {
        NetIncomingMsg cMsg = new NetIncomingMsg( this, nId, nPlayerInfo1Id, nPlayerInfo2Id, nTeamInfoId, nAddInfo, nAddObjectId );
        AddIncomingMessage( cMsg );
    }
    
    void AddIncomingMessage( String sText )
    {
        AddIncomingMessage( sText, (Vector)null );
    }
    
    void AddIncomingMessage( String sText, Vector vColor )
    {
        AddIncomingMessage( NetIncomingMsg._MSG_NON_STANDARD, sText, vColor, (String)null );
    }
    
    void AddIncomingMessage( int nId, String sText, Vector vColor, String sWave )
    {
        NetIncomingMsg cMsg = new NetIncomingMsg( this, nId, sText, -1 );
        if ( vColor != null )
            cMsg.SetRequestedColor( vColor );
        if ( sWave != null )
            cMsg.SetRequestedSound( sWave );
        AddIncomingMessage( cMsg );
    }
    
    void AddIncomingMessage( int nId, String sText )
    {
        AddIncomingMessage( nId, sText, -1 );
    }
    
    void AddIncomingMessage( int nId, String sText, int nPlayerInfo1Id )
    {
        AddIncomingMessage( nId, sText, nPlayerInfo1Id, -1 );
    }
    
    void AddIncomingMessage( int nId, String sText, int nPlayerInfo1Id, int nAddInfo )
    {
        NetIncomingMsg cMsg = new NetIncomingMsg( this, nId, sText, nPlayerInfo1Id, nAddInfo );
        AddIncomingMessage( cMsg );
    }

    void AddIncomingMessage( int nId, String sText, String sText2, int nPlayerInfo1Id )
    {
        NetIncomingMsg cMsg = new NetIncomingMsg( this, nId, sText, sText2, nPlayerInfo1Id );
        AddIncomingMessage( cMsg );
    }
    
    private void AddTextOrVoiceMessage( NetMsg cMsg, int nWriterInfoId )
    {
        if ( cMsg.m_sText != null )
        {
            AddIncomingMessage( NetIncomingMsg._TYPED_TEXT, 
                                cMsg.m_sText,
                                nWriterInfoId,
                                cMsg.m_nMode );
        }
        else
            AddVoiceMessage( cMsg.m_nVoice, nWriterInfoId );
    }
    
    private void AddVoiceMessage( int nId )
    {
        AddVoiceMessage( nId, -1 );
    }
    private void AddVoiceMessage( int nId, int nWriterInfoId )
    {
        if ( m_cVoiceManager == null )
            return;
        NetVoice cVoice = m_cVoiceManager.Get( nId );
        if ( cVoice != null )
            AddIncomingMessage( new NetVoiceMessage( this, cVoice, nWriterInfoId ) );
    }
    /**
     * Metoda rejestrujaca nowy komunikat w systemie, 
     * jezeli bedzie on gotowy do wyswietlenia to zostanie
     * wyslany do HUDa
     */
    void AddIncomingMessage( NetIncomingMsg cMsg )
    {
        if ( cMsg != null )
            m_cIncomingMsgManager.Add( cMsg );
    }
    
    void AddAdminMessage( String sText )
    {
        if ( GetHUDNet() != null )
            GetHUDNet().AddAdminMessage( sText );
    }
    /****
     * koniec obslugi wysylanie message'y
     * 
     */
    
    
    
    
    void MsgSendServerTypedCommand(ConsoleCommand cCommand)
    {
        MessageNew(_MSG_TYPED_COMMAND);
        MessagePackInt( cCommand.GetId() );
        cCommand.Pack( this );
        MessageSend();
        
        //dobrym zwyczajem po zabawie czyscimy komende
        cCommand.Clean();
    }
    
    void MsgRecvServerTypedCommand()
    {
        int nCommandId = MessageUnPackInt();
        ConsoleCommand cCommand = m_cConsoleCommandsManager.FindCommand( nCommandId );
        if ( cCommand != null )
            cCommand.UnPack( this );
        MessageEnd();
        
        if ( ( cCommand != null ) && ( Role == ERole._AUTHORITY ))
            m_cConsoleCommandsManager.ServerReceivedCommand( cCommand );
    }
   
    
    void MsgSendClientPlayerMessage(NetMsg cMsg, int nWriterInfoId, 
                                    int nHost,    boolean bSendHost )
    {
        MessageNew(_MSG_PLAYER_MESSAGE);
        MessagePackBoolean( cMsg.m_sText != null );
        if ( cMsg.m_sText != null )
            MessagePackString( cMsg.m_sText );
        else
            MessagePackInt( cMsg.m_nVoice );
        MessagePackInt( nWriterInfoId );
        if ( bSendHost )
            MessageSendHost( nHost );
        else
            MessageSendNotHost( nHost );
    }
    
    void MsgRecvClientPlayerMessage()
    {
        String sText = null;
        int nVoiceId = -1;
        if ( MessageUnPackBoolean() )
            sText   = MessageUnPackString();
        else
            nVoiceId = MessageUnPackInt();
        int nWriterInfoId = MessageUnPackInt();
        MessageEnd();
        
        if ( sText != null )
            AddIncomingMessage( NetIncomingMsg._TYPED_TEXT, 
                                sText,
                                nWriterInfoId );
        else
            AddVoiceMessage( nVoiceId, nWriterInfoId );
    }


    void MsgSendServerPlayerMessage(NetMsg cMsg)
    {
        MessageNew(_MSG_PLAYER_MESSAGE);
        MessagePackBoolean( cMsg.m_sText != null );
        if ( cMsg.m_sText != null )
        {
            MessagePackString( cMsg.m_sText );
            MessagePackInt( cMsg.m_nMode );
            MessagePackInt( cMsg.m_nDestHostId );
        }
        else
            MessagePackInt( cMsg.m_nVoice );
        MessageSend();
    }
    
    void MsgRecvServerPlayerMessage()
    {
        String sText        = null;
        int    nVoiceId     = -1;
        int    nMode        = -1;
        int    nDestHostId  = -1;
        if ( MessageUnPackBoolean() )
        {
            sText       = MessageUnPackString();
            nMode       = MessageUnPackInt();
            nDestHostId = MessageUnPackInt();
        }
        else
            nVoiceId = MessageUnPackInt();
        int    nHostWriter = MessageGetSenderId();
        MessageEnd();
        
        int nWriterInfoId = -1;
        NetPlayerInfo cWriterInfo = GetPlayerInfoList().GetPlayerByHostId(nHostWriter);
        if ( cWriterInfo != null )
            nWriterInfoId = cWriterInfo.GetNetId();
            
        NetMsg cMsg = null;
        if ( sText != null )
            cMsg = new NetMsg( sText, nMode, nDestHostId );
        else
            cMsg = new NetMsg( nVoiceId );
            
        EventPlayerMessage( cMsg, cWriterInfo );
    }
    
    void MsgSendClientCommandResults(ConsoleCommand cCommand)
    {
        MessageNew(_MSG_COMMAND_RESULTS);
        MessagePackInt( cCommand.GetId() );
        cCommand.PackResults( this );
        if ( cCommand.AreResultsToAll() )
            MessageSend();
        else
            MessageSendHost( cCommand.GetHost() );
    }
        
    void MsgRecvClientCommandResults()
    {
        int nCommandId = MessageUnPackInt();
        ConsoleCommand cCommand = m_cConsoleCommandsManager.FindCommand( nCommandId );
        if ( cCommand == null )
             cCommand = m_cConsoleCommandsManager.FindCommand( ConsoleCommandsManager._COMMAND_SERVER );
        cCommand.UnPackResults( this );
        MessageEnd();
        
        if ( cCommand != null )
        {
            cCommand.ShowResults();
            //dobrym zwyczajem po zabawie czyscimy komende
            cCommand.Clean();
        }
    }
    
    void MsgSendClientCommandErrors(ConsoleCommand cCommand)
    {
        MessageNew(_MSG_COMMAND_ERRORS);
        MessagePackInt( cCommand.GetId() );
        cCommand.PackErrors( this );
        MessageSendHost( cCommand.GetHost() );
    }

    void MsgRecvClientCommandErrors()
    {
        int nCommandId = MessageUnPackInt();
        ConsoleCommand cCommand = m_cConsoleCommandsManager.FindCommand( nCommandId );
        if ( cCommand == null )
             cCommand = m_cConsoleCommandsManager.FindCommand( ConsoleCommandsManager._COMMAND_SERVER );
        cCommand.UnPackErrors( this );
        MessageEnd();
        
        if ( cCommand != null )
        {
            cCommand.ShowErrors();
            //dobrym zwyczajem po zabawie czyscimy komende
            cCommand.Clean();
        }
    }

    void MsgSendClientForceAutoVoice(int nVoiceId,int nHostId)
    {
        MessageNew( _MSG_FORCE_AUTO_VOICE );
        MessagePackInt( nVoiceId );
        MessageSendHost( nHostId );
    }
    
    void MsgRecvClientForceAutoVoice()
    {
        int nVoiceId = MessageUnPackInt();
        MessageEnd();
        
        if ( IsLocalPlayerAlive() )
            LocalEventVoice( nVoiceId );
    }
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_MESSAGE_FROM_SERVER:
            LogD("_MSG_MESSAGE_FROM_SERVER\n");
            if (Role < ERole._AUTHORITY)
                MsgRecvClientMessageFromServer();
            break;
            
        case _MSG_PLAYER_MESSAGE:
            LogD("_MSG_PLAYER_MESSAGE\n");
            if (Role < ERole._AUTHORITY)
                MsgRecvClientPlayerMessage();
            else
                MsgRecvServerPlayerMessage();
            break;
        
        case _MSG_TYPED_COMMAND:
            LogD("_MSG_TYPED_COMMAND\n");
            MsgRecvServerTypedCommand();
            break;
            
        case _MSG_COMMAND_RESULTS:
            LogD("_MSG_COMMAND_RESULTS\n");
            MsgRecvClientCommandResults();
            break;
            
        case _MSG_COMMAND_ERRORS:
            LogD("_MSG_COMMAND_ERRORS\n");
            MsgRecvClientCommandErrors();
            break;
            
        case _MSG_FORCE_AUTO_VOICE:
            LogD("_MSG_FORCE_AUTO_VOICE\n");
            MsgRecvClientForceAutoVoice();
            break;
            
        default:
            MessageCancel();
        }
    }
    
    void MsgSendClientMessageFromServer(String sText,int nMode)
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_MESSAGE_FROM_SERVER);
        MessagePackString(sText);
        MessagePackInt(nMode);
        MessageSend();
    }

    void MsgRecvClientMessageFromServer()
    {
        String sText      = MessageUnPackString();
        int    nMode      = MessageUnPackInt();
        MessageEnd();
        
        if ( ServerAutoMessageMngr.AllowIncomingMsg( nMode ) )        
            AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, sText );
    }
    
    int  GetDominationBaseCnt(){ return 0; }
    
    
    void ShowWelcomeMessage()
    {
        /*
        AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                           " " );
        AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                           "Po zakonczeniu (lub wywaleniu sie) gry prosze o skopiowanie" );
        AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                           "pliku Crash.log do \\\\MACIEK\\temp\\CrashLogs. Dzieki..." );
        if ( ! m_cModule.AvailableInventoryMenu() )
        {
            AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                                "Nadal z przyczyn obiektywnych ;-) zablokowane jest InventoryRoom" );
        }
        
        AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                           " " );
        */
    }
    
    boolean SetPhysicsGranularity(float fPhysicsGranularity)
    {
        float   fOldVal = PlayerNet.sm_fPhysicsGranularity;
        boolean bRet = super.SetPhysicsGranularity( fPhysicsGranularity );
        if ( bRet )
        {
            AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                                "Ziarno fizyki zmienione z "+
                                fOldVal+" s. na "+ 
                                fPhysicsGranularity+" s.\n");    
        }
        return bRet;
    }
    
    boolean SetMaxResendMoveTime( float fMaxResendMoveTime )
    {
        float   fOldVal = PlayerNet._MAX_RESEND_MOVE_TIME;
        boolean bRet = super.SetMaxResendMoveTime( fMaxResendMoveTime );
        if ( bRet )
        {
            AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                                "PlayerNet._MAX_RESEND_MOVE_TIME zmienione z "+
                                fOldVal+" s. na "+ 
                                fMaxResendMoveTime+" s.\n");    
        }
        return bRet;
    }
    
    boolean SetAutoPlayersFire( boolean bFire )
    {
        boolean bRet = super.SetAutoPlayersFire( bFire );
        if ( bRet )
        {
            AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE, 
                                "PlayerNetTestBot.sm_bCanUseWeapon "+bFire+"");    
        }
        return bRet;
    }
    
    boolean SetClientShutdown()
    {
        boolean bRet = super.SetClientShutdown();
        if ( bRet )
        {
            AddIncomingMessage( NetIncomingMsg._SERVER_MESSAGE,"GameShutdown");    
        }
        return bRet;
    }
    
        
    void SetGameRestart()
    {
        super.SetGameRestart();
        if ( m_cGameStatusMsgManager != null && ModuleNet.IsGameGoing() )
            m_cGameStatusMsgManager.OnGameRestart();
    }
    void SetGameOver()
    {
        super.SetGameOver();
        
        SystemMessageHide();
        AddIncomingMessage( NetIncomingMsg._GAME_OVER );
        //TODO: podac wyniki        
        if ( m_cGameStatusMsgManager != null && ModuleNet.IsGameGoing() )
            m_cGameStatusMsgManager.OnGameOver();
    }
    
    void OnScoreChange()
    {
        if ( m_cGameStatusMsgManager != null && ModuleNet.IsGameGoing() )
            m_cGameStatusMsgManager.OnGameScoreChange();
    }
    void OnTeamChange()
    {
        if ( m_cGameStatusMsgManager != null && ModuleNet.IsGameGoing() )
            m_cGameStatusMsgManager.OnTeamChange();
    }
    
    /**
     * 
     * Called from ConsoleCommand's
     * 
     */
    
    
    /**
     * Called from CommandMapSetParam
     */
    void EventGameSettingsChanged()
    {
        OnUpdateGameSettings();
    }
    
    /**
     * Called from CommandMapNext
     */
    void EventRequestMapNext()
    {
        CallMethodIn( METHOD_ServerForceGameOver, 0.0f );
    }
    
    /**
     * Called from CommandMap
     */
    void EventRequestMap()
    {
        EventRequestMapNext();
    }

}

