public class NetGameInfoSMD extends NetGameInfoIndividual  
{
    NetGameInfoSMD()
    {
        super();
        
        SetGameType( ENetGameType.SMD );
    }

    public void OnDestroy()
    {
        super.OnDestroy();
        
        m_cGameSettingsSMD = null;
        m_cGameStatusMsgMngrSMD = null;
    }
    
    NetGameSettingsSMD m_cGameSettingsSMD = null;
    NetGameSettingsSMD GetGameSettingsSMD(){ return m_cGameSettingsSMD; }
    void SetGameSettings( NetGameSettings cGameSettings )
    {
        super.SetGameSettings( cGameSettings );
        if ( cGameSettings instanceof NetGameSettingsSMD )
             m_cGameSettingsSMD = (NetGameSettingsSMD)GetGameSettings(); 
        else
            CrashLog(" Niepoprawny obiekt GameSettings: "+cGameSettings+
                     " dla GameInfoSMD: "+this+"\n");
    }
    
    GameStatusMsgMngrSMD m_cGameStatusMsgMngrSMD = null;
    void SetGameStatusMsgManager(GameStatusMsgManager cManager)
    {
        super.SetGameStatusMsgManager( cManager );
        
        if ( cManager instanceof GameStatusMsgMngrSMD )
            m_cGameStatusMsgMngrSMD = (GameStatusMsgMngrSMD)cManager;
        else
            CrashLog(" Niepoprawny obiekt GameStatusMsgManager "+cManager+
                     " dla NetGameInfoSMD "+this+"\n");
    }
    
    void ServerInitGame() 
    {
        super.ServerInitGame();
        
        ServerInitDominationGame();
    }
    
    float GetDominationScoreUpdateTime()
    {
        return m_cGameSettingsSMD.GetScoreUpdateTime();
    }
    
      
    final static int _MSG_BASE_CAPTURED                 = 100;
    final static int _MSG_BASE_LOST                     = 101;
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_BASE_CAPTURED:
            LogD("_MSG_BASE_CAPTURED\n");
            MsgRecvClientBaseCaptured();
            break;
            
        case _MSG_BASE_LOST:
            LogD("_MSG_BASE_LOST\n");
            MsgRecvClientBaseLost();
            break;
        
        default:
            super.OnMessage();
        }
    }

    void MsgSendClientBaseCaptured( int nPlayerInfoId )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_BASE_CAPTURED);
        MessagePackInt( nPlayerInfoId );
        MessageSend();
    }
    
    void MsgRecvClientBaseCaptured()
    {
        int nPlayerInfoId = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._SMD_BASE_CAPTURED, 
                            nPlayerInfoId );
    }
    
    void MsgSendClientBaseLost( int nPlayerInfoId )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_BASE_LOST);
        MessagePackInt( nPlayerInfoId );
        MessageSend();
    }
    
    void MsgRecvClientBaseLost()
    {
        int nPlayerInfoId = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._SMD_BASE_LOST, 
                            nPlayerInfoId );
    }
    
    
    void EventPlayerReachedBase( DominationBase cBase, PlayerNet cPlayer )
    {
        if ( cBase == null )
            return;
        
        //sprawdzic czy baza jest na liscie
        if ( cBase != (DominationBase) m_aDominationBases.elementAt( cBase.GetIndex() )  )
            return;
        
        //sprawdzic, czy gracz jest w sensownej odleglosci od bazy
        
        NetPlayerInfo cPlayerInfo   = null;
        int           nPlayerInfoId = -1;
        if ( cPlayer != null )
        {
            cPlayerInfo = cPlayer.GetPlayerInfo();
            if (cPlayerInfo != null)
            {
                nPlayerInfoId = cPlayerInfo.GetNetId();
            }
            else
            {
                CrashLog("ERROR: "+this+" PlayerReachedBase "+cPlayer+" cNetPlayerInfo == null\n");
                return;
            }
        }
        else
        {
            CrashLog("ERROR: "+this+" PlayerReachedBase "+cPlayer+"\n");
            return;
        }
        
        if ( cBase.GetPlayerOwner() != cPlayerInfo )
        {
            //zmienic wlasciciela bazy
            SetBaseOwner( cBase, cPlayerInfo );
            
            //zmienic kolor bazy
            cBase.UpdateState();
            
            //policz na nowe warunki zapunktowania
            UpdateTimeToNextDominationScore();
                        
            //wyswietlic komunikaty
            //TODO:
            //jaka baza?
            AddIncomingMessage( NetIncomingMsg._SMD_BASE_CAPTURED, 
                                nPlayerInfoId );
            MsgSendClientBaseCaptured( nPlayerInfoId );
            
            //a moze komunikat tylko jak przejmnie sie _wszyskie_ bazy?
            //oczywiscie jak sie gra w trybie total
        }
    }
    
    void EventPlayerDied( PlayerNetBaseMortal victim, PlayerNetBaseMortal assassin, int nMode )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;

        //Klasa bazowa wyswietli odpowiednie komunikaty        
        super.EventPlayerDied( victim, assassin, nMode );
        
        if ( ! GetGameSettingsSMD().GetAllowDominateByDeadPlayer() )
            EventPlayerLostBaseIfBaseOwner( victim );
    }
    
    void EventPlayerLeft(PlayerNet player)
    {
        super.EventPlayerLeft( player );
        
        EventPlayerLostBaseIfBaseOwner( player );
    }
    
    
    void EventPlayerLostBaseIfBaseOwner(PlayerNetBaseMortal player)
    {
        NetPlayerInfo cPlayerInfo   = null;
        int           nPlayerInfoId = -1;
        if ( player != null )
        {
            cPlayerInfo = player.GetPlayerInfo();
            if (cPlayerInfo != null)
            {
                nPlayerInfoId = cPlayerInfo.GetNetId();
            }
            else
            {
                CrashLog("ERROR: "+this+" PlayerDied "+player+" cNetPlayerInfo == null\n");
                return;
            }
        }
        else
        {
            CrashLog("ERROR: "+this+" PlayerDied "+player+"\n");
            return;
        }
        
        int nBases = 0 ;
        //przegladnac wszystkie bazy.
        for ( int i=0; i<m_aDominationBases.size(); i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt(i);
            
            if ( cBase == null || cBase.GetPlayerOwner() != cPlayerInfo )
                continue;
                
            //zmienic wlasciciela bazy
            SetBaseOwner( cBase, (NetPlayerInfo)null );
            //zmienic kolor bazy
            cBase.UpdateState();
            //i zliczyc
            nBases++;
        }   
            
        if ( nBases > 0 )
        {
            //wyswietlic komunikaty
            //TODO: ktora baza? a moze grupowy komunikat
            if ( ! MessagesDisabled() )
            {
                AddIncomingMessage( NetIncomingMsg._SMD_BASE_LOST, 
                                    nPlayerInfoId );
                MsgSendClientBaseLost( nPlayerInfoId );
            }
            
            //policz na nowe warunki zapunktowania
            UpdateTimeToNextDominationScore();
        }
    }

    boolean HasBaseOwner(DominationBase cBase)
    { 
        return cBase != null && cBase.GetPlayerOwner() != null; 
    }
    
    void ServerUpdateGame()
    {
        super.ServerUpdateGame();
        
        if (Role < ERole._AUTHORITY) 
			return;
        
        if ( GetTimeToNextDominationScore()  > -1.0       && 
             GetTimeToNextDominationScore() <= GetGameTime() )
        {
            //policz punkty
            ServerScore();
            
            //policz na nowe warunki zapunktowania
            UpdateTimeToNextDominationScore();
        }
    }
    
    void ServerScore()
    {
        float fGameTime = GetGameTime();
        //w przypadku gdy gramy w trybie total, bedziemy zliczali
        //punkty tylko dla jednej (pierwszej) bazy
        boolean bScored = false;
            
        for (int i=0; i<m_aDominationBases.size() ; i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt( i );
                
            if ( cBase                       == null ||
                 cBase.GetPlayerOwner()      == null ||
                 cBase.GetTimeToNextScore()  <= -1.0f )
                continue;
            
            while ( cBase.GetTimeToNextScore() <= fGameTime )
            {
                /*LogD(" "+i+" "+
                     cBase.GetTimeToNextScore()+" "+fGameTime+" "+
                     bScored+" "+cBase.GetPlayerOwner()+
                     "\n");*/
                       
                if ( !IsModeDominationTotal() || !bScored )
                {
                    cBase.GetPlayerOwner().IncPoints();
                }
                
                cBase.AddTimeToNextScore( GetDominationScoreUpdateTime() );
            }
            
            bScored = true;
        }
    }
    
    void ServerStopGame()
    {
        super.ServerStopGame();
                
        for ( int i=0; i<m_aDominationBases.size(); i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt(i);
            
            //kasujemy wlasciciela
            SetBaseOwner( cBase, (NetPlayerInfo)null );
            
            //zmienic kolor bazy
            cBase.UpdateState();
        }
        
        //policz na nowe warunki zapunktowania
        UpdateTimeToNextDominationScore();
    }        
    
        
    
    boolean HasAllBasesTheSameOwner()
    {
        //przegladamy wszystkie bazy, czy maja tego samego wlasciciela
        NetPlayerInfo cPlayerOwner = null;
        int           nCntBases  = 0;
        for ( int i=0; i<m_aDominationBases.size(); i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt( i ) ;
                
            //jesli baza nie ma wlasciciela, to ja omijamy
            if ( cBase == null || cBase.GetPlayerOwner() == null )
                continue;
            
            //pierwszego znalezionego zapamietujemy jako "wzor"
            if ( cPlayerOwner == null )
                 cPlayerOwner = cBase.GetPlayerOwner();
            
            //zliczamy tych co maja "wzrocowego" wlasciciela
            if ( cPlayerOwner == cBase.GetPlayerOwner() )
                nCntBases ++;
        }
        return nCntBases == m_aDominationBases.size();
    }
       
    
    int GetDominationBaseLocalState(DominationBase cBase, NetPlayerInfo cPlayerInfo)
    {
        //jak gracz jeszcze sie nie stworzyl to tez da sie ustalic stan
        if ( cBase != null /*&& cPlayerInfo != null*/ )
        {            
            if ( cBase.GetPlayerOwner() == null )
                return DominationBase._STATE_FREE;
            else
            {
                if ( cBase.GetPlayerOwner() == cPlayerInfo )
                    return DominationBase._STATE_OWNER;
                else
                    return DominationBase._STATE_ENEMY;
            }
        }
        
        return DominationBase._STATE_UNKNOWN;
    }
    
    void LoadUsingMaterials()
    {
        super.LoadUsingMaterials();
        LoadDominationUsingMaterials();
    }
    
    void OnGetActiveTrigger()
    {
        //gracz szuka aktywnych triggerow, umozliwmy
        //mu odnalezienie obiektw DominationBase
        StoreFlagsAndEnableGetActiveTriggerForDominationObjs();
    }
    
    void OnPostGetActiveTrigger()
    {
        //gracz skonczyl szukac aktywnych triggerow, 
        //przywrocmy obiektom DominationBase zapamietane wczesniej flagi
        RestoreFlagsForDominationObjs();
    }
}
