public class NetTargetInfo
{
    //nick gracza
    private 
    String  m_sNick     = "";
    String  GetNick(){ return m_sNick; }
    
    //zdrowie w procentach
    private 
    float   m_nHealth   = 100;
    float   GetHealth(){ return m_nHealth; }
    
    private 
    int     m_nTeamId   = NetTeamInfo.NONE;
    int     GetTeamId(){ return m_nTeamId; }

    private
    String[] m_aPassengers = null;        
    
    final static Vector _COLOR_WHITE    = new Vector(1.0f,1.0f,1.0f);
    final static Vector _COLOR_RED      = new Vector(1.0f,0.6f,0.6f);
    final static Vector _COLOR_BLUE     = new Vector(0.6f,0.6f,1.0f);
    
    //kolor napisu
    private Vector  m_vColor    = new Vector( _COLOR_WHITE );
    
    NetTargetInfo(NetTargetInfo cInfo)
    {
        Set( cInfo.GetNick(), cInfo.GetHealth(), cInfo.GetTeamId() );
    }
    
    NetTargetInfo(String sNick, float fHealth)
    {
        Set( sNick, fHealth, NetTeamInfo.NONE );
    }
    
    void Set(String sNick, float fHealth,int nTeamId)
    {
        m_sNick   = sNick;
        m_nHealth = (int)fHealth;   
        if ( m_nHealth < 0 )
             m_nHealth = 0;
        else if ( m_nHealth > 100 )
                  m_nHealth = 100;
        m_nTeamId = nTeamId;
        SetColor();
        m_aPassengers = null;
    }
    
    void Set(String[] aPassengers,int nTeamId)
    {
        m_aPassengers   = aPassengers;
        m_nTeamId       = nTeamId;
        SetColor();
        m_sNick         = null;
    }
    
    
    void SetColor()
    {
        switch( m_nTeamId )
        {
        case NetTeamInfo.BLUE:
            m_vColor    = new Vector( _COLOR_BLUE );
            break;
        case NetTeamInfo.RED:
            m_vColor    = new Vector( _COLOR_RED );
            break;          
        default:
            m_vColor    = new Vector( _COLOR_WHITE );        
        }
    }
    
    
    
    Vector GetColor() { return m_vColor; }
    
    boolean Valid() 
    { 
        return (NetGameTools.IsStringValid(m_sNick)
               ||
               ((m_aPassengers != null) && (m_aPassengers.length>0) &&
                (NetGameTools.IsStringValid(m_aPassengers[0]))));
    }
    
    boolean Equals(NetTargetInfo cInfo)
    {
        if (cInfo == null)
            return false;
        
        if ( (cInfo.m_sNick != null) && (m_sNick!=null) )
        {
            return (
                    (cInfo.m_sNick.compareTo(m_sNick) == 0) &&
                    (cInfo.m_nHealth == m_nHealth) &&
                    (cInfo.m_nTeamId == m_nTeamId)
                    );
        }
        else
        if ( (cInfo.m_aPassengers != null) && (m_aPassengers != null) )
        {
            if ( cInfo.m_aPassengers.length != m_aPassengers.length )
                return false;
            for ( int i=0; i<m_aPassengers.length; ++i)
            {
                //jesli oba sa null to jedziemy dalej
                if ( (m_aPassengers[ i ] == null) &&
                     (cInfo.m_aPassengers[ i ] == null) )
                    continue;
                
                //jesli tylko jeden z nich to mamy roznice
                if ( (m_aPassengers[ i ] != null) ||
                     (cInfo.m_aPassengers[ i ] != null) )
                    return false;
                
                if ( ! m_aPassengers[ i ].equals(cInfo.m_aPassengers[ i ]) )
                    return false;
            }
            return true;
        }
        return false;
    }
    
    final static int _LINES = 6;
    String []m_aTexts = new String[ _LINES ];
    
    private float m_fTextLength = 0.0f;
    float GetTextLength(){ return m_fTextLength; }
    
    String GetText(Sprite cSprite)
    {
        int nLine = 0;
        if ( m_aPassengers != null )
        {
            for ( ; (nLine < m_aPassengers.length)&&(nLine < m_aTexts.length); ++nLine )
            {
                if (m_aPassengers[nLine] != null)
                    m_aTexts[ nLine ] = m_aPassengers[ nLine ];
                else
                    m_aTexts[ nLine ] = "";
                    
            }
        }
        else
        {
            m_aTexts[ nLine++ ]= m_sNick;
            //m_aTexts[ nLine++ ]= "("+m_nHealth+"%)";
        }
        
        for ( ; nLine < m_aTexts.length; ++nLine )
        {
            m_aTexts[ nLine ] = "";
        }
        
        float fMaxLength = 0.0f;
        for (int i=0;i<m_aTexts.length; i++) 
        {
            Vector vExt = new Vector();
            
            if (m_aTexts[i] != null)
                vExt = cSprite.GetTextExtent( m_aTexts[i] );
            
            if ( fMaxLength < vExt.fX )
                 fMaxLength = vExt.fX;
        }
        
        //TODO: zmienic albo zoptymalizowac to dodawanie spacji
        for (int i=0;i<m_aTexts.length; i++) 
        {
            if ( m_aTexts[i].compareTo("") == 0 )
                continue;
            for (int nSafety=0; nSafety<50 ; nSafety++ )
            {
                Vector vExt = new Vector();
                if (m_aTexts[i] != null)
                    vExt = cSprite.GetTextExtent( m_aTexts[i] );
                if ( vExt.fX >= fMaxLength )
                    break;
                m_aTexts[i] = " "+m_aTexts[i]+" ";
            }
        }
        
        m_fTextLength = fMaxLength;
        String sRet = null;
        for ( nLine = 0; nLine < m_aTexts.length; ++nLine )
        {
            if ( m_aTexts[ nLine ].compareTo("") == 0 )
                break;
            if ( sRet == null )
                sRet = "";
            else 
                sRet = sRet + "\n";
            sRet = sRet + m_aTexts[ nLine ];
        }
        return sRet;
    }
        
}
