import java.io.*;
public class NetServerInitInfo extends NetServerInitInfoBase
{
    static final String sm_sDefaultScriptFileName = "Data/ServerSettingsDefault.scr";

    static final int sm_nDefaultGamePort    = 27015;//CS//2176;
    static final int sm_nGamePortMin        = 2000;
    static final int sm_nGamePortMax        = 65000;
    static final int sm_nRemoteAccessShift  = 8;
    
    //nazwa pliku w ktorym znajduja sie ustawienia serwera (dla danego profilu)
    private String  m_sScriptFileName       = null;
    
    // adres serwera, moze byc ip lub hostname
    private String  m_sServerAddr           = "localhost";
    
    // port na ktorym dziala serwer
    private int     m_nGamePort             = sm_nDefaultGamePort;
    
    //haslo, jesli brak to == null
    private String  m_sPassword             = null;
        
    
    void SetServerAddr(String sAddr){ m_sServerAddr = sAddr; }
    String GetServerAddr(){ return m_sServerAddr; }
    
    void SetGamePort(int nPort){ m_nGamePort = nPort; }
    int GetGamePort(){ return m_nGamePort; }

    void SetPassword(String sPassword){ m_sPassword = sPassword; SetPassword(m_sPassword!=null); }
    String GetPassword(){ return m_sPassword; }
    boolean IsPassword(){ return m_sPassword!=null; }
    
    private
    boolean m_bInternetGame = false;
    void    SetInternetGame( boolean bVal ) { m_bInternetGame = bVal; }
    boolean IsInternetGame() { return m_bInternetGame; }
    
    NetServerInitInfo()
    {
        super();
        InitScriptFileName();
    }
    NetServerInitInfo(NetServerInitInfo cInfo)
    {
        super( cInfo );
        InitScriptFileName();
    }

    void InitScriptFileName()
    {
        m_sScriptFileName = NetPlayerProfile.GetCurrentServerSettingsFile();
    }
    
    void ParseDefaultScriptFile()
    {
        ParseScriptFile( sm_sDefaultScriptFileName );
    }
    
    void ParseScriptFile()
    {
        if ( ! ParseScriptFile( m_sScriptFileName ) )
        {
            ParseScriptFile( sm_sDefaultScriptFileName );
            SaveSettings();
        }
    }
    
    boolean ParseScriptFile(String sFile)
    {
        boolean bReturn = true;
        File cFile = new File( sFile );
        if (!cFile.exists())
        {
            GameObject.LogD("File " + sFile + " not exist !\n");
            bReturn = false;
        }
        else
        {
            //te informacje nie zawsze beda w pliku (tylko dla gry internetowej)
            //zatem przeforsujmy tu domyslne wartosci
            m_nGamePort = sm_nDefaultGamePort;
            //odczytany game port tu zapiszemy
            //i ew. ustawimy jak okaze sie ze mamy gre sieciowa
            //gra po LANie zawsze powinna byc na domyslnym porcie
            int nReadGamePort = m_nGamePort;
    
            ParseKeyword cRoot = GameObject.ParseFile( sFile );
            
            if (cRoot != null)
            {
                ParseKeyword cKey, cSubKey;
                for (cKey=cRoot; cKey!=null; cKey=cKey.GetNext())
                {
                    if ( cKey.Equals( "ServerName" ) )
                        SetServerName( new String( cKey.GetSParam(0) ) );
                    if ( cKey.Equals( "MaxPlayer" ) ) 
                        SetMaxPlayers( cKey.GetIParam(0) );
                    if ( cKey.Equals( "LoopLevels" ) ) 
                        SetLoopLevels( true );
                    if ( cKey.Equals( "Dedicated" ) ) 
                        SetDedicated( true );
                    if ( cKey.Equals( "Password" ) ) 
                        SetPassword( new String(cKey.GetSParam(0)) );
                    if ( cKey.Equals( "InternetGame" ) ) 
                        SetInternetGame( true );
                    if ( cKey.Equals( "GamePort" ) ) 
                        nReadGamePort = cKey.GetIParam(0);
                }
                
                if ( IsInternetGame() )
                    SetGamePort( nReadGamePort );
                
                //jezeli w linii komend jest wymusznie serwera
                //dedykowanego to to trzeba to ustawic
                //TODO: zastanowic sie czy to tak ma byc, zeby
                //po powtornym odpaleniu aplikcaji nie zmienilo
                //sie ustawienie w profilu
                if ( Game.cSingleton != null &&
                     Game.cSingleton.IsDedicatedServer() )
                {
                     SetDedicated( true );
                }
                
                cRoot.delete();
            }
            else
            {
                GameObject.LogD("Error while read/parse " + sFile + " file !\n");
                bReturn = false;
            }
        }
        return bReturn;
    }
    
    ScriptWriter m_cScriptWriter = new ScriptWriter();
    
    void SaveSettings()
    {
        m_cScriptWriter.Open( m_sScriptFileName );
                        
        m_cScriptWriter.Write("!ServerName(s)");
        m_cScriptWriter.Write("!MaxPlayer(i)");
        m_cScriptWriter.Write("!LoopLevels()");
        m_cScriptWriter.Write("!Dedicated()");
        m_cScriptWriter.Write("!Password(s)");
        m_cScriptWriter.Write("!InternetGame()");
        m_cScriptWriter.Write("!GamePort(i)");
        //TODO sprawdzic, czy nie mozna tego wyrzucic
        //m_cScriptWriter.Write("!CommPort(i)");
        
        m_cScriptWriter.NewLine();
            
        m_cScriptWriter.Write("ServerName(\"" + GetServerName() + "\")");
        m_cScriptWriter.Write("MaxPlayer(" + String.valueOf( GetMaxPlayers() ) + ")");
            
        if ( GetLoopLevels() )
            m_cScriptWriter.Write("LoopLevels()");
        
        if ( GetDedicated() )
            m_cScriptWriter.Write("Dedicated()");
        
        if ( IsPassword() )
            m_cScriptWriter.Write("Password(\"" + GetPassword() + "\")");
        
        if ( IsInternetGame() )
        {
            m_cScriptWriter.Write("InternetGame()");
            m_cScriptWriter.Write("GamePort(" + GetGamePort() + ")");
            //m_cScriptWriter.Write("CommPort(" + GetCommPort() + ")");
        }

        m_cScriptWriter.Close();
    }
    
    /**
     * Metoda statyczna weryfikujaca poprawnosc
     * portow (podczas ich proby zmiany w menu)
     */
    static boolean VerifyPorts(int nGamePort)
    {        
        return ((nGamePort >= sm_nGamePortMin)&&
                (nGamePort <= sm_nGamePortMax));
    }
               
    
}
