/** manager HUDComponentow w grze sieciowej
 */
 
public class HUDNetManager extends GameObject
{   
    /** NetGameInfo aktualnej gry...
     */
    NetGameInfoHud m_cNetGameInfo = null;

    final static byte _STATUS_UNKNOWN = 0x00;
    final static byte _STATUS_STARTED = 0x01;
    final static byte _STATUS_STOPPED = 0x02;
    
    /**status hudow
     */
    byte m_byStatus = _STATUS_UNKNOWN;
        
    /**Czy mozna pokazac nadchodzace wiadomosci, przestawiane na false gdy pokazujemy
     * okno logow*/
    static boolean s_bCanActivateIncommingMsg = true;
    
    /** tablica przechowujaca skladowe elementy HUD'a
     */
    HUDComponentNet m_aHUDComponentNet[] = null;
    
    /** poszczegolne komponenty huda */
    final static byte   _STATISTICS           = 0,
                        _INCOMING_MESSAGE     = 1,
                        _OUTGOING_MESSAGE     = 2,
                        _SYSTEM_MESSAGE       = 3,
                        _GAME_STATUS          = 4,
                        _VOICE_MENU           = 5,
                        _CONNECTION_STATUS    = 6,
                        _ADMIN_MESSAGE        = 7,
                        /** rozmiar tablicy z komponentami HUD'a */
                        _COMPONENT_CNT        = 8;
    
   

    public void OnCreate()
    {
        m_aHUDComponentNet = new HUDComponentNet[ _COMPONENT_CNT ];
    }
    
    public void OnDestroy()
    {
        if ( m_aHUDComponentNet != null )
            for (int i=0; i<m_aHUDComponentNet.length; i++)
            {
                if(m_aHUDComponentNet[i]!=null)
                {
                    m_aHUDComponentNet[i].OnDestroy();
                    m_aHUDComponentNet[i] = null;
                }
            }
    }
    
    //----------------------------------------------------------------------
    /** metoda inicjalizujaca HUDNet */
    public void Init(NetGameInfoHud cGameInfo)
    {
        // zapamietajmy sobie gameinfo 
        m_cNetGameInfo = cGameInfo;
        
        // jaki mamy tryb gry?
        int    nGameType  = m_cNetGameInfo.GetGameType();
        // na podstawie tego ustalimy klase huda do statystyk
        String sClassName = ENetGameType.GetHudNetClassName( nGameType );
        // i jedziemy z koksem...
        LogD("Creating hud statistics "+sClassName+"\n");
        m_aHUDComponentNet[_STATISTICS      ] = (HUDComponentNet)CreateObject( sClassName );
        m_aHUDComponentNet[_STATISTICS      ].Init( this, m_cNetGameInfo );
        // teraz jeszcze hudy, ktore sa w grze na stale
        m_aHUDComponentNet[_INCOMING_MESSAGE] = (HUDComponentNet)CreateObjectNoInit("HUDNetIncomingMessage");
        m_aHUDComponentNet[_INCOMING_MESSAGE].Init( this, m_cNetGameInfo );
        m_aHUDComponentNet[_INCOMING_MESSAGE].InitObject();
        
        m_aHUDComponentNet[_OUTGOING_MESSAGE] = (HUDComponentNet)CreateObjectNoInit("HUDNetOutgoingMessage");
        ((HUDNetOutgoingMessage)m_aHUDComponentNet[_OUTGOING_MESSAGE]).m_cHudIncomingMsg = ((HUDNetIncomingMessage)m_aHUDComponentNet[_INCOMING_MESSAGE]);
        m_aHUDComponentNet[_OUTGOING_MESSAGE].InitObject();
        m_aHUDComponentNet[_OUTGOING_MESSAGE].Init( this, m_cNetGameInfo );
        
        m_aHUDComponentNet[_SYSTEM_MESSAGE]   = (HUDComponentNet)CreateObjectNoInit("HUDNetSystemMessage");
        ((HUDNetSystemMessage)m_aHUDComponentNet[_SYSTEM_MESSAGE]).m_cHudIncomingMsg = ((HUDNetIncomingMessage)m_aHUDComponentNet[_INCOMING_MESSAGE]);        
        m_aHUDComponentNet[_SYSTEM_MESSAGE].InitObject();
        m_aHUDComponentNet[_SYSTEM_MESSAGE].Init( this, m_cNetGameInfo );
        
        m_aHUDComponentNet[_ADMIN_MESSAGE] = (HUDComponentNet)CreateObjectNoInit("HUDNetAdminMessage");
        m_aHUDComponentNet[_ADMIN_MESSAGE].Init( this, m_cNetGameInfo );
        m_aHUDComponentNet[_ADMIN_MESSAGE].InitObject();
        
        m_aHUDComponentNet[_GAME_STATUS]      = (HUDComponentNet)CreateObject("HUDNetGameStatus");
        m_aHUDComponentNet[_GAME_STATUS].Init( this, m_cNetGameInfo );
        
        m_aHUDComponentNet[_CONNECTION_STATUS]= (HUDComponentNet)CreateObject("HUDNetConnectionStatus");
        m_aHUDComponentNet[_CONNECTION_STATUS].Init( this, m_cNetGameInfo );
        Activate(_CONNECTION_STATUS);
        
        //bedziemy przechwytywac klawisze
        RegisterKeyRequest();
        //aktywujmy na module
        ActivateOnModule();
        // i co jakisz czas probujmy go odswiezyc...
        CallMethodEvery("CheckRefreshHuds",1.0f);
    }
    
    void Start()
    {
        if ( m_byStatus != _STATUS_STARTED )
            SetStatus( _STATUS_STARTED );
    }
    
    void Stop()
    {
        if ( m_byStatus != _STATUS_STOPPED )
            SetStatus( _STATUS_STOPPED );
    }
    
    void SetStatus(byte byStatus)
    {
        m_byStatus = byStatus;
        m_aHUDComponentNet[_STATISTICS      ].SetStatus( byStatus );
        m_aHUDComponentNet[_INCOMING_MESSAGE].SetStatus( byStatus );
        m_aHUDComponentNet[_OUTGOING_MESSAGE].SetStatus( byStatus );
        m_aHUDComponentNet[_SYSTEM_MESSAGE  ].SetStatus( byStatus );
        m_aHUDComponentNet[_GAME_STATUS     ].SetStatus( byStatus );
        m_aHUDComponentNet[_CONNECTION_STATUS].SetStatus( byStatus );
        m_aHUDComponentNet[_ADMIN_MESSAGE].SetStatus( byStatus );
    }
    
    // odwieza dane (jezeli sa aktualnie wyswietlane) z podanego komponentu
    // nalezy uwazac z odswiezaniem massageincoming, poniewaz
    // na razie jest to aktywowane na pewien czas i moze sie
    // cos pochrzanic
	void Refresh(int nComp)
	{
        if (m_aHUDComponentNet        == null ||
            m_aHUDComponentNet.length < nComp )
        {
            //todo: error
            return;
        }
             
		if ( m_aHUDComponentNet[ nComp ] == null )
        {
            //todo: error
            return;
        }
            
	    m_aHUDComponentNet[ nComp ].Refresh();
	}
    
    // odswieza wszystkie komponenty
    // uwaga podobnie jak powyzej
    void Refresh()
	{
		for ( int i=0; i<m_aHUDComponentNet.length; i++ )
            Refresh( i );
	}
    
    boolean IsActive(int nComp)
    {   
        if (m_aHUDComponentNet        == null ||
            m_aHUDComponentNet.length < nComp )
        {
            //todo: error
            return false;
        }
        
        if (m_aHUDComponentNet[ nComp ]==null)
        {
            CrashLog("ERROR: "+this+" IsActive "+nComp+" m_aHUDComponentNet[]==null\n");
            return false;
        }
        
        return m_aHUDComponentNet[ nComp ].bActiveFlag;
    }
    
    
    public void Activate(int nComp)
    {
        if (m_aHUDComponentNet        == null ||
            m_aHUDComponentNet.length < nComp )
        {
            //todo: error
            return;
        }
        
        if (m_aHUDComponentNet[nComp]!=null)
        {
            m_aHUDComponentNet[nComp].Activate();
            RefreshIfLocked();
        }
    }

    //----------------------------------------------------------------------
    public void Deactivate(int nComp)
    {
        if (m_aHUDComponentNet        == null ||
            m_aHUDComponentNet.length < nComp )
        {
            //todo: error
            return;
        }
        
        if (m_aHUDComponentNet[nComp]!=null)
        {
            m_aHUDComponentNet[nComp].Deactivate();
        }
    }
    
    public void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
    {
        if (IsActiveVoiceMenu() && bKeyState)
                return;
        
        if ( nKeyCode==EKey._TAB )
        {
            
            if (IsActiveStatistics() == bKeyState)
                return;
            
            if ( bKeyState )
                ActivateStatistics("");
            else
                DeactivateStatistics();
        }
        if ( nKeyCode == EGameControl.SHOW_ADMIN_MESSAGES)
        {
            if (IsActiveAdminMessage() == bKeyState)
                return;
            
            if ( bKeyState )
                Activate(_ADMIN_MESSAGE);
            else
                Deactivate(_ADMIN_MESSAGE);
        }
        
            
        /*if ( nKeyCode==EKey._X && bKeyState )
            ShowHistoryIncomingMessage();*/
    }
    
    boolean IsActiveVoiceMenu()
    {
        if (((ChromeModule)GetModule()).GetMainPlayer() != null)
        {
            HUDManager cHud = ((PlayerHUD)((ChromeModule)GetModule()).GetMainPlayer()).HUD;
            return cHud.GetHUDVoiceMenu() != null && cHud.GetHUDVoiceMenu().bActiveFlag;
        }
        return false;
    }
    
    /////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////
    // Statystyki
    /////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////

        
    boolean IsActiveStatistics()
    {
        return IsActive(_STATISTICS);
    }
    
    boolean IsActiveAdminMessage()
    {
        return IsActive(_ADMIN_MESSAGE);
    }
    
    boolean m_bStatisticsLocked = false;
    void ActivateStatistics(String sText)
    {
        ActivateStatistics(sText, false);
    }
    void ActivateStatistics(String sText, boolean bLock)
    {
        if (m_aHUDComponentNet[_STATISTICS]==null)
            return;
        
        SetTextStatistics( sText );
        m_bStatisticsLocked  = bLock;
        ActivateStatistics();
    }
    
    void ActivateStatistics()
    {
        if (m_aHUDComponentNet[_STATISTICS]==null)
            return;
        if (bTest)
            Thread.dumpStack();
        m_aHUDComponentNet[_STATISTICS].Activate();
        RefreshIfLocked();
    }

    boolean bTest = false;
    void DeactivateStatistics()
    {
        DeactivateStatistics(false);
    }
    void DeactivateStatistics(boolean bUnLock)
    {
        if (m_aHUDComponentNet[_STATISTICS]==null)
            return;
        if ( m_bStatisticsLocked && bUnLock == false )
            return;
        m_bStatisticsLocked = false;
        m_aHUDComponentNet[_STATISTICS].Deactivate();
    }
    
    //////////////////////////////////////////////////////////////////////////////
    // Statystiki end
    /////////////////////////////////////////////////////////////////////////////

    
    //////////////////////////////////////////////////////////////////////

    boolean IsActiveIncomingMessage()
    {
        return IsActive(_INCOMING_MESSAGE);
    }
    
    
	
	void CheckRefreshHuds()
    {
        RefreshStatistics();
        CheckTextsTime();
    }
    
    
    void RefreshStatistics()
    {
        Refresh(_STATISTICS);
    }
    
    void CheckTextsTime()
    {
        if (m_aHUDComponentNet[_INCOMING_MESSAGE]==null)
            return;
        
        m_aHUDComponentNet[_INCOMING_MESSAGE].CheckTextsTime();
        
        m_aHUDComponentNet[_ADMIN_MESSAGE].CheckTextsTime();
    }
    
    void SetGameRule(String sText)
    {
        m_aHUDComponentNet[_STATISTICS].SetRule( sText );
    }
    
    void AddIncomingMessage( String sText, Vector vColor )
    {
        SetTextIncoming( sText, vColor );
        Activate(_INCOMING_MESSAGE);
    }
    
    void AddAdminMessage( String sText, Vector vColor )
    {
        sText = ConsoleCommand.GetAdminPrefix()+" "+sText;
        SetTextAdmin( sText, vColor );
        Activate(_ADMIN_MESSAGE);
        
        //dodatkowo tez do okienka historii logow
        if ( (m_cNetGameInfo                        != null) && 
             (m_cNetGameInfo.GetMyPlayerNet()       != null) && 
             (m_cNetGameInfo.GetMyPlayerNet().HUD   != null) )
        {
            m_cNetGameInfo.GetMyPlayerNet().HUD.AddLogsText( sText, vColor );
        }
    }
    
    void AddAdminMessage(String sText)
    {
        AddAdminMessage( sText, NetMsg._COL_INMSG_ADMIN );
    }
    
    void ShowHistoryIncomingMessage()
    {
        if (m_aHUDComponentNet[_INCOMING_MESSAGE]==null)
            return;
        m_aHUDComponentNet[_INCOMING_MESSAGE].ShowHistory();
        Activate(_INCOMING_MESSAGE);
    }
    
    void AddSystemMessage( String sText )
    {
        if ( sText != null )
        {
            SetTextSystem( sText );
            Activate(_SYSTEM_MESSAGE);
        }
        else
            Deactivate(_SYSTEM_MESSAGE);
    }
    
    void SetTextStatistics( String sText )
    {
        if (m_aHUDComponentNet[_STATISTICS]==null)
            return;
        
        m_aHUDComponentNet[_STATISTICS].SetText( sText );
    }
    
    void SetTextIncoming( String sText, Vector vColor )
    {
        if (m_aHUDComponentNet[_INCOMING_MESSAGE]==null)
            return;
        m_aHUDComponentNet[_INCOMING_MESSAGE].SetText( sText, vColor );
    }
    
    void SetTextAdmin( String sText, Vector vColor )
    {
        NetLog.CommandAnswer(sText);
        if (m_aHUDComponentNet[_ADMIN_MESSAGE]==null)
            return;
        m_aHUDComponentNet[_ADMIN_MESSAGE].SetText( sText, vColor );
    }
    
    void SetTextSystem( String sText )
    {
        if (m_aHUDComponentNet[_SYSTEM_MESSAGE]==null)
            return;
        m_aHUDComponentNet[_SYSTEM_MESSAGE].SetText( sText );
    }
    
    void SetTextGameInfo( String sText, Vector vColor )
    {
        if (m_aHUDComponentNet[ _GAME_STATUS ]==null)
            return;
        m_aHUDComponentNet[ _GAME_STATUS ].SetInfoText( sText, vColor );
        Activate(_GAME_STATUS);
    }
    
    void SetTextGameScore( String sText, Vector vColor )
    {
        if (m_aHUDComponentNet[ _GAME_STATUS ]==null)
            return;
        m_aHUDComponentNet[ _GAME_STATUS ].SetScoreText( sText, vColor );
        Activate(_GAME_STATUS);
    }

    void SetConnectionStatus( byte byStatus )
    {
        if (m_aHUDComponentNet[_CONNECTION_STATUS]==null)
            return;
        m_aHUDComponentNet[_CONNECTION_STATUS].SetConnectionStatus( byStatus );
    }
    
//----------------------------------------------------------------------    
    
    //kolory do wyswietlania statystyk
    static Vector vColorPlayer          = new Vector( 1.0f, 1.0f, 1.0f );
    static Vector vColorMainPlayer      = new Vector( 1.0f, 1.0f, 0.0f );
    static Vector vColorExcludedPlayer  = new Vector( 1.0f, 0.0f, 0.0f );
        
    /** Procka zwraca referencje do jednego ze staticow HUDNetManagera
     * okreslajacych kolor do wyswietlania takestu
     * TODO: przerobic sposob rozpoznawania ownera
     */
    static Vector GetColorForPlayerInfo( NetPlayerInfo cPlayerInfo )
    {
        if ( cPlayerInfo != null )
        {
            if ( cPlayerInfo.IsExcluded() )
                return vColorExcludedPlayer;
        
            if ( cPlayerInfo.bNetIsOwner )
                return vColorMainPlayer;    
        }
        return vColorPlayer;
    }

    static Vector GetColorForTeamInfo(NetTeamInfo cTeamInfo)
    {
        if (cTeamInfo != null)
        {
             return cTeamInfo.GetTeamId() == NetTeamInfo.BLUE ?
                    new Vector(0.5f, 0.5f, 1f):
                    new Vector(1f, 0.5f, 0.5f);
        }
        return new Vector(1,1,1);
    }
    
    private boolean m_bLocked = false;
    boolean IsLocked(){ return m_bLocked; }
    
    void Lock(boolean bLock)
    {
        if ( m_bLocked == bLock )
            return;
        
        m_bLocked = bLock;
        Refresh();
    }
    
    void RefreshIfLocked()
    {
        if ( m_bLocked )
            Refresh();
    }
    
}
