public class HUDMapTeamDomination extends HUDMapTeam
{
    NetGameInfoCommon m_cGameInfoCommon = null;
    
    HudNetMapSpriteDomination m_aBases[] = null;
    
    int METHOD_ComputeBasesPos = 0;
    
    public void OnCreate()
    {
        METHOD_ComputeBasesPos = RegisterCallMethod( "ComputeBasesPos" );
        
        super.OnCreate();   
        
        if (m_bDisableInNetGame)
            return;
                
        m_aBases = new HudNetMapSpriteDomination[ NetGameInfoCommon.sm_nMaxDominationBases ];
        
        for ( int i=0; i<m_aBases.length; i++ )
        {
            HudNetMapSpriteDomination aBase = (HudNetMapSpriteDomination)CreateObject("HudNetMapSpriteDomination");
            
            m_cMap.AddChild( aBase );
            m_aBases[i] = aBase;
        }
        
        
    }
    
    void StartComputeObjectsPos()
    {
        super.StartComputeObjectsPos();
        
        DisableCallMethod( METHOD_ComputeBasesPos );
        CallMethodEvery( METHOD_ComputeBasesPos, 0.05f);
    }
    
    void SetCompassRef(HUDCompass cCompass)
    {
        if (m_bDisableInNetGame)
            return;
        
        super.SetCompassRef(cCompass);
        
        for ( int i=0; i<m_aBases.length; i++ )
        {
            m_aBases[i].m_cCompassElement = ((HUDCompassNet)cHUDCompasRef).AddWaypointNet(new Vector(), true, m_aBases[i]);
        }
        DrawDoUnderCursor();
    }
    
    public void OnDestroy()
    {
        if (m_bDisableInNetGame)
            return;
        
        if ( m_aBases != null )
        {
            for ( int i=0; i<m_aBases.length; i++ )
            {
                if ( m_aBases[i] != null )
                {
                    RemoveSprite( m_aBases[i] );
                    m_aBases[i].delete();
                    m_aBases[i] = null;
                }
            }
            m_aBases = null;
        }
        
        super.OnDestroy();
    }
    
    void SetNetGameInfo(NetGameInfo cGameInfo)
    {
        super.SetNetGameInfo( cGameInfo );
        
        if ( cGameInfo instanceof NetGameInfoCommon )
            m_cGameInfoCommon = (NetGameInfoCommon) cGameInfo;
    }
    
    public void Activate()
    {
		if (bActiveFlag)
			return;
		if (!cHUDManager.m_bActivateAllowed)
			return;
		
        super.Activate();   
		
    }
    
    
    void DrawDoUnderCursor()
    {
        for ( int i=0; i<m_nVisibleBases; i++ )
        {
            HudNetMapSpriteDomination cSpriteBase = m_aBases[i];
            if ( cSpriteBase == null )
                continue;
            if ( cSpriteBase.CanBeVisible() )
            {
                cSpriteBase.MoveTop();
                cSpriteBase.Show();
            }
            else
                cSpriteBase.Hide();
        }
        
        super.DrawDoUnderCursor();
    }
    
    public void HideAll()
	{
        super.HideAll();
        
        HideRemainingBases( 0 );
    }
        
    void ComputeBasesPos()
    {
        int nBaseSpriteIndex = 0;
        
        boolean bSettingsFound = false;
        boolean bBasesVisible  = false;

        //TODO: zoptymalizowac to
        if (m_cGameInfoCommon instanceof NetGameInfoTD)
        {
            NetGameSettingsTD cSettings = ((NetGameInfoTD)m_cGameInfoCommon).GetGameSettingsTD();
            
            if ( cSettings != null && cSettings.IsValid() )
            {
                bSettingsFound = true;
                bBasesVisible  = cSettings.GetBasesVisibleOnHudMap();
            }
        } else if (m_cGameInfoCommon instanceof NetGameInfoSMD)
        {
            NetGameSettingsSMD cSettings = ((NetGameInfoSMD)m_cGameInfoCommon).GetGameSettingsSMD();
            
            if ( cSettings != null && cSettings.IsValid() )
            {
                bSettingsFound = true;
                bBasesVisible  = cSettings.GetBasesVisibleOnHudMap();
            }
        }
                        
        if ( m_cGameInfoCommon == null  || 
             bSettingsFound    == false ||
             bBasesVisible     == false )
        {
            HideRemainingBases( nBaseSpriteIndex );
            if ( bSettingsFound )
                DisableCallMethod( METHOD_ComputeBasesPos );
            return;
        }
        
        for ( int i=0; i<m_cGameInfoCommon.GetDominationBaseCnt() && nBaseSpriteIndex < m_aBases.length; i++ )
        {
            DominationBase cBase = m_cGameInfoCommon.GetDominationBase( i );
            if ( cBase == null )
                continue;
                                    
            HudNetMapSpriteDomination cBaseSprite = m_aBases[ nBaseSpriteIndex ];
            switch ( cBase.GetLocalState() )
            {
            case DominationBase._STATE_FREE:
                cBaseSprite.SetMode( NetHudMapSprite._MODE_FREE );
                break;
            case DominationBase._STATE_BLUE:
                cBaseSprite.SetMode( NetHudMapSprite._MODE_BLUE );
                break;
            case DominationBase._STATE_RED:
                cBaseSprite.SetMode( NetHudMapSprite._MODE_RED );
                break;
            case DominationBase._STATE_OWNER:
                cBaseSprite.SetMode( NetHudMapSprite._MODE_OWNER );
                break;
            case DominationBase._STATE_ENEMY:
                cBaseSprite.SetMode( NetHudMapSprite._MODE_ENEMY );
                break;
            default:
                cBaseSprite.SetMode( NetHudMapSprite._MODE_FREE );
                continue;
            }
            
            ComputeSpritePos( cBaseSprite, cBase.GetPosition() );
            
            cBaseSprite.Show();
            
            nBaseSpriteIndex++;
        }
                
        HideRemainingBases( nBaseSpriteIndex );
    }
    
    int m_nVisibleBases = 0;
    
    void HideRemainingBases( int nVisibleBases )
    {
        m_nVisibleBases = nVisibleBases;
        
        for ( int i=nVisibleBases; i<m_aBases.length; i++ )
        {
            HudNetMapSpriteDomination cBaseSprite = m_aBases[i];
            if ( cBaseSprite == null )
                continue;
            
            cBaseSprite.SetMode( NetHudMapSprite._MODE_UNKNOWN );
            cBaseSprite.Hide();
        }
    }
}

