public class ModuleNetBaseQuery extends ModuleNetBase 
#if REMOTE_ACCESS        
    implements IRemoteGame
#endif    
{
#if REMOTE_ACCESS    
    GameRemoteAccessServer m_cRemoteAccessServer = null;
#endif
    
    int METHOD_HandleRemoteMessages = 0;
    
    void OnCreate()
    {
        METHOD_HandleRemoteMessages = RegisterCallMethod("HandleRemoteMessages");
        
        super.OnCreate();
    }
    
    void InitNetwork()
    {
        super.InitNetwork();
        
        if (Role == ERole._AUTHORITY)
        {
            // initialize ASE
            ASEQuery_Initialize(GetGameInitInfo().GetServerInfo().IsInternetGame());
            
            //TODO: tylko do gier sieciowych 
            
#if REMOTE_ACCESS                
            if ( GetGameInitInfo().GetServerInfo().IsInternetGame() )
            {
                //przejmujemy obiekt do komunikacji z zewnatrz
                m_cRemoteAccessServer = GameRemoteAccessServer.GetInstance();
                if ( m_cRemoteAccessServer == null )
                    m_cRemoteAccessServer = GameRemoteAccessServer.CreateInstance( GetGameInitInfo().GetServerInfo().GetGamePort() + NetServerInitInfo.sm_nRemoteAccessShift );
                if ( m_cRemoteAccessServer != null )
                     m_cRemoteAccessServer.Acquire( (IRemoteGame)this );
                NetLog.Test(" REMOTE ACCESS ACQUIRED on port "+m_cRemoteAccessServer.GetListenerPort()+"\n");
            }
#endif  
        }
    }
    
    public void OnDestroy()
    {
        super.OnDestroy();
        
        // close ASE
        ASEQuery_Shutdown();
        
#if REMOTE_ACCESS            
        //likwidujemy obiekt do komunikacji miedzy aplikacjami
        if ( m_cRemoteAccessServer != null )
        {
            //m_cRemoteAccessServer.Destroy();
            m_cRemoteAccessServer.Acquire( (IRemoteGame)null );
            m_cRemoteAccessServer = null;
        }
#endif        
    }
    
    //////////////////////////////////////////////////////////////////////////
    // ASE queries support
    
    //Status
    
    public void ASEQuery_OnWantStatus()
    {
        String sHostName = GetGameInitInfo().GetServerInfo().GetServerName();
        String sGameType = ENetGameType.GetGameName_NL( GetGameInitInfo().nGameType );
        String sLevelName = GetGameInitInfo().GetCurrentLevelName();
        String sGameVer = NetGameInfo.GetGameVersionNumberString();
        boolean bPassword = GetGameInitInfo().GetServerInfo().IsPassword();
        int nPlayers = m_cPlayerManager.GetPlayersCnt();
        int nMaxPlayers = GetGameInitInfo().GetServerInfo().GetMaxPlayers();
    
        ASEQuery_Status(sHostName, sGameType, sLevelName, sGameVer, bPassword, nPlayers, nMaxPlayers);
    }
    
    public void ASEQuery_Status(String     sHostName, 
                                String     sGameType, 
                                String     sLevelName, 
                                String     sGameVersion, 
                                boolean    bPassword, 
                                int        nPlayers, 
                                int        nMaxPlayers ) 
    {
#if REMOTE_ACCESS            
        if ( m_cRemoteQueryDataStatus != null )
            m_cRemoteQueryDataStatus.Set(sHostName,sGameType,sLevelName,sGameVersion,bPassword,nPlayers,nMaxPlayers);        
        else
#endif        
            super.ASEQuery_Status(sHostName,sGameType,sLevelName,sGameVersion,bPassword,nPlayers,nMaxPlayers);
    }
    
    //Rules
    
    public void ASEQuery_OnWantRules()
    {
        ASEQuery_AddRule( "Dedicated" , 
                          NetGameTools.Bool2YesNo( AmIDedicatedServer() ) );
        
        ASEQuery_AddRule( "Password Required" , 
                          NetGameTools.Bool2YesNo( GetGameInitInfo().GetServerInfo().IsPassword() ) );
        
        GetGameInfo().GetGameSettings().ASEQuery_OnWantRules( this );
    }

    public void ASEQuery_AddRule( String sRuleName, String sRuleValue )
    {
#if REMOTE_ACCESS            
        if ( m_cRemoteQueryDataRules != null )
            m_cRemoteQueryDataRules.Add( new RemoteQueryDataRule( sRuleName, sRuleValue ) );
        else
#endif        
            super.ASEQuery_AddRule( sRuleName, sRuleValue );
    }
    
    //Players

    public void ASEQuery_OnWantPlayers()
    {
        NetPlayerInfoList cList = GetGameInfo().GetPlayerInfoList();
        
        boolean bPoints = GetGameInfo().GetGameSettings().ScorePoints();
        
        for(int i=0; i<cList.GetSize(); ++i)
        {
            NetPlayerInfo cPlayerInfo = cList.GetPlayerByIndex(i);

            int nFrags = bPoints ? cPlayerInfo.GetPoints() : cPlayerInfo.GetFrags();
            
            ASEQuery_AddPlayer(cPlayerInfo.GetNick(), Text.Get( NetTeamInfo.GetName_NL(cPlayerInfo.GetTeamId()) ),
                               /*SKIN*/"", Integer.toString(nFrags), Integer.toString(cPlayerInfo.GetPing()), /*TIME*/"");
        }
    }
    
    public void ASEQuery_AddPlayer(String sName, String sTeam, String sSkin, String sScore, String sPing, String sTime)
    {
#if REMOTE_ACCESS            
        if ( m_cRemoteQueryDataPlayers != null )
            m_cRemoteQueryDataPlayers.Add( new RemoteQueryDataPlayer( sName, sTeam, sSkin, sScore, sPing, sTime ) );
        else
#endif        
            super.ASEQuery_AddPlayer( sName, sTeam, sSkin, sScore, sPing, sTime );
    }
        
    
    
    
    //////////////////////////////////////////////////////////////////

#if REMOTE_ACCESS        
    void HandleRemoteMessages()
    {
        if ( m_cRemoteAccessServer != null )
            m_cRemoteAccessServer.HandleReceivedMessages();
    }
    
    //Implementation of IRemoteGame
    
    public void OnRemoteMessageReceived()
    {
        CallOnFrameEnd( METHOD_HandleRemoteMessages );
    }
    
    RemoteQueryDataStatus m_cRemoteQueryDataStatus = null;
    RemoteQueryDataRules m_cRemoteQueryDataRules = null;
    RemoteQueryDataPlayers m_cRemoteQueryDataPlayers = null;
    
    public void RemoteQuery_OnWantStatus( RemoteQueryDataStatus cData )
    {
        m_cRemoteQueryDataStatus = cData;
        ASEQuery_OnWantStatus();
        m_cRemoteQueryDataStatus = null;
    }
    
    public void RemoteQuery_OnWantRules( RemoteQueryDataRules cData )
    {
        m_cRemoteQueryDataRules = cData;
        ASEQuery_OnWantRules();
        m_cRemoteQueryDataRules = null;
    }
    
    public void RemoteQuery_OnWantPlayers( RemoteQueryDataPlayers cData )
    {
        m_cRemoteQueryDataPlayers = cData;
        ASEQuery_OnWantPlayers();
        m_cRemoteQueryDataPlayers = null;
    }
#endif
    
}
