/** Obiekt zawierajacy zespol kontrolek do edycji 
 * Kontrolki te obslugiwane sa przez ekrany pochodne od MenuNetEditWindow
 * w chwili obecnej wykorzystywane w game settings:
 * - label (nazwa opcji)
 * - check box do uaktywniania / zdeaktywowania
 * - pole edycyjne (tekstowe lub liczbowe) wystepuje opcjonalnie
 * Obiekt nalezy stworzyc przez CreateObjectNoInit,
 * po czym zainicjowac go odpowiednimi parametrami (wymagane SetWindow,
 * reszta opcjonalnie) i wywolac InitObject.
 * Kontrolka przydzielane sa kolejne miejsca na okienku wg kolejnosci
 * wywolywania InitObject (a nie Create...!)
 */
public class MenuNetEditControlsSet extends UIControlObject
{
    /** Etykietka kontrolki*/
    private UIStatic              m_cLabel    = null;
    /** Check box*/
    private MenuNetCheckButton  m_cButton   = null;     
    /** Pole edycyjne */
    private MenuNetEdit         m_cEdit     = null;
    
    /** 
     * Parametry
     */
    
    /** Okienko na ktorym wystepuje kontrolka*/
    private MenuNetEditWindow   m_cWindow   = null;
    void SetWindow(MenuNetEditWindow cWindow) { m_cWindow = cWindow; }
    MenuNetEditWindow  GetWindow(){ return m_cWindow; }
    
    /** Napis na label'u */
    private String m_sLabelText = "";
    void SetLabelText( String sText ) 
	{ 
		m_sLabelText = sText;
		//m_cLabel.SetText(sText); 
	}
    
    /** Czy napis na label'u jest dlugi i ma byc w dwoch liniach */
    //TODO: dorobic automatyczne sprawdzanie dlugosci tekstu!
    private boolean m_bLabelMultiline = false;
    void SetLabelMultiline(){ m_bLabelMultiline = true; }
    
    /** Czy przy edycji wpisujemy same cyfry */
    private boolean m_bNumberOnly        = true;
	
    void SetNumberOnly(boolean bNumberOnly)
	{ 
		m_bNumberOnly = bNumberOnly;
		if(bNumberOnly)
		{
			m_cEdit.m_sAllowedChars = ChromeMenuWindow._DIGITS_ONLY;
		}
		else
		{
			m_cEdit.m_sAllowedChars = null;
		}
	}
    
    /** Pole edycyjne widoczne, albo poslugujemy sie tylko checkboxem */
    private boolean m_bEditVisible = true;
    void SetEditVisible(boolean bVisible){ m_bEditVisible = bVisible; }
    
    /** Czasem nie chcemy w ogole checkboxa a tylko pole edit
     */
    private boolean m_bCheckBoxVisible   = true;
    void    SetCheckBoxVisible(boolean bVisible){ m_bCheckBoxVisible = bVisible; }
    boolean GetCheckBoxVisible(){ return m_bCheckBoxVisible; }
    
    /** Ramka z etykieta moze byc dluga lub krotka */
    private boolean m_bBigSize = false;
    void    SetBigSize(boolean bBigSize){ m_bBigSize = bBigSize; }
    boolean IsBigSize(){ return m_bBigSize; }
    
    
    
    /** Albo edytujemy dane, albo tylko ogladamy 
     * domyslnie pobierane jest to z Window, mozna "recznie" zmienic
     * */
    private boolean m_bEditMode = false;
    void SetEditMode(boolean bEditMode){ m_bEditMode = bEditMode; }
    
    
    /** Jaki tekst bedzie w polu edit, gdy checkbox bedzie wylaczony */
    private String m_sUnavailable  = Text.Get("&MenuNetEditFieldNoLimit&");
    /** Jaka wartosc bedzie zwracala kontrolka, gdy checkbox bedzie wylaczony */
    private int    m_nUnavailable  = -1;
    void SetUnavailableValue(int nUnavailable,String sUnavailable)
    {
        SetUnavailableValue( nUnavailable );
        SetUnavailableValue( sUnavailable );
    }
    void SetUnavailableValue(int nUnavailable)
    {
        m_nUnavailable = nUnavailable;
    }
    void SetUnavailableValue(String sUnavailable)
    {
        m_sUnavailable = sUnavailable;
    }
    
    /** Minimalna wartosc pola edit (jesli w trybie NumberOnly)*/
    private int m_nMinValue     = 0;
    /** Maksymalna wartosc pola edit (jesli w trybie NumberOnly)*/
    private int m_nMaxValue     = 100;
    /** Domyslna wartosc pola edit (jesli w trybie NumberOnly)*/
    private int m_nDefaultValue = 50;
    /** Domyslna wartosc pola edit (jesli w trybie ! NumberOnly)*/
    private String m_sDefaultValue = "";
    
    void SetRangeValue(int nMin,int nMax,int nDefault)
    {
        SetRangeValue( nMin, nMax );
        SetDefaultValue( nDefault );
    }
    void SetRangeValue(int nMin,int nMax)
    {
        m_nMinValue     = nMin;
        m_nMaxValue     = nMax;
        SetDefaultValue( nMin );
    }
    void SetDefaultValue(int nDefault) { m_nDefaultValue = nDefault; }
    void SetDefaultValue(String sDefault) { m_sDefaultValue = sDefault; }
    
    /** Jak _wy_laczymy check boxa to pole edycyjne zmieni wartosc na m_sUnavailable
     * jak ponownie _w_laczymy check boxa, to pole edycyjne moze wrocic do poprzedniego
     * stanu (true) ewentualnie ustawi sie na m_nDefaultValue/m_sDefaultValue (false)
     */
    private boolean m_bRememberLastValue = true;
    void    SetRememberLastValue(boolean bRemember){ m_bRememberLastValue = bRemember; }
    
    /** Czy kontrolka zostala zainicjowana? */
    private boolean m_bInitialized = false;
    boolean Initialized() { return m_bInitialized; }
        
    /** poprzednia (zapamietana) wartosc pola edycyjnego */
    private String m_sLastValue = null;
    
    void InitControl(MenuNetGameSettings cParent)
	{
        if(m_cWindow != null)
			m_bEditMode   = m_cWindow.GetEditMode();
		else
			m_bEditMode = true;
		
		//pobrac elementy przez (UIWindow).GetElem(
		m_cLabel = (UIStatic)GetElem("ID_STATIC");
		m_cButton = (MenuNetCheckButton)GetElem("ID_CHECKBOX");
		m_cEdit = (MenuNetEdit)GetElem("ID_EDIT");
		
		//z zalozenia zawsze cyfry
		m_cEdit.m_sAllowedChars = ChromeMenuWindow._DIGITS_ONLY;
		m_cEdit.m_cParentSet = this;
		
		m_cButton.m_cParentSet = this;
        
        int nTextPosFlags = m_bLabelMultiline ?
                            ESpriteText.MULTILINE :
                            ESpriteText.HORIZ_LEFT | ESpriteText.VERT_MIDDLE;
        m_cLabel.SetTextPosFlags( nTextPosFlags );
        //na podstawie danych zapisanych przez menu wlaczyc lub wylaczyc kontrolki
		m_cLabel.SetText(m_sLabelText); 
        if (m_cLabel.GetTextExtent().fX >= m_cLabel.GetSize().fX)
            m_cLabel.SetTextPosFlags(ESpriteText.TOP_LEFT | ESpriteText.MULTILINE);
        else
            m_cLabel.SetTextPosFlags(ESpriteText.HORIZ_LEFT | ESpriteText.VERT_MIDDLE);

        m_cLabel.Show();
                
        m_cButton.Enable( m_bEditMode && m_bCheckBoxVisible );
        if ( m_bCheckBoxVisible )
            m_cButton.Show();
        else
            m_cButton.Hide();
        
        m_cEdit.Enable( m_bEditMode && m_bEditVisible );
        if ( m_bEditVisible )
            m_cEdit.Show();
        else
            m_cEdit.Hide();
		
		SetPos(cParent.GetCurrentSetPos());
		
        m_bInitialized = true;
    }
    
    void OnDestroy()
    { 
        super.OnDestroy();
        m_bInitialized = false;
    }
    
    /** Ustawienie wartosci pola edycyjnego liczba
     * Uzywane w trybach m_bNumberOnly m_bEditVisible
     */
	void SetValue(int nValue)
	{
		if ( ! m_bInitialized )
            return;
        m_cEdit.SetText( ""+nValue );
		
		boolean bCheck = GetInt() != m_nUnavailable;
        
		//wymuszamy aktywnosc zeby dalo sie ustawic
        m_cButton.Enable( true );
        m_cButton.SetCheck( bCheck );
        m_cButton.Enable( m_bEditMode && m_bCheckBoxVisible );
		
        //okreslamy dostepnosc pola edit, ewentualnie modyfikujemy jego wartosc
        if ( bCheck )
        {
            m_cEdit.Enable( m_bEditMode && m_bEditVisible );
			
			//musze zawolac ze wzgledu na bug w systemie menu
			m_cEdit.SetText( ""+nValue );
        }
        else
        {
            m_cEdit.Enable( false );
            if ( ! m_bCheckBoxVisible )
                LogD(" ERROR: m_bCheckBoxVisible == false , m_cEdit.Enable( false )!!!\n");
            if ( m_sUnavailable != null )
                m_cEdit.SetText( m_sUnavailable );
        }
	}
    
	/*void SetValue(int nValue)
    {
        if ( ! m_bInitialized )
            return;
        if ( ! m_bNumberOnly )
            LogD(" Kontrolka pracuje w trybie tekstowym!!!\n");
        
        //ustawiamy pole edit
        m_cEdit.SetText( ""+nValue );
        //i teraz odpowiednio check boxa
        boolean bCheck = GetInt() != m_nUnavailable;
        //wymuszamy aktywnosc zeby dalo sie ustawic
        m_cButton.Enable( true );
        m_cButton.SetCheck( bCheck );
        m_cButton.Enable( m_bEditMode && m_bCheckBoxVisible );
        //okreslamy dostepnosc pola edit, ewentualnie modyfikujemy jego wartosc
        if ( bCheck )
        {
            m_cEdit.Enable( m_bEditMode && m_bEditVisible );
        }
        else
        {
            m_cEdit.Enable( false );
            if ( ! m_bCheckBoxVisible )
                LogD(" ERROR: m_bCheckBoxVisible == false , m_cEdit.Enable( false )!!!\n");
            if ( m_sUnavailable != null )
                m_cEdit.SetText( m_sUnavailable );
            else
            {
                //TODO: co wtedy?
            }
        }
    }*/
    
    /** Ustawienie wartosci check boxa
     * Uzywane w trybach  ! m_bEditVisible
     */
    void SetValue(boolean bValue)
    {
        if ( ! m_bInitialized )
            return;
        //na czas ustawiania wartosci button mysi byc Enabled
        m_cButton.Enable( true );
        m_cButton.SetCheck( bValue );
        m_cButton.Enable( m_bEditMode && m_bCheckBoxVisible );
    }
    
    /** Ustawienie wartosci pola edycyjnego tekstem
     * Uzywane w trybach ! m_bNumberOnly m_bEditVisible
     */
    void SetValue(String sValue)
    {
        if ( ! m_bInitialized )
            return;
        
        m_cEdit.SetText( sValue );
    }
    
    /** Pobranie wartosci calkowitej
     */
    int GetInt()
    {
        if ( ! m_bInitialized )
            return -1;
        
		String sString = m_cEdit.GetText();
		
        if ( m_sUnavailable != null && 
             m_sUnavailable.compareTo( sString ) == 0 )
            return m_nUnavailable;
		if(sString.length() > 9 )
		{
			SetValue( ""+m_nMaxValue );
			return m_nMaxValue;
		}
			
        return Integer.parseInt( sString );
    }
    
    boolean GetBoolean()
    {
        if ( ! m_bInitialized )
            return false;
        return m_cButton.GetCheck();
    }
    
    String GetString()
    {
        if ( ! m_bInitialized )
            return "";
        return m_cEdit.GetText();
    }
        

    /** Handler wolany po skonczeniu edycji */
    void OnTypedValue()
    {
        //interesuje nas tylko w trybie liczbowym
        if ( m_bNumberOnly )
        {
            int nValue = GetInt();
            if ( nValue == m_nUnavailable )
                return;
            if ( nValue >= m_nMinValue && nValue <= m_nMaxValue )
                return;
            //jesli mamy wartosc poza zakresem, to ustawiamy defaulta
            //SetValue( m_nDefaultValue );
            int nVal = m_nDefaultValue;
            if ( nValue < m_nMinValue )
                nVal = m_nMinValue;
            else if ( nValue > m_nMaxValue )
                nVal = m_nMaxValue;
            SetValue( nVal );
        }
    }
    
    
    /** Handler wolany po kliknieciu na check boxa */
    void OnCheckButtonPushed(boolean bChecked)
    {
        if ( ! m_bInitialized )
            return;
        
        LogR(" "+bChecked+"\n");
        
        if ( m_bEditVisible )
        {
            if ( bChecked )
            {
                //wlaczylismy check boxa
                
                //jesli wczesniej nic nie zapamietalismy
                //to wstawiamy tu wartosc domyslna
                if ( m_sLastValue == null || m_sLastValue.compareTo("-1") == 0)
                {
                    if ( m_bNumberOnly )
                        m_sLastValue = ""+m_nDefaultValue;
                    else
                        m_sLastValue = m_sDefaultValue;
                }
                
                //przywracamy poprzednio zapamietana wartosc
                m_cEdit.SetText( m_sLastValue );
                //i udostepniamy do edycji to pole (jesl mozna)
                m_cEdit.Enable( m_bEditMode );
            }
            else
            {
                //wlaczylismy check boxa
                //zapamietajmy sobie aktualna wartosc pola edit
                if ( m_bRememberLastValue )
                    m_sLastValue = m_cEdit.GetText();
                //i wstawmy tu wartosc ktora ma byc dla trybu "niedostepny"
                m_cEdit.SetText( m_sUnavailable );
                //teraz pole edycyjne nie jest dostepne
                m_cEdit.Enable( false );
            }    
        }
    }
}
