public class ServerAutoMessageMngr extends GameObject
{
    static boolean m_bAdminConsole   = false;
    
    
    private 
    NetGameInfoHud m_cGameInfo = null;
    void SetGameInfo( NetGameInfoHud cGameInfo ) { m_cGameInfo = cGameInfo; }
    
    int METHOD_ShowInfo = 0;
    
    final static String sm_sMessagesScriptFile = "Data/ServerAutoMessages.scr";
    final static float  sm_fDefaultMsgsPeriod  = 60.0f;
    
    
    private float       m_fMsgPeriod    = sm_fDefaultMsgsPeriod;
    private ObjectsList m_cMessageList  = new ObjectsList();
    
    int m_nCurrentMsg = -1;
        
    
    void OnCreate()
    {
        METHOD_ShowInfo = RegisterCallMethod("ShowInfo");
        
        Parse( sm_sMessagesScriptFile );
        StartScheduleShowInfo();
    }
    
    void OnDestroy()
    {
        StopScheduleShowInfo();
        m_cGameInfo = null;
    }
    
    private void Parse(String sFile)
    {
        //parsujemy plik
        ParseKeyword cRoot = GameObject.ParseFile( sFile );
        if ( cRoot != null )
        {
            for ( ParseKeyword cKey = cRoot; cKey != null; cKey = cKey.GetNext() )
            {
                if (cKey.Equals("MessagesPeriod"))
                    m_fMsgPeriod = cKey.GetIParam(0);
                
                if (cKey.Equals("Message"))
                    m_cMessageList.Add( 
                      new ServerAutoMessageInfo( cKey.GetSParam(0), 
                                                 cKey.GetIParam(1),
                                                 cKey.GetIParam(2))
                                      );    
            }
            cRoot.delete();
        }
        else
        {
            //NetLog.Error("Read or parse error in file " + sFile);
        }
    }
    
    void StartScheduleShowInfo()
    {
        StopScheduleShowInfo();
        if ( m_cMessageList.GetSize() > 0 )
            CallMethodEvery( METHOD_ShowInfo, m_fMsgPeriod );
    }
    
    void StopScheduleShowInfo()
    {
        DisableCallMethod( METHOD_ShowInfo );
    }
    
    void ShowInfo()
    {
        if ( m_cGameInfo != null )
        {
            ServerAutoMessageInfo cMessage   = null;
            int                   nSafetyCnt = m_cMessageList.GetSize(); 
            do {
                m_nCurrentMsg = (m_nCurrentMsg + 1)%m_cMessageList.GetSize();
                cMessage = (ServerAutoMessageInfo)m_cMessageList.GetObjectAt( m_nCurrentMsg );
                cMessage.IncCurrentPriority();
                if ( cMessage.GetCurrentPriority() > 0 )
                    cMessage = null;
            } while ( (cMessage == null) && (nSafetyCnt-- > 0) );

            m_cGameInfo.EventServerMessage( cMessage.GetMessage(), cMessage.GetMode() );
            cMessage.ResetCurrentPriority();
        }
        else
            StopScheduleShowInfo();
    }

    static boolean AllowIncomingMsg( int nMode )
    {
        boolean bAllow = true;
        switch ( nMode )
        {
            case EServerAutoMessageMode._FULL_VERSION:
                 bAllow = NetGameVersion.IsFullVersion();
                 break;       
                 
            case EServerAutoMessageMode._DEMO:
                 bAllow = NetGameVersion.IsDemo();
                 break;
            default:          
                 bAllow = true;
        }
        return bAllow;
    }
}


class ServerAutoMessageInfo
{
    final static int _MAX_PRIORITY = 10;
    
    private
    String m_sText              = null;
    String GetText(){ return m_sText; }
    
    String GetMessage()
    { 
        return EServerAutoMessageMode.GetPrefix(m_nMode)+m_sText; 
    }
    
    
    private
    int    m_nDefaultPriority   = -1;
    
    private
    int    m_nMode = 0;
    int    GetMode(){ return m_nMode; }
    
    private
    int    m_nCurrentPriority = -1;
    int    GetCurrentPriority(){ return m_nCurrentPriority; }
    int    IncCurrentPriority(){ return m_nCurrentPriority--; }
    void   ResetCurrentPriority(){ m_nCurrentPriority = m_nDefaultPriority; }
    
    ServerAutoMessageInfo(String sText, int nDefaultPriority, int nMode)
    {
        m_sText             = sText;
        m_nDefaultPriority  = nDefaultPriority;
        if ( m_nDefaultPriority < 0 )
            m_nDefaultPriority = 0;
        else if ( m_nDefaultPriority > _MAX_PRIORITY )
            m_nDefaultPriority = _MAX_PRIORITY;
        m_nMode             = nMode;
    }
}
