public class DestroyableNet extends Destroyable
{
    /** dane potrzebne do respawnowania obiektu
     */
    //
    float m_fMaxHealth = 0.0f;
    //
    String m_sMeshName = null;
    

    final static int _STATE_SPAWNED     = 0;
    final static int _STATE_DEMOLISHED  = 1;
    final static int _STATE_DEAD        = 2;
        
    int     m_nState        = _STATE_SPAWNED;
    
    String GetStateName(int i)
    { 
        switch ( i )
        {
            case _STATE_SPAWNED: return "SPAWNED";
            case _STATE_DEMOLISHED:return "DEMOLISHED";
            case _STATE_DEAD:return "DEAD";
        }
        return "unknown ("+i+")";
    }
    
    /** Obiekt do ktorego jestesmy podlaczeni
     * zapamietywany w die (przy odlaczaniu) i wykorzystywany przy
     * kolejnym Respawn*/
    ControlObject m_cParent = null;
    
    boolean IsAlive(){ return m_nState != _STATE_DEAD; }
    
    /**
     * Kiedy ostatnio byl obiekt zaatakowany
     * jesli -1, to znaczy ze od spawnu jeszcze nigdy
     */
    private float m_fUnderAttackTime = -1.0f;
    
    /**
     * Ile czasu musi uplynac od poprzedniego ataku, zeby
     * podniesc alarm o ponownym natarciu
     */
    static 
    float sm_fMinAlertDeltaAttackTime = 20.0f;
    
    /**
     * Globalny licznik zmian stanow obiektow
     * Bedzie on wykorzystywany na klientach do rozstrzygania czy dana
     * akcja (ktora zleca nam serwer) ma sie wykonac czy nie (bo jest stara)
     */
    private static 
    ChangeStateCounter sm_cGlobalDestroyableStateCounter = new ChangeStateCounter();
    static void CleanGlobalDestroyableStateCounter(){sm_cGlobalDestroyableStateCounter.Clean();}
    static int  GetNextGlobalDestroyableStateCounter(){ return sm_cGlobalDestroyableStateCounter.GetNext();}
    
    /**
     * Indywidualny licznik zapamietujacy ostatnio
     * odebrana i wykonana akcje na tym obiekcie
     */
    private 
    ChangeStateCounter m_cDestroyableStateCounter = new ChangeStateCounter(this);
    boolean VerifyDestroyableStateCounter( int nCounter ){ return m_cDestroyableStateCounter.Verify( nCounter ); }

    
    public void OnCreate()
    {
        super.OnCreate();
        
        m_fMaxHealth = fHealth;
        m_sMeshName  = MeshName;
        
        SetFlag( EFlags._PORTAL_INDEPENDENT );
        
        bGreatHit           = true;
        bDeleteAfterDestroy = true;
    }
    
    public void OnDestroy()
    {
        super.OnDestroy();
        
        m_cParent = null;
    }
    
    void ServerRespawn()
    {
        if ( Role == ERole._AUTHORITY )
        {
            Respawn();
            MsgSendClientDestroyableChangeState();
        }
    }
    
    void Respawn()
    {
        // kasujemy fxy poprzednio dodane do obiektu
        CallMethodIn( "DeleteAttachedFXes", 0.0f );
        // kasujemy wszystko co bylo postawione na obiekcie
        CallMethodIn( "DeleteAllChildren", 0.0f );
                
        // przywracamy zdrowie
        fHealth = m_fMaxHealth;
        bDead   = false;
        
        // podmieniamy mesha
        MeshName = m_sMeshName;
        LoadMesh();
        //przywrocic kolor mesha
        //TODO: dobrze 1,1,1? i w ogole potrzebne
        SetMeshColor( new Vector(1.0f, 1.0f, 1.0f) );
                
        if ( m_cParent != null )
             m_cParent.AttachChildAndKeepWorldPos( this );
        
        m_fUnderAttackTime = GetTime() - sm_fMinAlertDeltaAttackTime;
        
        ActivateOnModule();
        
        m_nState = _STATE_SPAWNED;
    }
    
    private 
    PlayerNet m_cAssasinPlayer = null;
    PlayerNet GetAssasinPlayer(){ return m_cAssasinPlayer; }
                                
    public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
    {
        if ( Role == ERole._AUTHORITY )
        {
            m_cAssasinPlayer = ModuleNet.GetAssassinPlayerNetFromObject( Assassin );

            super.OnTakeDamage( fDamage, fPhSimDamage, Assassin, vHitLocation, 
                                vSpeed, nMeshElementorBoneID, nDamageType );
            
            CheckUnderAttack();
        }
    }
    
    private void CheckUnderAttack()
    {
        float fTime  = GetTime();
        float fDelta = fTime - m_fUnderAttackTime;
        if ( fDelta >= sm_fMinAlertDeltaAttackTime )
            UnderAttack();
        m_fUnderAttackTime = fTime;
    }
            
    
    void UnderAttack()
    {
    }
    
    public void Demolish()
    {
        super.Demolish();
        
        m_nState = _STATE_DEMOLISHED;
        
        if ( Role == ERole._AUTHORITY )
            MsgSendClientDestroyableChangeState();
    }
    
    public void Die()
    {
        m_cParent = GetParent(); 

        super.Die();
        
        m_nState = _STATE_DEAD;
                
        if ( Role == ERole._AUTHORITY )
            MsgSendClientDestroyableChangeState();
    }
    
    
    void ChangeState( int nState )
    {
        if ( m_nState == nState )
        {
            NetLog.Destroyable( "ChangeState skipped, state "+
                                GetStateName(m_nState),
                                this );
            return;
        }
        
        NetLog.Destroyable("ChangeState "+
                           GetStateName(m_nState)+
                           " -> "+
                           GetStateName(nState), 
                           this );
        
        
        switch ( nState )
        {
        case _STATE_SPAWNED:
            Respawn();
            break;
        case _STATE_DEMOLISHED:
            if ( m_nState == _STATE_DEAD )
                Respawn();
            Demolish();
            break;
        case _STATE_DEAD:
            Die();
            break;
        }
    }
    
    void DelayedBlow()
    {
        // odpalamy odbieranie energii obiektom w promieniu
        if( (fBlastRange > 0.0f) || (fBlastDamage > 0.0f) ) 
            Blast();
        
        //CallMethodIn("delete", 0.0f);
    }
    
    void Blast()
    {
        if ( Role == ERole._AUTHORITY )
            super.Blast();
	}

    
    public void OnReplicateServer()
    {
        if ( bNetInit )
		    MsgSendClientDestroyableState();
    }
    
    
    final int MSG_CLIENT_DESTROYABLE_STATE          = 20;
    final int MSG_CLIENT_DESTROYABLE_CHANGESTATE    = 21;
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case MSG_CLIENT_DESTROYABLE_STATE:
            NetLog.MsgIn("MSG_CLIENT_DESTROYABLE_STATE", this );
            MsgRecvClientDestroyableState();
            break;
            
        case MSG_CLIENT_DESTROYABLE_CHANGESTATE:
            NetLog.MsgIn("MSG_CLIENT_DESTROYABLE_CHANGESTATE", this );
            MsgRecvClientDestroyableChangeState();
            break;
            
        default:
            NetLog.MsgIn("CANCEL "+MessageGetId(), this );
            MessageCancel();
        }
    }
    
    
    
    void MsgSendClientDestroyableState()
    {
        NetLog.MsgOut("MSG_CLIENT_DESTROYABLE_STATE", this );
        MessageNew( MSG_CLIENT_DESTROYABLE_STATE );
        MessagePackInt( GetNextGlobalDestroyableStateCounter() );
        MessagePackState();
        MessageSend();
        NetLog.Destroyable( "SendState "+GetStateName(m_nState), 
                            this );
    }
    void MsgRecvClientDestroyableState()
    {
        int nStateCounter = MessageUnPackInt();
        int nState        = MessageUnPackState();
        MessageEnd();
        NetLog.Destroyable( "RecvState "+GetStateName(nState), 
                            this );
        
        if ( VerifyDestroyableStateCounter( nStateCounter ) )
        {
            //ta zmiana nastapila juz jakis czas temu, wiec trzeba
            //potraktowac to jako wczytuwanie z dysku :-)
            bLoadingSG = true;
            ChangeState( nState );
            bLoadingSG = false;
        }
    }
        
    
    void MsgSendClientDestroyableChangeState()
    {
        NetLog.MsgOut("MSG_CLIENT_DESTROYABLE_CHANGESTATE", this );
        MessageNew( MSG_CLIENT_DESTROYABLE_CHANGESTATE );
        MessagePackInt( GetNextGlobalDestroyableStateCounter() );
        MessagePackState();
        MessageSend();
        NetLog.Destroyable( "SendChangeState "+GetStateName(m_nState), 
                            this );
    }
    void MsgRecvClientDestroyableChangeState()
    {
        int nStateCounter = MessageUnPackInt();
        int nState        = MessageUnPackState();
        MessageEnd();
        NetLog.Destroyable( "RecvChangeState "+GetStateName(nState), 
                            this );
        
        if ( VerifyDestroyableStateCounter( nStateCounter ) )
            ChangeState( nState );
    }

    void MessagePackState()
    {
       MessagePackInt( m_nState ); 
    }
    
    int MessageUnPackState()
    {
        return MessageUnPackInt(); 
    }
    
}
