/**
 * Manager ma liste wszystkich spawnow w _losowej_ kolejnosci
 * Przy pobieraniu nowego przemiatana jest lista od 0 do konca
 * Pierwszy napotkany spawnpoint (ktory spelnia kryteria, np teamu)
 * jest zwracany do modulu go wykorzystania, po czym usuwany
 * jest z listy i wrzucany na jej koniec (konkretnie w losowe
 * miejsce w drugiej "polowie" dlugosci listy).
 * To wszystko zagwarantuje, ze
 *  - nie bedzie sytuacji ze ten sam spawnpoint bedzie uzywany 
 *    do kolejnych "urodzen"
 *  - kolejnosc zwracanych spawnow bedzie losowa
 */
public class NetSpawnPointManager
{
    /**
     * Level na ktorym pracyje mananager
     */
    private
    ModuleNetBase m_cModule = null;
    
    private
    NetPlayerInfoList m_cPlayerInfoList = null;
        
    /**
     * Znaleznione spawnpointy na levelu
     */
    private
    java.util.Vector m_aSpawnPoints = new java.util.Vector(); 
    
    /**
     * Punkt z ktorego bedziemy obserwowali gre po podlaczeniu
     */
    private
    NetSpawnPoint    m_cJoinPoint  = null;                         
        
    NetSpawnPointManager(ModuleNetBase cModule)
    {
        m_cModule         = cModule;
        //m_cPlayerInfoList = cModule.GetGameInfo().GetPlayerInfoList();
    
        //pobierzmy tymczasowa tablice spawnpointow
        GameObject []aSpawnPoints = cModule.GetObjectsOfClass("NetSpawnPoint");

        if ( m_aSpawnPoints != null )
        {
            //przepiszmy ta tablice do naszej listy
                    
            //jeden po drugim wskakuja  na liste
            for ( int i = 0 ; i < aSpawnPoints.length; i++ )
            {
                if ( ! (aSpawnPoints[ i ] instanceof NetSpawnPoint) )
                    continue;
                //ale wcale kolenosc nie jest zachowana
                //bo juz wstepnie mieszamy kolejnosc obiektow
                int nNewPos = m_aSpawnPoints.size() > 0 ?
                              GameObject.RandomInt( m_aSpawnPoints.size() ):
                              0 ;
                //wstawiamy w wylosowana pozycje
                m_aSpawnPoints.insertElementAt( (NetSpawnPoint)aSpawnPoints[ i ], nNewPos );
                //pierwszy z nich jest punktem do joina,
                //TODO: zrobi obsluge joincamery
                if ( m_cJoinPoint == null )
                    m_cJoinPoint = (NetSpawnPoint)aSpawnPoints[ i ];
            }
            //GameObject.CrashLog("Ilosc spawnpointow "+m_aSpawnPoints.size()+"\n");
        }
        else
            GameObject.CrashLog("Brak spawnpointow\n");
    }

    /**
     * Trzeba po sobie posprzatac
     */
    void Destroy()
    {
        m_cModule          = null;
        m_cPlayerInfoList  = null;
        
        m_aSpawnPoints.removeAllElements();
        m_aSpawnPoints     = null;
    }
    
    
    NetSpawnPoint GetRandomSpawnPoint(PlayerNet cPlayer)
    {
        NetPlayerInfo cPlayerInfo = cPlayer.GetPlayerInfo();
        
        if ( cPlayerInfo == null )
        {
            GameObject.CrashLog("ERROR: GetSpawnPoint , PlayerInfo == null \n");
            return null;;
        }

        //szukamy po kolei
        for ( int i = 0; i < m_aSpawnPoints.size(); i++ )
        {
            NetSpawnPoint cPoint = GetElement( i );
            
            if ( cPoint == null )
                continue;

            //czy spelnia wymogo stawiane przez NetGameInfo?
            if ( VerifySpawnPoint( cPoint, cPlayerInfo ) )
            {
                //ok, znalezlismy dobry spawn, 
                //trzeba go zwrocic, ale najpierw przestawic gdzies na koniec listy
                
                //usuwamy wiec go na chwile
                m_aSpawnPoints.removeElement( cPoint );
                //wyliczamy mu pozycje gdzies w drugiej polowie listy
                int nHalfListSize = (m_aSpawnPoints.size()) / 2 + 1;
                int nNewPos       = nHalfListSize + GameObject.RandomInt( nHalfListSize );
                // i wpisujemy go znowu na liste
                if ( nNewPos > m_aSpawnPoints.size() )
                     nNewPos = m_aSpawnPoints.size();
                m_aSpawnPoints.insertElementAt( cPoint, nNewPos );
                // zwracamy go jako znalezionego
                return cPoint;
            } 
        }
        //no coz, chyba cos nie tak (?)
        GameObject.CrashLog("ERROR: GetSpawnPoint , SpawnPoint not found \n");
        return null;
    }
    
    static Vector sm_vPos = new Vector();
    
    /** Minimalny dystans od gracza, zeby mozna sie bylo wyspawnowac
     */
    final static float _MIN_DISTANCE = 300.0f;
    
    boolean VerifySpawnPoint( NetSpawnPoint cPoint, NetPlayerInfo cPlayerInfo )
    {
        cPoint.SetDistanceToNearestPlayer( Float.MAX_VALUE );
        
        //pierw sprawdzamy czy odpowiedni dla teamu
        if ( ! m_cModule.GetGameInfo().VerifySpawnPoint( cPoint, cPlayerInfo ))
            return false;
        return true;
        /* TODO   
        //teraz bedziemy sprawdzac odleglosc od innych graczy
        Vector vPos         = cPlayerInfo.GetPlayer().GetPosition();
        float  fMinDistance = Float.MAX_VALUE;
            
        if ( m_cModule.GetGameInfo()                   != null &&
             m_cModule.GetGameInfo().GetPlayerInfoList != null )
        {
            for ( int i=0; i<m_cPlayerInfoList.GetSize(); i++ )
            {
                NetPlayerInfo cInfo = m_cPlayerInfoList.GetPlayerByIndex( i );
                if ( cInfo              != null   &&
                     cInfo.IsAlive()              &&
                     cInfo.GetPlayer()  != null )
                {
                    cInfo.GetPlayer().GetPosition( sm_vPos );
                    float fDistance = sm_vPos.Distance( vPos );
                    
                    if ( fMinDistance > fDistance )
                         fMinDistance = fDistance;
                }
            }
        }
        
        cPoint.SetDistanceToNearestPlayer( fMinDistance );
        return fMinDistance > _MIN_DISTANCE;
        */
    }
    
    NetSpawnPoint GetJoinPoint()
    {
        return m_cJoinPoint;
    }
    
    private NetSpawnPoint GetElement(int i)
    {
        if (m_aSpawnPoints == null || m_aSpawnPoints.size() < i)
            return null;
        return (NetSpawnPoint)m_aSpawnPoints.get( i );
    }
}
