public class PlayerNetBaseDriver extends PlayerNetBaseMover
{
    /**
     * Czy podczas rozwalenia pojazdu w ktorym znajduje
     * sie wlasnie gracz, powinien on dostac obrazenia
     * Jesli tak, to liczone sa one na podstawie zmiennej
     * ustawianej w pojezdzie i dostaepnej po zawolaniu
     * GetOccupantsDamageMulOnCrash() wzgledem pojazdu
     */
    static boolean sm_bDamageOnVehicleCrashed = true;

    int METHOD_ActionGetInDo = 0;
    
    public void OnDestroy()
    {
        METHOD_ActionGetInDo = RegisterCallMethod("ActionGetInDo");
            
        super.OnDestroy();
        
        if ( m_nState == _STATE_IN_VEHICLE )
        {
            ActionGetOutDo();
        }
    }
    
    /**
     * Globalny licznik zmian stanow obiektow
     * Bedzie on wykorzystywany na klientach do rozstrzygania czy dana
     * akcja (ktora zleca nam serwer) ma sie wykonac czy nie (bo jest stara)
     * TODO: dac szerszy opis
     */
    private static 
    ChangeStateCounter sm_cGlobalDriverCounter = new ChangeStateCounter();
    static void CleanGlobalDriverCounter() { sm_cGlobalDriverCounter.Clean(); }
    static int  GetNextGlobalDriverCounter(){ return sm_cGlobalDriverCounter.GetNext(); }
    /**
     * Indywidualny licznik zapamietujacy ostatnio
     * odebrana i wykonana akcje na tym obiekcie
     */
    private 
    ChangeStateCounter m_cDriverCounter = new ChangeStateCounter(this);
    int     GetDriverCounter(){ return m_cDriverCounter.Get(); }
    boolean VerifyDriverCounter( int nCounter ){ return m_cDriverCounter.Verify( nCounter ); }
    
    
	void PrepareHUDToGetIn(Vehicle vehicle)
	{
        super.PrepareHUDToGetIn(vehicle);
        
        if (cSeatInfo != null)
        {
            if ( ((cSeatInfo.GetRole() == EPassengerRole._VEHICLE_DRIVER) &&
                  (!(vehicle instanceof Walker))) ||
                  (cSeatInfo.GetRole() == EPassengerRole._VEHICLE_PASSENGER_IDLE)
               )
            {
                if (HUD.GetHUDCrossHair() != null)
                    HUD.GetHUDCrossHair().Deactivate();
            }        
        }
    }

	void PrepareHUDToGetOut()
	{
        super.PrepareHUDToGetOut();
        
        if (HUD != null)
        {
            if (HUD.GetHUDCrossHair() != null)
                HUD.GetHUDCrossHair().Activate();
        }
    }


    /**
     * Client
     * Flaga mowiaca o tym, ze jest to akcja ktora 
     * kazal nam wykonac serwer, czyli MUSI sie powiesc
     * JEZELI ma nowszy numerek niz gracz ktory z nami koliduje
     */
    boolean m_bGetInOutFromServer = false;
    
    void ActionGetIn(int nSeatVehicleToGetIn,int nSeatIndex)
    {
        GameObject cObject = GetObjectByNetId( nSeatVehicleToGetIn );
            
        if ( cObject == null ) 
        {
            NetLog.Error("ActionGetIn cObject == null");
            return;
        }
        
        if ( !( cObject instanceof ISeatVehicle ))
        {
            NetLog.Error( "ActionGetIn "+GetDriverInfo()+" obj:"+cObject+" is not instanceof ISeatVehicle");
            return;
        }
        
        ISeatVehicle cSeatVehicle = (ISeatVehicle)cObject;
        SeatInfo []  aSeats       = cSeatVehicle.GetSeats();
        SeatInfo     cSeatInfo    = ((aSeats != null)&&(aSeats.length>nSeatIndex))?
                                     aSeats[ nSeatIndex ]: null;
        if ( cSeatInfo == null )
        {
            NetLog.Error("ActionGetIn cSeatInfo == null");
            return;
        }
        ActionGetIn( cSeatInfo );
    }
    
    
    void ActionGetIn(SeatInfo cSeat)
    {
        if (!(cSeat.GetOccupiedObject() instanceof ISeatVehicle))
        {
            NetLog.PlayerDriver("GET_IN canceled "+GetDriverInfo()+" because "+
                                ((cSeat.GetOccupiedObject()!=null)?
                                 (cSeat.GetOccupiedObject().GetNetInfo()+" is not ISeatVehicle"):
                                 "cSeat.GetOccupiedObject()==null"));
            return;
        }
        
        ISeatVehicle cSeatVehicle  = (ISeatVehicle)cSeat.GetOccupiedObject();
        m_nSeatVehicleToGetInOut   = cSeatVehicle.GetNetId();
        SeatInfo [] aSeats         = cSeatVehicle.GetSeats();
        m_nSeatIndexToGetInOut     = -1;
        if ( aSeats != null )
            for ( int i=0; i<aSeats.length; ++i)
            {
                if ( aSeats[ i ] == cSeat )
                {
                    m_nSeatIndexToGetInOut = i;
                    break;
                }
            }
                                                  
                        
        if (Role == ERole._AUTHORITY)
        {
            // jestesmy serwerem
            if ( cSeatVehicle instanceof Vehicle )
            {
                Vehicle cVehicle = (Vehicle)cSeatVehicle;
                
                if ( !cVehicle.DoesCanDrive() )
                    return;
            
                //TODO: przeniesc sprawdzenie do GameSettings
                if ( cVehicle.m_bTeamExclusive )
                {
                    //pojazd jest na wylacznosc jednej z druzyn
                    //sprawdzmy kto jest w srodku
                    Pawn[] aOccupants = cVehicle.GetAllOccupants();
                    if (aOccupants != null)
                    {
                        //ktos jest, bierzemy pierwszego z brzegu
                        Pawn cPawn = aOccupants[ 0 ];
                        if ( cPawn instanceof PlayerNet )
                        {
                            PlayerNet cPlayer = (PlayerNet)cPawn;
                            
                            //jesli jest z innej druzyny to nie da sie wsiasc
                            if ( GetPlayerInfo().GetTeamId() != 
                                 cPlayer.GetPlayerInfo().GetTeamId() )
                                return;
                        }
                    }
                }
            }
                
            if (bNetIsOwner)
            {
                // to my chcemy wsiasc
                m_cSeatToGetInOut = cSeat;
            }
            else
            {
                // jezeli nie jestemy wlascicielem, to znaczy ze to 
                // jakis klient wykonal ta akcje,
                
                // pobiezmy siedzenie po jego indeksie
                
                aSeats             = cSeatVehicle.GetSeats();
                m_cSeatToGetInOut  = ((aSeats != null)&&(aSeats.length>m_nSeatIndexToGetInOut))?
                                     aSeats[ m_nSeatIndexToGetInOut ]: null;
                
                //i zweryfikujmy akcje wsiadania
                
                //nie mozna strzelac podczas wsiadania do pojazdu
                if (nFiringState != Weapon._NO_FIRING)
                {
                    NetLog.PlayerDriver("GET_IN canceled "+GetDriverInfo()+" because nFiringState == "+nFiringState);
                    return;
                }
                
                //TODO:
                //sprawdzic, jakies dodatkowe warunki czy da sie wsiasc, 
                //jezeli nie to wypad...
            }
            
            // i sami wykonujemy akcje
            NetLog.PlayerDriver("GET_IN "+GetDriverInfo()+
                                "\n vehicle: "+GetNetInfo(cSeatVehicle)+ 
                                "\n seat: "+m_cSeatToGetInOut.GetNetInfo());        
            super.ActionGetInDo();
            
            // jesli udalo sie wsiasc, to wysylamy klientom info o akcji
            if ( m_cSeatToGetInOut.GetOccupant() == this )
            {
                NetLog.PlayerDriver("GET_IN success -> sendclient "+GetDriverInfo());
                MsgSendClientAction( EPlayerAction.VEHICLE_GET_IN, 
                                     m_nSeatVehicleToGetInOut,
                                     m_nSeatIndexToGetInOut,
                                     GetNextGlobalDriverCounter() );
            }
            else
                NetLog.PlayerDriver("GET_IN failed "+GetDriverInfo());
        }
        else
        {
            // jestesmy klientami
            
            // wyslij requesta do serwera
            NetLog.PlayerDriver("GET_IN sendrequest "+GetDriverInfo()+
                                "\n vehicle "+GetNetInfo(cSeatVehicle)+
                                "\n seat "+cSeat.GetNetInfo());
            MsgSendServerActionRequest( EPlayerAction.VEHICLE_GET_IN, 
                                        m_nSeatVehicleToGetInOut,
                                        m_nSeatIndexToGetInOut
                                        );
        }
    }

    /** CLIENT wykonanie zmiany broni na kliencie
     * Pierw przygotowanie (znalezienie obiektow) pozniej prawdziwa akcja
     */
    void ActionGetInDo()
    {
        DisableCallMethod( METHOD_ActionGetInDo );
        
        if ( m_nSeatVehicleToGetInOut == -1 )
        {
            //takiej sytuacji byc nie powinno
            NetLog.Error( "ActionGetInDo "+GetDriverInfo()+" m_nSeatVehicleToGetInOut == -1");
            return;
        }
        
        GameObject cObject = GetObjectByNetId( m_nSeatVehicleToGetInOut );
            
        //if (cVehicle == null) 
        if ( cObject == null ) 
        {
            CallMethodIn( METHOD_ActionGetInDo, 0.1f );
            return;
        }
        
        if ( !( cObject instanceof ISeatVehicle ))
        {
            NetLog.Error( "ActionGetInDo "+GetDriverInfo()+" obj:"+cObject+" is not instanceof ISeatVehicle");
            return;
        }
        
        ISeatVehicle cSeatVehicle = (ISeatVehicle)cObject;
        SeatInfo [] aSeats = cSeatVehicle.GetSeats();
        m_cSeatToGetInOut  = ((aSeats != null)&&(aSeats.length>m_nSeatIndexToGetInOut))?
                             aSeats[ m_nSeatIndexToGetInOut ]: null;
        if ( m_cSeatToGetInOut == null )
        {
            NetLog.Error( "ActionGetInDo "+GetDriverInfo()+" to "+GetNetInfo(cSeatVehicle)+
                          " m_cSeatToGetInOut ("+m_nSeatIndexToGetInOut+") == null");
            return;
        }
            
        //zagwarantujmy sobie ze na kliencie akcje wykonaja sie na pewno
        //w dobrej kolejnosci
        if ( m_bGetInOutFromServer && 
             (m_cSeatToGetInOut.IsOccupied()) &&
             (m_cSeatToGetInOut.GetOccupant() instanceof PlayerNet) )
        {
            //jakis gracz ktos siedzi na siedzeniu na ktorym my chcemy usiasc
            PlayerNet cPlayer = (PlayerNet)m_cSeatToGetInOut.GetOccupant();
                
            //sprawdzmy ktory z nas tak naprawde tu powinien siedziec
            if ( cPlayer.GetDriverCounter() < GetDriverCounter() )
            {
                //on ma starszy stan niz my, wywalamy go z "naszego" siedzenia
                NetLog.PlayerDriver("GET_OUT force "+cPlayer.GetDriverInfo()+" because "+GetDriverInfo());
                cPlayer.ActionGetOutDo();
            }
            else
            {
                //on ma nowszy stan niz my, rezygnujemy z wsiadania
                NetLog.PlayerDriver("GET_IN canceled "+GetDriverInfo()+" because "+cPlayer.GetDriverInfo());
                return;
            }
        }

        NetLog.PlayerDriver("GET_IN "+GetDriverInfo()+
                            "\n vehicle: "+GetNetInfo(cSeatVehicle)+ 
                            "\n seat: "+m_cSeatToGetInOut.GetNetInfo());        
        super.ActionGetInDo();
        
        //jesli to akcja na kliencie (zadanie z serwera) to _musiala_ sie udac 
        if ( m_bGetInOutFromServer && (m_cSeatToGetInOut.GetOccupant() != this))
        {
            //tak sie nie stalo, to znaczy ze gdzies byl blad!!!!
            NetLog.Error("GET_IN failed "+GetDriverInfo()+
                         "\n vehicle: "+GetNetInfo(cSeatVehicle)+ 
                         "\n seat: "+m_cSeatToGetInOut.GetNetInfo());        
        }
    }

    
    void ActionGetOut()
    {
        // sprawdzmy czy faktycznie jestesmy w jakims pojezdzie
        if ( (m_nState != _STATE_IN_VEHICLE) || (cSeatInfo == null) )
        {
            if ( m_nState == _STATE_IN_VEHICLE )
                NetLog.Error( "ActionGetOut "+GetDriverInfo()+" cSeatInfo == null");
            else if ( cSeatInfo != null )
                NetLog.Error( "ActionGetOut "+GetDriverInfo()+" m_nState != _STATE_IN_VEHICLE");
            else 
                NetLog.Error( "ActionGetOut "+GetDriverInfo()+" not in ISeatVehicle");
            return ;
        }
            
        if (Role == ERole._AUTHORITY)
        {
            // jestesmy serwerem
            SeatInfo cSeat             = cSeatInfo;
            ISeatVehicle cSeatVehicle  = (ISeatVehicle)cSeatInfo.GetOccupiedObject();
                
            //TODO: jakies dodatkowe warunki na wysiadanie
            if ( (cSeatInfo.GetOccupant() == this) && cSeatInfo.CanGetOut() )
            {
                m_nSeatVehicleToGetInOut = cSeatVehicle.GetNetId();
                SeatInfo [] aSeats       = cSeatVehicle.GetSeats();
                m_nSeatIndexToGetInOut   = -1;
                if ( aSeats != null )
                    for ( int i=0; i<aSeats.length; ++i)
                    {
                        if ( aSeats[ i ] == cSeat )
                        {
                            m_nSeatIndexToGetInOut = i;
                            break;
                        }
                    }
                // i sami wykonujemy akcje
                ClearODE();
                ActionGetOutDo();    
                SetPhysicsLastUpdateTime( RoundToPhysicsGranularity(GetPhysicsLastUpdateTime() - sm_fPhysicsGranularity * 2.f) );
                //LogR("Ustawiam przy wysiadaniu (Server) LUT: " + GetPhysicsLastUpdateTime() + "\n");
                
                //sprawdzmy czy na pewno sie udalo
                if ( cSeat.GetOccupant() == null )
                {
                    NetLog.PlayerDriver("GET_OUT success -> sendclient "+GetDriverInfo());
                    // wysylamy klientom info o akcji
                    MsgSendClientAction( EPlayerAction.VEHICLE_GET_OUT, 
                                         m_nSeatVehicleToGetInOut, 
                                         m_nSeatIndexToGetInOut, 
                                         GetNextGlobalDriverCounter(),
                                         EMsgSendMode.ALL, 
                                         GetPhysicsLastUpdateTime() );
                }
                else
                {
                    NetLog.Error("GET_OUT failed "+GetDriverInfo()+
                                 "\n vehicle "+GetNetInfo(cSeatVehicle)+
                                 "\n seat "+cSeat.GetNetInfo());
                }
            }
            else
            {
                NetLog.PlayerDriver("GET_OUT canceled "+GetDriverInfo()+
                                    "\n vehicle "+GetNetInfo(cSeatVehicle)+
                                    "\n seat "+cSeat.GetNetInfo()+
                                    "\n CanGetOut() "+CanGetOut());
            }
                
        }
        else
        {
            // jestesmy klientami
            
            // wyslij requesta do serwera
            if ( cSeatInfo.GetOccupiedObject() instanceof ISeatVehicle )
            {
                ISeatVehicle cSeatVehicle = (ISeatVehicle)cSeatInfo.GetOccupiedObject();
                
                NetLog.PlayerDriver("GET_OUT sendrequest "+GetDriverInfo()+
                                    "\n vehicle "+GetNetInfo(cSeatVehicle)+
                                    "\n seat "+cSeatInfo.GetNetInfo());
                
                //TODO: ewentualnie dorzucic idenyfikator siedzenia
                MsgSendServerActionRequest( EPlayerAction.VEHICLE_GET_OUT, 
                                            cSeatVehicle.GetNetId() );
            }
        }
    }
    
    void ActionGetOutDo()
    {
        if ( (m_nState != _STATE_IN_VEHICLE) || (cSeatInfo == null) )
        {
            if ( m_nState == _STATE_IN_VEHICLE )
                NetLog.Error( "ActionGetOutDo "+GetDriverInfo()+" cSeatInfo == null");
            else if ( cSeatInfo != null )
                NetLog.Error( "ActionGetOutDo "+GetDriverInfo()+" m_nState != _STATE_IN_VEHICLE");
            else 
                NetLog.Error( "ActionGetOutDo "+GetDriverInfo()+" not in vehicle");
            return ;
        }
        
        //zapamietajmy sobie z czego wysiadamy, zeby mozna bylo
        //wyswietlic ewentualny komunikat o bledzie
        SeatInfo      cSeatInfoToGetOut     = cSeatInfo;
        ISeatVehicle  cSeatVehicleToGetOut  = (ISeatVehicle)cSeatInfoToGetOut.GetOccupiedObject();
        
        NetLog.PlayerDriver("GET_OUT "+GetDriverInfo()+
                            "\n vehicle: "+((cSeatVehicleToGetOut!=null)?GetNetInfo(cSeatVehicleToGetOut):"null")+ 
                            "\n seat: "+((cSeatInfoToGetOut!=null)?cSeatInfoToGetOut.GetNetInfo():"null"));
        super.ActionGetOutDo();
        
        //jesli to byla akcja zlecona z serwera po _musiala_ sie udac
        if ( m_bGetInOutFromServer && (m_nState == _STATE_IN_VEHICLE))
        {
            //jesli tak sie nie stalo, to znaczy ze byla jakas zwala
            NetLog.Error("GET_OUT failed "+GetDriverInfo()+
                         "\n vehicle: "+((cSeatVehicleToGetOut!=null)?GetNetInfo(cSeatVehicleToGetOut):"null")+ 
                         "\n seat: "+((cSeatInfoToGetOut!=null)?cSeatInfoToGetOut.GetNetInfo():"null"));
        }
    }
    
    /**
     * Akcja zmiany miejsca siedzenia.
     * Gracz musi sie znajdowac w jakims pojezdzie.
     * Pojazd ten powinien miec minimum 2 siedzenia.
     * Przynajmniej jedno z siedzen (o innej roli niz to na ktorym
     * siedzimy) musi byc wolne, zeby mozna sie bylo przesiasc.
     */
    void ActionChangeSeat()
    {
        // sprawdzmy czy faktycznie jestesmy w jakims pojezdzie
        if ( (m_nState != _STATE_IN_VEHICLE) || (cSeatInfo == null) )
        {
            if ( m_nState == _STATE_IN_VEHICLE )
                NetLog.Error( "ActionChangeSeat "+GetDriverInfo()+" cSeatInfo == null");
            else if ( cSeatInfo != null )
                NetLog.Error( "ActionChangeSeat "+GetDriverInfo()+" m_nState != _STATE_IN_VEHICLE");
            else 
                NetLog.Error( "ActionChangeSeat "+GetDriverInfo()+" not in ISeatVehicle");
            return ;
        }
            
        ISeatVehicle cSeatVehicle  = (ISeatVehicle)cSeatInfo.GetOccupiedObject();
        SeatInfo [] aSeats         = null; 
        if ( cSeatInfo.GetOccupant() == this )
        {
            //jezeli mozemy wysiasc, to sprawdzmy czy mozna sie 
            //na cos przesiasc
            aSeats = cSeatVehicle.GetSeats();
            if ( ( aSeats == null ) || ( aSeats.length < 2 ) )
            {
                //niestety nie ma odpowiedniej ilosci siedzien
                //TODO: comm
                return;
            }
        }
                
        
        if ( Role == ERole._AUTHORITY )
        {
            //sprawdzamy na ktorym siedzeniu siedzimy
            int nCurrentSeat = -1;
            if ( aSeats != null )
            {
                for ( int i = 0; i < aSeats.length; ++i )
                {
                    if ( aSeats[ i ] == cSeatInfo )
                    {
                        nCurrentSeat = i;
                        break;
                    }
                }
            }
            
            if ( nCurrentSeat > -1 )
            {
                //teraz bedziemy chcieli znalezc nastepne wolne
                //ktore spelnia inna role niz nasze
                int      nCurrentRole = cSeatInfo.GetRole();
                SeatInfo cTargetSeat  = null;
                int      nTargetSeat  = nCurrentSeat;
                do {
                    nTargetSeat = (nTargetSeat + 1) % aSeats.length;
                    if ( (aSeats[ nTargetSeat ] != null) &&
                         (aSeats[ nTargetSeat ].GetOccupant()==null) &&
                         (aSeats[ nTargetSeat ].GetRole()!=nCurrentRole) )
                    {
                        //znalezione wolne siedzenia
                        cTargetSeat = aSeats[ nTargetSeat ];
                        break;
                    }
                } while ( nTargetSeat != nCurrentSeat );
                        
                if ( cSeatInfo.CanGetOut() && (cTargetSeat != null) )
                {
                    //najpierw wysiadamy
                    ActionGetOut();
                    //i od razu wsiadamy na nowe siedzenia
                    ActionGetIn( cTargetSeat );    
                }
            }
        }
        else
        {
            // jestesmy klientami
            
            // wyslij requesta do serwera
            if ( cSeatInfo.GetOccupiedObject() instanceof ISeatVehicle )
            {
                cSeatVehicle = (ISeatVehicle)cSeatInfo.GetOccupiedObject();
                
                NetLog.PlayerDriver("CHANGE_SEAT sendrequest "+GetDriverInfo()+
                                    "\n vehicle "+GetNetInfo(cSeatVehicle)+
                                    "\n seat "+cSeatInfo.GetNetInfo());
                
                //TODO: ewentualnie dorzucic idenyfikator siedzenia
                MsgSendServerActionRequest( EPlayerAction.VEHICLE_CHANGE_SEAT, 
                                            cSeatVehicle.GetNetId() );
            }
        }
    }
    
    void GetIn(SeatInfo cSeat)
    {
        super.GetIn( cSeat );
        m_nState = _STATE_IN_VEHICLE;
        SetCorrectPosOnRender( false );
    }
    
    
    public void OnReceivedDataOnClient()
    {
        super.OnReceivedDataOnClient();
            
        if ((m_nState == _STATE_IN_VEHICLE) && (GetParent() == null))
        {
            String sLog = "!!! _STATE_IN_VEHICLE but no parent!!! "+GetDriverInfo();
            if ( (cSeatInfo != null) && (cSeatInfo.GetOccupant() == this) )
            {
                cSeatInfo.AttachOccupant();
                SitDownToDrive();
                sLog = sLog + " force attach to "+cSeatInfo.GetNetInfo();
            }
            //NetLog.PlayerDriver( sLog );                    
        }
    }
    
    
    boolean GetOut()
    {
        //jak wysiadamy z pozycji pasazera "obiboka" to kierunek
        //po GetOut mozemy miec utrzymany
        Vector vCameraDir = null;
        if ( bNetIsOwner && (Camera != null) &&
            (cSeatInfo != null) && (cSeatInfo.GetRole() == EPassengerRole._VEHICLE_PASSENGER_IDLE) )
        {
            vCameraDir = Camera.GetForwardVector();
            vCameraDir.Negate();
            vCameraDir.fY = 0f;
		    vCameraDir.Normalize();
        }
        
        // musimy skwantowac wartosc LastPhysicsUpdate, bo pojazdy moga nie uzywac stalego ziarna
        SetPhysicsLastUpdateTime(RoundToPhysicsGranularity(GetPhysicsLastUpdateTime()));
        if ( super.GetOut() )
        {
            SetPhysicsLastUpdateTime(RoundToPhysicsGranularity(GetPhysicsLastUpdateTime()));
            m_nState = _STATE_SPAWNED;
            CleanPlayerMotion();
            SetMotionMoveFromKeyState();
            if ( vCameraDir != null )
                SetDir( vCameraDir );
            SetCorrectPosOnRender( true );
            UpdateMotion();
            return true;
        }
        return false;
    }

    boolean SeatInfoCanGetOut()
    {
        boolean bRet = super.SeatInfoCanGetOut();
        
        if ( (bRet == false) && m_bGetInOutFromServer )
        {
            NetLog.PlayerDriver("!! GET_OUT SeatInfoCanGetOut() forced\n");
            bRet = true;
        }
        return bRet;        
    }
    

        
    Vector GetCheckedPosToGetOut(ISeatVehicle cVehicle,SeatInfo cSeat)
    {
        Vector vRet = super.GetCheckedPosToGetOut( cVehicle, cSeat );
        if ( (vRet == null) && m_bGetInOutFromServer )
        {
            NetLog.PlayerDriver("!! GET_OUT GetCheckedPosToGetOut() forced\n");
            vRet = GetPosition();
        }
        return vRet;
    }
    
    void SetupAnimsAndCamerasToGetOut()
    {
        //jak gra jest kasowana to nie wolamy tej procki
        if ( ModuleNet.IsGameGoing() )
            super.SetupAnimsAndCamerasToGetOut();
    }
    
    boolean UpdateMotionCanPlayAnim()
    {
        // mozna odtwarzac animacje tylko, jesli nie prowadzi pojazdu
        return (m_nState != _STATE_IN_VEHICLE) && super.UpdateMotionCanPlayAnim();
    }
    
    public void OnVehicleCollision(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed)
	{
        if ( Role < ERole._AUTHORITY )
        {
            ClearFlag(EFlags._BLOCK_MOVE);
            MsgSendServerVehicleCollision( Assassin ); // wysylamy pytanie, czy zabilismy kolesia na serverze
        }
        
        OnTakeDamage(fDamage, fDamage, Assassin, vHitLocation, vSpeed, EBones._NON_BONE, EnumDamageType._BLAST);
    }
    
    /**
     * Metoda nadpisana, zeby zadac obrazenia
     * wszystkim pasazerom pojazdu w momencie jego rozwalenia
     */
    void VehicleCrashed()
	{
        PlayerNet cAssassin = null;
        float     fDamage   = -1;
        
        //sprawdzmy czy nie powinnismy graczowi zadac obrazen
        //podczas wysiadania z rozwalonego pojazdu
		if ( (Role == ERole._AUTHORITY) && sm_bDamageOnVehicleCrashed )
        {
            Vehicle cVehicle = null;
            
            if ( (cSeatInfo != null) &&
                 (cSeatInfo.GetOccupiedObject() instanceof Vehicle) )
            {
                cVehicle = (Vehicle)cSeatInfo.GetOccupiedObject();
                if (cVehicle != null)
                {
                    //liczymy jakie obrazenia zadac graczowi
                    fDamage   = fMaxHealth * cVehicle.GetOccupantsDamageMaxHealthMulOnCrash(); 
                    //oraz komu ewentualnie te obrazenia przypisac.
                    cAssassin = cVehicle.GetPlayerNetAssassin();
                }
            }
        }
        
        //teraz wolamy supera 
        //tam jest wysiadanie z pojazdu
        super.VehicleCrashed();
        
        //i na koncu jesli trzeba to zadajemy obliczone obrazenia
        if ( (Role == ERole._AUTHORITY) && (fDamage > 0.0f) )
        {
            //i zadajemy mu bol...
            TakeDamage( fDamage, fDamage, 
                        cAssassin, null, null, 
                        EBones._NON_BONE, EnumDamageType._BULLET );
        }
	}
    
    String GetDriverInfo()
    {
        return GetDriverInfo( cSeatInfo );
    }
    String GetDriverInfo(SeatInfo cSeat)
    {
        return GetNetInfo()+" cnt:"+GetDriverCounter()+
               ((cSeat!=null)?(" seat:"+cSeat.GetNetInfo()):"");
    }
    
    private String GetNetInfo(ISeatVehicle cSeatVehicle)
    {
        if ( cSeatVehicle instanceof Vehicle )
            return ((Vehicle)cSeatVehicle).GetNetInfo();
        return ""+cSeatVehicle;
    }
    
    void MsgSendServerVehicleCollision(GameObject Assassin)
    {
        MessageNew( MSG_SERVER_VEHICLE_COLL );
        MessagePackInt( Assassin.GetNetId() );
        MessageSend();            
    }
    
    void MsgRecvServerVehicleCollision()
    {
        int nSenderId  = MessageGetSenderId();
        int nAssasinId = MessageUnPackInt();
        MessageEnd();            
        
        m_fTryToCheckCollisionTimeOut  = GetTime() + 1.0f;
        m_nTryToCheckCollisionSenderId = nSenderId;
        m_cTryToCheckCollisionAssassin = GetObjectByNetId( nAssasinId );
        TryToCheckCollision();
    }
    
    float m_fTryToCheckCollisionTimeOut       = 0.0f;
    int   m_nTryToCheckCollisionSenderId      = -1;
    GameObject m_cTryToCheckCollisionAssassin = null;
    
    void TryToCheckCollision()
    {
        //jezeli bez ode to
        //weryfikacja m_cTryToCheckCollisionAssassin
        //jesli zweryfikowano to takadamage
        
        if ( fHealth <= 0f )
        {
            MsgSendClientVehicleCollisionAnswer( m_nTryToCheckCollisionSenderId, true ); // gosc zginal na serverze
            return;
        }
        
        if ( GetTime() < m_fTryToCheckCollisionTimeOut )
            CallMethodIn("TryToCheckCollision", 0.1f);
        else
            MsgSendClientVehicleCollisionAnswer( m_nTryToCheckCollisionSenderId, false ); // gosc nie zginal na serverze
    }
    
    void MsgSendClientVehicleCollisionAnswer( int nSenderId, boolean bPlayerDied )
    {
        MessageNew( MSG_CLIENT_VEHICLE_COLL_ANSW );
        MessagePackBoolean( bPlayerDied );
        MessageSendHost( nSenderId );            
    }
    
    void MsgRecvClientVehicleCollisionAnswer()
    {
        boolean bPlayerDied = MessageUnPackBoolean();
        MessageEnd();      
        
        if (!bPlayerDied)
            SetFlag(EFlags._BLOCK_MOVE);
    }
}
