/**
 * Player obslugujacy efekty nad jego glowa (badz nad pojazdem
 * w ktorym sie znajduje). 
 * Za pomoca tych efektow mozna rozpoznac druzyne gracza,
 * oraz stwierdzic ze on wlasnie uzywa voice'a (w przyszlosci moze chata?)
 */
public class PlayerNetBaseIndicator extends PlayerNetBase
{
    /**
     * Czy gramy w tryb druzynowy?
     * To nam jest potrzebne zeby w ogole stwierdzic czy bedziemy
     * sie bawic w efekty nad glowami
     */
    private 
    boolean m_bTeamMode = false;
        
    
    /**
     * FX jaki bedziemy odpalali dla graczy
     * w zaleznosci od druzyny
     */
    String m_sIndicatorFXBlue = "FXPlayerNetIndicatorBlue";
    String m_sIndicatorFXRed  = "FXPlayerNetIndicatorRed";
    
    /**
     * Obiekt do pozycjonowania tego FX'a
     */
    NetSpritePositioner m_cIndicatorFXPositioner = null;
        
    int METHOD_IndicatorFX          = 0 ;

    public void OnCreate()
    {
        METHOD_IndicatorFX          = RegisterCallMethod( "IndicatorFX" );

        super.OnCreate();
        
        m_cIndicatorFXPositioner = (NetSpritePositioner)CreateObject("NetSpritePositioner");
        m_cIndicatorFXPositioner.ActivateOnModule();
        AttachChild( m_cIndicatorFXPositioner );
        UpdateIndicatorFXPositioner();
    }
    
    private String m_sNickForIndicator = " ";
    String GetNickForIndicator(){ return m_sNickForIndicator; }
    
    void SetName( String sName )
    {
        super.SetName( sName );
        UpdateNickForIndicator();
    }
    
    void OnPostCreate()
    {
        super.OnPostCreate();
        UpdateNickForIndicator();
    }
    
    void UpdateNickForIndicator()
    {
        m_sNickForIndicator = GetName();
        //skrocic
        if ( m_cIndicatorFXPositioner != null )
             m_cIndicatorFXPositioner.SetText( m_sNickForIndicator );
        
        //jesli siedzimy w pojezdzie to zaktualizujmy
        //tez tam nick, moze to nasz sie wyswietla
        if ( (cSeatInfo != null) &&
             (cSeatInfo.GetOccupiedObject() instanceof Vehicle) 
           )
            {
                Vehicle cVehicle = (Vehicle)cSeatInfo.GetOccupiedObject();
                if ( cVehicle != null )
                    cVehicle.UpdateNickForIndicator();
            }
    }
        
    
    void UpdateIndicatorFXPositioner()
    {
        if ( m_cIndicatorFXPositioner != null )
             m_cIndicatorFXPositioner.SetPosition( new Vector( 0.0f, FXPlayerNetIndicator._HEIGHT, 0.0f ) );
    }
    
    /**
     * Flaga mowiaca czy gracz jest w takim stanie (czy jest zywy) 
     * ze moze pojawiac sie nad nim idndykator
     */
    boolean m_bReadyForIndicator = false;
                      
    /** Zmiana stanu gracza ze wzgledu na 
     * mozliwosc "noszenia" identyfikatora
     */
    void ReadyForIndicator(boolean bReady)
    {
        m_bReadyForIndicator = bReady;
        //po kazdej zmianie stanu musimy sprawdzic
        //co z naszym indykatorem
        CheckIndicatorState();
    }
    
    /**
     * Metoda sprawdza jaki powinien byc stan indykatora
     */
    void CheckIndicatorState()
    {
        if ( bNetIsOwner )
            return;
        
        int nTeamId = (GetGameInfo() != null) ? 
                       GetGameInfo().GetTeamIndicator( GetPlayerInfo() ): NetTeamInfo.NONE;
        
        //jesli jestesmy gotowi na idnykator i dostalismy od systemu jakis
        //to go wyswietlamy, jesli nie to chowamy
        if ( m_bReadyForIndicator && (nTeamId != NetTeamInfo.NONE) )
            StartIndicator( nTeamId );
        else
            StopIndicator();                          
    }
    
    void StartIndicator(int nTeamId)
    {
        //sprawdzmy jaki FX bedzie trzeba odpalic
        String sFX = (nTeamId == NetTeamInfo.BLUE)? 
                      m_sIndicatorFXBlue : m_sIndicatorFXRed;
        
        //czy jestesmy w pojezdzie?
        if ((cSeatInfo != null) && (cSeatInfo.GetOccupiedObject() instanceof Vehicle))
        {
            //tak,
            
            //zatem sobie bezwzglednie wylaczamy
            StopIndicatorFX();
            
            //i sprawdzmy co to za pojazd
            Vehicle cVehicle = (Vehicle)cSeatInfo.GetOccupiedObject();
            
            //wlaczymy mu indykator jak nie ma w nim gracza glownego
            PlayerNet cMainPlayer = (GetGameInfo()!=null)?GetGameInfo().GetMyPlayerNet():null;
            if ( (cMainPlayer                               !=          null) &&
                 (cMainPlayer.cSeatInfo                     !=          null) &&
                 (cMainPlayer.cSeatInfo.GetOccupiedObject() instanceof Vehicle) )
            {
                //w jakim pojezdzie siedzi gracz glowny?
                Vehicle cMainPlayerVehicle = (Vehicle)cMainPlayer.cSeatInfo.GetOccupiedObject();
                if ( cMainPlayerVehicle == cVehicle )
                {
                    //w tym samym co my, wiec wylaczamy bezwzlglednie efekt i spadamy
                    cVehicle.StopIndicatorFX();
                    return;
                }
            }
            //nie ma gracza glownego w naszym pojezdzie
            
            //jezeli nie jest wlaczony to wlaczamy
            if ( ! cVehicle.IsIndicatorActive() )
                cVehicle.StartIndicatorFX( sFX );
        }
        else
        {
            //wlaczamy sobie lampke nad glowa
            StartIndicatorFX( sFX );
        }
    }
    
    void StopIndicator()
    {
        //wylaczamy sobie efekt
        StopIndicatorFX();
        
        //czy jestesmy w pojezdzie?
        if ((cSeatInfo != null) && (cSeatInfo.GetOccupiedObject() instanceof Vehicle))
        {
            //tak, jemu tez wylaczymy
            Vehicle cVehicle = (Vehicle)cSeatInfo.GetOccupiedObject();
            cVehicle.StopIndicatorFX();
        }
    }
    
    
    boolean m_bIndicatorActive = false;
    
    private void StartIndicatorFX(String sFX)
    {
        m_sIndicatorFX      = sFX;
        m_bIndicatorActive  = true;
        CallMethodEvery( METHOD_IndicatorFX , (float)((FXPlayerNetIndicator._DURATION-10.0)/1000.0) );
    }
    
    private void StopIndicatorFX()
    {
        m_bIndicatorActive  = false;
        DisableCallMethod( METHOD_IndicatorFX );
    }
    
    String m_sIndicatorFX  = null;
    Vector m_vIndicatorPos = new Vector(0,0,0);
    
    
    boolean bTestAllowIndicators = true;
    
    void IndicatorFX()
    {
        if ( !bTestAllowIndicators )
            return;
        m_cIndicatorFXPositioner.ExecuteFX( m_cIndicatorFXPositioner, 
                                            m_sIndicatorFX , 
                                            m_vIndicatorPos );
    }
    
   
    void SetNetGameInfo( NetGameInfo cGameInfo )
    {
        super.SetNetGameInfo( cGameInfo );
                
        CheckIndicatorState();
    }
    
  
        
    public void Respawn()
    {
        super.Respawn();
        ReadyForIndicator( true );
    }
    
    boolean GetOut()    
    {
        SeatInfo cSeat = cSeatInfo;
        boolean bRet = super.GetOut();
        if ( bRet )
        {
            if ((cSeat != null) && (cSeat.GetOccupiedObject() instanceof Vehicle))
            {
                Vehicle cVehicle = (Vehicle)cSeat.GetOccupiedObject();
                if ( cVehicle.GetAllOccupantsNum() <= 0 )
                    cVehicle.StopIndicatorFX();
                else
                {
                    //jezeli jestesmy graczem glownym to chociaz jeden
                    //z pozostalych w pojezdzie pasazerow powinien sprawdzic
                    //czy nie wlaczyc teraz indykatora
                    if ( bNetIsOwner )
                    {
                        Pawn []aPawns = cVehicle.GetAllOccupants();
                        if ( aPawns[0] instanceof PlayerNet )
                        {
                            PlayerNet cPlayer = (PlayerNet)aPawns[0];
                            cPlayer.CheckIndicatorState();
                        }
                    }
                }
            }
            
            CallOnFrameEnd("CheckIndicatorState");
        }
        return bRet;
    }
        
    
    void CleanPlayer()
    {
        super.CleanPlayer();
        ReadyForIndicator( false );
    }
    
    void PlayerDied(int nMode,int nParam)
    {
        ReadyForIndicator( false );
    }
    
    void GetIn(SeatInfo cSeatInfo)    
    {
        super.GetIn( cSeatInfo );
        if ( bNetIsOwner )
            StopIndicator();            
        else            
            CallOnFrameEnd("CheckIndicatorState");
    }
    
    

    void JustSpoken()
    {
        super.JustSpoken();
        
        if ( m_bIndicatorActive )
            SpokenFX();
        else
        {
            if ((cSeatInfo != null) && (cSeatInfo.GetOccupiedObject() instanceof Vehicle))
            {
                Vehicle cVehicle = (Vehicle)cSeatInfo.GetOccupiedObject();
                if ( cVehicle.IsIndicatorActive() )
                     cVehicle.SpokenFX();
            }        
        }
    }
    
    
    void SpokenFX()
    {
            ExecuteFX( m_cIndicatorFXPositioner, 
                       m_sIndicatorFX+"Spoken" , 
                       new Vector(  m_vIndicatorPos.fX, 
                                    m_vIndicatorPos.fY+FXPlayerNetIndicator._SPOKEN_ADD_HEIGHT,
                                    m_vIndicatorPos.fZ )
                      );
    }
    

}
