public class PlayerNetBaseMortal extends PlayerNetBaseIndicator
{
    PlayerNetBaseMortal thisPlayerNetBaseMortal = this;
    
    /**
    * przez jaki czas po respawnie gracz ma byc niesmiertelny
    */
    static float   sm_fImmortalTimeAfterRespawn    = 3.0f;

    /**
     * czy mozna odwolac akcje respawn i cofnac sie do InventoryRoom
     */ 
    static boolean sm_bActionRespawnCanBeCanceled  = true;
        
    /**
     * czy akcje w czasie smierci (spawnu i wejscia do inwentory)
     * maja byc blokowane do czasu zweryfikowania zawartosci inwentory
     */
    static boolean sm_bLockActionsInDieState       = true;
        
    /**
     * czy zablokowana akcja spawnu moze zostac odblokowana
     * po pewnym czasie mimo iz zawartosc inwentory nie zostala
     * jeszcze zweryfikowana
     * TODO: fixit, potrzebne na szybko tylko do dema.
     */
    static boolean sm_bForceAllowActionsInDieState = true;
    
    
    /**
     * Informacje o Damage'u, ktore bedziemy przekazywali
     * trupkowi po smierci
     */
    private
    DamageInfo m_cDamageInfo = new DamageInfo();
    DamageInfo GetDamageInfo(){ return m_cDamageInfo; }
 
    /** cialo trupka playera
     */
    private
    DeadBody      m_cDeadBody = null;
    DeadBody      GetDeadBody(){ return m_cDeadBody; }
    void          SetDeadBody(DeadBody cDeadBody){ m_cDeadBody = cDeadBody; }
    
    /** SERVER 
     *  Gracz, ktory zadal nam ostatnio rany 
     */
    private 
    PlayerNetBaseMortal m_cAssassin = null;
    
    /** czas ostatniego efektu postrzalu
     */
    private
    float m_fLastTakeDamageSpecialEffectsTime = 0.f;
    
    
    int METHOD_StopHitReaction              = 0;
    int METHOD_ClearImmortalAfterRespawn    = 0;
    
    PlayerNetBaseMortal()
    {
        super();
        
        //maksymalne zdrowie gracza        
        fMaxHealth = 200.0f;
        
        //wspolczynniki Damage'u, w oryginale bylo: { 1.f, 3.f, .6f, .6f};
        aDamageMultipliers[ BODY_PART_TORSO ] = 1.0f;
        aDamageMultipliers[ BODY_PART_HEAD  ] = 2.5f;
        aDamageMultipliers[ BODY_PART_LEG   ] = 0.7f;
        aDamageMultipliers[ BODY_PART_ARM   ] = 0.8f;
    }
    
    public void OnCreate()
    {
        METHOD_StopHitReaction           = RegisterCallMethod("StopHitReaction");
        METHOD_ClearImmortalAfterRespawn = RegisterCallMethod("ClearImmortalAfterRespawn");
        
        super.OnCreate();

        if (Role == ERole._AUTHORITY) // SERVER
        {
            // stwozmy obiekt trupka
            m_cDeadBody =(DeadBody)CreateObjectNoInit("DeadBody");
            m_cDeadBody.ServerInit( this );
        }
        
        //fizyka smierci bedzie wlaczona jesli juz to w trupku
        bUsePhysicsToDeathFall = false;
    }
    
    public void OnDestroy()
    {
        super.OnDestroy();
        
        if ( m_cDeadBody != null )
        {
            m_cDeadBody.delete();
            m_cDeadBody = null;
        }
        
        thisPlayerNetBaseMortal = null;
        m_cDamageInfo           = null;
        m_cAssassin             = null;
    }

    void DeactivateOnActivateDeadBody()
    {
        DeactivateOnModuleAlways();
        TermovisionOff();
        ImplantTermovision.RemoveFromSystem( this );
    }

    
    public void Respawn()
    {
        super.Respawn();
        
        bDead       = false;
        m_bJustDied = false;
        fHealth     = fMaxHealth;
        RefreshHealth();
        
        m_cAssassin = null;
        
        if (bNetIsOwner && m_cDeadBody != null)
            m_cDeadBody.OnPlayerRespawn();
    }
    
    /**SERVER Obsluga handlera odbierajacego energiie.
     */
    public void OnTakeDamage(float fDamage, float fPhSimDamage,  GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
    {
        if (Role < ERole._AUTHORITY)
        {
            // wolamy odebranie energii, ale ono odpali tylko efekty
            super.OnTakeDamage(fDamage, fPhSimDamage, Assassin, vHitLocation, vSpeed, nMeshElementorBoneID, nDamageType);
            
			return;
        }
        else
        {
            m_cDamageInfo.Set( fPhSimDamage, vHitLocation, vSpeed, nMeshElementorBoneID, nDamageType);
		    // ustalamy najpierw, kto w nas strzelil, jezeli to bedzie strzal smiertelny
		    // to automatycznie zostanie wyslana informacja kto nas zabil
		    m_cAssassin = ModuleNet.GetAssassinPlayerNetFromObject( Assassin );
            
		    // zapamietajmy nasz stan zdrowia
		    float fLastHealth = fHealth;
		
		    super.OnTakeDamage(fDamage, fPhSimDamage, Assassin, vHitLocation, vSpeed, nMeshElementorBoneID, nDamageType);

		    // jezeli zdrowie sie zmienilo, powiadommy o tym klienta wlasciciela
		    if ( fLastHealth != fHealth )
		    {
                Vector vAssassinPos = new Vector();
                if (Assassin instanceof ControlObject)
                    vAssassinPos = ((ControlObject)Assassin).GetPosition();
                
                //TODO: doslac gosciowi jeszcze kierunek postrzalu
		    	MsgSendClientHealth(fDamage, vAssassinPos, vSpeed);
		    }
            
            if (( fHealth > 0 ) && (m_cAssassin != null))
                GetGameInfo().EventPlayersDamaged( this, m_cAssassin );
        }
	}

    // metoda wolana z OnTakeDamage playera
    void TakeDamageSpecialEffects(float fDamage, Vector vAssassinPos, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
    {
        m_fLastTakeDamageSpecialEffectsTime = GetTime();
        
        super.TakeDamageSpecialEffects(fDamage, vAssassinPos, vHitLocation, vSpeed, nMeshElementorBoneID, nDamageType);
        
        if (( bNetIsOwner || IsLocalAutoControl()) && (m_nState != _STATE_IN_VEHICLE))
        {
            HitReaction( vSpeed );
        }
    }
    
    /**
     * Reakcja na postrzal:
     * -zwolnienie na jakis czas
     * -animacja odrzutu
     * -trzesienie kamery na podstawie damage'a
     */
    void HitReaction(Vector vHitDir)
    {
        // sprawdz, jaka animacje odpalic (zalezne od kierunku postrzalu)
        nHitReaction = BaseAnimations.A_TAKE_HIT_FRONT;
        if (vHitDir != null)
        {
            vHitDir.Normalize();
            vHitDir.Negate();
            float fFwdDot = vHitDir.Dot(GetForwardVectorVolatile());
            if (fFwdDot > NewAI._COS_2_PI_BY_8)
                nHitReaction = BaseAnimations.A_TAKE_HIT_FRONT;
            else if (fFwdDot < NewAI._COS_6_PI_BY_8)
                nHitReaction = BaseAnimations.A_TAKE_HIT_BACK;
            else if (vHitDir.Dot( GetRightVectorVolatile()) > 0 )
                nHitReaction = BaseAnimations.A_TAKE_HIT_LEFT;
            else
                nHitReaction = BaseAnimations.A_TAKE_HIT_RIGHT;
        }
        
        StartSlowDownOnRecoil();
        
        float fTime = GetAnimMorphTime(0)+Anims.GetLength( nHitReaction );
        CallMethodIn( METHOD_StopHitReaction, fTime );
    }
    
    void StopHitReaction()
    {
        if ( nHitReaction > 0 )
        {
            nHitReaction = 0;
            UpdateMotion();
        }
    }
    
    void StopSlowDown()
    {
        StopHitReaction();
        super.StopSlowDown();
    }
    
        
    ///////////////////////////////////////////////////////////////////////////////
    // DIE STATE - przeciazony z Player
    ///////////////////////////////////////////////////////////////////////////////
    
    int m_nDieStateMode  = EPlayerAction.DIE_MODE_NORMAL;
    int m_nDieStateParam = 0; 
    
    /**
     * Akcje wykonywane w stanie smierci ( respawn / inventory room )
     * na kliencie beda domyslnie zablokowane, az do czasy gdy
     * otrzymamy serwera liste inwentory oraz ta lista bedzie sie
     * zgadzala z tym co jest u nas (wszystkie obiekty do nas "splyna")
     * Ta flaga mowi czy aktualnie mamy pozwolenie na wykonania
     * jakis akcji, czy nie
     */
    private boolean m_bAllowActionsInDieState = false;
    
    /**
     * Jako, ze sprawdzanie zawartosci inwentory nie dziala
     * jeszcze poprawnie, do dema wpuszczamy gracza do gry
     * po pewnym czasie, mimo ze nie zatwierdzono zawartosci
     * jego inwentarza. Wywolanie tej metody otwiera furtke
     * do spawnu. TODO: tylko do dema
     */
    void ForceAllowActionsInDieState()
    {
        //zapamietajmy sobie stan logow
        int nLastMode = NetLog.GetMode( NetLog._TYPE_INVENTORY_OBJECT );
        //ustawmy na crashlogi
        NetLog.SetMode( NetLog._TYPE_INVENTORY_OBJECT, NetLog._MODE_CRASHLOG );
        //zawolajmy ta metode, to nam sie posypia logi
        CheckInventoryContentAlert();
        //i przywrocmy stan logow
        NetLog.SetMode( NetLog._TYPE_INVENTORY_OBJECT, nLastMode );
        
        //teraz juz nam sie uda wyspawnowac
        m_bAllowActionsInDieState = true;
        //zapomnimy tez o tej metodzie
        DisableCallMethod( "CheckInventoryContentAlert" );
    }
    
    //ale fajna nazwa zmiennej ;-)
    boolean m_bScheduleForceAllowActionsInDieStateOnInitFromServer = false;
        
    void ScheduleForceAllowActionsInDieState(float fTime)
    {
        //jesli juz jestesmy zainicjowani to jazda,
        //jesli nie to czekamy az serwer nam
        if ( IsInitializedFromServer() )
            CallMethodIn( "ForceAllowActionsInDieState", fTime );
        else
            m_bScheduleForceAllowActionsInDieStateOnInitFromServer = true;
    }
    
    void MsgRecvClientState(boolean bOnInit)
    {
        super.MsgRecvClientState( bOnInit );
        
        if ( m_bScheduleForceAllowActionsInDieStateOnInitFromServer )
        {
            m_bScheduleForceAllowActionsInDieStateOnInitFromServer = false;
            //jesli dopiero dostalismy inicjalizacje gracza, to dajemy
            //az 30 sekund na "splyniecie" wszystkich obiektow do inwentory
            ScheduleForceAllowActionsInDieState( 30.0f );
        }
    }
    
    void InventoryContentChecked()
    {
        super.InventoryContentChecked();
        m_bAllowActionsInDieState = true;
        DisableCallMethod( "ForceAllowActionsInDieState" );
    }
    
    /**
     * Metoda do nadpisania w PlayetNetBaseInventory.
     * Wolana po pewnym czasie od stanu smierci gdy jeszcze
     * nie mamy dobrzego inwentory. Bedzie ona mogla sprawdzac
     * dlaczego tak jest i ew. podjac jakies kroki aby temu zaradzic 
     */
    void CheckInventoryContentAlert()
    {
        
    }

    //jakie akcje mamy wykonac jak juz dostaniemy pozwolenie na akcje w DieState
    boolean m_bForceInventoryOnEnableActions = false;
    boolean m_bForceSpawnOnEnableActions     = false;        
    
    //czy probowano sie juz wyspalnowac
    boolean m_bActionRespawnRequested  = false;
    //boolean WasActionRespawnRequested(){ return m_bActionRespawnRequested; }
    
    //czy probowano juz odwolac request spawnowania
    boolean m_bActionRespawnCanceled   = false;
    
    //czy probowano sie juz wyspawnowac , info potrzebne dla NetGameInfo,
    //zeby wiedzialo, ze nie nalezy juz wyswietlac okna powitalnego (jak byl juz spawn)
    boolean WasPressedFireToRespawn()
    { 
        return (m_bActionRespawnRequested || 
                m_bForceSpawnOnEnableActions ||
                m_bForceInventoryOnEnableActions); }
        
    public class DieState extends PlayerState
    {
        
        
        public void OnEnter()
        {
            super.OnEnter();
            
            bDead        = true;
            m_nState     = _STATE_DIE;
            
            // "czyscimy" gracza
            CleanPlayer();
                        
            //Uwaga! DeactivateOnModule() i inne tego typu akcje teraz jest w 
            //DeadBody::Activate() zeby obie akcje "odegraly" sie rownoczesnie!
            
            HideHud();
            
            //RestartInventoryRoom();
            
            if ( Role == ERole._AUTHORITY && ModuleNet.IsGameGoing() )
                FillInventory();
            
            m_bActionRespawnRequested        = false;
            m_bActionRespawnCanceled         = false;
            m_bForceInventoryOnEnableActions = false;
            m_bForceSpawnOnEnableActions     = false;                    
            
            if ( Role == ERole._AUTHORITY || ( !sm_bLockActionsInDieState ) )
                m_bAllowActionsInDieState = true;                
            else
            {
                m_bAllowActionsInDieState = false;
                if ( bNetIsOwner )
                {
                    CheckInventoryContent();
                    //po 5 sekundach podniesiemy alarm i bedziemy serwera
                    //notyfikowac o naszych problemach
                    CallMethodIn( "CheckInventoryContentAlert", 5.0f );
                    
                    if ( sm_bForceAllowActionsInDieState )
                    {
                        //normalnie dajemy 15 sekund na "splyniecie" 
                        //wszystkich obiektow do inwentory
                        ScheduleForceAllowActionsInDieState( 15.0f );
                    }
                }
            }
        }
        
        void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
        {
            if ( bNetIsOwner && bKeyState )
            {
                //logi da sie wlaczyc zawsze, a ze w trybie smierci mamy
                //wylaczone hudy (nie odczytuja klawiszy) to musimy to zrobic tu
                if ( nKeyCode == EGameControl.LOGS && HUD != null )
                    HUD.SwitchHudLogs();
                    
                if ( m_nDieStateMode != EPlayerAction.DIE_MODE_GAMEOVER )
                {
                    if ( nKeyCode == EGameControl.FIRE) 
                    {
                        //wcisnieto fire, zeby sie wyspawnowac
                        OnSpawnPressed();
                    } 
                    else 
                    if ( (nKeyCode == EGameControl.INVENTORY) &&
                         ( GetGameInfo() != null ) &&
                         ( GetGameInfo().IsInventoryRoomAvailable() ) )
                    {
                        //wcisnieto Inventory
                        OnInventoryPressed();
                    }
                }
            }
        }
        
        /**
         * Wcisnieto fire, zeby wyspawnowac
         */
        void OnSpawnPressed()
        {
            //jezeli jestesmy w trybie wyboru druzyny, to robimy team balance
            if ( GetGameInfo() != null && GetGameInfo().IsChangeTeamMode() )
            { 
                GetGameInfo().LocalEventRequestChangeTeam();
                return ; 
            }
            
            //jezeli mamy pozwolenie na jakokolwiek akcje
            if ( m_bAllowActionsInDieState )
            {
                //zapamietujemy fakt zadania spawnu
                m_bActionRespawnRequested = true;
                //i wolamy akcje spawnu
                ActionRespawn();
            }
            else
            {
                //nie mozemy jeszcze wywolac tej akcji
                
                //czy wczesniej chcielismy juz wymusic akcje spawn
                if ( ! m_bForceSpawnOnEnableActions )
                {
                    //jeszcze nie, wiec zapamietajmy ten fakt
                    m_bForceSpawnOnEnableActions = true;
                    //i rozpocznijmy sprawdzanie kiedy bedzie mozna to zrobic
                    StartCheckAllowActionsInDieState();
                }
            }
        }
        
        void OnInventoryPressed()
        {
            //jezeli jestesmy w trybie wyboru druzyny, to robimy team balance
            if ( GetGameInfo() != null && GetGameInfo().IsChangeTeamMode() )
            { 
                GetGameInfo().LocalEventRequestChangeTeam();
                return ; 
            }
                 
            //jezeli mamy pozwolenie na jakokolwiek akcje
            if ( m_bAllowActionsInDieState )
            {
                //jesli juz wczesniej chcielismy sie spawnowac
                if ( m_bActionRespawnRequested )
                {
                    // tu mozna ewentualnie odwolac chec spawnu (moze zdarzymy)
                    if ( (! m_bActionRespawnCanceled ) && sm_bActionRespawnCanBeCanceled )
                    {
                        //na razie dopuszczamy tylko jednoktotnie taka akcje
                        //TODO: zrobic wielokrotny cancel - musi byc w nim
                        //obsluga InventoryContent i wlaczanie m_bAllowActionsInDieState
                        ActionRespawnCancel();
                    }
                    //po to, aby drugi raz nie dalo sie wejsc do inwentory
                    m_bActionRespawnCanceled = true;
                }
                else
                {
                    //to pierwszy raz
                    //zapamietujemy to
                    m_bActionRespawnRequested = true;
                    //i wchodzimy sobie lokalnie do inwentory room
                    SetInventoryMenu();
                }
            }
            else
            {
                //nie mozemy jeszcze wywolac tej akcji
                
                //czy wczesniej chcielismy juz wymusic akcje inventory
                if ( ! m_bForceInventoryOnEnableActions )
                {
                    //jeszcze nie, wiec zapamietajmy ten fakt
                    m_bForceInventoryOnEnableActions = true;
                    //i rozpocznijmy sprawdzanie kiedy bedzie mozna to zrobic
                    StartCheckAllowActionsInDieState();
                }
            }
        }
        
        /**
         * Metoda rozpoczynajaca sprawdzanie kiedy bedzie mozna
         * wywolac bezpiecznie jakas akcje (spawn/inwentory room)
         * Przy okazji pojawia sie hud z napisem "Please wait..."
         */
        void StartCheckAllowActionsInDieState()
        {
            StopCheckAllowActionsInDieState();
            CallMethodEvery( "CheckAllowActionsInDieState", 0.1f );
            
            ShowHUDNetOnPlaeseWait();
        }
        
        void StopCheckAllowActionsInDieState()
        {
            DisableCallMethod( "CheckAllowActionsInDieState" );
        }
        
        /**
         * Metoda czy mozna juz wywolac akcje spawn i/lub inwentory room
         * Jesli tak to konczy swoje (cykliczne) dzialanie i 
         * sprawdza czy mamy do wywolania jakas akcje (wymuszana wczesniej)
         */
        void CheckAllowActionsInDieState()
        {
            if ( m_bAllowActionsInDieState )
            {
                StopCheckAllowActionsInDieState();
                
                CheckForceAction();
            }
        }
        
        /**
         * Sprawdza czy wczesniej chcielismy wywolac jakas akcje,
         * jesli tak to ja wywolujemy teraz
         */
        void CheckForceAction()
        {
            if ( m_bForceInventoryOnEnableActions )
            {
                m_bForceInventoryOnEnableActions = false;    
                m_bForceSpawnOnEnableActions     = false;
                OnInventoryPressed();
            }
            else if ( m_bForceSpawnOnEnableActions )
            {
                m_bForceSpawnOnEnableActions     = false;
                m_bForceInventoryOnEnableActions = false;    
                OnSpawnPressed();
            }
        }
        
        
        public void OnInputMouseButton(int nButton, int nState)
        {
            ProcessMouseButtons( nButton, nState );
        }

        public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
        {
        }
    }
    
    void MessagePackDieState()
    {
        MessagePackInt( m_nDieStateMode );
        MessagePackInt( m_nDieStateParam );
    }
    
    void MessageUnPackDieState()
    {
        m_nDieStateMode  = MessageUnPackInt();
        m_nDieStateParam = MessageUnPackInt();
    }
    
    /////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////
    // Obsluga gracza na serwerze  // GNS
    //////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////
    
    
    /** Mnoznik obrazen, bedzie uwzglednial takze FriendlyFire
     */
    float GetDamageMultiplier( int nMeshElementorBoneID )
    {
        float fDamageMul = super.GetDamageMultiplier( nMeshElementorBoneID );
                
        if ( GetGameInfo() == null)
        {
            LogD(" m_cModule "+GetModuleNet()+" m_cGameInfo "+GetGameInfo()+"\n");
            return fDamageMul;
        }
            
        float fFriendlyFireMul = GetGameInfo().GetDamageMultiplier( this, m_cAssassin );
        return fDamageMul * fFriendlyFireMul;
    }
    
    /** po to by wymusic smierc na graczu przy zmianie druzyny
     * TODO: pokryc na pusta metode w stanach smierci
     */
    
    void ForceDieByYourSelf()
    {
        ForceDie( EPlayerAction.DIE_MODE_NORMAL );
    }
    void ForceDieOnLeftGame()
    {
        ForceDie( EPlayerAction.DIE_MODE_LEFTGAME );
    }
    void ForceDieOnChangeTeam()    
    {        
        ForceDie( EPlayerAction.DIE_MODE_CHANGETEAM ); 
    }
    void ForceDieOnGameOver()
    {
        ForceDie( EPlayerAction.DIE_MODE_GAMEOVER );
    }
    void ForceDieOnGameRestart()
    {
        ForceDie( EPlayerAction.DIE_MODE_GAMERESTART );
    }
    void ForceDieByVehiclePod()
    {
        m_cAssassin   = null;
        m_cDamageInfo = new DamageInfo( GetPosition() );
        Die( EPlayerAction.DIE_MODE_NORMAL );
    }
    void ForceDieByConsoleCommand()
    {
        m_cAssassin   = null;
        m_cDamageInfo = new DamageInfo( GetPosition() );
        Die( EPlayerAction.DIE_MODE_NORMAL );
    }
    void ForceDie(int nMode)
    {
        m_cAssassin   = this;
        m_cDamageInfo = new DamageInfo( GetPosition() );
        Die( nMode );
    }
    
    /**SERVER Wolane gdy gracz umiera. Moze zdazyc sie tylko na serwerze.
     */
    
    boolean m_bJustDied = false;
    
    public void Die() 
    {
        Die( EPlayerAction.DIE_MODE_NORMAL );
    }
    public void Die(int nMode) 
    {
        // ta metoda moze dzialac tylko na serwerze !!!!!!
        if (Role < ERole._AUTHORITY) 
			return;
        
        // zliczanie fragow w netgameinfo
        if (sCurrentState.compareTo("DieState") != 0)
        {
            m_bJustDied = true;
            GetGameInfo().EventPlayerDied( this, m_cAssassin, nMode );
        }
        
        
        //domyslnie parametrem jest losowa liczba do losowania animacji
        int nParam = RandomInt( 100 );
        if ( nMode == EPlayerAction.DIE_MODE_GAMEOVER ||
             nMode == EPlayerAction.DIE_MODE_GAMERESTART )
        {
            //w tych trybach przekazujemy jednak id winnera
            nParam = GetGameInfo().GetWinnerId();
        }
        
        //usmiercmy gracza (tu wypelni sie juz nowe inwentory)
        PlayerDied( nMode, nParam );
        //zapamietajmy jego stan
        SetStateFromServer( _STATE_DIE, GetTime() );
        // i powiedzmy o wszystkim klientom              
        MsgSendClientPlayerDied( nMode, nParam, GetRespawnCnt(), GetTime() );
        // do wlasciciela ponadto wysylamy liste zawartosci inventory
        MsgSendClientInventoryContent();
    }
    
    /**
     * Metoda wymuszajaca smierc z aktualnei ustawionymi parametrami
     * Wolana po odebraniu komunikatu z serwera
     */
    void PlayerDied()
    {
        if ( m_cDeadBody != null )
        {
            //ustawmy trupka na naszej aktualnej pozycji, bowiem
            //jest to smierc wymuszona dodatkowo i trupek moze nie miec
            //w danej chwili zadnych konkretnych parametrow.
            m_cDeadBody.SetPositionWorld( GetPosition() );
            m_cDeadBody.SetDir( GetForwardVector() );
        }
        PlayerDied( m_nDieStateMode, m_nDieStateParam );
    }
      
    void PlayerDied(int nMode,int nParam)
    {
        super.PlayerDied( nMode, nParam );

        m_nDieStateMode  = nMode;
        m_nDieStateParam = nParam;
        
        if ( m_nState == _STATE_IN_VEHICLE )
            ActionGetOutDo();
        else
        if ( m_nState == _STATE_BINOCULARS )
            StopUsingBinocularsDo();
        else
        if ( m_nState == _STATE_AIMING )
            RestoreViewAfterAiming();
        
        if (Role == ERole._AUTHORITY) 
        {
            //jaka domyslna animacja?
            int     nAnim     = BaseAnimations.A_DEATH_FRONT;
            boolean bLoopAnim = false;
            
            switch ( m_nDieStateMode )
            {
            case EPlayerAction.DIE_MODE_GAMEOVER:
                int nWinnerId = nParam;
                if ( GetGameInfo() == null )
                    break;
                nAnim = GetGameInfo().IsPlayerWinner( GetPlayerInfo(), nWinnerId )?
                        BaseAnimations.A_GLADDEN_12:
                        BaseAnimations.A_IDLE;
                bLoopAnim = true;
                break;
        
            case EPlayerAction.DIE_MODE_GAMERESTART:
                /* TODO: dorobic takie samo ale dotyczace ostatnio zdebytego punktu
                if ( GetGameInfo() == null )
                    break;
                nAnim = GetGameInfo().IsPlayerWinner( GetPlayerInfo(), nWinnerId )?
                        BaseAnimations.A_GLADDEN_1:
                        BaseAnimations.A_IDLE;*/
                nAnim     = BaseAnimations.A_IDLE;
                bLoopAnim = true;
                break;
            
            //pewnie poprostu umarl                
            default:
                switch (motion.iSpeed)
                {
                    case MotionState.SPEED_SWIM : 
                    case MotionState.SPEED_CRAWL :                      
                         nAnim = BaseAnimations.A_DEATH_CRAWL;
                         break;
                    case MotionState.SPEED_DUCK :
                         nAnim = BaseAnimations.A_DEATH_DUCK;
                         break;
                    default:
                         int nRandom = nParam;
                         nAnim = nRandom < 60 ? BaseAnimations.A_DEATH_FRONT:
                                (nRandom < 80 ? BaseAnimations.A_DEATH_LEFT:
                                                BaseAnimations.A_DEATH_RIGHT );
                }
            }
            
            if ( sCurrentState.compareTo("DieState") != 0 )
            {
                // rozrzucamy inwentory playera wokol siebie
                RemoveInventoryOnDie( nMode );
                // udostepnienie trupka
                if (m_cDeadBody != null )
                    m_cDeadBody.ServerActivate( nMode, nAnim, bLoopAnim );
                FlushReplicationManagerBuffers();
            }
        }
        
        if ( m_cFloor != null )
            DetachPlayerFromFloorDo();
        //TODO: uwzglednic to tez przy replikacji stanu
        SwitchParentTo( null );
        
        if ( bNetIsOwner )
        {
            int nCameraMode = ModuleNet._DEATH_CAMERA_MODE_NORMAL;
                        
            switch ( nMode )
            {
            case EPlayerAction.DIE_MODE_CHANGETEAM:
                nCameraMode         = ModuleNet._DEATH_CAMERA_MODE_CHANGETEAM;
                GetGameInfo().ActivateStatisticsOnDeath( 0.0f );
                break;
            case EPlayerAction.DIE_MODE_GAMEOVER:
                nCameraMode         = ModuleNet._DEATH_CAMERA_MODE_GAMEOVER;
                GetGameInfo().ActivateStatisticsOnGameOver();
                break;
            case EPlayerAction.DIE_MODE_GAMERESTART:
                nCameraMode         = ModuleNet._DEATH_CAMERA_MODE_GAMERESTART;
                GetGameInfo().ActivateStatisticsOnGameRestart();
                break;
            default:
                fHealth = 0.0f;
                GetGameInfo().ActivateStatisticsOnDeath( 2.0f );
            }
                        
            fHealth = 0.0f;
            RefreshHealth();
        
            if (nCameraMode != ModuleNet._DEATH_CAMERA_MODE_NORMAL &&
                sCurrentState.compareTo("DieState") != 0 &&
                GetModuleNet() != null )    
            {
                m_bCanUpdateMotionOnCameraModeChanged = false;
                GetModuleNet().ActivatePlayerDeathCamera( thisPlayerNetBaseMortal, nCameraMode );
                m_bCanUpdateMotionOnCameraModeChanged = true;
            }
        }
        
        if (sCurrentState.compareTo("DieState") != 0)    
            SetState("DieState");
    }
    

    private boolean m_bCanUpdateMotionOnCameraModeChanged = true;
    boolean CanUpdateMotionOnCameraModeChanged(){ return m_bCanUpdateMotionOnCameraModeChanged; }
    
    boolean InitDeathCameraEyeDummy()
    {
        // na klientach nie wlascicelach i na serwerze jest dumik do smierci
        /*if( bNetIsOwner )
            return super.InitDeathCameraEyeDummy();*/
        //dummik jest potrzebny tylko w deadbody
        return false;
    }
    

    /**
     * Jesli gracz nie zyje, to "oczy" musimy wyciagnac 
     * z obiektu trupka, w przeciwnym razie bedzie
     * bez zmian, czyli zawolamy super.Method(...) 
     */
    
    // wyciagamy pozycje oka
    void GetEyePos(Vector vPos)
    {
        if ( bDead && m_cDeadBody != null )
            m_cDeadBody.GetEyePos( vPos );
        else
            super.GetEyePos( vPos );
    }
    
    // wyciagamy kierunek oka
    void GetEyeForwardVector(Vector vDir)
    {
        if ( bDead && m_cDeadBody != null )
            m_cDeadBody.GetEyeForwardVector( vDir );
        else
            super.GetEyeForwardVector( vDir );
    }
    
   
  	/**SERVER Wyslanie do clienta (wlasciciela) informacji o aktualnym stanie zdrowia
     */
    void MsgSendClientHealth(float fDamage, Vector vAssassinPos, Vector vSpeed)
    {
        MessageNew( MSG_CLIENT_HEALTH );
        MessagePackFloat( fHealth );
        MessagePackFloat( fDamage );
        if ( vAssassinPos == null )
        {
            vAssassinPos = GetPosition();
            //robie na szybko tak myk, bo nie wiem czy moze byc taka sama pozycja
            vAssassinPos.fX += 1.0f;
        }
        MessagePackVector( vAssassinPos );
        MessagePackVector( vSpeed != null ? vSpeed : Vector.Up );
        MessageSend( EMsgSendMode.OWNER );
    }
    
    /**SERVER Wyslanie do clienta (wlasciciela) informacji o aktualnym stanie overloada
     */
    void MsgSendClientOverload(float fTime,float fOverload)
    {
        MessageNew( MSG_CLIENT_OVERLOAD );
        MessagePackFloat( fTime );
        MessagePackFloat( fOverload );
        MessageSend( EMsgSendMode.OWNER );
    }

    
    static float _TIME_BETWEEN_DAMAGE_EFFECT = 0.1f;
    
    /**CLIENT Odbior stanu zdrowia
     */
	void MsgRecvClientHealth()
    {
        float  fNewHealth   = MessageUnPackFloat();
        float  fDamage      = MessageUnPackFloat();
        Vector vAssassinPos = MessageUnPackVector();
        Vector vSpeed       = MessageUnPackVector();
        MessageEnd();
        
		if ( fHealth != fNewHealth )
		{
			fHealth = fNewHealth;
			RefreshHealth();
            
            if ( GetTime() - m_fLastTakeDamageSpecialEffectsTime > _TIME_BETWEEN_DAMAGE_EFFECT )
            {
                // wlaczamy efekt trafienia
                TakeDamageSpecialEffects( fDamage, vAssassinPos, 
                                          Vector.Zero, vSpeed, 
                                          EBones._NON_BONE, 0 );
            }
		}
	}
    
    /**CLIENT Odbior stanu overload
     */
	void MsgRecvClientOverload()
    {
        float fTime     = MessageUnPackFloat();
        float fOverload = MessageUnPackFloat();
        MessageEnd();
		
        if ( GetImplantSystem() != null )
             GetImplantSystem().Update( fTime, fOverload );
	}
        
    
    void MsgSendServerSwitchUnderWater( boolean bUnderWaterState )
    {
        MessageNew( MSG_SERVER_SWITCH_UNDER_WATER );
        MessagePackBoolean( bUnderWaterState );
        MessageSend();
    }
    
    void MsgRecvServerSwitchUnderWater()
    {
        boolean bUnderWaterState = MessageUnPackBoolean();
        MessageEnd();
        
        if ( bUnderWater != bUnderWaterState )
            SetUnderWater( bUnderWaterState );
    }

    void SetUnderWater(boolean bUnder)
    {
        boolean bLastUnderWater = bUnderWater;
                                 
        super.SetUnderWater( bUnder );
        
        if (  bNetIsOwner &&
             (Role < ERole._AUTHORITY) &&
             (bLastUnderWater != bUnderWater) &&
             (!bDead) )
            MsgSendServerSwitchUnderWater( bUnderWater );
    }
    
    void Drown()
    {
        //pokryte z playera
        if ( Role == ERole._AUTHORITY )
        {
            TakeDamage(10, 0, null, null, null, EBones._NON_BONE, EnumDamageType._DROWNING);
        }
    }

    /**
     * Niesmiertelnosc na pewien czas po respawnie
     */
    
    /**czy gracz jest niesmiertelny*/
    boolean IsImmortal(){ return (super.IsImmortal() || m_bImmortalAfterRespawn); }
    
    /**czy gracz ma aktualnie ustawiona flage nietykalnosci bo wlasnie sie wyspawnowal   */ 
    private boolean m_bImmortalAfterRespawn = false;
    
    /**zaplanuj niesmiertelnosc gracza na pewien czas ( jesli tak ma byc )*/
    void ScheduleImmortalAfterRespawn()
    {
        if ( sm_fImmortalTimeAfterRespawn > 0.0f )
             SetImmortalAfterRespawn( sm_fImmortalTimeAfterRespawn );
    }
    
    /**ustaw niesmiertelnosc na podany czas*/
    void SetImmortalAfterRespawn( float fTime )
    {
        m_bImmortalAfterRespawn = true;
        CallMethodIn( METHOD_ClearImmortalAfterRespawn, fTime );
    }
    /** wylacz niesmiertelnosc*/
    void ClearImmortalAfterRespawn()
    {
        m_bImmortalAfterRespawn = false;
    }
    
    
    //do nadpisania w PlayerNetBaseInventory
    void FillInventory(){}
    void SetInventoryMenu(){}
    void RestartInventoryRoom(){}
    void RemoveInventoryOnDie( int nDieMode ){} 
    void MsgSendClientPlayerDied(int nMode, int nParam,int nRespawnCnt,float fTime){}
        
    //do nadpisania w PlayerNetBaseMover
    void DelayedRotate(){}
    
    //TODO: tylko do testow
    void CheatDamage()
    {
        OnTakeDamage( 100.0f,//float fDamage, 
                      100.0f,//float fPhSimDamage,  
                      null,//GameObject Assassin, 
                      new Vector(0,1,0),//Vector vHitLocation, 
                      new Vector(1,1,1),//Vector vSpeed, 
                      0,//int nMeshElementorBoneID, 
                      0//int nDamageType)    
                      );
    }
    
    
    
    

}
