public class PlayerNetStateList
{
    static int sm_nMaxClientSentStates      = 256;
    static int sm_nMaxServerSimulatedStates = 256;

    PlayerNetBaseMover m_cPlayer = null;
        
    /** Lista stanow zapamietanych
     */
    PlayerNetState [] m_aStates = null;
    
    /** Ostatni stan zapisany w tablicy
     */
    int m_nLast = 0;
    
    /** Info czy w ogole cos w tablicy jest zapamietane 
     */
    boolean m_bStored = false;
     
    PlayerNetStateList(PlayerNetBaseMover cPlayer,int nMaxStates)
    {
        m_cPlayer = cPlayer;
        m_aStates = new PlayerNetState[ nMaxStates ];
    }
    
    void Start() // float fTime )
    {
        Clean(); // fTime );
    }
    
    void Clean() // float fTime )
    {
        for ( int i=0; i<m_aStates.length; i++ ) 
        {
            if ( m_aStates[i] != null )
                 m_aStates[i].Destroy();
            else
                m_aStates[i] = new PlayerNetState(); // m_cPlayer, fTime );
        }
        
        m_nLast   = 0;
        m_bStored = false;
    }
    
    void Destroy()
    {
        if ( m_aStates != null )
        {
            for ( int i=0; i< m_aStates.length; i++ ) 
            {
                if ( m_aStates[i] != null )
                {
                    m_aStates[ i ].Destroy();
                    m_aStates[ i ] = null;
                }
            }
            m_aStates = null;            
        }
        m_cPlayer = null;
        m_bStored = false;
    }

    /** Zapamietanie stanu gracza w podnym czasie
     */
    void Store( float fTimeStamp )
    {
        m_bStored = true;
        m_nLast++;
        if ( m_nLast >= m_aStates.length ) 
             m_nLast = 0;
        m_aStates[ m_nLast ].Store( m_cPlayer, fTimeStamp );
    }
    
    void StoreWithPortalSystemActiveFlag(float fTime)
    {
        Store( fTime );
        m_aStates[ m_nLast ].StorePortalSystemActiveFlag( m_cPlayer );
    }
    
    void Store( PlayerNetState cState )
    {
        m_bStored = true;
        m_nLast++;
        if ( m_nLast >= m_aStates.length ) 
             m_nLast = 0;
        if ( m_aStates[ m_nLast ] != null )
             m_aStates[ m_nLast ].Destroy();
        m_aStates[ m_nLast ] = cState;
    }
    
    PlayerNetFoundStateInfo m_cFoundStateInfo = new PlayerNetFoundStateInfo();
    
    PlayerNetFoundStateInfo GetStateInfo(float fTimeStamp, int nODEDoneUpdatesCount )
    {
        int nElems = 0;
        int i      = m_nLast;
        for( /**/ ; nElems <= m_aStates.length; nElems++, i-- )
        {
            if ( i < 0 ) 
                 i = m_aStates.length-1;
            if ( (m_aStates[i].fTimeStamp           == fTimeStamp) &&
                 (m_aStates[i].nODEDoneUpdatesCount == nODEDoneUpdatesCount) )
                break;
        }
    
        m_cFoundStateInfo.m_nIndex    = i;
        m_cFoundStateInfo.m_nElements = nElems;
            
        if (nElems > m_aStates.length)
        {
//            GameObject.CrashLog("ERROR! too many states to roll back!\n");

            /*for( i = m_nLast, nElems=0; nElems < m_aStates.length; nElems++, i-- )
            {
                if ( i < 0 ) 
                    i = m_aStates.length-1;
                if (m_aStates[i].fTimeStamp           == fTimeStamp)
                    GameObject.CrashLog("Znalazlem z odpowiednim czasem, ale z ODEUpdates: " + m_aStates[i].nODEDoneUpdatesCount + " a nie jak powinno by: " + nODEDoneUpdatesCount + "\n");
            }*/

            //TODO: obsluzyc jakos ten fakt.
            // cos poszlo nie tak !!!!!!! za mala tablica do zapamietywania stanu lub za duzy ping
            // FIXIT TODO no comprendo
            m_cFoundStateInfo.m_nIndex    = -1;
            m_cFoundStateInfo.m_nElements = -1;
        }
        
        return m_cFoundStateInfo;
    }
    
    PlayerNetState GetState(float fTimeStamp, int nODEDoneUpdatesCount )
    {
        PlayerNetFoundStateInfo cInfo = GetStateInfo( fTimeStamp, nODEDoneUpdatesCount );
        return ((cInfo.m_nIndex<m_aStates.length)&&(cInfo.m_nIndex>=0)) ? 
                m_aStates[ cInfo.m_nIndex ] : null;
    }
     
    static boolean sm_bCheckClimbingOnAdjust = true;
    /** Przesymulowanie gracza z wzieciem pod uwage podanego stanu cAdjustState z jakiegos tam czasu.
     */
    void ResimulateOnAdjust( PlayerNetAdjustState cAdjustState, float fLastUpdatePhysicsTime  )
    {
        Vector vPrevPos = m_cPlayer.GetLocalPosition();
        
        //zapamietujemy aktualnego parenta
        ControlObject cParent = m_cPlayer.GetParent();
        //wlaczamy gracza w tryb resymulacji
        m_cPlayer.m_bNetResimulate = true;
               
//        if ( fLastUpdatePhysicsTime - cAdjustState.fTimeStamp > 0.1f )
//            GameObject.CrashLog(" Adjust "+(fLastUpdatePhysicsTime - cAdjustState.fTimeStamp)+"\n");

        //GameObject.LogD(" Resimulate called for time " + String.valueOf(ClientState.fTimeStamp) + "\n");
        
        // Szukanie pierwszego elementu z czasem sprzed ClientState.fTimeStamp
        PlayerNetFoundStateInfo cFoundStateInfo = 
            GetStateInfo( cAdjustState.fTimeStamp, 
                          cAdjustState.nODEDoneUpdatesCount );
        if ( cFoundStateInfo == null || cFoundStateInfo.m_nIndex < 0 )
            return;
        
        int nIndex =  cFoundStateInfo.m_nIndex ;
        int nElems =  cFoundStateInfo.m_nElements; 
        
        //TODO: fixit
        //policzmy, o ile nas serwer chce zadjustowac na danej pozycji
        Vector vAdjustDiff = new Vector( cAdjustState.vPosition ); 
        vAdjustDiff.Sub( m_aStates[ nIndex ].vPosition );
        NetLog.PlayerMover(" IN: Adjust - Diff: " + vAdjustDiff + " Time: " + cAdjustState.fTimeStamp + " ODEUpdates: " + cAdjustState.nODEDoneUpdatesCount + "\n");
//        m_cPlayer.LogR(" wyszo po porownaniu: AdjustPos: " + cAdjustState.vPosition + "  ostatni stan Pos: " + 
//                       m_aStates[ nIndex ].vPosition + 
//                       " Time: " + m_aStates[ nIndex ].fTimeStamp + "\n");

        // zapisujemy informacje z AdjustState do stanu
        m_aStates[ nIndex  ].StoreAdjust( cAdjustState );
        if ( m_aStates[ nIndex ].cParent != cAdjustState.cParent )
        {
//            m_cPlayer.LogR("ERROR: Parenty sie roznia przy resymulacji: org:"+
//                     m_aStates[ nIndex ].cParent+" adj:"+cAdjustState.cParent+"\n");
        }
                
        // i odtwarzamy ten stan z czasu kiedy przyszedl AdjustState
        m_aStates[ nIndex ].Restore( m_cPlayer );
        // ustawiam flage portali taka jaka przyszla z serwera w adjuscie
        m_aStates[ nIndex ].RestorePortalSystemActiveFlag( m_cPlayer );

        //GameObject.LogD(" Wycofany stan pozycja=" + Player.GetPosition().toString() + " Predkosc=" + Player.GetSpeed() +"\n");
       
        //GameObject.LogD(" Resimulating actions\n");
        
        for ( m_nLast = nIndex; nElems > 0; nElems-- )
        {
            // odtwarzamy ruchy dla kolejnego stanu
            m_aStates[ m_nLast ].RestoreMotionAndDirAndLocalSpeed( m_cPlayer );
        
            // numerek nastepnego stanu po aktualnie odtwarzanym
            int   nNextState = (m_nLast+1)%m_aStates.length;
            // roznica czasow do przesymulowania
            float fDeltaTime =   m_aStates[ nNextState ].fTimeStamp 
                               - m_aStates[ m_nLast    ].fTimeStamp;

            m_cPlayer.UpdateMotion();
            m_cPlayer.UpdateState();
            float fLastPhysicsUpdate = m_aStates[ m_nLast ].fTimeStamp;
            if (m_aStates[m_nLast].nODEDoneUpdatesCount != 4)
                fLastPhysicsUpdate -= m_cPlayer.sm_fPhysicsGranularity;
            m_cPlayer.SetPhysicsLastUpdateTime(fLastPhysicsUpdate);
            m_cPlayer.StepUpdatePhysicsWithODE( fDeltaTime, true, m_aStates[ m_nLast ].nODEDoneUpdatesCount );
            m_cPlayer.UpdateAnims( fDeltaTime );
            
            // zapisujemy stan fizyki i pozycje
            int nTmp = m_aStates[ nNextState ].nODEDoneUpdatesCount;
            m_aStates[ nNextState ].StorePhysics( m_cPlayer );
            m_aStates[ nNextState ].StorePos( m_cPlayer );
            m_aStates[ nNextState ].StorePortalSystemActiveFlag( m_cPlayer );
            m_aStates[ nNextState ].nODEDoneUpdatesCount = nTmp;
            
            // przechodzimy do nastepnego stanu
            m_nLast = nNextState;
        }
        
        
        // ustawiamy z ostatniego zapamietanego stanu ustawienia motion
        m_aStates[ m_nLast ].RestoreMotionAndDirAndLocalSpeed( m_cPlayer );
        // pobieramy aktualna flage stanu portali
        m_aStates[ m_nLast ].RestorePortalSystemActiveFlag( m_cPlayer );
        // i na jej podstawie ustawiamy system portali
        m_cPlayer.ApplyPortalSystemActiveFlag();
            
        // updejtujemy jeszcze raz do stanu z chwili obecnej
        float fRemainingDeltaTime = fLastUpdatePhysicsTime - m_aStates[ m_nLast ].fTimeStamp;
        m_cPlayer.UpdateMotion();
        m_cPlayer.UpdateState();
        float fLastPhysicsUpdate = m_aStates[m_nLast].fTimeStamp;
        if (m_aStates[m_nLast].nODEDoneUpdatesCount != 4)
            fLastPhysicsUpdate -= m_cPlayer.sm_fPhysicsGranularity;
        m_cPlayer.SetPhysicsLastUpdateTime(fLastPhysicsUpdate);
        m_cPlayer.StepUpdatePhysicsWithODE(fRemainingDeltaTime, true, m_aStates[m_nLast].nODEDoneUpdatesCount);
        m_cPlayer.UpdateAnims( fRemainingDeltaTime );
                
        //wylaczamy tryb resymulacji
        m_cPlayer.m_bNetResimulate = false;
        //przywracamy parenta
        m_cPlayer.SwitchParentTo( cParent );
        
        if ( m_cPlayer.m_bUpdateEllipsoidRequired )
            m_cPlayer.UpdateEllipsoidAndMaxClimbAngle();
        
        //policzmy, o ile nas serwer efektywnie przesunal
        Vector vCorrect = m_cPlayer.GetLocalPosition();
        vCorrect.Sub( vPrevPos );
        if ( vCorrect.Len2() > 0.01f )
        {
            GameObject.LogD("IN: adjust: "+vAdjustDiff.toString()+
                            " correct: " + vCorrect.toString()+
                            cAdjustState.fTimeStamp+"\n");
        }
        
        //GameObject.LogD(" Resimulate finished\n");
        //GameObject.LogD(" Prev pos=" + vPrevPos.toString() + " new pos=" + Player.GetPosition().toString() + "\n");
    }    
}

class PlayerNetFoundStateInfo
{
    int m_nIndex    = -1;
    int m_nElements = -1;
}

