/** Kontener danych umozliwiajacy przesylanie ich przez siec
 * za pomoca systemu komunikatow
 */
//TODO: jesli bedzie taka potrzeba, to poszerzyc o wysylanie Byte'ow
public class NetDataContainer
{
    
    //lista z zapamietanymi Int'ami
    private ContainerObjectList m_cDataInt     = new ContainerObjectList();
    //lista z zapamietanymi Float'ami
    private ContainerObjectList m_cDataFloat   = new ContainerObjectList();
    //lista z zapamietanymi Bool'ami
    private ContainerObjectList m_cDataBoolean = new ContainerObjectList();
    //lista z zapamietanymi String'ami
    private ContainerObjectList m_cDataString  = new ContainerObjectList();
    
    //obiekt ktory bedzie posredniczyl w wyslaniu/odebraniu komunikatu
    private GameObject m_cMessenger   = null;
    //tryb w ktorym wysylamy NewFreeMessage( / NewMessage(
    private boolean    m_bFreeMessageMode = false;
    
    NetDataContainer()
    {
        Clear();
    }
    
    void Clear()
    {
        m_cDataInt.Clear();
        m_cDataFloat.Clear();
        m_cDataBoolean.Clear();
        m_cDataString.Clear();
    }
    
    /**
     * Metody dopisywanie danych
     */
    void PackInt(int nValue)
    {
        m_cDataInt.Add( new Integer( nValue ) );
    }
    
    void PackFloat(float fValue)
    {
        m_cDataFloat.Add( new Float( fValue ) );
    }
    
    void PackBoolean(boolean bValue)
    {
        m_cDataBoolean.Add( new Boolean( bValue ) );
    }
    
    void PackString(String sValue)
    {
        m_cDataString.Add( sValue );
    }
    
    /**
     * Metody odczytywania danych
     */
    int UnPackInt()
    {
        Object cObj = m_cDataInt.GetNext();
        if ( cObj != null )
            return ((Integer)cObj).intValue();
        return 0;
    }
    
    float UnPackFloat()
    {
        Object cObj = m_cDataFloat.GetNext();
        if ( cObj != null )
            return ((Float)cObj).floatValue();
        return 0.0f;
    }
    
    boolean UnPackBoolean()
    {
        Object cObj = m_cDataBoolean.GetNext();
        if ( cObj != null )
            return ((Boolean)cObj).booleanValue();
        return false;
    }
     
    String UnPackString()
    {
        return (String)m_cDataString.GetNext();
    }
    
    boolean CheckUnPackEnd()
    {
        if ( ! m_cDataFloat.IsAllDataReadAlready() )
            GameObject.LogD("ReadFloat "+m_cDataFloat.GetReadDataCnt()+"/"+m_cDataFloat.GetSize()+"\n");
        if ( ! m_cDataInt.IsAllDataReadAlready() )
            GameObject.LogD("ReadInt "+m_cDataInt.GetReadDataCnt()+"/"+m_cDataInt.GetSize()+"\n");
        if ( ! m_cDataString.IsAllDataReadAlready() )
            GameObject.LogD("ReadString "+m_cDataString.GetReadDataCnt()+"/"+m_cDataString.GetSize()+"\n");
        if ( ! m_cDataBoolean.IsAllDataReadAlready() )
            GameObject.LogD("ReadBoolean "+m_cDataBoolean.GetReadDataCnt()+"/"+m_cDataBoolean.GetSize()+"\n");
            
        return( m_cDataFloat.IsAllDataReadAlready()    &&
                m_cDataInt.IsAllDataReadAlready()      &&
                m_cDataString.IsAllDataReadAlready()   &&
                m_cDataBoolean.IsAllDataReadAlready() );
    }

    void FromMessage(GameObject cMessenger)
    {
        SetMessageParams( cMessenger, false );
        FromMessage();
    }
    
    void FromFreeMessage(GameObject cMessenger)
    {
        SetMessageParams( cMessenger, true );
        FromMessage();
    }
    
    private void FromMessage()
    {
        int nIntCnt     = MessageUnPackInt();
        for ( int i=0; i<nIntCnt; i++ )
            PackInt( MessageUnPackInt() );
        int nFloatCnt   = MessageUnPackInt();
        for ( int i=0; i<nFloatCnt; i++ )
            PackFloat( MessageUnPackFloat() );
        int nBooleanCnt = MessageUnPackInt();
        for ( int i=0; i<nBooleanCnt; i++ )
            PackBoolean( MessageUnPackBoolean() );
        int nStringCnt = MessageUnPackInt();
        for ( int i=0; i<nStringCnt; i++ )
            PackString( MessageUnPackString() );
    }
    
    int MessageUnPackInt()
    {
        if (m_bFreeMessageMode)
            return m_cMessenger.FreeMessageUnPackInt();
        return m_cMessenger.MessageUnPackInt();
    }
    float MessageUnPackFloat()
    {
        if (m_bFreeMessageMode)
            return m_cMessenger.FreeMessageUnPackFloat();
        return m_cMessenger.MessageUnPackFloat();
    }
    boolean MessageUnPackBoolean()
    {
        if (m_bFreeMessageMode)
            return m_cMessenger.FreeMessageUnPackBoolean();
        return m_cMessenger.MessageUnPackBoolean();
    }
    String MessageUnPackString()
    {
        if (m_bFreeMessageMode)
            return m_cMessenger.FreeMessageUnPackString();
        return m_cMessenger.MessageUnPackString();
    }    
    
    void ToMessage(GameObject cMessenger)
    {
        SetMessageParams( cMessenger, false );
        ToMessage();
    }
    
    void ToFreeMessage(GameObject cMessenger)
    {
        SetMessageParams( cMessenger, true );
        ToMessage();
    }
    void ToMessage()
    {
        int nIntCnt     = m_cDataInt.GetSize();
        int nFloatCnt   = m_cDataFloat.GetSize();
        int nBooleanCnt = m_cDataBoolean.GetSize();
        int nStringCnt  = m_cDataString.GetSize();

        m_cDataInt.ClearReadDataCnt();
        m_cDataFloat.ClearReadDataCnt();
        m_cDataBoolean.ClearReadDataCnt();
        m_cDataString.ClearReadDataCnt();
        
        MessagePackInt( nIntCnt );
        for ( int i=0; i<nIntCnt; i++ )
            MessagePackInt( UnPackInt() );
        MessagePackInt( nFloatCnt );
        for ( int i=0; i<nFloatCnt; i++ )
            MessagePackFloat( UnPackFloat() );
        MessagePackInt( nBooleanCnt );
        for ( int i=0; i<nBooleanCnt; i++ )
            MessagePackBoolean( UnPackBoolean() );
        MessagePackInt( nStringCnt );
        for ( int i=0; i<nStringCnt; i++ )
            MessagePackString( UnPackString() );
    }
    
    
    void MessagePackInt(int nValue)
    {
        if (m_bFreeMessageMode)
            m_cMessenger.FreeMessagePackInt( nValue );
        else
            m_cMessenger.MessagePackInt( nValue );
    }
    void MessagePackFloat(float fValue)
    {
        if (m_bFreeMessageMode)
            m_cMessenger.FreeMessagePackFloat( fValue );
        else
            m_cMessenger.MessagePackFloat( fValue );
    }
    void MessagePackBoolean(boolean bValue)
    {
        if (m_bFreeMessageMode)
            m_cMessenger.FreeMessagePackBoolean( bValue );
        else
            m_cMessenger.MessagePackBoolean( bValue );
    }
    void MessagePackString(String sValue)
    {
        if (m_bFreeMessageMode)
            m_cMessenger.FreeMessagePackString( sValue );
        else
            m_cMessenger.MessagePackString( sValue );
    }    
    
    void SetMessageParams( GameObject cMessenger, boolean bFreeMessageMode )
    {
        m_cMessenger        = cMessenger;
        m_bFreeMessageMode  = bFreeMessageMode;
    }
    
    
}
