import java.io.*;
import java.lang.Long;

public class NetServerInfoList extends ObjectsList
{
    private static String GetFileName() { return NetPlayerProfile.GetCurrentServersFileName(); }
	
    private boolean m_bServersValid = false;
    boolean IsValid(){ return m_bServersValid; }
        
    private long m_iRefreshTime = 0;
    void SetRefreshTimeNow(){ m_iRefreshTime = System.currentTimeMillis(); }
    
    
    void Clear()
    {
        super.Clear();
        SetRefreshTimeNow();
    }

    void Add(NetServerInfo cServerInfo)
    {
        if ( IsOnList( cServerInfo.sServerAddr , cServerInfo.GetGamePort() ) ||
             (!cServerInfo.IsCorrect()) )
            return;
        
        super.Add( cServerInfo );
    }

    private boolean IsOnList(String sIP, int nGamePort)
    {
        for ( int i=0; i<GetSize(); ++i )
        {
            NetServerInfo cServerInfo = (NetServerInfo)GetObjectAt( i );
            if ( cServerInfo.AddressEquals( sIP, nGamePort ) )
                return true;
        }
        
        return false;
    }
    
    ScriptWriter m_cScriptWriter = new ScriptWriter();
    
    void Save()
    {
        m_cScriptWriter.Open( GetFileName() );
        m_cScriptWriter.Write("!RefreshTime(s)");
		m_cScriptWriter.Write("!NetServerInfo(s,i,s,s,i,i,i,i,i,i,i)");
		m_cScriptWriter.NewLine();
		
		String sRefreshTime = Long.toString( m_iRefreshTime );
		m_cScriptWriter.Write( "RefreshTime(\""+sRefreshTime+"\")" );

        for ( int i=0; i<GetSize(); ++i )
        {
            NetServerInfo cServer = (NetServerInfo)GetObjectAt( i );
            String sServer = "NetServerInfo("+
                                        "\""+ cServer.sServerAddr+ "\","+
                                        cServer.GetGamePort()+","+
                                        "\""+ cServer.sServerName+ "\","+
                                        "\""+ cServer.sCurrentLevelName+ "\","+
                                        cServer.nNumberPlayerInGame+","+
                                        cServer.nMaxPlayersInGame+","+
                                        cServer.nGameType+","+
                                        cServer.nPing+","+
                                        cServer.nGameVersion+","+
                                        (cServer.bPassword?1:0)+","+
                                        cServer.nMapCRC+
                                        ")"; 
            m_cScriptWriter.Write( sServer );
        }
        
        m_cScriptWriter.NewLine();
        m_cScriptWriter.Close();
    }
    
    void Load()
    {
        Clear();
        
        String  sFile = GetFileName();
        File    cFile = new File( sFile );
        if (cFile.exists())
        {
            ParseKeyword cRoot = GameObject.ParseFile( sFile );
            
            if (cRoot != null)
            {
                for ( ParseKeyword cKey = cRoot; cKey != null; cKey = cKey.GetNext() )
                {
					if ( cKey.Equals( "RefreshTime" ) )
					{
						String sRefreshTime = cKey.GetSParam(0);
						m_iRefreshTime = Long.parseLong( sRefreshTime );
                        CheckServersValid();
					}
                    if ( cKey.Equals( "NetServerInfo" ) )
                    {
						 
                        NetServerInfo cServerInfo = m_bServersValid ? 
							new NetServerInfo(
                            cKey.GetSParam(0),//sHostName, 
                            cKey.GetSParam(2),//sServerName, 
                            cKey.GetIParam(1),//nPort,
                            cKey.GetSParam(3),//sLevelName, 
                            cKey.GetIParam(4),//nCurrPlayers, 
                            cKey.GetIParam(5),//nMaxPlayers, 
                            cKey.GetIParam(6),//nGameType,
                            cKey.GetIParam(7),//(int)fPing,
                            cKey.GetIParam(8),//nGameVersion,
                            (cKey.GetIParam(9)!=0),//bPassword,
                            0,
                            cKey.GetIParam(10),//nMapCRC,
                            true//bInternetServer
							):
							new NetServerInfo(
                            cKey.GetSParam(0),//sHostName, 
                            cKey.GetSParam(2),//sServerName, 
                            cKey.GetIParam(1)//nPort
                            );
                        
                        cServerInfo.bInternetServer = true;
                        Add( cServerInfo );
                    }
                }

                cRoot.delete();
            }
            else
            {
                NetLog.Test("Error while read/parse " + sFile + " file !\n");
            }
            
            
        }
        else
        {
            Save();
        }
    }
    
    void CheckServersValid()
    {
        long nTimeDelta = System.currentTimeMillis() - m_iRefreshTime;
        int nHours      = 0;
        int nMinutes    = 5;
        int nSeconds    = 10;
        int nMinTime    = (60*(60*(1000*nSeconds)+nMinutes)+nHours);
        m_bServersValid = nTimeDelta < nMinTime;
    }
                        
}


    
