public class AIControllerAnimal extends AIController
{

    public void InitController(NewAI ai)
    {
        super.InitController(ai);
        
        m_HearingSense = (HearingSense) CreateObject("HearingSense");
        m_HearingSense.Init(this);

        m_PainSense = (PainSense) CreateObject("PainSense");
        m_PainSense.Init(this);

        m_ObjectsSense = (ObjectsSense) CreateObjectNoInit("ObjectsSense");
        m_ObjectsSense._AI_SIGHT_RANGE = ((NewAIAnimal) ai).fSightRange;
        m_ObjectsSense._AI_RECOGNIZE_RANGE = ((NewAIAnimal) ai).fSightRange * ((NewAIAnimal) ai).fSightRangeRecognitionMult;
        m_ObjectsSense._AI_SIGHT_ANGLE = ((NewAIAnimal) ai).fSightAngle;
        m_ObjectsSense.InitObject();
        m_ObjectsSense.Init(this);
    }

    public void AddGenerators()
    {
        super.AddGenerators();

#if AI_DEBUG
        Generator debugFilter = new ScenariosFilterAnimal(this);
        m_DebugScenariosFilter.Set(debugFilter.m_GenScenarios);
#endif

        m_aGenerators = new Generator[] {
                           new CanAttack_Animal(this),
                           new IdleAllowed_Animal(this),
                           new PerformEditorActivity(this),
                           new ScenariosFilterAnimal(this),
                           new SilenceBroken_Animal(this),
                           };
    }
    
    
    void SetAlarmLevel(int iAlarmLevel)
    {
        super.SetAlarmLevel(iAlarmLevel);
        ((NewAIAnimal)m_MeshAI).bAggressive = (m_iCurrAlarmLevel == _ALARM_LEVEL_HIGH);
    }
    
    void TryToReduceAlarmLevel()
    {
        super.TryToReduceAlarmLevel();
        if (m_cActiveEnemy != null && 
            !ActiveEnemyReallyVisible() &&
            GetActiveEnemyDistance2() > ((NewAIAnimal)m_MeshAI).fMaxEnemyKeepingDistance *
                                        ((NewAIAnimal)m_MeshAI).fMaxEnemyKeepingDistance)
        {
            RemoveEnemy(m_cActiveEnemy);
            m_cActiveEnemy = null;
            SetAlarmLevel(_ALARM_LEVEL_LOW);
        }
    }
    
    /** Notyfikacja o kolizji z obiektem */
    public void CollisionMsg(ControlObject obstacle, Vector point, Vector normal)
    {
        super.CollisionMsg(obstacle, point, normal);
        if (obstacle instanceof ChromeActor)
        {
            ChromeActor actor = (ChromeActor)obstacle;
            EnemyInfo info = AddEnemy(actor);
            if (info != null)
                RecognizeEnemy(info);
        }
    }

    
    /** wrzucenie przekazanych przez zmysl ObjectsSense obiektow */
    void RedistributeObjects(ControlObject [] aObjs)
    {
        float fTime = GetTime();
        
        for (int i = 0; i < aObjs.length; i++)
        {
            if (aObjs[i] == m_MeshAI)
                continue;
            
            if ( (aObjs[i] instanceof NewAI) ||
                 (aObjs[i] instanceof Vehicle) ||
                 (aObjs[i] instanceof Player) )
            {
                AddToProperList((ChromeActor) aObjs[i], fTime);
            }
        }
    }

    
    // puste abstrakcyjne...
    void SetObjectToGetIn(ISeatVehicle cObjectToGetIn, EnumGetInType cGetInAs)  {}
    SeatInfo GetSeat() {return null;}
    public void GotIn() {}
    public void GotOut() {};
    boolean InsideVehicle() {return false;}
    void SetObjectToDrive(IDrivenVehicle cObjectToDrive) {}
    IDrivenVehicle GetDrivenVehicle() {return null;}
    void Drive() {}
    public void Checkpoint() {}
    public void MovingEnd() {}

    boolean InsideCannon() {return false;}
    void SetInCannon(boolean bInCannon) {}
    
    public boolean CanAttackFromCurrentHideout() {return false;}
    public ChromeActor HideoutsNear() {return null;}
    public Vector GetDestHideoutPos() {return null;}
    public boolean IsHidingBehindHideout() {return false;}
    public boolean IsBehindHideout() {return false;}
    public boolean CanHitActiveEnemyFromHideout() {return false;}
}
