import java.lang.reflect.*;
import java.util.*;

/** Animacje odpalane przez AI
 */
public abstract class AIAnimations extends Enum
{
    /** Tablica dostepnych animacji */
    protected int[] m_aAnims = null;
    
    /** Tablica z identyfikatorami animacji */
    protected String[] m_aAnimsNames = null;
    
    /** AI, ktorego dotycza te animacje */
    protected NewAI m_OwnerMesh = null;
    
    /** Nazwa glownej kosc AI */
    protected String m_sRootElementName = "";
    
    /** ID elementu glownej kosci AI */
    protected int m_nRootElementID = -1;
    
    /** Nazwa kosci zaczynajacej hierarchie torsu */
    protected String m_sTorsoElementName = "Spine";
    
    /** ID elementu kosci zaczynajacej hierarchie torsu AI */
    protected int m_nTorsoElementID = -1;
    
    
    
    
    
    
    public AIAnimations(int nVal)
    {
        super(nVal);
        // pobranie nazw stalych definiujacych id animacji i zapisanie ich w tablicy
        InitializeAnimsNames();
        // stworzenie odpowiednio duzej tablicy
        m_aAnims = new int[m_aAnimsNames.length];
    }
    

    /** Zainicjowanie tablicy m_aAnimsNames nazwami identyfikatorow animacji (A_*)
     */
    private void InitializeAnimsNames()
    {
        Field[] aFields = getClass().getFields();
        ArrayList aFieldsList = new ArrayList();
        // zapamietaj zmienne statyczne A_
        for (int i=0; i<aFields.length; i++)
        {
            if (aFields[i].getName().startsWith("A_") &&
                Modifier.isStatic(aFields[i].getModifiers()))
                aFieldsList.add(aFields[i]);
        }
        m_aAnimsNames = new String[aFieldsList.size()];
        for (int i=0; i<m_aAnimsNames.length; i++)
        {
            Field field = (Field)aFieldsList.get(i);
            try {
                m_aAnimsNames[field.getInt(this)] = field.getName();
            }
            catch (IllegalAccessException e)
            {
                GameObject.CrashLog("ERROR: IllegalAccessException during access to field " +
                                    getClass().getName() + "." + field.getName() + "\n");
            }
        }
    }
    
    
    /** Metoda sluzaca do inicjowania animacji. MUSI byc zawoalana po stworzeniu obiektu
     */
    public void Initialize(NewAI cOwnerMesh, String sAnimSeqFile,
                           String sRootElementName, String sTorsoElementName)
    {
        m_OwnerMesh = cOwnerMesh;

        m_sRootElementName = sRootElementName;
        m_sTorsoElementName = sTorsoElementName;
        m_nRootElementID = cOwnerMesh.GetElementID(m_sRootElementName);
        m_nTorsoElementID = cOwnerMesh.GetElementID(m_sTorsoElementName);

        cOwnerMesh.InitAnimSeqFile(sAnimSeqFile);
        InitializeImmutableAnimations();    
    }
    
    /** Metoda inicjujaca wszystke te animacje, ktore nie zmieniaja sie w czasie
     *  'zycia' AI
     */
    public abstract void InitializeImmutableAnimations();

    
    /** Metoda obliczajaca id animacji, ktore zaleza od stanu AI */
    public abstract void SetupAnimations(AIMotionState cMotion);
                              

    /** Odtworzenie animacji jeden raz */
    public final void Play(int nID)
    {
        Play(nID, 1);
    }

    /** Odtworzenie animacji podana ilosc razy */
    public final void Play(int nID, int nTimes)
    {
        if (m_aAnims[nID] > -1)
        {
/*            
            GameObject.CrashLog("Play : " + GetNameForID(nID) + 
                                "(" + m_OwnerMesh.GetAnimName(m_aAnims[nID]) + "), "
                                + nTimes + "\n");
*/            
/*
            Tools.ShowInfoTop(m_OwnerMesh.GetAnimName(m_aAnims[nID]) +
                              " (" + GetNameForID(nID) + ")", m_OwnerMesh.GetModule());
*/            
            m_OwnerMesh.StartAnim("");
            m_OwnerMesh.PlayAnim(m_aAnims[nID], nTimes);
        }
#if DEBUG
        else
        {
            //DBG:
//            GameObject.CrashLog("ERROR: ANIMS: Incorrect animID in PlayAnim(" + GetNameForID(nID) + ", " +
//                                nTimes + ")\n");
        }
#endif        
    }
    
    /** Odtwarzanie animacji w kolko */
    public final void PlayLoop(int nID)
    {
        Play(nID, -1);
    }
    
    
    /** Pobranie dlugosci animacji w [s] */
    public final float GetLength(int nID)
    {
        return m_OwnerMesh.GetAnimLength(m_aAnims[nID]);
    }
    
    
    /** Pobranie szybkosci animacji [fps] */
    public final float GetFreq(int nID)
    {
        return m_OwnerMesh.GetAnimFreq(m_aAnims[nID]);
    }

    
    /** Ustawienie szybkosci animacji [fps] */
    public final void SetFreq(int nID, float fFreq)
    {
        m_OwnerMesh.SetAnimFreq(m_aAnims[nID], fFreq);
    }

    
    /** Zatrzymanie odtwarzania animacji dla podanej kosci */
    public final void Stop(String sBoneName)
    {
        m_OwnerMesh.StopAnim(sBoneName);
    }
    
    
    public final String GetNameForID(int nID)
    {
        if (m_aAnimsNames == null || nID < 0 || nID >= m_aAnimsNames.length)
            return "Unknown anim name (ID: " + nID + ")";
        else
            return getClass().getName() + "." + m_aAnimsNames[nID];
    }
    
    
    public final int GetAnimationsNumber()
    {
        return m_aAnims.length;
    }
}

