public class NewAIWalkerHeavy extends NewAIWalker
{
	/** nazwy meshelementow luf */
	protected static String [] _BARRELS_NAMES = {"Gun L", "Gun R", "Rocket L", "Rocket R"};
	/** nazwy klas broni */
	protected static String [] _CLASS_NAMES = {"WeaponWalker_w_Gun", "WeaponWalker_w_Gun",
											   "WeaponWalker_w_Rocket", "WeaponWalker_w_Rocket"};
	/** indeksy meshelementow luf */
	protected int [] m_aBarrelsExits = null;
	/** bronie */
	protected Weapon [] m_aWeapons = null;

	/** typy broni walkera */
	protected int _WEAPON_PRIMARY = 0;
	protected int _WEAPON_SECONDARY = 2;
	
	/** biezacy rodzaj broni */
	protected int m_iCurrWeaponType = _WEAPON_PRIMARY;
	/** biezacy numer broni */
	protected int m_iCurrWeaponNum = 0;

	/** biezacy kat luf */
	float m_fCurrGunAngle = 0;

    public NewAIWalkerHeavy()
    {
        super();
        fHealth = fMaxHealth = 8000.f;
        fDamageThreshold = 160;

        ActiveElement = "";
        m_sControllerClassName = AIControllerWalkerHeavy.class.getName();

		m_sMeshElementToLookAt = "Walker Cockpit";
		
        /** Tablica predkosci poruszania sie w zaleznosci od MotionState.iSpeed
        *  STILL, SWIM, CRAWL, DUCK, WALK, RUN */
        m_aMotionVelocity = new float [] {0, 70, 70, 100, 165, 500};
        /** Mnozniki predkosci dla strafowania */
        m_aMotionVelocityStrafeMultiplier = new float[] {0.f, 1.0f, 1.f, .6f, .5f, .5f};
        /** Tablica predkosci katowych w zaleznosci od MotionState.iSpeed (dla Turn) */
        m_aMotionAngularVelocity = new float[] { 0, 0, 30, 90, 180, 300 };
        
        // elementy glowy (do ukrycia po smierci)
        m_aHeadElements = new String[] { "Walker Head", "Walker Cannon Link", "Walker L Cannon",
                             "Walker R Cannon", "Walker Cover", "Walker Inside", "Walker Cockpit", 
                             "WW_Fotel"};

        m_fHitRatioChangeSpeed = 0.7f;
        m_iPathFinderStampSize = 9;
        
        fBlowUpShakeStrength = 700;
        fBlowUpShakeDistMin = 700;
        fBlowUpShakeDistMax = 3000;
    }
    
    public void OnPostCreate()
    {
        SetEllipsoidNoExtents(180, 450, 225);

        super.OnPostCreate();
        
        CallMethodIn(METHOD_CallUpdateMotion, 0);
    }
    
    public void InitWalkerMesh()
    {
		super.InitWalkerMesh();
		
        m_nVertRotateElem = GetElementID("Walker Neck");
        m_nHorizRotateElem = GetElementID("Walker Head");
        m_nGunRotateElem = GetElementID("Walker Cannon Link");
        m_nLookFromElem = GetElementID("Walker Cockpit");
		
		m_aWeapons = new Weapon[_BARRELS_NAMES.length];
		m_aBarrelsExits = new int[_BARRELS_NAMES.length];
		
		for (int i = 0; i < _BARRELS_NAMES.length; i++)
	        m_aBarrelsExits[i] = GetElementID(_BARRELS_NAMES[i]);
        
        SetElementAnimType(m_nHorizRotateElem, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType(m_nVertRotateElem, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType(m_nGunRotateElem, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        
        CalcWalkAnimSpeed("Walker R Foot", "Walker L Foot");
    }
    
    public void Freeze()
    {
        if (!IsFrozen())
        {
            super.Freeze();
            
            for (int i = 0; i < _BARRELS_NAMES.length; i++)
            {
                if (m_aWeapons[i] != null)
                    m_aWeapons[i].FreezeCallMethods();
            }
        }
    }
    
    public void Unfreeze()
    {
        if (IsFrozen())
		{
            super.Unfreeze();
			for (int i = 0; i < _BARRELS_NAMES.length; i++)
			{
				if (m_aWeapons[i] != null)
	                m_aWeapons[i].UnfreezeCallMethods();
			}
		}
	}
			
    void InitializeAIWeapon()
    {
        super.InitializeAIWeapon();
		
		for (int i = 0; i < _BARRELS_NAMES.length; i++)
		{
			Weapon Gun = (Weapon) CreateObject(_CLASS_NAMES[i]);
	        Gun.AddToInventory(this, false);
			Gun.ActivateOnModule();
			m_aWeapons[i] = Gun;
			UseWeapon(Gun);

			AttachChild(Gun, m_aBarrelsExits[i]);
			ActiveWeapon.SetPosition(Vector.Zero);
			ActiveWeapon.SetDirLocal(Vector.Forward);
			ActiveWeapon.EnableRendering(false);
			ActiveWeapon.ClearFlag(EFlags._BLOCK_TRACE | EFlags._BLOCK_MOVE);
		}
		UseWeaponType(_WEAPON_PRIMARY);
    }
	
    void MessageWeapon(int nState)
    {
		super.MessageWeapon(nState);
		
        if ((nState & Weapon.STATE_SHOT) != 0) 
			SwitchToOtherWeaponType(m_iCurrWeaponType);
    }
	
	void UseWeaponType(int iWeaponType)
	{
		m_iCurrWeaponNum = iWeaponType;
		m_iCurrWeaponType = iWeaponType;
		UseWeapon(m_aWeapons[iWeaponType]);
	}
	
	int GetWeaponType()
	{
		return m_iCurrWeaponType;
	}
	
	void SwitchToOtherWeaponType(int iWeaponType)
	{
		if (m_iCurrWeaponType != iWeaponType)
			return;
		if (m_iCurrWeaponNum == m_iCurrWeaponType)
			m_iCurrWeaponNum++;
		else
			m_iCurrWeaponNum = m_iCurrWeaponType;
		ActiveWeapon = m_aWeapons[m_iCurrWeaponNum];
	}
	
    public void UseWeapon(Weapon weapon)
    {
		ActiveWeapon = weapon;
        m_Controller.ActiveWeaponChanged();
    }
    
    public Vector GetLookFromPoint()
    {
        return GetElementPos(m_nLookFromElem);
    }

    public Vector GetLookDir()
    {
        return GetElementForwardVector(m_nLookFromElem);
    }
    
    public Vector GetFireOrigin()
    {
		return ActiveWeapon.GetPosition();
    }
	
    public Vector GetVisibilityCheckFireOrigin()
    {
		Vector vTemp = GetPosition();
		vTemp.fY = GetFireOrigin().fY;
		return vTemp;
    }

    private Vector m_vDev = new Vector();
    private Vector m_vLastFireDir = new Vector();

    public Vector GetFireDir()
    {
        ChromeActor cEnemy = m_Controller.GetActiveEnemy();
        Vector vDir;
        if (cEnemy != null && ActiveWeapon != null)
        {
            vDir = cEnemy.GetPointToLookAt(0);
            vDir.Sub(GetFireOrigin());
            
            vDir.CrossStatic(Vector.Up, m_vDev);
            RotateVector(m_vDev, m_fFiringDevLineAngle, vDir, m_vDev);
            m_vDev.Mul(fMaxFireDeviation * m_fFiringDeviation);
            vDir.Add(m_vDev);
            vDir.Normalize();

            m_vLastFireDir.Set(vDir);

        }
        else
            vDir = super.GetFireDir();

        return vDir;
    }
    
	public Vector GetLastFireDir()
	{
		return m_vLastFireDir;
	}
    
    /** pokryte, by AI odpalalo efekt dokladnie tam, gdzie strzela */
    void ExecuteFXBarrel(String sFXName)
    {
        if (ActiveWeapon != null)
            ExecuteFX(ActiveWeapon, sFXName, ActiveWeapon.vBarrelElementPos);
        
        if (ActiveWeapon.m_bShowFXBeam)
        {
            if (GetLastFireDir().Len2() > 0f)
                ActiveWeapon.ExecuteFX(ActiveWeapon, "WeaponBeam", ActiveWeapon.vBarrelElementPos, 
                                       ActiveWeapon.VectorWorldToLocal(GetLastFireDir()));
            else
                ActiveWeapon.ExecuteFX(ActiveWeapon, "WeaponBeam", ActiveWeapon.vBarrelElementPos);
        }
    }
	
	/** sledzenie celu glowa i gunami */
    protected void UpdateTrackingObject()
    {
        Vector vTgtPos = m_TrackedObject.GetPointToLookAt(0);
        vTgtPos.Sub(GetElementPos(m_nHorizRotateElem));
        Vector vLocalDirHead = VectorWorldToLocal(vTgtPos);
        
        vTgtPos = m_TrackedObject.GetPointToLookAt(0);
        vTgtPos.Sub(GetElementPos(m_nVertRotateElem));        
        Vector vLocalDirNeck = VectorWorldToLocal(vTgtPos);
        
        float fHorizAngle = (float)(Math.atan2(vLocalDirHead.fX, vLocalDirHead.fZ) * Tools.RAD2DEG),
            fLenXZ = (float)Math.sqrt(vLocalDirNeck.fX * vLocalDirNeck.fX + vLocalDirNeck.fZ * vLocalDirNeck.fZ),
            fVertAngle = (float)(-Math.atan2(vLocalDirNeck.fY, fLenXZ) * Tools.RAD2DEG);
    
        if (Math.abs(fHorizAngle) > m_fMaxHorizHeadAngle 
            && !m_ActualMotion.IsMovingLinear())
        {
            Turn(fHorizAngle);
            fHorizAngle = 0;
        }
        RotateHead(fHorizAngle, fVertAngle);
         
        Vector vDirTarget = m_TrackedObject.GetPointToLookAt(0);
        vDirTarget.Sub(GetElementPos(m_aBarrelsExits[0]));
		vDirTarget.Set((float) Math.sqrt(vDirTarget.fX * vDirTarget.fX + vDirTarget.fZ * vDirTarget.fZ),
					   vDirTarget.fY,
					   0f);
		vDirTarget.Normalize();
		
		Vector vDirBarrel = GetElementForwardVector(m_aBarrelsExits[0]);
		vDirBarrel.Set((float) Math.sqrt(vDirBarrel.fX * vDirBarrel.fX + vDirBarrel.fZ * vDirBarrel.fZ),
					   vDirBarrel.fY,
					   0f);
		vDirBarrel.Normalize();
		
		float fGunAngle = Tools.acos(vDirTarget.Dot(vDirBarrel)) * Tools.RAD2DEG;
		if (vDirBarrel.fX * vDirTarget.fY - vDirTarget.fX * vDirBarrel.fY > 0) // cross.fZ
			fGunAngle = -fGunAngle;

		m_fCurrGunAngle += fGunAngle;
		
        RotateGun(m_fCurrGunAngle);
    }
    
    //*********************************************************
    //                     SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[NewAIWalkerHeavy.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_NEWAI_WALKER_HEAVY)
        {
            super.SGLoadChunk(cFC);

            int arraySize = cFC.LoadInt();
            if (arraySize < 0)
                m_aWeapons = null;
            else
            {
                m_aWeapons = new Weapon[arraySize];
                for (int i = 0; i < arraySize; i++)
                    m_aWeapons[i] = (Weapon)cFC.LoadObjectRef();
            }
            m_iCurrWeaponType = cFC.LoadInt();
            m_iCurrWeaponNum = cFC.LoadInt();
            m_fCurrGunAngle = cFC.LoadFloat();
            m_vLastFireDir = cFC.LoadVector();
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[NewAIWalkerHeavy.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_NEWAI_WALKER_HEAVY, cFCParent);
        
        super.SGSaveChunk(cFC);

        if (m_aWeapons != null)
        {
            cFC.SaveInt(m_aWeapons.length);
            for (int i = 0; i < m_aWeapons.length; i++)
                cFC.SaveObjectRef(m_aWeapons[i]);
        }
        else
            cFC.SaveInt(-1);

        cFC.SaveInt(m_iCurrWeaponType);
        cFC.SaveInt(m_iCurrWeaponNum);
        cFC.SaveFloat(m_fCurrGunAngle);
        cFC.SaveVector(m_vLastFireDir);
        
        cFC.delete();
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[NewAIWalkerHeavy.SGUpdate]\n");
        super.SGUpdate();
    }

}
