public class NewAIWalkerLight extends NewAIWalker
{
    protected int m_nBarrelExit = -1;
    

    public NewAIWalkerLight()
    {
        super();
        fHealth = fMaxHealth = 3000.f;
        fDamageThreshold = 60;

        ActiveElement = null;
        bForceWholeObjectCollision = true;
        m_sControllerClassName = AIControllerWalkerLight.class.getName();

        m_sMeshElementToLookAt = "Walker Cockpit";
        
        /** Tablica predkosci poruszania sie w zaleznosci od MotionState.iSpeed
        *  STILL, SWIM, CRAWL, DUCK, WALK, RUN */
        m_aMotionVelocity = new float [] {0, 0, 70, 100, 165, 330};
        /** Mnozniki predkosci dla strafowania */
        m_aMotionVelocityStrafeMultiplier = new float[] {1.f, 1.f, 1.f, 1.f, 1.f, 1.f};
        /** Tablica predkosci katowych w zaleznosci od MotionState.iSpeed (dla Turn) */
        m_aMotionAngularVelocity = new float[] { 90, 90, 45, 45, 60, 90 };
        
        // elementy glowy (do ukrycia po smierci)
        m_aHeadElements = new String[] { "Walker Head", "Walker Gun", "Walker Inside",
                             "Walker Cockpit", "WW_Fotel", "Walker Pane"};

        m_aWalkerAnimsSounds = new String[] {
                                  null, //A_STAND
                                  null, //A_FORWARD
                                  null, //A_BACKWARD
                                  null, //A_ROTATE_LEFT
                                  null, //A_ROTATE_RIGHT
                                  "Data/Sounds/Walker/Walker_Standby_k", //A_NORMAL2STANDBY
                                  null, //A_STANDBY
                                  "Data/Sounds/Walker/Walker_OpenCabin_k", //A_STANDBY2STANDBY_OPEN
                                  null, //A_STANDBY_OPEN
                                  "Data/Sounds/Walker/Walker_CloseCabin_k", //A_STANDBY_OPEN2STANDBY
                                  "Data/Sounds/Walker/Walker_WakeUp_k", //A_WAKEUP
                                  "Data/Sounds/Walker/Walker_NormUp_k", //A_NORMAL2UP
                                  null, //A_UP
                                  "Data/Sounds/Walker/Walker_UpNorm_k", //A_UP2NORMAL
                                  "Data/Sounds/Walker/Walker_NormDown_k", //A_NORMAL2DOWN
                                  null, //A_DOWN
                                  "Data/Sounds/Walker/Walker_DownNorm_k"  //A_DOWN2NORMAL
                                  };

        m_sStepSound = "Data/Sounds/Walker/Walker_Walk_k";
        m_sHeadLoopSound = "Data/Sounds/Walker/Walker_HeadLoop_k2";
        m_sGunRotateLoopSound = "Data/Sounds/Walker/Walker_GunRotateLoop_k";
        m_fHitRatioChangeSpeed = 0.8f;

        fBlowUpShakeStrength = 400;
        fBlowUpShakeDistMin = 200;
        fBlowUpShakeDistMax = 2000;
    }
    
    public void OnCreate()
    {
        super.OnCreate();
        SetEllipsoidNoExtents(180, 450, 225);
        SetFlag(EFlags._FXPARTICLE_COLLISION);
        SetBlobMapSize(300);
    }
    
    
    public void InitWalkerMesh()
    {
        super.InitWalkerMesh();
        m_nVertRotateElem = GetElementID("Walker Neck");
        m_nHorizRotateElem = GetElementID("Walker Head");
        m_nGunRotateElem = GetElementID("Walker Gun");
        m_nLookFromElem = GetElementID("Walker Cockpit");
        m_nBarrelExit = GetElementID("Barel");
        
        SetElementAnimType(m_nHorizRotateElem, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType(m_nVertRotateElem, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType(m_nGunRotateElem, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        
        SetAnimMorphTime(m_nHorizRotateElem, 0.3f, false);
        SetAnimMorphTime(m_nVertRotateElem, 0.3f, false);
        SetAnimMorphTime(m_nGunRotateElem, 0.3f, false);
        
        CalcWalkAnimSpeed("Walker R Foot", "Walker L Foot");
        CallMethodIn("InitializeDriver", 0);
    }


    
    void InitializeDriver()
    {
        // jesli nie ma ustawionego drivera a ma Waypointy - stworz jakiegos, 
        // zeby walker pusty nie ganial
        if (cDriver == null)
        {
            cDriver = (NewAIHuman)CreateObjectNoInit("NewAIHuman");
            cDriver.MeshName = "Data/Meshes/AI/LSoldier.3da";
            cDriver.SetName("Driver for " + GetName());
            cDriver.InitObject();
            cDriver.Activate();
        }
        
        if (cDriver != null)
        {
            //cDriver.Freeze();
            // cDriver.UpdatePathMap(false);
         
            ResetBodyInVehicle();
            
            cDriver.fDamageThreshold = 1e6f;
            cDriver.ClearFlag(EFlags._CAST_SHADOWS);
            if (cDriver.ActiveWeapon != null)
            {
                cDriver.ActiveWeapon.DeactivateOnModuleAlways();
                cDriver.ActiveWeapon.EnableRendering(false);
            }
            cDriver.bShowOnMoveDetector = false;
        }
        else
            bCanUseRadioCommunication = false;
    }
    
    void ResetBodyInVehicle()
    {
        // tu wazna jest kolejnosc zawolania tych metod, bo druga bazuje na pierwszej
		cDriver.Freeze();
		cDriver.UpdatePathMap(false);
        ResetInVehicleAnim();
        ResetInVehiclePos();
        CallMethodIn("ResetInVehiclePos", 0.1f);
    }

    void ResetInVehicleAnim()
    {
        int nDriveAnim = cDriver.GetAnimID("", "drive");
        int nDriveFrame = cDriver.GetFirstFrame(nDriveAnim);
		cDriver.ClearFlag(EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH);
		int nFlagNotStatic = cDriver.GetFlags() & EFlags._NOT_STATIC;
		cDriver.SetFlag(EFlags._NOT_STATIC);
        cDriver.SetFrame(nDriveAnim, nDriveFrame, -1);		
		if (nFlagNotStatic == 0)
			cDriver.ClearFlag(EFlags._NOT_STATIC);			
        cDriver.SetFlag(EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH);
    }
    
    void ResetInVehiclePos()
    {
        int iSeatID = GetElementID("Driver");
        Vector vNewPos = GetElementPos(iSeatID);
        
        if (vNewPos == null)
            return;
        
        Vector vDiff = cDriver.GetPosition();
        vDiff.Sub(cDriver.GetElementPos(cDriver.GetElementID("Bip01")));
        vNewPos.Add(vDiff);
            
        AttachChild(cDriver, iSeatID);
        cDriver.FromUpForwardPos(GetElementUpVector(iSeatID), 
                                 GetElementForwardVector(iSeatID),
                                 vNewPos);
    }
    
    public void Die()
    {
        super.Die();
        if (cDriver != null)
        {
            cDriver.Deactivate();
        }
    }
    
    
    public void Activate()
    {
        super.Activate();
        if (cDriver != null)
        {
            if (m_bDriverWasRenderableBeforeDeactivate)
                cDriver.Activate(); // to musi byc tak - a w new ai jest zabezp. przed null pointer (aicontroller)
        }
    }
    
    protected boolean m_bDriverWasRenderableBeforeDeactivate = true;
    public void Deactivate()
    {
        super.Deactivate();
        if (cDriver != null)
        {
            m_bDriverWasRenderableBeforeDeactivate = cDriver.IsRenderable();
            cDriver.Deactivate();
        }
    }
    
    void InitializeAIWeapon()
    {
        /** Podepnij guna tam gdzie trzeba i zniknij go */
        Weapon Gun = (Weapon) CreateObject("WeaponWalker_k");
        Gun.AddToInventory(this, false);
        UseWeapon(Gun);

        AttachChild(ActiveWeapon, "Barel");
        ActiveWeapon.SetPosition(Vector.Zero);
        ActiveWeapon.SetDirLocal(Vector.Forward);
        ActiveWeapon.EnableRendering(false);
        ActiveWeapon.ClearFlag(EFlags._BLOCK_TRACE | EFlags._BLOCK_MOVE);
    }
    
    public Vector GetLookFromPoint()
    {
        return GetElementPos(m_nLookFromElem);
    }

    public Vector GetLookDir()
    {
        return GetElementForwardVector(m_nLookFromElem);
    }
    
    public Vector GetFireOrigin()
    {
        return GetElementPos(m_nGunRotateElem);
    }
    
    public Vector GetVisibilityCheckFireOrigin()
    {
        Vector vTemp = GetPosition();
        vTemp.fY = GetFireOrigin().fY;
        return vTemp;
    }

    private Vector m_vDev = new Vector();
    private Vector m_vLastFireDir = new Vector();

    public Vector GetFireDir()
    {
        ChromeActor cEnemy = m_Controller.GetActiveEnemy();
        Vector vDir;
        if (cEnemy != null && ActiveWeapon != null)
        {
            vDir = cEnemy.GetPointToLookAt(0);
            vDir.Sub(GetFireOrigin());
            
            vDir.CrossStatic(Vector.Up, m_vDev);
            RotateVector(m_vDev, m_fFiringDevLineAngle, vDir, m_vDev);

            m_vDev.Mul(fMaxFireDeviation * m_fFiringDeviation);
            vDir.Add(m_vDev);

            vDir.Normalize();
            m_vLastFireDir.Set(vDir);
        }
        else
            vDir = super.GetFireDir();

        return vDir;
    }
    
    public Vector GetLastFireDir()
    {
        return m_vLastFireDir;
    }
    
    /** pokryte, by AI odpalalo efekt dokladnie tam, gdzie strzela
     */
    void ExecuteFXBarrel(String sFXName)
    {
        if (ActiveWeapon != null)
            ExecuteFX(ActiveWeapon, sFXName, ActiveWeapon.vBarrelElementPos);
        
        if (ActiveWeapon.m_bShowFXBeam)
        {
            if (GetLastFireDir().Len2() > 0f)
                ActiveWeapon.ExecuteFX(ActiveWeapon, "WeaponBeam", ActiveWeapon.vBarrelElementPos, 
                                       ActiveWeapon.VectorWorldToLocal(GetLastFireDir()));
            else
                ActiveWeapon.ExecuteFX(ActiveWeapon, "WeaponBeam", ActiveWeapon.vBarrelElementPos);
        }
    }

    
    //*********************************************************
    //                     SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[NewAIWalkerLight.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_NEWAI_WALKER_LIGHT)
        {
            super.SGLoadChunk(cFC);
            m_bDriverWasRenderableBeforeDeactivate = cFC.LoadBool();
            m_vLastFireDir = cFC.LoadVector();
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[NewAIWalkerLight.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_NEWAI_WALKER_LIGHT, cFCParent);
        
        super.SGSaveChunk(cFC);

        cFC.SaveBool(m_bDriverWasRenderableBeforeDeactivate);
        cFC.SaveVector(m_vLastFireDir);

        cFC.delete();
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[NewAIWalkerLight.SGUpdate]\n");
        super.SGUpdate();
		
		if (ActiveWeapon != null)
		{
			UseWeapon(ActiveWeapon);
			AttachChild(ActiveWeapon, "Barel");
			ActiveWeapon.EnableRendering(false);
			ActiveWeapon.ClearFlag(EFlags._BLOCK_TRACE | EFlags._BLOCK_MOVE);							
			if (cDriver != null)
				ResetBodyInVehicle();
		}
    }


}
