public class EditorLead extends Scenario
{
    static int _ID = _NUM_OF_SCENARIOS++;
    
    public final static float _UPDATE_INTERVAL = 1f,
                              _DESTINATION_EPSILON = 5f,
                              _BEFORE_AFTER_CHECK_INTERVAL = 2.f;
    
    static int init = _EDITOR_SCENARIOS.SetBit(_ID);

    
    
    void Initialize()
    {
        m_LastPathPosInfo = null;
        m_fLastPathPosUpdateTime = 0;
    }

    private static Vector tmp_vDir = new Vector();
	void PrepareScenario(AIController ai)
	{
        ChromeActor cEnemy = ai.GetActiveEnemy();

        mt_Lead leadData = (mt_Lead)ai.GetCurrActivity();
        Vector vDestPos = ai.GetCurrWaypoint().GetPosition();

        if (leadData.cObject != null)
        {
            if (ai.m_nLastPreparedScenario != _ID)
            {
                // zmiana scenariusza - zainicjuj
                Initialize();
                ai.m_nHelperScenariosInt = 1; // mozna odpalic action chain
            }
            
            boolean bShouldWait = leadData.cObject.GetDistance2To(ai.GetMesh()) >
                                  leadData.fMaxDistance * leadData.fMaxDistance &&
                                  ShouldWaitForObject(ai, leadData.cObject);
            
            boolean bShouldLook = (bShouldWait && leadData.bLookAtObjectWhileWaiting && 
                                   leadData.cObject instanceof ChromeActor);
            
            
            if (!bShouldWait)
                ai.AddTask(new TaskMotionStateChange((byte)leadData.cSpeed.GetValue()));

            if ((cEnemy != null) && (leadData.bCanShoot) )
            {
                ai.AddTask(new TaskTrackingStart(cEnemy)); 
                ai.AddTask(new TaskStartFire());		
            }
            else
            {
                if (bShouldLook)
                {
                    if (leadData.cSpeed.GetValue() == MotionState.SPEED_RUN)
                        ai.AddTask(new TaskMotionStateChange(MotionState.SPEED_WALK));

                    if (bShouldLook && leadData.aOnLookAtObject != null &&
                        ai.m_nHelperScenariosInt == 1)
                    {
                        ai.m_nHelperScenariosInt = 0;
                        ai.AddTask(new TaskInvokeActionChain(leadData.aOnLookAtObject));
                    }
                    
                    ai.AddTask(new TaskTrackingStart((ChromeActor)leadData.cObject, true));
                }
                else
                {
                    Vector vDir = new Vector(vDestPos);
                    vDir.Sub(ai.GetMesh().GetPositionVectorVolatile());
                    vDir.Normalize();
                    if (bShouldWait || vDir.Dot(ai.GetMesh().GetForwardVectorVolatile()) < 
                        ai.GetMesh().m_fMinAngleToTurnWhileMovingCos)
                    {
                        ai.AddTask(new TaskTurnAtDir(vDir));
                    }
                }
            }
            
            if (!bShouldWait)
            {
                ai.m_nHelperScenariosInt = 1;
                float fDestDist = leadData.fRadius + _DESTINATION_EPSILON;

                if (ai.GetMesh().GetDistance2To(ai.GetCurrWaypoint()) >
                    fDestDist * fDestDist)
                {
                    // Mozna podejsc
                    ai.AddTask(new TaskGoToObject(ai.GetCurrWaypoint(),
                                                _UPDATE_INTERVAL,
                                                leadData.fRadius));
                }
                else
                    ai.AddTask(new TaskEditorActivityEnd()); // zakoncz
            }
            else
            {
                //TODO: czekanie
                ai.AddTask(new TaskWait(_UPDATE_INTERVAL));
            }
            
            ai.AddCleaningTask(new TaskTrackingStop());
            ai.AddCleaningTask(new TaskStopFire());        
        }
        else
            GameObject.CrashLog("ERROR: Object to lead is null in mt_Lead (Waypoint: " +
                                ai.GetCurrWaypoint().GetName() + ")\n");
        
        super.PrepareScenario(ai);        
    }

    
    /** TODO: zrobic to na hashmapie po ai, bo nie moze miec takich pol zaleznych od ai, bo co gdy jest ich kilka? 
     *  TODO: dodac restartowanie ai - stworzenie nowych scenariuszy na start levelu
     */
    private PathPosInfo m_LastPathPosInfo = null;
    private float m_fLastPathPosUpdateTime = 0;

    protected boolean ShouldWaitForObject(AIController ai, ControlObject cLeadObject)
    {
        float fTime = ai.GetTime();
        if (m_LastPathPosInfo == null ||
            fTime - m_fLastPathPosUpdateTime > _BEFORE_AFTER_CHECK_INTERVAL)
        {
            m_fLastPathPosUpdateTime = fTime;
            m_LastPathPosInfo = GetObjectsNearestWaypoint(cLeadObject, ai.GetCurrWaypoint(), ai.GetMesh().GetWaypoint());
        }
        
        
        if (m_LastPathPosInfo.bIsBeforeCurrent) //obiekt jest z tylu
            return true;
        else 
        {
            Waypoint cCurrWpt = ai.GetCurrWaypoint();
            if (cCurrWpt == m_LastPathPosInfo.cWaypoint && // obiekt najblizej aktualnego
                cCurrWpt.cNext != null && // nie jest to ostatni (w ostatnim nie czeka)
                ai.GetMesh().GetDistance2To(cCurrWpt) < m_LastPathPosInfo.fDistance2) //ai blizej Wpt niz obiekt
                return true;
        }
        return false;
    }
    
    
    
    protected PathPosInfo GetObjectsNearestWaypoint(ControlObject cObject, Waypoint cTestedWaypoint, Waypoint cPathStart)
    {
        Waypoint cCurrWpt = cPathStart;
        boolean bCurrBeforeTested = cTestedWaypoint != cPathStart;
        PathPosInfo cNearest = new PathPosInfo(cCurrWpt, cCurrWpt.GetDistance2To(cObject), bCurrBeforeTested);
        float fDistance2;
            
        while (cCurrWpt != null)
        {
            if (cCurrWpt == cTestedWaypoint)
                bCurrBeforeTested = false;
            
            fDistance2 = cCurrWpt.GetDistance2To(cObject);
            if (fDistance2 < cNearest.fDistance2)
                cNearest.Set(cCurrWpt, fDistance2, bCurrBeforeTested);
            cCurrWpt = cCurrWpt.cNext;
        }
        return cNearest;
    }
    
    
    protected class PathPosInfo
    {
        public PathPosInfo(Waypoint cWaypoint, float fDistance2, boolean bIsBeforeCurrent)
        {
            Set(cWaypoint, fDistance2, bIsBeforeCurrent);
        }
        
        public void Set(Waypoint cWaypoint, float fDistance2, boolean bIsBeforeCurrent)
        {
            this.cWaypoint = cWaypoint;
            this.fDistance2 = fDistance2;
            this.bIsBeforeCurrent = bIsBeforeCurrent;
        }
        
        public Waypoint cWaypoint;
        public float fDistance2;
        public boolean bIsBeforeCurrent;
    }
    
    
    public String toString()
    {
        return "ELea";
    }
    
    int GetID()
    {
        return _ID;   
    }    
}
