public class EditorMovieFight extends Scenario
{
    static int _ID = _NUM_OF_SCENARIOS++;

    static int init = _EDITOR_SCENARIOS.SetBit(_ID);

	void PrepareScenario(AIController ai)
	{
        bhv_MovieFight fightData = (bhv_MovieFight)ai.GetCurrActivity();

        if (ai.m_nLastPreparedScenario != GetID())
            StartMovieFight(ai, fightData);

        
        ChromeActor objectToTrack = fightData.bTrackTarget ? fightData.cTarget : null; 
        if (objectToTrack == null || objectToTrack.IsDead())
            objectToTrack = ai.GetActiveEnemy();

        if (objectToTrack != null)
        {
            ai.AddCleaningTask(new TaskTrackingStart(objectToTrack, false, false));
            ai.AddCleaningTask(new TaskStartFire(true));
        }
        
        if ( (ai.GetActiveWeapon() != null) &&
             (ai.GetActiveWeaponAmmo() == 0) &&
             (ai.GetAmmoLeftForActiveWeapon() > 0) )
        {
            ai.AddTask(new TaskReload());
        }

        if (fightData.cEndCondition == null || fightData.cEndCondition.Evaluate())
            StopMovieFight(ai, fightData);
        else
            ai.AddTask(new TaskWait(0.5f));
        
        super.PrepareScenario(ai);        
    }

    
    protected void StartMovieFight(AIController ai, bhv_MovieFight fightData)
    {
        PlayElementsAnim(ai, fightData.sAnimName, fightData.sAnimatedElements);
    }
    
    protected void StopMovieFight(AIController ai, bhv_MovieFight fightData)
    {
        PlayElementsAnim(ai, null, fightData.sAnimatedElements); //stop anim

        ai.AddCleaningTask(new TaskTrackingStop());
        ai.AddCleaningTask(new TaskStopFire());
        ai.AddCleaningTask(new TaskEditorActivityEnd());
    }
    
    protected void PlayElementsAnim(AIController ai, String animName, String elementsList)
    {
        if (elementsList != null)
        {
            java.util.StringTokenizer st = new java.util.StringTokenizer(elementsList, ";");
            while (st.hasMoreTokens())
            {
                String elemName = st.nextToken();
                if (animName != null)
                    ai.GetMesh().PlayAnim(elemName, animName, -1);
                else
                    ai.GetMesh().StopAnim(elemName);
            }
        }
        else if (animName != null)
            ai.GetMesh().PlayAnim(animName, -1);
    }

    
    public String toString()
    {
        return "EMF";
    }
    
    int GetID()
    {
        return _ID;   
    }    
}
