/** zmysl odpowiedzialny za pobieranie w odstepach czasu obiektow
 * otaczajacych ai w celu dalszego przetworzenia
 */
public class ObjectsSense extends Sense
{
    /** min i max czas pomiedzy update'ami list celow, przeciwnikow i sprzymierzencow */
    static float _MIN_TIME_BTW_CHECK_TARGETS = 4f;
    static float _MAX_TIME_BTW_CHECK_TARGETS = 10f;

    static float _MIN_TIME_BTW_RECALCULATE_SITUATION = 0.1f;
    static float _MAX_TIME_BTW_RECALCULATE_SITUATION = 1f;
    
    /** zasieg widzenia AI */
    float _AI_SIGHT_RANGE = 0f;
    /** kwadrat zasiegu widzenia AI wyliczany w OnCreate() */
    float _AI_SIGHT_RANGE2 = 0f;
    
    /** odleglosc, z jakiej AI rozpoznaje przyjaciol / wrogow */
    float _AI_RECOGNIZE_RANGE = 0f;
    /** kwadrat odleglosci rozpoznawania */
    float _AI_RECOGNIZE_RANGE2 = 0f;

    /** polowa kata widzenia AI */
    float _AI_SIGHT_ANGLE = 60f;
    /** cos _AI_SIGHT_ANGLE wyliczany w OnCreate() */
    float _AI_SIGHT_ANGLE_COS = 60f;  //by Abak
    
    static GridManager m_GridManager = null;
    static CalcManager m_CalcManager = null;
    
    public void OnCreate()
    {
        SetAISightRange(_AI_SIGHT_RANGE);
        SetAISightAngle(_AI_SIGHT_ANGLE);
        SetAISightRecognizeRange(_AI_RECOGNIZE_RANGE);
        
        if ( (m_GridManager == null) || (m_GridManager.GetModule() != GetModule()) )
        {
            m_GridManager = (GridManager) CreateObject("GridManager");
            m_GridManager.ActivateOnModule();
            m_GridManager.SetTimePeriod(_MAX_TIME_BTW_CHECK_TARGETS);
        }
        
        if ( (m_CalcManager == null) || (m_CalcManager.GetModule() != GetModule()) )
        {
            m_CalcManager = (CalcManager) CreateObject("CalcManager");
            m_CalcManager.ActivateOnModule();
            m_CalcManager.SetTimePeriod( (_MAX_TIME_BTW_RECALCULATE_SITUATION +
                                          _MIN_TIME_BTW_RECALCULATE_SITUATION) / 2f);
        }
        
        // metoda wyciagajaca obiekty w otoczeniu
        CallMethodEveryMinMax("UpdateTargetsInRadius",
                              _MIN_TIME_BTW_CHECK_TARGETS,
                              _MAX_TIME_BTW_CHECK_TARGETS);
        // metoda update'ujaca informacje o obiektach z otoczenia
        CallMethodEveryMinMax("UpdateObjectsInformation",
                              _MIN_TIME_BTW_RECALCULATE_SITUATION,
                              _MAX_TIME_BTW_RECALCULATE_SITUATION);
    }
    
    void SetAISightRange(float fSightRange)
    {
        _AI_SIGHT_RANGE = fSightRange;
        _AI_SIGHT_RANGE2 = _AI_SIGHT_RANGE * _AI_SIGHT_RANGE;
    }
    
    void SetAISightRecognizeRange(float fSightRecognizeRange)
    {
        _AI_RECOGNIZE_RANGE = fSightRecognizeRange;
        _AI_RECOGNIZE_RANGE2 = fSightRecognizeRange * fSightRecognizeRange;
    }
    
    public void SetAISightAngle(float fSightAngle)
    {
        _AI_SIGHT_ANGLE = fSightAngle;
        _AI_SIGHT_ANGLE_COS = (float) Math.cos(_AI_SIGHT_ANGLE * Math.PI / 360f); // tak na prawde, to jest to cos polowy kata stozka widzenia
    }
    
    void UpdateTargetsInRadius()
    {
        m_GridManager.Request(this);
    }
    
    void DoUpdateTargetsInRadius()
    {
        m_Controller.GetMesh().ProfileBegin(NewAI.PROFILE_ObjectsMsg);
        
        Vector vMiddle = m_Controller.GetMesh().GetLookDir();
        vMiddle.Normalize();
        vMiddle.Mul(_AI_SIGHT_RANGE * 0.4f);
        vMiddle.Add(m_Controller.GetMesh().GetPositionVectorVolatile());
        
        // znajdz maksymalna widocznosc dla tego levelu
        ChromeModuleSingle module = (ChromeModuleSingle)GetModule();
        float fRange = Math.min(_AI_SIGHT_RANGE * module.GetVisibilityRangeFactor(),
                                module.GetVisibilityRangeThreshold());
        // a teraz obetnij ze wzgledu na przesuniecie w kierunku patrzenia
        fRange = Math.min(fRange, _AI_SIGHT_RANGE * 0.75f);
        
        ControlObject[] aObjs = GetObjectsInRadiusFromClass(vMiddle,
                                                            fRange + 500.f, //*1
                                                            "ChromeActor");
        //*1 -  stale powiekszenie, bo sight range wykorzystany przy recognition, a obiekty rzadko sie wyciagaja
        UpdateObjects(aObjs);
        
        m_Controller.GetMesh().ProfileEnd(NewAI.PROFILE_ObjectsMsg);
    }
    
    void UpdateObjectsInformation()
    {
        m_CalcManager.Request(this);
    }
    
    void DoUpdateObjectsInformation()
    {
        m_Controller.GetMesh().ProfileBegin(NewAI.PROFILE_CalcProfile);
        
        m_Controller.UpdateObjectsInformation();
        
        m_Controller.GetMesh().ProfileEnd(NewAI.PROFILE_CalcProfile);
    }
    
    public void Freeze()
    {
        super.Freeze();
        
        FreezeCallMethods();
        m_GridManager.Remove(this);
        m_CalcManager.Remove(this);
    }
    
    public void Unfreeze()
    {
        super.Unfreeze();
        
        UnfreezeCallMethods();
    }
    
    public void Die()
    {
        super.Die();
        
        DisableCallMethod("UpdateTargetsInRadius");
        DisableCallMethod("UpdateObjectsInformation");
    }
    
    public void Respawn()
    {
        super.Respawn();
        
        // metoda wyciagajaca obiekty w otoczeniu
        CallMethodEveryMinMax("UpdateTargetsInRadius",
                              _MIN_TIME_BTW_CHECK_TARGETS,
                              _MAX_TIME_BTW_CHECK_TARGETS);
        // metoda update'ujaca informacje o obiektach z otoczenia
        CallMethodEveryMinMax("UpdateObjectsInformation",
                              _MIN_TIME_BTW_RECALCULATE_SITUATION,
                              _MAX_TIME_BTW_RECALCULATE_SITUATION);
    }
    
    /** wyslanie komunikatu do kontrolera o zmianie obiektow */
    void UpdateObjects(ControlObject [] aObjs)
    {
        m_Controller.ObjectsListMsg(aObjs);
    }
    
    //*********************************************************
    //                      SaveGame
    //*********************************************************
    
/*    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[Sense.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_OBJECTS_SENSE)
        {
            super.SGLoadChunk(cFC);
        }
        
        cFC.delete();
    }    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[ObjectsSense.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_OBJECTS_SENSE, cFCParent);
        
        super.SGSaveChunk(cFC);
        
        cFC.delete();
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[ObjectsSense.SGUpdate]\n");
        super.SGUpdate();
    }
*/    
}
