public class TaskExecuteExtended extends Task
{
    static float _INTERP_MULT = 2f;
    
    TriggerObject m_Trigger = null;
	boolean m_bWasUsingPathfinder = false;
	boolean m_bWasMotionTypeCritical = false;
    Vector m_vStartPos = null;
    
    boolean m_bFastExec = false;
    boolean m_bExecutionStarted = false;

    
    
    public TaskExecuteExtended()
    {
        //SG
    }
    
    public TaskExecuteExtended(TriggerObject cTrigger)
    {
        this(cTrigger, false);
    }

	public TaskExecuteExtended(TriggerObject cTrigger, boolean bFast)
	{
		m_Trigger = cTrigger;
        m_bFastExec = bFast;
	}
	
    public void Perform()
    {
        super.Perform();
        
        m_bWasMotionTypeCritical = m_cAI.GetCurrActivity().IsCritical();
        m_vStartPos = m_cAI.GetMesh().GetPosition();
        m_cAI.GetCurrActivity().SetCritical(true);
        
        m_bWasUsingPathfinder = m_cAI.GetMesh().m_bUsePathfinder;
        m_cAI.GetMesh().UsePathfinder(false);
        
        // by nie zawolalo sie stop.... przez call method
        m_cAI.GetMesh().StopInterpolatePosition(false);
        m_cAI.GetMesh().StopGoTo();
        m_cAI.GetMesh().StopTurn();
        m_cAI.GetMesh().StopTrackingObject();
        
        // odswiezenie animacji i wylaczenie wolania UpdateMotion po czasie
        m_cAI.GetMesh().CallOrRemoveUpdateMotion();
        
        // UWAGA:
        // z powodu zmiany typu fizyki, jesli uzywane jest InterpolatePositionTo (tylko, 
        // jesli m_bFastExec == false trzeba zawolac StartExecution z opoznieniem, bo
        // Perform moze byc zawolane z handlera fizyki np. OnDestinationPos
        m_bExecutionStarted = (m_bFastExec);
        if (m_bFastExec)
            m_Trigger.StartExecution(m_cAI.GetMesh());
        m_cAI.StartTaskTime(0);
    }
    
    public String toString()
    {
        return "TaskExecuteExtended";
    }	    
    
    /** obsluguje komunikat wyslany z AIControllera */
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch (nNotice)
		{
    		case AIEvents._NOTICE_TASK_TIME_PASSED:
                 if (!m_bExecutionStarted)
                 {
                     m_bExecutionStarted = true;
                     m_Trigger.StartExecution(m_cAI.GetMesh());
                 }
                 else
                     PerformNextExecutionStep();
            break;            
            case AIEvents._NOTICE_TRIGGER_OBJECT_NOTIFICATION:
                 ServeTriggerObjectNotice();
			break;            
		}
    }	
    
    void ServeTriggerObjectNotice()
    {
        ExecutionInfo execInfo = m_cAI.GetCurrentExecutionInfo();
        NewAI mesh = m_cAI.GetMesh();
//        EnumExecuteMode infoInfo = new EnumExecuteMode(execInfo.iExecutionMode);
//GameObject.LogR("ServeTriggerObjectNotice: " + infoInfo + "\n");
        if (execInfo != null)
        {
            int nAnimToPlay = -1;
            boolean bLoopedAnim = false;
            Vector vDestPos = execInfo.vDestPos;
            Vector vDestDir = null;
            if (execInfo.vDestDir != null)
                vDestDir = m_Trigger.VectorLocalToWorld(execInfo.vDestDir);
                    
            switch (execInfo.iExecutionMode)
            {
            case EnumExecuteMode._SET_POSITION:
                if (vDestPos != null)
                    mesh.SetPositionWorld(m_Trigger.PointLocalToWorld(vDestPos));
                if (vDestDir != null) 
                    mesh.SetDir(vDestDir);
                PerformNextExecutionStep();
                return;
            case EnumExecuteMode._KEYBOARD_TYPE:
                nAnimToPlay = AIAnimationsHuman.A_KEYBOARD;
                break;
            case EnumExecuteMode._BUTTON_PRESS:
                nAnimToPlay = AIAnimationsHuman.A_BUTT_HIT;
                break;
            case EnumExecuteMode._MOVE_TO:
                nAnimToPlay = AIAnimationsHuman.A_FORWARD;
                bLoopedAnim = true;
                break;
            case EnumExecuteMode._TURN:
                nAnimToPlay = AIAnimationsHuman.A_ROTATE_LEFT;
                bLoopedAnim = true;
                break;
            case EnumExecuteMode._GET_IN:
                nAnimToPlay = AIAnimationsHuman.A_GET_IN;
                break;
            case EnumExecuteMode._GET_OUT:
                nAnimToPlay = AIAnimationsHuman.A_GET_OUT;
                break;
            case EnumExecuteMode._MOVE_BACK:
                nAnimToPlay = AIAnimationsHuman.A_FORWARD;
                bLoopedAnim = true;
                vDestPos = m_Trigger.PointWorldToLocal(m_vStartPos);
                break;
            }
            float fTime = 0.f;
            if (vDestDir != null)
            {
                if (m_bFastExec)
                    mesh.SetDir(vDestDir);
                else
                {
                    if (execInfo.iExecutionMode == EnumExecuteMode._TURN)
                    {
                        fTime = Math.max(fTime, mesh.Turn(vDestDir));
                        nAnimToPlay = -1;// animacja jest odpalana przez Turn
                    }
                    else
                    {
                        float fAngle = (float)Math.acos(MathUtils.crop(MathUtils.dot2D(mesh.GetForwardVectorVolatile(), 
                                                                                       vDestDir), 
                                                                       -1, 1));
                        float fAngSpeed = mesh.m_aMotionAngularVelocity[mesh.m_Motion.GetSpeed()];
                        fTime = Math.max(fTime, (float)Math.abs(fAngle) / fAngSpeed);
                        mesh.SetMaxAngSpeed(fAngSpeed);
                        mesh.SetDestDir(vDestDir);
                    }
                }
            }
			
            if (vDestPos != null)
            {
                Vector vWorldDestPos = m_Trigger.PointLocalToWorld(vDestPos);
                
                if (execInfo.iExecutionMode == EnumExecuteMode._GET_IN)
                {
                    if (nAnimToPlay != -1)
                    {
                        int nNonAIAnimID = m_cAI.GetMesh().m_Anims.m_aAnims[nAnimToPlay];
                        // sadzamy ai dokladnie na siedzenie
                        int iFirstFrame = m_cAI.GetMesh().GetFirstFrame(nNonAIAnimID);
                        int iLastFrame = m_cAI.GetMesh().GetLastFrame(nNonAIAnimID);
                        m_cAI.GetMesh().SetFrame(nNonAIAnimID, iLastFrame, -1);
                        Vector vDiff = m_cAI.GetMesh().GetPosition();
                        int iElemID = m_cAI.GetMesh().GetElementID("Bip01 Pelvis");
                        vDiff.Sub(m_cAI.GetMesh().GetElementPos(iElemID));
                        vWorldDestPos.Add(vDiff);
                        
                        m_cAI.GetMesh().SetFrame(nNonAIAnimID, iFirstFrame, -1);
                    }
                }
                
				if (execInfo.iExecutionMode == EnumExecuteMode._GET_OUT)
				{
					Vector vInCarPos = m_cAI.GetMesh().GetPosition();
					
					// sprawdzamy, gdzie mamy wysiasc
					Vector vMove = new Vector(Vector.Up);
					vMove.Negate();
					vMove.Mul(150f);
					Vector vStart = new Vector(vWorldDestPos);
					vStart.fY = vInCarPos.fY + 100f;
					
					m_cAI.GetMesh().SetPositionWorld(vStart);
					m_cAI.GetMesh().MoveByEllipsoid(vMove);
					vWorldDestPos.fY = vStart.fY + vMove.fY + 5f;
					m_cAI.GetMesh().SetPositionWorld(vInCarPos);
				}

                /** Pozycja (w pionie) triggera moze byc kijowa - lepiej utrzymac poczatkowa
                 */
                if (execInfo.iExecutionMode == EnumExecuteMode._MOVE_TO)
                    vWorldDestPos.fY = m_vStartPos.fY;
                if (m_bFastExec)
                    mesh.SetPositionWorld(vWorldDestPos);
                else
                {
                    fTime = Math.max(fTime, 
                                     mesh.InterpolatePositionTo(vWorldDestPos,
                                                                vDestDir,
                                                                mesh.GetSpeedValueForActualMotion() * _INTERP_MULT, 
                                                                true));
                }
            }
            
            if (!m_bFastExec)
            {
                if (nAnimToPlay != -1)
                {
                    if (bLoopedAnim)
                        mesh.PlayAIAnimLoop(nAnimToPlay);
                    else
                    {
                        mesh.PlayAIAnim(nAnimToPlay);
                        fTime = Math.max(fTime, mesh.GetAIAnimLength(nAnimToPlay));
                    }
                }
                m_cAI.StartTaskTime(fTime);
            }
            else
            {
                if (nAnimToPlay != -1)
                {
                    int nNonAIAnimID = m_cAI.GetMesh().m_Anims.m_aAnims[nAnimToPlay];
                    int iLastFrame = m_cAI.GetMesh().GetLastFrame(nNonAIAnimID);
                    m_cAI.GetMesh().SetFrame(nNonAIAnimID, iLastFrame, -1);
                    m_cAI.GetMesh().StopAIAnim(false);
                }
                PerformNextExecutionStep();
            }
        }
        else
            PerformNextExecutionStep();
    }
    
    
    void PerformNextExecutionStep()
    {
        ExecutionInfo execInfo = m_cAI.GetCurrentExecutionInfo();
        m_Trigger.ExecutionStepDone(m_cAI.GetMesh(), 
                                    execInfo);
        if (execInfo == null)
        {
            if (m_bWasUsingPathfinder)
                m_cAI.GetMesh().UsePathfinder(true);
            m_cAI.GetCurrActivity().SetCritical(m_bWasMotionTypeCritical);
            m_cAI.Notify(AIEvents._NOTICE_END_TASK);
        }
    }

    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskExecuteExtended.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_EXECUTE_EXTENDED)
        {
            super.SGLoadChunk(cFC);
            m_Trigger = (TriggerObject)cFC.LoadObjectRef();
            m_bWasUsingPathfinder = cFC.LoadBool();
            m_bWasMotionTypeCritical = cFC.LoadBool();
            m_vStartPos = cFC.LoadVector();
            m_bFastExec = cFC.LoadBool();
            m_bExecutionStarted = cFC.LoadBool();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskExecuteExtended.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_EXECUTE_EXTENDED, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveObjectRef(m_Trigger);
        cFC.SaveBool(m_bWasUsingPathfinder);
        cFC.SaveBool(m_bWasMotionTypeCritical);
        cFC.SaveVector(m_vStartPos);
        cFC.SaveBool(m_bFastExec);
        cFC.SaveBool(m_bExecutionStarted);
        
        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskExecuteExtended.SGUpdate]\n");
        super.SGUpdate();
        //
    }
    
}
