public class TaskGetInOut extends Task
{
	static Vector _MOVE_DIFF = new Vector(0, -80, 0);
	
	static int _GET_IN_TYPE = 0;
	static int _GET_OUT_TYPE = 1;
	
	int m_iType = _GET_IN_TYPE;
	
	int m_iAnim = -1;
	float m_fMoveTime = 0f;
	Vector m_vEndPos = null;
	Vector m_vStartPos = null;
	Vector m_vCurrPos = null;
	Vector m_vDir = null;
	float m_fStartTime = 0;
	int m_iFlags = 0;
    
    String m_sAISeatName = "Seat"; // nazwa meshelementu siedzienia obiektu (inaczej "dupy"), ktory ma usiasc
    int m_iAISeatID = -1;
    
    
    public TaskGetInOut()
    {
        //SG
    }
	
    TaskGetInOut(int iType, int iAnimation)
	{
		m_iType = iType;
		m_iAnim = iAnimation;
		SetUpdating(0.01f);
	}
    
    /** wykonuje zadanie */
    public void Perform()
    {
		super.Perform();

		if (m_iAnim != -1)
		{
			m_fMoveTime = m_cAI.GetMesh().m_Anims.GetLength(m_iAnim) * 0.8f; // wsiadamy troche szybciej
			m_cAI.GetMesh().PlayAIAnim(m_iAnim, 1);
		}
		
		m_vStartPos = m_cAI.GetMesh().GetPosition();
		
		if (m_iType == _GET_IN_TYPE)
		{
			if (m_cAI.GetSeat() != null)
				((TriggerObject) m_cAI.GetSeat().GetOccupiedObject()).Execute(m_cAI.GetMesh());
			
			m_vEndPos = new Vector(m_cAI.GetSeat().GetSeatPos());
            m_iAISeatID = m_cAI.GetSeat().GetOccupiedObject().GetElementID(m_sAISeatName);
            if (m_iAISeatID != -1)
            {
                Vector vDiff = new Vector(m_vStartPos);
                vDiff.Sub(m_cAI.GetMesh().GetElementPos(m_iAISeatID));
                m_vEndPos.Add(vDiff);
            }
		}
		else
			m_vEndPos = new Vector(m_cAI.GetSeat().GetStartPos());
        if (m_iAISeatID == -1)
    		m_vEndPos.Add(_MOVE_DIFF);
		m_vCurrPos = new Vector(m_vStartPos);
		m_vDir = new Vector(m_vEndPos);
		m_vDir.Sub(m_vStartPos);

		m_fStartTime = m_cAI.GetMesh().GetTime();
		
		m_iFlags = m_cAI.GetMesh().GetFlags();
		m_cAI.GetMesh().ClearFlag(EFlags._COLLISION);
		m_cAI.GetMesh().ClearFlag(EFlags._BLOCK_TRACE);
		m_cAI.GetMesh().ClearFlag(EFlags._STOPPED_BY_PATHFINDER);
		
		m_cAI.StartTaskTime(m_fMoveTime);
    }
	
	void Update()
	{
		float fTime = m_cAI.GetMesh().GetTime();
		
		m_vCurrPos.Set(m_vDir);
		float fValue = (fTime - m_fStartTime) / m_fMoveTime;
		if (fValue > 1f)
			fValue = 1f;
		m_vCurrPos.Mul(fValue);
		m_vCurrPos.Add(m_vStartPos);
		
		m_cAI.GetMesh().SetPositionWorld(m_vCurrPos);
	}
    
	void OnEndTask()
	{
		super.OnEndTask();
		
		m_cAI.GetMesh().SetPositionWorld(m_vEndPos);
		m_cAI.GetMesh().SetFlags(m_iFlags);
	}
	
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch(nNotice)
		{
		    case AIEvents._NOTICE_TASK_TIME_PASSED:
	    		m_cAI.Notify(AIEvents._NOTICE_END_TASK);
    		break;            
		}
    }	    
	
    public String toString()
    {
        return 	"TaskGetInOut";
    }
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskGetInOut.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_GET_IN_OUT)
        {
			super.SGLoadChunk(cFC);
            m_iAnim = cFC.LoadInt();
            m_fMoveTime = cFC.LoadFloat();
            m_vEndPos = cFC.LoadVector();
            m_vStartPos = cFC.LoadVector();
            m_vCurrPos = cFC.LoadVector();
            m_vDir = cFC.LoadVector();
            m_fStartTime = cFC.LoadFloat();
            m_iFlags = cFC.LoadInt();
            m_iAISeatID = cFC.LoadInt();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskGetInOut.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_GET_IN_OUT, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveInt(m_iAnim);
        cFC.SaveFloat(m_fMoveTime);
        cFC.SaveVector(m_vEndPos);
        cFC.SaveVector(m_vStartPos);
        cFC.SaveVector(m_vCurrPos);
        cFC.SaveVector(m_vDir);
        cFC.SaveFloat(m_fStartTime);
        cFC.SaveInt(m_iFlags);
        cFC.SaveInt(m_iAISeatID);
        
        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskGetInOut.SGUpdate]\n");
        super.SGUpdate();
        //
    }

}
