public class TaskGoToObject extends Task
{
    /** dzielnik odleglosci, by obliczyc czas przebiegniecia z punktu do punktu	 */
	final static float _SPEED = 500;  
	
	/** docelowy obiekt */
	ControlObject m_Object = null;
	
	/** czas trwaia zadania */
	float m_fEndTime = 0.0f;
	
	/** minimalna odleglosc, na jaka nalezy podejsc do obiektu */
	float m_fMinDist = 0.f;
	
    /** Znacznik, ze sledzimy obiekt - czekamy na uplyniecie czasu zadania - nie przerywamy
     *  po dojsciu
     */
    boolean m_bTrackObjectPos;
    
    /** czas, co jaki updateuje sie pozycja obiektu */	
	final float _UPDATE_TIME = 1.f;
    
    Vector m_vLastObjPos = null;
				
    public TaskGoToObject()
    {
        //SG
    }
    
	public TaskGoToObject(ControlObject cObject, float fEndTime, float fMinDist)
    {
        this(cObject, fEndTime, fMinDist, false);
    }
    
	public TaskGoToObject(ControlObject cObject, float fEndTime, float fMinDist, boolean bTrackObjectPos)
	{
		m_Object = cObject;
		m_fEndTime = fEndTime;
		m_fMinDist = fMinDist;
        m_bTrackObjectPos = bTrackObjectPos;
        SetUpdating(_UPDATE_TIME);
	}
    
    /** wykonuje zadanie     */
    public void Perform()
    {
		super.Perform();		

        if (m_fEndTime <= 0)
        {
            float fDist;
            
            NewAI mesh = m_cAI.GetMesh();
            
            if (mesh.GetAIGroup() != null)
                fDist = mesh.GetDistanceTo(mesh.GetInFormationPosition(m_Object.GetPositionVectorVolatile(), null));
            else
		        fDist = mesh.GetDistanceTo(m_Object);				
		    m_fEndTime = (fDist / _SPEED) * 1.5f;				
        }
        
		m_cAI.StartTaskTime(m_fEndTime);
        
		Update();
    }
	
	public void Update()
	{
		if (m_Object != null)
		{
			Vector vPos = m_Object.GetPosition();
            
			if (vPos == null)
				Cancel(false);
            else
                vPos = m_cAI.GetMesh().GetInFormationPosition(vPos, null);
			
			boolean bMove = m_cAI.GetMesh().m_Motion.IsMovingLinear();
			
			if (m_cAI.GetMesh().GetDistanceTo(vPos) > m_fMinDist)
            {
                if ( (m_vLastObjPos == null) ||
                     ((m_vLastObjPos != null) && (vPos.Distance2(m_vLastObjPos) > 1f)) )
    				m_cAI.GetMesh().GoToPosition(vPos);
            }
			else
			{
				if (bMove)
                {
                    m_cAI.GetMesh().StopGoTo();
                    ServeNotice(AIEvents._NOTICE_DEST_POS);
                }
			}
            
            if (m_vLastObjPos == null)
                m_vLastObjPos = new Vector(vPos);
            else
                m_vLastObjPos.Set(vPos);
		}
	}
	
    public void Cancel(boolean bFast)
    {
        m_cAI.GetMesh().StopGoTo();
        super.Cancel(bFast);
    }
    
    /** obsluguje komunikat wyslany z AIControllera     */
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch(nNotice)
		{
        case AIEvents._NOTICE_DEST_POS:
			if (!m_bTrackObjectPos)
                m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            break;
		case AIEvents._NOTICE_TASK_TIME_PASSED:
            m_cAI.GetMesh().StopGoTo();
            m_cAI.Notify(AIEvents._NOTICE_END_TASK);
			break;            
		}
    }	    

    public String toString()
	{
		return 	"GoToObject("+m_Object.GetName()+","+m_fEndTime+","+m_fMinDist+")";
	}
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskGoToObject.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_GO_TO_OBJECT)
        {
			super.SGLoadChunk(cFC);
            m_Object = (ControlObject)cFC.LoadObjectRef();
            m_fEndTime = cFC.LoadFloat();
            m_fMinDist = cFC.LoadFloat();
            m_bTrackObjectPos = cFC.LoadBool();
            m_vLastObjPos = cFC.LoadVector();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskGoToObject.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_GO_TO_OBJECT, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveObjectRef(m_Object);
	    cFC.SaveFloat(m_fEndTime);
	    cFC.SaveFloat(m_fMinDist);
	    cFC.SaveBool(m_bTrackObjectPos);
	    cFC.SaveVector(m_vLastObjPos);
	
        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskGoToObject.SGUpdate]\n");
        super.SGUpdate();
        //
    }
    
}
