public class TaskRotateHead_W extends Task
{
    protected float m_fHorizAngle,
        m_fVertAngle,
        m_fMorphTime,
        m_fOldHorizAnimMorphTime,
        m_fOldVertAnimMorphTime;
    protected NewAIWalker m_WalkerMesh;
        
    public TaskRotateHead_W()
    {
        //SG
    }
    
    public TaskRotateHead_W(float fHorizAngle, float fVertAngle)
    {
        this(fHorizAngle, fVertAngle, -1f);
    }

    /** Obrocenie glowy walkera DO podanych katow w czasie zaleznym od fQuasiSpeed
     * @param fHorizAngle kat w poziomie do jakiego ma obrocic sie glowa walkera
     * @param fVertAngle kat w pionie do jakiego ma obrocic sie glowa walkera
     * @param fMorphTime czas przeznaczony na obrot do tego kata
     */
    public TaskRotateHead_W(float fHorizAngle, float fVertAngle, float fMorphTime)
    {
        m_fHorizAngle = fHorizAngle;
        m_fVertAngle = fVertAngle;
        m_fMorphTime = fMorphTime;
    }

    
	/** wykonuje zadanie     */
    public void Perform()
    {
        super.Perform();

        m_WalkerMesh = (NewAIWalker)m_cAI.GetMesh();
        // zapamietaj czasy morphingu
        m_fOldHorizAnimMorphTime = m_WalkerMesh.GetAnimMorphTime(m_WalkerMesh.m_nHorizRotateElem);
        m_fOldVertAnimMorphTime = m_WalkerMesh.GetAnimMorphTime(m_WalkerMesh.m_nVertRotateElem);
        
        float fTime;
        if (m_fMorphTime > 0)
        {
            fTime = m_fMorphTime;
            // ustaw nowe na podstawie predkosci obrotowej
            m_WalkerMesh.SetAnimMorphTime(m_WalkerMesh.m_nHorizRotateElem, fTime, true);
            m_WalkerMesh.SetAnimMorphTime(m_WalkerMesh.m_nVertRotateElem, fTime, true);
        }
        else
        {
            fTime = (float)Math.max(m_fOldHorizAnimMorphTime,
                                    m_fOldVertAnimMorphTime);
        }
        m_WalkerMesh.RotateHead(m_fHorizAngle, m_fVertAngle);
        m_cAI.StartTaskTime(fTime);
    }
	
    protected void CleanUp()
    {
        if (m_fMorphTime > 0)
        {
            m_WalkerMesh.SetAnimMorphTime(m_WalkerMesh.m_nHorizRotateElem, m_fOldHorizAnimMorphTime, true);
            m_WalkerMesh.SetAnimMorphTime(m_WalkerMesh.m_nVertRotateElem, m_fOldVertAnimMorphTime, true);
        }
    }
    
    public void Cancel(boolean bQuick)
    {
        super.Cancel(bQuick);
        m_WalkerMesh.RotateHead(0,0);
        CleanUp();
    }
    
    /** obsluguje komunikat wyslany z AIControllera     */
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch(nNotice)
		{
        case AIEvents._NOTICE_TASK_TIME_PASSED:
            CleanUp();
			m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            break;
		}
    }	    

    public String toString()
	{
		return 	getClass().getName() + "(" +
                m_fHorizAngle + ", " +
                m_fVertAngle + ", " +
                m_fMorphTime + ")";
	}
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskRotateHead_W.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_ROTATE_HEAD_W)
        {
            super.SGLoadChunk(cFC);
            m_fHorizAngle = cFC.LoadFloat();
            m_fVertAngle = cFC.LoadFloat();
            m_fMorphTime = cFC.LoadFloat();
            m_fOldHorizAnimMorphTime = cFC.LoadFloat();
            m_fOldVertAnimMorphTime = cFC.LoadFloat();
            m_WalkerMesh = (NewAIWalker)cFC.LoadObjectRef();
      }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskRotateHead_W.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_ROTATE_HEAD_W, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveFloat(m_fHorizAngle);
        cFC.SaveFloat(m_fVertAngle);
        cFC.SaveFloat(m_fMorphTime);
        cFC.SaveFloat(m_fOldHorizAnimMorphTime);
        cFC.SaveFloat(m_fOldVertAnimMorphTime);
        cFC.SaveObjectRef(m_WalkerMesh);

        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskRotateHead_W.SGUpdate]\n");
        super.SGUpdate();
        //
    }
    
}
