public class MathUtils
{
    
    public final static float _SMALL_03 = 0.001f;
    
    /////////////////////////////////////////////////////////////////////////////
    
    public final static boolean CanHappen(int nPercentage)
    {
        return (GameObject.RandomInt(100) < nPercentage);
    }
    
    public final static int sign(float fValue)
    {
        return (fValue > 0) ? 1 : ((fValue < 0) ? -1 : 0);
    }
    
    public final static float crop(float fValue, float fMin, float fMax)
    {
        return (fValue < fMin) ? fMin : (fValue > fMax) ? fMax : fValue;
    }
    
    public final static float dist2D(Vector v1, Vector v2)
    {
        float fDx = v1.fX - v2.fX,
              fDz = v1.fZ - v2.fZ;
        return (float)Math.sqrt(fDx * fDx + fDz * fDz);
    }
	
    public final static float dist2D2(Vector v1, Vector v2)
    {
        return (v1.fX - v2.fX) * (v1.fX - v2.fX) + (v1.fZ - v2.fZ) * (v1.fZ - v2.fZ);
    }
    
    
    /** Obliczenie iloczynu skalarnego rzutow dwoch wektorow na plaszczyzne XZ
     * Wektory nie sa modyfikowane. Iloczyn normalizuje rzuty wektorow.
     */
    public final static float dot2D(Vector v1, Vector v2)
    {
        float x1 = v1.fX, z1 = v1.fZ,
            x2 = v2.fX, z2 = v2.fZ,
            fLen1 = (float)Math.sqrt(x1 * x1 + z1 * z1),
            fLen2 = (float)Math.sqrt(x2 * x2 + z2 * z2),
            fOOLen;

        if (fLen1 > 0)
        {
            fOOLen = 1.f / fLen1;
            x1 *= fOOLen;
            z1 *= fOOLen;
        }
        else
            return 0;
        
        if (fLen2 > 0)
        {
            fOOLen = 1.f / fLen2;
            x2 *= fOOLen;
            z2 *= fOOLen;
        }
        else
            return 0;
        
        return x1 * x2 + z1 * z2;
    }
	
	/** vEnd - koniec wektora, dla ktorego szukamy rzutu na prosta
	 * wyznaczona przez wektor znormalizowany vDir
	 * vStart - poczatek rzutowanego wektora
	 * vDir - znormalizowany kierunek prostej
	 */
	public static Vector ProjVector(Vector vStart, Vector vEnd, Vector vDir)
	{
		Vector vTemp = new Vector(vEnd);
		vTemp.Sub(vStart);
		float fTemp = vTemp.Len();
		vTemp.Normalize();
		Vector vReturn = new Vector(vDir);
		vReturn.Mul(vTemp.Dot(vDir) * fTemp);
		vReturn.Add(vStart);
		return vReturn;
	}
    
    
    protected static Vector m_CLPI_vDirProj = new Vector();
    protected static Vector m_CLPI_vDirOrig = new Vector();
    protected static Vector m_CLPI_vRet = new Vector();
    /** Obliczenie punktu przeciecia prostej wyznaczonej przez dwa punkty z plaszczyzna
     * (do obliczenia rzutu punktu na plaszczyzne w kierunku od podanego punktu)
     * @param vLineStart pierwszy punkt prostej
     * @param vLineEnd drugi punkt prostej (rzutowany)
     * @param vPlaneOrigin punkt na plaszczyznie
     * @param vPlaneNormal normalna do plaszczyzny (nie musi byc znormalizowana)
     * @return [volatile] punkt przeciecia prostej z plaszczyzna (rzut punktu vLineEnd na plaszczyzne)
     */
    public static Vector CalcLinePlaneIntersection(Vector vLineStart, Vector vLineEnd, 
                                                   Vector vPlaneOrigin, Vector vPlaneNormal)
    {
        m_CLPI_vDirProj.Set(vLineEnd);
        m_CLPI_vDirProj.Sub(vLineStart);
        
        m_CLPI_vDirOrig.Set(vPlaneOrigin);
        m_CLPI_vDirOrig.Sub(vLineStart);

        float fNdotDirProj = vPlaneNormal.Dot(m_CLPI_vDirProj),
            fNdotDirOrig = vPlaneNormal.Dot(m_CLPI_vDirOrig);
        
        if (Math.abs(fNdotDirProj) > _SMALL_03)
        {
            m_CLPI_vRet.Set(m_CLPI_vDirProj);
            m_CLPI_vRet.Mul(fNdotDirOrig / fNdotDirProj);
            m_CLPI_vRet.Add(vLineStart);
        }
        else
        {
            GameObject.CrashLog("ERROR: CalcLinePlaneIntersection - line parrallel to plane!\n");
            m_CLPI_vRet.Set(Vector.Zero);
        }

        return m_CLPI_vRet;
    }
    
    
    private static Vector m_vRandomPos = new Vector();

    /** Obliczenie losowego wektora z ograniczeniem na dlugosc
     * @param fMinDist minimalna odleglosc pozycji koncowej od poczatkowej
     * @param fMaxDist maksymalna odleglosc pozycji koncowej od poczatkowej
     * @return obliczona pozycja (do wykorzystania tymczasowego - volatile)
     */
    public final static Vector CalcRandomMoveVolatile(float fMinDist, float fMaxDist)
    {
        m_vRandomPos.fX = GameObject.RandomFloat(-1f, 1f);
        m_vRandomPos.fY = 0f;
        m_vRandomPos.fZ = GameObject.RandomFloat(-1f, 1f);
        float fLen = m_vRandomPos.Len();
        if (fLen < _SMALL_03)
        {
            m_vRandomPos.Set(Vector.Forward);
            fLen = 1.f;
        }
        m_vRandomPos.Mul(GameObject.RandomFloat(fMinDist, fMaxDist) / fLen);
        return m_vRandomPos;
    }
}
