/** klasa odpowiedzialna za objectives, metody w niej sie znajdujace
 * wystarcza, by sterowac objectivami i wszystkimi zaznaczeniami lokacji
 */
public class ObjectivesManager extends NetObjectContainer implements ISaveGame
{
    static int _START_AUTO_NUMBER = 1000000;
    
	/** przesuniecie w X tekstu sprite'a glownego / grupy */
	static int _TEXT_INDENT_X = 10;
	/** przesuniecie w X tekstu sprite'a podrzednego */
	static int _TEXT_SUB_INDENT_X = 10;
	/** przesuniecie w Y tekstu pierwszego wyswietlanego na mapie sprite'a */
	static int _TEXT_START_INDENT_Y = 5;
	/** odstepy pomiedzy kolejnymi tekstami (objectivami) */
	static int _TEXT_INDENT_Y = 5;
	
	/** lista obiektow klasy SingleObjective - wszystkich objectives, jest ona
	 * wypelniana na podstawie informacji o objectives w gameinitinfosingle,
	 * czyli na podstawie obiektow klasy ObjectiveDescription */
	java.util.Vector m_Objectives = null;
	
	/** lista obiektow DefLocation - wszystkich lokacji, ktore maja byc
	 * wyswietlone na mapie, lista ta tworzona jest na podstawie informacji
	 * o lokacjach w gameinitinfosingle, czyli na podstawie obiektow klasy
	 * LocationDescription
	 */
	java.util.Vector m_Locations = null;
	
	/** modul, do ktorego beda nalezaly tworzone przez managera sprite'y */
    static ChromeModule m_Module = null;
	
    String m_sLastMessage = "";
    float m_fLastMessageTime = 0;
    
	ObjectivesManager()
	{
		//SG!
	}
    
    PlayerHUD GetMainPlayer()
    { 
        if ( m_Module == null )
            return null;
        return  (m_Module.GetMainPlayer() instanceof PlayerHUD) ?
                (PlayerHUD)m_Module.GetMainPlayer() : null;
    }
    
    NetGameInfo GetNetGameInfo()
    {
        if ( m_Module instanceof ModuleNet )
            return ((ModuleNet)m_Module).GetGameInfo();
        return null;
    }

    
	boolean m_bSpritesAttached = false;
	
    static int GetNextAutoNumber()
    {
        return _START_AUTO_NUMBER++;
    }
    
	/** ustalenie objectiva glownego jako spelnionego
	 */
	void SetAccomplished(int iObjNo)
	{
		SetAccomplished(iObjNo, 0);
	}
	
	/** ustalenie objectiva (tez podobjectiva) jako spelnionego
	 */
	void SetAccomplished(int iGroupNo, int iSubObjNo)
	{
		SingleObjective obj = FindObjective(iGroupNo, iSubObjNo);
		
		if (obj != null)
		{
			obj.SetAccomplished();
	        TextMessage("&OBJECTIVE_ACCOMPLISHED&");
            
            if ( m_bOnServer )
            {
                ServerObjectiveUpdated( _OBJECTIVE_ACCOMPLISHED, 
                                        iGroupNo, 
                                        iSubObjNo );
            }
		}
	}
	
	/** ustalenie objectiva glownego (ale nie posiadajacego dzieci) jako spelnionego
	 */
	void SetUnaccomplished(int iObjNo)
	{
		SetUnaccomplished(iObjNo, 0);
	}
	
	/** ustalenie objectiva (tez podobjectiva) jako spelnionego
	 */
	void SetUnaccomplished(int iGroupNo, int iSubObjNo)
	{
		SingleObjective obj = FindObjective(iGroupNo, iSubObjNo);
		
		if (obj != null)
		{
			obj.SetUnaccomplished();
	        TextMessage("&OBJECTIVE_UNACCOMPLISHED&");
            
            if ( m_bOnServer )
            {
                ServerObjectiveUpdated( _OBJECTIVE_UNACCOMPLISHED, 
                                        iGroupNo, 
                                        iSubObjNo );
            }
		}
	}
    
	/** wyswietlenie wszystkich objectives na hudzie
	 */
	void EnableAllObjectives()
	{
		CheckSpritesAttached();
			
		for (int i = 0; i < m_Objectives.size(); i++)
		{
			SingleObjective obj = (SingleObjective) m_Objectives.elementAt(i);
			if (!obj.m_bIsSubObjective)
				obj.Enable();
		}
		
		for (int i = 0; i < m_Locations.size(); i++)
		{
			DefLocation loc = (DefLocation) m_Locations.elementAt(i);
			if ( !loc.HasObjective() )
				EnableLocation(loc.m_iGroupNo, loc.m_iObjectiveNo,false);
		}
	}
	
	void DisableAllObjectives()
	{
		CheckSpritesAttached();
			
		for (int i = 0; i < m_Objectives.size(); i++)
		{
			SingleObjective obj = (SingleObjective) m_Objectives.elementAt(i);
			obj.Disable();
		}
		
		for (int i = 0; i < m_Locations.size(); i++)
		{
			DefLocation loc = (DefLocation) m_Locations.elementAt(i);
			if ( !loc.HasObjective() )
				DisableLocation(loc.m_iGroupNo, loc.m_iObjectiveNo,false);
		}
	}
	
	/** wyswietlenie glownego objective na hudzie (jezeli jest to objective
	 * posiadajacy dzieci, to wyswietlenie takze jego dzieci)
	 */
	void EnableObjective(int iObjNo)
	{
		EnableObjective(iObjNo, 0);
	}
	
	/** wyswietlenie objective na hudzie (jezeli jest to objective
	 * posiadajacy dzieci, to wyswietlenie takze jego dzieci)
	 */
	void EnableObjective(int iGroupNo, int iSubObjNo)
	{
		CheckSpritesAttached();
		
		SingleObjective obj = FindObjective(iGroupNo, iSubObjNo);
		
		if (obj != null)
			obj.Enable();
		RecalculatePositions();
        
        TextMessage("&OBJECTIVE_UPDATED&");
        
        if ( m_bOnServer )
        {
            ServerObjectiveUpdated( _OBJECTIVE_ENABLED, 
                                    iGroupNo, 
                                    iSubObjNo );
        }
	}
	
    
	/** schowanie glownego objective na hudzie (jezeli jest to objective
	 * posiadajacy dzieci, to schowanie takze jego dzieci)
	 */
	void DisableObjective(int iObjNo)
	{
		DisableObjective(iObjNo, 0);
	}
	
	/** schowanie objective na hudzie (jezeli jest to objective
	 * posiadajacy dzieci, to schowanie takze jego dzieci)
	 */
	void DisableObjective(int iGroupNo, int iSubObjNo)
	{
		CheckSpritesAttached();
		
		SingleObjective obj = FindObjective(iGroupNo, iSubObjNo);
		
		if (obj != null)
			obj.Disable();
		RecalculatePositions();
        
        if ( m_bOnServer )
        {
            ServerObjectiveUpdated( _OBJECTIVE_DISABLED, 
                                    iGroupNo, 
                                    iSubObjNo );
        }
	}

    /** Sprawdza czy podany objective jest wlaczony
     */
    boolean IsEnabledObjective(int iGroupNo)
    {
        return IsEnabledObjective(iGroupNo, 0);
    }

    /** Sprawdza czy podany objective jest wlaczony
     */
    boolean IsEnabledObjective(int iGroupNo, int iSubObjNo)
	{
        SingleObjective obj = FindObjective(iGroupNo, iSubObjNo);
		
        return (obj != null) ? obj.IsEnabled() : false;
    }
    
	/** wyswietlenie lokacji na mapie (ktora nie jest podpieta do zadnego
	 * objective - jezeli jest podpieta, to wyswietli sie automatycznie wraz
	 * z objective). Mozna wyswietlac tez lokacje zwiazane z objective (ale 
	 * wtedy opis objective moze zostac niewyswietlony)
	 */
    void EnableLocation(int iGroupNo)
    {
        EnableLocation( iGroupNo, false );
    }
	void EnableLocation(int iGroupNo, boolean bNet)
	{
		EnableLocation(iGroupNo, 0, bNet);
	}
	
	/** wyswietlenie lokacji na mapie (ktora nie jest podpieta do zadnego
	 * objective - jezeli jest podpieta, to wyswietli sie automatycznie wraz
	 * z objective). Mozna wyswietlac tez lokacje zwiazane z objective (ale 
	 * wtedy opis objective moze zostac niewyswietlony)
	 */
	void EnableLocation(int iGroupNo, int iSubLocNo, boolean bNet)
	{
		CheckSpritesAttached();
		
		DefLocation loc = FindLocation(iGroupNo, iSubLocNo);
		
		if (loc != null)
		{
            loc.Enable( bNet );
            
            if ( loc.CanShow() )
			    loc.ShowLocation();
            
            if ( m_bOnServer )
            {
                ServerObjectiveUpdated( _LOCATION_ENABLED, 
                                        iGroupNo, 
                                        iSubLocNo );
            }
		}
	}
    
	/** schowanie lokacji na mapie (tez prawdopodobnie nie podpietej)
	 */
    void DisableLocation(int iGroupNo)
	{
		DisableLocation(iGroupNo, 0, false);
	}
    
	void DisableLocation(int iGroupNo,boolean bNet)
	{
		DisableLocation(iGroupNo, 0, bNet);
	}
	
	/** schowanie lokacji na mapie (tez prawdopodobnie nie podpietej)
	 */
	void DisableLocation(int iGroupNo, int iSubLocNo,boolean bNet)
	{
		CheckSpritesAttached();
		
		DefLocation loc = FindLocation(iGroupNo, iSubLocNo);
		
		if (loc != null)
		{
			loc.Disable( bNet );
			loc.HideLocation();
            
            if ( m_bOnServer )
            {
                ServerObjectiveUpdated( _LOCATION_DISABLED, 
                                        iGroupNo, 
                                        iSubLocNo );
            }
		}
	}
    
    /** podaje czy location jest aktywny czy nie
     */
    boolean IsEnabledLocation(int iGroupNo)
    {
        return IsEnabledLocation(iGroupNo, 0);
    }
    
    boolean IsEnabledLocation(int iGroupNo, int iSubLocNo)
	{
        DefLocation loc = FindLocation(iGroupNo, iSubLocNo);
		
        return (loc != null) ? loc.IsEnabled() : false;
    }
    
	/** ustawienie lokacji na mapie jako wykonanej
	 */
	void AccomplishedLocation(int iGroupNo)
	{
		AccomplishedLocation(iGroupNo, 0);
	}
	
	/** ustawienie lokacji na mapie jako wykonanej
	 */
	void AccomplishedLocation(int iGroupNo, int iSubLocNo)
	{
		DefLocation loc = FindLocation(iGroupNo, iSubLocNo);
		
		if (loc != null)
		{
			loc.TrackObjectPosition(false);
			loc.SetAccomplishedTex();
		}
	}

	/** ustawienie lokacji na mapie jako nie wykonanej
	 */
	void UnaccomplishedLocation(int iGroupNo)
	{
		UnaccomplishedLocation(iGroupNo, 0);
	}
	
	/** ustawienie lokacji na mapie jako niewykonanej
	 */
	void UnaccomplishedLocation(int iGroupNo, int iSubLocNo)
	{
		DefLocation loc = FindLocation(iGroupNo, iSubLocNo);
		
		if (loc != null)
		{
			loc.TrackObjectPosition(true);
			loc.SetUnaccomplishedTex();
		}
	}

    boolean IsAccomplishedLocation(int iGroupNo)
    {
        return IsAccomplishedLocation(iGroupNo, 0);
    }
    
    boolean IsAccomplishedLocation(int iGroupNo, int iSubLocNo)
    {
    	DefLocation loc = FindLocation(iGroupNo, iSubLocNo);
		
		if (loc != null)
		{
			return loc.IsAccomplished();
		}
        return false;
    }
    
	ObjectivesManager(ChromeModule mod)
	{
		m_Module = mod;
	}
	
	/** metoda przeksztalcajaca opisy objectives i locations na odpowiednie sprite'y -
	 * teksty i zaznaczenia lokacji w sprite'ach */
	void Init(java.util.Vector objectives, java.util.Vector locations)
	{
        //GameObject.CrashLog("[ObjectivesManager.Init]\n");
        
		// objectives
		SingleObjective.SetObjectivesManager(this);
		m_Objectives = new java.util.Vector();
		for (int i = 0; i < objectives.size(); i++)
		{
			ObjectiveDescription objDes = (ObjectiveDescription) objectives.elementAt(i);
			SingleObjective obj = null;
			
			if (objDes.m_iObjectiveGroupNo == 0)
			{
				// sprawdzamy, czy mamy do czynienia z grupa objectives czy objective glownym
				if (objDes.m_bIsGroup)
					obj = new SingleObjective(objDes.m_iObjectiveNo, objDes.m_ObjectiveInfo, true);
				else
					obj = new SingleObjective(objDes.m_iObjectiveNo, objDes.m_ObjectiveInfo, false);
				obj.m_bEnableOnStart = true;
			}
			else
			{
				obj = new ObjectiveGroup(objDes.m_iObjectiveNo,
										 objDes.m_iObjectiveGroupNo,
										 objDes.m_ObjectiveInfo);
				obj.m_bEnableOnStart = true;
			}
					
			m_Objectives.addElement(obj);
		}
		
		// locations
		m_Locations = new java.util.Vector();
		for (int i = 0; i < locations.size(); i++)
		{
			LocationDescription locDes = (LocationDescription) locations.elementAt(i);
			
			if (locDes.m_TrackedName == null)
			{
				DefLocation location = (DefLocation) m_Module.CreateObject("DefLocation");
				location.m_bEnableOnStart = true;
				location.InitLocation(locDes.m_iLocationGroupNo,
									  locDes.m_iLocationNo,
									  locDes.m_TextStringID,
									  locDes.m_vWorldPos,
									  locDes.m_ToolTip);
				m_Locations.addElement(location);
			}
			else
			{
				// sledzimy obiekt
				DefLocation location = (DefLocation) m_Module.CreateObject("DefLocation");
				location.m_bEnableOnStart = true;
				location.InitLocation(locDes.m_iLocationGroupNo,
									  locDes.m_iLocationNo,
									  locDes.m_TextStringID,
									  locDes.m_TrackedName,
									  locDes.m_ToolTip);
				m_Locations.addElement(location);
			}
		}
        
        // objectives edytorowe
        GameObject [] objTab = m_Module.GetObjectsOfClass("Objective");
		for (int i = 0; i < objTab.length; i++)
		{
			SingleObjective obj = null;
            Objective editorObj = (Objective) objTab[i];
            
            obj = new SingleObjective(editorObj.iObjectiveOrderNumber, editorObj.sTextID, false);
			obj.m_bEnableOnStart = editorObj.bEnableOnStart;
            
            boolean bInserted = false;
            // szukamy miejsca, gdzie powinnismy wstawic ten objective
            for (int j = 0; j < m_Objectives.size(); j++)
            {
                SingleObjective curr = (SingleObjective) m_Objectives.get(j);
                if (obj.m_iObjectiveNo <= curr.m_iObjectiveNo)
                {
                    m_Objectives.insertElementAt(obj, j);
                    bInserted = true;
                    break;
                }
            }
            if (!bInserted)
    			m_Objectives.addElement(obj);
		}
        
        objTab = m_Module.GetObjectsOfClass("SubObjective");
		for (int i = 0; i < objTab.length; i++)
		{
            SubObjective editorObj = (SubObjective) objTab[i];
            if (editorObj.cParent == null)
            {
                GameObject.CrashLog("ObjectivesManager ERROR: subobjective doesnt have parent !!!\n");
                continue;
            }
            
            FindObjective(editorObj.cParent.iObjectiveOrderNumber, 0).SetAsGroupObjective(true);
            
            ObjectiveGroup obj = new ObjectiveGroup(editorObj.cParent.iObjectiveOrderNumber,
                                                    editorObj.iObjectiveOrderNumber,
                                                    editorObj.sTextID);
			obj.m_bEnableOnStart = editorObj.bEnableOnStart;
			
            boolean bInserted = false;
            // szukamy miejsca, gdzie powinnismy wstawic ten objective
            for (int j = 0; (j < m_Objectives.size()) && (!bInserted); j++)
            {
                SingleObjective curr = (SingleObjective) m_Objectives.get(j);
                if (obj.m_iObjectiveNo == curr.m_iObjectiveNo)
                {
                    // przesuwamy sie na kolejny subobjective, jezeli taki istnieje
                    while (!bInserted)
                    {
                        j++;
                        if (j == m_Objectives.size())
                        {
                            
                			m_Objectives.addElement(obj);
                            bInserted = true;
                            break;
                        }
                        curr = (SingleObjective) m_Objectives.get(j);
                        if (curr instanceof ObjectiveGroup)
                        {
                            ObjectiveGroup currSubObjective = (ObjectiveGroup) curr;
                            if (obj.m_iObjectiveGroupNo <= currSubObjective.m_iObjectiveGroupNo)
                            {
                                m_Objectives.insertElementAt(obj, j);
                                bInserted = true;
                                break;
                            }
                        }
                        else
                        {
                            m_Objectives.insertElementAt(obj, j);
                            bInserted = true;
                            break;
                        }
                    }
                }
            }
		}
        
        // locations edytorowe
        objTab = m_Module.GetObjectsFromClassModule("EditorLocation", m_Module);
		for (int i = 0; i < objTab.length; i++)
		{
            EditorLocation editorLocation = (EditorLocation) objTab[i];
            
            if ( (editorLocation.bTrackDynamic) && (editorLocation.cTrackedObject == null) )
            {
                GameObject.CrashLog("ObjectivesManager ERROR: location doesnt have target !!!\n");
                continue;
            }
            
			DefLocation location = (DefLocation) m_Module.CreateObject("DefLocation");
			location.m_bEnableOnStart = editorLocation.bEnableOnStart;
            
            String m_MeshID = editorLocation.cPicture.GetStringValue();

            int iNo;
            int iSubNo;
            if (editorLocation.m_iAutoNumber != 0)
            {
                iNo = editorLocation.m_iAutoNumber;
                iSubNo = 0;
            }
            else
            {
                if (editorLocation.cObjective instanceof SubObjective)
                {
                    iNo = ((SubObjective)editorLocation.cObjective).cParent.iObjectiveOrderNumber;
                    iSubNo = editorLocation.cObjective.iObjectiveOrderNumber;
                }
                else
                {
                    iNo = editorLocation.cObjective.iObjectiveOrderNumber;
                    iSubNo = 0;
                }
            }
            
            if (!editorLocation.bTrackDynamic)
            {
                location.InitLocation(iNo,
                                      iSubNo,
                                      m_MeshID,
                                      editorLocation.GetPosition(),
                                      editorLocation.sTipID);
            }
            else
            {
			    location.InitLocation(iNo,
			    					  iSubNo,
			    					  m_MeshID,
			    					  editorLocation.cTrackedObject.GetName(),
			    					  editorLocation.sTipID);
            }
    	    m_Locations.addElement(location);
        }
		
		// wlaczamy objectives na start
		EnableObjectivesOnStart();
		// wlaczamy locations na start
		EnableLocationsOnStart();
        
        //HideAll();
	}
	
	/** uaktywnienie objectives, ktore maja byc na starcie ustawione */
	void EnableObjectivesOnStart()
	{
		for (int i = 0; i < m_Objectives.size(); i++)
		{
			SingleObjective obj = (SingleObjective) m_Objectives.get(i);
			
			if (obj instanceof ObjectiveGroup)
			{
				ObjectiveGroup group = (ObjectiveGroup) obj;
				if (obj.m_bEnableOnStart)
					EnableObjective(group.m_iObjectiveNo, group.m_iObjectiveGroupNo);
				else
					DisableObjective(group.m_iObjectiveNo, group.m_iObjectiveGroupNo);
			}
			else
			{
				if (obj.m_bEnableOnStart)
					EnableObjective(obj.m_iObjectiveNo);
				else
					DisableObjective(obj.m_iObjectiveNo);
			}
		}
	}
	
	void EnableLocationsOnStart()
	{
		for (int i = 0; i < m_Locations.size(); i++)
		{
			DefLocation location = (DefLocation) m_Locations.get(i);
			
			boolean bEnable = location.m_bEnableOnStart;
			
			SingleObjective obj = FindObjective(location.m_iGroupNo, location.m_iObjectiveNo);
			if (obj != null)
				bEnable = obj.m_bEnableOnStart;
			
			if (bEnable)
				EnableLocation(location.m_iGroupNo, location.m_iObjectiveNo,false);
			else
				DisableLocation(location.m_iGroupNo, location.m_iObjectiveNo,false);
		}
	}
	
	/** dolaczenie tekstow i sprite'ow punktow na mapie do sprite'a, na ktorym beda
	 * wyswietlane (czyli zapewne do jakiejs mapy z miejscem na opis objectives i tekstura mapy)
	 */
	void AttachTextsToParent(Sprite parent)
	{
		for (int i = 0; i < m_Objectives.size(); i++)
			((SingleObjective) m_Objectives.elementAt(i)).AttachTextToParent(parent);
	}

    void AttachLocationsToParent(Sprite parent)
    {
        for (int i = 0; i < m_Locations.size(); i++)
			((DefLocation) m_Locations.elementAt(i)).AttachLocationToParent(parent);
    }
    
	void RecalculatePositions()
	{
		float fY = _TEXT_START_INDENT_Y;
		
        if (GetMainPlayer() == null)
            return;
        
        Sprite cObjectives = GetMainPlayer().HUD.GetHUDMap().GetObjectivesSprite();
        
		for (int i = 0; i < m_Objectives.size(); i++)
		{
			SingleObjective obj = (SingleObjective) m_Objectives.elementAt(i);
            
			fY += obj.RecalculatePosition(fY, cObjectives.GetSize()); // ustalamy wysokosc okna z tekstem
		}
	}
	
	SingleObjective FindObjective(int iObjectiveNo, int iGroupNo)
	{
		for (int i = 0; i < m_Objectives.size(); i++)
		{
			SingleObjective objective = (SingleObjective) m_Objectives.elementAt(i);
			
			if (objective.m_iObjectiveNo == iObjectiveNo)
			{
				if (objective.m_bIsSuperObjective)
				{
					if (iGroupNo == 0)
						return objective;
				}
				else
				{
					if (objective.m_bIsSubObjective)
					{
						ObjectiveGroup group = (ObjectiveGroup) objective;
						if (group.m_iObjectiveGroupNo == iGroupNo)
							return objective;
					}
					else
						return objective;
				}
			}
		}
		return null;
	}

	java.util.Vector FindSubObjectives(int iObjectiveNo)
	{
		java.util.Vector vector = new java.util.Vector();
		
		for (int i = 0; i < m_Objectives.size(); i++)
		{
			SingleObjective objective = (SingleObjective) m_Objectives.elementAt(i);
			
			if ( (objective.m_iObjectiveNo == iObjectiveNo) && (!objective.m_bIsSuperObjective) )
				vector.addElement(objective);
		}
		return vector;
	}
	
	DefLocation FindLocation(int iGroupNo, int iSubLocNo)
	{
		for (int i = 0; i < m_Locations.size(); i++)
		{
			DefLocation location = (DefLocation) m_Locations.elementAt(i);
			
			if ( (location.m_iGroupNo == iGroupNo) && (location.m_iObjectiveNo == iSubLocNo) )
				return location;
		}
		return null;
	}
	
	java.util.Vector FindSubLocations(int iObjectiveNo)
	{
		java.util.Vector vector = new java.util.Vector();
		
		for (int i = 0; i < m_Locations.size(); i++)
		{
			DefLocation location = (DefLocation) m_Locations.elementAt(i);
			
			if ( (location.m_iGroupNo == iObjectiveNo) && (location.m_iObjectiveNo != 0) )
				vector.addElement(location);
		}
		return vector;
	}
	
    private boolean m_bVisible = false;
    boolean IsVisible(){ return m_bVisible; }
    
	void ShowAll()
	{
        CheckSpritesAttached();
        m_bVisible = true;
            
        for (int i = 0; i < m_Objectives.size(); i++)
		{
            if ( ! (m_Objectives.elementAt(i) instanceof SingleObjective) )
                continue;
            SingleObjective cObjective = (SingleObjective)m_Objectives.elementAt(i);
            
            if ( cObjective.CanShow() )
				 cObjective.ShowThisOnly();
		}
        
		for (int i = 0; i < m_Locations.size(); i++)
		{
            if ( ! ( m_Locations.elementAt(i) instanceof DefLocation))
                continue;
			DefLocation cLocation = (DefLocation)m_Locations.elementAt(i);
            
			if ( cLocation.HasObjective() )
                continue;
            
            if ( cLocation.CanShow() )
                cLocation.ShowLocation();
		}        
	}
	
	void HideAll()
	{
        m_bVisible = false;
        
		for (int i = 0; i < m_Objectives.size(); i++)
			((SingleObjective) m_Objectives.elementAt(i)).HideThisOnly();
		for (int i = 0; i < m_Locations.size(); i++)
		{
			DefLocation loc = (DefLocation) m_Locations.elementAt(i);
			loc.HideLocation();
		}
	}

    void HideAllTooltips()
    {
        for (int i = 0; i < m_Locations.size(); i++)
		{
            DefLocation loc = (DefLocation) m_Locations.elementAt(i);
			loc.HideToolTip();
        }
    }
    
	boolean IsAccomplished(int iObjNo)
	{
		return IsAccomplished(iObjNo, 0);
	}

	boolean IsAccomplished(int iGroupNo, int iSubObjNo)
	{
		SingleObjective obj = FindObjective(iGroupNo, iSubObjNo);
		
		if (obj != null)
			return obj.IsAccomplished();
		return true;
	}
    
    boolean m_bMesssagesActive = true;
    
    void SetMessagesActive(boolean bActive)
    {
        m_bMesssagesActive = bActive;
    }
    
    
    
    void TextMessage(String sText)
    {
        if ( ! m_bMesssagesActive )
            return;
        
        PlayerHUD cPlayer = GetMainPlayer();
        
        if ( cPlayer == null )
        {
            //GameObject.LogD(" Player == null !!!\n");
            return;
        }

        //sprawdzamy czy czasem nie rzucamy tego samego komunikatu w tej samej ramce
        if (m_sLastMessage.compareTo(sText) == 0 && 
            m_fLastMessageTime == cPlayer.GetTime())
            return;
        
        if (sText.equals("&OBJECTIVE_UPDATED&") ||
            sText.equals("&OBJECTIVE_ACCOMPLISHED&") ||
            sText.equals("&OBJECTIVE_UNACCOMPLISHED&"))
        {
            cPlayer.TryPlayObjectiveChangeSound();
        }
        
        m_sLastMessage = new String(sText);
        m_fLastMessageTime = cPlayer.GetTime();
       
        cPlayer.TextMessage( sText );
		// dodajemy info do okna logow
		if ( (cPlayer.HUD               != null) &&
			 (cPlayer.HUD.GetHUDLogs()  != null) )
        {
	        cPlayer.HUD.GetHUDLogs().InputText( sText );        
        }
    }
 
    void CheckSpritesAttached()
    {
        if ( ! m_bSpritesAttached )
        {
            if ( GetMainPlayer() != null &&
                 GetMainPlayer().HUD != null &&
                 GetMainPlayer().HUD.GetHUDMap() != null &&
                 GetMainPlayer().HUD.GetHUDMap().GetObjectivesSprite() != null)
            {
                m_bSpritesAttached = true;
			    AttachTextsToParent( GetMainPlayer().HUD.GetHUDMap().GetObjectivesSprite() );
                AttachLocationsToParent( GetMainPlayer().HUD.GetHUDMap().m_cMap );
                RecalculatePositions();
            }
        }
            
    }
    
    /** 
     * Obsluga w sieci
     * 
     * W grze sieciowej niektore objectivy (locations) beda
     * specyficzne dla druzyny w szeregach ktorej aktualnie
     * znajduje sie gracz. Dlatego tez dla pozostalych graczy
     * (tych z innej druzyny) nalezy niektore z nich ukryc.
     * W tym celu wprowadzono flage m_bEnableNet (do SingleObjective
     * oraz DefLocation) i w zaleznosci od trybu gry oraz druzyny
     * gracza bedzie mozna odpowiednio aktywowac wybrane elementy
     * 
     * Podzial objectivow bedzie wygladal nastepujaco:
     * -   1..99  - wspolne w grach typu single (nieteamowych)
     * - 100..199 - dla druzyny BLUE w grach teamowych
     * - 200..299 - dla druzyny RED w grach teamowych
     */
    
    private boolean m_bOnServer   = false;
    private boolean m_bInit       = false;
    private boolean m_bTeamGame   = false;
    private int     m_nPlayerTeam = NetTeamInfo.NONE;
    
    void StartOnServer()
    {
        m_bOnServer = true;
        UpdateEnableNet();
    }
    
    void StartOnClient()
    {
        m_bOnServer = false;
        UpdateEnableNet();
    }
    
    void SetEnableNet(boolean bTeamGame,int nPlayerTeam)
    {
        m_bInit       = true;
        m_bTeamGame   = bTeamGame;
        m_nPlayerTeam = nPlayerTeam;
        
        UpdateEnableNet();
    }
        
    boolean CheckEnableNet(int nNumber)
    {
        if ( ! m_bInit )
            return false;
            
        if ( m_bTeamGame )
        {
            switch ( m_nPlayerTeam )
            {
            case NetTeamInfo.BLUE:
                return (nNumber >= 100 && nNumber < 200);
            case NetTeamInfo.RED:
                return (nNumber >= 200 && nNumber < 300);    
            default:
                return false;
            }
        }
            
        return true;
    }
    
    void UpdateEnableNet()
    {
        for (int i = 0; i < m_Objectives.size(); i++)
		{
            if ( ! (m_Objectives.elementAt(i) instanceof SingleObjective) )
                continue;
            SingleObjective cObjective = (SingleObjective)m_Objectives.elementAt(i);
            
            if ( CheckEnableNet( cObjective.m_iObjectiveNo ))
                cObjective.Enable( true );
            else
                cObjective.Disable( true );
		}
        
		for (int i = 0; i < m_Locations.size(); i++)
		{
            if ( ! ( m_Locations.elementAt(i) instanceof DefLocation))
                continue;
			DefLocation cLocation = (DefLocation)m_Locations.elementAt(i);
            
			if ( cLocation.HasObjective() )
                continue;

            int nNumber = cLocation.m_iObjectiveNo;
            if ( nNumber == 0 )
                nNumber = cLocation.m_iGroupNo;

            if ( CheckEnableNet( nNumber ) )
                cLocation.Enable( true );
            else
                cLocation.Disable( true );
		}        
        
        RecalculatePositions();
        Refresh();
    }
    
    void CopyDataToContainer()
    {
        //nadmiarowo, jesli skypty o klienta beda takie same to moze wyleciec
        m_cContainer.PackInt( m_Objectives.size() );
        for (int i = 0; i < m_Objectives.size(); i++)
		{
            if ( ! (m_Objectives.elementAt(i) instanceof SingleObjective) )
                continue;
            SingleObjective cObjective = (SingleObjective)m_Objectives.elementAt(i);
    
            //nadmiarowo, jesli skypty o klienta beda takie same to moze wyleciec
            m_cContainer.PackInt( cObjective.m_iObjectiveNo );
            //nadmiarowo, jesli skypty o klienta beda takie same to moze wyleciec
            m_cContainer.PackInt( cObjective.m_iObjectiveGroupNo );
            
            m_cContainer.PackBoolean( cObjective.IsEnabled( false ) );
            m_cContainer.PackBoolean( cObjective.IsAccomplished() );
		}
        
        int nLocations = 0;
		for (int i = 0; i < m_Locations.size(); i++)
		{
            if ( ! ( m_Locations.elementAt(i) instanceof DefLocation))
                continue;
			DefLocation cLocation = (DefLocation)m_Locations.elementAt(i);
            
			if ( cLocation.HasObjective() )
                continue;
            
            nLocations++;
        }
        //nadmiarowo, jesli skypty o klienta beda takie same to moze wyleciec
        m_cContainer.PackInt( nLocations );
        for (int i = 0; i < m_Locations.size(); i++)
		{
            if ( ! ( m_Locations.elementAt(i) instanceof DefLocation))
                continue;
			DefLocation cLocation = (DefLocation)m_Locations.elementAt(i);
            
			if ( cLocation.HasObjective() )
                continue;
        
            //nadmiarowo, jesli skypty o klienta beda takie same to moze wyleciec
            m_cContainer.PackInt( cLocation.m_iObjectiveNo );
            //nadmiarowo, jesli skypty o klienta beda takie same to moze wyleciec
            m_cContainer.PackInt( cLocation.m_iGroupNo );
            m_cContainer.PackBoolean( cLocation.IsEnabled( false ) );
        }
    }
    
    void CopyDataFromContainer()
    {
        SetMessagesActive( false );

        int nObjectives = m_cContainer.UnPackInt();
        if ( nObjectives != m_Objectives.size() )
            GameObject.LogD(" nObjectives:"+nObjectives+" "+m_Objectives.size()+"\n");
        
        for (int i = 0; i < m_Objectives.size(); i++)
		{
            if ( ! (m_Objectives.elementAt(i) instanceof SingleObjective) )
                continue;
            SingleObjective cObjective = (SingleObjective)m_Objectives.elementAt(i);
    
            int nObjectiveNo = m_cContainer.UnPackInt();
            if ( nObjectiveNo != cObjective.m_iObjectiveNo )
                GameObject.LogD(" nObjectiveNo:"+nObjectiveNo+" "+cObjective.m_iObjectiveNo+"\n");                
            int nObjectiveGroupNo = m_cContainer.UnPackInt();
            if ( nObjectiveGroupNo != cObjective.m_iObjectiveGroupNo )
                GameObject.LogD(" nObjectiveGroupNo:"+nObjectiveGroupNo+" "+cObjective.m_iObjectiveGroupNo+"\n");                
            boolean bEnabled        = m_cContainer.UnPackBoolean();
            boolean bAccomplished   = m_cContainer.UnPackBoolean();
            
            if (bEnabled)
                cObjective.Enable( false );
            else
                cObjective.Disable( false );
            
            if (bAccomplished)
                cObjective.SetAccomplished();
            else
                cObjective.SetUnaccomplished();
		}
        
        int nLocations = 0;
		for (int i = 0; i < m_Locations.size(); i++)
		{
            if ( ! ( m_Locations.elementAt(i) instanceof DefLocation))
                continue;
			DefLocation cLocation = (DefLocation)m_Locations.elementAt(i);
            
			if ( cLocation.HasObjective() )
                continue;
            
            nLocations++;
        }
        
        int nRecvLocations = m_cContainer.UnPackInt();
        if ( nLocations != nLocations )
            GameObject.LogD(" nRecvLocations:"+nRecvLocations+" "+nLocations+"\n");
        for (int i = 0; i < m_Locations.size(); i++)
		{
            if ( ! ( m_Locations.elementAt(i) instanceof DefLocation))
                continue;
			DefLocation cLocation = (DefLocation)m_Locations.elementAt(i);
            
			if ( cLocation.HasObjective() )
                continue;
            
            int nObjectiveNo = m_cContainer.UnPackInt();
            if ( nObjectiveNo != cLocation.m_iObjectiveNo )
                GameObject.LogD(" nObjectiveNo:"+nObjectiveNo+" "+cLocation.m_iObjectiveNo+"\n");                
            int nGroupNo = m_cContainer.UnPackInt();
            if ( nGroupNo != cLocation.m_iGroupNo )
                GameObject.LogD(" nGroupNo:"+nGroupNo+" "+cLocation.m_iGroupNo+"\n");                
            boolean bEnabled = m_cContainer.UnPackBoolean();
                        
            if (bEnabled)
                cLocation.Enable( false );
            else
                cLocation.Disable( false );
        }
        
        SetMessagesActive( true );
        RecalculatePositions();
        Refresh();
    }
    
    
    void Refresh()
    {
        if ( m_bVisible )
        {
            HideAll();
            ShowAll();
        }
    }
    
    final static int _OBJECTIVE_ACCOMPLISHED    = 0;
    final static int _OBJECTIVE_UNACCOMPLISHED  = 1;
    final static int _OBJECTIVE_ENABLED         = 2;
    final static int _OBJECTIVE_DISABLED        = 3;
    final static int _LOCATION_ENABLED          = 4;
    final static int _LOCATION_DISABLED         = 5;
    
    void ServerObjectiveUpdated( int nUpdate, int iGroupNo, int iSubObjNo )
    {
        NetGameInfo cGameInfo = GetNetGameInfo();    
        if ( cGameInfo != null )
            cGameInfo.EventObjectiveUpdated( _OBJECTIVE_ACCOMPLISHED, 
                                             iGroupNo, 
                                             iSubObjNo,
                                             m_bMesssagesActive );
    }
    
    
        
    void ObjectiveUpdated(int       nUpdate, 
                          int       iGroupNo, 
                          int       iSubObjNo,
                          boolean   bForceMesssagesActive)
    {
        boolean bMesssagesActive = m_bMesssagesActive;
        
        m_bMesssagesActive = bForceMesssagesActive;
        switch ( nUpdate )
        {
        case _OBJECTIVE_ACCOMPLISHED:
            SetAccomplished( iGroupNo, iSubObjNo );        
            break;
        case _OBJECTIVE_UNACCOMPLISHED:
            SetUnaccomplished( iGroupNo, iSubObjNo );        
            break;
        case _OBJECTIVE_ENABLED:
            EnableObjective( iGroupNo, iSubObjNo );        
            break;
        case _OBJECTIVE_DISABLED:
            DisableObjective( iGroupNo, iSubObjNo );        
            break;
        case _LOCATION_ENABLED:
            EnableLocation( iGroupNo, iSubObjNo, false );        
            break;
        case _LOCATION_DISABLED:
            DisableLocation( iGroupNo, iSubObjNo, false );        
            break;
        }
        
        m_bMesssagesActive = bMesssagesActive;
    }
    
    void Destroy()
    {
		for (int i = 0; i < m_Objectives.size(); i++)
        {
            ((SingleObjective) m_Objectives.elementAt(i)).Destroy();
        }
	
        for (int i = 0; i < m_Locations.size(); i++)
        {
            ((DefLocation) m_Locations.elementAt(i)).Destroy();
        }
    }
    
    /*
    void Log()
    {
        GameObject.LogD("\n");
        GameObject.Log(" m_bInit "+m_bInit+"\n");
        GameObject.Log(" m_bSpritesAttached "+m_bSpritesAttached+"\n");
        GameObject.Log(" m_bVisible "+m_bVisible+"\n");
        GameObject.Log(" m_Objectives.size() "+m_Objectives.size()+"\n");
        for (int i = 0; i < m_Objectives.size(); i++)
		{
            if ( ! (m_Objectives.elementAt(i) instanceof SingleObjective) )
                continue;
            SingleObjective cObjective = (SingleObjective)m_Objectives.elementAt(i);
        
            GameObject.Log(" "+i+" "+
                cObjective.m_iObjectiveNo+" "+cObjective.m_iObjectiveGroupNo+" "+
                cObjective.IsEnabled()+" "+cObjective.IsEnabled(true)+" "+
                cObjective.CanShow()+"\n");
		}
    }
    */
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[ObjectivesManager.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_OBJECTIVES_MANAGER)
        {
			m_bMesssagesActive = cFC.LoadBool();
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[ObjectivesManager.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_OBJECTIVES_MANAGER, cFCParent);
		
		cFC.SaveBool(m_bMesssagesActive);
		
		cFC.delete();
	}							
    
    public void SGUpdate()
    {
        m_sLastMessage = "";
        m_fLastMessageTime = 0;
    }
}
