/** Bazowy obiekt emitujacy dzwieki.
 * Nalezy podziedziczyc po tym obiekcie i pokryc metody Init() oraz Chceck().
 */
public class SoundEmitter extends MeshObject implements iact_EnableDisable
{
    /** Okresla czy obiekt jest normalnym MeshObjectem czy tylko fakeiem */
    public boolean bNormalMeshObject = false;//#SOUND_EMITTER
    
    /** Odleglosc na jakiej dzwiek wybrzmiewa pelna glosnoscia */
    public float fMinDistance = 1000.0f;//#SOUND_EMITTER
    
    /** Odleglosc od kamery w jakiej dzwiek ma wygasnac */
    public float fMaxDistance = 3000.0f;//#SOUND_EMITTER
    
    /** Jesli true, to klasa nie modyfikuje zasiegow dzwieku.
     * Potrzebne, jesli uzywamy do tego celu skryptu (sounds.ini) */
    public boolean bScriptControlled = true;//#SOUND_EMITTER

/************************************************************************************/    

    /** Czas co jaki jest sprawdzane wylaczanie i wlaczanie dzwiekow */
    float fCheckTime = 1.0f;
    
    /** Maksymalna predkosc kamery - wprowadzona w celu lepszego wlaczania i wylaczania dzwiekow */
    float fCameraMaxSpeed = 600.0f;
    
    /** Aktualna odleglosc do kamery wyliczana na poczatku updejtu dzwiekow */
    float fDistanceToCamera = 100000.0f;
    
    /** Flaga, czy gracz jest w srodku budynku */
    static public boolean bListenerInsideBuilding = false;
    
    /** Flaga mowiaca, ze dzwieki maja przestac sie odgrywac */
    static boolean bSoundsDisabled = false;
    
    /** flaga mowiaca, czy ten emiter powinien przestac wydawac dzwiek */
    boolean bSoundDisabled = false;
    
    SoundEmitter()
    {
        MeshName = "Data/Interface/Wykrzyknik.3da";
    }
    
    /** W(y)laczenie wszystkich soundemiterow
     */
    static void EnableSounds(boolean bEnable)
    {
        bSoundsDisabled = !bEnable;
    }
    
    ChromeModule cModule = null;
    
    void OnCreate()
    {
        cModule = (ChromeModule)GetModule();
        if(!bNormalMeshObject)
        {
            // ustawiamy parametry pozwalajace na cicha egzystencje bez zwalniania gry
            ClearFlag(EFlags._COLLISION);
            ClearFlag(EFlags._BLOCK_MOVE);
            ClearFlag(EFlags._BLOCK_TRACE);
            SetFlag(EFlags._NO_ANIMATION);
            SetFlag(EFlags._NO_BODY_PHYSICS);
            SetPhysicsType(EPhysics._NONE);
            SetPhysicsActive(false);
            EnableRendering(false);
        }
        
        // wolamy metode, ktora ma byc pokryta w poszczegolnych implementacjach 
        // charaketerystycznych emiterow
        CallOnFrameEnd("Init");

        Reinitialize();
		
		DetachFromParentAndKeepWorldPos();
    }
    
    void Reinitialize()
    {
        DisableCallMethods();
        
        // wlaczamy sprawdzanie kiedy dzwieki wylaczyc
        CallMethodEveryMinMax("Check", fCheckTime, 3.f*fCheckTime);
        
        // wymuszamy pierwszego checka
        CallMethodIn("Check", fCheckTime);
    }
    
    /** Metoda inicjalizujaca odgrywane dzwieki. Do pokrycia w klasach pochodnych
     */
    void Init()
    {
        LogD("Member not implmented in inherited class!\n");
    }
    
    
    /** Metoda oblicza i wpisuje do odpowiedniego pola odleglosc do kamery.
     * Gdy nie ma kamery to zwraca 0.
     */
    void UpdateDistanceToCamera()
    {
        // liczymy odleglosc do aktualnej kamery
        Camera cActiveCamera = cModule.GetActiveViewCamera();
        fDistanceToCamera = (cActiveCamera != null) ? cActiveCamera.GetPositionVolatile().Distance(GetPositionVectorVolatile()) : 1000000.0f;
    }
    
    /** Metoda sprawdza czy dzwiek wylaczyc lub wlaczyc
     */
    void Check()
    {
        LogD("Member not implmented in inherited class!\n");
    }

    /**********************************************
     ** implementacja interfejsu iact_EnableDisable
     ***********************************************/    
    public void Enable()
    {
        bSoundDisabled = false;
    }
    
    public void Disable()
    {
        bSoundDisabled = true;
    }
    
    public boolean IsEnabled()
    {
        return bSoundDisabled;
    }
}
