/** klasa odpowiadajaca za fizyke pojazdow latajacych
 */
public class FlyingVehiclePhysics extends GameObject
{
	/** STAE */
	
	static float _MIN_PHYSICS_UPDATE_GRAIN = 0.05f; //minimalne ziarno updatu fizyki pojazdu
	float _UPDOWN_FLY_PRECISION = 10f;				//dopuszczalny blad gora / dol
	float _ACCEL_UPDOWN = 1000f;					//przyspieszenie poruszania sie gora / dol
	float _MAX_UPDOWN_SPEED = 2000f;				//predkosc maksymalna poruszania sie gora / dol
	float _MOVE_PRECISION = 1000f;					//dokladnosc odleglosci od celu w cm, kiedy uznajemy, ze dolecielismy
	float _MAX_FW_SPEED = 4000f;					//maksymalna predkosc do przodu w cm/s
	float _CURR_MAX_FW_SPEED = 2000f;				//maksymalna biezaca predkosc do przodu w cm/s - musi byc mniejsza od _MAX_FW_SPEED
													//powinna byc ustalona przed wykorzystaniem pojazdu i nie zmieniana w trakcie zycia
	float _ACCEL_FW = 1000f;						//przyspieszenie przy locie do przodu cm/s2
	float _ACCEL_BW = 1200f;						//przyspieszenie przy hamowaniu cm/s2
	static float _DEG_PRECISION = 1f;				//dokladnosc obrotu w stopniach
	float _MAX_TURN_SPEED = 120f;					//maksymalna predkosc obrotu w stopniach/s
	float _ACCEL_TURN = 40f;						//przyspieszenie obrotu w stopniach/s
	float _MAX_VERT_ANGLE = 45f;					//maksymalne przechylenie gora / dol pojazdu
	float _MAX_HORIZ_ANGLE = 30f;					//maksymalne przechylenie lewo / prawo pojazdu
	float _AIM_AT_ENEMY_ANGLE_PRECISION = 30f;		//kat, po przekroczeniu ktorego pojazd obraca sie na wroga przy celowaniu na niego 
	static float _RAD_TO_DEG = 180f / 3.1415926f;	//konwersja radiany - stopnie
	static float _NO_VALUE = -1f;					//stala oznaczajaca wartosc nieokreslona (niemozliwa do obliczenia)
	static int _TURN_LEFT = -1;						//flagi kierunku, w ktorym sie pojazd obraca
	static int _NO_TURN = 0;
	static int _TURN_RIGHT = 1;
	static int _DEACCEL = -1;						//flagi ciagu silnikow
	static int _NO_ACCEL = 0;
	static int _ACCEL = 1;
	
	/** ZMIENNE */
	
	float m_fCurrSpeed = 0f;						//biezaca predkosc
	int m_iAccelFlag = _NO_ACCEL;					//flaga mowiaca, czy przyspieszamy (1) czy zwalniamy (-1) czy nic nie robimy (0)
	int m_iTurnFlag = _NO_TURN;						//flaga mowiaca, czy skrecamy
	float m_fCurrTurnSpeed = 0f;					//predkosc skretu w stopniach/s
	int m_iUpDownFlag = _NO_ACCEL;					//flaga mowiaca, czy wznosimy sie, czy opadamy
	float m_fCurrUpDownSpeed = 0f;					//aktualna predkosc lotu gora / dol
	FlyingVehicle m_Owner = null;					//pojazd, ktorym kieruje TA fizyka
	Vector m_vDestPoints[] = null;					//punkty docelowe pojazdu
	int m_iCurrPoint = 0;							//biezacy indeks punktu docelowego pojazdu
	boolean m_bStopTurn = true;						//czy w stanie moveto w ostatnim punkcie mamy czekac, az skonczymy sie obracac,
													//czy tez potem bedziemy kontynuowac lot i nie musimy czekac na koniec obrotu
	float m_fLastT = 0;								//ostatni czas update fizyki
	Vector m_vLandDestDir = null;					//kierunek, w ktorym chcemy wyladowac
	Vector m_vVehicleForward = null;				//wektor forward pojazdu
	Vector m_vVehiclePos = null;					//wektor pozycji pojazdu
	boolean m_bPreciseMoving = false;				//czy ma dokladnie dolatywac w ostatnim punkcie
	Vector m_vStartLandPos = null;					//wektor, gdzie chcemy wystartowac / wyladowac
	public boolean m_bSlowDownToPoint = true;		//czy zwalniac, gdy zblizamy sie do punktu docelowego - wazne przy patrolowaniu - by sie nie
													//zatrzymywac w kolejnych punktach, tylko przelatywac na full speed
	private boolean m_bStarting = false;			//wiadomosc dla stanu startstate czy startujemy, czy ladujemy
	
	/** METODY */
	
	/** inicjacja fizyki, musi byc zrobiona jednorazowo dla pojazdu, ktory jej uzywa */
	void InitPhysics(FlyingVehicle owner)
	{
		m_Owner = owner;
	}
	
	boolean bActivatePhysics = false;
	
	/** aktywacja fizyki - wolana zazwyczaj po aktywowaniu pojazdu */
	void ActivatePhysics()
	{
		bActivatePhysics = true;
		ActivateOnModule();
		m_fLastT = GetTime();
	}
	
	/** deaktywacja fizyki - wolana zazwyczaj po deaktywowaniu (schowaniu) pojazdu */
	void DeactivatePhysics()
	{
		bActivatePhysics = false;
		CallMethodIn("DeactivateOnModule", 0f);
	}
	
	/** powoduje wstrzymanie wszelkich manewrow pojazdu - pojazd zatrzyma sie w pozycji wyjsciowej */
	void Reset()
	{
		m_iAccelFlag = _NO_ACCEL;
		m_iTurnFlag = _NO_TURN;
		m_iUpDownFlag = _NO_ACCEL;
		m_fCurrSpeed = 0;
		m_fCurrTurnSpeed = 0;
		m_fCurrUpDownSpeed = 0f;
		m_iCurrPoint = 0;
	}
	
	/** ustawienie przyspieszenia wznoszenia sie / opadania pojazdu */
	void SetAccelUpDown(float fAccel)
	{
		_ACCEL_UPDOWN = fAccel;
	}
	
	/** ustalenie promienia okregu, w ktorym uznajemy, ze pojazd dotarl do punktu docelowego */
	void SetMovePrecision(float fMovePrecision)
	{
		_MOVE_PRECISION = fMovePrecision;
	}
	
	/** ustalenie maksymalnej predkosci lotu w plaszczyznie xoz - jest to maksymalna
	 * mozliwa predkosc pojazdu. Jezeli chcemy w trakcie lotu zmienic predkosc
	 * maksymalna pojazdu, nalezy uzyc SetCurrMaxForwardSpeed() */
	void SetMaxForwardSpeed(float fNewSpeed)
	{
		_MAX_FW_SPEED = fNewSpeed;
	}
    

    float GetMaxForwardSpeed()
    {
        return _MAX_FW_SPEED;
    }
    
	/** ustalenie maksymalnej biezacej predkosci lotu w plaszczyznie xoz - musi byc
	 * mniejsza od _MAX_FW_SPEED. */
	void SetCurrMaxForwardSpeed(float fNewSpeed)
	{
		_CURR_MAX_FW_SPEED = fNewSpeed;
        if (_CURR_MAX_FW_SPEED > _MAX_FW_SPEED)
        {
            _CURR_MAX_FW_SPEED = _MAX_FW_SPEED;
			CrashLog("_CURR_MAX_FW_SPEED > _MAX_FW_SPEED !!!\n");
        }
	}
	
	void SetMaxUpDownSpeed(float fNewSpeed)
	{
		_MAX_UPDOWN_SPEED = fNewSpeed;
	}
	
	/** rozkaz przemieszczenia sie do punktu, bStopTurn mowi, czy
	 * wychodzimy ze stanu po tym, jak predkosc zmaleje do zera czy z dodatkowy jeszcze
	 * warunkiem, ze predkosc katowa == 0 - jezeli wychodzimy bez sprawdzania
	 * predkosci katowej, nalezy w nastepnym stanie ja dosymulowac (tak aby osiagnela 0) */
	void MoveTo(Vector vDestPoint, boolean bStopTurn)
	{
		MoveTo(vDestPoint, bStopTurn, true);
	}
	
	void MoveTo(Vector vDestPoint, boolean bStopTurn, boolean bSlowDownToPoint)
	{
		if (m_Owner == null)
			return;
		
		m_bSlowDownToPoint = bSlowDownToPoint;
		
		m_bStopTurn = bStopTurn;
		
		m_vDestPoints = new Vector[1]; // jest tylko jeden punkt do ktorego sie poruszamy
		m_vDestPoints[0] = new Vector(vDestPoint);
		SetState("GotoPointState");
	}
	
	void MoveTo(Vector vDestPoint)
	{
		MoveTo(vDestPoint, false);
	}
	
	void PatrolPoints(Vector vPoints[])
	{
		if (m_Owner == null)
			return;
		if (vPoints == null)
			return;

		m_bSlowDownToPoint = (vPoints.length <= 1);
		m_bStopTurn = true;
		m_vDestPoints = vPoints;
		SetState("GotoPointState");
	}
	
	void StartFly()
	{
        m_Owner.PlayFXDustStart();
		SetState("StartState");
	}
	
	/** wystartowanie pojazdu tak, aby osiagnal wysokosc zadana w vPos */
	void StartFly(Vector vDestPos)
	{
		m_vStartLandPos = vDestPos;
		
		m_bStarting = true;
        m_Owner.PlayFXDustStart();
		SetState("StartState");
	}
	
	void StartLand(Vector vDestPos, Vector vDestDir)
	{
		m_vStartLandPos = vDestPos;
		m_vLandDestDir = vDestDir;
		m_bStarting = false;
		SetState("StartState");
	}

	/** stan poruszania sie do punktu bez pozniejszego namierzania sie na cel */
	public class GotoPointState extends State
	{
		boolean m_bStopping = false;
	
		public void OnEnter()
		{
			m_iCurrPoint = 0;
			m_iAccelFlag = _ACCEL;
			m_bStopping = false;
		}
		
		public void OnFrame()
		{
			float fTime = GetTime();
			do
			{
				Physics_MakeUpTurnDecision(_DEG_PRECISION);
			
				Physics_MakeUpDownDecision(m_Owner.GetPosition(),
										   m_vDestPoints[m_iCurrPoint].fY);
				if (CheckMoveDecision())
					break;

			} while (UpdatePhysics(fTime));
		}
        
		/** zwraca true, jezeli doleci na miejsce */
		boolean CheckMoveDecision()
		{
			if (!m_bSlowDownToPoint)
			{
				float fDist = Physics_CalcDist();
		
				if (fDist < _MOVE_PRECISION)
				{
					if (m_iCurrPoint == m_vDestPoints.length - 1)
						m_Owner.OnDestinationPos(); // przetworzylismy ostatni punkt sciezki
					else
					{
						m_Owner.OnDestinationPos(m_iCurrPoint);
						m_iCurrPoint++;
						// jezeli dopiero bedziemy przetwarzac ostatni punkt sciezki
						if (m_iCurrPoint == m_vDestPoints.length - 1)
							m_bSlowDownToPoint = true;
					}
				}
				else
				{
					if (!Physics_EnoughTimeToTurn())
						m_iAccelFlag = _DEACCEL;
					else
						m_iAccelFlag = _ACCEL;
				}
			}
			else
			{
				if (!m_bStopping)
				{
					float fDist = Physics_CalcDist();
			
					if (fDist < _MOVE_PRECISION)
					{
						// zwalniamy, bo dolecielismy na miejsce
						m_bStopping = true;
						m_iAccelFlag = _DEACCEL;
					}
					else
					{
						if (!Physics_EnoughTimeToTurn())
							m_iAccelFlag = _DEACCEL;
						else
							m_iAccelFlag = _ACCEL;
					}
				}
				else
				{
					if (m_bPreciseMoving)
					{
						if (m_fCurrSpeed == 0)
						{
							SetState("PrecisionGotoState");
							return true;
						}
					}
					else
					{
						if (m_fCurrSpeed == 0)
						{
							if (m_bStopTurn)
							{
								if (m_fCurrTurnSpeed == 0)
								{
									m_Owner.OnDestinationPos();
									return true;
								}
							}
							else
							{
								m_Owner.OnDestinationPos();
								return true;
							}
						}
					}
				}
			}
			return false;
		}
		
		//*********************************************************
		//					SaveGame
		//*********************************************************
				
		public void SGLoad(FileChunk cParentFC)
		{
			FileChunk.Log("[FlyingVehiclePhysics$GotoPointState] : SGLoad\n");
			
			FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
			if (cFC.GetID() == ESGChunksChrome._CHK_FLYING_VEHICLE_GOTO_POINT_STATE)
			{
				super.SGLoad(cFC);
				
				m_bStopping = cFC.LoadBool();
			}
			
			cFC.delete();
		}
	
		public void SGSave(FileChunk cParentFC)
		{
			FileChunk.Log("[FlyingVehiclePhysics$GotoPointState] : SGSave\n");
			
			FileChunk cFC = NewChunk(ESGChunksChrome._CHK_FLYING_VEHICLE_GOTO_POINT_STATE, cParentFC);
			
			super.SGSave(cFC);
			
			cFC.SaveBool(m_bStopping);
			
			cFC.delete();
		}		
		
		public void SGUpdate()
		{
			FileChunk.Log("[FlyingVehiclePhysics$GotoPointState] : SGUpdate\n");
			
		}		
	}
	
	/** stan dokladnego poruszania sie do punktu */
	public class PrecisionGotoState extends GotoPointState
	{
		Vector vStart = null;
		Vector vEnd = null;
		float fStartTime;
		float fEndTime;
		float fDist;
		Vector vDir = null;
		
		public void OnEnter()
		{
			fStartTime = GetTime();
			vStart = m_Owner.GetPosition();
			vEnd = new Vector(m_vDestPoints[m_iCurrPoint]);
			vDir = new Vector(vEnd);
			vDir.Sub(vStart);
			fDist = vDir.Len();
			vDir.Normalize();
			fEndTime = fStartTime + vStart.Distance(vEnd) / 200f;
			
			m_iTurnFlag = _NO_TURN;
		}
		
		public void OnFrame()
		{
			float fTime = GetTime();
			do
			{
				if (UpdatePosition())
					break;
			} while (UpdatePhysics(fTime));
		}
		
		public boolean UpdatePosition()
		{
			float fTime = GetTime();
			
			float fValue = Smoothstep(fStartTime, fEndTime, fTime);

			Vector vPos = new Vector(vDir);
			vPos.Mul(fDist * fValue);
			vPos.Add(vStart);
			m_Owner.SetPosition(vPos);
			if (fValue >= 1f)
			{
				Finish();
				return true;
			}
			else
				return false;
		}
		
		float Smoothstep(float a, float b, float x)
		{
		    if (x < a) return 0;
		    if (x >= b) return 1;
		    x = (x-a)/(b-a);
		    return (x*x * (3-2*x));
		}    
		
		void Finish()
		{
			m_Owner.OnDestinationPos();
		}
		
		public void SGLoad(FileChunk cParentFC)
		{
			FileChunk.Log("[FlyingVehiclePhysics$PrecisionGotoPointState] : SGLoad\n");
			
			FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
			if (cFC.GetID() == ESGChunksChrome._CHK_FLYING_VEHICLE_GOTO_POINT_STATE)
			{
				super.SGLoad(cFC);

				vStart = cFC.LoadVector();
				vEnd = cFC.LoadVector();
				vDir = cFC.LoadVector();

				fStartTime = cFC.LoadFloat();
				fEndTime = cFC.LoadFloat();
				fDist = cFC.LoadFloat();				
			}
			
			cFC.delete();
		}
	
		public void SGSave(FileChunk cParentFC)
		{
			FileChunk.Log("[FlyingVehiclePhysics$PrecisionGotoPointState] : SGSave\n");
			
			FileChunk cFC = NewChunk(ESGChunksChrome._CHK_FLYING_VEHICLE_GOTO_POINT_STATE, cParentFC);
			
			super.SGSave(cFC);
			
			cFC.SaveVector(vStart);
			cFC.SaveVector(vEnd);
			cFC.SaveVector(vDir);

			cFC.SaveFloat(fStartTime);
			cFC.SaveFloat(fEndTime);
			cFC.SaveFloat(fDist);
				
			cFC.delete();
		}		
		
		public void SGUpdate()
		{
			FileChunk.Log("[FlyingVehiclePhysics$PrecisionGotoPointState] : SGUpdate\n");		
		}				
	}
	
	/** stan calkowitej bezczynnosci */
	public class IdleState extends State
	{
		public void OnEnter()
		{
			m_iAccelFlag = _NO_ACCEL;
			m_iTurnFlag = _NO_TURN;
			m_iUpDownFlag = _NO_ACCEL;
		}
	}
	
	/** stan wznoszenia sie i opadania (w szczegolnym przypadku start i ladowanie) */
	public class StartState extends State
	{
		float fStartHeight = 0f; //wysokosc docelowa
		boolean bSetLandingAngle = false; //potrzebne, gdy chcemy przed wyladowaniem ustawic forward pojazdu
		Vector vLandDestPoint = null;
        float fLastFryUpdateTime = 0;
        boolean bLandFXSpawned = false;
        float _DIST_TO_SPAWN_LAND_DUST = 200f;
        
        public void OnEnter()
		{
			m_iAccelFlag = _NO_ACCEL;
			m_iTurnFlag = _NO_TURN;

            bLandFXSpawned = false;

			fStartHeight = m_vStartLandPos.fY;
			
			// jezeli mamy przy ladowaniu sie ustawic w kierunku wektora
			if (m_vLandDestDir != null)
			{
				bSetLandingAngle = true;
				m_vDestPoints = new Vector[1];
				m_iCurrPoint = 0;
				m_vDestPoints[m_iCurrPoint] = m_Owner.GetPosition();
				m_vDestPoints[m_iCurrPoint].Add(m_vLandDestDir);
			}
			else
				bSetLandingAngle = false;
            
            fLastFryUpdateTime = 0;
		}
        
		public void OnFrame()
		{
            float fDistBefore = 0f;
            float fDistAfter = 0f;
            
			float fTime = GetTime();
			do
			{
				if (bSetLandingAngle)
				{
					Physics_MakeUpTurnDecision(1f);
					if (java.lang.Math.abs(Physics_CalcAngle()) < 1f)
					{
						bSetLandingAngle = false;
						m_iTurnFlag = _NO_TURN;
					}
				}
                
                // sprawdzamy, czy nie "minelismy" ziemi, wtedy uznajemy, ze wyladowalismy
                fDistAfter = Physics_CalcUpDownDist(m_vVehiclePos, fStartHeight);
                if (fDistBefore * fDistAfter < 0)
                {
                    m_fCurrUpDownSpeed = 0f;
                    m_iUpDownFlag = _NO_ACCEL;
                    m_vVehiclePos.fY = fStartHeight;
                }
        
				if (CheckStartDecision(m_Owner.GetPosition(), fStartHeight))
					break;
                
                fDistBefore = Physics_CalcUpDownDist(m_vVehiclePos, fStartHeight);
			} while (UpdatePhysics(fTime));

            if ( (!bLandFXSpawned) && (m_Owner.m_bLanding) )
            {
			    float fCurrDistance = Physics_CalcUpDownDist(m_Owner.GetPosition(), fStartHeight);
                if (Math.abs(fCurrDistance) < _DIST_TO_SPAWN_LAND_DUST)
                {
                    m_Owner.PlayFXDustLand();
                    bLandFXSpawned = true;
                }
            }
            
            // Palenie aktorow
            FryingActors();
		}
		
        void FryingActors()
        {
            float fTime = GetTime();
            if (m_Owner.m_fActorFryingUpdateTime > 0 &&
                fTime - fLastFryUpdateTime > m_Owner.m_fActorFryingUpdateTime)
            {
                fLastFryUpdateTime = fTime;
                m_Owner.FryActors();
            }
        }
                
		/** podjecie decyzji, czy skonczylismy startowac, czy jeszcze nie */
		boolean CheckStartDecision(Vector vPos, float fHeight)
		{
            Physics_MakeUpDownDecision(vPos, fHeight);
			
			if ( (m_fCurrUpDownSpeed == 0f) && (m_iUpDownFlag == _NO_ACCEL) )
			{
				float fCurrDistance = Physics_CalcUpDownDist(vPos, fHeight);
				if (java.lang.Math.abs(fCurrDistance) <= _UPDOWN_FLY_PRECISION)
				{
					if (!m_bStarting)
						m_Owner.VehicleLanded( GetTime() );
					else
						m_Owner.VehicleStarted( GetTime() );
					m_vLandDestDir = null;
					return true;
				}
			}
			return false;
		}
		
		public void SGLoad(FileChunk cParentFC)
		{
			FileChunk.Log("[FlyingVehiclePhysics$GotoPointState] : SGLoad\n");
			
			FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
			if (cFC.GetID() == ESGChunksChrome._CHK_FLYING_VEHICLE_GOTO_POINT_STATE)
			{
				super.SGLoad(cFC);
				
				fStartHeight = cFC.LoadFloat();					
				bSetLandingAngle = cFC.LoadBool();				
				vLandDestPoint = cFC.LoadVector();					
				fLastFryUpdateTime = cFC.LoadFloat();
				bLandFXSpawned = cFC.LoadBool();
				_DIST_TO_SPAWN_LAND_DUST = cFC.LoadFloat();					
			}
			
			cFC.delete();
		}
	
		public void SGSave(FileChunk cParentFC)
		{
			FileChunk.Log("[FlyingVehiclePhysics$StartState] : SGSave\n");
			
			FileChunk cFC = NewChunk(ESGChunksChrome._CHK_FLYING_VEHICLE_GOTO_POINT_STATE, cParentFC);
			
			super.SGSave(cFC);
			
			cFC.SaveFloat(fStartHeight);					
			cFC.SaveBool(bSetLandingAngle);				
			cFC.SaveVector(vLandDestPoint);					
			cFC.SaveFloat(fLastFryUpdateTime);
			cFC.SaveBool(bLandFXSpawned);
			cFC.SaveFloat(_DIST_TO_SPAWN_LAND_DUST);					
			
			cFC.delete();
		}		
		
		public void SGUpdate()
		{
			FileChunk.Log("[FlyingVehiclePhysics$StartState] : SGUpdate\n");
			
		}				
	}
	
	public void OnFrame()
	{
		float fTime = GetTime();
		while (UpdatePhysics(fTime))
			; // do update
	}
	
	/** zwraca true, jezeli jeszcze zostal czas do update */
	boolean UpdatePhysics(float fCurrentT)
	{
		// symulujemy fizyke na pewnym okreslonym stalym ziarnie
		float fDT = fCurrentT - m_fLastT;
		
		if (fDT == 0f)
			return false;
		
		// te wartosci bedziemy przeksztalcac w krokach
		m_vVehicleForward = m_Owner.GetForwardVector();
		m_vVehiclePos = m_Owner.GetPosition();

		if (fDT >= _MIN_PHYSICS_UPDATE_GRAIN)
		{
			UpdateOneStep(_MIN_PHYSICS_UPDATE_GRAIN);
			m_fLastT += _MIN_PHYSICS_UPDATE_GRAIN;
			fDT -= _MIN_PHYSICS_UPDATE_GRAIN;
		}
		else
		{
			UpdateOneStep(fDT);
			m_fLastT += fDT;
			fDT = 0f;
		}
        
		// fizyka zostala przesymulowana, teraz okreslamy polozenie i orientacje pojazdu
		// na podstawie m_vVehiclePos i m_vVehicleForward
		float fUpDown = m_fCurrSpeed / _MAX_FW_SPEED / 3f;
		if (fUpDown > 1f)
			fUpDown = 1f;
		fUpDown *= _MAX_VERT_ANGLE;
		
		float fLeftRight = m_fCurrTurnSpeed  / _MAX_TURN_SPEED;
		fLeftRight *= _MAX_HORIZ_ANGLE;
		
		Vector vUp = new Vector(Vector.Up);
		Vector vRightAxis = m_vVehicleForward.Cross(Vector.Up);
		vRightAxis.Normalize();

		vUp = RotateVector(Vector.Up, -fUpDown, vRightAxis);
		m_vVehicleForward = RotateVector(m_vVehicleForward, -fUpDown, vRightAxis);
		vUp = RotateVector(vUp, fLeftRight, m_vVehicleForward);
		
		m_Owner.FromUpForwardPos(vUp, m_vVehicleForward, m_vVehiclePos);
		
        // Update predkosci Ownera (zalozenie: fizyka ownera jest wylaczona!!!)
        Vector vSpeed = m_Owner.GetForwardVector();
        vSpeed.Mul(m_fCurrSpeed);
        vSpeed.fY = m_fCurrUpDownSpeed;
        m_Owner.SetSpeed(vSpeed);
		
		return (fDT > 0f);
	}
    
	void UpdateOneStep(float fStep)
	{
		Physics_UpdateTurnSpeed(fStep);
				
		Physics_UpdateSpeed(fStep);

		Physics_UpdateUpDownSpeed(fStep);
		
		Physics_UpdatePosAndOrientation(fStep);
	}
	
	int GetPathPointsNum()
	{
		if (m_vDestPoints == null)
			return 0;
		return m_vDestPoints.length;
	}
	
	/** update predkosci */
	void Physics_UpdateSpeed(float fDT)
	{
        //Log("UpdateSpeed m_fCurrSpeed " + m_fCurrSpeed + " m_iAccelFlag " + m_iAccelFlag);
        
		if (m_iAccelFlag == _ACCEL)
		{
			if (m_fCurrSpeed < _CURR_MAX_FW_SPEED)
			{
				m_fCurrSpeed += fDT * _ACCEL_FW;
				if (m_fCurrSpeed > _CURR_MAX_FW_SPEED)
					m_fCurrSpeed = _CURR_MAX_FW_SPEED;
			}
			else
			if (m_fCurrSpeed > _CURR_MAX_FW_SPEED)
			{
				m_fCurrSpeed -= fDT * _ACCEL_BW;
				if (m_fCurrSpeed < _CURR_MAX_FW_SPEED)
					m_fCurrSpeed = _CURR_MAX_FW_SPEED;
			}
		}
		else
		if (m_iAccelFlag == _DEACCEL)
		{
			if (m_fCurrSpeed > 0f)
			{
				m_fCurrSpeed -= fDT * _ACCEL_BW;
				if (m_fCurrSpeed < 0f)
					m_fCurrSpeed = 0;
			}
		}
        //Log(" -> " + m_fCurrSpeed + " fDT " + fDT + "\n");
	}
	
	/** update predkosci obracania sie */
	void Physics_UpdateTurnSpeed(float fDT)
	{
		if (m_iTurnFlag == _TURN_LEFT)
		{
			m_fCurrTurnSpeed -=  fDT * _ACCEL_TURN;
			if (m_fCurrTurnSpeed < -_MAX_TURN_SPEED)
				m_fCurrTurnSpeed = -_MAX_TURN_SPEED;
		}
		else
		if (m_iTurnFlag == _TURN_RIGHT)
		{
			m_fCurrTurnSpeed +=  fDT * _ACCEL_TURN;
			if (m_fCurrTurnSpeed > _MAX_TURN_SPEED)
				m_fCurrTurnSpeed = _MAX_TURN_SPEED;
		}
		else
		{
			if (m_fCurrTurnSpeed > 0f)
			{
				m_fCurrTurnSpeed -=  fDT * _ACCEL_TURN;
				if (m_fCurrTurnSpeed < 0f)
					m_fCurrTurnSpeed = 0f;
			}
			
			if (m_fCurrTurnSpeed < 0f)
			{
				m_fCurrTurnSpeed +=  fDT * _ACCEL_TURN;
				if (m_fCurrTurnSpeed > 0f)
					m_fCurrTurnSpeed = 0f;
			}
		}
	}

	/** update predkosci poruszania sie gora / dol */
	void Physics_UpdateUpDownSpeed(float fDT)
	{
		if (m_iUpDownFlag < 0)
		{
			m_fCurrUpDownSpeed -=  fDT * _ACCEL_UPDOWN;
			if (m_fCurrUpDownSpeed < -_MAX_UPDOWN_SPEED)
				m_fCurrUpDownSpeed = -_MAX_UPDOWN_SPEED;
		}
		else
		if (m_iUpDownFlag > 0)
		{
			m_fCurrUpDownSpeed +=  fDT * _ACCEL_UPDOWN;
			if (m_fCurrUpDownSpeed > _MAX_UPDOWN_SPEED)
				m_fCurrUpDownSpeed = _MAX_UPDOWN_SPEED;
		}
		else
		if (m_iUpDownFlag == 0)
		{
			if (m_fCurrUpDownSpeed > 0f)
			{
				m_fCurrUpDownSpeed -=  fDT * _ACCEL_UPDOWN;
				if (m_fCurrUpDownSpeed < 0f)
					m_fCurrUpDownSpeed = 0f;
			}
			
			if (m_fCurrUpDownSpeed < 0f)
			{
				m_fCurrUpDownSpeed +=  fDT * _ACCEL_UPDOWN;
				if (m_fCurrUpDownSpeed > 0f)
					m_fCurrUpDownSpeed = 0f;
			}
		}
	}
	
	/** wyznaczenie nastepna pozycje i orientacje pojazdu na podstawie
	 * wyliczonych wartosci */
	void Physics_UpdatePosAndOrientation(float fDT)
	{
		Vector vForward = new Vector(m_vVehicleForward);
		vForward.fY = 0f;
		vForward.Normalize();
		
		Vector vPos = new Vector(m_vVehiclePos);
		
		vForward = m_Owner.RotateVector(vForward, -m_fCurrTurnSpeed * fDT, Vector.Up);
		
		if (m_fCurrSpeed != 0f)
		{
			Vector vMove = new Vector(vForward);
			vMove.Mul(m_fCurrSpeed * fDT);
			vPos.Add(vMove);
		}
		
		vPos.fY += m_fCurrUpDownSpeed * fDT;
		
		m_vVehicleForward.Set(vForward);
		m_vVehiclePos.Set(vPos);
	}
	
	/** odleglosc, jaka zajmie nam hamowanie od biezacej predkosci do 0. */
	float Physics_CalcDistToStop()
	{
		float fTimeToStop = m_fCurrSpeed / _ACCEL_BW; // czas w jakim predkosc zmaleje do zera
		return fTimeToStop * (m_fCurrSpeed - (_ACCEL_BW * fTimeToStop) / 2f);
	}
	
	/** kat, jaki zakreslimy zwalniajac szybkosc obrotu do 0. */
	float Physics_CalcAngleToStop(float fAccelTurn)
	{
		float fTurnSpeed = java.lang.Math.abs(m_fCurrTurnSpeed);
		float fTimeToStop = fTurnSpeed / fAccelTurn;
		return fTimeToStop * (fTurnSpeed - (fAccelTurn * fTimeToStop) / 2f);
	}

	/** ustawia ciag pionowy w zaleznosci od tego, na jakiej wysokosci jestesmy
	 * i na jaka wysokosc chcemy sie wzniesc */
	void Physics_MakeUpDownDecision(Vector vPos, float fHeight)
	{
		// ile nam zajmie zahamowanie
		float fDistanceToStop = Physics_CalcHeightToStop(_ACCEL_UPDOWN);
		// biezaca odleglosc do przelecenia, by znalezc sie na wysokosci fHeight
		float fCurrDistance = Physics_CalcUpDownDist(vPos, fHeight);
		
		if ( (Math.abs(fCurrDistance) > _UPDOWN_FLY_PRECISION) ||
			 (fDistanceToStop > Math.abs(fCurrDistance)) )
		{
			if (fCurrDistance > 0)
			{
				// jestesmy nad punktem docelowym
				if (m_fCurrUpDownSpeed > 0)
					m_iUpDownFlag = _DEACCEL;
				else
				{
					if (fDistanceToStop > fCurrDistance)
						m_iUpDownFlag = _ACCEL;
					else
						m_iUpDownFlag = _DEACCEL;
				}
			}
					
			if (fCurrDistance < 0)
			{
				// jestesmy pod punktem docelowym
				if (m_fCurrUpDownSpeed < 0)
					m_iUpDownFlag = _ACCEL;
				else
				{
					if (fDistanceToStop > java.lang.Math.abs(fCurrDistance))
						m_iUpDownFlag = _NO_ACCEL;
					else
						m_iUpDownFlag = _ACCEL;
				}
			}
		}
		else
			m_iUpDownFlag = _NO_ACCEL;
	}
	
	void Physics_MakeUpTurnDecision(float fAnglePrecision)
	{
		float fCurrAngle = Physics_CalcAngle();
		
		// jezeli obracamy sie, sprawdzamy, czy nie powinnismy zaczac zwalniac
		if (m_iTurnFlag != _NO_TURN)
		{
			float fAngleToStop = Physics_CalcAngleToStop(_ACCEL_TURN);
			
			if (fCurrAngle > 0)
			{
				if (m_fCurrTurnSpeed < 0)
					m_iTurnFlag = _TURN_RIGHT;
				else
				{
					if (fAngleToStop > fCurrAngle)
						m_iTurnFlag = _NO_TURN;
				}
			}
			else
			if (fCurrAngle < 0)
			{
				if (m_fCurrTurnSpeed > 0)
					m_iTurnFlag = _TURN_LEFT;
				else
				{
					if (fAngleToStop > -fCurrAngle)
						m_iTurnFlag = _NO_TURN;
				}
			}
		}
		else
		{
			if (m_fCurrTurnSpeed == 0f)
			{
				if (java.lang.Math.abs(fCurrAngle) > fAnglePrecision)
				{
					if (fCurrAngle > 0)
						m_iTurnFlag = _TURN_RIGHT;
					else
						m_iTurnFlag = _TURN_LEFT;
				}
				else
					m_iUpDownFlag = _NO_TURN;
			}
		}
	}
	
	/** oblicza odleglosc od vehicla do pewnej wysokosci */
	float Physics_CalcUpDownDist(Vector vPos, float fHeight)
	{
		return (vPos.fY - fHeight);
	}

	/** wysokosc, jaka przebedziemy hamujac ruch */
	float Physics_CalcHeightToStop(float fAccel)
	{
		float fUpDownSpeed = java.lang.Math.abs(m_fCurrUpDownSpeed);
		float fTimeToStop = fUpDownSpeed / fAccel;
		return fTimeToStop * (fUpDownSpeed - (fAccel * fTimeToStop) / 2f);
	}
	
	/** okresla na podstawie biezacego punktu docelowego, kierunek obracania sie */
	int Physics_CalcTurnDirection()
	{
		Vector vForward = m_Owner.GetForwardVector();
		vForward.fY = 0f;
		vForward.Normalize();
		Vector vDestDir = new Vector(m_vDestPoints[m_iCurrPoint]);
		vDestDir.Sub(m_Owner.GetPosition());
		vDestDir.fY = 0f;
		vDestDir.Normalize();
			
		if (vDestDir.Cross(vForward).fY > 0)
			return _TURN_RIGHT;
		else
			return _TURN_LEFT;
	}

	/** kat, jaki jest pomiedzy forward pojazdu a wektorem do biezacego punktu docelowego */
	float Physics_CalcAngle()
	{
		Vector vForward = m_Owner.GetForwardVector();
		vForward.fY = 0f;
		vForward.Normalize();
		Vector vDestDir = new Vector(m_vDestPoints[m_iCurrPoint]);
		vDestDir.Sub(m_Owner.GetPosition());
		vDestDir.fY = 0f;
		vDestDir.Normalize();
		float fReturn = vDestDir.Dot(vForward);
		if (fReturn < -1f)
			fReturn = -1;
		if (fReturn > 1f)
			fReturn = 1f;
		fReturn = (float) java.lang.Math.acos(fReturn);
		if (vDestDir.Cross(vForward).fY < 0f)
			fReturn *= -1f;
		fReturn *= _RAD_TO_DEG;
		return fReturn;
	}
	
	/** odleglosc pojazdu do biezacego punktu docelowego na plaszczyznie */
	float Physics_CalcDist()
	{
		Vector vDestDir = new Vector(m_vDestPoints[m_iCurrPoint]);
		vDestDir.Sub(m_Owner.GetPosition());
		vDestDir.fY = 0;
		return vDestDir.Len();
	}
	
	boolean Physics_EnoughTimeToTurn()
	{
		boolean bReturn = true;
		float fAngle = Physics_CalcAngle();
		float fDist = Physics_CalcDist();

		// jezeli mamy bardzo mala szybkosc skretu, wychodzimy by nie bylo infinity
		if (java.lang.Math.abs(m_fCurrTurnSpeed) < 1f)
			return true;

		// mamy mala szybkosc poruszania sie, wychodzimy
		if (m_fCurrSpeed < 100f)
			return true;
		
		float fTurnTime = Physics_CalcAngle() / m_fCurrTurnSpeed;
		// jezeli obracamy sie w druga strone, niz powinnismy, wychodzimy
		if (fTurnTime < 0)
			return true;
	
		// jezeli czas obrotu jest wiekszy niz czas dolecenia na miejsce
		if (fTurnTime > fDist / m_fCurrSpeed)
			return false;
		else
			return true;
	}
	
   //*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean bActivateSG = false;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[FlyingVehiclePhysics.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_FLYING_VEHICLE_PHYSICS)
        {	
			super.SGLoadChunk(cFC);
			
			// biezaca predkosc
			m_fCurrSpeed = cFC.LoadFloat();
			// flaga mowiaca, czy przyspieszamy (1) czy zwalniamy (-1) czy nic nie robimy (0)
			m_iAccelFlag = cFC.LoadInt();
			// flaga mowiaca, czy skrecamy
			m_iTurnFlag = cFC.LoadInt();
			// predkosc skretu w stopniach/s
			m_fCurrTurnSpeed = cFC.LoadFloat();
			// flaga mowiaca, czy wznosimy sie, czy opadamy
			m_iUpDownFlag = cFC.LoadInt();
			// aktualna predkosc lotu gora / dol
			m_fCurrUpDownSpeed = cFC.LoadFloat();
			//punkty docelowe pojazdu
			if (cFC.LoadBool())
			{
				int nSize = cFC.LoadInt();
				
				if ((m_vDestPoints == null) || (nSize != m_vDestPoints.length))
					m_vDestPoints = new Vector[nSize];

				for (int i=0; i < m_vDestPoints.length; i++)
					m_vDestPoints[i] = cFC.LoadVector();
			}
			else
				m_vDestPoints = null;
			// biezacy indeks punktu docelowego pojazdu
			m_iCurrPoint = cFC.LoadInt();
			// czy w stanie moveto w ostatnim punkcie mamy czekac, az skonczymy sie obracac,
			m_bStopTurn = cFC.LoadBool();
			// ostatni czas update fizyki
			m_fLastT = cFC.LoadFloat();													
			// kierunek, w ktorym chcemy wyladowac
			m_vLandDestDir = cFC.LoadVector();
			// wektor forward pojazdu
			m_vVehicleForward = cFC.LoadVector();
			// wektor pozycji pojazdu
			m_vVehiclePos = cFC.LoadVector();
			// czy ma dokladnie dolatywac w ostatnim punkcie
			m_bPreciseMoving = cFC.LoadBool();
			// wektor, gdzie chcemy wystartowac / wyladowac
			m_vStartLandPos = cFC.LoadVector();
			// czy zwalniac, gdy zblizamy sie do punktu docelowego - wazne przy patrolowaniu
			m_bSlowDownToPoint = cFC.LoadBool();
			// wiadomosc dla stanu startstate czy startujemy, czy ladujemy	
			m_bStarting = cFC.LoadBool();
			//pojazd, ktorym kieruje TA fizyka
			m_Owner = (FlyingVehicle)cFC.LoadObjectRef();
			
			bActivateSG = cFC.LoadBool();											
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[FlyingVehiclePhysics.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_FLYING_VEHICLE_PHYSICS, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		// biezaca predkosc
		cFC.SaveFloat(m_fCurrSpeed);
		// flaga mowiaca, czy przyspieszamy (1) czy zwalniamy (-1) czy nic nie robimy (0)
		cFC.SaveInt(m_iAccelFlag);
		// flaga mowiaca, czy skrecamy
		cFC.SaveInt(m_iTurnFlag);
		// predkosc skretu w stopniach/s
		cFC.SaveFloat(m_fCurrTurnSpeed);
		// flaga mowiaca, czy wznosimy sie, czy opadamy
		cFC.SaveInt(m_iUpDownFlag);
		// aktualna predkosc lotu gora / dol
		cFC.SaveFloat(m_fCurrUpDownSpeed);
		//punkty docelowe pojazdu
		cFC.SaveBool(m_vDestPoints != null);
		if (m_vDestPoints != null)
		{
			cFC.SaveInt(m_vDestPoints.length);
			for (int i=0; i < m_vDestPoints.length; i++)
				cFC.SaveVector(m_vDestPoints[i]);
		}
		// biezacy indeks punktu docelowego pojazdu
		cFC.SaveInt(m_iCurrPoint);
		// czy w stanie moveto w ostatnim punkcie mamy czekac, az skonczymy sie obracac,
		cFC.SaveBool(m_bStopTurn);
		// ostatni czas update fizyki
		cFC.SaveFloat(m_fLastT);													
		// kierunek, w ktorym chcemy wyladowac
		cFC.SaveVector(m_vLandDestDir);
		// wektor forward pojazdu
		cFC.SaveVector(m_vVehicleForward);
		// wektor pozycji pojazdu
		cFC.SaveVector(m_vVehiclePos);
		// czy ma dokladnie dolatywac w ostatnim punkcie
		cFC.SaveBool(m_bPreciseMoving);
		// wektor, gdzie chcemy wystartowac / wyladowac
		cFC.SaveVector(m_vStartLandPos);
		// czy zwalniac, gdy zblizamy sie do punktu docelowego - wazne przy patrolowaniu
		cFC.SaveBool(m_bSlowDownToPoint);
		// wiadomosc dla stanu startstate czy startujemy, czy ladujemy	
		cFC.SaveBool(m_bStarting);
		//pojazd, ktorym kieruje TA fizyka		
		cFC.SaveObjectRef(m_Owner);
		
		cFC.SaveBool(bActivatePhysics);
				
		cFC.delete();
    }	    					
	
	public void SGUpdate()	
	{
		// zakomentowa jesli cos sie wali /**/
		super.SGUpdate();
	
		if (!bActivatePhysics & bActivateSG)
			ActivatePhysics();
		else if (bActivatePhysics & !bActivateSG)
			DeactivatePhysics();		
		/**/
	}
}