public class SeatInfo implements ISaveGame
{
	float m_fBeginPosRadius = 250f;					// promien kuli, w obrebie ktorej mozna wsiasc
	int m_iRole = EPassengerRole._VEHICLE_NONE;		// rola, ktora pelni obiekt siedzacy na tym miejscu
	TriggerObject m_ObjectToOccupy = null;			// obiekt, do ktorego wsiadamy / wysiadamy
	int m_iSeatID = -1;								// numer meshelementu do siedzenia
	int m_iToSeatID = -1;							// numer meshelementu od ktorego zaczynamy wsiadanie
	int m_iAnimType = EGetInOutAnimType._UNKNOWN;	// typ animacji do odegrania
	Pawn m_Occupant = null;							// obiekt, ktory siedzi na tym miejscu
	Pawn m_Reserver = null;                         // obiekt, ktory zarezerwowal to miejsce
	boolean m_bWaitForNoticeBeforeGetOut = true;    // znacznik, ze obiekt siedzacy ma poczekac na Notice przed wysiadaniem
    boolean m_bCanGetOut = true;                    // znacznik, ze obiekt siedzacy moze wysiasc
	Vector m_vMoveDir = new Vector(0f, 0f, 0f);		// wektor (ukl. lokalny) przesuniecia punktu, z ktorego mozna wsiadac
    int m_iDieAnimType = AIAnimationsHuman.A_DIE_DRIVE_B; // animacja smierci do odpalenia
    String m_sTip = null;                           //nazwa wyswietlana na hudzie
    
	// Tylko na potrzeby zapisywania i odczytu SaveGame !
	public SeatInfo()
	{
	}
    
    public SeatInfo(TriggerObject occupyObject, String sSeatName, int iRole,
                    int iAnimType)
    {
        this(occupyObject, sSeatName, iRole, iAnimType, true);
    }
    
	public SeatInfo(TriggerObject occupyObject, String sSeatName, int iRole,
                    int iAnimType, boolean bWaitForNoticeBeforeGetOut)
	{
        
		if (sSeatName == null)
		{
			GameObject.CrashLog("SeatInfo: sSeatName == null\n");
			return;
		}
		
        m_sTip = EPassengerRole.GetRoleNameForTip(iRole);
		        
        if (occupyObject == null)
		{
			GameObject.CrashLog("SeatInfo: vehicleObject == null\n");
			return;
		}
		
		m_ObjectToOccupy = occupyObject; 
		
		m_iSeatID = m_ObjectToOccupy.GetElementID(sSeatName);
		if (m_iSeatID == -1)
		{
			GameObject.CrashLog("SeatInfo: nie istnieje element o nazwie:" + sSeatName + " w " + m_ObjectToOccupy + "\n");
			return;
		}
		
		m_iAnimType = iAnimType;
		m_iRole = iRole;
        m_bWaitForNoticeBeforeGetOut = bWaitForNoticeBeforeGetOut;
	}
	
	public SeatInfo(TriggerObject occupyObject, String sSeatName, int iRole,
					int iAnimType, Vector vMoveDir, boolean bWaitForNoticeBeforeGetOut)
	{
		this(occupyObject, sSeatName, iRole, iAnimType, vMoveDir);
        
        m_bWaitForNoticeBeforeGetOut = bWaitForNoticeBeforeGetOut;
	}
	
	public SeatInfo(TriggerObject occupyObject, String sSeatName, int iRole,
					int iAnimType, Vector vMoveDir)
	{
		this(occupyObject, sSeatName, iRole, iAnimType);
		
		m_vMoveDir.Set(vMoveDir);
	}
	
	public Vector GetStartPos()
	{
		if (m_iToSeatID != -1)
			return m_ObjectToOccupy.GetElementPos(m_iToSeatID);
		if (m_iSeatID != -1)
		{
			Vector vStartPos = null;
				
			if (m_vMoveDir.Len2() > 0f)
			{
				vStartPos = m_ObjectToOccupy.VectorLocalToWorld(m_vMoveDir);
				vStartPos.Add(m_ObjectToOccupy.GetElementPos(m_iSeatID));
			}
			else
			{			
				vStartPos =  m_ObjectToOccupy.GetRightVector();
				if (m_iAnimType == EGetInOutAnimType._RIGHT_SIDE)
					vStartPos.Negate();
				vStartPos.Mul(GetMoveInRadius());
				vStartPos.Add(m_ObjectToOccupy.GetElementPos(m_iSeatID));
			}
			
			return vStartPos;
		}
		return null;
	}
	
	public Vector GetStartLocalPos()
	{
		Vector vStartLocalPos = GetStartPos();
		if (vStartLocalPos != null)
			return m_ObjectToOccupy.PointWorldToLocal(vStartLocalPos);
		return null;
	}
	
	public Vector GetStartDir()
	{
		if (m_iToSeatID != -1)
        {
            return m_ObjectToOccupy.GetElementForwardVector(m_iToSeatID);
        }
		if (m_iSeatID != -1)
			return m_ObjectToOccupy.GetElementForwardVector(m_iSeatID);
		return null;
	}
	
	public Vector GetStartLocalDir()
	{
		Vector vStartLocalDir = GetStartDir();
		if (vStartLocalDir != null)
			return m_ObjectToOccupy.VectorWorldToLocal(vStartLocalDir);
		return null;
	}

	public Vector GetSeatPos()
	{
		return m_ObjectToOccupy.GetElementPos(m_iSeatID);
	}
	
	public Vector GetSeatLocalPos()
	{
		if (m_iSeatID != -1)
			return m_ObjectToOccupy.PointWorldToLocal(m_ObjectToOccupy.GetElementPos(m_iSeatID));
		return null;
	}
	
	public Vector GetSeatDir()
	{
		if (m_iSeatID != -1)
			return m_ObjectToOccupy.GetElementForwardVector(m_iSeatID);
		return null;
	}

	public Vector GetSeatLocalDir()
	{
		if (m_iSeatID != -1)
			return m_ObjectToOccupy.VectorWorldToLocal(GetSeatDir());
		return null;
	}
	
	public Vector GetSeatUpDir()
	{
		if (m_iSeatID != -1)
			return m_ObjectToOccupy.GetElementUpVector(m_iSeatID);
		return null;
	}
	
    int GetSeatElement()
    {
        return m_iSeatID;
    }
    
	public int GetRole()
	{
		return m_iRole;
	}
	
	public void Reserve(Pawn reserver)
	{
		if ( (m_Reserver == null) || (reserver == null) )
			m_Reserver = reserver;
		else
		{
			GameObject.CrashLog("Nie mozna zarezerwowac: SeatInfo.m_Reserver != null\n");
		}
	}
	
	public Pawn GetReserver()
	{
		return m_Reserver;
	}
	
	public boolean IsOccupied()
	{
		return (m_Occupant != null);
	}
	
	public Pawn GetOccupant()
	{
		return m_Occupant;
	}
	
	public TriggerObject GetOccupiedObject()
	{
		return m_ObjectToOccupy;
	}

	public float GetMoveInRadius()
	{
		return m_fBeginPosRadius;
	}
	
    public boolean AttachOccupant()
    {
		if (m_iSeatID != -1 && m_Occupant != null)
        {
            m_ObjectToOccupy.AttachChildAndKeepWorldPos(m_Occupant, m_iSeatID);
            return true;
        }
        else
            return false;
    }
    
    public boolean WaitForNoticeBeforeGetOut()
    {
        return m_bWaitForNoticeBeforeGetOut;
    }
    
    public void SetCanGetOut(boolean bCanGetOut)
    {
        m_bCanGetOut = bCanGetOut;
    }
    
    public boolean CanGetOut()
    {
        return m_bCanGetOut;
    }
	
	void Occupy(Pawn occupant)
	{
		m_Occupant = occupant;
	}

	public String GetInfo()
	{
		return "SEAT: " + this + "\n" +
			   "m_fBeginPosRadius:" + m_fBeginPosRadius + ", m_iRole:" + m_iRole +
			   ", m_Occupant:" + m_Occupant + ", m_Reserver:" + m_Reserver;
	}
    
    public int GetDieAnimID()
    {
        return m_iDieAnimType;
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[SeatInfo.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_SEAT_INFO)
        {				
			// obiekt, ktory siedzi na tym miejscu		
			m_Occupant = (Pawn)GameObject.cSG.LoadObjectRef(cFC);
			// obiekt, ktory zarezerwowal to miejsce
			m_Reserver = (Pawn)GameObject.cSG.LoadObjectRef(cFC);
			// znacznik, ze obiekt siedzacy moze wysiasc
			m_bCanGetOut = cFC.LoadBool();
			
			// rola, ktora pelni obiekt siedzacy na tym miejscu
			m_iRole = cFC.LoadInt();
			// obiekt, do ktorego wsiadamy / wysiadamy
			m_ObjectToOccupy = (TriggerObject)GameObject.cSG.LoadObjectRef(cFC);
			// numer meshelementu do siedzenia
			m_iSeatID = cFC.LoadInt();
			// numer meshelementu od ktorego zaczynamy wsiadanie
			m_iToSeatID = cFC.LoadInt();
			// typ animacji do odegrania
			m_iAnimType = cFC.LoadInt();

			// znacznik, ze obiekt siedzacy ma poczekac na Notice przed wysiadaniem
			m_bWaitForNoticeBeforeGetOut = cFC.LoadBool();
			// znacznik, ze obiekt siedzacy moze wysiasc
			m_bCanGetOut = cFC.LoadBool();
			// wektor (ukl. lokalny) przesuniecia punktu, z ktorego mozna wsiadac
			m_vMoveDir = cFC.LoadVector();
            
            m_fBeginPosRadius = cFC.LoadFloat();
        }
		
		FileChunk.Log("[SeatInfo.SGLoadChunk] - END\n");
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[SeatInfo.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_SEAT_INFO, cFCParent);
	
		// obiekt, ktory siedzi na tym miejscu		
		GameObject.cSG.SaveObjectRef(cFC,m_Occupant);
		// obiekt, ktory zarezerwowal to miejsce
		GameObject.cSG.SaveObjectRef(cFC,m_Reserver);
		// znacznik, ze obiekt siedzacy moze wysiasc
		cFC.SaveBool(m_bCanGetOut);		
	
		// rola, ktora pelni obiekt siedzacy na tym miejscu
		cFC.SaveInt(m_iRole);
		// obiekt, do ktorego wsiadamy / wysiadamy
		GameObject.cSG.SaveObjectRef(cFC,m_ObjectToOccupy);
		// numer meshelementu do siedzenia
		cFC.SaveInt(m_iSeatID);
		// numer meshelementu od ktorego zaczynamy wsiadanie
		cFC.SaveInt(m_iToSeatID);
		// typ animacji do odegrania
		cFC.SaveInt(m_iAnimType);

		// znacznik, ze obiekt siedzacy ma poczekac na Notice przed wysiadaniem
		cFC.SaveBool(m_bWaitForNoticeBeforeGetOut);
		// znacznik, ze obiekt siedzacy moze wysiasc
		cFC.SaveBool(m_bCanGetOut);
		// wektor (ukl. lokalny) przesuniecia punktu, z ktorego mozna wsiadac
		cFC.SaveVector(m_vMoveDir);

        cFC.SaveFloat(m_fBeginPosRadius);
        
		FileChunk.Log("[SeatInfo.SGSaveChunk] - END\n");
	
        cFC.delete();
    }
	
	public void SGUpdate() 
	{
		AttachOccupant();
        m_sTip = EPassengerRole.GetRoleNameForTip(m_iRole);
	}
    
    String GetNetInfo()
    { 
        return ""+((m_Occupant!=null)?(" "+m_Occupant.GetNetInfo()):"")+ 
                  ((m_ObjectToOccupy!=null)?(" "+m_ObjectToOccupy.GetNetInfo()):"");
        
    }
}
