import java.util.*;

/** Pojazd. Pawn moze do niego wsiasc i jezdzic. */
public class Vehicle extends ChromeActor implements IDrivenVehicle, ISeatVehicle, 
    ipar_MotionSpeed, ipar_MovieVehicle
{
	static float RAD_TO_DEG = 180f / 3.1415926f;
	int N_PHYSICS_UPDATES_PER_S = 20;
	float PHYSICS_GRAIN = 0f;
	
///////////////////////////////////////////////////////////////////////////////
// kamera i wyglad pojazdu
///////////////////////////////////////////////////////////////////////////////
	/** parametry kamery znajdujacej sie nad pojazdem */
	float m_fObservedSightVAngle = 15f;
	float m_fOverObservedHeight = 300f;
	float m_fOverObservedBack = 800f;

    String sBlobMaterial = "blobs.mat";
    float fBlobMapSize = 220;

	/** czy symulowac pojazd (fizyka) */
	boolean m_bSimulate = true;
	
///////////////////////////////////////////////////////////////////////////////
// wsiadanie / wysiadanie
///////////////////////////////////////////////////////////////////////////////
	
	/** informacja o siedzeniach w pojezdzie */
	java.util.ArrayList aSeatsInfo = new java.util.ArrayList();
	
    /** krok z jaka bedziemy sie "oddalac" w poszukiwaniach */
    static float _GETOUT_DISTANCEDELTA = 50.0f;
    /** kat o jaki bedziemy sie obracac */
    static float _GETOUT_ANGLEDELTA = 10.0f;
    
    /** Akcje odpalane jak kierowca wsiada do pojazdu */
    public Action[] aOnGotInDriver  = null;//#\VEHICLE;Actions executed when driver enters car
    
    /** Akcje odpalane jak pasaer wsiada do pojazdu */
    public Action[] aOnGotInPassenger  = null;//#\VEHICLE;Actions executed when passenger enters car

    public Action[] aOnGetOutDriver = null;//#\VEHICLE;Actions executed when driver leaves car
    
    public Action[] aOnGetOutPassenger = null;//#\VEHICLE;Actions executed when passenger leaves car
    
    /** czy wsiadamy do samochodu jako kierowca, czy tylko jako pasazer */
	boolean bExecutorAsPassenger = false;//#VEHICLE;When set to true player will enter vehicle as passenger otherwise as driver
    
    // minimalna (w procentach) energia potrzebna do jazdy
    public float fMinHealthCoeffToDrive = 0.2f;//#VEHICLE;Health threshold below which car can't be driven

    // minimalna (w procentach) energia potrzebna do jazdy w sieci
    public float fMinNetHealthCoeffToDrive = 0.0f;//#Net;Health threshold below which car can't be driven in net game
    
    static float _TIME_TO_GET_OUT_IF_VEHICLE_ALREADY_DESTROYED = 5.0f;
    
    public Action[] aOnCantDrive = null;//#\VEHICLE;Actions executed when vehicle crashes
    
///////////////////////////////////////////////////////////////////////////////
// symulacja
///////////////////////////////////////////////////////////////////////////////
	/** fizyka pojazdu */
	public PhysicsVehicle m_Physics = null;
	
	/** czy fizyka jest aktywna w danym momencie */
	boolean m_bPhysicsActive = false;

	/** czy silnik jest wlaczony */
    public boolean m_bEngineOn = false;

    /** Stan ruchu */
    VehicleMotionState motion = null;

    /** Ostatni czas symulacji */
    protected float fLastT = 0;

	/** czy pojazd moze jechac (czy np. nie jest uszkodzony) */
	public boolean m_bCanDrive = true;
    public boolean DoesCanDrive(){ return m_bCanDrive; }
    
    /** Typ pojazdu */
    public byte nVehicleType = EVehicleType._CAR;
    
    /** Paliwo [percent] */
    public float fFuel = 100.0f;//#VEHICLE;% of fuel
    
    /** Zuzycie paliwa (liczone w [%/s]) */
    public float fFuelConsumption = 0.01f; //#VEHICLE;%/s
	
	/** potrzebne do update na stalym ziarnie */
	public float m_fAllFrameTime = 0;

    public boolean m_bVehicleInMovie = false;
///////////////////////////////////////////////////////////////////////////////
// symulacja jazdy po sciezce patrolowej
///////////////////////////////////////////////////////////////////////////////
	/** numerek biezacego punktu patrolowego */
	int m_iCurrPatrolPoint = 0;
	/** wartosc uzywana, gdy sciezka jest w obie strony */
	int m_iAddValue = 1;
	/** czy jazda po sciezce zostala zakonczona */
	boolean m_bDriveFinished = false;
	/** czy w danej chwili przerywamy patrolowanie sciezki, zeby sie zatrzymac */
	boolean m_bBreakPatrol = false;
	
	/** czy w danej chwili patroluje */
	boolean m_bIsPatrolling = false;

	/** sciezka pojazdu */
	Path m_VehiclePath = null;
	/** zapamietana wartosc poslizgu vehicle */
	float m_fOldSlideValue;
	float m_fOldSlowDownValue;
	float m_fOldAccelValue;
	float m_fOldBrakeValue;
	
	static float _PATH_SLOW_VALUE = 5000f;
	
///////////////////////////////////////////////////////////////////////////////
//  dzwieki
///////////////////////////////////////////////////////////////////////////////
    /** Nazwa dzwieku odpalanego przy starcie silnika */
    public String m_sSoundStartName = null;
    /** Nazwa dzwieku odpalanego podczas jazdy */
    public String m_sSoundLoopName = null;
    /** Zasieg dzwieku silnika */
    public float m_fSoundLoopMin = 1000.f;//#SOUND
    /** Zasieg dzwieku silnika */
    public float m_fSoundLoopMax = 5000.f;//#SOUND
    /** Nazwa dzwieku odpalanego podczas wylaczania silnika */
    public String m_sSoundStopName = null;
	/** Nazwa dzwieku odpalanego podczas hamowania */
    public String m_sSoundBrakeName = null;
    /** Nazwa dzwieku odpalanego podczas przyspieszania */
    public String m_sSoundAccelName = null;
	/** Nazwa dzwieku odpalanego podczas kolizji */
    public String m_sSoundHitName = null;
	/** Nazwa dzwieku odpalanego podczas poslizgu */
    public String m_sSoundSlideName = null;
    
    /** Czas trwania dzwieku startu */
    float m_fSoundVehicleStartDuration = 0.f;
    
    /** identyfikator dzwieku odpalanego podczas jazdy */
    public int m_nSoundLoopId = 0;

    boolean m_bPlayEngineStartStopSound = true;
    
    /** mnozniki czestotliwosci dzwieku m_sSoundLoopName */
    /** czestotliwosc dzwieku przy predkosci fWheelsSpeed == 0.0f */
    public float m_fMinFreq     = 1.0f;
    /** czestotliwosc dzwieku przy predkosci fWheelsSpeed == fMaxSpeed */
    public float m_fMaxFreq     = 1.1f;
    /** czestotliwosc dzwieku przy predkosci fWheelsSpeed == fMaxBackSpeed */
    public float m_fMaxFreqBack = 1.1f;
    
    /** informacja o tym czy autko odpalilo dzwieki ruszania,
     * na poczatku ustawione na false zeby przy pierwszym wejsciu
     * do StillState nie byl odpalany dzwiek wylaczania silnika */
    boolean m_bSoundsStarted = false;
    
///////////////////////////////////////////////////////////////////////////////
// update HUD'a
///////////////////////////////////////////////////////////////////////////////
	
///////////////////////////////////////////////////////////////////////////////
// akcje edytorowe
///////////////////////////////////////////////////////////////////////////////
	/** Wartosci szybkosci edytorowych ([STILL, SWIM,] CRAWL, DUCK, WALK, RUN) dla pojazdu */
	float[] m_aSpeeds = null;
    //znacznik czy ma byc fizyka ustawiana wg powyzszych ustawien
    boolean m_bUseMotionSpeed = true;
	
	boolean m_bEditorAction = false;
	IVehiclePassenger m_VehicleController = null;

///////////////////////////////////////////////////////////////////////////////
// FXy
///////////////////////////////////////////////////////////////////////////////

    public String sBlowUpFX = "FXBlowBarrel";//#FX;
    public String sBlowUpSound = "Data/Sounds/Weapon/GrenadeExplosion";//#FX;
    
    // Trzesienie kamera
    public float fBlowUpShakeStrength = 0;//#FX;
    public float fBlowUpShakeDistMin = 0;//#FX;
    public float fBlowUpShakeDistMax = 0;//#FX;

    
    //jaka czesc obrazen pojazdu trafia do pasazerow
    float m_fOccupantsDamageMul = 0.25f;
    //dla Singla
    float m_fSingleOccupantsDamageMul = 0.25f;
    //to samo ale dla sieci
    public float m_fNetOccupantsDamageMul = 0.0f;//#Net

    //jaka czesc blast damage trafia do kierowcy,gunnera/pasazera
    public float m_fOccupantsBlastDamageMul = 1.0f;//#Net;Player(drivier,gunner,passenger) damage mult in blast damage
    float GetOccupantsBlastDamageMul(){ return m_fOccupantsBlastDamageMul; }
    
    //jakie obrazenia (w stosunku do MaxHealth) dostaja pasazerowie przy 
    //rozwalaniu pojazdu (wykorzystywane w grze sieciowej)
    float m_fOccupantsDamageMaxHealthMulOnCrash = 1.0f;//#Net
    float GetOccupantsDamageMaxHealthMulOnCrash(){ return m_fOccupantsDamageMaxHealthMulOnCrash; }
        
    
    /** Czestotliwosc odpalania FX dymu */
    public float fFXSmokeUpdateTime = 0.5f;//#Vehicle
    /** Rodzaj odpalanego FX (nazwa sekwencji) */
    public String sFXSmokeName = "Smoke1";//#Vehicle
    /** Czy odpalac efekt dymu przy pojezdzie niezdolnym do jazdy */
    public boolean bSpawnSmokeFX = true;//#Vehicle
    
    int METHOD_UpdateSmokeFX = 0;
    boolean m_bSmokeSpawned = false;

    /* parametry do bujajacej sie w osi Y kamery */
    float m_fAttenuation = 0.92f;
    float m_fForceFactor = 150f;
    float m_fAmplitudeCoeff = 0.3f;

    /** wielkosc stampa na pathmapie tego pojazdu */
    int m_iStampSize = -1;
    
    /** Pod na ktorym spawnowany jest pojazd */
    public VehiclePod m_cPod = null;//#Net
    
    /** Czy w danej chwili gracze tylko jednego z teamow moga nim jezdzic?*/
    //to w zasadzie mialo byc wyciagniete do menu game settings, ale
    //na sztywno zostaje na TRUE!
    boolean m_bTeamExclusive = true;
    
    /**Czy w danej chwili kierowca jest lokalny gracz (do sieci) */
    boolean m_bLocalPlayerDriver = false;

    public boolean m_bMakeVehicleNoiseInMovie = false;//#Vehicle
    
    String sVehicleProfilingPage = "Vehicle Profiling";
    static boolean bInitProfiling = true;
    static int PROFILE_GeneralA = 0;
    static int PROFILE_GeneralB = 0;
    static int PROFILE_GeneralC = 0;

    Vector m_vFXDir = new Vector(Vector.Up);
    Vector m_vFXParticlesDir = new Vector(Vector.Up);
    Vector m_vPositionModifier = new Vector(Vector.Zero);
        
    /**
     * Wrak pojazdu
     */
    String       m_sWreckMeshName = null;
    VehicleWreck m_Wreck          = null;
    boolean      m_bWreckVisible  = false;
    
    /** Akcje odpalane jak pojazd skoliduje z playerem */
    public Action[] aOnCollisionWithPlayer  = null;//#\VEHICLE;Actions executed when the vehicle collides with Player
    
	Vehicle()
    {
        super();
        
        fHealth        = 2000.0f;
        
        m_sSoundAccelName = "Data/Sounds/Vehicles/VehicleAccel";
		m_sSoundBrakeName = "Data/Sounds/Vehicles/VehicleBrake";
	    m_sSoundHitName	  = "Data/Sounds/Vehicles/VehicleHit";
		m_sSoundSlideName = "Data/Sounds/Vehicles/VehicleSlide";
        m_sSoundStartName = "Data/Sounds/Vehicles/JeepEngineStart";
        m_sSoundLoopName  = "Data/Sounds/Vehicles/JeepEngineLoop";
        m_sSoundStopName  = "Data/Sounds/Vehicles/JeepEngineStop";
        m_fMinFreq        = 0.6f;
        m_fMaxFreq        = 1.3f;
        m_fMaxFreqBack    = 1.2f;
        m_fSoundVehicleStartDuration = 0.f;
		
		bIsActiveTarget = true;

        bIsAITarget = true;
        
		m_aSpeeds = new float[] { 0f, 210f, 630f, 1050f, 1575f, 2100f };
    }

    int METHOD_ServerHideDo = 0;
    int METHOD_IndicatorFX  = 0 ;
    
   
    void OnCreate()
    {
        METHOD_ServerHideDo     = RegisterCallMethod( "ServerHideDo" );
        METHOD_IndicatorFX      = RegisterCallMethod( "IndicatorFX" );
            
        super.OnCreate();
		
		bSG = true;
		
        SetLighting(true);
        
		fMaxHealth = fHealth;
		fLastT = GetTime();
        motion = new VehicleMotionState();

		N_PHYSICS_UPDATES_PER_S = 30;
		PHYSICS_GRAIN = 1f / (float) N_PHYSICS_UPDATES_PER_S;

		CreatePhysics();
		
		SetPhysicsType(EPhysics._NONE);
		SetFlag(EFlags._BLOCK_TRACE); 
		
        SetFlag(EFlags._COLLISION);
        SetFlag(EFlags._BLOCK_MOVE);
		
        ClearFlag(EFlags._FXPARTICLE_COLLISION);
		
        
		SetCollHandler(ECollFlags.OBJCOL_HANDLER_FULLINFOMOVE);
		SetCollAction(ECollFlags.OBJCOL_ACTION_SLIDE);
		SetCollFlags(ECollFlags.OBJCOL_FLAG_NOTERRAIN);
		SetCollFlags(ECollFlags.OBJCOL_FLAG_ELLNORMAL);
        SetCollFlags(ECollFlags.OBJCOL_FLAG_TESTTERRAINTYPE);
        SetCollFlags(ECollFlags.OBJCOL_FLAG_PARTICLENONAA);

        ClearFlag(EFlags._NO_ANIMATION);
        ClearFlag(EFlags._NO_ANIMATION_REFRESH);
        
		if (m_Physics != null && m_bUseMotionSpeed)
        {
            m_Physics.m_Params._WHEEL_MOVE_SPEED_COEF = 1f;
            m_Physics.m_Params.SetMaxSpeed(m_aSpeeds[MotionState.SPEED_RUN]);
        }
	            
        RegisterKeyRequest();
        
        m_bModuleNet = GetModule() instanceof ModuleNet;
        //TODO: fixit
        m_fOccupantsDamageMul = m_bModuleNet ? m_fNetOccupantsDamageMul : m_fSingleOccupantsDamageMul;
		
		m_fOldSlideValue = m_Physics.m_Params._SLIDE_SUPRESS;
        
        SetState("StillState");
		
		CallMethodIn("TryToAttachDriver", 0f);

        
		// ustawiamy recznie tryb (akcje generujemy dynamicznie w zaleznosci, czy wsiadamy, czy wysiadamy
        bExtendedExecutionMode = true;

        SetFlag(EFlags._NOT_STATIC);
        
        METHOD_UpdateSmokeFX = RegisterCallMethod("UpdateSmokeFX");
        
        if (m_iStampSize > 0)
            UpdatePathMap(true, m_iStampSize, true);
        
        //CrashLog("[Vehicle.OnCreate] OnCreateDone\n");

        if (bInitProfiling)
        {
            if (PROFILE_GeneralA == 0)
                PROFILE_GeneralA = ProfileAdd(sVehicleProfilingPage, "GeneralA");
            if (PROFILE_GeneralB == 0)
                PROFILE_GeneralB = ProfileAdd(sVehicleProfilingPage, "GeneralB");
            if (PROFILE_GeneralC == 0)
                PROFILE_GeneralC = ProfileAdd(sVehicleProfilingPage, "GeneralC");
            bInitProfiling = false;
        }
        
        SetCorrectPosOnRender( true );
        m_fInterpolateSpeedOnMoveMul = 1.05f;
        
        if (m_Physics != null)
            m_Physics.AlignSpringsAndWheelsToTerrain();
		
		SetMaxInterpolateDistance(500f);
		
		CreateSpeaker();
        
        m_cIndicatorFXPositioner = (NetSpritePositioner)CreateObject("NetSpritePositioner");
        m_cIndicatorFXPositioner.ActivateOnModule();
        AttachChild( m_cIndicatorFXPositioner );
        UpdateIndicatorFXPositioner();
    }

   
    /**metoda mowi czy w pojezdzie jest bron, z ktorej moze strzelac player, jesli nie, to
     * na hudzie nie pojawi sie celownik po wejsciu do pojazdu*/
    public boolean IsArmed()
    {
        return false;
    }
    
    public void SetState(String sStateName)
    {
        // stan crashed state jest stanem bez wyjscia
        if (CurrentState instanceof CrashedState)
            return;
        super.SetState(sStateName);
    }
    
	/** wstawienie kierowcy na samym poczatku levelu */
	public void TryToAttachDriver()
	{
		ChromeActor driver = GetFirstOccupant(EPassengerRole._VEHICLE_DRIVER);
		if (driver != null)
		{
			if (driver instanceof Player)
			{
				Pawn player = (Pawn) driver;
				player.cSeatInfo.Occupy(null);
				Execute(player); // aby ktos mogl wsiasc, cDriver musi byc = null
			}
		}
	}
	
	void CreatePhysics()
	{
		m_Physics = new PhysicsVehicle(this);
        m_Physics.Init();
	}

    /**
     * przepisanie parametrow pojazdu do fizyki,
     * przyda sie to w sieci, gdy na niektorych hostach nie bedzie
     * na biezaco aktyalizowana pelna fizyka, w momencie gdy bedziemy
     * chcieli zaczac jej uzywac, trzeba ja zaincjowac (pobrac z pojazdu
     * pozycje, normalna, itp...) */
    void InitPhysicsParamsFromVehicle()
	{
        //if ( m_bModuleNet )
        {
            if (m_Physics == null)
            {
                LogD(" m_Physics == null!!!!!!!!!!!\n");
                return;
            }
        
            m_Physics.InitParamsFromVehicle();
        }
    }
    
    
    public Vector GetSpeed()
    {
        return new Vector(m_Physics.m_vMoveDir);
    }
    
    void SetPlayEngineStartStopSound(boolean bPlayEngineStartStopSound)
    {
        m_bPlayEngineStartStopSound = bPlayEngineStartStopSound;
    }
    
	public void SetVehicleForMovie(boolean bIsInMovie, boolean bPlayEngineStartStopSound)
	{
        SetPlayEngineStartStopSound(bPlayEngineStartStopSound);
        
        m_bVehicleInMovie = bIsInMovie;
        
		if (bIsInMovie)
			SetState("MovieSimulateState");
		else
        {
			if (m_bPhysicsActive)
				SetPhysicsVehicleActive(false);
			SetState("StillState");
        }
        
        SetPlayEngineStartStopSound(true); // w innym wypadku niz film zawsze ma odgrywac dzwiek
	}
    
    public void OnFrame()
    {
        GlobalOnFrame();
    }
    
    Vector m_vLastFramePos = new Vector();
    
    Vector m_vGOFPos = new Vector();
    Vector m_vGOFFd = new Vector();
    
    /** ten onframe powinien byc wolany z OnFrame w kazdym stanie
     */
    public void GlobalOnFrame()
    {
        /*GetPosition( sm_vTmpPos );
        if ( m_vLastFramePos.Distance2(sm_vTmpPos) > 0.0f )
        {
            NetLog.Vehicle(""+GetNetInfo()+" Pos changed "+m_vLastFramePos+" -> "+sm_vTmpPos+"\n");
            m_vLastFramePos.Set( sm_vTmpPos );
        }*/
        
        if(!m_bModuleNet)
        {
            GetPosition( m_vGOFPos );
			GetForwardVector( m_vGOFFd );
			UpdateSpeakerPosDir(m_vGOFPos, m_vGOFFd);
        }
    }

///////////////////////////////////////////////////////////////////////////////
// wsiadanie / wysiadanie
///////////////////////////////////////////////////////////////////////////////
	/** dodanie informacji o nowym siedzeniu */
	public void AddSeatInfo(SeatInfo info)
	{
		if (info != null)
			aSeatsInfo.add(info);
	}
	
	/** usuniecie informacji o wszystkich siedzeniach */
	public void DeleteAllSeatsInfos()
	{
		aSeatsInfo.clear();
	}
	
	/** pobranie informacji o siedzeniu danego typu */
	public SeatInfo GetFirstFreeSeat(int nPassengerRole)
	{
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if ( ((info.GetRole() == nPassengerRole) || (nPassengerRole == EPassengerRole._VEHICLE_ANY )) &&
				  (!info.IsOccupied()) &&
				 (info.GetReserver() == null) )
				return info;
		}
		return null;
	}
	
	int GetFreeSeatsNum()
	{
		int nNum = 0;
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if ( (info.GetOccupant() == null) && (info.GetReserver() == null) )
				nNum++;
		}
		return nNum;
	}
    
    int GetSeatIndex(SeatInfo cSeat)
    {
        return aSeatsInfo.indexOf( cSeat );
    }
    
    SeatInfo GetSeatByIndex(int nIndex)
    {
        if ( nIndex >= 0 && nIndex < aSeatsInfo.size() )
            return (SeatInfo)aSeatsInfo.get( nIndex );
        return null;
    }
    

	public SeatInfo [] GetSeats()
	{
		int nNum = aSeatsInfo.size();
		if (nNum == 0)
			return null;
		
		SeatInfo tab[] = new SeatInfo[nNum];
		for (int i = 0; i < aSeatsInfo.size(); i++)
			tab[i] = (SeatInfo) aSeatsInfo.get(i);
		return tab;
	}
	
	public SeatInfo [] GetFreeSeats()
	{
		int nNum = GetFreeSeatsNum();
		if (nNum == 0)
			return null;
		
		SeatInfo tab[] = new SeatInfo[nNum];
		int nNdx = 0;
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if ( (info.GetOccupant() == null) && (info.GetReserver() == null) )
			{
				tab[nNdx] = info;
				nNdx++;
			}
		}
		return tab;
	}
	
	/** pobranie pierwszego obiektu zajmujacego miejsce w roli nPassengerRole */
	public Pawn GetFirstOccupant(int nPassengerRole)
	{
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if ( (info.GetRole() == nPassengerRole) && (info.IsOccupied()) )
				return info.GetOccupant();
		}
		return null;
	}
	
	/** zwraca ilosc obiektow zajmujacych miejsca w podanej roli */
	int GetOccupantsNum(int nPassengerRole)
	{
		int nNum = 0;
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if ( (info.GetRole() == nPassengerRole) && (info.IsOccupied()) )
				nNum++;
		}
		return nNum;
	}
	
	/** zwraca ilosc wszystkich obiektow zajmujacych jakies miejsca */
	int GetAllOccupantsNum()
	{
		int nNum = 0;
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if (info.IsOccupied())
				nNum++;
		}
		return nNum;
	}
    
    int GetAllAliveOccupantsNum()
    {
		int nNum = 0;
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if ( (info.IsOccupied()) && (!info.GetOccupant().bDead) )
				nNum++;
		}
		return nNum;
    }
	
	/** zwraca wszystkie obiekty, ktore zajmuja miejsca w danej roli */
	public Pawn [] GetOccupants(int nPassengerRole)
	{
		int nOccupants = GetOccupantsNum(nPassengerRole);
		if (nOccupants == 0)
			return null;
		Pawn []tab = new Pawn[nOccupants];
		int nNdx = 0;
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if ( (info.GetRole() == nPassengerRole) && (info.IsOccupied()) )
			{
				tab[nNdx] = info.GetOccupant();
				nNdx++;
			}
		}
		return tab;
	}
	
	/** zwraca wszystkie obiekty, ktore zajmuja miejsca */
	public Pawn [] GetAllOccupants()
	{
		int nOccupants = GetAllOccupantsNum();
		if (nOccupants == 0)
			return null;
		
		Pawn []tab = new Pawn[nOccupants];
		int nNdx = 0;
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if (info.IsOccupied())
			{
				tab[nNdx] = info.GetOccupant();
				nNdx++;
			}
		}
		return tab;
	}
	
	/** czy w pojezdzie znajduje sie aktor */
	public SeatInfo GetSeatOccupiedBy(Pawn actor)
	{
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if (info.GetOccupant() == actor)
				return info;
		}
		return null;
	}
	
	public SeatInfo GetSeatReservedBy(Pawn actor)
	{
		for (int i = 0; i < aSeatsInfo.size(); i++)
		{
			SeatInfo info = (SeatInfo) aSeatsInfo.get(i);
			if (info.GetReserver() == actor)
				return info;
		}
		return null;
	}
	
    float _CORRECTION = 150f;
    /** Matoda zwraca miejsce kolo pojazdu gdzie moze obiekt wysiasc */ 
    public Vector GetCheckedPosToGetOut(SeatInfo info)
    {
		if (info.GetOccupant() == null)
			return info.GetStartPos();

		int iFlags = GetFlags();
        ClearFlag(EFlags._BLOCK_MOVE);

        // najpierw probujemy wysiasc do miejsca startowego
        Vector vPos = CanGetOutToPos(info, info.GetSeatPos(), info.GetStartPos());
        
        if (vPos != null)
        {
            SetFlags(iFlags);
            return vPos;
        }
        
        // probujemy wysiasc po ekstentach
        Vector vExtents[] = { new Vector(), new Vector(), new Vector(), new Vector() };
        GetMeshExtents(vExtents[0], vExtents[1]);
        
        vExtents[2].Set(vExtents[0]);
        vExtents[2].fZ = (vExtents[1].fZ + vExtents[0].fZ) / 2f;
        vExtents[2].fX -= _CORRECTION;
        
        vExtents[3].Set(vExtents[1]);
        vExtents[3].fZ = vExtents[2].fZ;
        vExtents[3].fX += _CORRECTION;
        
        vExtents[0].fX = (vExtents[3].fX + vExtents[2].fX) / 2f;
        vExtents[0].fZ -= _CORRECTION;
        
        vExtents[1].fX = vExtents[0].fX;
        vExtents[1].fZ += _CORRECTION;
        
        for (int i = 0; i < vExtents.length; i++)
        {
            vExtents[i] = PointLocalToWorld(vExtents[i]);
            vExtents[i].fY = info.GetSeatPos().fY;
        }
            
        for (int i = 0; i < vExtents.length; i++)
        {
            vPos = CanGetOutToPos(info, info.GetSeatPos(), vExtents[i]);
            if (vPos != null)
            {
                SetFlags(iFlags);
                return vPos;
            }
        }
        
        SetFlags(iFlags);
        
        Vector vMin = new Vector();
        Vector vMax = new Vector();
        GetMeshExtentsInWorld(vMin, vMax);
        vPos = GetPosition();
        vPos.fY += Math.abs(vMax.fY - vMin.fY) + 10f;
        return vPos;
    }
    
    float _GET_OUT_TOLERANCE = 0.2f;
    float m_fDownTestVectorLen = -300f;
    
    Vector CanGetOutToPos(SeatInfo info, Vector vStartPos, Vector vDestPos)
    {
        if (GetTerrainType(vDestPos.fX, vDestPos.fZ) == 0)
            return null;
        
        Vector vMove = new Vector(vDestPos);
        vMove.Sub(vStartPos);
        vMove.fY = 0;
        float fMoveLen = vMove.Len();
        
        // stara pozycja occupanta
        Vector vOldOccupantPos = info.GetOccupant().GetPosition();
        info.GetOccupant().SetPositionWorld(vStartPos);
        
        info.GetOccupant().MoveByEllipsoid(vMove);
        if (vMove.Len() < fMoveLen * (1f - _GET_OUT_TOLERANCE))
        {
            info.GetOccupant().SetPositionWorld(vOldOccupantPos);
            return null;
        }
        
        Vector vStart = new Vector(vStartPos);
        vStart.Add(vMove);
        
        vMove = new Vector(0f, m_fDownTestVectorLen, 0f);
        info.GetOccupant().SetPositionWorld(vStart);
        if (info.GetOccupant().MoveByEllipsoid(vMove))
        {
            info.GetOccupant().SetPositionWorld(vOldOccupantPos);
            return null; // nie mozna obiektu postawic na ziemi
        }
        info.GetOccupant().SetPositionWorld(vOldOccupantPos);
       
        vStart.Add(vMove);
        return vStart;
    }
	
	Vector Collision(Vector vStart, Vector vEnd, MeshObject objExclude)
	{
		Vector vCollPoint = new Vector(vEnd);
		TraceObjectsExclude(vStart, vCollPoint, objExclude);
		TraceTerrain(vStart, vCollPoint);
		TraceTrees(vStart, vCollPoint);
		return vCollPoint;
	}
	
    /** Wejscie do pojazdu (siec) */
    void ActionExecute(Actor executor)
    {
        super.Execute(executor);

		if (!(executor instanceof Pawn))
		{
            CrashLog("[Vehicle.ActionExecute]: Vehicle can be executed only by Pawn\n");
			return;
		}

		Pawn pawn = (Pawn) executor;
		
		SeatInfo seat = GetCurrentPointedSeat(executor);
		if (seat == null)
			return;
        
        
        if ( m_bModuleNet )
        {
            //jesli nikt nie rezerwuje siedzenia, albo to my
            //wtedy probujemy wsiadac
            if ((seat.GetReserver() == null) ||
                (seat.GetReserver() == pawn) )
			    pawn.ActionGetIn(seat);
        }
        else
        {
            //dlaczego tu jest proba zarezerwowania siedzenia?!!!!!!
            //w sieci tego nie wolno robic, bo moze nam sie nie 
            //udac wsiasc i wtedy nikt nie usiadzie na siedzenie zarezerwowane
            //dla nas!!!!
            
            //dla sieci dorabiam osobna wersje (powyzej)
            //a to zostawiam po staremu bo moze ktos to wola w singlu
            pawn.PrepareToGetIn(seat);
        
		    if (seat.GetReserver() == pawn)
			    pawn.ActionGetIn(seat);
        }
    }
    
    /** Wejscie do pojazdu */
    public void Execute(Actor executor)
    {
        super.Execute(executor);

		if (!(executor instanceof Pawn))
		{
            CrashLog("[Vehicle.ActionExecute]: Vehicle can be executed only by Pawn\n");
			return;
		}
		
		Pawn pawn = (Pawn) executor;

		SeatInfo seat = GetCurrentPointedSeat(executor);
		if (seat == null)
		{
			CrashLog("Vehicle.Execute(): seat == null \n");
			return;
		}
		
		pawn.PrepareToGetIn(seat);
		if (seat.GetReserver() == executor)
			pawn.GetIn(seat);
		
		return;
    }
	
    /** Wejscie do pojazdu */
    public void Execute(Actor executor, int iSeatType)
    {
        super.Execute(executor);

		if (!(executor instanceof Pawn))
		{
            CrashLog("[Vehicle.ActionExecute]: Vehicle can be executed only by Pawn\n");
			return;
		}
		
		Pawn pawn = (Pawn) executor;

		SeatInfo seat = GetFirstFreeSeat(iSeatType);
		if (seat == null)
		{
			CrashLog("Vehicle.Execute(): seat == null \n");
			return;
		}
		
		pawn.PrepareToGetIn(seat);
		if (seat.GetReserver() == executor)
			pawn.GetIn(seat);
		
		return;
    }
	
    public void StartExecution(Actor executor)
    {
        if (!bExtendedExecutionMode)
        {
            Execute(executor);
            return;
        }

		Pawn pawn = null;
		
		if (executor instanceof Pawn)
		{
			pawn = (Pawn) executor;
		}
		else
			return;

		if (pawn.GetSeat().GetOccupant() == null)
		{
			if (pawn.GetSeat().GetReserver() == executor)
			{
				// wsiadamy - tworzymy dynamicznie tablice akcji
				Vector vLocalStartPos = pawn.GetSeat().GetStartLocalPos();
				Vector vLocalStartDir = pawn.GetSeat().GetStartLocalDir();
				Vector vLocalEndPos = pawn.GetSeat().GetSeatLocalPos();
				Vector vLocalEndDir = pawn.GetSeat().GetSeatLocalDir();

				aExecutionSteps = new ExecutionInfo[]
									 {
									 new ExecutionInfo(EnumExecuteMode._MOVE_TO, _IEM_EXECUTE, 
									 vLocalStartPos, vLocalStartDir),
									 new ExecutionInfo(EnumExecuteMode._GET_IN, _IEM_NONE,
									 vLocalEndPos, vLocalEndDir),
									 };
				InitExecutionActions();
			}
		}
		else
		if (pawn.GetSeat().GetOccupant() == executor)
		{
            // wysiadamy
			Vector vLocalStartPos = pawn.GetSeat().GetStartLocalPos();
			Vector vLocalStartDir = pawn.GetSeat().GetStartLocalDir();
			aExecutionSteps = new ExecutionInfo[]
								 {
								 new ExecutionInfo(EnumExecuteMode._GET_OUT, _IEM_NONE, 
								 vLocalStartPos, vLocalStartDir),
								 };
			InitExecutionActions();
		}
		
        ExecutionInfo currentExecutionInfo = null;
        if (aExecutionSteps != null && aExecutionSteps.length > 0)
            currentExecutionInfo = GetFirstValidExecutionInfo(aExecutionSteps[0]);
        executor.TriggerObjectNotification(this, currentExecutionInfo);
    }
	
    /** Aktywny jesli nie ma kierowcy lub pasazera i nie jest rozwalony */
    public boolean IsActive(ControlObject Activator)
    {
		if (!(Activator instanceof Pawn))
			return false;
		if (GetSeatOccupiedBy((Pawn) Activator) != null)
			return false;
		if ( (GetFirstFreeSeat(EPassengerRole._VEHICLE_ANY) == null) || (fHealth <= 0) || (!m_bCanDrive) )
			return false;
		if (!bActiveTrigger)
			return false;
		
        SeatInfo seat = GetCurrentPointedSeat(Activator);
		if ( (seat != null) && (seat.GetReserver() == null) )
			return true;

		return false;
    }
	
	SeatInfo GetCurrentPointedSeat(ControlObject Activator)
	{
		if (!(Activator instanceof Pawn))
			return null;
		
		Pawn pawn = (Pawn) Activator;
		Vector vActivatorPos = Activator.GetPositionVectorVolatile();
	
		SeatInfo reservedSeat = GetSeatReservedBy(pawn);
		if (reservedSeat != null)
		{
			if (vActivatorPos.Distance(reservedSeat.GetStartPos()) < reservedSeat.GetMoveInRadius())
				return reservedSeat;
		}
		
		SeatInfo [] seats = GetFreeSeats();
		if (seats == null)
			return null;
		
		for (int i = 0; i < seats.length; i++)
		{
			if (vActivatorPos.Distance(seats[i].GetStartPos()) < seats[i].GetMoveInRadius())
				return seats[i];
		}
		return null;
	}

    protected void SetCanGetOut(boolean bCanGetOut)
    {
		for (int i = 0; i < aSeatsInfo.size(); i++)
			((SeatInfo)aSeatsInfo.get(i)).SetCanGetOut(bCanGetOut);
        
    }
	
	/** komunikat wysylany przez pawnera informujacy, ze wsiadl do pojazdu */
	public void PawnGotIn(SeatInfo info, Pawn pawn)
	{
        if (pawn.GetConflictSide() != EnumConflictSide._NEUTRAL)
		    ChangeConflictSide(pawn.GetConflictSide()); // dla pierwszego wsiadajacego zmieniamy conflict side
		
		if (info.GetRole() == EPassengerRole._VEHICLE_DRIVER)
			InvokeActionChain(aOnGotInDriver);
		else
		if (info.GetRole() == EPassengerRole._VEHICLE_PASSENGER)
			InvokeActionChain(aOnGotInPassenger);
        
        Pawn driver = GetFirstOccupant( EPassengerRole._VEHICLE_DRIVER );
        if (info.GetRole() == EPassengerRole._VEHICLE_DRIVER)
        {
            m_bLocalPlayerDriver = (driver != null && driver.bNetIsOwner);
            //na serwerze "odsluchujemy" replikacje, tylko jak w pojezdzic
            //jest jakis gracz-klient
            if ( Role == ERole._AUTHORITY )
                EnableIncomingReplication( ! m_bLocalPlayerDriver );
        }
        pawn.ClearFlag(EFlags._CAST_SHADOWS);
        
        if (Role == ERole._AUTHORITY) 
            RemoveVehicleFromAutoHide();
	}
    
    
	
	/** komunikat wysylany przez pawnera informujacy, ze wysiadl on z pojazdu */
	public void PawnGotOut(SeatInfo info, Pawn pawn)
	{
		if (GetAllAliveOccupantsNum() == 0)
			ChangeConflictSide(EnumConflictSide._NEUTRAL); // dla pierwszego wsiadajacego zmieniamy conflict side
		
        InvokeActionChain((info.GetRole() == EPassengerRole._VEHICLE_DRIVER) ?
						  aOnGetOutDriver : aOnGetOutPassenger);
        
		if (info.GetRole() == EPassengerRole._VEHICLE_DRIVER)
        {
			StopSounds();
            m_bLocalPlayerDriver = false;
            if ( Role == ERole._AUTHORITY )
                EnableIncomingReplication( false );
        }
        
        pawn.SetFlag(EFlags._CAST_SHADOWS);
        
        if ( (Role == ERole._AUTHORITY) && (GetAllOccupantsNum()<=0))
            CheckAddVehicleToAutoHide();
	}
    
	
	/** zeby nie mozna bylo wysiasc z pojazdu */
    public void Enable()
    {
        super.Enable();
		
        SetCanGetOut(true);
    }
    
	/** zeby nie mozna bylo wysiasc z pojazdu */
    public void Disable()
    {
        super.Disable();
		
        SetCanGetOut(false);
    }

    String GetTip()
    {
        if (GetSeats().length > 1)
        {
            Player cPlayer = ((ChromeModule)GetModule()).GetMainPlayer();
            if (cPlayer != null)
            {
                SeatInfo seat = GetCurrentPointedSeat(cPlayer);
                if (seat != null )
                    return seat.m_sTip;
            }
        }
        return "&GET_IN_VEHICLE&";
    }
    
    String GetAdditionalTip()
    {
        String aPassengers = "";
        if (GetModule() instanceof ModuleNet)
        {
            //bierzemy kolejno: kierowce, pasazera z dzialkiem, pozostalych pasazerow
            for ( int nPassengerRole =  EPassengerRole._MIN; 
                  nPassengerRole <= EPassengerRole._MAX;
                  ++  nPassengerRole )
            {
                //pobieramy wszystkich o podanej roli
                Pawn[] aPawns = GetOccupants( nPassengerRole );
                
                //jesli cos znalezlismy to pobieramy ich nicki i dodajemy do wyniku
                if ( aPawns != null && aPawns.length > 0)
                {
                    aPassengers += ("  " + EPassengerRole.GetRoleName(nPassengerRole) + "\n");
                    for ( int i = 0; i < aPawns.length; ++i )
                    {
                        if ( aPawns[ i ] != null )
                        {
                            //poszczegolnych pasazerow oddzielam nowa linia
                            aPassengers = (aPassengers + "      " + aPawns[ i ].GetName() + "\n");
                        }
                    }
                }
            }
        }
        return aPassengers;
    }
	
///////////////////////////////////////////////////////////////////////////////
// rozwalka
///////////////////////////////////////////////////////////////////////////////
    void VehicleCrashed()
    {
        SeatInfo seats[] = GetSeats();
        if (seats != null)
        {
            for (int i = 0; i < seats.length; i++)
                seats[i].SetCanGetOut(true);
        }
                
		Pawn [] tab = GetAllOccupants();
		if (tab != null)
		{
			for (int i = 0; i < tab.length; i++)
				tab[i].VehicleCrashed();
		}
		
        StopFXSmokeUpdate();
        
        SetState("CrashedState");
        SetCorrectPosOnRender( false );
        bActiveTrigger  = false;
        bIsActiveTarget = false;
        m_bCanDrive     = false;
        m_nState        = _CRASHED;
        if (Role == ERole._AUTHORITY)
        {
            CheckAddVehicleToAutoRespawn();
            MsgSendClientCrash();
        }
    }
    
    /** Rozwalka samochodu */
    public void Die()
    {
        if (Role < ERole._AUTHORITY)
            return;
        
        super.Die();

        VehicleCrashed();
        
        //uwaga! w VehicleCrashed() wszyscy opuszczaja
        //pojazd, zatem to nie ma prawa sie odegrac!
		Pawn [] occupants = GetAllOccupants();
		if (occupants != null)
		{
			// na serwerze mozemy zdecydowac czy chcemy rozwalic tez kierowce czy nie
			for (int i = 0; i < occupants.length; i++)
				occupants[i].Die();
		}
    }

	void TakeDamage(float fDamage, int nDamageType)
	{
		OnTakeDamage(fDamage, fDamage, null, null, null, 0, nDamageType);
	}
	
    private
    PlayerNet m_cPlayerNetAssassin = null;
    PlayerNet GetPlayerNetAssassin(){ return m_cPlayerNetAssassin; }
   
    // poprawione OnTakeDamage //////////////////////////////////////////////////////////////
    void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
    {
        if ( Role < ERole._AUTHORITY )
            return;

        // powiadom przeciwnika, ze trafil
        if (Assassin instanceof ChromeActor)
            ((ChromeActor)Assassin).NotifyHit(this, nDamageType);

        if (bDead)
            return;
        
        if ( m_bModuleNet )
            m_cPlayerNetAssassin = ModuleNet.GetAssassinPlayerNetFromObject( Assassin );
        
       
        // odbieramy energie normalnie
        Damage( fDamage, nDamageType );
        float fOccupantsDamage = m_fOccupantsDamageMul * fDamage;
        OccupantsDamage( fOccupantsDamage, Assassin );
        
        CheckCanDrive();
    }
   
   void OccupantsDamage( float fDamage, GameObject Assassin )
   {
        Pawn [] occupants = GetAllOccupants();
		if (occupants != null)
		{
			for (int i = 0; i < occupants.length; i++)
			{
				if (occupants[i] instanceof Player)
			        occupants[i].TakeDamage( fDamage, fDamage, 
                                             Assassin, null, null, 
                                             EBones._NON_BONE, EnumDamageType._BULLET);
			}
		}
   }
    
    void CheckCanDrive()
    {
		Pawn []occupants = GetAllOccupants();
		boolean bPlayerInside = false;
		if (occupants != null)
		{
			for (int i = 0; i < occupants.length; i++)
			{
				if (occupants[i] instanceof Player)
				{
					bPlayerInside = true;
					break;
				}
			}
		}
        float fMinHealthCoeff = m_bModuleNet ?
                                fMinNetHealthCoeffToDrive:
                                fMinHealthCoeffToDrive;
        
		if ( bPlayerInside || m_bModuleNet )
		{
			// wysadzamy playera, zanim dojdzie do wybuchu
			if ( m_bCanDrive && 
				 fHealth > 0 && 
				 m_Physics != null &&
				 fHealth < fMaxHealth * fMinHealthCoeff )
			{
				CannotDrive();
			}
		}
		else
		{
            if (fHealth <= fMaxHealth * fMinHealthCoeff)
                StartFXSmokeUpdate();
			if (m_bCanDrive && fHealth <= 0)
				CannotDrive();
		}
    }
	
	void CannotDrive()
	{
		m_bCanDrive = false;
		StartFXSmokeUpdate();
        TextMessageDestroyed();
        
        InvokeActionChain(aOnCantDrive);
        
        m_nState = _CANNOTDRIVE;
        if ( Role == ERole._AUTHORITY )
        {
		    Pawn []occupants = GetAllOccupants();
		    if (occupants != null)
		        for (int i = 0; i < occupants.length; i++)
			        occupants[i].ActionGetOut();
            
            CheckAddVehicleToAutoRespawn();
            MsgSendClientCannotDrive();
        }
	}
    
    void OnCollisionWithPlayer()
    {
        InvokeActionChain(aOnCollisionWithPlayer);
    }
    
    void StartFXSmokeUpdate()
    {
        StopFXSmokeUpdate();
        if (m_bSmokeSpawned || !bSpawnSmokeFX)
            return;
        m_bSmokeSpawned = true;
        CallMethodEvery(METHOD_UpdateSmokeFX, fFXSmokeUpdateTime);
    }
    
    void StopFXSmokeUpdate()
    {
        DisableCallMethod(METHOD_UpdateSmokeFX);
        m_bSmokeSpawned = false;
    }
    
    void UpdateSmokeFX()
    {
        ExecuteFX(sFXSmokeName, GetPosition());
    }
	
    void AlreadyCannotDrive()
	{
		TextMessageAlreadyDestroyed();
		
		Pawn []occupants = GetAllOccupants();
		if (occupants == null)
			return;

		for (int i = 0; i < occupants.length; i++)
			occupants[i].ActionGetOut();
	}
    
	void TextMessageDestroyed()
	{
		Pawn []occupants = GetAllOccupants();
		if (occupants == null)
			return;
		
		for (int i = 0; i < occupants.length; i++)
			occupants[i].TextMessage("&VEHICLE_DESTROYED&");
	}
	
	void TextMessageAlreadyDestroyed()
	{
		Pawn []occupants = GetAllOccupants();
		if (occupants == null)
			return;
		
		for (int i = 0; i < occupants.length; i++)
			occupants[i].TextMessage("&VEHICLE_ALREADY_DESTROYED&");
	}

///////////////////////////////////////////////////////////////////////////////
// implementacja interfejsu Appear
///////////////////////////////////////////////////////////////////////////////
	boolean m_bIsGridIndependent = false;
    boolean bOnAppearShowWreckOnly = false; // czy w momencie appear pokazywac tylko wrak
    
    public void Appear(ControlObject cDestinationObj)
	{
        if (bOnAppearShowWreckOnly)
        {
            // przed deaktywacja bylo widac tylko wrak, wiec przywracamy tylko wrak
            ShowWreck();
            bOnAppearShowWreckOnly = false;
            StartFXSmokeUpdate();
        }
        else
        {
            super.Appear(cDestinationObj);
        
            boolean bWasPhysicsActiveBeforeDisappear = false;
        
            // sprawdzamy czy gracz siedzi w samochodziku
            if(GetFirstOccupant(EPassengerRole._VEHICLE_DRIVER) instanceof Player)
                bWasPhysicsActiveBeforeDisappear = true;
        
            ActivateVehicle(true, m_bIsGridIndependent, bWasPhysicsActiveBeforeDisappear);
        }
	}
    
    public void Disappear(ControlObject cDestinationObj)
	{
        if (m_bWreckVisible)
        {
            // jezeli widac wrak, to nie widac pojazdu, ale na disappear wrak tez musimy schowac
            HideWreck();
            bOnAppearShowWreckOnly = true;
        }
        else
        {
		    super.Disappear(cDestinationObj);
		
		    m_bIsGridIndependent = IsGridIndependent();
		    ActivateVehicle(false, m_bIsGridIndependent, false);
        }
	}
	
///////////////////////////////////////////////////////////////////////////////
// symulacja
///////////////////////////////////////////////////////////////////////////////
    public void SetAllFramePeriod(float fAllFrameTime)
    {
        m_fAllFrameTime = fAllFrameTime;
    }
    
	public float GetNextTimePeriod()
	{
		m_fAllFrameTime -= PHYSICS_GRAIN;
			
		if (m_fAllFrameTime >= 0f)
			return PHYSICS_GRAIN;
		if (m_fAllFrameTime > -PHYSICS_GRAIN)
			return (m_fAllFrameTime + PHYSICS_GRAIN);
		return 0f;
	}
	
	public void ActivateVehicle(boolean bActivate, boolean bGridIndependent, boolean bActivatePhysics)
	{
		if (bActivate)
		{
            if (m_iStampSize > 0)
            {
                if (!IsUpdatingPathMap())
                    UpdatePathMap(true, m_iStampSize, true);
            }

            if (bGridIndependent)
				ActivateOnModuleGridIndependent();
			else
				ActivateOnModule();
			
            if (m_bSmokeSpawned)
                UnfreezeCallMethod(METHOD_UpdateSmokeFX);

            // pobieramy nowy "stary" czas symulacji
            fLastT = GetTime();
		}
		else
        {
            if (m_iStampSize > 0)
            {
                if (IsUpdatingPathMap())
                    UpdatePathMap(false, m_iStampSize, true);
            }

            if (m_bSmokeSpawned)
                FreezeCallMethod(METHOD_UpdateSmokeFX);
            
            DeactivateOnModuleAlways();
        }

        if (bActivate && bActivatePhysics)
    		SetPhysicsVehicleActive(bActivate);
        else
    		SetPhysicsVehicleActive(false);
	}
    
	void SetPhysicsVehicleActive(boolean bActive)
	{
		m_bPhysicsActive = bActive;
		
		if (m_bPhysicsActive)
        {
			StartSounds();
            InitPhysicsParamsFromVehicle();
        }
		else
			StopSounds();
	}
    
    java.util.ArrayList m_TempObjects = new java.util.ArrayList();
    boolean m_BoolTab[] = new boolean[5];

    public void TurnOffPassengersCollisions()
    {
        ControlObject children[] = GetChildren();
        
        m_TempObjects.clear();
        
        if (children == null)
            return;
        
        if (children.length > m_BoolTab.length)
        {
            m_BoolTab = null;
            m_BoolTab = new boolean[children.length];
        }
        
        for (int i = 0; i < children.length; i++)
        {
            m_BoolTab[i] = children[i].IsSetFlag(EFlags._COLLISION);
            m_TempObjects.add(children[i]);
            if (m_BoolTab[i])
                children[i].ClearFlag(EFlags._COLLISION);
        }
    }
    
    public void TurnOnPassengersCollisions()
    {
        for (int i = 0; i < m_TempObjects.size(); i++)
        {
            if (m_BoolTab[i])
                ((ControlObject) m_TempObjects.get(i)).SetFlag(EFlags._COLLISION);
        }
    }

    

    float m_fTestGranularity = 0.0f;
    
	/** ogolna symulacja */
    public void Simulate(boolean bDriver)
    {
		if (!m_bSimulate)
			return;
		
        byte iAcc = motion.iAcc;
        float fTurn = motion.fTurn;
                
		if (!m_bCanDrive)
		{
            iAcc = 0;
			fTurn = 0;
		}
        else
		if (!m_bEngineOn)
            iAcc = 0;
        
        float fCurrentT = GetTime();
        float fdT = fCurrentT - fLastT;
        fLastT    = fCurrentT;
                
        TurnOffPassengersCollisions();
        if (m_bModuleNet)
        {
            float fGranularity = GetModule().GetPhUpdateGranularity();
            
            //TODO: fixit
            if ( m_fTestGranularity > 0.0f )
                fGranularity = m_fTestGranularity;
            
            // jezdzimy na module sieciowym
            if ( bDriver && m_bLocalPlayerDriver )
            {
                // pojazd ma kierowce i ja jestem tym kierowca
                
                // zatem dokonajmy symulacji
				while ( fdT >= fGranularity )
                {
					m_Physics.Simulate( iAcc, fTurn, fGranularity );
                    fdT -= fGranularity;
                }
				
                // i pobieramy z fizyki parametry do zreplikowania innym
                m_fCurrDirAngle = m_Physics.m_fCurrDirAngle;
                m_fCurrSpeed    = m_Physics.m_fCurrSpeed;
                m_iAcc          = iAcc;
                m_fTurn         = fTurn;
				m_fRealSpeed	= m_Physics.m_fRealSpeed;
				m_fReplicationTime	= GetTime();
				m_fAngleSpeed	= m_Physics.m_fAngleSpeed;
				m_fRealAccel	= m_Physics.m_fRealAccel;
            }
            else if ( bDriver == false && Role == ERole._AUTHORITY )
            {
                // kierowcy juz nie ma (moze wysiadl?) ale ja jestem serwerem
                
                // wiec musze doprowadzic do zatrzymania pojazdu 
                while ( fdT >= fGranularity )
                {
                    m_Physics.Simulate( (byte)0, 0f, fGranularity );
                    fdT -= fGranularity;
                }
                                
                // i przekazac odpowiednie informacje innym
                m_fCurrDirAngle = m_Physics.m_fCurrDirAngle;
                m_fCurrSpeed    = m_Physics.m_fCurrSpeed;
                m_iAcc          = iAcc;
                m_fTurn         = fTurn;
				m_fRealSpeed	= m_Physics.m_fRealSpeed;
				m_fReplicationTime	= GetTime();
				m_fAngleSpeed	= m_Physics.m_fAngleSpeed;
				m_fRealAccel	= m_Physics.m_fRealAccel;
            }
            else
            {
                // albo kierowca jest, ale nie u mnie,
                // albo go nie ma a ja jestem klientem
                
                // poprostu interpretuje przychodzace dane
                m_Physics.m_fCurrDirAngle = m_fCurrDirAngle;
                m_Physics.m_fCurrSpeed    = m_fCurrSpeed; 
				m_Physics.m_fRealSpeed	  = m_fRealSpeed;
				m_Physics.m_fRealAccel    = m_fRealAccel;
				m_Physics.m_fAngleSpeed	  = m_fAngleSpeed;
                // i na ich podstawie robie symulacje
                while ( fdT >= fGranularity )
                {
                    m_Physics.NetSimulate( fdT, Role == ERole._AUTHORITY );
                    fdT -= fGranularity;
                }
            }
            
            fLastT -= fdT;
            SetPhysicsLastUpdateTime( fLastT );
        }
        else
        {
			SetAllFramePeriod(fdT);
			float fTimePeriod = GetNextTimePeriod();
			while (fTimePeriod > 0f)
			{
				m_Physics.Simulate( iAcc, fTurn, fTimePeriod );
				fTimePeriod = GetNextTimePeriod();
			}
        }
        TurnOnPassengersCollisions();
    }
    
    public float fMinMotionNoiseRadius = 400;
    public float fMaxMotionNoiseRadius = 3000;
    public float fPredictionNoiseSecs = 2;
    public float fMinMotionNoiseTime = 0.2f;

    
    private float m_fLastVehicleNoiseTime = 0;
    private Vector m_vNoisePos = new Vector();

    // tests only
    private LaserPointer[] m_aVis = null;
    private int _VIS_STEPS = 100;
    boolean bNoiseVis = false;
    
    void MakeVehicleNoise()
    {
        float fSpeed = m_Physics.m_fCurrSpeed;
        if (fSpeed > 1)
        {
            float fSpeedCoef = Tools.InterpLinear(0, m_Physics.GetMaxSpeed(), fSpeed);
            float fUpdateTime = fPredictionNoiseSecs * 0.5f -
                                Math.max(0, 1 - fSpeedCoef) * 
                                (fPredictionNoiseSecs * 0.5f - fMinMotionNoiseTime);

            float fTime = GetTime();
            if (fTime - m_fLastVehicleNoiseTime > fUpdateTime)
            {
                float fRadius = fSpeed * fPredictionNoiseSecs;
                if (fRadius < fMinMotionNoiseRadius)
                    fRadius = fMinMotionNoiseRadius;
                else if (fRadius > fMaxMotionNoiseRadius)
                        fRadius = fMaxMotionNoiseRadius;
                m_vNoisePos.Set(GetForwardVectorVolatile());
                m_vNoisePos.Mul(fRadius);
                m_vNoisePos.Add(GetPositionVectorVolatile());
                
                m_fLastVehicleNoiseTime = fTime;
                MakeNoiseOnPosition(m_vNoisePos, fRadius, Pawn._NOISE_DANGEROUS_OBJECT_NEARBY);
                
                if (bNoiseVis)
                {
                    if (m_aVis == null)
                    {
                        m_aVis = new LaserPointer[_VIS_STEPS];
                        for (int i=0; i<_VIS_STEPS; i++)
                        {
                            m_aVis[i] = Tools.CreateLaserPointer(GetModule(), LaserPointer._BEAM_RED);
                            m_aVis[i].SetSize(3);
                        }
                    }
                    if (m_aVis != null)
                    {
                        Vector vVisStart = new Vector(m_vNoisePos);
                        Vector vVisEnd = new Vector();
                        vVisStart.fY += 70;
                        for (int i=0; i<_VIS_STEPS; i++)
                        {
                            float fAngle = 2.f * (float)Math.PI * (float)i / (float)_VIS_STEPS;
                            vVisEnd.fX = (float)Math.sin(fAngle) * fRadius + vVisStart.fX;
                            vVisEnd.fZ = (float)Math.cos(fAngle) * fRadius + vVisStart.fZ;
                            vVisEnd.fY = vVisStart.fY;
                            m_aVis[i].PointFromTo(vVisStart, vVisEnd);
                        }
                    }
                }
            }
        }
    }
    

	/** probujemy sparsowac to, co jest w polu tekstowym - zwracamy,
	 * czy obsluzylismy to czy nie */
	boolean TryToCallPointHandler()
	{
		String sText = m_VehiclePath.GetPoint(m_iCurrPatrolPoint).sTextField;
		if ( (sText == null) || (sText.equals("")) )
			return false;

        StringTokenizer cItemsIter = new StringTokenizer(sText, " ,", false);
		
		if (!cItemsIter.hasMoreTokens())
			return false;
		
		String s = cItemsIter.nextToken();
		float f1, f2;
		if ( !TextToUpperCase(s).equals("RANDOMWAIT") )
		{
			CallMethodIn(s, 0f);
			return true;
		}
			
		if (!cItemsIter.hasMoreTokens())
		{
			CrashLog("Bledna nazwa stringa dla punktu w sciezce:" +
					 m_VehiclePath.GetPathName() + "\n");
			return false;
		}
		
		try
		{
			f1 = Float.valueOf(cItemsIter.nextToken()).floatValue();
		}
		catch (java.lang.NumberFormatException e)
		{
			CrashLog("Bledny parametr dla stringa dla punktu w sciezce:" +
					 m_VehiclePath.GetPathName() + "\n");
			return false;
		}
		
		if (!cItemsIter.hasMoreTokens())
		{
			CrashLog("Bledny parametr dla stringa dla punktu w sciezce:" +
					 m_VehiclePath.GetPathName() + "\n");
			return false;
		}
		
		try
		{
			f2 = (float) Float.valueOf(cItemsIter.nextToken()).floatValue();
		}
		catch (java.lang.NumberFormatException e)
		{
			CrashLog("Bledny parametr dla stringa dla punktu w sciezce:" +
					 m_VehiclePath.GetPathName() + "\n");
			return false;
		}
		
		if (f2 < f1)
			return false;
			
		PatrolBreak();
		DisableCallMethod("PatrolResume");
		CallMethodIn("PatrolResume", GameObject.RandomFloat(f1, f2));
		return true;
	}
	
	void SlowDownOnCurve(float fDist, Vector vDir, Vector vDestDir)
	{
		float fDot = vDir.Dot(vDestDir);

		motion.iAcc = 1;
		m_Physics.m_Params._SLOW_COEF = m_fOldSlowDownValue;
		
		// jezeli kat jest wiekszy niz 30 stopni, zwalniamy, by sie nakierowac na punkt
		// (warunek dla duzej odleglosci)
		if ( (m_Physics.m_fCurrSpeed > 600) && (fDot < 0.866f) )
		{
			m_Physics.m_Params._SLOW_COEF = ((float)Math.abs(Math.acos(fDot)) * 57f / 90f) * _PATH_SLOW_VALUE;
			motion.iAcc = 0;
			return;
		}
		
		// warunek dla bliskiej odleglosci
		if ( (m_Physics.m_fCurrSpeed > 200f) &&
			 (fDist / m_Physics.m_fCurrSpeed <
			  Math.abs(Math.acos(fDot)) * 28.645f / m_Physics.m_Params._MAX_TURN_ANGLE) )
		{
			m_Physics.m_Params._SLOW_COEF = ((float)Math.abs(Math.acos(fDot)) * 57f / 90f) * _PATH_SLOW_VALUE;
			motion.iAcc = 0;
		}
	}
        
	/** do przeciazenia w klasach pochodnych - musi byc zawolane super z podklasy */
	void OnDestinationPos(int iDestPointNum)
	{
		// powiadamiamy kontroler, ze dojechalismy do kolejnego punktu
		if ( (m_bEditorAction) && (m_VehicleController != null) )
			m_VehicleController.Checkpoint();

		// powiadamiamy pozostalych pasazerow (AI), ze dojechalismy do nastepnego punktu
		Pawn [] occupants = GetAllOccupants();
		if (occupants != null)
		{
			for (int i = 0; i < occupants.length; i++)
			{
				if ( (occupants[i] instanceof NewAI) &&
                     (((NewAI) occupants[i]).GetController() != m_VehicleController) )
				{
					((NewAI) occupants[i]).GetController().Checkpoint();
				}
			}
		} 
		
		m_iCurrPatrolPoint = iDestPointNum + m_iAddValue;
		
		if (m_iCurrPatrolPoint == m_VehiclePath.GetPointsNum())
		{
			if (m_VehiclePath.IsPathLooped())
				m_iCurrPatrolPoint = 0;
			else
			if (m_VehiclePath.IsPathOneWay())
			{
				m_bDriveFinished = true;
				motion.Stop();
				RestoreVehicleParamsBeforePatrol();
			}
			else
			if (m_VehiclePath.IsPathTwoWays())
			{
				m_iAddValue = -1;
				m_iCurrPatrolPoint = m_VehiclePath.GetPointsNum() - 2;
				if (m_iCurrPatrolPoint < 0)
					m_iCurrPatrolPoint = 0;
			}
		}
		else
		if (m_iCurrPatrolPoint == 0)
		{
			m_iAddValue = 1;
			m_iCurrPatrolPoint = 1;
			if (m_VehiclePath.GetPointsNum() < 2)
				m_iCurrPatrolPoint = 0;
		}
	}
	
	/** wolane, kiedy pojazd zatrzyma sie po skonczeniu patrolowania -
	 * - do przeciazenia w klasach pochodnych */
	void OnDestinationPos()
	{
		// powiadamiamy kontroler, ze dojechalismy do ostatniego punktu
		if ( (m_bEditorAction) && (m_VehicleController != null) )
			m_VehicleController.MovingEnd();
		else
		{
			CrashLog("Vehicle.OnDestinationPos().m_bEditorAction:" + m_bEditorAction + "\n");
			CrashLog("Vehicle.OnDestinationPos().m_VehicleController:" + m_VehicleController + "\n");
		}
		
		// powiadamiamy pozostalych pasazerow (AI), ze dojechalismy do ostatniego punktu
		Pawn [] occupants = GetAllOccupants();
		if (occupants != null)
		{
			for (int i = 0; i < occupants.length; i++)
			{
				if ( (occupants[i] instanceof NewAI) &&
                     (((NewAI) occupants[i]).GetController() != m_VehicleController) )
				{
					((NewAI) occupants[i]).GetController().MovingEnd();
				}
			}
		} 
	}
	
	/** obsluga kolizji */
	public void OnCollisionFullInfoMove(ControlObject obj, Vector vPoint, Vector vNormal, Vector vMove)
	{
        //LogD("\n");
		if (m_bPhysicsActive)
        {
            m_cPlayerNetAssassin = null;
			m_Physics.HandleCollision(obj, vPoint, vNormal, vMove);
        }
	}
	
	/** umozliwianie cofania po pewnym czasie */
	void EnableBackLater(float fTime)
	{
		CallMethodIn("EnableBack", fTime);
	}
	
	void EnableBack()
	{
		DisableCallMethod("EnableBack");
		if (m_Physics != null)
			 m_Physics.EnableBack();
	}
	
///////////////////////////////////////////////////////////////////////////////
// stany
///////////////////////////////////////////////////////////////////////////////
	/** StillState - stoimy */
    public class StillState extends State
    {
        public void OnEnter()
        {
            if (m_bGameLoading)
            {
                m_bGameLoading = false;
                return;
            }
            
            nTeamID = 0;
            if ( m_bModuleNet ) 
                UnregisterKeyRequest();
        }
        
        public void OnFrame()
        {
            GlobalOnFrame();
        }
    }

    /** UNCARED DRIVE STATE - Jazda samopas. Moze w koncu stanac i przejsc do StillState */
    public class UncaredDriveState extends State
    {
        public void OnEnter()
        {
            if (m_bGameLoading)
            {
                m_bGameLoading = false;
                return;
            }
            
            LogLine("[Vehicle.UncaredDriveState]");

            nTeamID = 0;
            //zatrzymujemy sie - nikt nie przytrzymuje kierownicy ani gazu
            motion.Stop();
            
            // przepiszmy dla pewnosci parametry pojazdu do fizyki,
            // przyda sie to w sieci, do zmiany "opiekujacego" sie pojazdem
            InitPhysicsParamsFromVehicle();
        }

        public void OnFrame()
        {
            GlobalOnFrame();
			if ( (m_Physics != null) && (m_Physics.m_fCurrSpeed == 0f) )
                SetState("StillState");
            Simulate(false);
            MakeVehicleNoise();
        }
    }
    
    /** DrivenState - Kierowany przez Pawna */
    public class DrivenState extends State
    {
        public void OnEnter()
        {
            if (m_bGameLoading)
            {
                m_bGameLoading = false;
                return;
            }
            
			Pawn driver = GetFirstOccupant(EPassengerRole._VEHICLE_DRIVER);
            if (driver instanceof ChromeActor)
                nTeamID = driver.nTeamID;
            
            fLastT = m_bModuleNet ? GetModule().GetPhysicsLastUpdateTime() : GetTime();
            SetPhysicsLastUpdateTime( fLastT );
            SetPhysicsVehicleActive(true);            
            
            if (m_bModuleNet) 
            {
                if ( (driver != null) && (driver.bNetIsOwner) )
                    RegisterKeyRequest();
            }
        }
        
        public void OnFrame()
        {
            GlobalOnFrame();
            Simulate(true);
            MakeVehicleNoise();
        }
    }

    
        //mk: hhmm, a nie powinno to byc w Vehiclu, co by bylo bardziej uniwersalnie?
    void CreateWreck()
    {
        if (m_Wreck != null)
            return;
        if (m_sWreckMeshName == null)
            return;
        m_Wreck = (VehicleWreck) CreateObject("VehicleWreck");
        m_Wreck.m_cOwner = this;
		m_Wreck.LoadMesh( m_sWreckMeshName );
        m_Wreck.SetFlag(EFlags._BLOCK_MOVE | EFlags._BLOCK_TRACE |
                      EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH);
        m_Wreck.ClearFlag(EFlags._COLLISION);
        m_Wreck.ClearFlag(EFlags._BLOCK_PARTICLES);
        m_Wreck.SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        m_Wreck.SetCollHandler(ECollFlags.OBJCOL_HANDLER_NONE);
        m_Wreck.SetTraceCollType(ECollTraceFlags._HULL);
		m_Wreck.SetLighting(true);
        m_Wreck.SetFlag(EFlags._CAST_SHADOWS);
		m_Wreck.EnableRendering(false);
    }
    
    void ShowWreck()
    {
        if (m_Wreck == null)
            return;
        m_Wreck.ActivateOnModule();
		m_Wreck.SetFlag(EFlags._COLLISION);
        m_Wreck.EnableRendering(true);
        m_Wreck.FromUpForwardPos(Vector.Up, GetForwardVector(), GetPosition());
        m_bWreckVisible = true;
    }
    
    void ServerHideWreck()
    {
        HideWreck();
        
        if ( Role == ERole._AUTHORITY )
            MsgSendClientHideWreck();
    }
    void HideWreck()
    {
        if (m_Wreck == null)
            return;
        
        m_Wreck.DeactivateOnModuleAlways();
		m_Wreck.ClearFlag(EFlags._COLLISION);
        m_Wreck.EnableRendering(false);
        m_bWreckVisible = false;
        StopFXSmokeUpdate();
    }

    /** CrashedState - Rozwalka - normalnie wyjsc sie juz z tego stanu 
     * nie da, jedynie poprzez zawolanie Respawn()     
     **/
    public class CrashedState extends State
    {
        public void OnEnter()
        {
            if (m_bGameLoading)
            {
                m_bGameLoading = false;
                return;
            }
            
            SetPhysicsVehicleActive(false);
            nTeamID = 0;
            
            Vector vTemp = GetPosition();
            vTemp.Add(m_vPositionModifier);
            
            ExecuteFX(sBlowUpFX, vTemp, m_vFXDir);
            ExecuteParticleSystem(sBlowUpFX, vTemp, m_vFXParticlesDir);
            PlaySound(sBlowUpSound);

            if (fBlowUpShakeStrength > 0)
                ShakePlayersCamera(fBlowUpShakeStrength, fBlowUpShakeDistMin, fBlowUpShakeDistMax);
            
            CallMethodIn("DisableAllActivities", 0.2f);
        }
        
        public void DisableAllActivities()
        {
            EnableRendering(false);
            DeactivateOnModuleAlways();
            DisableCallMethods();
            CallMethodIn("DeleteAttachedFXes", 10f);
        }

		// poprawione OnTakeDamage //////////////////////////////////////////////////////////////
		void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
		{
		}
    }
	
///////////////////////////////////////////////////////////////////////////////
// symulacja jazdy po sciezce patrolowej
///////////////////////////////////////////////////////////////////////////////
    /** PatrolPathState - Jazda sciezka */
    public class PatrolPathState extends State
    {
        public void OnEnter()
        {
            if (m_bGameLoading)
            {
                m_bGameLoading = false;
                return;
            }
            
			m_bIsPatrolling = true;
			
            fLastT = GetTime();
			if (!m_bPhysicsActive)
	            SetPhysicsVehicleActive(true);
			m_iCurrPatrolPoint = 0;
			m_iAddValue = 1;
			m_bDriveFinished = false;
			motion.iAcc = 1;
        }
        
        public void OnFrame()
        {
            GlobalOnFrame();
            SimulatePathMove();
            MakeVehicleNoise();
        }
    }
	
	void SimulatePathMove()
	{
		if (!m_bSimulate)
			return;
		
        byte iAcc = motion.iAcc;
        float fTurn = motion.fTurn;
                
		if (!m_bCanDrive)
		{
            iAcc = 0;
			fTurn = 0f;
		}
        else
		if (!m_bEngineOn)
            iAcc = 0;
		else
		{
			float fCurrentT = GetTime();
			float fdT = fCurrentT - fLastT;
			fLastT = fCurrentT;
			
            if(fdT > 1.f) CrashLog("Czas updejtu fizyki wiekszy niz 1s\n");
            
			SetAllFramePeriod(fdT);
			float fTimePeriod = GetNextTimePeriod();
            TurnOffPassengersCollisions();
			while (fTimePeriod > 0f)
			{
				if (!m_bDriveFinished)
				{
					Vector vCurrDir = GetForwardVector();
					vCurrDir.fY = 0;
					vCurrDir.Normalize();
					Vector vCurrPos = GetPosition();
					Vector vToDestVect = new Vector(m_VehiclePath.GetPoint(m_iCurrPatrolPoint).vPosition);
					vToDestVect.Sub(vCurrPos);
					vToDestVect.fY = 0;
					float fDist = vToDestVect.Len();
					vToDestVect.Normalize();
			
					if (fDist < m_VehiclePath.GetPoint(m_iCurrPatrolPoint).fRadius)
					{
						TryToCallPointHandler();
						OnDestinationPos(m_iCurrPatrolPoint);
					}
			
					if (!m_bDriveFinished)
					{
						float fDot = vToDestVect.Dot(vCurrDir);
						fTurn = (float) java.lang.Math.acos(fDot) * RAD_TO_DEG / m_Physics.m_Params._MAX_TURN_ANGLE;

						if (fTurn > 1f)
							fTurn = 1f;
						
						if (vToDestVect.Cross(vCurrDir).fY < 0)
							fTurn *= -1f;
						
						if (m_bBreakPatrol)
						{
							if (m_Physics.m_fCurrSpeed > 0f)
								iAcc = -1;
							else
								iAcc = 0;
						}
						else
							SlowDownOnCurve(fDist, vCurrDir, vToDestVect);
					}
					else
					{
						iAcc = motion.iAcc;
						fTurn = motion.fTurn;
					}
				}
				
				m_Physics.Simulate( iAcc, fTurn, fTimePeriod );
				fTimePeriod = GetNextTimePeriod();
			}
            TurnOnPassengersCollisions();
		}
		
		if ( (m_bDriveFinished) && (m_Physics.m_fCurrSpeed == 0) )
		{
			SetState("UncaredDriveState");
			OnDestinationPos();
		}
	}

	Vector m_vLastPos = null;
	Vector m_vLastDir = null;
        
    /** MovieSimulateState - stan jazdy specjalnie do filmow, gdzie pojazd
     * ma sie odpowiednio ustawiac do podloza i odpalac FX'y */
    public class MovieSimulateState extends State
    {
        public void OnEnter()
        {
            if (m_bGameLoading)
            {
                m_bGameLoading = false;
                return;
            }
            
            fLastT = GetTime();
			if (!m_bPhysicsActive)
				SetPhysicsVehicleActive(true);
			m_vLastPos = GetPosition();
			m_vLastDir = GetForwardVector();
        }
        
        public void OnFrame()
        {
            GlobalOnFrame();
		    MovieSimulate();
            if (m_bMakeVehicleNoiseInMovie)
                MakeVehicleNoise();
        }
		
		public void MovieSimulate()
		{
			if (!m_bSimulate)
				return;
            
            // liczymy czas, jaki minal
			float fCurrentT = GetTime();
			float fdT = fCurrentT - fLastT;
			if (fdT == 0f)
				return;
            if (fdT > 1f)
            {
    			fLastT = fCurrentT;
                return;
            }
			
            // obliczamy predkosc
			Vector vPos = GetPosition();
			float fSpeed = vPos.Distance(m_vLastPos) / fdT;
			m_vLastPos.Set(vPos);

            // obliczamy kat, o jaki sie obrocilismy od ostatniej ramki
			Vector vForward = GetForwardVector();
			float fAngle = 0f;
			Vector vUpDir = null;
            if (vForward.Distance(m_vLastDir) > 0.0001f)
			{
				vUpDir = m_vLastDir.Cross(vForward);
				vUpDir.Normalize();
				fAngle = (float) java.lang.Math.acos(vForward.Dot(m_vLastDir)) * RAD_TO_DEG;
                if (vUpDir.fY > 0)
                    fAngle *= -1f;
			}
            
            float fAngleSpeed = fAngle / fdT;

			if ( (m_Physics != null) && (m_bPhysicsActive == true) )
			{
				SetAllFramePeriod(fdT);
				float fTimePeriod = GetNextTimePeriod();
				while (fTimePeriod > 0f)
				{
					m_Physics.MovieSimulate(fSpeed, fAngleSpeed, fTimePeriod);
					fTimePeriod = GetNextTimePeriod();
				}
			}
			
			m_vLastDir.Set(vForward);
			fLastT = fCurrentT;
		}		
    }

    float GetCameraAttenuation()
    {
        return m_fAttenuation;
    }
    
    float GetCameraForceFactor()
    {
        return m_fForceFactor;
    }
    
    float GetAmplitudeCoeff()
    {
        return m_fAmplitudeCoeff;
    }

///////////////////////////////////////////////////////////////////////////////
//  dzwieki
///////////////////////////////////////////////////////////////////////////////
    void StartSounds()
    {
		m_bSoundsStarted = true;
		
        if ( (m_sSoundStartName != null) && (m_bPlayEngineStartStopSound) )
        {
            // jest dzwiek odpalania autka, wiec najpierw jego odegramy
            PlaySound(m_sSoundStartName);
            
            // policzmy za jaki czas mamy wlaczyc dzwiek lopa-silnika
            //float fWaitTime = GetWaveDuration(m_sSoundStartName) - 0.1f;
            float fWaitTime = m_fSoundVehicleStartDuration - 0.1f;
			
			if (m_bCanDrive)
				CallMethodIn("AttachLoopSound", fWaitTime);
			else
			{
				CallMethodIn("StopSounds", fWaitTime);
                CallMethodIn("AlreadyCannotDrive" , fWaitTime + _TIME_TO_GET_OUT_IF_VEHICLE_ALREADY_DESTROYED);
			}
        }
        else
        {
			if (m_bCanDrive)
				AttachLoopSound();
			else
			{
				StopSounds();
				CallMethodIn("AlreadyCannotDrive" , _TIME_TO_GET_OUT_IF_VEHICLE_ALREADY_DESTROYED);
			}
        }
    }
    
    void AttachLoopSound()
    {
        if (( m_sSoundLoopName != null ) && 
            (m_nSoundLoopId == 0) &&
            (m_bSoundsStarted))
        {
             m_nSoundLoopId = AttachSound(m_sSoundLoopName, m_fSoundLoopMin, m_fSoundLoopMax);
             UpdateSounds();
             m_bEngineOn = true;  
        }
    }
    
    void DetachLoopSound()
    {
        if (m_nSoundLoopId != 0)
        {
            DetachSound(m_nSoundLoopId);            
            m_nSoundLoopId = 0;
            m_bEngineOn = false;            
        }
    }
    
    void StopSounds()
    {
        if (m_bSoundsStarted)
        {
            if ( (m_sSoundStopName != null) && (m_bPlayEngineStartStopSound) )
                PlaySound(m_sSoundStopName);
            DetachLoopSound();
            m_bSoundsStarted = false;
        }
    }
    
    void UpdateSounds()
    {
        if ( (m_nSoundLoopId != 0)  && (m_Physics != null) )
        {
            // zmieniamy czestotliwosc
            float fFreq, fSpeedCoeff;
            if (m_Physics.m_fCurrSpeed  >= 0.0f)
                fSpeedCoeff = m_Physics.m_fCurrSpeed/m_Physics.m_Params.GetMaxSpeed();
            else
                fSpeedCoeff = m_Physics.m_fCurrSpeed/m_Physics.m_Params._MIN_SPEED;
            
            if(fSpeedCoeff > 1.0f) fSpeedCoeff = 1.0f;
            if(fSpeedCoeff < 0.0f) fSpeedCoeff = 0.0f;
            
            fFreq = m_fMinFreq + (m_fMaxFreq-m_fMinFreq)*fSpeedCoeff;
            
            SetSoundFreq(m_nSoundLoopId, fFreq);
        }
    }

	void HitSound()
	{
	    if (m_sSoundHitName != null)
			PlaySound(m_sSoundHitName);
	}
	
	void SlideSound()
	{
	    if (m_sSoundSlideName != null)
			PlaySound(m_sSoundSlideName);
	}
	
    void AccelSound()
	{
		if (m_sSoundAccelName != null)
			PlaySound(m_sSoundAccelName);
	}
    
	void BrakeSound()
	{
		if (m_sSoundBrakeName != null)
			PlaySound(m_sSoundBrakeName);
	}

    void LoadSoundParams(String sFile)
    {
        ParseKeyword Root = ParseFile(sFile);
        
        if (Root == null)
		{
            CrashLog("No file " + sFile + "\n");
			return;
		}
		
        ParseKeyword Key;
        for(Key = Root; Key != null; Key = Key.GetNext())
        {
            if (Key.Equals("MinFreq")) 
                m_fMinFreq = Key.GetFParam(0);
            if (Key.Equals("MaxFreq")) 
                m_fMaxFreq = Key.GetFParam(0);
            if (Key.Equals("MaxFreqBack")) 
                m_fMaxFreqBack = Key.GetFParam(0);
        }
        Root.delete();
    }

///////////////////////////////////////////////////////////////////////////////	
// jezdzenie pojazdu po sciezce patrolowej
///////////////////////////////////////////////////////////////////////////////
	/** ustawienie sciezki pojazdu */
	void SetVehiclePath(Path path)
	{
		m_VehiclePath = path;
		
		CheckRadiuses();
	}
	
	void CheckRadiuses()
	{
		if (m_VehiclePath != null)
		{
			for (int i = 0; i < m_VehiclePath.GetPointsNum(); i++)
			{
				if (m_VehiclePath.GetPoint(i).fRadius == 0f)
					m_VehiclePath.GetPoint(i).fRadius = 200f;
			}
		}
	}
	
	/** (depreciated) probujemy zaladowac sciezke o nazwie, jaka ma pojazd */
	void TryToLoadPath()
	{
		Path path = GetModule().cPathsManager.GetPathByName(GetName());
		if (path != null)
			SetVehiclePath(path);
	}
	
	/** tworzenie sciezki pojazdu z waypointow */
	void TryToLoadPath(Waypoint waypoint)
	{
		Path path = new Path();
		path.SetPathOneWay();
        mt_Move defMotion = new mt_Move();
        mt_Move motion;
        Waypoint firstWaypoint = waypoint;
        
		while (waypoint != null && 
               (waypoint.cMotion instanceof mt_Move || waypoint.cMotion == null))
		{
            if (waypoint.cMotion == null)
                motion = defMotion;
            else
                motion = (mt_Move) waypoint.cMotion;
            motion.bCritical = true;
			path.AddPathPoint(new PathPoint(waypoint.GetPosition(),
											motion.fRadius,
											motion.cSpeed.GetValue()));
			waypoint = waypoint.cNext;
            if (waypoint == firstWaypoint)
                break;
		}
		if (path != null)
			SetVehiclePath(path);
	}
	
    boolean m_bValuesSetForPatrol = false;
    
	/** ustawiamy poslizg vehicle na bardzo maly */
	void SetVehicleParamsForPatrol()
	{
        if (m_bValuesSetForPatrol)
            return;
        m_bValuesSetForPatrol = true;
		m_fOldSlideValue = m_Physics.m_Params._SLIDE_SUPRESS;
        m_Physics.m_Params._SLIDE_SUPRESS = m_Physics.m_Params._SLIDE_COEF;
		m_fOldSlowDownValue = m_Physics.m_Params._SLOW_COEF;
		m_fOldAccelValue = m_Physics.m_Params._ACCELERATION_COEF;
		m_Physics.m_Params._ACCELERATION_COEF = 700f;
		m_fOldBrakeValue = m_Physics.m_Params._BRAKE_COEF;
		m_Physics.m_Params._BRAKE_COEF = 2500f;
	}
	
	/** przywrocenie starego poslizgu - zawolac np. przed zmiana stanu na inny */
	void RestoreVehicleParamsBeforePatrol()
	{
        if (!m_bValuesSetForPatrol)
            return;
        m_bValuesSetForPatrol = false;
		m_bIsPatrolling = false;
        m_Physics.m_Params._SLIDE_SUPRESS = m_fOldSlideValue;
		m_Physics.m_Params._SLOW_COEF = m_fOldSlowDownValue;
		m_Physics.m_Params._ACCELERATION_COEF = m_fOldAccelValue;
		m_Physics.m_Params._BRAKE_COEF = m_fOldBrakeValue;
	}

	/** rozpoczecie patrolowania */
	void PatrolStart()
	{
		if (m_VehiclePath == null)
			TryToLoadPath();
		if (m_VehiclePath == null)
		{
			CrashLog("Pojazd nie ma przypisanej sciezki patrolowej !!!\n");
			return;
		}

		SetVehicleParamsForPatrol();
		SetState("PatrolPathState");
	}

	/** zaprzestanie patrolowania */
	void PatrolEnd()
	{
		RestoreVehicleParamsBeforePatrol();
		SetState("UncaredDriveState");
	}
	
	/** zatrzymanie pojazdu na chwile */
	public void PatrolBreak()
	{
		m_bBreakPatrol = true;
	}
	
	/** wznowienie poruszania sie pojazdu */
	public void PatrolResume()
	{
		m_bBreakPatrol = false;
	}
	
///////////////////////////////////////////////////////////////////////////////
// akcje edytorowe
///////////////////////////////////////////////////////////////////////////////
	/** tworzymy sciezke patrolowa i zaczynamy jechac */
	public void EditorPatrol(Waypoint waypoint, IVehiclePassenger controller)
	{
		if (m_bEditorAction)
		{
			// ustalamy predkosc pojazdu w drodze do punktu
			if ( (m_iCurrPatrolPoint >= 0) && (m_iCurrPatrolPoint < m_VehiclePath.GetPointsNum()) )
			{
				int iSpeed = m_VehiclePath.GetPoint(m_iCurrPatrolPoint).nFlags;
				if ( (iSpeed >= 0) && (iSpeed <= m_aSpeeds.length) )
					m_Physics.SetCurrMaxSpeed(m_aSpeeds[iSpeed]);
			}
			
			if (m_bBreakPatrol)
			{
				// zatrzymalismy sie, teraz ruszamy ponownie
				PatrolResume();
			}
		}
		else
		{
			// zaczynamy jazde
			TryToLoadPath(waypoint);
			if (m_VehiclePath == null)
			{
				CrashLog("Pojazd nie ma przypisanej sciezki patrolowej !!!\n");
				return;
			}
			
			m_bEditorAction	= true;
			m_VehicleController = controller;
			SetVehicleParamsForPatrol();
			SetState("PatrolPathState");
			
			// tutaj m_iCurrPatrolPoint == 0
			int iSpeed = m_VehiclePath.GetPoint(m_iCurrPatrolPoint).nFlags;
			if ( (iSpeed >= 0) && (iSpeed <= m_aSpeeds.length) )
				m_Physics.SetCurrMaxSpeed(m_aSpeeds[iSpeed]);
		}
	}
	
	/**	zatrzymanie sie (jezeli jedziemy edytorowo) i wylaczenie silnika */
	public void FinishDrive()
	{
		m_bEditorAction	= false;
		RestoreVehicleParamsBeforePatrol();
		m_Physics.SetCurrMaxSpeed(m_Physics.GetMaxSpeed());
        SetState("UncaredDriveState");
	}
	
///////////////////////////////////////////////////////////////////////////////
// update HUD'a
///////////////////////////////////////////////////////////////////////////////
	void SetUpHudComponentToUpdate(HUDManager hud)
	{
		if (hud != null)
		{
			hud.SetUniversalHUDActor(this, Text.Get("&HUD_VEHICLE_HEALTH&"), fMaxHealth, 0);
		}
	}
	
	void StopUpdatingHud(HUDManager hud)
	{
		DisableCallMethod("UpdateHUD");
		if (hud != null)
		{
			hud.ClearUniversalHUDActor(this);
		}
	}	
///////////////////////////////////////////////////////////////////////////////
// siec
///////////////////////////////////////////////////////////////////////////////
    static final int MSG_CLIENT_VEHICLE_STATE = 0; // server wysyla informacje o stanie
    static final int MSG_CLIENT_RESPAWN       = 1; // server wysyla informacje o spawnie
    static final int MSG_CLIENT_CANNOT_DRIVE  = 2; //
    static final int MSG_CLIENT_CRASH         = 3; // server wysyla informacje o smierci
    static final int MSG_CLIENT_HIDE          = 4;
    static final int MSG_CLIENT_HIDE_WRECK    = 5;
    static final int MSG_SERVER_COLLISION     = 6; // klient, na ktorym jest symulowany pojazd informuje server o kolizji
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
         case MSG_CLIENT_RESPAWN:
            LogD(" MSG_CLIENT_RESPAWN\n");
            MsgRecvClientRespawn();            
            break;
            
        case MSG_CLIENT_CANNOT_DRIVE:
			LogD(" MSG_CLIENT_CANNOT_DRIVE\n");
			MsgRecvClientCannotDrive();
            break;
            
        case MSG_CLIENT_CRASH:
			LogD(" MSG_CLIENT_CRASH\n");
			MsgRecvClientCrash();
            break;
            
        case MSG_CLIENT_HIDE:
			LogD(" MSG_CLIENT_HIDE\n");
			MsgRecvClientHide();
            break;            
            
        case MSG_CLIENT_HIDE_WRECK:
			LogD(" MSG_CLIENT_HIDE_WRECK\n");
			MsgRecvClientHideWreck();
            break;            
        
        case MSG_SERVER_COLLISION:
            LogD(" MSG_SERVER_COLLISION\n");
			MsgRecvServerCollision();
            break;            
        
        /* TODO
        case MSG_CLIENT_VEHICLE_STATE:
            LogD(" MSG_CLIENT_VEHICLE_STATE\n");
            MsgRecvClientVehicleState();            
            break;            */
            
        default:
            MessageCancel();
        }
    }
    

    boolean m_bModuleNet = false;

    // parametry, ktory beda:
    // -pobierany z fizyki na wlascicielu
    // -replikowany na serwer, jesli wlasciciel jest klientem
    // -replikowany z serwera na klientow niewlascicieli
    // -na klientach nie wlascicielach wstawiany do fizyki
    float m_fCurrDirAngle = 0.0f;
    float m_fCurrSpeed    = 0.0f;
    float m_fMoveSpeed    = 0.0f;
    byte  m_iAcc          = 0;
    float m_fTurn         = 0.0f;
	float m_fRealSpeed	  = 0f;
	float m_fReplicationTime	  = 0f;
	float m_fAngleSpeed	  = 0f;
	float m_fRealAccel	  = 0f;
	
    static int sm_nVehicleMoveSet   = -1;
    static int sm_nVehicleHealthSet = -1;
    public void ReplicationInit()
    {
        //TODO: PositionAtTime
        //sm_nVehicleMoveSet   = NewReplicationSet("nativePosition;nativeDir;nativeUp;m_fCurrDirAngle;m_fCurrSpeed;m_iAcc;m_fTurn");
        sm_nVehicleMoveSet   = NewReplicationSet("nativePositionMinEverySec;nativeDir;m_fCurrDirAngle;m_fCurrSpeed;m_iAcc;m_fTurn;m_fRealSpeed;m_fReplicationTime;m_fAngleSpeed;m_fRealAccel");
        sm_nVehicleHealthSet = NewReplicationSet("fHealth");
        
        /* uzyc ponizszych metod do ograniczenia przesylania replikowanych danych
        SetReplicationSetMinPeriod( sm_nVehicleMoveSet, 0.0f );
        SetReplicationSetMinPeriodUnrelevantMul( sm_nVehicleMoveSet, 5.0f );
        */
        
        //SetReplicationSetMinPeriod( sm_nVehicleHealthSet, 10.0f );
        //SetReplicationSetMinPeriodUnrelevantMul( sm_nVehicleHealthSet, 10.0f );
        
        
        SetNetPriority( 2.0f );
        SetMinRelevantLevel( 0.01f );
    }

    static boolean sm_bLimitReplicationTimes = true;
    
    boolean bDontReplicate = false;
   
    float fLastReplicationTime = 0.0f;
    public float sm_ReplicateInverval = 0.5f;
        
	float m_fLastReplicatedRealAccel = 0f;
	
    void CheckIfShouldReplicate()
    { 
        if(!sm_bLimitReplicationTimes) return;
           
        float fNow = GetTime();
        if ( (fNow - fLastReplicationTime >  sm_ReplicateInverval) ||
			 (Math.abs(m_fRealAccel - m_fLastReplicatedRealAccel) > 1f) )
        {
			m_fLastReplicatedRealAccel = m_fRealAccel;
            fLastReplicationTime = fNow;
            bDontReplicate = false;
        }
        else
        {
            if(fNow == fLastReplicationTime)
                return;
            else                                                      
                bDontReplicate = true;
        }
    }

    /** Replikacja odpowiednich zestawow dla odpowiednich warunkow */
    public void OnReplicateServer()
    {
        CheckIfShouldReplicate();
        
        if(bDontReplicate) return;
        
        //TODO: w inicie przeslac stan wraka
        
        Pawn cDriver    = GetFirstOccupant( EPassengerRole._VEHICLE_DRIVER );
        Pawn cPassenger = GetFirstOccupant( EPassengerRole._VEHICLE_PASSENGER );
        if ( bNetInit || cDriver == null || !cDriver.bNetIsOwner )
        {
            //TODO: wyslac msg ze stanem
            Replicate( sm_nVehicleMoveSet );
        }
        if (  bNetInit || 
            ( cDriver    != null && cDriver.bNetIsOwner) ||
            ( cPassenger != null && cPassenger.bNetIsOwner) )
        {
            Replicate( sm_nVehicleHealthSet );
        }
    }
    
    public void OnReplicateClient()
    {
        if ( m_bLocalPlayerDriver )
        {
            Replicate( sm_nVehicleMoveSet );
        }
    }
             
    public boolean IsRelevantConstans(){ return false; }
    
    /** Okreslenie warunku kiedy obiekt jest symulowany na cliencie
     * Wolane dla kazdego clienta */
    public float GetRelevantLevel()
    {
        if ( fNetForceRelevantLevel >= 0.0f )
            return fNetForceRelevantLevel;
            
        Pawn cDriver    = GetFirstOccupant(EPassengerRole._VEHICLE_DRIVER);
        Pawn cPassenger = GetFirstOccupant(EPassengerRole._VEHICLE_PASSENGER);
        /*if ( cDriver    == null || !cDriver.bNetIsOwner ||
             cPassenger == null || !cPassenger.bNetIsOwner )
            return 1.0f;*/
        if ( ((cDriver    != null) && cDriver.bNetIsOwner) ||
             ((cPassenger != null) && cPassenger.bNetIsOwner) )
            return 1.0f;
        
        float fRel = 1.0f-((GetCurrentClientDistance() - GetModule().fMinDistToCalculateRelevant)/
                           (GetModule().fMaxDistToCalculateRelevant-GetModule().fMinDistToCalculateRelevant));
        if ( fRel > 1.0f )
             fRel = 1.0f;
        return fRel;
    }
            
    public boolean DeleteIfNotRelevant()
    {
        return false;
    }


    /**
     * Nadpisane zeby nie wysylac pozycji na poziomie ChromeActora
     * przy spawnie
     */
    void MsgSendClientState()
    {
    }
    
    
    Vector m_vRecvUp  = null;
    Vector m_vRecvDir = null;
    Vector m_vRecvPos = null;
    
    void MessagePackPos()
    {
        MessagePackVector( GetUpVector() );
        MessagePackVector( GetForwardVector() );
        MessagePackVector( GetPosition() );
    }
    
    void MessageUnPackPos()
    {
        m_vRecvUp  = MessageUnPackVector();
        m_vRecvDir = MessageUnPackVector();
        m_vRecvPos = MessageUnPackVector();
    }
    
    void ApplyReceivedPos()
    {
        FromUpForwardPos( m_vRecvUp, m_vRecvDir, m_vRecvPos );
        ResetInterpolatePosAndDir();
    }
    
    /*
    void MsgSendClientVehicleState()
    {
    }
    
    void MsgRecvClientVehicleState()
    {
    }*/
    
    void MsgSendClientRespawn(boolean bOnLevelRestart)
    {
        MessageNew( MSG_CLIENT_RESPAWN );
        MessagePackInt( GetNextGlobalVehicleStateCounter() );
        MessagePackPos();
        MessagePackBoolean( bOnLevelRestart );
        MessageSend();
    }
    
    void MsgRecvClientRespawn()
    {
        int nStateCounter = MessageUnPackInt();
        MessageUnPackPos();
        boolean bOnLevelRestart = MessageUnPackBoolean();
        MessageEnd();
        
        if ( VerifyVehicleStateCounter( nStateCounter ) )
        {
            ApplyReceivedPos();
            Respawn( bOnLevelRestart );
        }
    }
    
    void MsgSendClientCannotDrive()
    {
        MessageNew( MSG_CLIENT_CANNOT_DRIVE );
        MessagePackInt( GetNextGlobalVehicleStateCounter() );
        MessagePackPos();
        MessageSend();
    }
    
    void MsgRecvClientCannotDrive()
    {
        int nStateCounter = MessageUnPackInt();
        MessageUnPackPos();
        MessageEnd();
        
        if ( VerifyVehicleStateCounter( nStateCounter ) )
        {
            //TODO: jezeli nie jest wyspawnowany to go wyspawnowac
            ApplyReceivedPos();
            CannotDrive();
        }
    }

    void MsgSendClientCrash()
    {
        LogD("\n");
        MessageNew( MSG_CLIENT_CRASH );
        MessagePackInt( GetNextGlobalVehicleStateCounter() );
        MessagePackPos();
        MessageSend();
    }
    
    void MsgRecvClientCrash()
    {
        int nStateCounter = MessageUnPackInt();
        MessageUnPackPos();
        MessageEnd();

        if ( VerifyVehicleStateCounter( nStateCounter ) )
        {
            //TODO: jezeli nie jest wyspawnowany to go wyspawnowac
		    ApplyReceivedPos();
            VehicleCrashed();
        }
    }
    
    void MsgSendClientHide()
    {
        MessageNew( MSG_CLIENT_HIDE );
        MessagePackInt( GetNextGlobalVehicleStateCounter() );
        MessageSend();
    }
    
    void MsgRecvClientHide()
    {
        int nStateCounter = MessageUnPackInt();
        MessageEnd();
        
        if ( VerifyVehicleStateCounter( nStateCounter ) )
            Hide();
    }
    
    void MsgSendClientHideWreck()
    {
        MessageNew( MSG_CLIENT_HIDE_WRECK );
        MessagePackInt( GetNextGlobalVehicleStateCounter() );
        MessageSend();
    }
    
    void MsgRecvClientHideWreck()
    {
        int nStateCounter = MessageUnPackInt();
        MessageEnd();
        
        if ( VerifyVehicleStateCounter( nStateCounter ) )
            HideWreck();
    }
        
    void MsgSendServerVehicleCollision(float fDamage)
    {
        MessageNew( MSG_SERVER_COLLISION );
        MessagePackFloat( fDamage );
        MessagePackInt( this.GetNetId() );
        MessageSend();            
    }
    
	void MsgRecvServerCollision()
    {
        float fDamage = MessageUnPackFloat();
        int iVehicleId = MessageUnPackInt();
        MessageEnd();            

        Vehicle damagedVehicle = (Vehicle) GetObjectByNetId( iVehicleId );
        //CrashLog("damagedVehicle:" + damagedVehicle + "\n");
        damagedVehicle.TakeDamage( fDamage, EnumDamageType._BULLET );	
    }
    
    /**
     * 
     * Czasy spawnowania pojazdow
     * 
     */
    
    /**
     * Czas opoznienia FXa spawnowania pojazdu (czas przez ktory
     * gracz bedzie 'ostrzegany' o nadchodzacym spawnie (odpowiedni
     * dzwiek, miganie poda) 
     * Po tym czasie pojawia sie FX spawnu
     */
    private 
    static  float  sm_fSpawnFXDelayTime = 3.5f;
    static  double GetSpawnFXDelayTimeInMiliseconds(){ return (double)(sm_fSpawnFXDelayTime * 1000.0f); }
    
    /**
     * Czas opoznienia pomiedzy rozpoczeciem odpalania
     * FX spawnu a faktycznym pojawieniem sie pojazdu
     * w swoim miejscu (na podzie)
     */
    private 
    static  float  sm_fSpawnShowDelayTime = 1.0f;

    /**
     * Zwraca calkowity czas pomiedzy rozpoczeciem
     * spawnowania (miganie poda i dzwiek ostrzegawczy)
     * do momentu pojawienia sie pojazdu (na podzie)
     */
    static  float  GetSpawnDelayTime(){ return sm_fSpawnFXDelayTime+sm_fSpawnShowDelayTime; }
    
    
    /**
     * 
     * Czasy chowania pojazdow
     * 
     */
    
    /**
     * Czas opoznienia FXa spawnowania pojazdu (czas przez ktory
     * gracz bedzie 'ostrzegany' o nadchodzacym spawnie (odpowiedni
     * dzwiek, miganie poda) 
     * Po tym czasie pojawia sie FX spawnu
     */
    private 
    static  float  sm_fHideFXDelayTime = 3.5f;
    static  double GetHideFXDelayTimeInMiliseconds(){ return (double)(sm_fHideFXDelayTime * 1000.0f); }
                
    
    /**
     * Calkowity czas opoznienia znikania pojazdu 
     * czas pomiedzy rozpoczeciem spawnowania (dzwiek ostrzegawczy)
     * do momentu zniekniecie pojazdu
     * Uwaga: Jako ze FX spawnu i schowania jest taki 
     * sam, zatem zwraca ta samo wartosc co GetSpawnDelayTime()
     * TODO: jak beda osobne FXy do spawnu i chowania pojazdow
     * to zmienic
     */
    static  float  GetHideDelayTime(){ return GetSpawnDelayTime(); }
    
    
    
    void ServerHide( float fDelayTime )
    {
        ForceStillState();
        
        Hide();
        MsgSendClientHide();
        
        DisableCallMethod( METHOD_ServerHideDo );
        CallMethodIn( METHOD_ServerHideDo, fDelayTime );
    }
    
    void Hide()
    {
        m_bCanDrive   = false;
        m_nState      = _HIDDEN;
    }
    
    void ServerHideDo()
    {
        Vector vPos = GetPosition();
        vPos.fY -= 1000.0f;
        SetPosition( vPos );
        
        DisableCallMethod( METHOD_ServerHideDo );
        CheckAddVehicleToAutoRespawn( true );
    }
    
    void ServerRespawn( VehicleSpawnInfo cSpawnInfo, 
                        float            fDelayTime, 
                        boolean          bOnLevelRestart )
    {
        DisableCallMethod( METHOD_ServerHideDo );
        ForceStillState();
                
        if ( bOnLevelRestart )
            fDelayTime = 0.0f;
        
        super.ServerRespawn( cSpawnInfo, fDelayTime );
        
        //i dajemy mu max energii
        fHealth = cSpawnInfo.m_fHealth;
        m_nRespawnCnt++;
        
        if ( bOnLevelRestart )
        {
            m_nRespawnCnt = 0;
            Enable();
        }
        
        Respawn( bOnLevelRestart );
        MsgSendClientRespawn( bOnLevelRestart );
    }
    
    void Respawn( boolean bOnRestartLevel )
    {
        //wolamy respawn do poda,
        //on sie "zaswiecici" jakim czasie ma sie wylaczyc
        //i usmiercic graczy na nim stojacych
        //no chyba ze to tryb bOnRestartLevel, to tylko ustawi material na normalny
        if ( m_cPod != null )
            m_cPod.Respawn( bOnRestartLevel, this );
        
        SetStateSpawned();
    }
    
    void SetStateSpawned()
    {
        ForceStillState();
        
		m_vLastInterpolatedDir = null;
		m_fLastSimulatedSpeed = 0f;
		m_fRealAccel = 0f;
		m_fRealSpeed = 0f;
		
        StopFXSmokeUpdate();
        fHealth = fMaxHealth;
        m_cPlayerNetAssassin = null;
        m_bCanDrive     = true;
        bDead           = false;
        bIsActiveTarget = true;
        bActiveTrigger  = true;
        Enable();
        EnableCallMethods();
        InitPhysicsParamsFromVehicle();
        EnableRendering(true);
        ActivateOnModule();
        SetCorrectPosOnRender( true );
        HideWreck();
        
        // jeep wybucha, ale jak sie zrespawnuje, to efekty trzeba usunac
        DisableCallMethod("DeleteAttachedFXes");
        DeleteAttachedFXes();
        
        //wolam celowo super bo ze stanu CrashedState normalnie nie da sie wyjsc
        super.SetState("StillState");
        
        //kliencie powinni byc zwarci i gotowi na otrzymywanie 
        //informacji poprzez replikacje
        EnableIncomingReplication( Role < ERole._AUTHORITY );
        
        m_nState = _SPAWNED;
    }
    
    
    boolean bOld = false;
    
    void ForceStillState()
    {
        if ( bOld )
            return;
        
        if ( CurrentState instanceof DrivenState || 
             CurrentState instanceof UncaredDriveState ) 
        {
            super.SetState( "StillState" );
        }
        EnableIncomingReplication( false );
        m_Physics.Clear();
    }
    
    private int m_nRespawnCnt = 0;
    void ResetRespawnCnt(){ m_nRespawnCnt = 0; }
    int  GetRespawnCnt(){ return m_nRespawnCnt; }
    
    final static int _SPAWNED      = 0;
    final static int _CANNOTDRIVE  = 1;
    final static int _CRASHED      = 2;
    final static int _HIDDEN       = 3;
    int   m_nState = _SPAWNED;
    
        
    void CheckAddVehicleToAutoRespawn()
    {
        CheckAddVehicleToAutoRespawn( false );
    }
    
    void CheckAddVehicleToAutoRespawn(boolean bNow)
    {
        if ( GetModule() instanceof ModuleNet )
            ((ModuleNet)GetModule()).CheckAddVehicleToAutoRespawn( this, bNow );    
    }
    
    void CheckAddVehicleToAutoHide()
    {
        if ( GetModule() instanceof ModuleNet )
            ((ModuleNet)GetModule()).CheckAddVehicleToAutoHide( this );    
    }
    
    void RemoveVehicleFromAutoHide()
    {
        if ( GetModule() instanceof ModuleNet )
            ((ModuleNet)GetModule()).RemoveVehicleFromAutoHide( this );    
    }

    /////////////////////////////////////////////////////////////////////////////
    // Implementacja ipar_MotionSpeed
    
    public void SetMotionSpeedValue(int iMotionSpeed, float fNewSpeedValue)
    {
        if (iMotionSpeed >=0 && iMotionSpeed < m_aSpeeds.length)
        {
            m_aSpeeds[iMotionSpeed] = fNewSpeedValue;
        }
    }
    
    public void MultiplyMotionSpeedValue(int iMotionSpeed, float fSpeedValueMulitplier)
    {
        if (iMotionSpeed >=0 && iMotionSpeed < m_aSpeeds.length)
        {
            m_aSpeeds[iMotionSpeed] *= fSpeedValueMulitplier;
        }
    }
    
    void SetMotionMoveFromKeysState( Player cDriver )
    {
        if ( cDriver == null || ! cDriver.bNetIsOwner )
            return;
        if ( motion == null )
            return;
        motion.SetMoveFromKeysState( GetKeyState( EGameControl.FORWARD ),
                                     GetKeyState( EGameControl.BACKWARD ),
                                     GetKeyState( EGameControl.STRAFE_RIGHT),
                                     GetKeyState( EGameControl.STRAFE_LEFT) );
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    boolean m_bGameLoading = false; // tego bron boze nie sejwowac
	int m_nSoundLoopIdSG = 0;
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Vehicle.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_VEHICLE)
        {	
			super.SGLoadChunk(cFC);

            m_bSimulate = cFC.LoadBool();
            int iTemp = cFC.LoadInt();
			for (int i = 0; i < iTemp; i++)
				aSeatsInfo.set(i, (SeatInfo) cFC.LoadObjectRef());
            m_Physics = (PhysicsVehicle) cFC.LoadObject(m_Physics, this);
            m_bPhysicsActive = cFC.LoadBool();
            m_bEngineOn = cFC.LoadBool();
			motion = (VehicleMotionState)cFC.LoadObjectRef();
            fLastT = cFC.LoadFloat();
            m_bCanDrive = cFC.LoadBool();
            fFuel = cFC.LoadFloat();
            m_fAllFrameTime = cFC.LoadFloat();
            m_iCurrPatrolPoint = cFC.LoadInt();
            m_iAddValue = cFC.LoadInt();
            m_bDriveFinished = cFC.LoadBool();
            m_bBreakPatrol = cFC.LoadBool();
            m_bIsPatrolling = cFC.LoadBool();
            m_VehiclePath = (Path) cFC.LoadObjectRef();
            m_fOldSlideValue = cFC.LoadFloat();
            m_fOldSlowDownValue = cFC.LoadFloat();
            m_fOldAccelValue = cFC.LoadFloat();
            m_fOldBrakeValue = cFC.LoadFloat();
            m_nSoundLoopIdSG = cFC.LoadInt();
            m_bPlayEngineStartStopSound = cFC.LoadBool();
            m_bSoundsStarted = cFC.LoadBool();
            iTemp = cFC.LoadInt();
            for (int i = 0; i < iTemp; i++)
                m_aSpeeds[i] = cFC.LoadFloat();
            m_bUseMotionSpeed = cFC.LoadBool();
            m_bEditorAction = cFC.LoadBool();
            m_VehicleController = (IVehiclePassenger) cFC.LoadObjectRef();
            m_bIsGridIndependent = cFC.LoadBool();
            m_fLastVehicleNoiseTime = cFC.LoadFloat();
            m_vNoisePos = cFC.LoadVector();
            
            m_vLastPos = cFC.LoadVector();
            m_vLastDir = cFC.LoadVector();
            
            m_bVehicleInMovie = cFC.LoadBool();

            m_bValuesSetForPatrol = cFC.LoadBool();
            
            m_bSmokeSpawned = cFC.LoadBool();
            
            bOnAppearShowWreckOnly = cFC.LoadBool();
            
            m_bGameLoading = true;
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Vehicle.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_VEHICLE, cFCParent);
		
		super.SGSaveChunk(cFC);
		
        cFC.SaveBool(m_bSimulate);
        cFC.SaveInt(aSeatsInfo.size());
		for (int i = 0; i < aSeatsInfo.size(); i++)
			cFC.SaveObjectRef((SeatInfo) aSeatsInfo.get(i));
        cFC.SaveObject(m_Physics);
        cFC.SaveBool(m_bPhysicsActive);
        cFC.SaveBool(m_bEngineOn);
		cFC.SaveObjectRef(motion);
        cFC.SaveFloat(fLastT);
        cFC.SaveBool(m_bCanDrive);
        cFC.SaveFloat(fFuel);
        cFC.SaveFloat(m_fAllFrameTime);
        cFC.SaveInt(m_iCurrPatrolPoint);
        cFC.SaveInt(m_iAddValue);
        cFC.SaveBool(m_bDriveFinished);
        cFC.SaveBool(m_bBreakPatrol);
        cFC.SaveBool(m_bIsPatrolling);
        cFC.SaveObjectRef(m_VehiclePath); // paths are depreciated, this is why we can save it
        cFC.SaveFloat(m_fOldSlideValue);
        cFC.SaveFloat(m_fOldSlowDownValue);
        cFC.SaveFloat(m_fOldAccelValue);
        cFC.SaveFloat(m_fOldBrakeValue);
        cFC.SaveInt(m_nSoundLoopId);
        cFC.SaveBool(m_bPlayEngineStartStopSound);
        cFC.SaveBool(m_bSoundsStarted);
        cFC.SaveInt(m_aSpeeds.length);
        for (int i = 0; i < m_aSpeeds.length; i++)
            cFC.SaveFloat(m_aSpeeds[i]);
        cFC.SaveBool(m_bUseMotionSpeed);
        cFC.SaveBool(m_bEditorAction);
        cFC.SaveObjectRef((ISaveGame)m_VehicleController);
        cFC.SaveBool(m_bIsGridIndependent);
        cFC.SaveFloat(m_fLastVehicleNoiseTime);
        cFC.SaveVector(m_vNoisePos);
        
        cFC.SaveVector(m_vLastPos);
        cFC.SaveVector(m_vLastDir);
	
        cFC.SaveBool(m_bVehicleInMovie);
        
        cFC.SaveBool(m_bValuesSetForPatrol);

        cFC.SaveBool(m_bSmokeSpawned);
        
        cFC.SaveBool(bOnAppearShowWreckOnly);
            
        cFC.delete();
    }	    			
    
	public void SGUpdate()
	{
		FileChunk.Log("[Vehicle.SGSaveChunk]\n");

		super.SGUpdate();

        boolean bEngineWasOn = m_bEngineOn;
        
		DetachLoopSound();
			
		//odswiezenie speakera
		RemoveSpeaker();
		CreateSpeaker();
		
		if (bEngineWasOn)
			AttachLoopSound();
        
        if (m_Physics != null)
            m_Physics.SGUpdate();
        
        m_bGameLoading = false;
	}
    
    Vector  m_vActuallyUp = new Vector();
    Vector  m_vRenderUp   = new Vector();
    boolean m_bOnRenderUp = false;
    
	float m_fInterpDirWeight = 0.1f;
	
	boolean m_bSim = true;
	boolean m_bCorrectPos = true;
	
    void OnRender()
    {
        if ( ! m_bCorrectPosOnRender )
		{
			GetPosition( m_vActuallyPosition );
			GetForwardVector( m_vActuallyDir );
			UpdateSpeakerPosDir(m_vActuallyPosition, m_vActuallyDir);
			
            return;
		}
        
        //pobieramy oryginalne pozycje
        GetPosition( m_vActuallyPosition );
        GetForwardVector( m_vActuallyDir );
        GetUpVector( m_vActuallyUp );
            
        if ( m_bLocalPlayerDriver )
        {
            //dla naszego pojazdu robimy tylko dosymulowanie fizyki
            
            //ustawiamy je jako potencjalne do renderu
            m_vRenderPosition.Set( m_vActuallyPosition );    
            m_vRenderDir.Set( m_vActuallyDir );    
            m_vRenderUp.Set( m_vActuallyUp );    
            //dosymulowujemy fizyke modyfikujac parametry
            AdditionalSimulateOnRender( m_vRenderPosition, m_vRenderDir, null, m_vRenderUp );
            //ustawiamy w nowej pozycji
            FromUpForwardPos( m_vRenderUp, m_vRenderDir, m_vRenderPosition );
            //i zapamietujemy ten fakt
            m_bOnRenderPosition = true;
            m_bOnRenderDir      = true;
            m_bOnRenderUp       = true;
        }
        else
        {
            //pojazdy "obce" bedziemy interpolowac
            
            if ( m_bInterpolateRenderPos )
            {
				if (m_bSim)
				{
					//pobieramy pozycje
					GetPosition( m_vActuallyPosition );
					//zapamietujemy tu
					sm_vPos.Set( m_vActuallyPosition );
					SimulatePos(sm_vPos);
					if (m_bCorrectPos)
						CorrectVehiclePos(m_vActuallyPosition, sm_vPos);
					InterpolateRenderPos( sm_vPos );
					//wynik zapamietujemy jako pozycje do renderu
					m_vRenderPosition.Set( sm_vPos ); 
					//ustawiamy w nowej pozycji
					SetPositionWorld( m_vRenderPosition );
					//i zaznaczmy, zeby w OnPostRender przywrocic m_vActuallyPosition
					m_bOnRenderPosition = true;
				}
				else
				{
					GetPosition(m_vRenderPosition);
					m_vActuallyPosition.Set(m_vRenderPosition);
				}
			}
			else
			{
				GetPosition(m_vRenderPosition);
				m_vActuallyPosition.Set(m_vRenderPosition);
			}

            if ( m_bInterpolateRenderDir )
            {
                //pobieramy faktyczny kierunek
				if (m_bSim)
				{
					GetForwardVector( m_vActuallyDir );
					sm_vDir.Set(m_vActuallyDir);
					//i przekazujemy do wylicznia kierunku na potrzeby renderu
					SimulateDir(sm_vDir);
					
					if (m_vLastInterpolatedDir == null)
					{
						m_vLastInterpolatedDir = new Vector(sm_vDir);
						m_vRenderDir.Set(sm_vDir);
					}
					else
					{
						Interpolate(m_vLastInterpolatedDir, sm_vDir, m_vRenderDir, m_fInterpDirWeight );
						m_vLastInterpolatedDir.Set(m_vRenderDir);
					}
					// i ustawiamy goscia w ten kierunek
					SetDir( m_vRenderDir );
					// i zapamietujemy fakt ze gracz jest obrocony na potrzeby renderingu
					m_bOnRenderDir = true;
				}
				else
				{
					GetForwardVector(m_vRenderDir);
					m_vActuallyDir.Set(m_vRenderDir);
				}
			}
			else
			{
				GetForwardVector(m_vRenderDir);
				m_vActuallyDir.Set(m_vRenderDir);
			}
        }
		UpdateSpeakerPosDir(m_vRenderPosition, m_vRenderDir);
        SetRenderTime(  GetTime() );
    }
	
	Vector m_vLastInterpolatedDir = null;
	Vector vTemp = new Vector();
	Vector m_vLastReplPos = new Vector();
	Vector m_vLastReplDirFromPos = new Vector();
	float _FAKE_PHYSICS_GRAIN = 0.05f;
	Vector m_vLastSimulatedPos = new Vector();
	Vector m_vLastSimulatedDir = new Vector();
	float m_fLastSimulatedTime = 0f;
	float m_fLastSimulatedSpeed = 0f;
	float _ADD_SIM_MAX_TIME = 1f;
	
	void SimulatePos(Vector vPos)
    {
		// sprawdzamy, czy zmienila sie wartosc zmiennej pozycji od ostatniej ramki, zeby wyliczyc kierunek przesuniecia
		if (!m_vLastReplPos.IsEqual(vPos))
		{
			m_vLastReplDirFromPos.Set(vPos);
			m_vLastReplDirFromPos.Sub(m_vLastReplPos);
			m_vLastReplDirFromPos.Normalize();
			m_vLastReplPos.Set(vPos);

			// czas, do ktorego dosymulowalismy
			m_fLastSimulatedTime = GetTime();
			m_vLastSimulatedPos.Set(vPos);
			m_vLastSimulatedDir.Set(m_vLastReplDirFromPos);
			m_fLastSimulatedSpeed = m_fRealSpeed;
		}
		
		float fDeltaTime = GetTime() - m_fReplicationTime;
		
		if (fDeltaTime > 0f)
		{
			if ( (Math.abs(fDeltaTime) < _ADD_SIM_MAX_TIME) && ShouldAddSimulate())
			{
				float fTimeToSimulate = GetTime() - m_fLastSimulatedTime;
				
//				ClearCollFlags(ECollFlags.OBJCOL_FLAG_NOTERRAIN);
				while (fTimeToSimulate >= _FAKE_PHYSICS_GRAIN)
				{
					vPos.Set(FakePhysicsSimulateStep(_FAKE_PHYSICS_GRAIN, true));
					
					fTimeToSimulate -= _FAKE_PHYSICS_GRAIN;
					m_fLastSimulatedTime += _FAKE_PHYSICS_GRAIN;
				}
				
				if (fTimeToSimulate > 0)
				{
					// dosymulowanie koncowki
					vPos.Set(FakePhysicsSimulateStep(fTimeToSimulate, false));
				}
//				SetCollFlags(ECollFlags.OBJCOL_FLAG_NOTERRAIN);
			}
		}
		else
			vPos.Set(m_vLastSimulatedPos);
	}
	
	float _TERRAIN_HEIGHT_DELTA = 50f;
	
	void CorrectVehiclePos(Vector vReplicatedPos, Vector vRenderPos)
	{
		if (vReplicatedPos.fY >= TerrainHeight(vReplicatedPos.fX, vReplicatedPos.fZ) - _TERRAIN_HEIGHT_DELTA)
		{
			float fTerrainHeight = TerrainHeight(vRenderPos.fX, vRenderPos.fZ) - _TERRAIN_HEIGHT_DELTA;
			if (vRenderPos.fY < fTerrainHeight)
				vRenderPos.fY = fTerrainHeight;
		}
	}
	
	Vector vFakeTemp = new Vector();
//	Vector vSaveSimPos = new Vector();
//	Vector vDownMove = new Vector();
//	float m_fAdjustLen = 10f;
									  
	Vector FakePhysicsSimulateStep(float fDeltaTime, boolean bSavePosAndDir)
	{
		vTemp.Set(m_vLastSimulatedDir);
		vTemp = RotateVector(vTemp, m_fAngleSpeed * fDeltaTime, GetUpVector());
		vTemp.Normalize();
		if (bSavePosAndDir)
			m_vLastSimulatedDir.Set(vTemp);
		float fSpeed = m_fLastSimulatedSpeed + m_fRealAccel * fDeltaTime;
		vTemp.Mul(fSpeed * fDeltaTime);

		vFakeTemp.Set(m_vLastSimulatedPos);
		vFakeTemp.Add(vTemp);
		
/*		// zachowujemy pozycje
		GetPosition(vSaveSimPos);
		
		vFakeTemp.Set(m_vLastSimulatedPos);
		vFakeTemp.fY += 100f;
		SetPositionWorld(vFakeTemp);
		
		// opuszczamy samochod na elipsoide
		SetCollAction(ECollFlags.OBJCOL_ACTION_STOP);
		vDownMove.Set(0f, -100f, 0f);
		MoveByEllipsoidPure(vDownMove);
		vFakeTemp.Add(vDownMove);
		SetPositionWorld(vFakeTemp);
		
		// przesuwamy samochod w kierunku jego jazdy
		SetCollAction(ECollFlags.OBJCOL_ACTION_SLIDE);
		MoveByEllipsoidPure(vTemp);
		vFakeTemp.Add(vTemp);
		SetPositionWorld(vFakeTemp);
		
		// sprawdzamy, czy ellipsoida opiera sie na ziemi
		vDownMove.Set(0f, -5f, 0f);
		SetCollAction(ECollFlags.OBJCOL_ACTION_STOP);
		boolean bMoved = MoveByEllipsoidPure(vDownMove);
		SetCollAction(ECollFlags.OBJCOL_ACTION_SLIDE);
		
		// jezeli sie opiera, ustawiamy samochod na terenie trace'ami
		if (!bMoved)
		{
			vDownMove.Set(vFakeTemp);
			vDownMove.fY -= m_fAdjustLen;
			m_Physics.TestTrack(vFakeTemp, vDownMove);
			vDownMove.Set(m_Physics.m_collInfo.vEnd);
			vFakeTemp.fY = vDownMove.fY + m_Physics._CORRECTION;
		}
		
		SetPositionWorld(vSaveSimPos);
*/		
		if (bSavePosAndDir)
		{
			m_vLastSimulatedPos.Set(vFakeTemp);
			m_fLastSimulatedSpeed = fSpeed;
		}
		return vFakeTemp;
	}
	
	boolean ShouldAddSimulate()
	{
		return (Math.abs(m_fCurrSpeed) > 0.01f);
	}

	void SimulateDir(Vector vDir)
	{
		float fDeltaTime = GetTime() - m_fReplicationTime;
		
		if ( (fDeltaTime > 0f) && (Math.abs(fDeltaTime) < 1f) )
			vDir.Set(RotateVector(vDir, m_fAngleSpeed * fDeltaTime, GetUpVector()));
	}
	
    void Interpolate(Vector vBegin, Vector vEnd, Vector vOut, float fWeight )
    {
		vTemp.Set(vBegin);
		vTemp.Mul(1f - fWeight);
		vOut.Set(vEnd);
		vOut.Mul(fWeight);
		vOut.Add(vTemp);
		
//		vOut.Set(vEnd);
    }
	
    void OnPostRender()
    {
        /**
         * Po renderze przywracamy stan obiektu
         * ktory zastalismy w OnRender         * 
         */
        
        
        if ( m_bOnRenderPosition ||
             m_bOnRenderDir      || 
             m_bOnRenderUp )
        {
            // to przypracamy faktyczna pozycje
            FromUpForwardPos( m_vActuallyUp, m_vActuallyDir, m_vActuallyPosition );
            m_bOnRenderPosition = false;
            m_bOnRenderDir      = false;
            m_bOnRenderUp       = false;
        }
    }
    
    void AdditionalSimulateOnRender(Vector vRenderPos,Vector vRenderDir, Vector vRenderRight,Vector vRenderUp)
    {
        //jaka jest roznica czasu pomiedzy czasem aktualnym
        //a czasem symulacji fizyki?
        float fDeltaTime = GetTime() - GetPhysicsLastUpdateTime();

        //jesli sa jakies rozbieznosci, to zrobimy korekte pozycji
        if ( (fDeltaTime > 0.0f) && (Math.abs( fDeltaTime ) < 1.0f) )
        {
            TurnOffPassengersCollisions();
            m_Physics.AdditionalSimulate( m_iAcc, m_fTurn, fDeltaTime, 
                                          vRenderPos, vRenderDir, vRenderRight, vRenderUp );
            TurnOnPassengersCollisions();
        }
        else
        {
            // jezeli czas jest rowny 0 to pobierz aktualne wertosci wektorow
            // potrzebne m.in. kamerze, updateujacej swoja pozycje na podstawie AdditionalSimulation
            GetPosition( vRenderPos );
            GetForwardVector( vRenderDir );
            GetRightVector( vRenderRight );
            GetUpVector( vRenderUp );
        }
    } 
    
    float GetInterpolateSpeed()
    {
        return m_Physics.m_Params.GetMaxSpeed() *  m_fInterpolateSpeedOnMoveMul;
    }
    
    /**
     * Pobranie pozycji pojazdu w renderze
     * wolane z kamery, zatem obsluguje tylko dosymulowanie
     * fizyki bez zadnej interpolacji
     */
    public void GetRenderPosition(Vector vRenderPos, Vector vRenderDir, Vector vRenderRight)
    {
        if (m_bModuleNet && m_bCorrectPosOnRender)
        {
            if (m_bLocalPlayerDriver)
                AdditionalSimulateOnRender( vRenderPos, vRenderDir, vRenderRight, null );
            else
            {
                if ( (m_bInterpolateRenderPos) && (vRenderPos != null) )
                {
                    //pobieramy pozycje
                    GetPosition( vRenderPos );
					SimulatePos(vRenderPos);
					if (m_bCorrectPos)
						CorrectVehiclePos(m_vActuallyPosition, vRenderPos);
                    InterpolateRenderPos( vRenderPos );
                }

                if ( (m_bInterpolateRenderDir) && (vRenderDir != null) )
				{
					Vector vActuallyDir = new Vector();
                    GetForwardVector( vActuallyDir );
					SimulateDir(vActuallyDir);
					if (m_vLastInterpolatedDir != null)
						Interpolate(m_vLastInterpolatedDir, vActuallyDir, vRenderDir, m_fInterpDirWeight );
					else
						vRenderDir.Set(vActuallyDir);
				}
				
                if (vRenderRight != null)
                    GetRightVector( vRenderRight );
            }
        }
        else
            super.GetRenderPosition( vRenderPos, vRenderDir, vRenderRight );
    }
    
    public void OnAudioReinitialize()
    {
        if (m_nSoundLoopId != 0)
        {
            m_nSoundLoopId = 0;
            AttachLoopSound();
        }
            
    }
	
	ControlObject m_cSpeaker = null;
	
    void CreateSpeaker()
    {
		if (m_cSpeaker != null)
		{
            m_cSpeaker.delete();
            m_cSpeaker = null;
		}
		
        m_cSpeaker = (ControlObject) CreateObject("ControlObject");
        m_cSpeaker.SetName("SPEAKER");
        m_cSpeaker.ActivateOnModule();
        m_cSpeaker.ClearFlag(EFlags._COLLISION);
        m_cSpeaker.ClearFlag(EFlags._BLOCK_TRACE);
        m_cSpeaker.ClearFlag(EFlags._BLOCK_MOVE);
    }
    
    void RemoveSpeaker()
    {
        if (m_cSpeaker != null)
        {
			m_cSpeaker.DetachAllSounds();
            m_cSpeaker.delete();
            m_cSpeaker = null;
        }
    }
	
	void UpdateSpeakerPosDir(Vector vPos, Vector vDir)
	{
        if (m_cSpeaker != null)
			m_cSpeaker.FromUpForwardPos(Vector.Up, vDir, vPos);
	}

	void OnDestroy()
	{
		super.OnDestroy();
		
		RemoveSpeaker();
        if ( m_cIndicatorFXPositioner != null )
        {
            m_cIndicatorFXPositioner.delete();
            m_cIndicatorFXPositioner = null;
        }
	}
	
	int AttachSound(String sSoundName) { return m_cSpeaker.AttachSound(sSoundName); }
	int AttachSound(String sSoundName, float fMin, float fMax) { return m_cSpeaker.AttachSound(sSoundName, fMin, fMax); }
	void AttachOneShotSound(String sSoundName) { m_cSpeaker.AttachOneShotSound(sSoundName); }
	void AttachOneShotSound(String sSoundName, float fVolume) { m_cSpeaker.AttachOneShotSound(sSoundName, fVolume); }
	void AttachOneShotSound(String sSoundName, float fMin, float fMax) { m_cSpeaker.AttachOneShotSound(sSoundName, fMin, fMax); }
	void DetachSound(int iSoundID) { m_cSpeaker.DetachSound(iSoundID); }
	void DetachAllSounds() { m_cSpeaker.DetachAllSounds(); }
	void SetSoundVolume(int iSoundID, float fVolume) { m_cSpeaker.SetSoundVolume(iSoundID, fVolume); }
	void SetSoundFreq(int iSoundID, float fFreq) { m_cSpeaker.SetSoundFreq(iSoundID, fFreq); }
	void StopSound(int iSoundID) { m_cSpeaker.StopSound(iSoundID); }
	void PlaySound(int iSoundID) { m_cSpeaker.PlaySound(iSoundID); }
	boolean IsSoundPlaying(int iSoundID) { return m_cSpeaker.IsSoundPlaying(iSoundID); }
    
    /**
     * Globalny licznik zmian stanow obiektow
     * Bedzie on wykorzystywany na klientach do rozstrzygania czy dana
     * akcja (ktora zleca nam serwer) ma sie wykonac czy nie (bo jest stara)
     */
    private static 
    ChangeStateCounter sm_cGlobalVehicleStateCounter = new ChangeStateCounter();
    static void CleanGlobalVehicleStateCounter(){sm_cGlobalVehicleStateCounter.Clean();}
    static int  GetNextGlobalVehicleStateCounter(){ return sm_cGlobalVehicleStateCounter.GetNext(); }
    
    /**
     * Indywidualny licznik zapamietujacy ostatnio
     * odebrana i wykonana akcje na tym obiekcie
     */
    private 
    ChangeStateCounter m_cVehicleStateCounter = new ChangeStateCounter(this);
    boolean VerifyVehicleStateCounter( int nCounter ){ return m_cVehicleStateCounter.Verify( nCounter ); }
    
    
    /**
     * Indykator dla pokazanie _swoich_ pojazdow
     */

    void SetFXX(float fX)
    {
        m_vIndicatorFXPositionerPos.fX = fX;
        UpdateIndicatorFXPositioner();
    }
    
    void SetFXY(float fY)
    {
        m_vIndicatorFXPositionerPos.fY = fY;
        UpdateIndicatorFXPositioner();
    }
    
    void SetFXZ(float fZ)
    {
        m_vIndicatorFXPositionerPos.fZ = fZ;
        UpdateIndicatorFXPositioner();
    }
    
    void ShowFXPos()
    {
        CrashLog(" "+GetNetInfo()+" "+m_vIndicatorFXPositionerPos+"\n");
    }
    
                
    NetSpritePositioner m_cIndicatorFXPositioner = null;

    Vector m_vIndicatorFXPositionerPos = new Vector( 0.0f, 300.0f, 0.0f );
    String m_sIndicatorFX  = null;
    Vector m_vIndicatorPos = new Vector(0,0,0);
    boolean m_bIndicatorActive  = false;
    boolean IsIndicatorActive(){ return m_bIndicatorActive; }
    
    void UpdateIndicatorFXPositioner()
    {
        if ( m_cIndicatorFXPositioner != null )
             m_cIndicatorFXPositioner.SetPosition( m_vIndicatorFXPositionerPos );
    }
    
    void UpdateNickForIndicator()
    {
        String sNick = " ";
        Pawn[] aPawns = GetAllOccupants();
        if ( (aPawns != null) && (aPawns.length>0)) 
        {
            if ( aPawns[ 0 ] instanceof PlayerNet )
            {
                PlayerNet cPlayer = (PlayerNet)aPawns[ 0 ];
                sNick = cPlayer.GetNickForIndicator();
            }
        }
        if ( m_cIndicatorFXPositioner != null )
             m_cIndicatorFXPositioner.SetText( sNick );
    }
            
    
    void StartIndicatorFX(String sFX)
    {
        m_sIndicatorFX      = sFX;
        m_bIndicatorActive  = true;
        UpdateNickForIndicator();
        CallMethodEvery( METHOD_IndicatorFX , (float)((FXPlayerNetIndicator._DURATION-10.0)/1000.0) );
    }
    
    
    
    void IndicatorFX()
    {
        m_cIndicatorFXPositioner.ExecuteFX( m_cIndicatorFXPositioner, 
                                            m_sIndicatorFX , 
                                            m_vIndicatorPos );
    }
    
    void SpokenFX()
    {
            ExecuteFX( m_cIndicatorFXPositioner, 
                       m_sIndicatorFX+"Spoken" , 
                       new Vector(  m_vIndicatorPos.fX, 
                                    m_vIndicatorPos.fY+FXPlayerNetIndicator._SPOKEN_ADD_HEIGHT,
                                    m_vIndicatorPos.fZ )
                      );
    }
    

    
    void StopIndicatorFX()
    {
        m_bIndicatorActive  = false;
        DisableCallMethod( METHOD_IndicatorFX );
    }

}                      

    

