public class GroundVehicle extends ChromeActor
{
	float _MAX_SPEED = 2000f;
	float _MIN_SPEED = -500f;
	float _ACCEL = 500f;
	float _DEACCEL = 1000f;
	float _ACCEL_SUPRESS = 200f;
	float _MAX_TURN_SPEED = 70f;
	float _TURN_ACCEL = 70f;
	float _TURN_ACCEL_SUPRESS = 200f;
	
	float _HALF_WHEEL_SIZE = 0;
	float _MAX_WHEEL_POS = 70f;
	float _MIN_WHEEL_POS = -20f;
	float _SPRING_MOVE_SPD = 500f;
	
	float _NORMAL_ALIGN_SPEED = 60f;
	
	float m_fCurrSpeed = 0;
	float m_fCurrTurnSpeed = 0;
	Vector m_vCurrSpeedDir = null;
	
	VehicleController m_Controller = null;

	int m_aHelpers[] = null;
	int m_aWheels[] = null;
	int m_aTurnWheels[] = null;
	float m_aSpringsLen[] = null;
	float m_aNewSpringsLen[] = null;

	public void OnCreate()
	{
		super.OnCreate();
		
        SetPhysicsType(EPhysics._NEWFLY);
		SetCollType(ECollFlags.OBJCOL_TYPE_PERFACE);
        SetPhysicsActive(true);
        SetSpeed(new Vector(0, 0, 1));
        SetSpeedValue(0);
		SetMaxClimbAngle(90);
		SetEllipsoidNoExtents(100, 300, 150);
		SetMass(40f);
		SetCollAction(ECollFlags.OBJCOL_ACTION_SLIDE);

		SetupHelpers();
		SetupWheels();
		
		m_Controller = (VehicleController) CreateObject("VehicleController");
		m_Controller.SetVehicle(this);
		m_Controller.ActivateOnModule();
		
		SetupTemp();
	}
	
	/** Metoda wypelniajaca tablice m_aHelpers. Nalezy pokryc w klasach
	 * pochodnych (dla pojazdow o innych meshach).
	 */
	public void SetupHelpers()
	{
		m_aHelpers = new int[4];
		
        m_aHelpers[0] = GetElementID("kolobase_PP");
		m_aHelpers[1] = GetElementID("kolobase_LP");
		m_aHelpers[2] = GetElementID("kolobase_LT");
		m_aHelpers[3] = GetElementID("kolobase_PT");
	}
	
	/** Metoda wypelniajaca tablice m_aWheels. Nalezy pokryc w klasach
	 * pochodnych (dla pojazdow o innych meshach).
	 */
	public void SetupWheels()
	{
		m_aWheels = new int[4];
		
		m_aWheels[0] = GetElementID("kolo_PP");
		m_aWheels[1] = GetElementID("kolo_LP");
		m_aWheels[2] = GetElementID("kolo_LT");
        m_aWheels[3] = GetElementID("kolo_PT");
		
        if (m_aWheels[0] != -1)
			_HALF_WHEEL_SIZE = GetElementPos(m_aWheels[0]).fY - GetPosition().fY;
		
		m_aSpringsLen = new float[4];
		m_aNewSpringsLen = new float[4];
	}
	
	public void SetupTemp()
	{
		m_vStart = new Vector();
		m_vEnd = new Vector();
		m_Coll = new CollReturn();
		m_vSuspensionDir = new Vector();
		m_vVehicleNormal = new Vector(0, 1f, 0);
		m_vNewVehicleNormal = new Vector(0, 1f, 0);
	}
	
	void UpdateNormalSpringsWheels(float fDeltaT)
	{
		CalculateNormalSprings();	// obliczamy docelowe w danej chwili ustawienie sprezyn dla kazdego kola pojazdu
		UpdateNormal(fDeltaT);
//		UpdateSprings(fDeltaT);		// modyfikujemy ustawienie sprezyn zgodnie z uplywem czasu
//		UpdateWheels(fDeltaT);		// obracamy i skrecamy kola
	}

	Vector m_vVehicleNormal = null;
	Vector m_vNewVehicleNormal = null;
	Vector m_vSuspensionDir = null;
	Vector m_vStart = null;
	Vector m_vEnd = null;
	CollReturn m_Coll = null;

	void CalculateNormalSprings()
	{
		// jezeli nie ma helperow, nie mozemy obliczyc normalnej ani ustawien kol
		if (m_aHelpers == null)
			return;
		
		m_vSuspensionDir.Set(m_vVehicleNormal);
		m_vSuspensionDir.Negate();
		m_vSuspensionDir.Mul(2f * _HALF_WHEEL_SIZE + _MAX_WHEEL_POS - _MIN_WHEEL_POS);

		CollReturn collReturn = null;
		m_vNewVehicleNormal.Set(Vector.Zero);

		// trace z kol
		for (int i = 0; i < m_aHelpers.length; i++)
		{
			m_vStart.Set(m_vVehicleNormal);
			m_vStart.Mul(-_MIN_WHEEL_POS + _HALF_WHEEL_SIZE);
			m_vStart.Add(GetElementPos(m_aHelpers[i]));
			m_vEnd.Set(m_vStart);
			m_vEnd.Add(m_vSuspensionDir);
			
			collReturn = TestTrack(m_vStart, m_vEnd);
			
			if (collReturn != null)
			{
				m_vNewVehicleNormal.Add(collReturn.m_vNormal);
				if (collReturn.m_fDistance < 2 * _HALF_WHEEL_SIZE)
					m_aNewSpringsLen[i] = _MIN_WHEEL_POS;
				else
					m_aNewSpringsLen[i] = collReturn.m_fDistance - 2f * _HALF_WHEEL_SIZE + _MIN_WHEEL_POS;
			}
			else
				m_aNewSpringsLen[i] = _MAX_WHEEL_POS;
		}
		
		// obliczamy nowa normalna pojazdu
		if ( m_vNewVehicleNormal.Len2() > 0.01f )
			m_vNewVehicleNormal.Normalize();
		else
			m_vNewVehicleNormal.Set(Vector.Up);
	}
	
	void UpdateNormal(float fDeltaT)
	{
		float fDot = m_vVehicleNormal.Dot(m_vNewVehicleNormal);
		if (fDot > 0.99999f)
			return;
		
		Vector m_NormalsCross = m_vVehicleNormal.Cross(m_vNewVehicleNormal);
		m_NormalsCross.Normalize();
		
		float fAngleBtwNormals = (float) java.lang.Math.acos(fDot);
		float fAngleToTurn = _NORMAL_ALIGN_SPEED * fDeltaT;
		if (fAngleToTurn > fAngleBtwNormals)
			fAngleToTurn = fAngleBtwNormals;
		m_vVehicleNormal = RotateVector(m_vVehicleNormal, fAngleToTurn, m_NormalsCross);
		
		FromUpForwardPos(m_vVehicleNormal, GetForwardVector(), GetPosition());
	}
	
	void UpdateSprings(float fDeltaT)
	{
		for (int i = 0; i < m_aWheels.length; i++)
		{
			if (m_aSpringsLen[i] < m_aNewSpringsLen[i])
			{
				m_aSpringsLen[i] += _SPRING_MOVE_SPD * fDeltaT;
				if (m_aSpringsLen[i] > m_aNewSpringsLen[i])
					m_aSpringsLen[i] = m_aNewSpringsLen[i];
			}
			else
			{
				m_aSpringsLen[i] -= _SPRING_MOVE_SPD * fDeltaT;
				if (m_aSpringsLen[i] < m_aNewSpringsLen[i])
					m_aSpringsLen[i] = m_aNewSpringsLen[i];
			}
		}
	}

	CollReturn TestTrack(Vector vStart, Vector vEnd)
	{
		boolean bCollided = false;
		
		Collision coll = TraceTerrain(vStart, vEnd);
		if (coll != null)
		{
			bCollided = true;
			m_Coll.m_vNormal.Set(coll.vNormal);
			m_Coll.m_vEnd.Set(vEnd);
			m_Coll.m_fDistance = coll.fDistance;
			m_Coll.m_vNormal.Set(coll.vNormal);
		}
		
		coll = TraceObjectsExclude(vStart, vEnd, this);
		if (coll != null)
		{
			m_Coll.m_vNormal.Set(coll.vNormal);
			m_Coll.m_vEnd.Set(vEnd);
			m_Coll.m_fDistance = coll.fDistance;
			m_Coll.m_vNormal.Set(coll.vNormal);
		}
		
		if (!bCollided)
			return null;
		else
			return m_Coll;
	}
}

/** klasa przechowujaca dane o kolizji
 */
class CollReturn
{
	Vector m_vEnd = new Vector();
	Object m_Obj = null;
	float m_fDistance = 0;
	Vector m_vNormal = new Vector();
}