public class Walker extends Vehicle
{
    //////////////////////////////////////////////////
    // Indeksy do tablicy animacji
    /** Poruszanie sie */
    public final static int A_STAND                 = 0;
    public final static int A_FORWARD               = 1;
    public final static int A_BACKWARD              = 2;
    public final static int A_ROTATE_LEFT           = 3;
    public final static int A_ROTATE_RIGHT          = 4;
    
    /** Zmiana stanu */
    public final static int A_NORMAL2STANDBY        = 5;
    public final static int A_STANDBY               = 6;
    public final static int A_STANDBY2STANDBY_OPEN  = 7;
    public final static int A_STANDBY_OPEN          = 8;
    public final static int A_STANDBY_OPEN2STANDBY  = 9;
    public final static int A_WAKEUP                = 10;
    
    public final static int A_NORMAL2UP             = 11;
    public final static int A_UP                    = 12;
    public final static int A_UP2NORMAL             = 13;
    public final static int A_NORMAL2DOWN           = 14;
    public final static int A_DOWN                  = 15;
    public final static int A_DOWN2NORMAL           = 16;
    //ilosc animacji
    public final static int A_CNT                   = 17;
    
    /** Tablica dostepnych animacji */
    protected int[] m_aAnims = new int[A_CNT];
    
    PhysicsWalker m_cPhysicsWalker = null;

    /** Lista broni walkera */
    WeaponWalkerList m_aWeapons = new WeaponWalkerList( this );
    
    /**
     * Mesh elementy
     */
    /** Element do obracania glowa w pionie*/
    String m_sVertRotateElem  = "Walker Neck";
    int m_nVertRotateElem = -1;
    
    /** Element do obracania glowa w poziomie*/
    String m_sHorizRotateElem = "Walker Head";
    int m_nHorizRotateElem = -1;
    
    /** Element do celowania w walkera */
    String m_sMeshElementToLookAt = "Walker Cockpit";
	int m_nElemPointToLookAt = -1;
    
    /** wielkosc elipsoidy*/
    Vector m_vEllipsoid = new Vector( 180.0f, 450.0f, 225.0f );
        
    
    /**
     * Dzwieki
     * 
     */
    /** Dzwieki dla poszczegolnych animacji walkera - tablica indeksowana
     *  intem o wartosci AIAnimationsWalker.A_* (pozycja == null dla animacji bez dzwieku)
     */
    String[] m_aWalkerAnimsSounds = null;

    /** Dzwiek krokow walkera  */
    String m_sStepSound = null;
    
    /** Dzwiek do odpalania w loopie przy obracaniu 'glowy' walkera  */
    String m_sHeadLoopSound = null;
    /** ID odpalonego dzwieku m_sHeadLoopSound */
    protected int m_nHeadRotateSoundID = -1;


    /**
     * katy pod ktorym moze patrzec sie kamera
     */
    //na boki
    float m_fMaxHAngle =  15.0f;//#Walker
    //w dol
    float m_fMaxVAngle =  20.0f;//#Walker
    //pod gore
    float m_fMinVAngle = -10.0f;//#Walker
    
    /**
     * Metody do zarejestrowania
     * 
     */
    int METHOD_EnableWalk           = 0;
    int METHOD_StopHeadRotateSound  = 0;
    int METHOD_StopGunRotateSound   = 0;
    int METHOD_PlayStepSound        = 0;
    int METHOD_CheckWeaponsToApply  = 0;

    
    /** 
     * Smierc walkera 
     * 
     */
    /** FX odpalany po rozwaleniu Walkera */
    String m_sDieBlowFX = "FXBlowWalker";
    /** Dzwiek odpalany po rozwalenio Walkera */
    String m_sDieBlowSound = "Data/Sounds/Explosion/WalkerDieExplosion.wav";
    /** Tablica nazw meshelementow glowy walkera (do ukrycia po smierci) */
    String[] m_aHeadElements = null;

    
    /**
     * Predkosci poruszania sie
     */
    //predkosc chodzenie do przodu    
    float m_fWalkSpeed     = 210.0f;
    //predkosc chodzenie do tylu    
    float m_fWalkSpeedBack = 130.0f;
    //predkosc obracania sie (stopnie na sekunde)
    float m_fRotateSpeed   = 30.0f;
    
    //Droga przebywana w jednej klatce animacji kroku walkera
    float m_fWalkAnimSpeed = 1.0f;
    //wspolczynnik pomiedzy wyliczona predkoscia animacji a to co trzeba zastosowac
    float m_fWalkAnimSpeedMul = 1.0f;
    
    Walker()
    {
        MeshName = "Data/Meshes/Ai/Walker_w.3da";
                
        fHealth    = 
        fMaxHealth = 3500.f;
        bForceWholeObjectCollision = true;
        
        //fBlobMapSize = 220f;
	    //sBlobMaterial = "blobs_jeep.mat";
        
		// dzwieki TODO
        m_sSoundStartName = null;
        m_sSoundLoopName  = null;
        m_sSoundStopName  = null;
                
        m_sSoundAccelName = null;
		m_sSoundBrakeName = null;
        m_sSoundSlideName = null;
	}

	void OnCreate()
    {
        METHOD_EnableWalk           = RegisterCallMethod("EnableWalk"); 
        METHOD_StopHeadRotateSound  = RegisterCallMethod("StopHeadRotateSound");
        METHOD_StopGunRotateSound   = RegisterCallMethod("StopGunRotateSound");
        METHOD_PlayStepSound        = RegisterCallMethod("PlayStepSound");
        METHOD_CheckWeaponsToApply  = RegisterCallMethod("CheckWeaponsToApply");

		bSG = true;
		
        super.OnCreate();
        
		if (m_Physics != null)
			UpdatePathMap( true, 7 );
		
        SetFlag(EFlags._CAST_SHADOWS);
        SetBlobMaterial(sBlobMaterial);
        SetBlobMapSize(fBlobMapSize);

        SetTraceCollType(ECollTraceFlags._HULL);
        SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        ClearFlag(EFlags._NO_ANIMATION);
        SetFlag(EFlags._CAST_SHADOWS | EFlags._BLOCK_TRACE);        
        
        CreateWeapons();
        InitAnimations();
        InitMesh();
        
        m_fDownTestVectorLen = -500f;
        
        SetMaxClimbAngle(60f);
    }
    
    void OnDestroy()
    {
        super.OnDestroy();
        m_aWeapons.Destroy();
    }

    public boolean IsArmed()
    {
        return true;
    }
    
    void AttachLoopSound()
    {
        super.AttachLoopSound();
        m_bEngineOn = true;             
    }
    
    void DetachLoopSound()
    {
        super.DetachLoopSound();
        m_bEngineOn = false;            
    }
    
    public class CrashedState extends Vehicle.CrashedState
    {
        public void OnEnter()
        {
            SetPhysicsVehicleActive(false);
                        
            if ( m_sDieBlowFX != null )
            {
                ExecuteFX( m_sDieBlowFX, GetElementPos( m_nHorizRotateElem ), Vector.Up );
                ExecuteParticleSystem( m_sDieBlowFX, GetElementPos( m_nHorizRotateElem ), Vector.Up);
            }

            if ( m_sDieBlowSound != null)
                PlaySound( m_sDieBlowSound );
                
            if (fBlowUpShakeStrength > 0)
                ShakePlayersCamera(fBlowUpShakeStrength, fBlowUpShakeDistMin, fBlowUpShakeDistMax);

            HideHead();
            PlayAnimOnce( A_STANDBY );
            
            DeleteAttachedFXes();
            DisableCallMethods();
        }
		
		public void SGUpdate()
		{
			FileChunk.Log("[Walker$CrashedState] : SGUpdate\n");
			
			SetPhysicsVehicleActive(false);
			
			HideHead();
			
            DeleteAttachedFXes();
            DisableCallMethods();			
		}
    }
    
    void Respawn( boolean bOnLevelRestart )
    {
        super.Respawn( bOnLevelRestart );
        UnhideHead();
    }

    
    void CreatePhysics()
	{
        m_Physics        = 
        m_cPhysicsWalker = new PhysicsWalker( this );
         
        m_Physics.Init();
        
        m_Physics.sWheelSmokeFX = null;
        m_Physics.bWheelSmokeFX = false;
        
        //kola nie beda skrecaly :-)
        m_Physics.m_Params._TURNING_SPEED       = 0.0f;
        
        //ale walker bedzie sie obracal
        m_Physics.m_Params._ROTATE_SPEED        = m_fRotateSpeed;			
        
        m_Physics.m_Params.SetMaxSpeed(m_fWalkSpeed);
        m_Physics.m_Params._MIN_SPEED_TO_COLLIDE=  m_fWalkSpeed + 1;//nie dostajemy obrazen po kolizji
	    m_Physics.m_Params._CURR_MAX_SPEED      =  m_fWalkSpeed;		
	    m_Physics.m_Params._MIN_SPEED           =  -m_fWalkSpeedBack;	
        
        //to nam zapewni ze w Vehicle.Java nie zostana wpisane
        //predkosci m_aSpeeds[]
        m_bUseMotionSpeed = false;
	}

    
    void CreateWeapons()
    {
        //Do nadpisania
    }
    
    /**
     * Metoda tworzy obiekt broni a nastepnie wola
     * inna metode, ktora podpina ta bron do walkera
     * Obslugiwane tylko na serwerze
     */
    void CreateNewWeapon( Class   cWeaponClass, 
                          String  sWalkerElementBarrelExit, 
                          int     nMode )
    {
        if ( cWeaponClass != null && Role == ERole._AUTHORITY )
        {
            Weapon cWeapon = (Weapon)CreateObject( cWeaponClass.getName() );
            ApplyNewWeapon( cWeapon, sWalkerElementBarrelExit, nMode );
        }
    }
    
    /**
     * Podpiecie broni do walkera
     * Obslugiwane
     * - na serwerze bezposrednio po stworzeniu broni
     * - na klientach po zreplikowaniu walkera i broni
     */
    void ApplyNewWeapon( Weapon cWeapon, String sWalkerElementBarrelExit, int nMode )
    {
        AttachChild( cWeapon, GetElementID( sWalkerElementBarrelExit ) );
        cWeapon.SetWalkerMode( this );
        m_aWeapons.Add( new WeaponWalker( cWeapon, nMode, sWalkerElementBarrelExit ) );
    }
    
    
    void InitAnimations()
    {
        String sRootElementName = "";
        m_aAnims[A_STAND]                = GetAnimID( sRootElementName, "normal" );
        m_aAnims[A_FORWARD]              = GetAnimID( sRootElementName, "walk" );
        m_aAnims[A_BACKWARD]             = GetAnimID( sRootElementName, "walk_back" );
        m_aAnims[A_ROTATE_LEFT]          = GetAnimID( sRootElementName, "rotate_left" );
        m_aAnims[A_ROTATE_RIGHT]         = GetAnimID( sRootElementName, "rotate_right" );
        m_aAnims[A_NORMAL2STANDBY]       = GetAnimID( sRootElementName, "normal-standby" );
        m_aAnims[A_STANDBY]              = GetAnimID( sRootElementName, "Standby" );
        m_aAnims[A_STANDBY2STANDBY_OPEN] = GetAnimID( sRootElementName, "standby-standby_open" );
        m_aAnims[A_STANDBY_OPEN]         = GetAnimID( sRootElementName, "standby_open" );
        m_aAnims[A_STANDBY_OPEN2STANDBY] = GetAnimID( sRootElementName, "Standby_open-standby" );
        m_aAnims[A_WAKEUP]               = GetAnimID( sRootElementName, "Wakeup" );
        m_aAnims[A_NORMAL2UP]            = GetAnimID( sRootElementName, "normal-up" );
        m_aAnims[A_UP]                   = GetAnimID( sRootElementName, "up" );
        m_aAnims[A_UP2NORMAL]            = GetAnimID( sRootElementName, "up-normal" );
        m_aAnims[A_NORMAL2DOWN]          = GetAnimID( sRootElementName, "normal-down" );
        m_aAnims[A_DOWN]                 = GetAnimID( sRootElementName, "down" );
        m_aAnims[A_DOWN2NORMAL]          = GetAnimID( sRootElementName, "down-normal" );
        
        CalcWalkAnimSpeed("Walker R Foot", "Walker L Foot");
        
        SetFrame( m_aAnims[ A_STANDBY ], GetFirstFrame( m_aAnims[ A_STANDBY ] ), -1 );
        SetAnimMorphTime( 0.5f );
    }
    
    /** Obliczenie drogi przebywanej w jednej klatce animacji chodzenia
     * i ustawienie predkosci animacji A_FORWARD i A_BACKWARD na podstawie 
     * zdefiniowanej predkosci walkera m_fWalkSpeed i m_fWalkSpeedBack
     * @param sFoot1 nazwa elementu stopy #1 
     * @param sFoot2 nazwa elementu stopy #2
     */
    protected void CalcWalkAnimSpeed(String sFoot1, String sFoot2)
    {
        int nFoot1 = GetElementID( sFoot1 ),
            nFoot2 = GetElementID( sFoot2 );
        int nAnimForwardId  = m_aAnims[ A_FORWARD  ];
        // ustaw pierwsza klatke animacji chodzenia
        SetFrame( nAnimForwardId, GetFirstFrame(nAnimForwardId), -1 );
        // pobierz pozycje 'stop'
        Vector vDist = GetElementPos( nFoot1 );
        vDist.Sub( GetElementPos( nFoot2 ) );
        // zmien na uklad lokalny -> fZ bedzie przebywana droga
        vDist = VectorWorldToLocal(vDist);
        float fDist = (float)Math.abs(vDist.fZ);
        m_fWalkAnimSpeed = 2 * fDist / (GetAnimLength( nAnimForwardId ) * 
                                        GetAnimFreq(   nAnimForwardId ) );
        UpdateWalkAnimsSpeed();
    }
        
    void UpdateWalkAnimsSpeed()
    {
        //liczymy predkosc animacji A_FORWARD
        SetAnimFreq( m_aAnims[ A_FORWARD  ], m_fWalkAnimSpeedMul * m_fWalkSpeed / m_fWalkAnimSpeed );
        //liczymy predkosc animacji A_BACKWARD
        SetAnimFreq( m_aAnims[ A_BACKWARD ], m_fWalkAnimSpeedMul * m_fWalkSpeedBack / m_fWalkAnimSpeed );
    }
           
    void InitMesh()
    {
        m_nVertRotateElem    = GetElementID( m_sVertRotateElem  );
        m_nHorizRotateElem   = GetElementID( m_sHorizRotateElem );
        m_nElemPointToLookAt = GetElementID( m_sMeshElementToLookAt );
        
        SetElementAnimType( m_nHorizRotateElem, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType( m_nVertRotateElem,  (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
                
        SetAnimMorphTime( m_nHorizRotateElem, 0.3f, false );
        SetAnimMorphTime( m_nVertRotateElem,  0.3f, false );
        
        SetEllipsoidNoExtents( m_vEllipsoid.fX, m_vEllipsoid.fY, m_vEllipsoid.fZ );
    }
    
    void PlayAnimOnce( int nAnim )
    {
        PlayAnim( m_aAnims[ nAnim ], 1 );
        PlayAnimSound( nAnim );
        DisableCallMethod( METHOD_PlayStepSound );
        DisableCallMethod( "PlayAnimLater" );
    }
    
    
    void PlayAnimLoop( int nAnim, boolean bStepAnimation )
    {
        if ( Role == ERole._AUTHORITY || m_bLocalPlayerDriver )
            MsgSendAnim( nAnim, bStepAnimation );
            
        PlayAnim( m_aAnims[ nAnim ], -1 );
        PlayAnimSound( nAnim );
        if ( bStepAnimation )
        {
            m_nLastStepLoopAnimId =  m_aAnims[ nAnim ];
            //zawolajmy recznie tak jakby skonczyl sie cykl
            //do odpalanie dzwiekow chodzenia
            OnAnimCycle( m_nLastStepLoopAnimId, -1 );
        }
        else
        {
            DisableCallMethod( METHOD_PlayStepSound );
            m_nLastStepLoopAnimId = -1;
        }
        DisableCallMethod( "PlayAnimLater" );
    }
    
    int m_nAnimToPlay = -1;
    void PlayAnimLater( int nAnim, float fDelay )
    {
        m_nAnimToPlay = nAnim;
        DisableCallMethod( "PlayAnimLater" );
        CallMethodIn( "PlayAnimLater", fDelay );
    }
    
    void PlayAnimLater()
    {
        DisableCallMethod("PlayAnimLater");
        PlayAnimOnce( m_nAnimToPlay );
    }
    
    int m_nLastStepLoopAnimId = -1;

    void OnAnimCycle(int nAnimId, int nCount)
    {
        DisableCallMethod( METHOD_PlayStepSound );
        if ( nAnimId == m_nLastStepLoopAnimId )
        {
            //odpalamy dzwiek na rozpoczecie kroku
            //TODO: mozna sie zastanowic czy odpalac pierwszy dzwiek 
            //po rozpoczeciu chodzenia ( dla niego nCount == -1 ) 
            //bowiem bo w przypadku szybkiego wciskania i puszczania 
            //klawisza bedzie sie on odpalal czesto
            PlayStepSound();
            //to zeby sie odegral dzwiek w polowie kroku
            CallMethodIn( METHOD_PlayStepSound, 
                          GetAnimLength( nAnimId ) / 2 );
        }
    }
    
    void PlayAnimSound( int nAnim )
    {
        if (m_aWalkerAnimsSounds != null && 
            m_aWalkerAnimsSounds[ nAnim ] != null)
        {
            PlaySound( m_aWalkerAnimsSounds[ nAnim ] );
        }
    }
    
    void PlayStepSound()
    {
        if (m_sStepSound != null)
            PlaySound( m_sStepSound );
    }
    
    void PlayHeadRotateSound(float fTime)
    {
        if (m_nHeadRotateSoundID < 0)
        {            
            if (m_sHeadLoopSound != null)
                m_nHeadRotateSoundID = AttachSound(m_sHeadLoopSound);
        }
        else
            PlaySound(m_nHeadRotateSoundID);
        DisableCallMethod( METHOD_StopHeadRotateSound );
        CallMethodIn( METHOD_StopHeadRotateSound, fTime );
    }
    
    void StopHeadRotateSound()
    {
        if (m_nHeadRotateSoundID > 0)
            StopSound(m_nHeadRotateSoundID);
    }
    
    void HideHead()
    {
        if ( m_aHeadElements == null )
            return;
        for (int i = 0; i < m_aHeadElements.length; i++)
        {
            int nId = GetElementID( m_aHeadElements[ i ] );
            HideElement( nId );
            EnableCollisionsElement(nId, false);
        }
    }
    
    void UnhideHead()
    {
        if ( m_aHeadElements == null )
            return;
        for (int i = 0; i < m_aHeadElements.length; i++)
        {
            int nId = GetElementID( m_aHeadElements[ i ] );
            UnhideElement( nId );
            EnableCollisionsElement(nId, true);
        }
    }
    
    public void PawnGotIn(SeatInfo info, Pawn pawn)
	{
		super.PawnGotIn(info, pawn);
        
        PlayAnimOnce( A_WAKEUP );
        //TODO: jakis czas na odpalenie wszystkich animacji
        CallMethodIn( METHOD_EnableWalk, 2.0f );
        
        m_aWeapons.SetWeaponsOwner( pawn );
	}
   
    public void PawnGotOut(SeatInfo info, Pawn pawn)
	{
		super.PawnGotOut( info, pawn );

        m_aWeapons.ForceStopFire();
        m_aWeapons.SetWeaponsOwner( null );
        
        DisableWalk();
        RotateHead( 0.0f, 0.0f );
        PlayAnimOnce( A_NORMAL2STANDBY );
        //PlayAnimLater( A_NORMAL2STANDBY, 2.0f );	
    }
    
    void EnableWalk()
    {
        m_cPhysicsWalker.EnableWalk( true );
    }
    
    void DisableWalk()
    {
        DisableCallMethod( METHOD_EnableWalk );
        m_cPhysicsWalker.EnableWalk( false );
		m_fCurrSpeed = 0f;
    }
        
    void SetFire(int nMode,boolean bFire)
    {
        m_aWeapons.SetFire( nMode, bFire );
    }
        
        
    float GetMaxHorizAngle(){ return m_fMaxHAngle; };
    float GetMinVertAngle(){ return m_fMinVAngle; };
    float GetMaxVertAngle(){ return m_fMaxVAngle; };
            
    Vector m_vTargetPoint = new Vector();
    //dlatego tu robie Set() bo na niektorych hostach prawdziwy kierunek
    //bedzie przechowywany tylko w zmiennych replikowanych m_fTarget?
    Vector GetTargetPoint(){ m_vTargetPoint.Set(m_fTargetX,m_fTargetY,m_fTargetZ); return m_vTargetPoint; }
    
    //tez zmienne bedziemy replikowac na pozostalych ludkow
    float m_fTargetX = 0.0f, 
          m_fTargetY = 0.0f, 
          m_fTargetZ = 0.0f;
                                         
    public void SetTargetPoint(Vector vTarget)
    {
        m_vTargetPoint.Set( vTarget );
        
        //to sobie tak zapisujemy zeby zreplikowac do klientow
        m_fTargetX = vTarget.fX;
        m_fTargetY = vTarget.fY;
        m_fTargetZ = vTarget.fZ;
    }
    
    public float m_fMinVertHeadAngle = -25;
    public float m_fMaxVertHeadAngle = 16;
    public float m_fMaxHorizHeadAngle = 40;

    public float m_fHorizVertAddAngle = -20;
    
    float m_fLastHorizAngle = 0.0f;
    float m_fLastVertAngle  = 0.0f;
    float m_fHorizAngle = 0.0f;
    float m_fVertAngle  = 0.0f;
    
    //mnoznik obtotu glowa w pionie
    float fVertMul = 0.5f;

    public void RotateHead(float fHorizAngle, float fVertAngle)
    {
        fVertAngle *= fVertMul;
        
        // obetnij wartosci do tych 'bezpiecznych' ze wzgledu na ograniczenia modelu
        fHorizAngle = MathUtils.crop( fHorizAngle, -m_fMaxHorizHeadAngle, m_fMaxHorizHeadAngle );
        fVertAngle  = MathUtils.crop( fVertAngle,   m_fMinVertHeadAngle,  m_fMaxVertHeadAngle  );

        //RotateElementWithAnim( m_nVertRotateElem,  Vector.Right, fVertAngle  );
        //RotateElementWithAnim( m_nHorizRotateElem, Vector.Up,    fHorizAngle );

        //od tej chwili to jest obowiazujace ustawienie
        m_fHorizAngle = fHorizAngle;
        m_fVertAngle  = fVertAngle;
    }
    
    public void RotateHeadDo(float fHorizAngle, float fVertAngle)
    {
        RotateElementWithAnim( m_nVertRotateElem,  Vector.Right, fVertAngle  );
        RotateElementWithAnim( m_nHorizRotateElem, Vector.Up,    fHorizAngle );
    }
    
    public void OnPostUpdate()
    {
        float fDelta = Math.abs(m_fHorizAngle - m_fLastHorizAngle) +
                       Math.abs(m_fVertAngle -  m_fLastVertAngle);
        //LogR("OnPostUpdate "+fDelta+" "+m_fHorizAngle+" "+m_fVertAngle+" "+
         //    m_fLastHorizAngle+" "+m_fLastVertAngle+
         //    "\n");
        if ( fDelta > 0.0f )
        {
            RotateHeadDo( m_fHorizAngle, m_fVertAngle );
            /** Odpalenie dzwieku obrotu glowa
             */
            if ( fDelta > 3.0f )
            {
                PlayHeadRotateSound((float)Math.max(GetAnimMorphTime(m_nHorizRotateElem),
                                                    GetAnimMorphTime(m_nVertRotateElem)));
            }
            m_fLastHorizAngle = m_fHorizAngle;
            m_fLastVertAngle  = m_fVertAngle;
        }
        
        if ( m_bPrimaryWeaponLastFiring != m_bPrimaryWeaponFiring )
        {
            SetFire( WeaponWalker._MODE_PRIMARY, m_bPrimaryWeaponFiring );
            m_bPrimaryWeaponLastFiring = m_bPrimaryWeaponFiring;
        }
    
        if ( m_bSecondaryWeaponLastFiring != m_bSecondaryWeaponFiring )
        {
            SetFire( WeaponWalker._MODE_SECONDARY, m_bSecondaryWeaponFiring );
            m_bSecondaryWeaponLastFiring = m_bSecondaryWeaponFiring;
        }
    }
        
    void DriverIsDead()
	{
	}
	
	void PassengerIsDead()
	{
	}
	    
    public Vector GetPointToLookAt(int nPoint)
    {
		return GetElementPos( m_nElemPointToLookAt );
    }

    float GetMainGunAccuracy()
    {
        if (m_aWeapons != null)
        {
            for (int i = 0; i < m_aWeapons.GetSize(); i++)
            {
                WeaponWalker cW = m_aWeapons.Get(i);
                if (cW.GetMode() == WeaponWalker._MODE_PRIMARY)
                    return cW.m_cWeapon.fBaseAcc;
            }
            return 1;
        }
        else
        {
            return 0;
        }
    }
    
    /**
     * to nalezy zawolac przy kazdej zmianie predkosci chodzenia walkera.
     */
    void SpeedChanged()
    {
        m_Physics.m_Params._ROTATE_SPEED   = m_fRotateSpeed;			
        m_Physics.m_Params.SetMaxSpeed(m_fWalkSpeed);
        m_Physics.m_Params._MIN_SPEED_TO_COLLIDE = m_fWalkSpeed + 1;//nie dostajemy obrazen po kolizji
	    m_Physics.m_Params._CURR_MAX_SPEED =  m_fWalkSpeed;		
	    m_Physics.m_Params._MIN_SPEED      =  -m_fWalkSpeedBack;        
        UpdateWalkAnimsSpeed();
    }

    float _GET_OUT_DOWN_CORRECTION = -200f;
    
    public Vector GetCheckedPosToGetOut(SeatInfo info)
    {
		if (info.GetOccupant() == null)
			return info.GetStartPos();

		int iVehicleFlags = GetFlags();
        ClearFlag(EFlags._BLOCK_MOVE);

		int iPassengerFlags = info.GetOccupant().GetFlags();
        info.GetOccupant().ClearFlag(EFlags._BLOCK_MOVE);
        
        Vector vPos = null;
        
        // probujemy wysiasc po ekstentach
        Vector vExtents[] = { new Vector(), new Vector(), new Vector(), new Vector() };
        GetMeshExtents(vExtents[1], vExtents[0]);
        
        vExtents[2].Set(vExtents[1]);
        vExtents[2].fZ = (vExtents[0].fZ + vExtents[1].fZ) / 2f;
        vExtents[2].fX -= _CORRECTION;
        
        vExtents[3].Set(vExtents[0]);
        vExtents[3].fZ = vExtents[2].fZ;
        vExtents[3].fX += _CORRECTION;
        
        vExtents[1].fX = (vExtents[3].fX + vExtents[2].fX) / 2f;
        vExtents[1].fZ -= _CORRECTION;
        
        vExtents[0].fX = vExtents[1].fX;
        vExtents[0].fZ += _CORRECTION;
        
        for (int i = 0; i < vExtents.length; i++)
        {
            vExtents[i] = PointLocalToWorld(vExtents[i]);
            vExtents[i].fY = info.GetSeatPos().fY + _GET_OUT_DOWN_CORRECTION;
        }
        
        Vector vFromPos = info.GetSeatPos();
        vFromPos.fY += _GET_OUT_DOWN_CORRECTION;
        
        for (int i = 0; i < vExtents.length; i++)
        {
            vPos = CanGetOutToPos(info, vFromPos, vExtents[i]);
            if (vPos != null)
            {
                SetFlags(iVehicleFlags);
                info.GetOccupant().SetFlags(iPassengerFlags);
                return vPos;
            }
        }
        
        SetFlags(iVehicleFlags);
        info.GetOccupant().SetFlags(iPassengerFlags);
        
        Vector vMin = new Vector();
        Vector vMax = new Vector();
        GetMeshExtentsInWorld(vMin, vMax);
        vPos = GetPosition();
        vPos.fY += Math.abs(vMax.fY - vMin.fY) + 10f;
        return vPos;
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	int m_nHeadRotateSoundIDSG = -1;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Walker.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_WALKER)
        {	
			super.SGLoadChunk(cFC);
		
			m_cPhysicsWalker.SGLoadChunk(cFC);
			
			// Lista broni walkera
			m_aWeapons = (WeaponWalkerList)cFC.LoadObject(m_aWeapons,this);
			
			// ID odpalonego dzwieku m_sHeadLoopSound
			m_nHeadRotateSoundIDSG = cFC.LoadInt();

			//predkosc chodzenie do przodu    
			m_fWalkSpeed = cFC.LoadFloat();
			//predkosc chodzenie do tylu
			m_fWalkSpeedBack = cFC.LoadFloat();
			//predkosc obracania sie (stopnie na sekunde)
			m_fRotateSpeed = cFC.LoadFloat();
    
			//Droga przebywana w jednej klatce animacji kroku walkera
			m_fWalkAnimSpeed = cFC.LoadFloat();
			//wspolczynnik pomiedzy wyliczona predkoscia animacji a to co trzeba zastosowac
			m_fWalkAnimSpeedMul = cFC.LoadFloat();			
        }
		
		cFC.delete();
    }
	    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Walker.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_WALKER, cFCParent);
		
		super.SGSaveChunk(cFC);

		m_cPhysicsWalker.SGSaveChunk(cFC);

		// Lista broni walkera
		cFC.SaveObject(m_aWeapons);
           
		// ID odpalonego dzwieku m_sHeadLoopSound
        if (m_nHeadRotateSoundID > 0 && // jesli pobrany jez id dzwieku
            !IsSoundPlaying(m_nHeadRotateSoundID)) // a nie jest on odgrywany
        {
            cFC.SaveInt(-1);// zaznacz, ze nie odgrywa
        }
        else
    		cFC.SaveInt(m_nHeadRotateSoundID); // zapisz wlasciwe
        
		//predkosc chodzenie do przodu    
		cFC.SaveFloat(m_fWalkSpeed);
		//predkosc chodzenie do tylu
		cFC.SaveFloat(m_fWalkSpeedBack);
		//predkosc obracania sie (stopnie na sekunde)
		cFC.SaveFloat(m_fRotateSpeed);
    
		//Droga przebywana w jednej klatce animacji kroku walkera
		cFC.SaveFloat(m_fWalkAnimSpeed);
		//wspolczynnik pomiedzy wyliczona predkoscia animacji a to co trzeba zastosowac
		cFC.SaveFloat(m_fWalkAnimSpeedMul);
		
        cFC.delete();
    }	    				
	
    /** Metoda wywolujaca odpowiednie akcje po odczycie danych
     */		
	public void SGUpdate()
	{
		FileChunk.Log("[Walker.SGUpdate]\n");
		
		super.SGUpdate();	
		
		if (m_nHeadRotateSoundID > 0)
		{
			DetachSound(m_nHeadRotateSoundID);
			m_nHeadRotateSoundID = -1;
		}
		
		if (m_nHeadRotateSoundIDSG > 0)
		{
			FileChunk.Log("[Walker.SGUpdate] HeadLoopSound attached! \n");
			m_nHeadRotateSoundID = AttachSound(m_sHeadLoopSound);
		}
	}			
    
    
    static int sm_nWalkerState = -1;
    
    //TODO: zoptymalizowac te sety
    public void ReplicationInit()
    {
        super.ReplicationInit();
        
        sm_nWalkerState = NewReplicationSet("m_fHorizAngle;m_fVertAngle;m_bPrimaryWeaponFiring;m_bSecondaryWeaponFiring;m_fTargetX;m_fTargetY;m_fTargetZ");   
    }
  
    /** Replikacja odpowiednich zestawow dla odpowiednich warunkow */
    public void OnReplicateServer()
    {
        super.OnReplicateServer();
        
        if ( bNetInit )
        {
            MsgSendClientWeaponsInfo();
        }
        
        if(bDontReplicate) return;
        
		Pawn driver = GetFirstOccupant( EPassengerRole._VEHICLE_DRIVER );
        if ( driver == null || !driver.bNetIsOwner )
        {
            Replicate(  sm_nWalkerState );
        }
    }
    
    public void OnReplicateClient()
    {
        super.OnReplicateClient();
        		
        if ( m_bLocalPlayerDriver )
        {
            Replicate( sm_nWalkerState );
        }
    }
    
    
    void MsgSendClientWeaponsInfo()
    {
        MessageNew( MSG_WEAPONSINFO );
        MessagePackInt( m_aWeapons.GetSize() );
        for ( int i=0; i<m_aWeapons.GetSize(); i++ )
        {
            WeaponWalker cWeapon = m_aWeapons.Get( i );
            /*LogR("MsgSendClientWeaponsInfo "+cWeapon+" "+cWeapon.GetMode()+
                 " "+cWeapon.GetWalkerElementBarrelExit()+"\n");*/
            MessagePackInt( cWeapon.GetWeapon().GetNetId() );
            MessagePackInt( cWeapon.GetMode() );
            MessagePackString( cWeapon.GetWalkerElementBarrelExit() );
        }
        MessageSend();
    }
    
    void MsgRecvClientWeaponsInfo()
    {
        int nCnt = MessageUnPackInt();
        for ( int i=0; i<nCnt; i++ )
        {
            int    nNetId   = MessageUnPackInt();
            int    nMode    = MessageUnPackInt();
            String sElement = MessageUnPackString();
            m_aWeaponsToApply.addElement( new WeaponWalkerInfo( nNetId, nMode, sElement ) );
            /*LogR("MsgRecvClientWeaponsInfo "+nNetId+" "+nMode+
                 " "+sElement+"\n");*/
        }
        MessageEnd();
        
        CallMethodEvery( METHOD_CheckWeaponsToApply, 1.0f );
    }
    
    void MsgSendAnim( int nAnim, boolean bStepAnimation )
    {
        MessageNew( MSG_ANIM );
        MessagePackInt( nAnim );
        MessagePackBoolean( bStepAnimation );
        MessageSend();
    }
    
    void MsgRecvAnim()
    {
        int     nAnim = MessageUnPackInt();
        boolean bStep = MessageUnPackBoolean();
        MessageEnd();
                
        if ( ! m_bLocalPlayerDriver && 
             m_cPhysicsWalker.IsWalkEnabled() )
        {
            PlayAnimLoop( nAnim, bStep );
        }
    }
    
    static final int MSG_WEAPONSINFO = 10; 
    static final int MSG_ANIM        = 11; 
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
         case MSG_WEAPONSINFO:
            LogD(" MSG_WEAPONSINFO\n");
            MsgRecvClientWeaponsInfo();
            break;
            
         case MSG_ANIM:
            LogD(" MSG_ANIM\n");
            MsgRecvAnim();
            break;
            
         default:
            super.OnMessage();
        }
    }
    
    java.util.Vector m_aWeaponsToApply = new java.util.Vector();
    
    void CheckWeaponsToApply()
    {
        for ( int i=0; i<m_aWeaponsToApply.size(); i++ )
        {
            boolean bRemove = true;
            
            if ( m_aWeaponsToApply.elementAt( i ) instanceof WeaponWalkerInfo )
            {
                WeaponWalkerInfo cInfo = (WeaponWalkerInfo)m_aWeaponsToApply.elementAt( i );
                Weapon cWeapon = GetWeaponByNetId( cInfo.GetNetId() );
                if ( cWeapon != null )
                    ApplyNewWeapon( cWeapon, cInfo.GetWalkerElementBarrelExit(),cInfo.GetMode() );
                else
                    bRemove = false;
            }
                    
            if ( bRemove )
            {
                m_aWeaponsToApply.removeElementAt( i );
                i --;
            }
        }
        if ( m_aWeaponsToApply.size() <= 0 )
            DisableCallMethod( METHOD_CheckWeaponsToApply );
    }
    
    boolean m_bPrimaryWeaponFiring       = false;
    boolean m_bSecondaryWeaponFiring     = false;
    boolean m_bPrimaryWeaponLastFiring   = false;
    boolean m_bSecondaryWeaponLastFiring = false;

    void SetWeaponFiring( int nMode, boolean bFiring )
    {
        if ( nMode == WeaponWalker._MODE_PRIMARY )
            m_bPrimaryWeaponFiring = bFiring;
        else
            m_bSecondaryWeaponFiring = bFiring;
    }
    
    public void OnAudioReinitialize()
    {
        m_nHeadRotateSoundID = -1;
    }
	
	boolean ShouldAddSimulate()
	{
		return m_cPhysicsWalker.IsWalkEnabled() && super.ShouldAddSimulate();
	}
}





